/* Copyright (C) 2001 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

///////////////////////////////////////////////////////////////////////////////
// @doc     ArchMgrRun
// @module  Architecture Manager Daemon | This module contains the
//          implementation of the ArchMgrRun class. The Architecture Manager
//          Daemon...
//
//          This module is required to build on NT, Windows 2000 and Solaris.
//
///////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////
// OpenSTA includes.
#include <stdafx.h>
#include <ArchMgrRun.h>
#pragma warning (disable:4786)
#pragma warning (disable:4503)

///////////////////////////////////////////////////////////////////////////////
// Standard includes.
using namespace std;

///////////////////////////////////////////////////////////////////////////////
// @mfunc   The constructor is responsible for...
//
// @parm    Pointer to CORBA server.
//
ArchMgrRun::ArchMgrRun(CORBAServer* pCorba, HANDLE hOSTADaemon,
    DWORD dwOSTADaemonPid, CString strOSTAHome) :
    m_bRunning(true), m_pCorba(pCorba), m_hOSTADaemon(hOSTADaemon),
    m_dwOSTADaemonPid(dwOSTADaemonPid), m_bTrace(FALSE), m_strOSTAHome(strOSTAHome)
{
    // Start Architecture Manager CORBA servant.
    StartArchMgr();
}

///////////////////////////////////////////////////////////////////////////////
// @mfunc   Destructor.
//
ArchMgrRun::~ArchMgrRun()
{
}

///////////////////////////////////////////////////////////////////////////////
// @mfunc   This method starts the daemon process running.  The daemon can
//          block further input or allow processing to continue.  If processing
//          is stopped there is NO way to stop the daemon other than to
//          terminate the application.
//
void ArchMgrRun::Run()
{
    // The Architecture Manager Daemon has been started.
    m_bRunning = true;

    // Accept requests.
    m_pCorba->Run();
}

///////////////////////////////////////////////////////////////////////////////
// @mfunc   This method is responsible for starting the Architecture Manager
//          Daemon CORBA object.
//
void ArchMgrRun::StartArchMgr()
{
    const char* where = "ArchMgrRun::StartArchMgr";

    try
    {
        // Start ArchMgr.
        m_pArchMgr = new ArchMgrImpl(m_pCorba, m_hOSTADaemon);
    }
    catch(CORBA::Exception&)
    {
        throw CyrDaemonException("Error starting CORBA services.",
            where, CYR_CORBA_INIT_FAILED );
    }
    catch(CyrCORBAException ex)
    {
        throw CyrDaemonException(ex.what().c_str(), where, ex.status());
    }
    catch(...)
    {
        throw CyrDaemonException("Unknown exception", where, CYR_UNKNOWN_ERROR);
    }
}
