/* Copyright (C) 2001 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

///////////////////////////////////////////////////////////////////////////////
// OpenSTA includes.
#include <stdafx.h>
#include <ArchMgrDmn.h>
#include <ArchMgr_i.h>
#include <ArchMgrHostFile.h>
#include <CECInetInfo.h>
#include <ArchMgrRun.h>
#include <Solutions.h>
#include <OstaTraceDef.h>

#pragma warning (disable:4786)
#pragma warning (disable:4503)

///////////////////////////////////////////////////////////////////////////////
// Standard includes.
#include <winsock.h>
#include <fstream>
using namespace std;

ArchMgrImpl::ArchMgrImpl(CORBAServer* pCorba, HANDLE hOSTADaemon)
    : m_pCorba(pCorba), m_pObjid(NULL), m_bOpen(TRUE), m_hOSTADaemon(hOSTADaemon)
{
    char strReg[64];
    CECInetInfo inet;
    m_pObjid = m_pCorba->ActivateObject(this);
    sprintf(strReg, "%s", inet.GetIPAddressAsString().c_str());
    m_strRegisterName = strReg;
    m_pCorba->NameObject(this->_this(), ArchMgr::CLASS_NAME, strReg);
}

ArchMgrImpl::~ArchMgrImpl()
{
    m_pCorba->UnNameObject(ArchMgr::CLASS_NAME,m_strRegisterName.c_str());
    Deactivate();
}

void ArchMgrImpl::Deactivate()
{
    if(m_pObjid)
    {
        m_pCorba->DeActivateObject(*m_pObjid);
        m_pObjid = NULL;
    }
}

///////////////////////////////////////////////////////////////////////////////
// @mfunc   Start the OpenSTA Daemon process.
//
CORBA::Boolean ArchMgrImpl::StartOSTADaemon()
{
    WriteLogMsg(TRCLEV_LOW, string("Start OpenSTA Daemon requested."));

    // Start the OpenSTA Daemon (cyrdmn.exe) for the localhost.
    DWORD dwOSTADaemonPid;
    if (CreateOSTADaemon(&m_hOSTADaemon, &dwOSTADaemonPid) != 0) return 0;

    return 1;
}

///////////////////////////////////////////////////////////////////////////////
// @mfunc   Stop the OpenSTA Daemon process.
//
CORBA::Boolean ArchMgrImpl::StopOSTADaemon()
{
    WriteLogMsg(TRCLEV_LOW, string("Stop OpenSTA Daemon requested."));
    // Terminate the OpenSTA Daemon process and clean the Naming Service.
    if (m_hOSTADaemon != 0)
    {
        TerminateProcess(m_hOSTADaemon, 0);
        m_hOSTADaemon = 0;
        CleanNS();
        WriteLogMsg(TRCLEV_LOW, string("OpenSTA Daemon stopped."));
    }
    else
    {
        WriteLogMsg(TRCLEV_LOW, string("OpenSTA Daemon was not running."));
    }

    return 1;
}

///////////////////////////////////////////////////////////////////////////////
// @mfunc   Restart the OpenSTA Daemon process.
//
CORBA::Boolean ArchMgrImpl::RestartOSTADaemon()
{
    // Terminate the existing OpenSTA Daemon process.
    if (m_hOSTADaemon != 0)
    {
        TerminateProcess(m_hOSTADaemon, 0);
        m_hOSTADaemon = 0;

        // Update the Naming Service.
        CleanNS();
    }

    // Start a new OpenSTA daemon process.
    return 1;
}

///////////////////////////////////////////////////////////////////////////////
// @mfunc   Stop the Architecture Manager daemon.
//
CORBA::Boolean ArchMgrImpl::Stop(CORBA::Boolean bCleanNames)
{
    WriteLogMsg(TRCLEV_LOW, string("Stop Architecture Manager Daemon requested."));

    // Terminate the OpenSTA Daemon process.
    WriteLogMsg(TRCLEV_LOW, string("Stopping OpenSTA Daemon."));
    if (m_hOSTADaemon != 0)
    {
        TerminateProcess(m_hOSTADaemon, 0);
        WriteLogMsg(TRCLEV_LOW, string("Stopped OpenSTA Daemon."));
        m_hOSTADaemon = 0;

        // Update the Naming Service, if requested (this is not required if Omninames
        // is also to be shutdown).
        if (bCleanNames) CleanNS();
    }
    else
    {
        WriteLogMsg(TRCLEV_LOW, string("OpenSTA Daemon was not running."));
    }

    // Exit.
    WriteLogMsg(TRCLEV_LOW, string("Architecture Manager Daemon stoped; exiting normally."));
    exit(0);
}

char* ArchMgrImpl::FileHostProperties()
{
    WriteLogMsg(TRCLEV_HIGH, string("Write Host Properties file requested."));
	// initialize MFC and print and error on failure
	if (!AfxWinInit(::GetModuleHandle(NULL), NULL, ::GetCommandLine(), 0))
	{
		// TODO: change error code to suit your needs
        WriteLogMsg(TRCLEV_NONE, string("FATAL ERROR: MFC initialization failed."));
        return 0;
	}
	else
	{
		// TODO: code your application's behavior here

        // Initialize Windows sockets - must call WSACleanup() when done.
	    WORD      VersionRequested;
	    WSADATA   stWSAData;
	    VersionRequested = MAKEWORD(1, 1);
        if (WSAStartup (VersionRequested, &stWSAData) != 0) 
        {
            WriteLogMsg(TRCLEV_NONE, string("FATAL ERROR: WSAStartup failed."));
            return CORBA::string_dup("");
        }

        // Get SYSTEM Data.
        const DWORD MAXBUFSIZ = 256;
        DWORD dwSize = MAXBUFSIZ;

        TCHAR szComputerName[MAXBUFSIZ];
        GetComputerName(szComputerName, &dwSize);

        dwSize = MAXBUFSIZ;
        TCHAR szUserName[MAXBUFSIZ];
        GetUserName(szUserName, &dwSize);

        TCHAR szOSName[MAXBUFSIZ];
        TCHAR szOSVersion[MAXBUFSIZ];
        GetOSInfo (szOSName, szOSVersion);

        TCHAR szArchitectureName[MAXBUFSIZ];
        TCHAR szNumberOfProcessors[MAXBUFSIZ];
        GetSystemData (szArchitectureName, szNumberOfProcessors);

        MEMORYSTATUS memstat;
        GlobalMemoryStatus(&memstat);

        CString strTotalMemory;
        strTotalMemory.Format("%ld KB", memstat.dwTotalPhys / 1024);

        CString strFreeMemory;
        strFreeMemory.Format("%ld KB", memstat.dwAvailPhys / 1024);

        CString strMemoryLoad;
        strMemoryLoad.Format("%d %%", memstat.dwMemoryLoad);

        // Get HOST data.
        char *pOstaHome;
        TCHAR szOstaHome[MAXBUFSIZ] = "";
        pOstaHome = getenv("OPENSTA_HOME");
        if (pOstaHome != NULL) strcpy(szOstaHome, pOstaHome);

        // Get Repository data.

        // Get Repository path.
        HKEY hKey;
        DWORD dwKeyValLen;
        unsigned char* szRepositoryPath = NULL;
        RegOpenKeyEx( HKEY_LOCAL_MACHINE,
            CURRENT_PROJECT_KEY, 0, KEY_QUERY_VALUE, &hKey );
        RegQueryValueEx(hKey, "Path", NULL, NULL, NULL, &dwKeyValLen);
        szRepositoryPath = new unsigned char[dwKeyValLen];
        RegQueryValueEx(hKey, "Path", NULL, NULL, szRepositoryPath, &dwKeyValLen);
        RegCloseKey(hKey);
        std::string strRepositoryPath ((char *) szRepositoryPath);
        delete [] szRepositoryPath;

        // Get host name of machine we are running on.
        char hostname[256];
        gethostname(hostname,256);
        hostent* host=gethostbyname(hostname);

        // Get IP address (with dots replaced by underscores) of machine we are running on.
	    CString strIPAddrUnderscores = "";
        in_addr* addr;
        if(host)
        {
            addr=((struct in_addr*)((host->h_addr_list)[0]));
		    strIPAddrUnderscores = inet_ntoa(*addr);
            strIPAddrUnderscores.Replace('.','_');
        }

        // Cleanup windows sockets.
        WSACleanup();

		string strHostInfo = "";
        strHostInfo +="[FILE-HEADER]\r\n";
        strHostInfo +="File Version=1\r\n";
 
        strHostInfo +="[SYSTEM-MAIN-LIST]\r\n";
		strHostInfo +="Computer Name=" + string(szComputerName) + "\r\n";
        strHostInfo +="User Name=" + string(szUserName) + "\r\n";
        strHostInfo +="OS Name=" + string(szOSName) + "\r\n";
        strHostInfo +="OS Version=" + string(szOSVersion) + "\r\n";
        strHostInfo +="Architecture=" + string(szArchitectureName) + "\r\n";
        strHostInfo +="Number of Processors=" + string(szNumberOfProcessors) + "\r\n";
        strHostInfo +="Total Memory=" + string(strTotalMemory) + "\r\n";
        strHostInfo +="Free Memory=" + string(strFreeMemory) + "\r\n";
        strHostInfo +="Memory Load=" + string(strMemoryLoad) + "\r\n";

        strHostInfo +="[HOST-MAIN-LIST]\r\n";
        strHostInfo +="OpenSTA Home= " + string(szOstaHome) + "\r\n";

        strHostInfo +="[REPOSITORY-MAIN-LIST]\r\n";
        strHostInfo +="Repository Path= " + strRepositoryPath;
		
		return CORBA::string_dup(strHostInfo.c_str());
	}
    return CORBA::string_dup("");;
}

BOOL ArchMgrImpl::GetOSInfo(char *pOSName, char *pOSVersion)
{
   OSVERSIONINFOEX osvi;
   BOOL bOsVersionInfoEx;

   // Try calling GetVersionEx using the OSVERSIONINFOEX structure,
   // which is supported on Windows 2000.
   //
   // If that fails, try using the OSVERSIONINFO structure.

   ZeroMemory(&osvi, sizeof(OSVERSIONINFOEX));
   osvi.dwOSVersionInfoSize = sizeof(OSVERSIONINFOEX);

   if( !(bOsVersionInfoEx = GetVersionEx ((OSVERSIONINFO *) &osvi)) )
   {
      // If OSVERSIONINFOEX doesn't work, try OSVERSIONINFO.

      osvi.dwOSVersionInfoSize = sizeof (OSVERSIONINFO);
      if (! GetVersionEx ( (OSVERSIONINFO *) &osvi) ) 
         return FALSE;
   }

   switch (osvi.dwPlatformId)
   {
      case VER_PLATFORM_WIN32_NT:

      // Test for the product.

         if ( osvi.dwMajorVersion <= 4 )
            strcpy(pOSName, "Microsoft Windows NT");

         if ( osvi.dwMajorVersion == 5 )
            strcpy(pOSName, "Microsoft Windows 2000");

      // Test for workstation versus server.

         if( bOsVersionInfoEx )
         {
            if ( osvi.wProductType == VER_NT_WORKSTATION )
               strcat(pOSName, " Professional");

            if ( osvi.wProductType == VER_NT_SERVER )
               strcat(pOSName, " Server");
         }
         else
         {
            HKEY hKey;
            char szProductType[80];
            DWORD dwBufLen;

            RegOpenKeyEx( HKEY_LOCAL_MACHINE,
               "SYSTEM\\CurrentControlSet\\Control\\ProductOptions",
               0, KEY_QUERY_VALUE, &hKey );
            RegQueryValueEx( hKey, "ProductType", NULL, NULL,
               (LPBYTE) szProductType, &dwBufLen);
            RegCloseKey( hKey );
            if ( lstrcmpi( "WINNT", szProductType) == 0 )
               strcat(pOSName, " Workstation");
            if ( lstrcmpi( "SERVERNT", szProductType) == 0 )
               strcat(pOSName, " Server");
         }

      // Display version, service pack (if any), and build number.

         sprintf (pOSVersion, "%d.%d %s (Build %d)",
            osvi.dwMajorVersion,
            osvi.dwMinorVersion,
            osvi.szCSDVersion,
            osvi.dwBuildNumber & 0xFFFF);

         break;

      case VER_PLATFORM_WIN32_WINDOWS:

         if ((osvi.dwMajorVersion > 4) || 
            ((osvi.dwMajorVersion == 4) && (osvi.dwMinorVersion > 0)))
         {
             strcpy(pOSVersion, "Microsoft Windows 98");
         } 
         else strcpy(pOSVersion,"Microsoft Windows 95");

         break;

      case VER_PLATFORM_WIN32s:

         strcpy(pOSVersion, "Microsoft Win32s");
         break;
   }
   return TRUE; 
}

void ArchMgrImpl::GetSystemData(char *pArchitectureName, char *pNumberOfProcessors)
{
    SYSTEM_INFO sysinf;
    GetSystemInfo (&sysinf);

    switch (sysinf.wProcessorArchitecture)
    {
    case PROCESSOR_ARCHITECTURE_INTEL:
    {
        strcpy (pArchitectureName, "Intel");
        switch (sysinf.wProcessorLevel)
        {
        case 3:
        {
            strcat(pArchitectureName, " 80386");
            break;
        }
        case 4:
        {
            strcat(pArchitectureName, " 80486");
            break;
        }
        case 5:
        {
            strcat(pArchitectureName, " Pentium");
            break;
        }
        case 6:
        {
            strcat(pArchitectureName, " Pentium Pro/Pentium II");
            break;
        }
        }
        break;
    }
    case PROCESSOR_ARCHITECTURE_MIPS:
    {
        strcpy (pArchitectureName, "MIPS");
        switch (sysinf.wProcessorLevel)
        {
        case 0004:
        {
            strcat(pArchitectureName, " R4000");
            break;
        }
        }
        break;
    }
    case PROCESSOR_ARCHITECTURE_ALPHA:
    {
        strcpy (pArchitectureName, "Alpha");
        switch (sysinf.wProcessorLevel)
        {
        case 21064:
        {
            strcat(pArchitectureName, " 21064");
            break;
        }
        case 21066:
        {
            strcat(pArchitectureName, " 21066");
            break;
        }
        case 21164:
        {
            strcat(pArchitectureName, " 21164");
            break;
        }
        }
        break;
    }
    case PROCESSOR_ARCHITECTURE_PPC:
    {
        strcpy (pArchitectureName, "PPC");
        switch (sysinf.wProcessorLevel)
        {
        case 1:
        {
            strcat(pArchitectureName, " 601");
            break;
        }
        case 3:
        {
            strcat(pArchitectureName, " 603");
            break;
        }
        case 4:
        {
            strcat(pArchitectureName, " 604");
            break;
        }
        case 6:
        {
            strcat(pArchitectureName, " 603+");
            break;
        }
        case 9:
        {
            strcat(pArchitectureName, " 604+");
            break;
        }
        case 20:
        {
            strcat(pArchitectureName, " 620");
            break;
        }
        }
        break;
    }
    default:
    {
        strcpy (pArchitectureName, "Unknown");
    }
    }


    sprintf (pNumberOfProcessors, "%d", sysinf.dwNumberOfProcessors);
}

///////////////////////////////////////////////////////////////////////////////
// @mfunc   Clean the Naming Service. This function examines all the objects
//          registered with the OpenSTA Naming Service (Omninames) and 
//          deregisters any objects that cannot be found, e.g. because the
//          host process has terminated.
//
void ArchMgrImpl::CleanNS()
{
    WriteLogMsg (TRCLEV_LOW, "Deregister non-existent objects from Naming Service.");
    CosNaming::NamingContext_var context;

    try
    {
        // Narrow the object returned by resolve_initial_references()
        // to a CosNaming::NamingContext object:
        context = m_pCorba->GetNamingContext();
        if (CORBA::is_nil(context))
        {
            WriteLogMsg(TRCLEV_NONE, "Failed to narrow naming context.");
            return;
        }
    }
    catch(CORBA::ORB::InvalidName)
    {
        WriteLogMsg (TRCLEV_NONE, "Service required is invalid [does not exist].");
    }

    // Get a list of all contexts on NameService
    CosNaming::BindingIterator_var bi;  // Binding iterator
    CosNaming::BindingList_var bl;      // Binding list
    const CORBA::ULong CHUNK = 100;     // Chunk size

    context->list(CHUNK, bl, bi);
    CleanNSChunk(bl, context);

    if (!CORBA::is_nil(bi))
    {
        CORBA::Boolean bMore;
        do
        {
            bMore = bi->next_n(CHUNK, bl);
            CleanNSChunk(bl, context);
        } while (bMore);
        bi->destroy();
    }

    WriteLogMsg (TRCLEV_LOW, "Deregister non-existent objects from Naming Service complete.");
}

void ArchMgrImpl::CleanNSChunk(const CosNaming::BindingList &bl, const CosNaming::NamingContext_var context)
{
    std::string strMsg; // WriteLogMsg message buffer

    for (CORBA::ULong isub = 0; isub < bl.length(); isub++)
    {
        CORBA::Object_var obj;
        try
        {
            // Resolve name object
            obj = context->resolve(bl[isub].binding_name);
        }
        catch (CORBA::Exception &e)
        {
            // Here if object cannot be found. So de-register.
            strMsg = "Cannot resolve " + bl[isub].binding_name[0].id +
                "," + bl[isub].binding_name[0].kind + " Caught: " + CORBAExString(e).c_str();
            WriteLogMsg(TRCLEV_NONE, strMsg);
            context->unbind(bl[isub].binding_name);
        }
        catch(...)
        {
            ; // Catch - and ignore - all other exceptions
        }

        //Retry on CORBA::Exceptions break out on anything else
        bool deregister=false;
        strMsg = "Checking Object: " + bl[isub].binding_name[0].id + "," + bl[isub].binding_name[0].kind;
        WriteLogMsg (TRCLEV_HIGH, strMsg);

        deregister=false;//set to false on each try
        try// Check for existence
        {
            if (CORBA::is_nil(obj))
            {
                strMsg = "Object is NIL " + 
                    bl[isub].binding_name[0].id + "," + bl[isub].binding_name[0].kind;
                WriteLogMsg (TRCLEV_NONE, strMsg);
                deregister=true;
            }
            else if(obj->_non_existent())    //object does not exist
            {
                strMsg = "Object No Longer Exists " + 
                    bl[isub].binding_name[0].id + "," + bl[isub].binding_name[0].kind;
                WriteLogMsg (TRCLEV_NONE, strMsg);
                deregister=true;
            }
        }
        catch( CORBA::Exception &e)
        {
            // Here if object cannot be found. So de-register.
            strMsg = "CORBA::Exception checking " + bl[isub].binding_name[0].id +
                "," + bl[isub].binding_name[0].kind + " Caught: " + CORBAExString(e).c_str();
            WriteLogMsg (TRCLEV_NONE, strMsg);
            deregister = true;
        }
        catch (...)
        {
            ; // Catch - and ignore -all other exceptions.
        }

        // If the object was not found then deregister it from the Naming Service
        if (deregister)
        {
            strMsg = "Deregistering object " +
                bl[isub].binding_name[0].id + "," +
                bl[isub].binding_name[0].kind;
            WriteLogMsg (TRCLEV_HIGH, strMsg);
            context->unbind(bl[isub].binding_name);

        }
    }
}