/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

///////////////////////////////////////////////////////////////////////////////
// @doc     VDK_MODULE
// @module  Simple Time Service VDK | This module provides the Time for the
//          node on which it is situated in seconds since Jan 1900.
//
//          Typically VDKs will be loaded by the Cyrano Daemon application
//          and utilise the Factory within that application.
//
///////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////
// Cyrano includes
#include <Daemon/CyrVdk.h>
#include "TimeService_i.h"
///////////////////////////////////////////////////////////////////////////////
// Global variables
TimeServiceImpl* TimeS=NULL;
HINSTANCE g_hInstance = NULL;

///////////////////////////////////////////////////////////////////////////////
// @func   This function initialises Name service cleaner
// @rdesc A standard Cyrano error code.
VDK_EXP CyrVdkInitialise( const char* factory, CORBAServer* pServer )
{
    TimeS=new TimeServiceImpl(pServer);
    return CYR_SUCCESS;
}

///////////////////////////////////////////////////////////////////////////////
// @func  This function stops the  Name service cleaner
// @rdesc A standard Cyrano error code.
VDK_EXP CyrVdkUninitialise()
{
    delete TimeS;
    return CYR_SUCCESS;
}


VDK_EXP GetModuleVersion(string& strVersion)
{
	DWORD dwHandle;
	CHAR currentFileName[2*MAX_PATH];
	if ( GetModuleFileName(g_hInstance, currentFileName, 2*MAX_PATH) == 0 )
	{
		return -1;
	}
	DWORD dwSize = GetFileVersionInfoSize((LPTSTR)currentFileName, &dwHandle);
	BYTE* pVersionInfo= new BYTE[dwSize];
	GetFileVersionInfo((LPTSTR)currentFileName, 0, dwSize, pVersionInfo);

	struct LANGANDCODEPAGE
	{
	  WORD wLanguage;
	  WORD wCodePage;
	} *lpTranslate;

	// Read the list of languages and code pages.
	UINT cbTranslate;
	VerQueryValue(pVersionInfo, 
              TEXT("\\VarFileInfo\\Translation"),
              (LPVOID*)&lpTranslate,
              &cbTranslate);

	char SubBlock[512];
	sprintf(SubBlock, "\\StringFileInfo\\%04x%04x\\FileVersion", 
			lpTranslate[0].wLanguage,
	        lpTranslate[0].wCodePage);
	
	LPVOID lpBuffer;
	UINT dwBytes;
	// Retrieve file description for language and code page "i". 
	VerQueryValue(pVersionInfo, SubBlock, &lpBuffer, &dwBytes); 

	string strVer = (LPTSTR)lpBuffer;

	strVersion = currentFileName;
	int iPos = strVersion.find_last_of("\\");
	if ( iPos == string::npos )
		iPos = strVersion.find_last_of(":");
	if ( iPos != string::npos )
		strVersion.erase(0, iPos);

	strVersion += " ";
	strVersion += strVer;

	return 0;
}



extern "C"
BOOL WINAPI DllMain(HINSTANCE hInstance, DWORD dwReason, LPVOID /*lpReserved*/)
{
    if (dwReason == DLL_PROCESS_ATTACH)
    {
		g_hInstance = hInstance;

        CyrtoolsInitialize();
    }
    else if (dwReason == DLL_PROCESS_DETACH)
    {
        CyrtoolsUninitialize();
    }
    return TRUE;
}
