/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

///////////////////////////////////////////////////////////////////////////////
// @doc     VDK_MODULE
// @module  Injector Service VDK | This module provides services for
//          controlling the execution of Task Groups on an Injector and
//          for obtaining status information for Executers running on an
//          Injector.
//
//          VDKs are loaded by the OpenSTA Daemon when it starts up.
//
///////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////
// OpenSTA includes.
#include <Daemon/CyrVdk.h>
#include "InjectorControl_i.h"
#include "HTTPInjectorStatus_i.h"
#include "StepByStep_Control_i.h"

///////////////////////////////////////////////////////////////////////////////
// Global variables
//
// @globalv InjectorControlImpl* | InjectorControl | Pointer to Injector
//          Control object.
InjectorControlImpl* InjectorControl = NULL;

// @globalv HTTPInjectorStatusImpl* | InjectorStatus | Pointer to InjectorStatus object.
// 
         
HTTPInjectorStatusImpl* InjectorStatus = NULL;
//InjectorStatusImpl* InjectorStatus = NULL;

// @globalv StepByStepControlImpl* | stepByStepControl | Pointer to step-by-step control
//          object.
StepByStepControlImpl* pStepByStepControl = NULL;

HINSTANCE g_hInstance = NULL;

///////////////////////////////////////////////////////////////////////////////
// @func    This function initialises the OpenSTA Injector Service VDK.
//
// @rdesc   A standard OpenSTA error code.
//
// @parm    The name of the parent Factory that all Providers (allocated using
//          the Factory mechanism) within this DLL will register themselves
//          with.
// @parm    Pointer to CORBA server object.
//
VDK_EXP CyrVdkInitialise( const char* factory, CORBAServer* pServer )
{
#ifndef _WIN32
    CyrtoolsInitialize();
#endif
    // Create the InjectorControl object for this Injector
    InjectorControl = new InjectorControlImpl(pServer);

    // Create the HTTPInjectorStatus object for this Injector.
    InjectorStatus = new HTTPInjectorStatusImpl(pServer, *InjectorControl);
	//InjectorStatus = new InjectorStatusImpl(pServer, *InjectorControl);

    // Create the StepByStepControl object for this Injector
    pStepByStepControl = new StepByStepControlImpl(pServer, *InjectorControl);

    return (InjectorControl && InjectorStatus && pStepByStepControl ? CYR_SUCCESS
        : CYR_UNKNOWN_ERROR);
}

///////////////////////////////////////////////////////////////////////////////
// @func    This function stops the Injector Service VDK. It deletes the
//          InjectorControl and InjectorStaus objects.
//
// @rdesc   A standard OpenSTA error code.
//
VDK_EXP CyrVdkUninitialise()
{
    delete InjectorStatus;
    delete pStepByStepControl;
    delete InjectorControl;
#ifndef _WIN32
    CyrtoolsUninitialize();
#endif
    return CYR_SUCCESS;
}


VDK_EXP GetModuleVersion(string& strVersion)
{
	DWORD dwHandle;
	CHAR currentFileName[2*MAX_PATH];
	if ( GetModuleFileName(g_hInstance, currentFileName, 2*MAX_PATH) == 0 )
	{
		return -1;
	}
	DWORD dwSize = GetFileVersionInfoSize((LPTSTR)currentFileName, &dwHandle);
	BYTE* pVersionInfo= new BYTE[dwSize];
	GetFileVersionInfo((LPTSTR)currentFileName, 0, dwSize, pVersionInfo);

	struct LANGANDCODEPAGE
	{
	  WORD wLanguage;
	  WORD wCodePage;
	} *lpTranslate;

	// Read the list of languages and code pages.
	UINT cbTranslate;
	VerQueryValue(pVersionInfo, 
              TEXT("\\VarFileInfo\\Translation"),
              (LPVOID*)&lpTranslate,
              &cbTranslate);

	char SubBlock[512];
	sprintf(SubBlock, "\\StringFileInfo\\%04x%04x\\FileVersion", 
			lpTranslate[0].wLanguage,
	        lpTranslate[0].wCodePage);
	
	LPVOID lpBuffer;
	UINT dwBytes;
	// Retrieve file description for language and code page "i". 
	VerQueryValue(pVersionInfo, SubBlock, &lpBuffer, &dwBytes); 

	string strVer = (LPTSTR)lpBuffer;

	strVersion = currentFileName;
	int iPos = strVersion.find_last_of("\\");
	if ( iPos == string::npos )
		iPos = strVersion.find_last_of(":");
	if ( iPos != string::npos )
		strVersion.erase(0, iPos);

	strVersion += " ";
	strVersion += strVer;

	return 0;
}




#ifdef _WIN32
extern "C"
BOOL WINAPI DllMain(HINSTANCE hInstance, DWORD dwReason, LPVOID /*lpReserved*/)
{
    if (dwReason == DLL_PROCESS_ATTACH)
    {
		g_hInstance = hInstance;

        CyrtoolsInitialize();
    }
    else if (dwReason == DLL_PROCESS_DETACH)
    {
        CyrtoolsUninitialize();
    }
    return TRUE;
}
#endif
