/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

#ifndef _GlobalVariableImpl_H_
#define _GlobalVariableImpl_H_
#include "GlobalVariable.h"
#include "CECListVar.h"
#include "CECFileVar.h"
#include "CECRangeVar.h"
#include "CECArrayVar.h"
#include "CECSimpleVar.h"
#include "CyrException.h"
#include <string>
#include <OSTAORB/OSTAORB.h>    
#define VAR_CATCH_ALL catch(const CyrException & e) \
{ \
throw CyrGlobalVariableException(e.what().c_str());\
} \
catch(const CORBA::Exception&)\
{\
throw;\
}\
catch(const exception &e) \
{ \
throw CyrGlobalVariableException(e.what());\
} \
catch(...)\
{\
throw CyrGlobalVariableException("Unknown exception thrown in Variable handling");\
};


class GlobalVariableImpl:public virtual POA_GlobalVariable,public PortableServer::RefCountServantBase
{
public:
    GlobalVariableImpl(CORBAServer* pcorba);
    virtual ~GlobalVariableImpl();
    virtual char*  get_value(const char*  expression);
    virtual CORBA::Long  set_value(const char*  newval, const char*  expression);
    virtual void next_value();
    virtual void generate_value();
    virtual void reset_seed();
protected:
    CECProcessVar*      m_Variable;
};

class GlobalRangeVarImpl:public GlobalVariableImpl
{
public:
    GlobalRangeVarImpl(CORBAServer* corba,string name,int options,
                       string values,unsigned long seed)
:GlobalVariableImpl(corba)
    {
        m_Variable=new  CECRangeVar(name,options,values,seed);
    }
    virtual ~GlobalRangeVarImpl()
    {}
    ;

};
class GlobalListVarImpl:public GlobalVariableImpl
{
public:
    GlobalListVarImpl(CORBAServer* corba,string name,int size,int options,
                      string values,unsigned long seed)
:GlobalVariableImpl(corba)
    {
        m_Variable=new CECListVar(name,size,options,values,seed);
    }
    virtual ~GlobalListVarImpl()
    {}
    ;
};

class GlobalFileVarImpl:public GlobalVariableImpl
{
public:
    GlobalFileVarImpl(CORBAServer* corba,string name,int size,int options,string file)
:GlobalVariableImpl(corba)
    {
        m_Variable=new CECFileVar(name,size,options,"",file);
    }
    virtual ~GlobalFileVarImpl()
    {}
};

class GlobalArrayVarImpl:public GlobalVariableImpl
{
public:
    GlobalArrayVarImpl(CORBAServer* corba,string name,int size,int options,string indecies)
:GlobalVariableImpl(corba)
    {
        m_Variable=new CECArrayVar(name,size,options,indecies);
    }
    virtual ~GlobalArrayVarImpl()
    {}
    ;
};

class GlobalSimpleVarImpl:public GlobalVariableImpl
{
public:
    GlobalSimpleVarImpl(CORBAServer* corba,string name,int size,int options)
:GlobalVariableImpl(corba)
    {
        m_Variable=new CECSimpleVar(name,size,options);
    }
    virtual ~GlobalSimpleVarImpl()
    {}
    ;
};
#endif
