/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

#include "TPMutex_i.h"
//mutex for locking actions shared for lock and trylock, protects wrinting of 'key'
static omni_mutex mutex;
/////////////////////////////////////////////////////
// @mfunc Constructor
TPMutexImpl::TPMutexImpl(CORBAServer* pcorba,string MutexName)
        :semaphore(1),m_regname(MutexName),m_corba(pcorba)
{
    m_corba->ActivateObject(this);
    m_corba->NameObject(this->_this(),TPMutex::CLASS_NAME,m_regname.c_str());
}
/////////////////////////////////////////////////////
// @mfunc Destructor
TPMutexImpl::~TPMutexImpl()
{
    m_corba->UnNameObject(TPMutex::CLASS_NAME,m_regname.c_str());
}
/////////////////////////////////////////////////////
// @mfunc Lock Mutex
// @rdesc key required to unlock mutex
CORBA::Long  TPMutexImpl::lock()
{
    omni_mutex_lock guard(mutex);
    //done by guard: mutex.acquire();
    semaphore.wait();//wait for lock to become free
    key=rand();//generate key used to unlock
    //done by guard: mutex.release();
    return key;
}
/////////////////////////////////////////////////////
// @mfunc Try and Lock Mutex
// @rdesc key required to unlock mutex or -1 if unable to lock
CORBA::Long  TPMutexImpl::trylock()
{
    omni_mutex_lock guard(mutex);
    int have_locked=semaphore.trywait();
    CORBA::Long  ret;
    if(have_locked==1)//have mangaged to acquire semaphore
    {
        key=rand();//generate key used to unlock
        ret=key;
    }
    else //already acquired
    {
        ret=-1;
    }
    return ret;
}
/////////////////////////////////////////////////////
// @mfunc unLock Mutex
// @parm key required to unlock mutex
void TPMutexImpl::unlock(CORBA::Long  mtxid)
{
    static omni_mutex umutex;//unlocking mutex
    omni_mutex_lock guard(umutex);
    //done by guard:mutex.acquire();
    if(mtxid!=key)
    {
        //done by guard: mutex.release();
        throw TPMutexException("Wrong Key to unlock this mutex",-1);
    }
    semaphore.post();
    //done by guard: mutex.release();
}
