/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

///////////////////////////////////////////////////////////////////////////////
// @doc CyrFFApi
///////////////////////////////////////////////////////////////////////////////

#ifndef _CECFFImpactProject_h
#define _CECFFImpactProject_h

///////////////////////////////////////////////////////////////////////////////
// Cyrano includes
#include "CECFFProject.h"

///////////////////////////////////////////////////////////////////////////////
// @class   CECFFImpactProject | This class represents a particular type of
//          project within OpenSTARep.  This project type is an Impact project.
// @devnote The main difference is a collection of build parameters.  I have
//          absolutely no idea what they represent.  Hence the lack of comments
//          until such time as someone who knows what they are fills them in.
// @base    public | CECFFProject
class CECFFImpactProject : public CECFFProject
{
    // @access Public Members
public:
    // @cmember Constructor for retrieving existing Project
    CECFFImpactProject( CECFFServer* server, const char* name );
    // @cmember Constructor for creating a new Project
    CECFFImpactProject( CECFFServer* server, const char* name, const char* desc );
    virtual ~CECFFImpactProject();  // @cmember Destructor

    bool GetBuildDebug();
    void SetBuildDebug( bool flag );
    long GetBuildMaxSteps();
    void SetBuildMaxSteps( long steps );
    bool GetBuildList();
    void SetBuildList( bool flag );
    bool GetBuildCheckSteps();
    void SetBuildCheckSteps( bool flag );

    // @access Protected Members
protected:
    void ReadBuildSettings();   // @cmember Read build settings from disk
    void WriteBuildSettings();  // @cmember Write build settings to disk

    // @access Private Members
private:
    bool m_debug;
    bool m_list;
    bool m_checkSteps;
    long m_steps;
};

#endif
