/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

///////////////////////////////////////////////////////////////////////////////
// @doc     CyrFFApi
// @module  Project Implementation | This module contains the implementation of
//          the CECFFProject class.  This class is simply a C++ wrapper around
//          the OpenSTARep COM interface IProject.
//
///////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////
// Cyrano includes
#include "CECFFProject.h"
#include <Repository/CyrStHelper.inl>

///////////////////////////////////////////////////////////////////////////////
// Standard includes
#include <direct.h>

///////////////////////////////////////////////////////////////////////////////
// @mfunc   This constructor retrieves an existing Project from the OpenSTARep
//          database.  To do this it requires the Server object currently in
//          use and the name of the project.  It retrieves the underlying
//          IStServer interface from this object and can then loop through all
//          known Projects until the correct IProject interface is found.
// @parm    Pointer to the parent CECFFServer object.
// @parm    The name of this Project.
CECFFProject::CECFFProject( CECFFServer* parent, const char* name )
{
    // Error information
    const char* where = "CECFFProject::CECFFProject";

    // Attempt to retrieve project
    if ( RetrieveProject( parent, name ) == false )
        throw CyrRepositoryException( "Cannot locate project", where,
                                      CYR_PROJECT_NOT_FOUND );
    // Create full path name
    string fullPath = GetRootDirectory().c_str();
    fullPath += name;

    // Actually create the root folder, this may error if dir already exists but
    // that shouldn't cause a problem
    _mkdir( fullPath.c_str() );
    m_name=name;
    m_desc="";
}

bool CECFFProject::RetrieveProject( CECFFServer* parent, const char* name )
{
    // Error information
    const char* where = "CECFFProject::RetrieveProject";

    // Retrieve server interface
    m_parent = parent;
    return true;
}

///////////////////////////////////////////////////////////////////////////////
// @mfunc   This method constructs a completely new Project within the
//          OpenSTARep database.
// @parm    Pointer to the parent CECFFServer object.
// @parm    The name of the new Project.
// @parm    The description of the new Project.
CECFFProject::CECFFProject( CECFFServer* parent, const char* name,
                            const char* desc )
{
    // Error information
    const char* where = "CECFFProject::CECFFProject";
    m_name=name;
    m_desc=desc;
}

///////////////////////////////////////////////////////////////////////////////
// @mfunc   The destructor is responsible for destroying the underlying COM
//          interface.
CECFFProject::~CECFFProject()
{}

///////////////////////////////////////////////////////////////////////////////
// @mfunc   This method returns the current description of the underlying COM
//          Project interface.
// @rdesc   The current description.

string CECFFProject::GetDescription() const
{
    return m_desc;
}

///////////////////////////////////////////////////////////////////////////////
// @mfunc   This method returns the name of the Project as recognised by
//          OpenSTARep.  It should be the same name as the one passed into the
//          constructor and therefore should not be needed.  However, it has
//          been added here for completeness.
// @rdesc   The name of the Project.
string CECFFProject::GetName() const
{
    return m_name;
}

CECFFServer* CECFFProject::GetParentServer()
{
    return m_parent;
}
