/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

#ifndef _OSTA_ORB_H_
#define _OSTA_ORB_H_

///////////////////////////////////////////////////////////////////////////////
// Cyrano includes
#include <definition.h>
#include <CyrException.h>
///////////////////////////////////////////////////////////////////////////////
// CORBA includes
#include <omniORB3/CORBA.h>
#include <omniORB3/Naming.hh>
///////////////////////////////////////////////////////////////////////////////
//Standard includes
#include <ostream>
#include <string>
#include <vector>
using std::string;
using std::vector;
#ifdef typename
#undef typename
#endif
class CORBAServer
{
public:

    typedef struct RegName
    {
	string Id;
	string Kind;
    };
    typedef vector<RegName>  NameList_t;

    CORBAServer(int argc,char** argv, bool bUseNameService = true);
    CORBAServer(const CORBAServer& srv);
    virtual ~CORBAServer();
    void Run();
    PortableServer::ObjectId* ActivateObject(PortableServer::Servant p_servant);
    void DeActivateObject(const PortableServer::ObjectId& oid);
    void NameObject(CORBA::Object_ptr objref,const char* classname,const char* regname);
    void UnNameObject(const char* classname,const char* regname);
    CORBA::Object_ptr FindObjectWithName(const char* classname,const char* regname);
    NameList_t GetListOfNames(const char* type=NULL);
	CORBA::Object_ptr stringToObject(const string & iorString) ;
	string objectToString(CORBA::Object_ptr obj) ;
    CosNaming::NamingContext_var GetNamingContext() {return m_rootContext;};

private:
    CORBA::ORB_var m_orb;
    PortableServer::POA_var m_poa;
    PortableServer::POAManager_var m_manager;
    CosNaming::NamingContext_var m_rootContext;
    static int refcount;

};

///////////////////////////////////////////////////////////////////////////////
// @func Convert CORBA::Exception so that a text string can be output to ostreams
//	Based on code from 
//	"Advanced CORBA Programming with C++(Michi Henning,Steve Vinoski)"
static inline std::ostream& operator<<(std::ostream &os,const CORBA::Exception &e)
{   
    CORBA::Any anytmp;
    anytmp<<=e;
    CORBA::TypeCode_var tc = anytmp.type();
    const char* p=tc->name();
    if(*p != '\0' )
	os << p;
    else
	os << tc->id();
    return os;
}

///////////////////////////////////////////////////////////////////////////////
// @func Convert CORBA::Exception to a std::string
//	Based on code from 
//	"Advanced CORBA Programming with C++(Michi Henning,Steve Vinoski)"
inline const char* CorbaExceptionString( const CORBA::Exception& e )
{   
    CORBA::Any anytmp;
    anytmp<<=e;
    CORBA::TypeCode_var tc = anytmp.type();
    CORBA::String_var p=tc->name();
    if(strcmp("",(const char*)p)==0)
	p=tc->id();
    return CORBA::string_dup(p);
}
inline string CORBAExString(const CORBA::Exception &e)
{   
    CORBA::Any anytmp;
    anytmp<<=e;
    CORBA::TypeCode_var tc = anytmp.type();
    string p=tc->name();
    if(p.empty())
	p=tc->id();
    return p;
}

#endif


