/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

//==================================================================< Pragmas >
#ifdef  WIN32
#  pragma warning(disable : 4514)
#endif  // WIN32


//=================================================================< Includes >
#ifndef __DEFINITION__
#  include "definition.h"
#endif  // __DEFINITION__

#ifdef  WIN32
#  ifndef   _WINDOWS_
#    define WIN32_LEAN_AND_MEAN
#    define NOGDICAPMASKS
#    define NOVIRTUALKEYCODES
#    define NOWINMESSAGES
#    define NOWINSTYLES
#    define NOSYSMETRICS
#    define NOMENUS
#    define NOICONS
#    define NOKEYSTATES
#    define NOSYSCOMMANDS
#    define NORASTEROPS
#    define NOSHOWWINDOW
#    define OEMRESOURCE
#    define NOATOM
#    define NOCLIPBOARD
#    define NOCOLOR
#    define NOCTLMGR
#    define NODRAWTEXT
#    define NOGDI
#    define NOKERNEL
//#    define   NOUSER
#    define NONLS
#    define NOMB
#    define NOMEMMGR
#    define NOMETAFILE
#    define NOMINMAX
#    define NOMSG
#    define NOOPENFILE
#    define NOSCROLL
//#    define   NOSERVICE
#    define NOSOUND
#    define NOTEXTMETRIC
#    define NOWH
#    define NOWINOFFSETS
#    define NOCOMM
#    define NOKANJI
#    define NOHELP
#    define NOPROFILER
#    define NODEFERWINDOWPOS
#    define NOMCX
#    pragma warning(disable : 4201)
#    include <windows.h>
#    pragma warning(default : 4201)
#  endif    // _WINDOWS_
#endif  // WIN32

#ifdef  UNIX
#  include <unistd.h>
#  include <signal.h>
#  include <errno.h>
#  include <iostream.h>
#endif  // UNIX

#ifndef __MTHREAD__
#  include "mthread.hpp"
#endif

#ifndef __SERVICE__
#  include "service.hpp"
#endif  // __SERVICE__

#ifndef __XALLOC__
#  include "xalloc.hpp"
#endif  // __XALLOC__

#include <iostream>
using std::endl ;
using std::cerr ;

//=====================================================< Definition of Macros >
#define SERVICE_CONSOLE_STOP    0


//=====================================================================< Data >
#ifdef  WIN32
extern  OSVERSIONINFO   osVersionInfo ;
#endif  // WIN32


//======================================================< Definition of class >

///////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////// Class CTServiceInfo //
///////////////////////////////////////////////////////////////////////////////
// Class used to create and control service informations.
class CTServiceInfo : public CTMTSafe
{
    //-----------------------------------------------------------------------------
public:                     // Constructor(s) and destructor.
    CTServiceInfo (const string& programName, const string& serviceName, CTService& service) ;
    ~CTServiceInfo ()
    { }
    ;

public:                     // Get methods.
    bool                    isInstalled (void) ;

public:                     // Controls methods.
    void                    console (void) ;
    void                    service (void) ;
    void                    install (void) ;
    void                    uninstall (void) ;
    void                    control (CTService::enumServiceControl eServiceControl) ;
#ifdef  WIN32
    void                    mainService (void) ;
    void                    controlService (DWORD dwCommand) ;
#endif  // WIN32
#ifdef  UNIX
    void                    controlService (int iSig) ;
#endif  // UNIX

    //-----------------------------------------------------------------------------
private:                    // Operator(s).
    CTServiceInfo&          operator= (const CTServiceInfo&) ;

private:                    // Members.
    string              programName_ ;
    string              serviceName_ ;
#ifdef  WIN32
    string              displayName_ ;
    SERVICE_STATUS          serviceStatus_ ;
    SERVICE_STATUS_HANDLE   hServiceStatus_ ;
#endif  // WIN32
    CTService&              service_ ;

    //-----------------------------------------------------------------------------
}

; // CTServiceInfo

///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////// End of Class CTServiceInfo //
///////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////// Globals Functions //
///////////////////////////////////////////////////////////////////////////////

#ifdef  WIN32
void WINAPI serviceMainFunction (DWORD, LPTSTR *) ;
void WINAPI serviceControlFunction (DWORD dwCommand) ;
BOOL consoleControleHandler (DWORD) ;
#endif  // WIN32
#ifdef  UNIX
void serviceControlFunction (int iSig) ;
#endif  // UNIX


///////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////// End of Globals Functions //
///////////////////////////////////////////////////////////////////////////////


//=====================================================================< Data >
CTServiceInfo*  GpProcessServiceInfo = NULL ;


//====================================================================< Class >

///////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////// Class CTServiceInfo //
///////////////////////////////////////////////////////////////////////////////
// Class used to create and control service informations.

//============================================< Constructor(s) and destructor >
//-----------------------------------------------------------------------------
// CTServiceInfo::CTServiceInfo
//
// Constructor.
//
// Arguments : programName : name of the program.
//             serviceName : name of the service.
//             service     : service.
// Return    : none.
//-----------------------------------------------------------------------------
CTServiceInfo::CTServiceInfo (const string& programName, const string& serviceName, CTService& service)
        : programName_ (programName), serviceName_ (serviceName), service_ (service)
{
#ifdef  WIN32
    // Save the real name.
    displayName_ = serviceName_ ;

    // Make service name correct for windows.
    size_t  index ;
    while ( (index = serviceName_.find_first_of ("/\\") ) != string::npos)
    {
        serviceName_[index] = '-' ;
    }
#endif  // WIN32
    // Get the program name.
    programName_.erase (0, programName_.find_last_of ('\\') + 1) ;
    programName_.erase (0, programName_.find_last_of ('/') + 1) ;
} // CTServiceInfo::CTServiceInfo

//=========================================================< Control methods >
//-----------------------------------------------------------------------------
// CTServiceInfo::console
//
// Start the service as a console.
//
// Arguments : none.
// Return    : none.
//-----------------------------------------------------------------------------
void CTServiceInfo::console (void)
{
#ifdef  WIN32
    // Save the this pointer.
    GpProcessServiceInfo = this ;

    // Set the console controle handler.
    if (SetConsoleCtrlHandler ( (PHANDLER_ROUTINE) consoleControleHandler, TRUE) == 0)
    {   // Error from SetConsoleCtrlHandler ().
        char zBuf[255] ;
        FormatMessage (FORMAT_MESSAGE_FROM_SYSTEM, NULL, GetLastError (), MAKELANGID (LANG_NEUTRAL, SUBLANG_DEFAULT), (LPTSTR) zBuf, sizeof (zBuf), NULL) ;
        CharToOem (zBuf, zBuf) ;
        throw CTServiceException (string ("console: SetConsoleCtrlHandler failed: ") + zBuf, __FILE__, __LINE__, GetLastError () ) ;
    }
#endif  // WIN32

    // Initialize the service.
    service_.init () ;

    // Execute the service.
    service_.main () ;

    // Exit the service.
    service_.exit () ;

#ifdef  WIN32
    // Remove the console controle handler.
    if (SetConsoleCtrlHandler ( (PHANDLER_ROUTINE) consoleControleHandler, FALSE) == 0)
    {   // Error from SetConsoleCtrlHandler ().
        char zBuf[255] ;
        FormatMessage (FORMAT_MESSAGE_FROM_SYSTEM, NULL, GetLastError (), MAKELANGID (LANG_NEUTRAL, SUBLANG_DEFAULT), (LPTSTR) zBuf, sizeof (zBuf), NULL) ;
        CharToOem (zBuf, zBuf) ;
        throw CTServiceException (string ("console: SetConsoleCtrlHandler failed: ") + zBuf, __FILE__, __LINE__, GetLastError () ) ;
    }
#endif  // WIN32
} // CTServiceInfo::console

//-----------------------------------------------------------------------------
// CTServiceInfo::service
//
// service the service as a service.
//
// Arguments : none.
// Return    : none.
//-----------------------------------------------------------------------------

void CTServiceInfo::service (void)
{
#ifdef  WIN32
    if (osVersionInfo.dwPlatformId == VER_PLATFORM_WIN32_NT)
    {
        // Connect to the service control manager.
        SC_HANDLE hServiceControlManager = OpenSCManager (NULL, NULL, SC_MANAGER_ALL_ACCESS) ;
        if (hServiceControlManager == NULL)
        {   // Error from OpenSCManager ().
            char zBuf[255] ;
            FormatMessage (FORMAT_MESSAGE_FROM_SYSTEM, NULL, GetLastError (), MAKELANGID (LANG_NEUTRAL, SUBLANG_DEFAULT), (LPTSTR) zBuf, sizeof (zBuf), NULL) ;
            CharToOem (zBuf, zBuf) ;
            throw CTServiceException (string ("service: OpenSCManager failed: ") + zBuf, __FILE__, __LINE__, GetLastError () ) ;
        }

        // Lock the control services database.
        SC_LOCK hServiceControlLocked = LockServiceDatabase (hServiceControlManager) ;
        if ( (hServiceControlLocked == NULL) && (GetLastError () != ERROR_SERVICE_DATABASE_LOCKED) )
        {   // Error from LockServiceDatabase ().
            CloseServiceHandle (hServiceControlManager) ;
            char zBuf[255] ;
            FormatMessage (FORMAT_MESSAGE_FROM_SYSTEM, NULL, GetLastError (), MAKELANGID (LANG_NEUTRAL, SUBLANG_DEFAULT), (LPTSTR) zBuf, sizeof (zBuf), NULL) ;
            CharToOem (zBuf, zBuf) ;
            throw CTServiceException (string ("service: LockServiceDatabase failed: ") + zBuf, __FILE__, __LINE__, GetLastError () ) ;
        }

        // Is another process starting the service ?
        if (hServiceControlLocked)
        {
            // Unlock the control services database.
            if (!UnlockServiceDatabase (hServiceControlLocked) )
            {   // Error from UnlockServiceDatabase ().
                CloseServiceHandle (hServiceControlManager) ;
                char zBuf[255] ;
                FormatMessage (FORMAT_MESSAGE_FROM_SYSTEM, NULL, GetLastError (), MAKELANGID (LANG_NEUTRAL, SUBLANG_DEFAULT), (LPTSTR) zBuf, sizeof (zBuf), NULL) ;
                CharToOem (zBuf, zBuf) ;
                throw CTServiceException (string ("service: UnlockServiceDatabase failed: ") + zBuf, __FILE__, __LINE__, GetLastError () ) ;
            }

            // Open the service.
            SC_HANDLE hService = OpenService (hServiceControlManager, TEXT (serviceName_.c_str () ), SERVICE_ALL_ACCESS) ;
            if (hService == NULL)
            {   // Error from OpenService ().
                CloseServiceHandle (hServiceControlManager) ;
                char zBuf[255] ;
                FormatMessage (FORMAT_MESSAGE_FROM_SYSTEM, NULL, GetLastError (), MAKELANGID (LANG_NEUTRAL, SUBLANG_DEFAULT), (LPTSTR) zBuf, sizeof (zBuf), NULL) ;
                CharToOem (zBuf, zBuf) ;
                throw CTServiceException (string ("service: OpenService failed: ") + zBuf, __FILE__, __LINE__, GetLastError () ) ;
            }

            // Start the service.
            if (!StartService (hService, 0, NULL) )
            {   // Error from StartService ().
                CloseServiceHandle (hService) ;
                CloseServiceHandle (hServiceControlManager) ;
                char zBuf[255] ;
                FormatMessage (FORMAT_MESSAGE_FROM_SYSTEM, NULL, GetLastError (), MAKELANGID (LANG_NEUTRAL, SUBLANG_DEFAULT), (LPTSTR) zBuf, sizeof (zBuf), NULL) ;
                CharToOem (zBuf, zBuf) ;
                throw CTServiceException (string ("service: StartService failed: ") + zBuf, __FILE__, __LINE__, GetLastError () ) ;
            }
            CloseServiceHandle (hService) ;
            CloseServiceHandle (hServiceControlManager) ;
        }
        else
        {
            CloseServiceHandle (hServiceControlManager) ;

            SERVICE_TABLE_ENTRY serviceTable [] =
                {
                    { (char*) TEXT (serviceName_.c_str () ), (LPSERVICE_MAIN_FUNCTION) serviceMainFunction},
                    {NULL, NULL}
                } ;

            // Save the this pointer.
            GpProcessServiceInfo = this ;

            // Try to connect to the service control manager.
            if (!StartServiceCtrlDispatcher (serviceTable) )
            {   // Error from StartServiceCtrlDispatcher ().
                CloseServiceHandle (hServiceControlManager) ;
                char zBuf[255] ;
                FormatMessage (FORMAT_MESSAGE_FROM_SYSTEM, NULL, GetLastError (), MAKELANGID (LANG_NEUTRAL, SUBLANG_DEFAULT), (LPTSTR) zBuf, sizeof (zBuf), NULL) ;
                CharToOem (zBuf, zBuf) ;
                throw CTServiceException (string ("service: StartServiceCtrlDispatcher failed: ") + zBuf, __FILE__, __LINE__, GetLastError () ) ;
            }
        }
    }
    else
    {
        throw CTServiceException ("service: services not supported by Windows 95 and Win32s.", __FILE__, __LINE__) ;
    }
#endif  // WIN32
#ifdef  UNIX
    // Fork the process.
    pid_t pidChild = fork () ;
    if (pidChild == -1)
    {   // Error from fork ().
        throw CTServiceException (string ("service: fork failed: ") + strerror (errno), __FILE__, __LINE__, errno) ;
    }
    else if (pidChild == 0)
    {   // Child process for "xxx -daemon".
        setpgrp () ;
        if (signal (SIGHUP, SIG_IGN) == SIG_ERR)
        {   // Error from signal ().
            throw CTServiceException (string ("service: signal failed: ") + strerror (errno), __FILE__, __LINE__, errno) ;
        }
        if (signal (SIGQUIT, SIG_IGN) == SIG_ERR)
        {   // Error from signal ().
            throw CTServiceException (string ("service: signal failed: ") + strerror (errno), __FILE__, __LINE__, errno) ;
        }
        if (signal (SIGTERM, serviceControlFunction) == SIG_ERR)
        {   // Error from signal ().
            throw CTServiceException (string ("service: signal failed: ") + strerror (errno), __FILE__, __LINE__, errno) ;
        }
        if (signal (SIGUSR1, serviceControlFunction) == SIG_ERR)
        {   // Error from signal ().
            throw CTServiceException (string ("service: signal failed: ") + strerror (errno), __FILE__, __LINE__, errno) ;
        }
        if (signal (SIGCONT, serviceControlFunction) == SIG_ERR)
        {   // Error from signal ().
            throw CTServiceException (string ("service: signal failed: ") + strerror (errno), __FILE__, __LINE__, errno) ;
        }

        // Save the this pointer.
        GpProcessServiceInfo = this ;

        // Run the service.
        console () ;
    }
#endif  // UNIX
} // CTServiceInfo::service

//-----------------------------------------------------------------------------
// CTServiceInfo::install
//
// Install the service.
//
// Arguments : none.
// Return    : none.
//-----------------------------------------------------------------------------

void CTServiceInfo::install (void)
{
#ifdef  WIN32
    if (osVersionInfo.dwPlatformId == VER_PLATFORM_WIN32_NT)
    {
        SC_HANDLE   hService ;
        SC_HANDLE   hServiceControlManager ;
        char        szPath[512] ;

        // Get the module name.
        if (!GetModuleFileName (NULL, szPath, sizeof (szPath) ) )
        {   // Error from GetModuleFileName ().
            char zBuf[255] ;
            FormatMessage (FORMAT_MESSAGE_FROM_SYSTEM, NULL, GetLastError (), MAKELANGID (LANG_NEUTRAL, SUBLANG_DEFAULT), (LPTSTR) zBuf, sizeof (zBuf), NULL) ;
            CharToOem (zBuf, zBuf) ;
            throw CTServiceException (string ("install: DuplicateHandle failed: ") + zBuf, __FILE__, __LINE__, GetLastError () ) ;
        }

        // Connect to the service control manager.
        hServiceControlManager = OpenSCManager (NULL, NULL, SC_MANAGER_ALL_ACCESS) ;
        if (hServiceControlManager == NULL)
        {   // Error from OpenSCManager ().
            char zBuf[255] ;
            FormatMessage (FORMAT_MESSAGE_FROM_SYSTEM, NULL, GetLastError (), MAKELANGID (LANG_NEUTRAL, SUBLANG_DEFAULT), (LPTSTR) zBuf, sizeof (zBuf), NULL) ;
            CharToOem (zBuf, zBuf) ;
            throw CTServiceException (string ("install: OpenSCManager failed: ") + zBuf, __FILE__, __LINE__, GetLastError () ) ;
        }

        // Create the service.
        hService = CreateService (hServiceControlManager, TEXT (serviceName_.c_str () ), displayName_.c_str (), SERVICE_ALL_ACCESS, SERVICE_WIN32_OWN_PROCESS, SERVICE_DEMAND_START, SERVICE_ERROR_NORMAL, szPath, NULL, NULL, NULL, NULL, NULL) ;
        if (hService == NULL)
        {   // Error from CreateService ().
            CloseServiceHandle (hServiceControlManager) ;
            char zBuf[255] ;
            FormatMessage (FORMAT_MESSAGE_FROM_SYSTEM, NULL, GetLastError (), MAKELANGID (LANG_NEUTRAL, SUBLANG_DEFAULT), (LPTSTR) zBuf, sizeof (zBuf), NULL) ;
            CharToOem (zBuf, zBuf) ;
            throw CTServiceException (string ("install: CreateService failed: ") + zBuf, __FILE__, __LINE__, GetLastError () ) ;
        }
        CloseServiceHandle (hService) ;
        CloseServiceHandle (hServiceControlManager) ;
    }
    else
    {
        throw CTServiceException ("install: services not supported by Windows 95 and Win32s.", __FILE__, __LINE__) ;
    }
#endif  // WIN32
} // CTServiceInfo::install

//-----------------------------------------------------------------------------
// CTServiceInfo::uninstall
//
// Uninstall the service.
//
// Arguments : none.
// Return    : none.
//-----------------------------------------------------------------------------

void CTServiceInfo::uninstall (void)
{
#ifdef  WIN32
    if (osVersionInfo.dwPlatformId == VER_PLATFORM_WIN32_NT)
    {
        SC_HANDLE   hService ;
        SC_HANDLE   hServiceControlManager ;

        // Connect to the service control manager.
        hServiceControlManager = OpenSCManager (NULL, NULL, SC_MANAGER_ALL_ACCESS) ;
        if (hServiceControlManager == NULL)
        {   // Error from OpenSCManager ().
            char zBuf[255] ;
            FormatMessage (FORMAT_MESSAGE_FROM_SYSTEM, NULL, GetLastError (), MAKELANGID (LANG_NEUTRAL, SUBLANG_DEFAULT), (LPTSTR) zBuf, sizeof (zBuf), NULL) ;
            CharToOem (zBuf, zBuf) ;
            throw CTServiceException (string ("uninstall: OpenSCManager failed: ") + zBuf, __FILE__, __LINE__, GetLastError () ) ;
        }

        // Open the service.
        hService = OpenService (hServiceControlManager, TEXT (serviceName_.c_str () ), SERVICE_ALL_ACCESS) ;
        if (hService == NULL)
        {   // Error from OpenService ().
            CloseServiceHandle (hServiceControlManager) ;
            char zBuf[255] ;
            FormatMessage (FORMAT_MESSAGE_FROM_SYSTEM, NULL, GetLastError (), MAKELANGID (LANG_NEUTRAL, SUBLANG_DEFAULT), (LPTSTR) zBuf, sizeof (zBuf), NULL) ;
            CharToOem (zBuf, zBuf) ;
            throw CTServiceException (string ("uninstall: OpenService failed: ") + zBuf, __FILE__, __LINE__, GetLastError () ) ;
        }

        // Get service status.
        if (!QueryServiceStatus (hService, &serviceStatus_) )
        {   // Error from QueryServiceStatus ().
            CloseServiceHandle (hService) ;
            CloseServiceHandle (hServiceControlManager) ;
            char zBuf[255] ;
            FormatMessage (FORMAT_MESSAGE_FROM_SYSTEM, NULL, GetLastError (), MAKELANGID (LANG_NEUTRAL, SUBLANG_DEFAULT), (LPTSTR) zBuf, sizeof (zBuf), NULL) ;
            CharToOem (zBuf, zBuf) ;
            throw CTServiceException (string ("uninstall: QueryServiceStatus failed: ") + zBuf, __FILE__, __LINE__, GetLastError () ) ;
        }

        if (serviceStatus_.dwCurrentState != SERVICE_STOPPED)
        {   // Try to stop the service.
            if (!ControlService (hService, SERVICE_CONTROL_STOP, &serviceStatus_) )
            {   // Error from ControlService ().
                CloseServiceHandle (hService) ;
                CloseServiceHandle (hServiceControlManager) ;
                char zBuf[255] ;
                FormatMessage (FORMAT_MESSAGE_FROM_SYSTEM, NULL, GetLastError (), MAKELANGID (LANG_NEUTRAL, SUBLANG_DEFAULT), (LPTSTR) zBuf, sizeof (zBuf), NULL) ;
                CharToOem (zBuf, zBuf) ;
                throw CTServiceException (string ("uninstall: ControlService failed: ") + zBuf, __FILE__, __LINE__, GetLastError () ) ;
            }

            // Wait for the service to stop.
            CTCurrentThread::sleep (1000) ;
            while (QueryServiceStatus (hService, &serviceStatus_) )
            {
                if (serviceStatus_.dwCurrentState != SERVICE_STOP_PENDING)
                {
                    break ;
                }
                CTCurrentThread::sleep (1000) ;
            }

            // Verify the service is stopped.
            if (serviceStatus_.dwCurrentState != SERVICE_STOPPED)
            {   // Error from serviceStatus_.dwCurrentState.
                CloseServiceHandle (hService) ;
                CloseServiceHandle (hServiceControlManager) ;
                throw CTServiceException ("uninstall: service failed to stop.", __FILE__, __LINE__) ;
            }
        }

        // Now remove the service.
        if (!DeleteService (hService) )
        {   // Error from DeleteService ().
            CloseServiceHandle (hService) ;
            CloseServiceHandle (hServiceControlManager) ;
            char zBuf[255] ;
            FormatMessage (FORMAT_MESSAGE_FROM_SYSTEM, NULL, GetLastError (), MAKELANGID (LANG_NEUTRAL, SUBLANG_DEFAULT), (LPTSTR) zBuf, sizeof (zBuf), NULL) ;
            CharToOem (zBuf, zBuf) ;
            throw CTServiceException (string ("uninstall: DeleteService failed: ") + zBuf, __FILE__, __LINE__, GetLastError () ) ;
        }

        CloseServiceHandle (hService) ;
        CloseServiceHandle (hServiceControlManager) ;
    }
    else
    {
        throw CTServiceException ("uninstall: services not supported by Windows 95 and Win32s.", __FILE__, __LINE__) ;
    }
#endif  // WIN32
} // CTServiceInfo::uninstall

//-----------------------------------------------------------------------------
// CTServiceInfo::control
//
// Control the service.
//
// Arguments : eServiceControl : control command to send.
// Return    : none.
//-----------------------------------------------------------------------------

void CTServiceInfo::control (CTService::enumServiceControl eServiceControl)
{
#ifdef  WIN32
    if (osVersionInfo.dwPlatformId == VER_PLATFORM_WIN32_NT)
    {
        // Connect to the service control manager.
        SC_HANDLE hServiceControlManager = OpenSCManager (NULL, NULL, SC_MANAGER_ALL_ACCESS) ;
        if (hServiceControlManager == NULL)
        {   // Error from OpenSCManager ().
            char zBuf[255] ;
            FormatMessage (FORMAT_MESSAGE_FROM_SYSTEM, NULL, GetLastError (), MAKELANGID (LANG_NEUTRAL, SUBLANG_DEFAULT), (LPTSTR) zBuf, sizeof (zBuf), NULL) ;
            CharToOem (zBuf, zBuf) ;
            throw CTServiceException (string ("control: OpenSCManager failed: ") + zBuf, __FILE__, __LINE__, GetLastError () ) ;
        }

        // Open the service.
        SC_HANDLE hService = OpenService (hServiceControlManager, TEXT (serviceName_.c_str () ), SERVICE_ALL_ACCESS) ;
        if (hService == NULL)
        {   // Error from OpenService ().
            CloseServiceHandle (hServiceControlManager) ;
            char zBuf[255] ;
            FormatMessage (FORMAT_MESSAGE_FROM_SYSTEM, NULL, GetLastError (), MAKELANGID (LANG_NEUTRAL, SUBLANG_DEFAULT), (LPTSTR) zBuf, sizeof (zBuf), NULL) ;
            CharToOem (zBuf, zBuf) ;
            throw CTServiceException (string ("control: OpenService failed: ") + zBuf, __FILE__, __LINE__, GetLastError () ) ;
        }

        // Control the service.
        DWORD   dwControl ;
        DWORD   dwCurrentState ;
        switch (eServiceControl)
        {
        case CTService::SRVCTRL_STOP :
            dwControl = SERVICE_CONTROL_STOP ;
            dwCurrentState = SERVICE_STOPPED ;
            break ;
        case CTService::SRVCTRL_PAUSE :
            dwControl = SERVICE_CONTROL_PAUSE ;
            dwCurrentState = SERVICE_PAUSED ;
            break ;
        case CTService::SRVCTRL_CONTINUE :
            dwControl = SERVICE_CONTROL_CONTINUE ;
            dwCurrentState = SERVICE_RUNNING ;
            break ;
        default :
            dwControl = SERVICE_CONTROL_CONTINUE ;
            dwCurrentState = SERVICE_RUNNING ;
        }
        if (!ControlService (hService, dwControl, &serviceStatus_) )
        {   // Error from ControlService ().
            CloseServiceHandle (hService) ;
            CloseServiceHandle (hServiceControlManager) ;
            char zBuf[255] ;
            FormatMessage (FORMAT_MESSAGE_FROM_SYSTEM, NULL, GetLastError (), MAKELANGID (LANG_NEUTRAL, SUBLANG_DEFAULT), (LPTSTR) zBuf, sizeof (zBuf), NULL) ;
            CharToOem (zBuf, zBuf) ;
            throw CTServiceException (string ("control: ControlService failed: ") + zBuf, __FILE__, __LINE__, GetLastError () ) ;
        }

        // Waiting for the end of the service.
        while (serviceStatus_.dwCurrentState != dwCurrentState)
        {
            // Wait for the service to control.
            CTCurrentThread::sleep (1000) ;

            if (!QueryServiceStatus (hService, &serviceStatus_) )
            {   // Error from QueryServiceStatus ().
                CloseServiceHandle (hService) ;
                CloseServiceHandle (hServiceControlManager) ;
                char zBuf[255] ;
                FormatMessage (FORMAT_MESSAGE_FROM_SYSTEM, NULL, GetLastError (), MAKELANGID (LANG_NEUTRAL, SUBLANG_DEFAULT), (LPTSTR) zBuf, sizeof (zBuf), NULL) ;
                CharToOem (zBuf, zBuf) ;
                throw CTServiceException (string ("control: QueryServiceStatus failed: ") + zBuf, __FILE__, __LINE__, GetLastError () ) ;
            }
        }

        CloseServiceHandle (hService) ;
        CloseServiceHandle (hServiceControlManager) ;
    }
    else
    {
        throw CTServiceException ("control: services not supported by Windows 95 and Win32s.", __FILE__, __LINE__) ;
    }
#endif  // WIN32
#ifdef  UNIX
    // Create pipe files.
    int iPipeFds[2] ;
    if (pipe (iPipeFds) == -1)
    {   // Error from pipe ().
        throw CTServiceException (string ("control: pipe failed: ") + strerror (errno), __FILE__, __LINE__, errno) ;
    }

    // Fork the process.
    pid_t pidChild = fork () ;
    if (pidChild == -1)
    {   // Error from fork ().
        throw CTServiceException (string ("control: fork failed: ") + strerror (errno), __FILE__, __LINE__, errno) ;
    }
    else if (pidChild == 0)
    {   // Child process for "ps -ef".

        // Close the standard output.
        if (close (1) == -1)
        {   // Error from close ().
            throw CTServiceException (string ("control: close failed: ") + strerror (errno), __FILE__, __LINE__, errno) ;
        }

        // Duplicate the pipe file 2 to stdout.
        if (dup (iPipeFds[1]) == -1)
        {   // Error from dup ().
            throw CTServiceException (string ("control: dup failed: ") + strerror (errno), __FILE__, __LINE__, errno) ;
        }

        // Close the pipe file 1.
        if (close (iPipeFds[0]) == -1)
        {   // Error from close ().
            throw CTServiceException (string ("control: close failed: ") + strerror (errno), __FILE__, __LINE__, errno) ;
        }

        // Execute the "ps" command in the child process memory space.
        if (execlp ("ps", "ps", "-ef", (char*) 0) == -1)
        {   // Error from execlp ().
            throw CTServiceException (string ("control: execlp failed: ") + strerror (errno), __FILE__, __LINE__, errno) ;
        }
    }
    else
    {   // Parent process for service control.

        // Close the standard input.
        if (close (0) == -1)
        {   // Error from close ().
            throw CTServiceException (string ("control: close failed: ") + strerror (errno), __FILE__, __LINE__, errno) ;
        }

        // Duplicate the pipe file 1 to stdin.
        if (dup (iPipeFds[0]) == -1)
        {   // Error from dup ().
            throw CTServiceException (string ("control: dup failed: ") + strerror (errno), __FILE__, __LINE__, errno) ;
        }

        // Close the pipe file 2.
        if (close (iPipeFds[1]) == -1)
        {   // Error from close ().
            throw CTServiceException (string ("control: close failed: ") + strerror (errno), __FILE__, __LINE__, errno) ;
        }

        // Get the service process identifier.
        string  poub ;
        pid_t       pidService = 0 ;
        if (!cin.eof () )
        {
            poub.readLine (cin) ;
            while (!cin.eof () )
            {
                cin >> poub ;
                cin >> pidService ;
                poub.readLine (cin) ;
                if ( (poub.contains (programName_) ) && (pidService != getpid () ) && (pidService != pidChild) )
                {
                    break ;
                }
                pidService = 0 ;
            }
        }

        // Verify the service process identifier is correct.
        if (pidService == 0)
        {   // Error from pidService.
            throw CTServiceException ("control: service is not running.", __FILE__, __LINE__) ;
        }

        // Control the service.
        int iSig ;
        switch (eServiceControl)
        {
        case CTService::SRVCTRL_STOP :
            iSig = SIGTERM ;
            break ;
        case CTService::SRVCTRL_PAUSE :
            iSig = SIGUSR1 ;
            break ;
        case CTService::SRVCTRL_CONTINUE :
            iSig = SIGCONT ;
            break ;
        }

        // Send the signal to the service process.
        if (kill (pidService, iSig) == -1)
        {   // Error from kill ().
            throw CTServiceException (string ("control: kill failed: ") + strerror (errno), __FILE__, __LINE__, errno) ;
        }
    }
#endif  // UNIX
} // CTServiceInfo::control

//-----------------------------------------------------------------------------
// CTServiceInfo::isInstalled
//
// Is the service installed ?
//
// Arguments : none.
// Return    : bInstalled : service installed or not.
//-----------------------------------------------------------------------------

bool CTServiceInfo::isInstalled (void)
{
    bool    bInstalled ;
#ifdef  WIN32
    if (osVersionInfo.dwPlatformId == VER_PLATFORM_WIN32_NT)
    {
        SC_HANDLE   hService ;
        SC_HANDLE   hServiceControlManager ;

        // Connect to the service control manager.
        hServiceControlManager = OpenSCManager (NULL, NULL, SC_MANAGER_ALL_ACCESS) ;
        if (hServiceControlManager == NULL)
        {   // Error from OpenSCManager ().
            char zBuf[255] ;
            FormatMessage (FORMAT_MESSAGE_FROM_SYSTEM, NULL, GetLastError (), MAKELANGID (LANG_NEUTRAL, SUBLANG_DEFAULT), (LPTSTR) zBuf, sizeof (zBuf), NULL) ;
            CharToOem (zBuf, zBuf) ;
            throw CTServiceException (string ("isInstalled: OpenSCManager failed: ") + zBuf, __FILE__, __LINE__, GetLastError () ) ;
        }

        // Open the service.
        hService = OpenService (hServiceControlManager, TEXT (serviceName_.c_str () ), SERVICE_ALL_ACCESS) ;
        bInstalled = (hService != NULL) ;
        if (hService)
        {
            CloseServiceHandle (hService) ;
        }

        CloseServiceHandle (hServiceControlManager) ;
    }
    else
    {
        bInstalled = false ;
    }
#endif  // WIN32
#ifdef  UNIX
    bInstalled = false ;
#endif  // UNIX

    return bInstalled ;
} // CTServiceInfo::isInstalled

#ifdef  WIN32
//-----------------------------------------------------------------------------
// CTServiceInfo::mainService
//
// Windows service main function.
//
// Arguments : none.
// Return    : none.
//-----------------------------------------------------------------------------
void CTServiceInfo::mainService (void)
{
    // Register the service control routine.
    hServiceStatus_ = RegisterServiceCtrlHandler (TEXT (serviceName_.c_str () ), (LPHANDLER_FUNCTION) serviceControlFunction) ;
    if (hServiceStatus_ == NULL)
    {   // Error from RegisterServiceCtrlHandler ().
        char zBuf[255] ;
        FormatMessage (FORMAT_MESSAGE_FROM_SYSTEM, NULL, GetLastError (), MAKELANGID (LANG_NEUTRAL, SUBLANG_DEFAULT), (LPTSTR) zBuf, sizeof (zBuf), NULL) ;
        CharToOem (zBuf, zBuf) ;
        throw CTServiceException (string ("mainService: RegisterServiceCtrlHandler failed: ") + zBuf, __FILE__, __LINE__, GetLastError () ) ;
    }

    // Tell the control services manager that we are initializing the service
    memset (&serviceStatus_, 0, sizeof (SERVICE_STATUS) ) ;
    serviceStatus_.dwServiceType = SERVICE_WIN32_OWN_PROCESS ;
    serviceStatus_.dwControlsAccepted = 0 ;
    serviceStatus_.dwCurrentState = SERVICE_START_PENDING ;
    serviceStatus_.dwWin32ExitCode = NO_ERROR ;
    serviceStatus_.dwServiceSpecificExitCode = 0 ;
    serviceStatus_.dwCheckPoint = 1 ;
    serviceStatus_.dwWaitHint = 5000 ;
    if (!SetServiceStatus (hServiceStatus_, &serviceStatus_) )
    {   // Error from SetServiceStatus ().
        char zBuf[255] ;
        FormatMessage (FORMAT_MESSAGE_FROM_SYSTEM, NULL, GetLastError (), MAKELANGID (LANG_NEUTRAL, SUBLANG_DEFAULT), (LPTSTR) zBuf, sizeof (zBuf), NULL) ;
        CharToOem (zBuf, zBuf) ;
        throw CTServiceException (string ("mainService: SetServiceStatus failed: ") + zBuf, __FILE__, __LINE__, GetLastError () ) ;
    }

    try
    {
        // Initialize the service.
        service_.init () ;
    }
    catch (CTServiceException& e)
    {
        cerr << e << endl ;
    }

    // Tell to the control services manager that initialization is terminated.
    serviceStatus_.dwControlsAccepted = SERVICE_ACCEPT_STOP | SERVICE_ACCEPT_SHUTDOWN | SERVICE_ACCEPT_PAUSE_CONTINUE ;
    serviceStatus_.dwCurrentState = SERVICE_RUNNING ;
    serviceStatus_.dwWin32ExitCode = NO_ERROR ;
    serviceStatus_.dwCheckPoint = 0 ;
    serviceStatus_.dwWaitHint = 0 ;
    if (!SetServiceStatus (hServiceStatus_, &serviceStatus_) )
    {   // Error from SetServiceStatus ().
        char zBuf[255] ;
        FormatMessage (FORMAT_MESSAGE_FROM_SYSTEM, NULL, GetLastError (), MAKELANGID (LANG_NEUTRAL, SUBLANG_DEFAULT), (LPTSTR) zBuf, sizeof (zBuf), NULL) ;
        CharToOem (zBuf, zBuf) ;
        throw CTServiceException (string ("mainService: SetServiceStatus failed: ") + zBuf, __FILE__, __LINE__, GetLastError () ) ;
    }

    try
    {
        // Execute the service.
        service_.main () ;

        // Exit the service.
        service_.exit () ;
    }
    catch (CTServiceException& e)
    {
        cerr << e << endl ;
    }

    // Tell control services manager that the service is terminated.
    if ( (serviceStatus_.dwCurrentState == SERVICE_RUNNING) || (serviceStatus_.dwCurrentState == SERVICE_STOP_PENDING) )
    {
        serviceStatus_.dwCurrentState = SERVICE_STOPPED ;
        serviceStatus_.dwWin32ExitCode = NO_ERROR ;
        serviceStatus_.dwCheckPoint = 0 ;
        serviceStatus_.dwWaitHint = 0 ;
        if (!SetServiceStatus (hServiceStatus_, &serviceStatus_) )
        {   // Error from SetServiceStatus ().
            char zBuf[255] ;
            FormatMessage (FORMAT_MESSAGE_FROM_SYSTEM, NULL, GetLastError (), MAKELANGID (LANG_NEUTRAL, SUBLANG_DEFAULT), (LPTSTR) zBuf, sizeof (zBuf), NULL) ;
            CharToOem (zBuf, zBuf) ;
            throw CTServiceException (string ("mainService: SetServiceStatus failed: ") + zBuf, __FILE__, __LINE__, GetLastError () ) ;
        }
    }
} // CTServiceInfo::mainService

//-----------------------------------------------------------------------------
// CTServiceInfo::controlService
//
// Windows service control function.
//
// Arguments : dwCommand : Command to do.
// Return    : none.
//-----------------------------------------------------------------------------
void CTServiceInfo::controlService (DWORD dwCommand)
{
    switch (dwCommand)
    {
    case SERVICE_CONSOLE_STOP :
        service_.controlNotify (CTService::SRVCTRL_STOP) ;
        break ;
    case SERVICE_CONTROL_STOP :
    case SERVICE_CONTROL_SHUTDOWN :
        // Shutdown or stop the service.
        serviceStatus_.dwCurrentState = SERVICE_STOP_PENDING ;
        serviceStatus_.dwWin32ExitCode = NO_ERROR ;
        serviceStatus_.dwCheckPoint = 1 ;
        serviceStatus_.dwWaitHint = 5000 ;
        if (!SetServiceStatus (hServiceStatus_, &serviceStatus_) )
        {   // Error from SetServiceStatus ().
            char zBuf[255] ;
            FormatMessage (FORMAT_MESSAGE_FROM_SYSTEM, NULL, GetLastError (), MAKELANGID (LANG_NEUTRAL, SUBLANG_DEFAULT), (LPTSTR) zBuf, sizeof (zBuf), NULL) ;
            CharToOem (zBuf, zBuf) ;
            throw CTServiceException (string ("controlService: SetServiceStatus failed: ") + zBuf, __FILE__, __LINE__, GetLastError () ) ;
        }
        service_.controlNotify (CTService::SRVCTRL_STOP) ;
        ++ serviceStatus_.dwCheckPoint ;
        break ;
    case SERVICE_CONTROL_PAUSE :
        // Suspend the service.
        serviceStatus_.dwCurrentState = SERVICE_PAUSE_PENDING ;
        serviceStatus_.dwWin32ExitCode = NO_ERROR ;
        serviceStatus_.dwCheckPoint = 1 ;
        serviceStatus_.dwWaitHint = 5000 ;
        if (!SetServiceStatus (hServiceStatus_, &serviceStatus_) )
        {   // Error from SetServiceStatus ().
            char zBuf[255] ;
            FormatMessage (FORMAT_MESSAGE_FROM_SYSTEM, NULL, GetLastError (), MAKELANGID (LANG_NEUTRAL, SUBLANG_DEFAULT), (LPTSTR) zBuf, sizeof (zBuf), NULL) ;
            CharToOem (zBuf, zBuf) ;
            throw CTServiceException (string ("controlService: SetServiceStatus failed: ") + zBuf, __FILE__, __LINE__, GetLastError () ) ;
        }
        service_.controlNotify (CTService::SRVCTRL_PAUSE) ;
        serviceStatus_.dwCurrentState = SERVICE_PAUSED ;
        serviceStatus_.dwCheckPoint = 0 ;
        break ;
    case SERVICE_CONTROL_CONTINUE :
        // Resume the service.
        serviceStatus_.dwCurrentState = SERVICE_CONTINUE_PENDING ;
        serviceStatus_.dwWin32ExitCode = NO_ERROR ;
        serviceStatus_.dwCheckPoint = 1 ;
        serviceStatus_.dwWaitHint = 5000 ;
        if (!SetServiceStatus (hServiceStatus_, &serviceStatus_) )
        {   // Error from SetServiceStatus ().
            char zBuf[255] ;
            FormatMessage (FORMAT_MESSAGE_FROM_SYSTEM, NULL, GetLastError (), MAKELANGID (LANG_NEUTRAL, SUBLANG_DEFAULT), (LPTSTR) zBuf, sizeof (zBuf), NULL) ;
            CharToOem (zBuf, zBuf) ;
            throw CTServiceException (string ("controlService: SetServiceStatus failed: ") + zBuf, __FILE__, __LINE__, GetLastError () ) ;
        }
        service_.controlNotify (CTService::SRVCTRL_CONTINUE) ;
        serviceStatus_.dwCurrentState = SERVICE_RUNNING ;
        serviceStatus_.dwCheckPoint = 0 ;
        break ;
    case SERVICE_CONTROL_INTERROGATE :
        if (serviceStatus_.dwCheckPoint > 0)
        {
            ++ serviceStatus_.dwCheckPoint ;
        }
        break ;
    default :
        break ;
    }

    if ( (dwCommand != SERVICE_CONSOLE_STOP) && (!SetServiceStatus (hServiceStatus_, &serviceStatus_) ) )
    {   // Error from SetServiceStatus ().
        char zBuf[255] ;
        FormatMessage (FORMAT_MESSAGE_FROM_SYSTEM, NULL, GetLastError (), MAKELANGID (LANG_NEUTRAL, SUBLANG_DEFAULT), (LPTSTR) zBuf, sizeof (zBuf), NULL) ;
        CharToOem (zBuf, zBuf) ;
        throw CTServiceException (string ("controlService: SetServiceStatus failed: ") + zBuf, __FILE__, __LINE__, GetLastError () ) ;
    }
} // CTServiceInfo::controlService
#endif  // WIN32
#ifdef  UNIX
//-----------------------------------------------------------------------------
// CTServiceInfo::controlService
//
// Unix service control function.
//
// Arguments : iSig : signal caught.
// Return    : none.
//-----------------------------------------------------------------------------
void CTServiceInfo::controlService (int iSig)
{
    switch (iSig)
    {
    case SIGTERM :
        service_.controlNotify (CTService::SRVCTRL_STOP) ;
        break ;
    case SIGUSR1 :
        service_.controlNotify (CTService::SRVCTRL_PAUSE) ;
        raise (SIGSTOP) ;
        break ;
    case SIGCONT :
        service_.controlNotify (CTService::SRVCTRL_CONTINUE) ;
        break ;
    }
} // CTServiceInfo::controlService
#endif  // UNIX

///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////// End of Class CTServiceInfo //
///////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////// Class CTService //
///////////////////////////////////////////////////////////////////////////////
// Class used to create and control a service.

//=====================================================================< Data >
const char* const CTService::zStartModeLabel_[3] =
#ifdef  WIN32
    { "unknow", "console", "service"
    } ;
#endif  // WIN32
#ifdef  UNIX
{ "unknow", "console", "daemon" } ;
#endif  // UNIX


//============================================< Constructor(s) and destructor >
//-----------------------------------------------------------------------------
// CTService::CTService
//
// Constructor.
//
// Arguments : programName :        name of the program.
//             serviceDisplayName : name of the service.
// Return    : none.
//-----------------------------------------------------------------------------
CTService::CTService (const string& programName, const string& serviceDisplayName)
        : pServiceInfo_ (NULL), eStartMethode_ (START_UNDEFINED)
{
    // Create the service object.
    try
    {
        pServiceInfo_ = (void*) new CTServiceInfo (programName, serviceDisplayName, *this) ;
    }
    catch (CTxalloc&)
    {
        throw CTServiceException ("CTService: new CTServiceInfo (programName, serviceDisplayName, *this) failed.", __FILE__, __LINE__) ;
    }
} // CTService::CTService

//-----------------------------------------------------------------------------
// CTService::~CTService
//
// Destructor.
//
// Arguments : none.
// Return    : none.
//-----------------------------------------------------------------------------
CTService::~CTService ()
{
    if (pServiceInfo_)
    {
        // Delete the service object.
        delete (CTServiceInfo*) pServiceInfo_ ;
        pServiceInfo_ = NULL ;
    }
} // CTService::~CTService

//=========================================================< Control methods >
//-----------------------------------------------------------------------------
// CTService::start
//
// Start the service.
//
// Arguments : eStartMethode : start service as service or as console.
// Return    : none.
//-----------------------------------------------------------------------------
void CTService::start (const enumStartMethode eStartMethode)
{
    // Verify the service object is correct.
    if (pServiceInfo_ == NULL)
    {   // Error from pServiceInfo_.
        throw CTServiceException ("start: pServiceInfo_ is NULL.", __FILE__, __LINE__) ;
    }

    // Get a reference to service object.
    CTServiceInfo&  serviceInfo = * ( (CTServiceInfo*) pServiceInfo_) ;

    // Start the service object.
    switch (eStartMethode)
    {
    case START_UNDEFINED :
        if (getStartMode () == START_SERVICE)
        {   // Start the service object as service.
            serviceInfo.service () ;
        }
        else
        {   // Start the service object as console.
            serviceInfo.console () ;
        }
        break ;
    case START_SERVICE :
        // Start the service object as service.
        serviceInfo.service () ;
        break ;
    case START_CONSOLE :
        // Start the service object as console.
        serviceInfo.console () ;
        break ;
    }
} // CTService::start

//-----------------------------------------------------------------------------
// CTService::install
//
// Install the service.
//
// Arguments : none.
// Return    : none.
//-----------------------------------------------------------------------------
void CTService::install (void)
{
    // Verify the service object is correct.
    if (pServiceInfo_ == NULL)
    {   // Error from pServiceInfo_.
        throw CTServiceException ("install: pServiceInfo_ is NULL.", __FILE__, __LINE__) ;
    }

    // Get a reference to service object.
    CTServiceInfo&  serviceInfo = * ( (CTServiceInfo*) pServiceInfo_) ;

    // Install the service.
    serviceInfo.install () ;
} // CTService::install

//-----------------------------------------------------------------------------
// CTService::uninstall
//
// Uninstall the service.
//
// Arguments : none.
// Return    : none.
//-----------------------------------------------------------------------------
void CTService::uninstall (void)
{
    // Verify the service object is correct.
    if (pServiceInfo_ == NULL)
    {   // Error from pServiceInfo_.
        throw CTServiceException ("install: pServiceInfo_ is NULL.", __FILE__, __LINE__) ;
    }

    // Get a reference to service object.
    CTServiceInfo&  serviceInfo = * ( (CTServiceInfo*) pServiceInfo_) ;

    // Uninstall the service.
    serviceInfo.uninstall () ;
} // CTService::uninstall

//-----------------------------------------------------------------------------
// CTService::control
//
// Control the service.
//
// Arguments : eServiceControl : control command to send.
// Return    : none.
//-----------------------------------------------------------------------------
void CTService::control (enumServiceControl eServiceControl)
{
    // Verify the service object is correct.
    if (pServiceInfo_ == NULL)
    {   // Error from pServiceInfo_.
        throw CTServiceException ("control: pServiceInfo_ is NULL.", __FILE__, __LINE__) ;
    }

    // Get a reference to service object.
    CTServiceInfo&  serviceInfo = * ( (CTServiceInfo*) pServiceInfo_) ;

    // Stop the service object.
    serviceInfo.control (eServiceControl) ;
} // CTService::control

//-----------------------------------------------------------------------------
// CTService::getStartMode
//
// Get the service start mode.
//
// Arguments : none.
// Return    : eStartMethode_ : service start mode.
//-----------------------------------------------------------------------------
CTService::enumStartMethode CTService::getStartMode (void)
{
    // Verify the service object is correct.
    if (pServiceInfo_ == NULL)
    {   // Error from pServiceInfo_.
        throw CTServiceException ("getStartMode: pServiceInfo_ is NULL.", __FILE__, __LINE__) ;
    }

    if (eStartMethode_ == START_UNDEFINED)
    {
        // Get a reference to service object.
        CTServiceInfo&  serviceInfo = * ( (CTServiceInfo*) pServiceInfo_) ;

        // Is the service installed.
        try
        {
            eStartMethode_ = (serviceInfo.isInstalled () ) ? START_SERVICE : START_CONSOLE ;
        }
        catch (CTServiceException&)
        {
            eStartMethode_ = START_CONSOLE ;
        }
    }

    // Return the service start mode.
    return eStartMethode_ ;
} // CTService::getStartMode

///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////// End of Class CTService //
///////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////// Globals Functions //
///////////////////////////////////////////////////////////////////////////////

#ifdef  WIN32
//-----------------------------------------------------------------------------
// serviceMainFunction
//
// Windows service main function.
//
// Arguments : none.
// Return    : none.
//-----------------------------------------------------------------------------
void WINAPI serviceMainFunction (DWORD, LPTSTR *)
{
    try
    {
        if (GpProcessServiceInfo)
        {
            GpProcessServiceInfo->mainService () ;
        }
        else
        {   // Error from GpProcessServiceInfo.
            throw CTServiceException ("serviceMainFunction: GpProcessServiceInfo is NULL.", __FILE__, __LINE__) ;
        }
    }
    catch (CTServiceException& e)
    {
        cerr << e << endl ;
    }
} // serviceMainFunction

//-----------------------------------------------------------------------------
// serviceControlFunction
//
// Windows service control function.
//
// Arguments : dwCommand : Command to do.
// Return    : none.
//-----------------------------------------------------------------------------
void WINAPI serviceControlFunction (DWORD dwCommand)
{
    try
    {
        if (GpProcessServiceInfo)
        {
            GpProcessServiceInfo->controlService (dwCommand) ;
        }
        else
        {   // Error from GpProcessServiceInfo.
            throw CTServiceException ("serviceControlFunction: GpProcessServiceInfo is NULL.", __FILE__, __LINE__) ;
        }
    }
    catch (CTServiceException& e)
    {
        cerr << e << endl ;
    }
} // serviceControlFunction

//-----------------------------------------------------------------------------
// consoleControleHandler
//
// Console control function.
//
// Arguments : DWORD : Event type.
// Return    : TRUE.
//-----------------------------------------------------------------------------
BOOL consoleControleHandler (DWORD)
{
    try
    {
        if (GpProcessServiceInfo)
        {
            if (osVersionInfo.dwPlatformId == VER_PLATFORM_WIN32_NT)
            {
                GpProcessServiceInfo->controlService (SERVICE_CONSOLE_STOP) ;
            }
        }
        else
        {   // Error from GpProcessServiceInfo.
            throw CTServiceException ("consoleControleHandler: GpProcessServiceInfo is NULL.", __FILE__, __LINE__) ;
        }
    }
    catch (CTServiceException& e)
    {
        cerr << e << endl ;
    }
    return TRUE ;
} // consoleControleHandler

#endif  // WIN32
#ifdef  UNIX
//-----------------------------------------------------------------------------
// serviceControlFunction
//
// Unix service control function.
//
// Arguments : iSig : signal caught.
// Return    : none.
//-----------------------------------------------------------------------------
void serviceControlFunction (int iSig)
{
    try
    {
        if (signal (iSig, serviceControlFunction) == SIG_ERR)
        {   // Error from signal ().
            throw CTServiceException (string ("serviceControlFunction: signal failed: ") + strerror (errno), __FILE__, __LINE__, errno) ;
        }
        if (GpProcessServiceInfo)
        {
            GpProcessServiceInfo->controlService (iSig) ;
        }
        else
        {   // Error from GpProcessServiceInfo.
            throw CTServiceException ("serviceControlFunction: GpProcessServiceInfo is NULL.", __FILE__, __LINE__) ;
        }
    }
    catch (CTServiceException& e)
    {
        cerr << e << endl ;
    }
} // serviceControlFunction
#endif  // UNIX

///////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////// End of Globals Functions //
///////////////////////////////////////////////////////////////////////////////
