/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/
/*===========================================================================*/
/*									     */
/*			     **** C O M M O N ****   			     */
/*									     */
/*===========================================================================*/
/*									     */
/*			Module Name: PSL_GET_MESSAGE_UNIX		     */
/*			============					     */
/*									     */
/* Description:								     */
/* ============								     */
/*									     */
/*	Return the message text or message prefix for a given message	     */
/*	inserting the Utility Name if required				     */
/*									     */
/*									     */
/*									     */
/*===========================================================================*/
/*
 *  Include PSL macro definitions
 */
#include <psl_macros.h>

/*
 *  THIS MODULE IS UNIX SPECIFIC AND THEREFORE IS ONLY COMPILED UNDER UNIX
 */
#ifdef UNIX

/*
 *  Include useful standard header files
 */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>		
#include <locale.h>		
#include <nl_types.h>
#include <errno.h>

/*
 *  Include message handling consant definitions and a file defining the
 *  supported Sub-systems and their facilities
 */
#include <pslmsg_prmdef.h>
#include <facilities.h>

/*
 *  Flags for STRADD_MSGTXT().
 */
#define STRADD_COPY 1
#define STRADD_CONCAT 2

/*
 *  Function prototypes.
 */
#include <psl_rtndef.h>                         /* PSL Routines */

/*
 *  The following three function prototypes were required under the version of
 *  the SUN compiler pre SUNPro C 2.0.1. They are left here for reference.
 *
 *  nl_catd catopen(char *name, long oflag);
 *
 *  unsigned long catclose(nl_catd catd);
 *
 *  char *catgets(nl_catd catd,
 *	      long set_num,
 *	      long msg_num,
 *	      char *s);
 */

static Boolean GET_FACILITY(unsigned long vms_fac_num,
			    unsigned long *unix_set_num_ptr,
			    char *sub_system);

static void STRADD_MSGTXT(char *msgbuf,
			  unsigned long bufsiz,
			  char* msgtxt,
			  unsigned long flags);

static nl_catd		msg_desc = (nl_catd)-1;			      
					/* File descriptor for msg file 1 */
static char		cur_subsys[16]; /* Subsystem name of current open */
					/* ... message file */

/*
 *  Global data
 */
static char	utlnam[21];		/* Utility name */
#ifndef VMS_MSG_FORMAT
static char	utlnam_lc[21];		/* Utility name in lower case */
#endif /* VMS_MSG_FORMAT */

/*
 *  Define a macro which says that we should close the message file each
 *  time, after we are finished with it.
 */
#define CLOSING_MSGFILE_EACH_TIME 1           


/****************************************************************************/
/*									    */
/*  PSL_GET_MESSAGE - Get a message from the catfile UNIX message file	    */
/*									    */
/*  Calling Convention:							    */
/*									    */
/*	iret = PSL_GET_MESSAGE (msgnum, msgbuf, buflen, msgflg)		    */
/*									    */
/*  Where:								    */
/*									    */
/*    iret	=   return status					    */
/*    msgnum	=   message number (VMS message number)			    */
/*    msgbuf	=   pointer to message text buffer			    */
/*    buflen	=   length of msgbuf					    */
/*    msgflg	=   flags indicating the required parts of the message	    */
/*									    */
/*    N.B. The message number specified points to a particular facility     */
/*    in a particular sub system. The supported sub systems and facilities  */
/*    are held in the include file facilities.h.                            */
/*									    */
/****************************************************************************/
unsigned long PSL_GET_MESSAGE (
unsigned long	msgnum,
char		*msgbuf,
unsigned long	buflen,
unsigned long	msgflg)
{
/*
 * Local Data,
 */
static char	sub_system[16];		/* Sub system name */
unsigned long	facility;		/* VMS facility number */
unsigned long	msgkey;			/* Low 16 bits of VMS msg number */
unsigned long	status;			/* VMS message status */
char		msg_file[255];		/* Base name of message files */
unsigned long	unix_set;		/* Unix set number */
char		msgtxt[255];		/* Message text */
char		facnam[20];		/* Facility name */
char		msgnam[20];		/* Message symbolic name */
Boolean		statusn;		/* Boolean return status */ /*23X295*/
char		*chrptr;		/* Useful character pointer */
int             i;                      /* Loop variable */

/*
 *  Get the VMS facility number, message number + status and status
 */
    facility = (msgnum >> 16) & 0XFFF;
    msgkey = msgnum & 0X7FFF;
    status = msgnum & 0X7;

/*
 *  Get the unix set number and sub system from the VMS facility number
 */
    statusn = GET_FACILITY(facility, &unix_set, sub_system);
    if (!statusn)
    {
	(void) strcpy(msgtxt, "Facility not found");
	goto error;
    }
/*
 *  Construct the message filename from the subsytem name
 *
 *  This file is:
 *
 *  "<psl_root>/lib/<subsystem>.msgcat"
 */
    PSL_ADDROOT ("/lib/", msg_file, sizeof (msg_file));
                                                /* Start with the PSL root dir link */
    (void) strcat(msg_file, sub_system);        /* Add name of message file */
    (void) strcat(msg_file, ".msgcat");         /* Plus an extension */

/*
 *  Open the message file if it is not already open
 */
    if (strcmp(cur_subsys, sub_system) != 0)	/* Req'd msg file not open */
    {
#ifndef CLOSING_MSGFILE_EACH_TIME
	if (msg_desc != (nl_catd)-1)		/* Previous msg file to close */
	{
    	    if (catclose(msg_desc) != 0)	/* Close & check for error */
	    {
		(void) strcpy(msgtxt, "Error1 closing message file");
		(void) strcat(msgtxt, msg_file);
		(void) strcat(msgtxt, ": ");			      
		(void) strcat(msgtxt, strerror(errno));		  
		goto error;
	    }
	    msg_desc = (nl_catd)-1;				      
	}
#endif
	msg_desc = catopen(msg_file, 0);	/* Open new msg file */
	if (msg_desc == (nl_catd) -1)		/* Error opening msg file */
	{
            (void)sprintf(msgtxt,
                          "Failed to open message catalogue %s - errno = %d",
                          msg_file, errno);
	    goto error;
	}
	(void) strcpy(cur_subsys, sub_system);  /* Remember which msg file is open */
    }


/*
 *  Extract the message name and text.
 */
    chrptr = catgets(msg_desc,			/* Message file 1 descriptor */
		     (long)unix_set,		/* Set number of message */
		     (long)msgkey,		/* Message number to locate */
		     "Message Number");		/* Text if message not found */
    (void) strcpy(msgtxt, chrptr);

    if ((*chrptr) == '\0')                      /* Message not found */
    {
	(void) strcpy(msgtxt, "NOMSG,Message Number");
    }
    else if (strcmp(msgtxt, "Message Number") == 0)
						/* Could not locate msg file */
    {
	(void) strcpy(msgtxt, "Message file ");
	(void) strcat(msgtxt, msg_file);
	(void) strcat(msgtxt, " not available");
	goto error;
    }
    
/*
 *  Search the recovered text for the first occurrance of ','
 *  The message name and text are divided by a ','
 */
    chrptr = (char*)strchr(msgtxt, ',');
    *chrptr = '\0';
    (void) strcpy(msgnam, msgtxt);
    (void) strncpy(msgtxt, (chrptr + 1), 254);
#ifndef VMS_MSG_FORMAT
    for (i = 0;
         (( i < sizeof(msgnam)) && (msgnam[i] != '\0'));
         i++)
    {
        msgnam[i] = tolower(msgnam[i]);
    }
#endif /* !VMS_MSG_FORMAT */

/*
 *  If a message prefix was requested then get the facility name
 */
    if ( (msgflg & PSLMSG_PREFIX) != 0)
    {
        
/*
 *  Get the facility name from the message file
 */
	chrptr = catgets(msg_desc,		/* Message file 2 descripor */
			 (long)unix_set,	/* Set no. of facility name */
			 (long)32767, 		/* Msg no. of facility name */
			 "NONAME");             /* Text if fac name not found */
	(void) strcpy(facnam, chrptr);

	if (strlen(facnam) == 0)                /* Facility name not found */
	{
	    (void) strcpy(facnam, "NONAME");
	}
	else if (strcmp(facnam, "NONAME") == 0)
                                                /* Could not locate msg file */
	{
	    (void) strcpy(msgtxt, "Message file ");
	    (void) strcat(msgtxt, msg_file);
	    (void) strcat(msgtxt, " not available");
	    goto error;
	}
#ifndef VMS_MSG_FORMAT
        for (i = 0;
             ((i < sizeof(facnam)) && (facnam[i] != '\0'));
             i++)
        {
            facnam[i] = tolower(facnam[i]);
        }
#endif /* !VMS_MSG_FORMAT */
    }
/*
 *  N.B.
 *  ====
 *  At this point we would normally close the message file but it is much
 *  more efficient to leave it open as it is likely we will be asked for
 *  another message.
 *
 *  Close the message file
 */
#ifdef CLOSING_MSGFILE_EACH_TIME
    if (catclose(msg_desc) != 0)
    {
	(void) strcpy(msgtxt, "Error2 closing message file");
	(void) strcat(msgtxt, msg_file);
        (void) strcat(msgtxt, ": ");
        (void) strcat(msgtxt, strerror(errno));
	goto error;
    }
    (void)memset(cur_subsys, '\0', sizeof(cur_subsys));               
    msg_desc = (nl_catd)-1;					      
#endif /*CLOSING_MSGFILE_EACH_TIME*/

/*
 *  Construct the message
 */
    msgbuf[0] = '\0';			    /* Start with null string */
    if (((msgflg & PSLMSG_PREFIX) != 0)	    /* Construct the prefix if req'd */
    ||  ((msgflg & PSLMSG_CODE) != 0))	    /* or if CODE req'd */    
    {
#ifdef VMS_MSG_FORMAT
        STRADD_MSGTXT(msgbuf, buflen, "%", STRADD_COPY);
					    /* Add % */
#endif
	if ((msgflg & PSLMSG_UTLNAM) != 0)  /* Add utility_name if requested */
	{
#ifdef VMS_MSG_FORMAT
	    STRADD_MSGTXT(msgbuf, buflen, utlnam, STRADD_CONCAT);
#else
	    STRADD_MSGTXT(msgbuf, buflen, utlnam_lc, STRADD_COPY);
#endif /* VMS_MSG_FORMAT */
	}

/*
 *   If PREFIX then add the facility name to the message, for CODE we do not
 */
	else if ((msgflg & PSLMSG_PREFIX) != 0)	
				/* Add facility_name for prefix only*/
	{
#ifdef VMS_MSG_FORMAT
	    STRADD_MSGTXT(msgbuf, buflen, facnam, STRADD_CONCAT);
#else
	    STRADD_MSGTXT(msgbuf, buflen, facnam, STRADD_COPY);
#endif
	}
#ifndef VMS_MSG_FORMAT
	    STRADD_MSGTXT(msgbuf, buflen, ": ", STRADD_CONCAT);
#endif
	switch (status)			    /* Add the -status- */
	{
#ifdef VMS_MSG_FORMAT
	    case 0:			    /* Warning */
	    {
		STRADD_MSGTXT(msgbuf, buflen, "-W-", STRADD_CONCAT);
                break;
	    }
	    case 1:			    /* Success */
	    {
		STRADD_MSGTXT(msgbuf, buflen, "-S-", STRADD_CONCAT);
		break;
	    }
	    case 2:			    /* Error */
	    {
		STRADD_MSGTXT(msgbuf, buflen, "-E-", STRADD_CONCAT);
		break;
	    }
	    case 3:			    /* Informational */
	    {
		STRADD_MSGTXT(msgbuf, buflen, "-I-", STRADD_CONCAT);
		break;
	    }
	    case 4:			    /* Fatal */
	    {
		STRADD_MSGTXT(msgbuf, buflen, "-F-", STRADD_CONCAT);
		break;
	    }
#else
	    case 0:			    /* Warning */
	    {
		STRADD_MSGTXT(msgbuf, buflen, "warn", STRADD_CONCAT);
                break;
	    }
	    case 1:			    /* Success */
	    {
		STRADD_MSGTXT(msgbuf, buflen, "ok", STRADD_CONCAT);
		break;
	    }
	    case 2:			    /* Error */
	    {
		STRADD_MSGTXT(msgbuf, buflen, "error", STRADD_CONCAT);
		break;
	    }
	    case 3:			    /* Informational */
	    {
		STRADD_MSGTXT(msgbuf, buflen, "info", STRADD_CONCAT);
		break;
	    }
	    case 4:			    /* Fatal */
	    {
		STRADD_MSGTXT(msgbuf, buflen, "fatal", STRADD_CONCAT);
		break;
	    }
#endif /* VMS_MSG_FORMAT */
	}
#ifndef VMS_MSG_FORMAT
        STRADD_MSGTXT(msgbuf, buflen, " (", STRADD_CONCAT);
#endif /* !VMS_MSG_FORMAT */
	STRADD_MSGTXT(msgbuf, buflen, msgnam, STRADD_CONCAT);
							/* Add message name */
#ifndef VMS_MSG_FORMAT
        STRADD_MSGTXT(msgbuf, buflen, ")", STRADD_CONCAT);
#endif /* !VMS_MSG_FORMAT */
    	if ((msgflg & PSLMSG_TEXT) != 0)		/* If text req'd */
	{
	    STRADD_MSGTXT(msgbuf, buflen, ", ", STRADD_CONCAT);
							/* Add , and a space */
	    STRADD_MSGTXT(msgbuf, buflen, msgtxt, STRADD_CONCAT);
						    /* ... add message text */
	}
    }
    else					   /* Message text only req'd */
    {
	STRADD_MSGTXT(msgbuf, buflen, msgtxt, STRADD_COPY);
    }

    return 1;				    /* Return successful */

/*
 *  We jump to this label if an error occurs. The error message in msgbuf.
 */

error:
    (void) strcpy(msgbuf, msgtxt);

    return 0;				    /* Return unsuccessful */
}


/****************************************************************************/
/*									    */
/*	GET_FACILITY - Get a facilities unix SET number and sub system	    */
/*									    */
/*	Calling Convention:						    */
/*									    */
/*	    iret = GET_FACILITY(vms_fac_num, unix_set_num, sub_system)	    */
/*									    */
/*	Where:								    */
/*									    */
/*	  iret		   =   Message found (PTRUE), not found (PFALSE)    */
/*	  vms_fac_num	   =   VMS facility number			    */
/*	  unix_set_num_ptr =   pointer to returned UNIX SET number	    */
/*	  sub_system	   =   the sub system name for this facility	    */
/*									    */
/****************************************************************************/
static Boolean GET_FACILITY(
unsigned long vms_fac_num,
unsigned long *unix_set_num_ptr,
char	      *sub_system_ptr)
{
/*
 *  Local data,
 */
unsigned short	    isub;
unsigned short	    jsub;

/*
 *  Subtract 2048 from the specified facility number.
 *  VMS reserves the first 2048 facilities for itself.
 */
    vms_fac_num -= 2048;
/*
 *  Search the lookup table for the specified facility
 */
    for (isub = 0;
	 isub < no_of_subsystems;
	 isub++)
    {
	if ((vms_fac_num >= fac_table[isub].vms_facility_min) &&
	    (vms_fac_num <= fac_table[isub].vms_facility_max))
							/* Facility found */
	{
	    *unix_set_num_ptr = vms_fac_num
				- fac_table[isub].vms_facility_min + 1;
	    (void) strcpy(sub_system_ptr, fac_table[isub].sub_system);
/*
 *	    Ensure subsystem in lowercase.
 */
	    for (jsub = 0;
		 jsub < strlen(sub_system_ptr);
		 jsub++)
	    {
		*(sub_system_ptr + jsub) =
		    psl_tolower(*(sub_system_ptr + jsub));
	    }
	    return PTRUE;				    /* Successful */
	}
    }
/*
 *  Facility not found
 */
    *unix_set_num_ptr = 0;		/* No set number */
    *sub_system_ptr = '\0';			/* No sub system */

    return PFALSE;			/* Return unsuccessful */
}


/****************************************************************************/
/*									    */
/*	STRADD_MSGTXT - Get a facilities unix SET number and filename	    */
/*									    */
/*	Calling Convention:						    */
/*									    */
/*	    GET_FACILITY(msg_str, buf_len, msg_txt, flags)		    */
/*									    */
/*	Where:								    */
/*									    */
/*	  msg_str	=	Pointer to message string		    */
/*	  buf_len	=	Buffer size				    */
/*	  msg_txt	=	Pointer to string to add to msg_str	    */
/*	  flags		=	Flag indicating copy or concatenate	    */
/*				STRADD_COPY or STRADD_CONCAT		    */
/*									    */
/****************************************************************************/
static void STRADD_MSGTXT(
char	      *msg_str,
unsigned long buf_len,
char	      *msg_txt,
unsigned long flags)
{
/*
 *  Local Data,
 */

unsigned long	length;

/*
 *  Copy or add according to the flags
 */
    if (flags == STRADD_COPY)			/* Copy msg_txt into msg_str */
    {
/*
 *  If there is not enough room in msg_str to add all of msg_txt then only
 *  add characters until msg_str is full.
 */
        if ( strlen(msg_txt) > (size_t)(buf_len - 1))
						/* Not enough room for msg_txt*/
	{
	    length = buf_len - 1 - (unsigned long)strlen(msg_str);
						/* Reduce copy string */
	}
	else					/* Enough room for msg_txt */
	{
	    length = (unsigned long)strlen(msg_txt);
	}
	for (; length != 0; msg_str++, msg_txt++, length--)	    
	    *msg_str = *msg_txt;
	*msg_str = '\0';
    }
    else if (flags == STRADD_CONCAT)		/* Add msg_txt to msg_str */
    {
/*
 *  If the message string is already full then return immediately
 */
        if (strlen(msg_str) >= (size_t)(buf_len - 1)) return;
/*
 *  If there is not enough room in msg_str to add all of msg_txt then only
 *  add characters until msg_str is full.
 */
	if ( (strlen(msg_str) + strlen(msg_txt)) > (size_t)(buf_len - 1))
						/* Not enough room for msg_txt*/
	{
	    length = buf_len - 1 - (unsigned long)strlen(msg_str);
						/* Reduce add string */
	}
	else					/* Enough room for msg_txt */
	{
	    length = (unsigned long)strlen(msg_txt);
	}
	msg_str = strchr(msg_str, '\0');
	for (; length != 0; msg_str++, msg_txt++, length--)	    /*31X094*/
	    *msg_str = *msg_txt;
	*msg_str = '\0';
    }

    return;
}


/****************************************************************************/
/*									    */
/*  PSL_GET_UTILNAME - Retrieve utility name				    */
/*									    */
/*  Calling Convention:							    */
/*									    */
/*	(void) PSL_GET_UTILNAME (utlnam_ptr, max_size);			    */
/*									    */
/*  Where:								    */
/*									    */
/*    utlnam_ptr = Pointer to buffer to hold utility name.		    */
/*    max_size   = Size of buffer.					    */
/*									    */
/*	This routine retrieves the utility name used by PSL_GET_MESSAGE     */
/*									    */
/****************************************************************************/
void	PSL_GET_UTILNAME (char *utlnam_ptr,
			  long max_size)
{
/*
 *	Copy utility name and ensure it is null-terminated
 */
	(void) strncpy (utlnam_ptr, utlnam, max_size - 1);
	*(utlnam_ptr + max_size - 1) = '\0';
	return;
}

/****************************************************************************/
/*									    */
/*  PSL_SET_UTILNAME - Set up utility name				    */
/*									    */
/*  Calling Convention:							    */
/*									    */
/*	(void) PSL_SET_UTILNAME (util_name)				    */
/*									    */
/*  Where:								    */
/*									    */
/*    util_name	=   Pointer to utility name				    */
/*									    */
/*	This routine stores the utility name for use by PSL_GET_MESSAGE     */
/*									    */
/****************************************************************************/
void	PSL_SET_UTILNAME (char *util_name)
{
#ifndef VMS_MSG_FORMAT
    int                 i;                      /* loop variable */
#endif /* VMS_MSG_FORMAT */
    
/*
 *	Set the locale for date/time, messages etc.
 */
	(void) setlocale (LC_ALL, "");

/*
 *	Copy utility name and ensure it is null-terminated
 */
	(void) strncpy (utlnam, util_name, (sizeof(utlnam) - 1));
	utlnam[sizeof(utlnam) - 1] = '\0';
#ifndef VMS_MSG_FORMAT
        (void)memcpy(utlnam_lc, utlnam, sizeof(utlnam_lc));
        for (i = 0;
             (i < sizeof(utlnam_lc) &&
              (utlnam_lc[i] != '\0'));
             i++)
        {
            utlnam_lc[i] = tolower(utlnam_lc[i]);
        }
#endif /* VMS_MSG_FORMAT*/        
        return;
}

#endif
