#ifndef PSL__SCL_COMDEF_LOADED
#define PSL__SCL_COMDEF_LOADED 1
/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

/*
 *  scl_comdef.h:
 *  =============
 *
 *  Include File containing SCL Common Data Structure.
 */

/* */
/*  File Logical Unit Numbers. */
/* */
#define            OBJ_UNIT 1       /* Script object file */
#define            SRC_UNIT 2       /* Source file unit */
#define        DEP_UNIT 3       /* Dependency file unit */
#define            INCLUDE_START_UNIT 4 /* Include file start unit */
/* */
/*  ...other constants. */
/* */
#define            MAX_INPUT_RATE 999   /* Maximum input rate in */
/* characters per second x 10 */
#define            MAX_TIME_PERIOD 32767    /* Maximum WAIT period   */
/* (arbitrary limit) */
/* */
/*  This symbol defines the maximum number of operands */
/*  in a command or conditional clause. */
/* */
/*  This number is used in creating the table of operand */
/*  descriptors for general use GEN_OP_DESCS. */
/* */
#define            MAX_VPS_OPD 5    /* Max operands in VP$SCREEN */
#define            MAX_DO_OPD 5 /* Max operands for a DO command */
#define            SCRIPT_NAME_LENGTH 16    /* Script name length        */
#define            MAX_INC_DEPTH 10 /* INclude depth maximum */
/* */
/*  The number of operands in a Log command is limited in order to */
/*  provide a definite limit on the maximum size of the FAO control */
/*  string and parameter list generated at run-time. A reasonably */
/*  large number has been chosen, but should it be insufficient, a */
/*  higher limit can be set. This will require changes to the */
/*  run-time service routine, which will need to allow for a longer */
/*  FAO control string and parameter list. */
/* */
#define            MAX_LOG_OPD 30   /* Maximum number of operands in */
/* a Log command */
/* */
/*  The SCL_COM record structure */
/*  ============================ */
/* */
struct scl_com
{
    /* */
    /*  Command Line Parsing information. */
    /* */
    struct scl_cmd          *cmd_ptrps;
    /* Command Lind Info. */
    char                    input_namec256[256];  /* Name of SCL file(s) */
    /* */
    /*  Minor Command Qualifiers */
    /*  ======================== */
    /* */
    char                    cmnd_chr_valuec256[256];
    char                    ctrl_chr_valuec256[256];
    char                    filler1b4[4];           /* Filler /alignment */
    PSL_U_LONGWORD_T        max_stepsl;             /* Value for step checks */
    char                    include_valuec256[256]; /* Value on /INCLUDE qualifier */
    char                    include_namec256[256];  /* Name of include file */
    char                    object_valuec256[256];  /* Value on /OBJECT qualifier */
    char                    obj_namec256[256];      /* Name of object file */
    char                    dep_namec256[256];      /* Name of dependency file */
    char                    scl_namec256[256];      /* Name current SCL file */
    char                    trimmed_namec40[40];    /* Trimmed name of current */
    /* SCL file */
    /*
     *  Define an array for the various files SCL may open:
     *
     *   1 x Script Object,
     *   1 x Script Source,
     *   1 x Dependency,
     *  10 x Include files,
     *   1 x spare
     */
    union files {
        PSL_LONGWORD_T          unitl14[14];
        FILE                    *ptrpf14[14];
    } files;
    PSL_U_LONGWORD_T        include_depthl;
    /* */
    /*  Context for the listing file. */
    /* */
    struct list_ctx         *lst_contextps;
    /* */
    /*  Context for the wildcarding of input files */
    /* */
    struct psl_vm_llb       *find_contextps;
    /* */
    /*  CONTROL/T Message text buffer. */
    /* */
    char                    ctrlt_messagec256[256];
    /* */
    /*  END-OF-FILE flag */
    /* */
    PSL_U_LONGWORD_T        eof_flagl;
    /* */
    /*  Define a pointer to the TPARSE structure. This contains data required */
    /*  to actually parse a source file. */
    /* */
    struct scl_tpa          *tpa_ptrps;
    /* */
    /*  Line number of line read from file. */
    /* */
    PSL_U_LONGWORD_T        srclinl10[10];
    /*
     *  Source line number on which the current definition, command or label started:
     *  - If in an #INCLUDE file, the line number is the line number (in the
     *    top level script) of the #INCLUDE command.
     *  - This value is reset when SCL starts to parse a new definition, command
     *    or label.
     *  - This value is stored in the 'srclin' parameter of each
     *    TOF record. It identifies the start of the SCL definition, command
     *    or label corresponding to the TOF record.
     */
    PSL_U_LONGWORD_T        tof_srclinl;
    /* */
    /*  Linked list ID. for the source file names */
    /* */
    struct psl_vm_llb       *srcfil_ctxps;
    PSL_U_LONGWORD_T        srcfil_cntl;
    /* */
    /*  ...and linked list data for a list of modules and a list of labels. */
    /* */
    struct psl_vm_llb       *module_ctxps;
    PSL_U_LONGWORD_T        module_cntl;
    struct psl_vm_llb       *labels_ctxps;
    PSL_U_LONGWORD_T        labels_cntl;
    struct psl_vm_llb       *variables_ctxps;

    /*
     *  Count of items in the script dependency linked list.
     */
    PSL_U_LONGWORD_T        dep_cntl;

    /*
     *  Script depdendency linked list.
     */
    struct psl_vm_llb       *dep_llps;

    /* */
    /*    Define a pointer to the Diagnostic stack structure. */
    /* */
    struct diagstk          *diag_ptrps;
    /* */
    /*    Pointer to the /dev/tty file device for CONFIRM. */
    /* */
    FILE                    *ttyfilpf;
    /* */
    /*  Full include filename */
    /* */
    char                    exp_src_filc256[256];
    /* */
    /*  */
    /* */
    /*    This array is used to hold the source file index of the file which we */
    /*    were compiling BEFORE we encountered an INCLUDE. We need to save this so */
    /*    that it can be reset to its original value on return from the included */
    /*    file. The source file index is used during the encoding of the diagnose */
    /*    information to identify the entry in the table of source file name which */
    /*    relates to the file currently being processed. */
    /* */
    PSL_U_LONGWORD_T        src_fil_indxl10[10];
    /* */
    /*    /TYPE flags. */
    /* */
    Boolean                 type_scriptn;
    Boolean                 type_threadn;
    Boolean                 type_testn;

    /*
     *  Pointer to TOF line buffer. This is used in writing a script object ('tof') file.
     */
    char                    *toflinpc;

    /*
     *  Script name.
     */
    char            scrnam_c20[20];

    /*
     *    Filler for alignment and to allow for expansion.
     */
    char                    filler2b240[240];
} ;
/**/
/*  End of C Include File */
/**/
#endif  /* PSL__SCL_COMDEF_LOADED */
