#ifndef PSL__SCL_TBLDEF_LOADED
#define PSL__SCL_TBLDEF_LOADED 1
/* Copyright (C) 2000-2001 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

/*
 *  scl_tbldef.h:
 *  =============
 *
 *  SCL Symbol Table Definitions.
 */

/* */
/*    THE OPERAND HEADER */
/*    ================== */
/* */
/*    The operand header structure contains data relevant to all breeds of */
/*    operand descriptors. */
/* */
struct oph
{
    /* */
    /*    Addressing Mode */
    /* */
    PSL_U_LONGWORD_T        addrl;
    /* */
    /*    Data type */
    /* */
    PSL_U_LONGWORD_T        dtypl;
    /* */
    /*    Data size */
    /* */
    PSL_U_LONGWORD_T        sizel;
    /* */
    /*    Variable options bitmask */
    /* */
    PSL_LONGWORD_T          optl;
    /* */
    /*    Flag identifying the operand within a command, definition or statement. */
    /*    This value is optional and command, definition or statement specific. */
    /* */
    PSL_LONGWORD_T          idl;
    /* */
    /*    Pad to quadword boundary. */
    /* */
    PSL_LONGWORD_T          paddingl;

} ;

/* */
/*    THE OPERAND DESCRIPTOR */
/*    ====================== */
/* */
/*    Operand descriptors are used to save operand data during */
/*    parsing. Below is a set of symbols defining the field offsets */
/*    within an operand descriptor. */
/* */
struct opd
{
    /* */
    /*    The rest of an operand descriptor is determined by the operand type. */
    /*    We therefore define unions to overlay this information. */
    /* */
    union uopd {
        /*
         *    Function Operand.
         */
        struct fnc
        {
            /*
             *    Function ID. Symbol names TOF_FNC_ID_*.
             */
            PSL_LONGWORD_T          idl;

            /*
             *    Parameter count.
             */
            PSL_LONGWORD_T          prmcntl;
        }
        fnc;
        /* */
        /*    Immediate Integer Operand */
        /* */
        struct imi
        {
            /* */
            /*    Integer value */
            /* */
            PSL_LONGWORD_T          vall;
        }
        imi;
        /*    Test Operand. */
        /* */
        struct tst
        {
            /*
             *    This field contains a pointer to the SCL symbol table entry for the
             *    test during parsing.
             */
            struct tsttbl           *tsttbl_ptrps;
        }
        tst;
        /*    Test Thread Operand. */
        /* */
        struct tstthr
        {
            /*
             *    This field contains a pointer to the SCL symbol table entry for the
             *    test thread during parsing.
             */
            struct tstthrtbl        *tstthrtbl_ptrps;
        }
        tstthr;

        /* */
        /*    Timer Operand */
        /* */
        struct tim
        {
            /*
             *    This field contains a pointer to the SCL symbol table entry for the
             *    timer during parsing.
             */
            struct timtbl           *timtbl_ptrps;
        }
        tim;

        /*
         *    FAT (File Access Table) Operand (i.e. File or Pipe).
         */
        struct fat
        {
            /*
             *    This field contains a pointer to the File Access Table entry for the
             *    item during parsing.
             */
            struct fattbl           *fattbl_ptrps;
        }
        fat;

        /* */
        /*    Immediate Character Operand */
        /* */
        struct imc
        {
            /*
             *    Character value. The space for the character value is allocated
             *    using MALLOC (since we know when the operand is created how long
             *    it is) and MUST be freed when the operand data structure is FREE'd
             *    to avoid memory leakage.
             */
            char                    *val_pc;
        }
        imc;
        /* */
        /*    Character string (in code buffer) */
        /* */
        struct ims
        {
            /* */
            /*    Address of character string in the object code buffer */
            /* */
            PSL_U_LONGWORD_T        ptrl;
        }
        ims;
        /* */
        /* */
        /*    Variable Fields */
        /* */
        /*    Define the layout of the operand descriptor for a V4.0 variable. */
        /*    This structure is now obsolete and has been superseded by the following */
        /*    structure which will be used for all scripts V4.1 onwards. */
        /* */
        struct varv40
        {
            /* */
            /*    This field contains the offset of the variable in the */
            /*    data area, if it is a local variable. If it is a shareable variable */
            /*    it contains the offset of the ADDRESS of the data in the Shareable */
            /*    Variable Address Table (created at run-time). */
            /* */
            PSL_U_LONGWORD_T        offl;
            /* */
            /*    This field defines the offset of the start of any value list for */
            /*    the variable. */
            /* */
            PSL_U_LONGWORD_T        values_offl;
            /* */
            /*    This field defines the offset of the first byte beyond the end of any */
            /*    value list for the variable. */
            /* */
            PSL_U_LONGWORD_T        values_endl;
            /* */
            /*    This field defines the number of values in any value list for the */
            /*    variable. */
            /* */
            PSL_U_LONGWORD_T        values_cntl;
            /* */
            /*    This field defines the low range value for any range specified for this */
            /*    variable. */
            /* */
            PSL_LONGWORD_T          low_rangel;
            /* */
            /*    This field defines the high range value for any range specified for this */
            /*    variable. */
            /* */
            PSL_LONGWORD_T          high_rangel;
        }
        varv40;
        /* */
        /*    Define the layout of the operand descriptor for a V4.1 onwards variable. */
        /*    The only difference is that the operand descriptor has been extended to */
        /*    include a longword count of the array dimensions. */
        /* */
        struct var
        {
            /* */
            /*    This field defines the offset of the variable within the LDA, SDA or */
            /*    shareable variable lookup table. */
            /* */
            PSL_U_LONGWORD_T        offl;
            union uvar {
                /* */
                /* Structure changed to allow records, but retain compatibility. */
                /* */
                /* */
                /*    Non record Variable */
                /* */
                struct nrec
                {
                    /* */
                    /*    This field defines the offset of the start of any value list for */
                    /*    the variable. */
                    /* */
                    PSL_U_LONGWORD_T        values_offl;
                    /* */
                    /*    This field defines the offset of the first byte beyond the end of any */
                    /*    value list for the variable. */
                    /* */
                    PSL_U_LONGWORD_T        values_endl;
                    /* */
                    /*    This field defines the number of values in any value list for the */
                    /*    variable. */
                    /* */
                    PSL_U_LONGWORD_T        values_cntl;
                    /* */
                    /*    This field defines the low range value for any range specified for this */
                    /*    variable. */
                    /* */
                    PSL_LONGWORD_T          low_rangel;   /* Low range value */
                    /* */
                    /*    This field defines the high range value for any range specified for this */
                    /*    variable. */
                    /* */
                    PSL_LONGWORD_T          high_rangel;
                }
                nrec;
                /* */
                /*    Record Variable */
                /* */
                struct rec
                {
                    /* */
                    /* Record Index */
                    /* */
                    PSL_LONGWORD_T          indexl;
                    PSL_LONGWORD_T          dummy1l;
                    PSL_LONGWORD_T          dummy2l;
                    PSL_LONGWORD_T          dummy3l;
                    PSL_LONGWORD_T          dummy4l;
                }
                rec;
            } uvar;
            /* */
            /*    This field contains a value which defines the number of dimensions for */
            /*    the array. */
            /* */
            PSL_LONGWORD_T          dimcntl;
            /*
             *    This field contains a pointer to the SCL symbol table entry for the
             *    variable during parsing.
             */
            struct symtbl           *symtbl_ptrps;
        }
        var;
        /* */
        /* */
        /* */
        /*    Operator Operand */
        /* */
        /*    This operand descriptor is used to store an OPERATOR. */
        /* */
        struct opr
        {
            /* */
            /*    Operator */
            /* */
            PSL_U_LONGWORD_T        operatorl;
        }
        opr;
        /* */
        /* */
    } uopd;
} ;

/* */
/*    VP$SCREEN DESCRIPTOR (LGR) */
/*    ========================== */
/* */
/*    This table is used within LGR for maintaining a record of the last usage */
/*    of a VP$SCREEN operand so that the individual commands can work on it. */
/* */
/*    It is set up by the DECODE_OPERAND routine to contain the base line */
/*    and column number and the depth and width of the screen. */
/* */
struct vp_screen_dsc
{
    /* */
    /*    Base line value */
    /* */
    PSL_U_WORD_T            base_linew;
    /* */
    /*    Base column value */
    /* */
    PSL_U_WORD_T            base_columnw;
    /* */
    /*    Depth in lines of screen */
    /* */
    PSL_U_WORD_T            screen_depthw;
    /* */
    /*    Width in columns of screen */
    /* */
    PSL_U_WORD_T            screen_widthw;
} ;

/* */
/*    VARIABLES SYMBOL TABLE ENTRY RECORD */
/*    =================================== */
/* */
/*    The VAR_OFFL field contains the offset of the variable in the */
/*    data area, if it is a local variable. If it is a shareable variable */
/*    it contains the offset of the ADDRESS of the data in the Shareable */
/*    Variable Address Table (created at run-time). */
/* */
/*    The VAR_OPTL field is used to flag variable options. It contains */
/*    a bitmap, in which each bit represents a variable option. */
/* */
/* */
/*    The bits are interpreted as follows: */
/* */
/*    VOPT_RANDOM_M */
/*      Clear = RANDOM character values (Default) */
/*          Set   = REPEATABLE RANDOM character values */
/*    VOPT_SCOPE_M */
/*      Clear = Scope defaulted (Default) */
/*          Set   = Scope specified */
/*    VOPT_LIST_M */
/*      Clear = No value list defined (Default) */
/*          Set   = Value list defined */
/*    VOPT_RANGE_M */
/*      Clear = No value range defined (Default) */
/*          Set   = Value range defined */
/*    VOPT_SEED_M */
/*      Clear = No seed specified */
/*          Set   = Seed specified */
/* */
struct symtbl
{
    /* */
    /*    Maximum number of entries in the Variables Symbol table */
    /* */
#define            MAX_VARIABLES 8000
    /* */
    /*    Variable Name - This must be a valid V-TEST dataname. */
    /* */
    char                    var_namec16[16];
    /*
     *  Source line number on which the current definition started.
     */
    PSL_U_LONGWORD_T        var_tof_srclinl;
    /*
     *    Length of name.
     */    
    PSL_U_LONGWORD_T        var_nlengthl;
    /* */
    /*    Variable Addressing Mode - The variable addressing mode. */
    /* */
    /* */
    PSL_U_LONGWORD_T        var_addrl;
    PSL_U_LONGWORD_T        var_addrl2;
    /* */
    /*    The addressing modes are: */
    /* */
    /*   *   ADDR_IMMEDIATE (0)          Immediate */
    /*   *   ADDR_LOCAL     (1)          Local Data Area */
    /*   *   ADDR_GLOBAL    (2)          Global Data Area */
    /*       ADDR_VP$OUTPUT (3)          VP$OUTPUT */
    /*       ADDR_NULL      (4)          No operand */
    /*   *   ADDR_SCRIPT    (5)          Script Data Area */
    /*       ADDR_TIMER     (6)          Timer */
    /*       ADDR_FUNCTION  (7)          Function */
    /* */
    /*    The addressing modes marked with a "*" are the only addressing */
    /*    modes that may appear in the Variables Symbol Table. */
    /* */
    /*    The symbols are defined in the macro VTEST_PRMDEF.MAR. */
    /* */
    /* */
    /*    Variable Size - The variable length in bytes. All integer variables */
    /*    are two bytes long. */
    /* */
    PSL_U_LONGWORD_T        var_sizel;
    /* */
    /*    Variable Data Type - The variable data type. */
    /* */
    PSL_U_LONGWORD_T        var_typel;
    /* */
    /*    The valid variable data types are: */
    /* */
    /*      DTYP_INTEGER   (1)          Integer */
    /*      DTYP_CHARACTER (2)          Character */
    /* */
    /*    These symbols are defined in the file VTEST_PRMDEF.H */
    /* */
    /* */
    /*    Variable Offset/Address Offset - If the variable is a local or */
    /*    script variable this field contains offset of the variable in Local */
    /*    Data Area and Script Data Area respectively. If it is a shareable variable, */
    /*    however, it contains the offset of the variable's address in Shareable */
    /*    Variables Address Table (created at run-time). */
    /* */
    PSL_U_LONGWORD_T        var_offl;
    /* */
    /*    Variable Options Mask - This field contains a mask of variable options. */
    /* */
    PSL_LONGWORD_T          var_optl;
    /* */
    /*    The bits are interpreted as follows: */
    /* */
    /*    VOPT_RANDOM_M */
    /*              Clear = RANDOM character values (Default) */
    /*              Set   = REPEATABLE RANDOM character values */
    /*    VOPT_SCOPE_M */
    /*              Clear = Scope defaulted (Default) */
    /*              Set   = Scope specified */
    /*    VOPT_LIST_M */
    /*              Clear = No value list defined (Default) */
    /*              Set   = Value list defined */
    /*    VOPT_RANGE_M */
    /*              Clear = No value range defined (Default) */
    /*              Set   = Value range defined */
    /*    VOPT_SEED_M */
    /*              Clear = No seed specified */
    /*              Set   = seed specified */
    /* */
    /* */
    /*    Variable Value List Offset - Offset of value list in bytes from the */
    /*    start of the value list buffer. */
    /* */
    PSL_U_LONGWORD_T        var_values_offl;
    /* */
    /*    Variable Value List End Offset - Offset in bytes of */
    /*    the byte beyond end of the variable's value list from the */
    /*    start of the value list buffer. */
    /* */
    PSL_U_LONGWORD_T        var_values_endl;
    /* */
    /*    Variable Value Count - The number of values in the variable's value list. */
    /* */
    PSL_U_LONGWORD_T        var_values_cntl;
    /* */
    /*    Low Range Value - Low range value for a variable with a defined */
    /*    value range. */
    /* */
    PSL_LONGWORD_T          var_low_rangel;
    /* */
    /*    High Range Value - High range value for a variable with a defined */
    /*    value range. */
    /* */
    PSL_LONGWORD_T          var_high_rangel;
    /* */
    /*    Prompt string. */
    /* */
    char                    var_promptc50[50];
    /* */
    /*    Filler bytes */
    /* */
    char                    var_fillerb2[2];
    /* */
    /*    Length of prompt */
    /* */
    PSL_U_LONGWORD_T        var_plengthl;
    /* */
    /*    Seed value */
    /* */
    PSL_LONGWORD_T          var_seedl;
    /* */
    /*    Dimensions for array */
    /* */
    PSL_LONGWORD_T          var_array_dimcntl;
    /* */
    /*    Array Bounds array */
    /* */
    PSL_LONGWORD_T          var_array_bounds1l10[10];
    /* */
    /*    Array Bounds array */
    /* */
    PSL_LONGWORD_T          var_array_bounds2l10[10];
    /*
     *    Index of parent 'record' in Variables Symbol Table.
     *    0 = variable os NOT a field within a record.
     */
    PSL_U_LONGWORD_T        var_recidxl;
    /* */
    /*    Record index */
    /* */
    PSL_U_LONGWORD_T        var_indexl;
    /* */
    /*    Number of items */
    /* */
    PSL_U_LONGWORD_T        var_rec_iteml;
    /* */
    /*  Offset of start of format array */
    /* */
    PSL_LONGWORD_T          *var_start_rec_arraypl;
    /*
     *  Pointer to filename (set to the filename associated with a
     *  file variable).
     *
     *  N.B. The space allocated for this filename must be freed
     *   when finished with the symbol table.
     */
    char            *var_filnam_pc;
} ;

/* */
/*    LABELS Symbol Table */
/*    =================== */
/* */
/*    The Labels Symbol Table entries have the following format */
/* */
/*      Character*16    Label name */
/*  Unsigned long   Offset of instruction after label */
/*      Unsigned long   Address of last unresolved reference to label */
/* */
/*    The label name field will always set, but only one of the other two */
/*    fields may be set at any time. */
/* */
/*    If a label definition is encountered, before it has been referenced, */
/*    an entry will be created for it in the Labels Symbol Table. The */
/*    entry will contain the label name and the offset of the next */
/*    instruction in the object code buffer. The unresolved reference field */
/*    will be set to zero. */
/* */
/*    If a reference to a label is encountered before the label definition */
/*    itself, the label offset field will be set to zero and a reference */
/*    chain established. Each element in the chain either contains the */
/*    address of the previous reference to the label, or zero, marking */
/*    the end of the chain. If the chain header contains zero the chain */
/*    is empty.  */
/* */
struct labtbl
{
    /* */
    /*    Max number entries allowed in the Labels Symbol Table */
    /* */
#define            MAX_LABELS 255
    /* */
    /*    Label Name - This must be a valid V-TEST dataname. */
    /* */
    char                    lab_namec16[16];
    /* */
    /*    Label Name length - This is the length in chars of the label name. */
    /* */
    PSL_U_LONGWORD_T        lab_nlengthl;
    /*
     *    Label Defined Flag - This field is set PTRUE if the label has been
     *    defined and PFALSE, if only references to the label have been
     *    encountered.
     */
    Boolean         lab_defn;
} ;

/*
 *    SCRIPT Dependency List
 *    ======================
 *
 *    This PSL linked list is used to hold all script dependencies that
 *    are not stored in another table within SCL.
 */
struct sdep
{
    /*
     *    Dependency 'name' - For example, script, include file, data file or screen image file.
     */
    char                    sdep_namec256[256];
    /*
     *    Dependency type (DEP_TYP_*).
     */
    unsigned long       sdep_typl;
    /*
     *    Filler for quadword alignment.
     */
    char                    sdep_filler1c4[4];
    /*
     *    Dependency type specific data, e.g. target type for 'executed scripts'.
     *    This should NOT include a leading separator character.
     */
    char                    sdep_datac256[256];
    /*
     *  Padding for quadword alignment.
     */
    /***  char                    sdep_paddingc4[4]; ***/
} ;


/* */
/*    SUBROUTINE Symbol Table    */
/*    ======================= */
/* */
/*    The Subroutines Symbol Table entries have the following format */
/* */
/*        16 Bytes  Subroutine name */
/*         4 Bytes  Offset of instruction after Subroutine */
/*         4 Bytes  Address of last unresolved reference to subroutine */
/* */
/*    The subroutine name field will always set, but only one of the other two */
/*    fields may be set at any time. */
/* */
/*    If a Subroutine definition is encountered, before it has been referenced */
/*    an entry will be created for it in the Subroutines Symbol Table. The */
/*    entry will contain the subroutine name and the offset of the next */
/*    instruction in the object code buffer. The unresolved reference field */
/*    will be set to zero. */
/* */
/*    If a reference to a subroutine is encountered before the definition */
/*    itself, the subroutine offset field will be set to zero and a reference */
/*    chain established. Each element in the chain either contains the */
/*    address of the previous reference to the subroutine or zero, marking */
/*    the end of the chain. If the chain header contains zero the chain */
/*    is empty.  */
/* */
struct subtbl
{
    /* */
    /*    Max number entries allowed in the Subroutines Symbol Table */
    /* */
#define            MAX_SUBROUTINES 255
    /* */
    /*    Subroutine name */
    /* */
    char                    sub_namec20[20];
    /*
     *    Subroutine Defined Flag - This field is set PTRUE if the subroutine has
     *    been defined and PFALSE, if only references to the subroutine have been
     *    encountered.
     */
    Boolean         sub_defn;
    /* */
    /*    Count of parameters */
    /* */
    PSL_U_LONGWORD_T        sub_param_cntl;
    /* */
    /*  Padding for expansion and quadword alignment. */
    /* */
    char                    sub_paddingc4[4];
} ;

/*    TESTS Symbol Table */
/*    ================== */
/* */
/*    Tests Symbol table entry structure. */
/* */
struct tsttbl
{
    /* */
    /*    Test name. */
    /* */
    char                    tst_namec68[68];
    /*
     *  Source line number on which the current definition started.
     */
    PSL_U_LONGWORD_T        tst_tof_srclinl;
    /* */
    /*    Pointer to Parameter structure. */
    /* */
    struct prm              *tst_prm_ptrps;
    /* */
    /*    Pointer to Test Header structure. */
    /* */
    struct tlh              *tst_tlh_ptrps;
    /*
     *    Execute Test reference flag:
     *    - PTRUE  = Test or sub-test referenced within an EXECUTE command.
     *    - PFALSE = Test or sub-test NOT referenced within an EXECUTE command.
     */
    Boolean                 tst_exe_refn;
    /*
     *    Wait for Test reference flag:
     *    - PTRUE  = Test or sub-test referenced within a WAIT FOR TEST command.
     *    - PFALSE = Test or sub-test NOT referenced within a WAIT FOR TEST command.
     */
    Boolean                 tst_wait_refn;
    /* */
    /*      This field indicates if there was any platform specific */
    /*      code in the test. */
    /* */
    Boolean                 tst_plat_spec_coden;
    /* */
    /*      This field indicates if there was any operting system specific */
    /*      code in the test. */
    /* */
    Boolean                 tst_opsys_spec_coden;
    /*
     *  This field indicates whether the test is DECLARED as a TEST or SUB-TEST
     *  - PTRUE  = SUB-TEST
     *  - PFALSE = TEST
     */
    Boolean                 tst_subtstn;

    /*
     *  This field indicates, for sub-tests only, whether the SUB-TEST has been DEFINED.
     *  - PTRUE  = Has been defined
     *  - PFALSE = Has NOT been defined
     */
    Boolean                 tst_definedn;

    /* */
    /*  Padding for expansion and quadword alignment. */
    /* */
    char                    tst_paddingc4[4];
    /* */
} ;

/*
 *    TEST THREADS Symbol Table
 *    =========================
 *
 *    This table is used to hold Thread declarationd within a Test script.
 */
struct tstthrtbl
{
    /*
     *    Test Thread Name - This must be a valid OpenSTA dataname.
     */
    char                    tstthr_namec68[68];
    /*
     *  Source line number on which the current definition started.
     */
    PSL_U_LONGWORD_T        tstthr_tof_srclinl;
    /*
     *    Pointer to Test Thread Structure.
     */
    struct tthr             *tstthr_tthr_ptrps;
    /*
     *    Pointer to Terminal Structure.
     */
    struct tdf              *tstthr_tdf_ptrps;
    /*
     *    Execute Test Thread reference flag:
     *    - PTRUE  = Test Thread referenced within an EXECUTE command.
     *    - PFALSE = Test Thread NOT referenced within an EXECUTE command.
     */
    Boolean                 tstthr_exe_refn;

    /*
     *    Definition/declaration flag
     *    - PTRUE  = Test Thread definition local.
     *    - PFALSE = Test Thread definition external (this is a declaration only)
     */
    Boolean                 tstthr_defn;
    /*
     *    Padding for quadword alignment.
     */
    char                    tstthr_paddingc4[4];
} ;

/*    THREADS Symbol Table */
/*    ==================== */
/* */
/*    This table is used to hold Thread definitions defined within a */
/*    Thread script. */
/* */
struct thrtbl
{
    /* */
    /*    Thread name. */
    /* */
    char                    thr_namec20[20];
    /*
     *  Source line number on which the current definition started.
     */
    PSL_U_LONGWORD_T        thr_tof_srclinl;
    /* */
    /*    Pointer to Terminal Structure. */
    /* */
    struct tdf              *thr_tdf_ptrps;

    /*  Pad to quadword. */
    /*
     *  Padding for quadword alignment.
     */
    char                    thr_paddingc8[8];
} ;

/* */
/*    TIMERS Symbol Table */
/*    =================== */
/* */
/*    The timers symbol table currently only has a single entry - the name of */
/*    the timer. */
/* */
struct timtbl
{
    /*
     *    Timer Name - This must be a valid OpenSTA dataname.
     */
    char                    tim_namec68[68];
    /*
     *  Source line number on which the current definition started.
     */
    PSL_U_LONGWORD_T        tim_tof_srclinl;
} ;

/* */
/*    FILE/PIPE Access Symbol Table */
/*    ============================= */
/* */
/*    The File/Pipe Access Symbol Table entries have the following format */
/* */
/*    16 Bytes  File/Pipe Id name */
/*     4 Bytes  Length of name */
/*     4 Bytes  Referenced flag: */
/* */
/*    There are two separate tables.  The File Table will contain up to */
/*    MAX_FILES entries and the Pipe Table will contain up to MAX_PIPES */
/*    entries. */
/* */
struct fattbl
{
    /* */
    /*    File/Pipe name */
    /* */
    char                    fat_namec20[20];
    /*
     *  Source line number on which the current definition started.
     */
    PSL_U_LONGWORD_T        fat_tof_srclinl;
    /* */
    /*    Referenced flag */
    /*    Note that the flag is set to various constant values for files */
    /*    and to various bitmasks for pipes. */
    /* */
    PSL_U_LONGWORD_T        fat_refflgl;
#define            ACCESSED_FILE 0  /* File accessed */
#define            DEFINED_FILE 1   /* File defined (FILE statement) */
#define            OPEN_FILE 2  /* File opened */
#define            PASSED_FILE 3    /* File passed as a parameter */
#define            PIPATT_SYNC 1    /* Pipe is synchronous */
#define            PIPATT_PRM 2 /* Pipe passed as parameter */
#define            PIPATT_OPEN 4    /* Pipe has been opened */
#define            PIPATT_ACC 8 /* Pipe has been accessed */
} ;

/*
 *    SHAREABLE VARIABLES Symbol Table
 *    ================================
 *
 *    This table is used to hold data relating to variables that may be shared
 *    by different scripts or threads, i.e. Thread, Global or Testwide variables.
 */
struct shrvartbl
{
    /*
     *    Variable Data Type - The variable data type.
     */
    PSL_U_LONGWORD_T        shrvar_dtypl;
    /*
     *    The valid variable data types are:
     *
     *      DTYP_INTEGER   (1)          Integer
     *      DTYP_CHARACTER (2)          Character
     *      DTYP_RECORD    (9)          Record
     *    These symbols are defined in the file SCL_PRMDEF.H
     *
     *
     *    Variable Size - The variable length in bytes. All integers are classed
     *    as long values.
     */
    PSL_U_LONGWORD_T        shrvar_sizel;
    /*
     *    Variable Name - This must be a valid V-TEST dataname.
     */
    char                    shrvar_namec16[16];
    /*
     *    It identifies the variables addressing mode. It may be set to ADDR_THREAD,
     *    ADDR_GLOBAL or ADDR_TESTWIDE.
     */
    PSL_U_LONGWORD_T        shrvar_addrl;
} ;

/* */
/*    SCRIPT PARAMETERS Symbol Table */
/*    ============================== */
/* */
struct prmtbl
{
    /* */
    /*    Variable data name */
    /* */
    char                    prm_namec16[16];
    /* */
    /*    Length of name */
    /* */
    PSL_U_LONGWORD_T        prm_nlengthl;
    /* */
    /*    Variable data type */
    /* */
    PSL_U_LONGWORD_T        prm_dtypl;
    /* */
    /*    ...and its addressing mode */
    /* */
    PSL_U_LONGWORD_T        prm_addrl;
    /* */
    /*    ...and its size */
    /* */
    PSL_U_LONGWORD_T        prm_sizel;
    /* */
    /*    ...and its offset */
    /* */
    PSL_U_LONGWORD_T        prm_offsetl;
    /* */
    /*    ...and a prompt string */
    /* */
    char                    prm_promptc50[50];
    /* */
    /*    Filler bytes */
    /* */
    char                    prm_fillerb2[2];
    /* */
    /*    Length of prompt */
    /* */
    PSL_U_LONGWORD_T        prm_plengthl;
    /* */
    /*    Echo/noecho flag */
    /* */
    PSL_U_LONGWORD_T        prm_echol;
    /* */
    /*    Uppercase flag */
    /* */
    PSL_U_LONGWORD_T        prm_uppercasel;
} ;

/*
 *    COMPLEX CONDITION (IF) Parsing Stack
 *    ====================================
 *
 *    The Complex Condition Parsing Stack is used to hold data during
 *    the parsing of a complex conditional expression.
 *
 *    An entry is pushed onto the stack when a left parenthesis '('
 *    is encountered and popped from it when a right parenthesis ')'
 *    is encountered.
 */
struct ifstack
{
    /*
     *    'Else' command parsed flag.
     */
    Boolean             else_parsedn;
    /*
     *    This is a filler record to pad the stack to a quadword boundary.
     */
    char                    paddingb4[4];
} ;

/* */
/*    DO Parsing Stack */
/*    ================ */
/* */
struct dostack
{
    /* */
    /*    This field is used during parsing of a DO command, to maintain a record */
    /*    of the OFFSET of the first executable instruction in the DO loop. It */
    /*    is filled in by the DO command and referenced by the ENDDO command. */
    /* */
    char                    *start_addrpc;    /* DO loop start address */
    /* */
    /*    This field is used during parsing of a DO loop. On its first iteration, */
    /*    the DO loop branches to the END of the DO loop in order that the */
    /*    condition test may be performed. This field contains the ADDRESS of the */
    /*    GOTO instruction which performs this branch. During processing of the */
    /*    ENDDO instruction, this field is used so that the GOTO may be resolved. */
    /* */
    char                    *end_addrpc;  /* DO loop END reference address */
    /* */
    /*    This field holds the pointer to the LL which holds the operands which */
    /*    togethor define the DO loop that is the CONTROL, START, END and STEP */
    /*    operands. */
    /* */
    struct psl_vm_llb       *do_operand_llps;
    /* */
    /*    This is a filler record to pad the stack to a quadword boundary. */
    /* */
    char                    fillerb4[4];
} ;

/* */
/*    SWITCH Parsing Stack */
/*    ==================== */
/* */
struct swstack
{
    /* */
    /*    Address of end_switch offset */
    /* */
    char                    *end_switch_offpc;
    /* */
    /*    Offset of last case parsed */
    /* */
    char                    *last_case_offpc;
    /* */
    /*    Current break address */
    /* */
    PSL_LONGWORD_T          *break_addrpl;
    /* */
    /*    This field holds the pointer to the linked list which holds the */
    /*    operands which together define SWITCH variable. */
    /* */
    struct psl_vm_llb       *sw_operand_llps;
    /* */
    /*    Switch Variable data type */
    /* */
    PSL_U_LONGWORD_T        sw_dtypl;
    /* */
    /*    Count of cases in switch */
    /* */
    PSL_U_LONGWORD_T        case_countl;
    /* */
    /*    Count of breaks parsed */
    /* */
    PSL_U_LONGWORD_T        break_countl;
    /* */
    /*    Indicates if there has been a default case at */
    /*    the current level. */
    /* */
    PSL_U_LONGWORD_T        def_case_fgl;
} ;

/* */
/*    VARIABLES Symbol Table */
/*    ====================== */
/* */
/*    This symbol table contains all of the variables, TIMERS and FILES defined */
/*    within an SCL source file. It is appended to the generated script either */
/*    when /DIAGNOSE has been specified or when the script is an SQL mode one. */
/* */
struct var_symtbl
{
    /* */
    /*    This field contains the null terminated name of the symbol */
    /*    (variable/timer/file) */
    /* */
    char                    symbol_namec20[20];
    /* */
    /*    If this symbol is a variable, this field will define the Variable */
    /*    addressing mode */
    /* */
    PSL_U_LONGWORD_T        addrl;
    /* */
    /*    Variable size      */
    /* */
    PSL_U_LONGWORD_T        sizel;
    /* */
    /*    Variable data type     */
    /* */
    PSL_U_LONGWORD_T        typel;
    /* */
    /*    Variable offset (local/script) or address offset (thread/global/testwide) */
    /* */
    PSL_U_LONGWORD_T        offl;
    /* */
    /*    This field contains the options bitmask specified for this variable when */
    /*    it was declared. */
    /* */
    PSL_U_LONGWORD_T        optl;
    /* */
    /*    Variable Value List Offset - Offset of value list in bytes from the */
    /*    start of the value list buffer. */
    /* */
    PSL_U_LONGWORD_T        values_offl;
    /* */
    /*    Variable Value List End Offset - Offset in bytes of */
    /*    the byte beyond end of the variable's value list from the */
    /*    start of the value list buffer. */
    /* */
    PSL_U_LONGWORD_T        values_endl;
    /* */
    /*    Variable Value Count - The number of values in the variable's value list. */
    /* */
    PSL_U_LONGWORD_T        values_cntl;
    /* */
    /*    Low Range Value - Low range value for a variable with a defined */
    /*    value range. */
    /* */
    PSL_LONGWORD_T          low_rangel;
    /* */
    /*    High Range Value - High range value for a variable with a defined */
    /*    value range. */
    /* */
    PSL_LONGWORD_T          high_rangel;
    /* */
    /*    Pad to quadword */
    /* */
    PSL_LONGWORD_T          paddingl;
} ;

/* */
/*    LABEL Symbol Table */
/*    ================== */
/* */
/*    This symbol table provides a translation between a LABEL and its offset */
/*    within the generated SCP file. */
/* */
/*    Currently it is only generated if /DIAGNOSE is specified. */
/* */
struct lab_symtbl
{
    /* */
    /*    This field contains the null terminated name of the label. */
    /* */
    char                    label_namec20[20];
    /* */
    /*    Label offset */
    /* */
    PSL_U_LONGWORD_T        offl;
} ;

/* */
/*    SCRIPT MODULES Symbol Table */
/*    =========================== */
/* */
/*    This symbol table contains the names of the modules which combine to form */
/*    this script (main + include files) and their index. */
/* */
struct mod_symtbl
{
    /* */
    /*    This field contains the null terminated module name */
    /* */
    char                    module_namec20[20];
    /* */
    /*    Module index */
    /* */
    PSL_U_LONGWORD_T        idxl;
} ;
/**/
/*  End of C Include File */
/**/
#endif  /* PSL__SCL_TBLDEF_LOADED */
