/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

/*
 * scl_wrt_routines.c:
 * ===================
 *
 * This module contains all the routines specific to creating and writing a
 * script object ('tof') file.
 */

#include <psl_macros.h>
#include <vtest_macros.h>

/*
 *  Standard ANSI headers.
 */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>
#include <ctype.h>

/*
 *  Function Prototypes.
 */
#include <psl_rtndef.h>         /* PSL routines */
#include <trc_rtndef.h>         /* Tracing Routines */
#include "scl_rtndef.h"         /* SCL routines */
#include <depprmdef.h>          /* Script dependency parameters */

static void     GET_VAROPT  (struct scl_com *com_ptr,
                             struct symtbl *var_ptr,
                             unsigned long *optval_ptr);

static unsigned long    WRT_DEP_LINE    (struct scl_com *com_ptr,
        char *depnam_ptr,
        enum deptyp_t deptyp,
        char *depdat_ptr);

static unsigned long    WRT_DEP_RECS    (struct scl_com *com_ptr);

static unsigned long    WRT_DEP_THRS    (struct scl_com *com_ptr);

static unsigned long    WRT_DEP_TSTS    (struct scl_com *com_ptr);

static unsigned long    WRT_SCRTOF_HDR  (struct scl_com *com_ptr);

static unsigned long    WRT_THRTOF_FILE (struct scl_com *com_ptr,
        struct thrtbl *thr_ptr);

static unsigned long    WRT_THRTOF_HDR  (struct scl_com *com_ptr,
        struct thrtbl *thr_ptr);

static unsigned long    WRT_TOF_CHRLIS  (struct scl_com *com_ptr,
        struct symtbl *var_ptr);

static unsigned long    WRT_TOF_COD (struct scl_com *com_ptr);

static unsigned long    WRT_TOF_FILS    (struct scl_com *com_ptr);

static unsigned long    WRT_TOF_FIL (struct scl_com *com_ptr,
                                     struct fattbl *fat_ptr);

static unsigned long    WRT_TOF_FLDDEF  (struct scl_com *com_ptr,
        struct symtbl *var_ptr);

static unsigned long    WRT_TOF_FLDDEFFOFF (
    struct scl_com *com_ptr,
    struct symtbl *var_ptr);

static unsigned long    WRT_TOF_FLDDEFREC (
    struct scl_com *com_ptr,
    struct symtbl *var_ptr);

static unsigned long    WRT_TOF_INTLIS  (struct scl_com *com_ptr,
        struct symtbl *var_ptr);

static unsigned long    WRT_TOF_INTRNG  (struct scl_com *com_ptr,
        struct symtbl *var_ptr);

static unsigned long    WRT_TOF_PIPS    (struct scl_com *com_ptr);

static unsigned long    WRT_TOF_PIP (struct scl_com *com_ptr,
                                     struct fattbl *fat_ptr);

static unsigned long    WRT_TOF_PIPOPT  (struct scl_com *com_ptr,
        struct fattbl *fat_ptr);

static unsigned long    WRT_TOF_RECDEF  (struct scl_com *com_ptr,
        struct symtbl *var_ptr);

static unsigned long    WRT_TOF_RECHDR (struct scl_com *com_ptr,
                                        char *usrlin_ptr,
                                        PSL_U_LONGWORD_T tof_srclin);

static unsigned long    WRT_TOF_SCRRECS (struct scl_com *com_ptr);

static unsigned long    WRT_TOF_SUBTST  (struct scl_com *com_ptr,
        struct tsttbl *tst_ptr);

static unsigned long    WRT_TOF_THRRECS (struct scl_com *com_ptr,
        struct thrtbl *thr_ptr);

static unsigned long    WRT_TOF_THR (struct scl_com *com_ptr,
                                     struct thrtbl *thr_ptr);

static unsigned long    WRT_TOF_TIMS    (struct scl_com *com_ptr);

static unsigned long    WRT_TOF_TIM (struct scl_com *com_ptr,
                                     struct timtbl *tim_ptr);

static unsigned long    WRT_TOF_TSTS    (struct scl_com *com_ptr);

static unsigned long    WRT_TOF_TST (struct scl_com *com_ptr,
                                     struct tsttbl *tst_ptr);

static unsigned long    WRT_TOF_TSTPRMS (struct scl_com *com_ptr,
        struct tsttbl *tst_ptr);

static unsigned long    WRT_TOF_TSTTHRS (struct scl_com *com_ptr);

static unsigned long    WRT_TOF_TSTTHR  (struct scl_com *com_ptr,
        struct tstthrtbl *tstthr_ptr);

static unsigned long    WRT_TOF_THRPRM  (struct scl_com *com_ptr,
        struct tdp *tdp_ptr);

static unsigned long    WRT_TOF_VARS    (struct scl_com *com_ptr);

static unsigned long    WRT_TOF_VAR (struct scl_com *com_ptr,
                                     struct symtbl *var_ptr);

static unsigned long    WRT_TOF_VARDEF  (struct scl_com *com_ptr,
        struct symtbl *var_ptr);

static unsigned long    WRT_TOF_VARDEFDIM (
    struct scl_com *com_ptr,
    struct symtbl *var_ptr);

static unsigned long    WRT_TOF_VARDEFVAL (
    struct scl_com *com_ptr,
    struct symtbl *var_ptr);

static unsigned long    WRT_TOF_VARNAM (struct scl_com *com_ptr,
                                        struct symtbl *var_ptr);

static unsigned long    WRT_TOF_VAROPT (struct scl_com *com_ptr,
                                        struct symtbl *var_ptr);

static unsigned long    WRT_TOF_VARSIZ (struct scl_com *com_ptr,
                                        struct symtbl *var_ptr);
/*
 *  Structure definitions and parameters.
 */
#include <shrmsg.h>         /* PSL messages */
#include "scl_cmd.h"            /* Command Line Data Structure */
#include "scl_comdef.h"         /* Global Data structure */
#include "scl_tof.h"            /* SCL TOF data */
#include "scl_tpacom.h"         /* Tparse common Data */
#include <psldate_prmdef.h>     /* PSL date/time symbols */
#include <psldiag_prmdef.h>     /* Diagnostics paramters */
#include <pslspn_prmdef.h>      /* PSL_SPAWN symbols */
#include <pslfil_prmdef.h>      /* PSL_FILE symbols */
#include <psl_trc_prmdef.h>     /* PSL tracing parameters */
#include <psl_vm_prmdef.h>      /* PSL linked list arameters */
#include "scl_tlodef.h"         /* Test structures */
#include <scpdef.h>         /* SCP Header record */
#include <tofprmdef.h>          /* TOF parameters */
#include <vtest_prmdef.h>       /* VTEST Parameters */
#include "scl_tbldef.h"         /* SCL table definitions */

/******************************************************************************
 * @func   This routine writes a Vardef options record to the script object
 *      ('tof') file.
 *
 * @rdesc   Unsigned long PSL routine return status.
 * @parm    Pointer to the scl common data structure.
 * @parm    Pointer to variable's symbol table entry.
 */
static void GET_VAROPT (struct scl_com *com_ptr,
                        struct symtbl *var_ptr,
                        unsigned long *optval_ptr)
{
    struct scl_tpa   *tpa_ptr;      /* Tparse data */
    unsigned long    optval = 0;        /* Options value */

    /*
     *  Copy pointers for ease of reference.
     */
    tpa_ptr = (*com_ptr).tpa_ptrps; /* Tparse data pointer */

    /*
     *  Set the datatype bits.
     */
    if ((*var_ptr).var_typel == DTYP_CHARACTER)
    {
        optval |= VTCHAR;
    }
    else if ((*var_ptr).var_typel == DTYP_INTEGER)
    {
        optval |= VTINT;
    }

    /*
     *  Set the data source bits.
     */
    if (((*var_ptr).var_optl & VOPT_LIST) != 0)
    {
        optval |= VSLIST;
    }
    else if (((*var_ptr).var_optl & VOPT_RANGE) != 0)
    {
        optval |= VSRANGE;
    }
    else
    {
        optval |= VSNONE;
    }

	if ((*var_ptr).var_optl & VOPT_TESTWIDE)
    {
        optval |= VOTESTWIDE;
    }
    /*
     *  Set the selection order bits.
     *  The order is 'repeatable random' if specified as such. The order is
     *  'random' if a value list or range has been specified AND it is NOT explicitly
     *  specified as 'repeatable random'. The 'auto-next' option is always disabled
     *  by SCL - this flag distiguishes between the SCL way of handling value lists
     *  and ranges and the old SQLR way of handling them, which must be maintained for
     *  upwards compatibility.
     */
    if (((*var_ptr).var_optl & VOPT_RANDOM) != 0)
    {
        optval |= VOREPRAND;
    }
    else if (((*var_ptr).var_optl & (VOPT_LIST|VOPT_RANGE)) != 0)
    {
        optval |= VORAND;
    }

    optval |= VONOAUTONEXT;

    /*
     *  Set the variable scope bits.
     */
    if ((*var_ptr).var_addrl == ADDR_GLOBAL)
    {
        optval |= VCGLOBAL;
    }
    else if ((*var_ptr).var_addrl == ADDR_LOCAL)
    {
        optval |= VCLOCAL;
    }
    else if ((*var_ptr).var_addrl == ADDR_SCRIPT)
    {
        optval |= VCSCRIPT;
    }
    else if ((*var_ptr).var_addrl == ADDR_THREAD)
    {
        optval |= VCTHREAD;
    }


    /*
     *  Set noecho value bit.
     */
    if (((*var_ptr).var_optl & VOPT_ECHO) != 0) optval |= VONOECHO;

    /*
     *  Set uppercase value bit.
     */
    if (((*var_ptr).var_optl & VOPT_UPPERCASE) != 0) optval |= VOUPPERCASE;

    /*
     *  Set file variable bit.
     */
    if (((*var_ptr).var_optl & VOPT_FILE) != 0) optval |= VSFILE;

    /*
     *  Return the option value.
     */
    *optval_ptr = optval;

    return;
}

/******************************************************************************
 * @func    This routine creates and writes a dependency file for script and
 *      test type scripts. It also logs an informational message if the
 *      file is created successfully and -v has been specified.
 *
 * @rdesc   Unsigned long PSL routine return status:
 *      PSL_NORMAL      - Success
 *      PSL_OPENOUT     - Unable to create file.
 *      PSL_CLOSEOUT    - Unable to close file.
 * @parm    Pointer to the scl common data structure.
 */
unsigned long WRT_DEP_FILE (struct scl_com *com_ptr)
{
    /*
     *  Local Data.
     */
    char    *file_ptr;  /* Pointer to file name */
    char    *related_ptr;   /* Pointer to related file name */
    char    *expdep_ptr;    /* Pointer to expanded dependency file name */
    char    relext [8]; /* Expected related file extension string */
    char    deffil [8]; /* Default filename */
    char    *nam_ptr;   /* Pointer to file name */
    char    *namend_ptr;    /* Pointer to file name end */
    long    cstatus;    /* C routine return status */
    struct scl_cmd  *scl_cmd_ptr;   /* Pointer to command line structure */

    /*
     *  Copy pointers for ease of reference.
     */
    scl_cmd_ptr = (*com_ptr).cmd_ptrps;

    /*
     *  Set up the expanded dependency file specification in expdep_ptr.
     */
    file_ptr    = (*com_ptr).object_valuec256;
    /* Identify file to open */
    related_ptr = (*com_ptr).scl_namec256;  /* Identify related file */
    expdep_ptr  = (*com_ptr).dep_namec256;  /* Expanded dependency file name */

    if ((*com_ptr).type_testn)          /* If test script */
    {
        (void) strcpy (relext, ".tss");     /* Related file extension */
        (void) strcpy (deffil, ".tsd");     /* Default filename */
    }
    else                    /* Else must be script script */
    {
        (void) strcpy (relext, ".scl");     /* Related file extension */
        (void) strcpy (deffil, ".scd");     /* Default filename */
    }

    PSL_FILE_PARSE (file_ptr, deffil, related_ptr, relext, expdep_ptr, 255, 0);

    /*
     *  expdep_ptr now points to the expanded dependency file name. However, because
     *  we use the -o switch for both the object file name (TOF) AND the dependency
     *  filename, we could end up with the same name for both files, e.g. if an explicit
     *  filename (not just a pathname) has been specified using -o. To overcome this we
     *  explicitly strip off ANY extension from the expanded dependency filename and
     *  append the dependency extension. This is not very elegant, but it should have
     *  the desired effect.
     *
     *  The following filename formats are assumed:
     *
     *  WIN32: ...\scriptname{.extension}
     *     scriptname is the string between the last '\' and the next '.'.
     *  UNIX: .../scriptname{.extension}
     *     scriptname is the string between the last '/' and the next '.'.
     */
#ifdef WIN32
    nam_ptr = strrchr (expdep_ptr, '\\');
#elif UNIX
nam_ptr = strrchr (expdep_ptr, '/');
#else
#  error Platform not supported at line __LINE__ (Cannot obtain dependency file name)
#endif

    if (nam_ptr != NULL)            /* If end of path found */
    {
        nam_ptr++;              /* ...skip end of path delimiter */
        namend_ptr = strchr (nam_ptr, '.'); /* ...look for file extension */
        if (namend_ptr != NULL)         /* If file extension found */
        {
            *namend_ptr = '\0';         /* ...strip off file extension */
        }

        (void) strcat (expdep_ptr, deffil); /* Append dependency extension */
    }

    /*
     *  Open dependency file for write access - report error and return if open fails.
     */
    (*com_ptr).files.ptrpf14 [DEP_UNIT] = fopen (expdep_ptr, "wp");

    if ((*com_ptr).files.ptrpf14 [DEP_UNIT] == NULL)
    {
        PSL_LOG_CIOERR ((*com_ptr).diag_ptrps,
                        PSL_OPENOUT,
                        expdep_ptr);
        PSL_OUTPUT_DIAG ((*com_ptr).diag_ptrps, 0, DIOUFL_LSTERR);
        return PSL_OPENOUT;         /* Return with status */
    }

    /*
     *  Report an informational message, indicating that the script dependency file has
     *  been created, if -v was specified.
     */
    if ((*scl_cmd_ptr).logn)
    {
        PSL_PUTDIAG ((*com_ptr).diag_ptrps, 1, PSL_CREATED, expdep_ptr);
        PSL_OUTPUT_DIAG ((*com_ptr).diag_ptrps, 0, DIOUFL_LSTERR);
    }

    /*
     *  Write the records to the dependency file.
     */
    (void) WRT_DEP_RECS (com_ptr);

    /*
     *  Close dependency file - report error and return if close fails.
     */
    cstatus = fclose ((*com_ptr).files.ptrpf14 [DEP_UNIT]);

    if (cstatus == EOF)
    {
        PSL_LOG_CIOERR ((*com_ptr).diag_ptrps, PSL_CLOSEOUT,
                        (*com_ptr).dep_namec256);
        PSL_OUTPUT_DIAG ((*com_ptr).diag_ptrps, 0, DIOUFL_LSTERR);
        return PSL_CLOSEOUT;        /* Return with status */
    }

    return PSL_NORMAL;
}

/******************************************************************************
 * @func    This routine writes a line to the script dependency file.
 *
 * @rdesc   Unsigned long PSL routine return status:
 *      <PRE>
 *      PSL_NORMAL - Success
 *      PSL_WRITEERR - Error writing line to file
 *      </PRE>
 *
 * @parm    Pointer to scl common data structure.
 * @parm    Pointer to dependency name.
 * @parm    Dependency type.
 * @parm    Dependency type specific data (excluding leading separator character).
 */
static unsigned long WRT_DEP_LINE (struct scl_com *com_ptr,
                                   char *depnam_ptr,
                                   enum deptyp_t deptyp,
                                   char *depdat_ptr)
{
    /*
     *  Define the local data.
     */
    char    deplin [DEPBUFLEN];    /* Dependency line buffer */
    unsigned long    iret;          /* PSL routine return status */

    /*
     *  Create dependency line.
     */
    (void) sprintf (deplin, "%s%c%d", depnam_ptr, DEP_SEPCHR, deptyp);

    if (depdat_ptr != NULL) /* If pointer to some dependency data */
    {
        if (strlen(depdat_ptr) != 0) /* Dependency data is not a NULL string */
        {
            (void) strcat (deplin, DEP_SEPCHRS);
            (void) strcat (deplin, depdat_ptr);
        }
    }

    /*
     *  Write the line to the dependency file.
     */
    (void) fprintf ((*com_ptr).files.ptrpf14 [DEP_UNIT], "%s\n", deplin);

    if (ferror ((*com_ptr).files.ptrpf14 [DEP_UNIT]) != 0)
    {
        iret = PSL_WRITEERR;
        PSL_LOG_CIOERR ((*com_ptr).diag_ptrps,
                        iret,
                        (*com_ptr).obj_namec256);
        PSL_OUTPUT_DIAG ((*com_ptr).diag_ptrps, 0, DIOUFL_LSTERR);
    }
    else
    {
        iret = PSL_NORMAL;
    }

    /*
     *  Return with status.
     */
    return iret;
}

/******************************************************************************
 * @func   This routine writes all the records to the output type test or
 *      type script dependency file. A dependency is a simple, 'flat',
 *      ASCII text file. It has one line per dependency and each line has
 *      the following format:
 *
 *      <PRE>
 *      <dep_name>|<dep_type>
 *      </PRE>
 *
 *      e.g. MYTEST|4 (test "MYTEST").
 *
 *      Most dependencies are held in the script dependency linked list.
 *      However, for test scripts, test and thread dependencies are held
 *      in the test and test thread symbol tables.
 *
 * @rdesc   Unsigned long PSL routine return status.
 * @parm    Pointer to the scl common data structure.
 */
static unsigned long WRT_DEP_RECS (struct scl_com *com_ptr)
{
    /*
     *  Local data.
     */
    struct sdep  *dep_ptr;      /* Script dependency pointer */
    struct sdep  depbuf;        /* Script dependency buffer */
    unsigned long    iret;          /* Status return */
    unsigned long    isub;          /* Loop control */
    struct scl_cmd   *scl_cmd_ptr;      /* Pointer to command line structure */

    /*
     *  Copy pointers for ease of reference.
     */
    scl_cmd_ptr = com_ptr->cmd_ptrps;
    dep_ptr = &depbuf;

    /*
     *  Write type test specific dependency records.
     */
    if (com_ptr->type_testn)
    {
        iret = WRT_DEP_TSTS (com_ptr);              /* Write Test dependencies */
        if (!psl_test_status (iret)) goto wrt_dep_recs_end;

        iret = WRT_DEP_THRS (com_ptr);              /* Write thread dependencies */
        if (!psl_test_status (iret)) goto wrt_dep_recs_end;
    }

    /*
     *  Write a dependency record for each entry in the dependency list and when we have
     *  finished with the list, delete it.
     */
    if ((*com_ptr).dep_llps != NULL)
    {
        for (isub = 0; isub < (*com_ptr).dep_cntl; isub++)
        {
            iret = PSL_VM_REMOVE ((*com_ptr).dep_llps,
                                  (char *) &depbuf,
                                  NULL, NULL, PSL_VM_POS_BEG);
            if (psl_test_status (iret))
            {
                iret = WRT_DEP_LINE (com_ptr, (*dep_ptr).sdep_namec256,
                                     (*dep_ptr).sdep_typl, (*dep_ptr).sdep_datac256);
                if (!psl_test_status (iret)) goto wrt_dep_recs_end;
            }
        }

        PSL_VM_DELETE ((*com_ptr).dep_llps);
        (*com_ptr).dep_llps = NULL;
    }

    iret = PSL_NORMAL;

    /*
     *  All done - return with status.
     */
wrt_dep_recs_end:
    return iret;
}

/******************************************************************************
 * @func    This routine writes the thread dependency records to the script
 *      dependency file.
 *
 * @rdesc   Unsigned long PSL routine return status.
 * @parm    Pointer to the scl common data structure.
 */
static unsigned long WRT_DEP_THRS (struct scl_com *com_ptr)
{
    /*
     *  Local data.
     */
    struct scl_tpa   *tpa_ptr;      /* Tparse data */
    struct tstthrtbl *tstthr_ptr;       /* Test thread symbol table entry */
    struct tdf       *tdf_ptr;      /* Thread data */
    unsigned long    iret;          /* PSL routine return status */
    unsigned long    tstthr_cnt;        /* Test thread count */
    char    depdat [DEPBUFLEN];    /* Dependency specific data buffer */

    /*
     *  Copy pointers for ease of reference.
     */
    tpa_ptr = com_ptr->tpa_ptrps;   /* Tparse data pointer */

    /*
     *  Loop through the test thread symbol table writing a dependency record for
     *  each externally defined test thread that has been referenced by the script.
     */
    for (tstthr_ptr = tpa_ptr->tstthr_begps,
            tstthr_cnt = 1;
            tstthr_cnt <= tpa_ptr->tstthr_cntl;
            tstthr_ptr++, tstthr_cnt++)
    {
        if (!(tstthr_ptr->tstthr_defn)      /* If not locally defined test thread */
                &&  (tstthr_ptr->tstthr_exe_refn))  /* and test thread has been referenced */
        {
            tdf_ptr = tstthr_ptr->tstthr_tdf_ptrps;
            (void) sprintf (depdat, "%d", tdf_ptr->tdf_targetl);
            iret = WRT_DEP_LINE (com_ptr, tstthr_ptr->tstthr_namec68, DEP_TYP_THREAD, depdat);
            if (!psl_test_status (iret)) goto wrt_dep_thrs_end;
        }
    }

    iret = PSL_NORMAL;

    /*
     *  All done - return with status.
     */
wrt_dep_thrs_end:
    ;
    return iret;
}

/******************************************************************************
 * @func    This routine writes the test dependency records to the dependency
 *      file.
 *
 * @rdesc   Unsigned long PSL routine return status.
 * @parm    Pointer to the scl common data structure.
 */
static unsigned long WRT_DEP_TSTS (struct scl_com *com_ptr)
{
    /*
     *  Local data.
     */
    struct scl_tpa   *tpa_ptr;      /* Tparse data */
    struct tsttbl    *tst_ptr;      /* Test symbol table entry */
    unsigned long    iret;          /* PSL routine return status */
    unsigned long    tst_cnt;       /* Test count */

    /*
     *  Copy pointers for ease of reference.
     */
    tpa_ptr = com_ptr->tpa_ptrps;   /* Tparse data pointer */

    /*
     *  Loop through the test symbol table writing a dependency record for each
     *  external test declaration that has been referenced within the script.
     */
    for (tst_ptr = tpa_ptr->tst_begps,
            tst_cnt = 1;
            tst_cnt <= tpa_ptr->tst_cntl;
            tst_ptr++, tst_cnt++)
    {
        if (!(tst_ptr->tst_subtstn) /* If an EXTERNAL TEST declaration */
                &&  (tst_ptr->tst_exe_refn))    /* ...that has been referenced */
        {
            iret = WRT_DEP_LINE (com_ptr, tst_ptr->tst_namec68, DEP_TYP_TEST, NULL);
            if (!psl_test_status (iret)) goto wrt_dep_tsts_end;
        }
    }

    iret = PSL_NORMAL;

    /*
     *  All done - return with status.
     */
wrt_dep_tsts_end:
    ;
    return iret;
}

/******************************************************************************
 * @func    This routine creates and writes a script object file for script and
 *      test type scripts. It also logs an informational message if the
 *      file is created successfully and -v has been specified.
 *
 * @rdesc   Unsigned long PSL routine return status:
 *      PSL_NORMAL      - Success
 *      PSL_OPENOUT     - Unable to create file.
 *      PSL_CLOSEOUT    - Unable to close file.
 * @parm    Pointer to the scl common data structure.
 */
unsigned long WRT_SCRTOF_FILE (struct scl_com *com_ptr)
{
    /*
     *  Local Data.
     */
    char    *file_ptr;  /* Pointer to file name */
    char    *related_ptr;   /* Pointer to related file name */
    char    *expobj_ptr;    /* Pointer to expanded object file name */
    char    relext [8]; /* Expected related file extension string */
    char    defnam [8]; /* Default file name */
    long    cstatus;    /* C routine return status */
    struct scl_cmd  *scl_cmd_ptr;   /* Pointer to command line structure */

    /*
     *  Copy pointers for ease of reference.
     */
    scl_cmd_ptr = (*com_ptr).cmd_ptrps;

    /*
     *  Set up the expanded object file specification in expobj_ptr.
     */
    file_ptr    = (*com_ptr).object_valuec256;
    /* Identify file to open */
    related_ptr = (*com_ptr).scl_namec256;  /* Identify related file */
    expobj_ptr  = (*com_ptr).obj_namec256;  /* Expanded object file name */

    if ((*com_ptr).type_testn)          /* If test script */
    {
        (void) strcpy (relext, ".tss");
        (void) strcpy (defnam, ".tso");     /* Test object 'TOF' output */
    }
    else                    /* Else must be script script */
    {
        (void) strcpy (relext, ".scl");
        (void) strcpy (defnam, ".tof");     /* 'Regular' 'TOF' output */
    }

    PSL_FILE_PARSE (file_ptr, defnam, related_ptr, relext, expobj_ptr, 255, 0);

    /*
     *  Open object file for write access - report error and return if open fails.
     */
    (*com_ptr).files.ptrpf14 [OBJ_UNIT] = fopen (expobj_ptr, "wp");

    if ((*com_ptr).files.ptrpf14 [OBJ_UNIT] == NULL)
    {
        PSL_LOG_CIOERR ((*com_ptr).diag_ptrps,
                        PSL_OPENOUT,
                        expobj_ptr);
        PSL_OUTPUT_DIAG ((*com_ptr).diag_ptrps, 0, DIOUFL_LSTERR);
        return PSL_OPENOUT;         /* Return with status */
    }

    /*
     *  Report an informational message, indicating that the script object file has
     *  been created, if -v was specified.
     */
    if ((*scl_cmd_ptr).logn)
    {
        PSL_PUTDIAG ((*com_ptr).diag_ptrps, 1, PSL_CREATED,
                     (*com_ptr).obj_namec256);
        PSL_OUTPUT_DIAG ((*com_ptr).diag_ptrps, 0, DIOUFL_LSTERR);
    }

    /*
     *  Write the records to the script object ('tof') file.
     */
    (void) WRT_TOF_SCRRECS (com_ptr);

    /*
     *  Close object file - report error and return if close fails.
     */
    cstatus = fclose ((*com_ptr).files.ptrpf14 [OBJ_UNIT]);

    if (cstatus == EOF)
    {
        PSL_LOG_CIOERR ((*com_ptr).diag_ptrps, PSL_CLOSEOUT,
                        (*com_ptr).scl_namec256);
        PSL_OUTPUT_DIAG ((*com_ptr).diag_ptrps, 0, DIOUFL_LSTERR);
        return PSL_CLOSEOUT;        /* Return with status */
    }

    /*
     *  Exit routine.
     */
    return PSL_NORMAL;
}

/******************************************************************************
 * @func    This routine writes the 'tof' header record for a thread script or
 *      test script to the script object ('tof') file. The header record
 *      consists of the following lines:
 *
 *      <PRE>
 *      Format:             Example:
 *      tof.<TOF_version>           tof.7
 *      dbtype <TOF_ID>         dbtype 12
 *      date dd-mmm-yyyy hh:mm:ss       date 26-may-1999 14:38:00
 *      name <script_name>          name Test1
 *      description <script_description>    description My 1st test script
 *      </PRE>
 *
 * @rdesc   Unsigned long PSL routine return status.
 * @parm    Pointer to the scl common data structure.
 */
static unsigned long WRT_SCRTOF_HDR (struct scl_com *com_ptr)
{
    /*
     *    Local data.
     */
    struct scl_tpa   *tpa_ptr;      /* Tparse data */
    struct scl_tofh  *tofh_ptr;     /* 'tof' header record data pointer */
    struct scph  *scph_ptr;     /* SCPH pointer */
    unsigned long    iret;          /* PSL routine return status */
    unsigned long    psldat;        /* PSL date */
    unsigned long    psltim;        /* PSL time */
    unsigned long    tofid;         /* TOF ID */
    char    datbuf[24];        /* ASCII date/time buffer */
    char    *lin_ptr;      /* Pointer to line buffer */

    /*
     *  Copy pointers for ease of reference.
     */
    tpa_ptr = (*com_ptr).tpa_ptrps; /* Tparse data pointer */
    tofh_ptr = (*tpa_ptr).tofh_begps;   /* 'tof' header record data pointer */
    lin_ptr = (*com_ptr).toflinpc;   /* 'tof' line buffer pointer */
    scph_ptr = (*tpa_ptr).header_begps; /* Pick up script header pointer */

    /*
     *  Write TOF version record:
     *
     *  tof.<TOF_version>
     */
    (void) sprintf (lin_ptr, "tof.%s", TOF_CURVERS);
    iret = WRT_TOF_LINE (com_ptr, NULL);
    if (!psl_test_status (iret)) goto wrt_scrtof_hdr_err;

    /*
     *  Write database type record:
     *
     *  dbtype <TOF_ID>
     *
     *  Yes, this should contain the <TOF_ID>. The field name is now something of
     *  a misnomer as the field does NOT specify the database type, rather it
     *  specifies the TOF type.
     */
    if ((*com_ptr).type_testn)
    {
        tofid = TOF_ID_TEST;        /* Test script */
    }
    else if (((*scph_ptr).scph_mode_b & MODE_CHR_CELL) != 0)
    {
        tofid = TOF_ID_CHARCELL;    /* Character cell thread script */
    }
    else
    {
        tofid = TOF_ID_COMMAND;     /* Command mode thread script */
    }

    (void) sprintf (lin_ptr, "dbtype %d", tofid);
    iret = WRT_TOF_LINE (com_ptr, NULL);
    if (!psl_test_status (iret)) goto wrt_scrtof_hdr_err;

    /*
     *  Write date record. This contains the current date/time:
     *
     *  date dd-mmm-yyyy hh:mm:ss
     */
    PSL_GET_DATTIM (&psldat, &psltim);
    PSL_FMT_DATTIM (psldat, psltim, datbuf, sizeof (datbuf), PSDTYP_PSL,
                    (PSDFMT_DATE | PSDFMT_TIME | PSDFMT_CENTURY));
    (void) sprintf (lin_ptr, "date %s", datbuf);
    iret = WRT_TOF_LINE (com_ptr, NULL);
    if (!psl_test_status (iret)) goto wrt_scrtof_hdr_err;

    /*
     *  Write script name record.
     *
     *  name <script_name>
     */
    (void) sprintf (lin_ptr, "name %s", (*com_ptr).scrnam_c20);
    iret = WRT_TOF_LINE (com_ptr, NULL);
    if (!psl_test_status (iret)) goto wrt_scrtof_hdr_err;

    /*
     *  Write description record.
     *
     *  description <script_description>
     */
    (void) sprintf (lin_ptr, "description %s", (*tofh_ptr).tofh_sdsc_c52);
    iret = WRT_TOF_LINE (com_ptr, NULL);
    if (!psl_test_status (iret)) goto wrt_scrtof_hdr_err;

    /*
     *  If this is a test type script, write any test parameter records.
     */
    if ((*com_ptr).type_testn)
    {
        iret = WRT_TOF_TSTPRMS (com_ptr, (*tpa_ptr).tst_thisps);
        if (!psl_test_status (iret)) goto wrt_scrtof_hdr_err;
    }

    /*
     *    All done - return success
     */
    return PSL_NORMAL;

    /*
     *    Error return.
     */
wrt_scrtof_hdr_err:
    return iret;
}

/******************************************************************************
 * @func    This routine creates and writes a script object file for the
 *      supplied thread definition. It also logs an informational message
 *      if the file is created successfully and -v has been specified.
 *
 * @rdesc   Unsigned long PSL routine return status:
 *      PSL_NORMAL      - Success
 *      PSL_OPENOUT     - Unable to create file.
 *      PSL_CLOSEOUT    - Unable to close file.
 * @parm    Pointer to the scl common data structure.
 * @parm    Pointer to Thread Symbol Table entry for thread definition to be
 *      written to file.
 */
static unsigned long WRT_THRTOF_FILE (struct scl_com *com_ptr,
                                      struct thrtbl *thr_ptr)
{
    /*
     *  Local Data.
     */
    char    *file_ptr;  /* Pointer to file name */
    char    *expobj_ptr;    /* Pointer to expanded object file name */
    long    cstatus;    /* C routine return status */
    char    deffil [256];   /* Default file name */
    struct scl_cmd  *scl_cmd_ptr;   /* Pointer to command line structure */

    /*
     *  Copy pointers for ease of reference.
     */
    scl_cmd_ptr = (*com_ptr).cmd_ptrps;

    /*
     *  Set up the expanded object file specification in expobj_ptr.
     */
    file_ptr    = (*com_ptr).object_valuec256;  /* Identify file to open */
    expobj_ptr  = (*com_ptr).obj_namec256;  /* Expanded object file name */
    (void) sprintf (deffil, "%s.tho", (*thr_ptr).thr_namec20);

    PSL_FILE_PARSE (file_ptr, deffil, NULL, NULL, expobj_ptr, 255, 0);

    /*
     *  Open object file for write access - report error and return if open fails.
     */
    (*com_ptr).files.ptrpf14 [OBJ_UNIT] = fopen (expobj_ptr, "wp");

    if ((*com_ptr).files.ptrpf14 [OBJ_UNIT] == NULL)
    {
        PSL_LOG_CIOERR ((*com_ptr).diag_ptrps,
                        PSL_OPENOUT,
                        expobj_ptr);
        PSL_OUTPUT_DIAG ((*com_ptr).diag_ptrps, 0, DIOUFL_LSTERR);
        return PSL_OPENOUT;         /* Return with status */
    }

    /*
     *  Report an informational message, indicating that the script object file has
     *  been created, if -v was specified.
     */
    if ((*scl_cmd_ptr).logn)
    {
        PSL_PUTDIAG ((*com_ptr).diag_ptrps, 1, PSL_CREATED,
                     (*com_ptr).obj_namec256);
        PSL_OUTPUT_DIAG ((*com_ptr).diag_ptrps, 0, DIOUFL_LSTERR);
    }

    /*
     *  Write the records to the script object ('tof') file.
     */
    (void) WRT_TOF_THRRECS (com_ptr, thr_ptr);

    /*
     *  Close object file - report error and return if close fails.
     */
    cstatus = fclose ((*com_ptr).files.ptrpf14 [OBJ_UNIT]);

    if (cstatus == EOF)
    {
        PSL_LOG_CIOERR ((*com_ptr).diag_ptrps, PSL_CLOSEOUT,
                        (*com_ptr).scl_namec256);
        PSL_OUTPUT_DIAG ((*com_ptr).diag_ptrps, 0, DIOUFL_LSTERR);
        return PSL_CLOSEOUT;        /* Return with status */
    }

    /*
     *  Exit routine.
     */
    return PSL_NORMAL;
}

/******************************************************************************
 * @func    This routine creates and writes a script object file for each
 *      thread definition in a thread type script. It also logs an
 *      informational message if the file is created successfully and -v
 *      has been specified.
 *
 * @rdesc   Unsigned long PSL routine return status:
 *      PSL_NORMAL      - Success
 *      PSL_OPENOUT     - Unable to create file.
 *      PSL_CLOSEOUT    - Unable to close file.
 * @parm    Pointer to the scl common data structure.
 */
unsigned long WRT_THRTOF_FILES (struct scl_com *com_ptr)
{
    /*
     *  Local data.
     */
    struct scl_tpa   *tpa_ptr;      /* Tparse data */
    struct thrtbl    *thr_ptr;      /* Thread symbol table entry */
    unsigned long    iret;          /* PSL routine return status */
    unsigned long    thr_cnt;       /* Test thread count */

    /*
     *  Copy pointers for ease of reference.
     */
    tpa_ptr = com_ptr->tpa_ptrps;   /* Tparse data pointer */

    /*
     *  Loop through the thread symbol table writing a thread definition
     *  file for each entry.
     */
    for (thr_ptr = tpa_ptr->thr_begps,
            thr_cnt = 1;
            thr_cnt <= tpa_ptr->thr_cntl;
            thr_ptr++, thr_cnt++)
    {
        iret = WRT_THRTOF_FILE (com_ptr, thr_ptr);
        if (!psl_test_status (iret)) goto wrt_thrtof_files_end;
    }

    iret = PSL_NORMAL;

    /*
     *  All done - return with status.
     */
wrt_thrtof_files_end:
    ;
    return iret;
}
/******************************************************************************
 * @func    This routine writes the 'tof' header record for a thread definition
 *      script to the script object ('tof') file. The header record
 *      consists of the following lines:
 *
 *      <PRE>
 *      Format:             Example:
 *      tof.<TOF_version>           tof.7
 *      dbtype <TOF_ID>         dbtype 13
 *      date dd-mmm-yyyy hh:mm:ss       date 26-may-1999 14:38:00
 *      name <script_name>          name Test1
 *      </PRE>
 *
 * @rdesc   Unsigned long PSL routine return status.
 * @parm    Pointer to the scl common data structure.
 * @parm    Pointer to Thread Symbol Table entry for thread definition to be
 *      written to file.
 */
static unsigned long WRT_THRTOF_HDR (struct scl_com *com_ptr,
                                     struct thrtbl *thr_ptr)
{
    /*
     *    Local data.
     */
    struct scl_tpa   *tpa_ptr;      /* Tparse data */
    struct scl_tofh  *tofh_ptr;     /* 'tof' header record data pointer */
    unsigned long    iret;          /* PSL routine return status */
    unsigned long    psldat;        /* PSL date */
    unsigned long    psltim;        /* PSL time */
    char    datbuf[24];        /* ASCII date/time buffer */
    char    *lin_ptr;      /* Pointer to line buffer */

    /*
     *  Copy pointers for ease of reference.
     */
    tpa_ptr = (*com_ptr).tpa_ptrps; /* Tparse data pointer */
    tofh_ptr = (*tpa_ptr).tofh_begps;   /* 'tof' header record data pointer */
    lin_ptr = (*com_ptr).toflinpc;   /* 'tof' line buffer pointer */

    /*
     *  Write TOF version record:
     *
     *  tof.<TOF_version>
     */
    (void) sprintf (lin_ptr, "tof.%s", TOF_CURVERS);
    iret = WRT_TOF_LINE (com_ptr, NULL);
    if (!psl_test_status (iret)) goto wrt_thrtof_hdr_err;

    /*
     *  Write database type record:
     *
     *  dbtype TOF_ID_THREAD
     *
     *  Yes, this should contain the <TOF_ID>. The field name is now something of
     *  a misnomer as the field does NOT specify the database type, rather it
     *  specifies the TOF type.
     */
    (void) sprintf (lin_ptr, "dbtype %d", TOF_ID_THREAD);
    iret = WRT_TOF_LINE (com_ptr, NULL);
    if (!psl_test_status (iret)) goto wrt_thrtof_hdr_err;

    /*
     *  Write date record. This contains the current date/time:
     *
     *  date dd-mmm-yyyy hh:mm:ss
     */
    PSL_GET_DATTIM (&psldat, &psltim);
    PSL_FMT_DATTIM (psldat, psltim, datbuf, sizeof (datbuf), PSDTYP_PSL,
                    (PSDFMT_DATE | PSDFMT_TIME | PSDFMT_CENTURY));
    (void) sprintf (lin_ptr, "date %s", datbuf);
    iret = WRT_TOF_LINE (com_ptr, NULL);
    if (!psl_test_status (iret)) goto wrt_thrtof_hdr_err;

    /*
     *  Write script name record.
     *
     *  name <thread_name>
     */
    (void) sprintf (lin_ptr, "name %s", (*thr_ptr).thr_namec20);
    iret = WRT_TOF_LINE (com_ptr, NULL);
    if (!psl_test_status (iret)) goto wrt_thrtof_hdr_err;

    /*
     *    All done - return success
     */
    return PSL_NORMAL;

    /*
     *    Error return.
     */
wrt_thrtof_hdr_err:
    return iret;
}

/******************************************************************************
 * @func    This routine writes the 'list values' portion of a Vardef values
 *      record for a character variable to the script object ('tof')
 *      file. A Vardef values record has the following format:
 *
 *      <PRE>
 *      values <ltxt_delimiter>
 *      <values>            // This bit written by this routine
 *      <ltxt_delimiter>
 *      </PRE>
 *
 *      For a character list, <values> consists of a list of comma separated
 *      quoted character strings (delimited by either single or double
 *      quotes). These will be continued over multiple lines as necessary to
 *      fit within an arbitrarily fixed right margin, e.g.:
 *
 *      <PRE>
 *      "Lazaridis","Adebola","Bennett","Poole","Wreh","Marcello"
 *      "Francis"
 *      </PRE>
 *
 * @rdesc   Unsigned long PSL routine return status.
 * @parm    Pointer to the scl common data structure.
 * @parm    Pointer to variable's symbol table entry.
 */
static unsigned long WRT_TOF_CHRLIS (struct scl_com *com_ptr,
                                     struct symtbl *var_ptr)
{
    unsigned long    iret = PSL_NORMAL; /* PSL routine return status */
    struct scl_tpa   *tpa_ptr;      /* Tparse data */
    char    *lin_ptr;      /* Pointer to line buffer */
    char    *val_ptr;      /* Value pointer */
    char    *endval_ptr;       /* Pointer to byte beyond last value in list */
    long    vallen;        /* Value length */
    unsigned long    valpad;        /* Number of value padding characters */
    /*
     *  Create a static buffer to hold each value in encoded format. We do
     *  not know in advance how big the value is going to be so we allocate
     *  a buffer of the maximum size. As this is BIG, we do not want to
     *  allocate and deallocate it every time this routine is entered so
     *  we make it static.
     */
    static   char    valbuf [CHRBUFSIZ];    /* ASCII value buffer */

    /*
     *  Copy pointers for ease of reference.
     */
    tpa_ptr = com_ptr->tpa_ptrps;   /* Tparse data pointer */
    lin_ptr = com_ptr->toflinpc; /* 'tof' line buffer pointer */

    /*
     *  Read through the raw character value list and write out a comma separated
     *  list of quoted character string values. Flush the current line, if the next value
     *  will not fit on the current line.
     *
     *  The raw value list is a list of counted strings, in which a longword 'count'
     *  indicating the length of the value precedes each value. Each list entry is
     *  padded to ensure that the 'length' field is longword-aligned.
     */
    lin_ptr [0] = '\0';         /* Clear line buffer */
    endval_ptr = tpa_ptr->values_begpc + var_ptr->var_values_endl;

    for (val_ptr = tpa_ptr->values_begpc + var_ptr->var_values_offl;
            val_ptr < endval_ptr;
            val_ptr += (vallen + valpad))
    {
        vallen = (long) *val_ptr;           /* Get value length */
        val_ptr += sizeof (val_ptr);            /* Bump pointer past length field */

        CREATE_ENCCHRIMM (com_ptr, valbuf, sizeof(valbuf), val_ptr, vallen);
        /* Get value in encoded format */

        if ((strlen(lin_ptr) + (strlen(valbuf) + 1) > TOF_RIGHT_MARGIN) &&
            (strlen(lin_ptr) != 0))

        {
            (void) strcat (lin_ptr, ",");       /* Append comma to line buffer */
            iret = WRT_TOF_LINE (com_ptr, NULL);    /* Flush line buffer */
            if (!psl_test_status (iret)) goto wrt_tof_chrlis_end;
            lin_ptr [0] = '\0';             /* Clear line buffer */
        }

        if (strlen(lin_ptr) != 0) (void) strcat (lin_ptr, ",");
        (void) strcat (lin_ptr, valbuf);        /* Append value to line buffer */

        /* Calculate number of padding characters to align next vlaue on longword */
        valpad = (vallen % sizeof(long));       /* Get any remainder */
        valpad = ((sizeof(long) - valpad) % sizeof(long));  /* Convert to shortfall */
    }

    /*
     *  If anything left in line buffer, flush it to file.
     */
    if (strlen(lin_ptr) != 0)
    {
        iret = WRT_TOF_LINE (com_ptr, NULL);
        if (!psl_test_status (iret)) goto wrt_tof_chrlis_end;
    }

wrt_tof_chrlis_end:
    ;
    return iret;
}

/******************************************************************************
 * @func   This routine writes the 'tof' code object records to the script
 *      object ('tof') file.
 *
 * @rdesc   Unsigned long PSL routine return status.
 * @parm    Pointer to the scl common data structure.
 */
static unsigned long WRT_TOF_COD (struct scl_com *com_ptr)
{
    /*
     *    Local data.
     */
    struct scl_tpa   *tpa_ptr;      /* Tparse data */
    struct scl_tofl  *tofl_ptr;     /* Pointer TOF line data */
    unsigned  long   iret;          /* PSL routine status */

    /*
     *  Copy pointers for ease of reference.
     */
    tpa_ptr = (*com_ptr).tpa_ptrps; /* Tparse data pointer */

    /*
     *  If we do not have a TOF line linked list, all done.
     */
    if ((*tpa_ptr).tofl_llps == NULL) goto wrt_tof_cod_end;

    /*
     *  Loop through the TOF linked list writing TOF lines to the TOF file
     *  and deleting the list as we go.
     */
    do
    {
        iret = PSL_VM_REMOVE ((*tpa_ptr).tofl_llps,
                              (char *) &tofl_ptr,
                              NULL, NULL, PSL_VM_POS_BEG);
        if (psl_test_status (iret))
        {
            iret = WRT_TOF_LINE (com_ptr, tofl_ptr->tofl_linptrpc);
            if (!psl_test_status (iret)) goto wrt_tof_cod_err;

            FREE (tofl_ptr->tofl_linptrpc);
            FREE (tofl_ptr);

#ifdef PSL_TRACING
            if (PSL_TEST_TRACE (PTRC_USR_USER5))
            {
                PSL_OUTPUT_TRACE ("FREE: tofl - '%p' - WRT_TOF_COD\n", tofl_ptr);
            }
#endif  /* PSL_TRACING */
        }
    }
    while (psl_test_status (iret));     /* Loop while not an error */

    /*
     *  Delete the linked list.
     */
    (void) PSL_VM_DELETE ((*tpa_ptr).tofl_llps);
    (*tpa_ptr).tofl_llps = NULL;

    /*
     *  All done - return success
     */
wrt_tof_cod_end:
    ;
    return PSL_NORMAL;

    /*
     *  Error return.
     */
wrt_tof_cod_err:
    ;
    return iret;
}

/******************************************************************************
 * @func   This routine writes the 'tof' file definition records to the
 *      script object ('tof') file.
 *
 * @rdesc   Unsigned long PSL routine return status.
 * @parm    Pointer to the scl common data structure.
 */
static unsigned long WRT_TOF_FILS (struct scl_com *com_ptr)
{
    /*
     *  Local data.
     */
    struct scl_tpa   *tpa_ptr;      /* Tparse data */
    struct fattbl    *fat_ptr;      /* Files symbol table entry */
    unsigned long    iret;          /* PSL routine return status */
    unsigned long    file_cnt;      /* File count */

    /*
     *  Copy pointers for ease of reference.
     */
    tpa_ptr = com_ptr->tpa_ptrps;   /* Tparse data pointer */

    /*
     *  Loop through the files symbol table writing a file definition
     *  record for each entry.
     */
    for (fat_ptr = tpa_ptr->fat_begps,
            file_cnt = 1;
            file_cnt <= tpa_ptr->fat_cntl;
            fat_ptr++, file_cnt++)
    {
        iret = WRT_TOF_FIL (com_ptr, fat_ptr);
        if (!psl_test_status (iret)) goto wrt_tof_fils_end;
    }

    iret = PSL_NORMAL;

    /*
     *  All done - return with status.
     */
wrt_tof_fils_end:
    ;
    return iret;
}

/******************************************************************************
 * @func   This routine writes a single File record to the script object
 *      ('tof') file. A File record has the following format:
 *
 *      <PRE>
 *      Fildef
 *      name <file_id>
 *      </PRE>
 *
 * @rdesc   Unsigned long PSL routine return status.
 * @parm    Pointer to the scl common data structure.
 */
static unsigned long WRT_TOF_FIL (struct scl_com *com_ptr,
                                  struct fattbl *fat_ptr)
{
    struct scl_tpa   *tpa_ptr;      /* Tparse data */
    unsigned long    iret;          /* PSL routine return status */
    char    *lin_ptr;      /* Pointer to line buffer */

    /*
     *  Copy pointers for ease of reference.
     */
    tpa_ptr = com_ptr->tpa_ptrps;   /* Tparse data pointer */
    lin_ptr = com_ptr->toflinpc; /* 'tof' line buffer pointer */

    /*
     *  Write file definition header:
     *
     *  Fildef
     */
    iret = WRT_TOF_RECHDR (com_ptr, "Fildef", fat_ptr->fat_tof_srclinl);
    if (!psl_test_status (iret)) goto wrt_tof_fil_end;

    /*
     *  Write name record:
     *
     *  name <file_id>
     */
    (void) sprintf (lin_ptr, "name %s", fat_ptr->fat_namec20);
    iret = WRT_TOF_LINE (com_ptr, NULL);
    if (!psl_test_status (iret)) goto wrt_tof_fil_end;

    iret = PSL_NORMAL;

wrt_tof_fil_end:
    ;
    return iret;
}

/******************************************************************************
 * @func    This routine writes a single flddef record to the script object
 *      ('tof') file. A Flddef record has the following format:
 *
 *      <PRE>
 *      Flddef
 *      name <variable_name>
 *      options <option_value>
 *      size <variable_size>
 *      record <record_name>
 *      foffset <field_offset>
 *      </PRE>
 *
 * @rdesc   Unsigned long PSL routine return status.
 * @parm    Pointer to the scl common data structure.
 * @parm    Pointer to variable's entry in Variables Symbol Table.
 */
static unsigned long WRT_TOF_FLDDEF (struct scl_com *com_ptr,
                                     struct symtbl *var_ptr)
{
    struct scl_tpa   *tpa_ptr;      /* Tparse data */
    unsigned long    iret;          /* PSL routine return status */
    char    *lin_ptr;      /* Pointer to line buffer */

    /*
     *  Copy pointers for ease of reference.
     */
    tpa_ptr = com_ptr->tpa_ptrps;   /* Tparse data pointer */
    lin_ptr = com_ptr->toflinpc; /* 'tof' line buffer pointer */

    /*
     *  Write record definition header:
     *
     *  Flddef
     */
    iret = WRT_TOF_RECHDR (com_ptr, "Flddef", var_ptr->var_tof_srclinl);
    if (!psl_test_status (iret)) goto wrt_tof_flddef_end;

    /*
     *  Write variable name record.
     */
    iret = WRT_TOF_VARNAM (com_ptr, var_ptr);
    if (!psl_test_status (iret)) goto wrt_tof_flddef_end;

    /*
     *  Write options record.
     */
    iret = WRT_TOF_VAROPT (com_ptr, var_ptr);
    if (!psl_test_status (iret)) goto wrt_tof_flddef_end;

    /*
     *  Write size record.
     */
    iret = WRT_TOF_VARSIZ (com_ptr, var_ptr);
    if (!psl_test_status (iret)) goto wrt_tof_flddef_end;

    /*
     *  Write record name record.
     */
    iret = WRT_TOF_FLDDEFREC (com_ptr, var_ptr);
    if (!psl_test_status (iret)) goto wrt_tof_flddef_end;

    /*
     *  Write filed offset record.
     */
    iret = WRT_TOF_FLDDEFFOFF (com_ptr, var_ptr);
    if (!psl_test_status (iret)) goto wrt_tof_flddef_end;

    iret = PSL_NORMAL;
wrt_tof_flddef_end:
    ;
    return iret;
}

/******************************************************************************
 * @func    This routine writes a field definition 'field offset' record to the
 *      script object ('tof') file. A foffset record has the following
 *      format:
 *
 *      <PRE>
 *      foffset <field_offset>
 *      </PRE>
 *
 * @rdesc   Unsigned long PSL routine return status.
 * @parm    Pointer to the scl common data structure.
 * @parm    Pointer to variable's symbol table entry.
 */
static unsigned long WRT_TOF_FLDDEFFOFF (struct scl_com *com_ptr,
        struct symtbl *var_ptr)
{
    struct scl_tpa   *tpa_ptr;      /* Tparse data */
    struct symtbl    *rec_ptr;      /* Pointer to record variable's Symbol Table entry */
    unsigned long    iret;          /* PSL routine return status */
    char    *lin_ptr;      /* Pointer to line buffer */
    unsigned long    fldoff;        /* Field offset */

    /*
     *  Copy pointers for ease of reference.
     */
    tpa_ptr = com_ptr->tpa_ptrps;   /* Tparse data pointer */
    lin_ptr = com_ptr->toflinpc; /* 'tof' line buffer pointer */
    rec_ptr = tpa_ptr->variable_begps + var_ptr->var_recidxl;
    /* Pointer to record entry */

    /*
     *  Get field offset by subtracting record offset from current variable offset.
     */
    fldoff = var_ptr->var_offl - rec_ptr->var_offl;

    /*
     *  Write field offset record:
     *
     *  foffset <field_offset>
     */
    (void) sprintf (lin_ptr, "foffset %lu", fldoff);
    iret = WRT_TOF_LINE (com_ptr, NULL);
    if (!psl_test_status (iret)) goto wrt_tof_recfoff_end;

wrt_tof_recfoff_end:
    ;
    return iret;
}

/******************************************************************************
 * @func    This routine writes a field definition 'record name' record to the
 *      script object ('tof') file. A record name record has the following
 *      format:
 *
 *      <PRE>
 *      record <record_name>
 *      </PRE>
 *
 * @rdesc   Unsigned long PSL routine return status.
 * @parm    Pointer to the scl common data structure.
 * @parm    Pointer to variable's symbol table entry.
 */
static unsigned long WRT_TOF_FLDDEFREC (struct scl_com *com_ptr,
                                        struct symtbl *var_ptr)
{
    struct scl_tpa   *tpa_ptr;      /* Tparse data */
    struct symtbl    *rec_ptr;      /* Pointer to record variable's Symbol Table entry */
    unsigned long    iret;          /* PSL routine return status */
    char    recnam[TOFBUFLEN]; /* Parnet record name */
    char    *lin_ptr;      /* Pointer to line buffer */

    /*
     *  Copy pointers for ease of reference.
     */
    tpa_ptr = com_ptr->tpa_ptrps;   /* Tparse data pointer */
    lin_ptr = com_ptr->toflinpc; /* 'tof' line buffer pointer */
    rec_ptr = tpa_ptr->variable_begps + var_ptr->var_recidxl;
    /* Pointer to record entry */
    /*
     *  Write record name record:
     *
     *  record <record_name>
     */
    (void) memcpy (recnam, rec_ptr->var_namec16, rec_ptr->var_nlengthl);
    recnam[rec_ptr->var_nlengthl] = '\0';
    (void) sprintf (lin_ptr, "record %s", recnam);
    iret = WRT_TOF_LINE (com_ptr, NULL);
    if (!psl_test_status (iret)) goto wrt_tof_recnam_end;

wrt_tof_recnam_end:
    ;
    return iret;
}

/******************************************************************************
 * @func    This routine writes the 'list values' portion of a Vardef values
 *      record for an integer variable to the script object ('tof') file. A
 *      Vardef values record has the following format:
 *
 *      <PRE>
 *      values <ltxt_delimiter>
 *      <values>            // This bit written by this routine
 *      <ltxt_delimiter>
 *      </PRE>
 *
 *      For an integer list, <values> consists of a list of comma separated
 *      integer values. These will be continued over multiple lines as
 *      necessary to fit within the right margin, e.g.:
 *
 *      <PRE>
 *      100,200,300,400,500,600,700,
 *      800,900,1000,-100,-200,-300
 *      </PRE>
 *
 * @rdesc   Unsigned long PSL routine return status.
 * @parm    Pointer to the scl common data structure.
 * @parm    Pointer to variable's symbol table entry.
 */
static unsigned long WRT_TOF_INTLIS (struct scl_com *com_ptr,
                                     struct symtbl *var_ptr)
{
    unsigned long    iret = PSL_NORMAL; /* PSL routine return status */
    struct scl_tpa   *tpa_ptr;      /* Tparse data */
    char    *lin_ptr;      /* Pointer to line buffer */
    char    *val_ptr;      /* Value pointer */
    char    *endval_ptr;       /* Pointer to byte beyond last value in list */
    char    valbuf [TOFBUFLEN];    /* ASCII value buffer */

    /*
     *  Copy pointers for ease of reference.
     */
    tpa_ptr = com_ptr->tpa_ptrps;   /* Tparse data pointer */
    lin_ptr = com_ptr->toflinpc; /* 'tof' line buffer pointer */

    /*
     *  Loop through the list of integer values and write them to the script object file
     *  as a list of comma separated values. Add a trailing comma, flush the line and
     *  start a new line, if the next value will not fit on the current line.
     */
    lin_ptr [0] = '\0';         /* Clear line buffer */
    endval_ptr = tpa_ptr->values_begpc + var_ptr->var_values_endl;
    /* Pointer to byte beyond last value in list */
    for (val_ptr = tpa_ptr->values_begpc + var_ptr->var_values_offl;
            val_ptr < endval_ptr;
            val_ptr += sizeof(tpa_ptr->int_valuel))
    {
        (void) sprintf (valbuf, "%ld", *(long *)val_ptr);   /* Get value as ASCII */

        if ((strlen(lin_ptr) + strlen(valbuf) + 1) > TOF_RIGHT_MARGIN)
        {
            (void) strcat (lin_ptr, ",");       /* Append comma to line buffer */
            iret = WRT_TOF_LINE (com_ptr, NULL);    /* Flush line buffer */
            if (!psl_test_status (iret)) goto wrt_tof_intlis_end;
            lin_ptr [0] = '\0';             /* Clear line buffer */
        }

        if (strlen(lin_ptr) != 0) (void) strcat (lin_ptr, ",");
        (void) strcat (lin_ptr, valbuf);        /* Append value to line buffer */
    }

    /*
     *  If anything left in line buffer, flush it to file.
     */
    if (strlen(lin_ptr) != 0)
    {
        iret = WRT_TOF_LINE (com_ptr, NULL);
        if (!psl_test_status (iret)) goto wrt_tof_intlis_end;
    }

wrt_tof_intlis_end:
    ;
    return iret;
}

/******************************************************************************
 * @func    This routine writes the 'range values' portion of a Vardef values
 *      record to the script object ('tof') file. A Vardef values record
 *      has the following format:
 *
 *      <PRE>
 *      values <ltxt_delimiter>
 *      <values>            // This bit written by this routine
 *      <ltxt_delimiter>
 *      </PRE>
 *
 *      For an integer range, <values> consists of the low range value, a
 *      hyphen, then the high range value, e.g. 100-199 or -99--1.
 *
 * @rdesc   Unsigned long PSL routine return status.
 * @parm    Pointer to the scl common data structure.
 * @parm    Pointer to variable's symbol table entry.
 */
static unsigned long WRT_TOF_INTRNG (struct scl_com *com_ptr,
                                     struct symtbl *var_ptr)
{
    unsigned long    iret;          /* PSL routine return status */
    struct scl_tpa   *tpa_ptr;      /* Tparse data */
    char    *lin_ptr;      /* Pointer to line buffer */

    /*
     *  Copy pointers for ease of reference.
     */
    tpa_ptr = com_ptr->tpa_ptrps;   /* Tparse data pointer */
    lin_ptr = com_ptr->toflinpc; /* 'tof' line buffer pointer */

    /*
     *  Write '<values>'.
     */
    (void) sprintf (lin_ptr, "%ld-%ld", var_ptr->var_low_rangel, var_ptr->var_high_rangel);
    iret = WRT_TOF_LINE (com_ptr, NULL);
    if (!psl_test_status (iret)) goto wrt_tof_intrng_end;

wrt_tof_intrng_end:
    ;
    return iret;
}

/******************************************************************************
 * @func    This routine writes a line to the script object ('tof') file.
 * @rdesc   Unsigned long PSL routine return status:
 *      <PRE>
 *      PSL_NORMAL - Success
 *      PSL_WRITEERR - Error writing line to file
 *      </PRE>
 *
 * @parm    Pointer to scl common data structure.
 * @parm    Pointer to null-terminated script object ('tof') line string.
 */
unsigned long WRT_TOF_LINE (struct scl_com *com_ptr,
                            char *usrlin_ptr)
{
    /*
     *  Define the local data.
     */
    unsigned long    iret;          /* PSL routine return status */
    char    *lin_ptr;      /* Pointer to source line buffer */

    /*
     *  Use the default line buffer, unless a buffer is supplied in the call.
     */
    if (usrlin_ptr == NULL)
    {
        lin_ptr = (*com_ptr).toflinpc;
    }
    else
    {
        lin_ptr = usrlin_ptr;
    }

    /*
     *  Write the line to the script object ('tof') file.
     */
    (void) fprintf ((*com_ptr).files.ptrpf14 [OBJ_UNIT], "%s\n", lin_ptr);

    if (ferror ((*com_ptr).files.ptrpf14 [OBJ_UNIT]) != 0)
    {
        iret = PSL_WRITEERR;
        PSL_LOG_CIOERR ((*com_ptr).diag_ptrps,
                        iret,
                        (*com_ptr).obj_namec256);
        PSL_OUTPUT_DIAG ((*com_ptr).diag_ptrps, 0, DIOUFL_LSTERR);
    }
    else
    {
        iret = PSL_NORMAL;
    }

    /*
     *  Return with status.
     */
    return iret;
}

/******************************************************************************
 * @func   This routine writes the 'tof' pipe definition records to the
 *      script object ('tof') file.
 *
 * @rdesc   Unsigned long PSL routine return status.
 * @parm    Pointer to the scl common data structure.
 */
static unsigned long WRT_TOF_PIPS (struct scl_com *com_ptr)
{
    /*
     *  Local data.
     */
    struct scl_tpa   *tpa_ptr;      /* Tparse data */
    struct fattbl    *fat_ptr;      /* Pipes symbol table entry */
    unsigned long    iret;          /* PSL routine return status */
    unsigned long    pipe_cnt;      /* Pipe count */

    /*
     *  Copy pointers for ease of reference.
     */
    tpa_ptr = com_ptr->tpa_ptrps;   /* Tparse data pointer */

    /*
     *  Loop through the pipes symbol table writing a pipe definition
     *  record for each entry.
     */
    for (fat_ptr = tpa_ptr->pipe_begps,
            pipe_cnt = 1;
            pipe_cnt <= tpa_ptr->pipe_cntl;
            fat_ptr++, pipe_cnt++)
    {
        iret = WRT_TOF_PIP (com_ptr, fat_ptr);
        if (!psl_test_status (iret)) goto wrt_tof_pips_end;
    }

    iret = PSL_NORMAL;

    /*
     *  All done - return with status.
     */
wrt_tof_pips_end:
    ;
    return iret;
}

/******************************************************************************
 * @func   This routine writes a single Pipe record to the script object
 *      ('tof') file. A Pipe record has the following format:
 *
 *      <PRE>
 *      Pipdef
 *      name <pipe_id>
 *      options <option_value>
 *      </PRE>
 *
 * @rdesc   Unsigned long PSL routine return status.
 * @parm    Pointer to the scl common data structure.
 */
static unsigned long WRT_TOF_PIP (struct scl_com *com_ptr,
                                  struct fattbl *fat_ptr)
{
    struct scl_tpa   *tpa_ptr;      /* Tparse data */
    unsigned long    iret;          /* PSL routine return status */
    char    *lin_ptr;      /* Pointer to line buffer */

    /*
     *  Copy pointers for ease of reference.
     */
    tpa_ptr = com_ptr->tpa_ptrps;   /* Tparse data pointer */
    lin_ptr = com_ptr->toflinpc; /* 'tof' line buffer pointer */

    /*
     *  Write pipe definition header:
     *
     *  Pipdef
     */
    iret = WRT_TOF_RECHDR (com_ptr, "Pipdef", fat_ptr->fat_tof_srclinl);
    if (!psl_test_status (iret)) goto wrt_tof_pip_end;

    /*
     *  Write name record:
     *
     *  name <pipe_id>
     */
    (void) sprintf (lin_ptr, "name %s", fat_ptr->fat_namec20);
    iret = WRT_TOF_LINE (com_ptr, NULL);
    if (!psl_test_status (iret)) goto wrt_tof_pip_end;

    /*
     *  Write options record.
     */
    iret = WRT_TOF_PIPOPT (com_ptr, fat_ptr);
    if (!psl_test_status (iret)) goto wrt_tof_pip_end;

    iret = PSL_NORMAL;

wrt_tof_pip_end:
    ;
    return iret;
}

/******************************************************************************
 * @func   This routine writes a Pipe options record to the script object
 *      ('tof') file.
 *
 * @rdesc   Unsigned long PSL routine return status.
 * @parm    Pointer to the scl common data structure.
 * @parm    Pointer to pipe's symbol table entry.
 */
static unsigned long WRT_TOF_PIPOPT (struct scl_com *com_ptr,
                                     struct fattbl *fat_ptr)
{
    struct scl_tpa   *tpa_ptr;      /* Tparse data */
    unsigned long    iret;          /* PSL routine return status */
    unsigned long    optval = 0;        /* Option value */
    char    *lin_ptr;      /* Pointer to line buffer */

    /*
     *  Copy pointers for ease of reference.
     */
    tpa_ptr = (*com_ptr).tpa_ptrps; /* Tparse data pointer */
    lin_ptr = (*com_ptr).toflinpc;   /* 'tof' line buffer pointer */

    /*
     *  Get option value.
     */
    if ((fat_ptr->fat_refflgl & PIPATT_SYNC) != 0)
    {
        optval |= TOF_PIPOPT_SYNCHRONOUS;
    }

    /*
     *  Write options record:
     *
     *  options <options_value>
     */
    (void) sprintf (lin_ptr, "options %lu", optval);
    iret = WRT_TOF_LINE (com_ptr, NULL);
    if (!psl_test_status (iret)) goto wrt_tof_pipopt_end;

wrt_tof_pipopt_end:
    ;
    return iret;
}

/******************************************************************************
 * @func    This routine writes a single recdef record to the script object
 *      ('tof') file. A Recdef record has the following format:
 *
 *      <PRE>
 *      Recdef
 *      name <record_name>
 *      options <option_value>
 *      size <record_size>
 *      </PRE>
 *
 * @rdesc   Unsigned long PSL routine return status.
 * @parm    Pointer to the scl common data structure.
 * @parm    Pointer to variable's entry in Variables Symbol Table.
 */
static unsigned long WRT_TOF_RECDEF (struct scl_com *com_ptr,
                                     struct symtbl *var_ptr)
{
    unsigned long    iret;          /* PSL routine return status */

    /*
     *  Write record definition header:
     *
     *  Recdef
     */
    iret = WRT_TOF_RECHDR (com_ptr, "Recdef", var_ptr->var_tof_srclinl);
    if (!psl_test_status (iret)) goto wrt_tof_recdef_end;

    /*
     *  Write variable name record.
     */
    iret = WRT_TOF_VARNAM (com_ptr, var_ptr);
    if (!psl_test_status (iret)) goto wrt_tof_recdef_end;

    /*
     *  Write options record.
     */
    iret = WRT_TOF_VAROPT (com_ptr, var_ptr);
    if (!psl_test_status (iret)) goto wrt_tof_recdef_end;

    /*
     *  Write size record.
     */
    iret = WRT_TOF_VARSIZ (com_ptr, var_ptr);
    if (!psl_test_status (iret)) goto wrt_tof_recdef_end;

    iret = PSL_NORMAL;
wrt_tof_recdef_end:
    ;
    return iret;
}

/******************************************************************************
 * @func    This routine writes the 'TOF' record header lines to the script
 *          object ('tof') file. Its purpose is to isolate in a single place
 *          the code that writes the record separator - a blank line - to the
 *          file. It also writes the srclin line (which is common to all 'TOF'
 *          records).
 *
 * @rdesc   Unsigned long PSL routine return status from WRT_TOF_LINE.
 *
 * @parm    Pointer to scl common data structure.
 * @parm    Pointer to null-terminated record header line string.
 * @parm    SCL source line number on which the current definition started.
 */
static unsigned long WRT_TOF_RECHDR (struct scl_com *com_ptr,
                                     char *usrlin_ptr,
                                     PSL_U_LONGWORD_T tof_srclin)
{
    /*
     *  Local data.
     */
    unsigned long    iret;          /* PSL routine return status */
    char    linbuf = '\0';          /* Local TOF line buffer */
    char    srclin [TOFBUFLEN];     /* 'srclin' line buffer */

    /*
     *  Write a blank line to delimit record.
     */
    iret = WRT_TOF_LINE (com_ptr, &linbuf);
    if (!psl_test_status (iret)) goto wrt_tof_rechdr_end;

    /*
     *  Write record header.
     */
    iret = WRT_TOF_LINE (com_ptr, usrlin_ptr);
    if (!psl_test_status (iret)) goto wrt_tof_rechdr_end;

    /*
     *  Write 'srclin'.
     */
    (void) sprintf (srclin, "srclin %d", tof_srclin);
    iret = WRT_TOF_LINE (com_ptr, srclin);
    if (!psl_test_status (iret)) goto wrt_tof_rechdr_end;

    /*
     *  Return with status.
     */
wrt_tof_rechdr_end:
    return iret;
}

/******************************************************************************
 * @func   This routine writes all the 'tof' records to the output type test or
 *      type script script object file. A script object file is a 'tof'
 *      format file and has the following structure:
 *
 *      <PRE>
 *      'tof' header record
 *      {code object record(s)}
 *      {variable definition object record(s)}
 *      </PRE>
 *
 * @rdesc   Unsigned long PSL routine return status.
 * @parm    Pointer to the scl common data structure.
 */
static unsigned long WRT_TOF_SCRRECS (struct scl_com *com_ptr)
{
    /*
     *  Local data.
     */
    struct scl_tpa  *tpa_ptr;       /* TPARSE Data */
    unsigned long    iret;          /* Status return */
    struct scl_cmd   *scl_cmd_ptr;      /* Pointer to command line structure */

    /*
     *  Copy pointers for ease of reference.
     */
    tpa_ptr = (*com_ptr).tpa_ptrps; /* Tparse data pointer */
    scl_cmd_ptr = (*com_ptr).cmd_ptrps;

    /*
     *  Write the header record.
     */
    iret = WRT_SCRTOF_HDR (com_ptr);
    if (!psl_test_status (iret)) goto wrt_tof_scrrecs_end;

    /*
     *  Write the code object records.
     */
    iret = WRT_TOF_COD (com_ptr);
    if (!psl_test_status (iret)) goto wrt_tof_scrrecs_end;

    /*
     *  Write the file definition records.
     */
    iret = WRT_TOF_FILS (com_ptr);
    if (!psl_test_status (iret)) goto wrt_tof_scrrecs_end;

    /*
     *  Write the pipe definition records.
     */
    iret = WRT_TOF_PIPS (com_ptr);
    if (!psl_test_status (iret)) goto wrt_tof_scrrecs_end;

    /*
     *  Write the timer definition records.
     */
    iret = WRT_TOF_TIMS (com_ptr);
    if (!psl_test_status (iret)) goto wrt_tof_scrrecs_end;

    /*
     *  Write the test definition records.
     */
    iret = WRT_TOF_TSTS (com_ptr);
    if (!psl_test_status (iret)) goto wrt_tof_scrrecs_end;

    /*
     *  Write the test thread definition records.
     */
    iret = WRT_TOF_TSTTHRS (com_ptr);
    if (!psl_test_status (iret)) goto wrt_tof_scrrecs_end;

    /*
     *  Write the variable definition records.
     */
    iret = WRT_TOF_VARS (com_ptr);
    if (!psl_test_status (iret)) goto wrt_tof_scrrecs_end;

    iret = PSL_NORMAL;

    /*
     *  All done - return with status.
     */
wrt_tof_scrrecs_end:
    return iret;
}

/******************************************************************************
 * @func    This routine writes all the 'tof' records to the output type thread
 *      script object file. A script object file is a 'tof' format file and
 *      has the following structure:
 *
 *      <PRE>
 *      'tof' header record
 *      thread definition object record.
 *      </PRE>
 *
 *      Each thread definition file contains a single thread definition.
 *
 * @rdesc   Unsigned long PSL routine return status.
 * @parm    Pointer to the scl common data structure.
 */
static unsigned long WRT_TOF_THRRECS (struct scl_com *com_ptr,
                                      struct thrtbl *thr_ptr)
{
    /*
     *  Local data.
     */
    struct scl_tpa  *tpa_ptr;       /* TPARSE Data */
    unsigned long    iret;          /* Status return */
    struct scl_cmd   *scl_cmd_ptr;      /* Pointer to command line structure */

    /*
     *  Copy pointers for ease of reference.
     */
    tpa_ptr = (*com_ptr).tpa_ptrps; /* Tparse data pointer */
    scl_cmd_ptr = (*com_ptr).cmd_ptrps;

    /*
     *  Write the header record.
     */
    iret = WRT_THRTOF_HDR (com_ptr, thr_ptr);
    if (!psl_test_status (iret)) goto wrt_tof_thrrecs_end;

    /*
     *  Write the thread definition record.
     */
    iret = WRT_TOF_THR (com_ptr, thr_ptr);
    if (!psl_test_status (iret)) goto wrt_tof_thrrecs_end;

    iret = PSL_NORMAL;

    /*
     *  All done - return with status.
     */
wrt_tof_thrrecs_end:
    return iret;
}

/******************************************************************************
 * @func    This routine writes a single Thrdef record to the script object
 *      ('tof') file. A Thrdef record has the following format:
 *
 *      <PRE>
 *      Thrdef
 *      name <thread_name>
 *      target <target_id>
 *      {parameter(s)}
 *      </PRE>
 *
 *      where 'parameter(s)' are the Thrdef parameter records corresponding
 *      to the options specified in the thread definition.
 *
 * @rdesc   Unsigned long PSL routine return status.
 * @parm    Pointer to the scl common data structure.
 */
static unsigned long WRT_TOF_THR (struct scl_com *com_ptr,
                                  struct thrtbl *thr_ptr)
{
    struct scl_tpa   *tpa_ptr;      /* Tparse data */
    struct tdf   *tdf_ptr;      /* Pointer to thread data */
    unsigned long    iret;          /* PSL routine return status */
    char    *lin_ptr;      /* Pointer to line buffer */

    /*
     *  Copy pointers for ease of reference.
     */
    tpa_ptr = com_ptr->tpa_ptrps;   /* Tparse data pointer */
    lin_ptr = com_ptr->toflinpc; /* 'tof' line buffer pointer */
    tdf_ptr = thr_ptr->thr_tdf_ptrps;   /* Thread data pointer */

    /*
     *  Write thread definition header:
     *
     *  Thrdef
     */
    iret = WRT_TOF_RECHDR (com_ptr, "Thrdef", thr_ptr->thr_tof_srclinl);
    if (!psl_test_status (iret)) goto wrt_tof_thr_end;

    /*
     *  Write name record:
     *
     *  name <thread_name>
     */
    (void) sprintf (lin_ptr, "name %s", thr_ptr->thr_namec20);
    iret = WRT_TOF_LINE (com_ptr, NULL);
    if (!psl_test_status (iret)) goto wrt_tof_thr_end;

    /*
     *  Write target record:
     *
     *  target <target_id>
     */
    (void) sprintf (lin_ptr, "target %lu", tdf_ptr->tdf_targetl);
    iret = WRT_TOF_LINE (com_ptr, NULL);
    if (!psl_test_status (iret)) goto wrt_tof_thr_end;

    /*
     *  Write answerback record, if specified:
     *
     *  answerback <string>
     */
    if ((tdf_ptr->tdf_default_flags1l & TDF_THRDF1_ANSWERBACK) == 0)
    {
        (void) sprintf (lin_ptr, "answerback %s", tdf_ptr->tdf_answerbackc32);
        iret = WRT_TOF_LINE (com_ptr, NULL);
        if (!psl_test_status (iret)) goto wrt_tof_thr_end;
    }

    /*
     *  Write automatic synchronisation record, if specified:
     *
     *  autosync <string>
     */
    if ((tdf_ptr->tdf_default_flags2l & TDF_THRDF2_AUTO_SYNC) == 0)
    {
        (void) sprintf (lin_ptr, "autosync %lu", tdf_ptr->tdf_auto_syncl);
        iret = WRT_TOF_LINE (com_ptr, NULL);
        if (!psl_test_status (iret)) goto wrt_tof_thr_end;
    }

    /*
     *  Write baud rate record, if specified:
     *
     *  baudrate <string>
     */
    if ((tdf_ptr->tdf_default_flags1l & TDF_THRDF1_BAUD_RATE) == 0)
    {
        (void) sprintf (lin_ptr, "baudrate %lu", tdf_ptr->tdf_baud_ratel);
        iret = WRT_TOF_LINE (com_ptr, NULL);
        if (!psl_test_status (iret)) goto wrt_tof_thr_end;
    }

    /*
     *  Write character size record, if specified:
     *
     *  charsize <size>
     */
    if ((tdf_ptr->tdf_default_flags1l & TDF_THRDF1_CHARACTER_SIZE) == 0)
    {
        (void) sprintf (lin_ptr, "charsize %lu", tdf_ptr->tdf_character_sizel);
        iret = WRT_TOF_LINE (com_ptr, NULL);
        if (!psl_test_status (iret)) goto wrt_tof_thr_end;
    }

    /*
     *  Write detach on exit record, if specified:
     *
     *  detonexit <state>
     */
    if ((tdf_ptr->tdf_default_flags1l & TDF_THRDF1_DETACH_EXIT) == 0)
    {
        (void) sprintf (lin_ptr, "detonexit %lu", tdf_ptr->tdf_detach_exitl);
        iret = WRT_TOF_LINE (com_ptr, NULL);
        if (!psl_test_status (iret)) goto wrt_tof_thr_end;
    }

    /*
     *  Write device type record, if specified:
     *
     *  devtype <type>
     */
    if ((tdf_ptr->tdf_default_flags1l & TDF_THRDF1_DEVICE_TYPE) == 0)
    {
        (void) sprintf (lin_ptr, "devtype %lu", tdf_ptr->tdf_device_typel);
        iret = WRT_TOF_LINE (com_ptr, NULL);
        if (!psl_test_status (iret)) goto wrt_tof_thr_end;
    }

    /*
     *  Write emulator model record, if specified:
     *
     *  emmodel <string>
     */
    if ((tdf_ptr->tdf_default_flags2l & TDF_THRDF2_EMULMOD) == 0)
    {
        (void) sprintf (lin_ptr, "emmodel %s", tdf_ptr->tdf_emulator_modelc16);
        iret = WRT_TOF_LINE (com_ptr, NULL);
        if (!psl_test_status (iret)) goto wrt_tof_thr_end;
    }

    /*
     *  Write emulator type record, if specified:
     *
     *  emtype <string>
     */
    if ((tdf_ptr->tdf_default_flags2l & TDF_THRDF2_EMULTYP) == 0)
    {
        (void) sprintf (lin_ptr, "emtype %s", tdf_ptr->tdf_emulator_typec16);
        iret = WRT_TOF_LINE (com_ptr, NULL);
        if (!psl_test_status (iret)) goto wrt_tof_thr_end;
    }

    /*
     *  Write function key controls record, if specified:
     *
     *  fkeycontrols <size>
     */
    if ((tdf_ptr->tdf_default_flags1l & TDF_THRDF1_FKEY_CONTROL) == 0)
    {
        (void) sprintf (lin_ptr, "fkeycontrols %lu", tdf_ptr->tdf_fkey_controll);
        iret = WRT_TOF_LINE (com_ptr, NULL);
        if (!psl_test_status (iret)) goto wrt_tof_thr_end;
    }

    /*
     *  Write hangup on detach record, if specified:
     *
     *  hangondet <state>
     */
    if ((tdf_ptr->tdf_default_flags1l & TDF_THRDF1_HANGUP_DETACH) == 0)
    {
        (void) sprintf (lin_ptr, "hangondet %lu", tdf_ptr->tdf_hangup_detachl);
        iret = WRT_TOF_LINE (com_ptr, NULL);
        if (!psl_test_status (iret)) goto wrt_tof_thr_end;
    }

    /*
     *  Write input timing type record, if specified:
     *
     *  inptiming <type>
     */
    if ((tdf_ptr->tdf_default_flags1l & TDF_THRDF1_INPUT_TIMING) == 0)
    {
        (void) sprintf (lin_ptr, "inptiming %lu", tdf_ptr->tdf_input_timingl);
        iret = WRT_TOF_LINE (com_ptr, NULL);
        if (!psl_test_status (iret)) goto wrt_tof_thr_end;
    }

    /*
     *  Write internet name/address record, if specified:
     *
     *  inetnam <string>
     */
    if ((tdf_ptr->tdf_default_flags2l & TDF_THRDF2_TCPIPADDR) == 0)
    {
        (void) sprintf (lin_ptr, "inetnam %s", tdf_ptr->tdf_fqtcpip_addressc128);
        iret = WRT_TOF_LINE (com_ptr, NULL);
        if (!psl_test_status (iret)) goto wrt_tof_thr_end;
    }

    /*
     *  Write IOM record, if specified:
     *
     *  iom <state>
     */
    if ((tdf_ptr->tdf_default_flags2l & TDF_THRDF2_ENABLE_IOM) == 0)
    {
        (void) sprintf (lin_ptr, "inptimtype %lu", tdf_ptr->tdf_enable_ioml);
        iret = WRT_TOF_LINE (com_ptr, NULL);
        if (!psl_test_status (iret)) goto wrt_tof_thr_end;
    }

    /*
     *  Write login record, if specified:
     *
     *  login <type>
     */
    if ((tdf_ptr->tdf_default_flags1l & TDF_THRDF1_LOGIN_TYPE) == 0)
    {
        (void) sprintf (lin_ptr, "logintype %lu", tdf_ptr->tdf_login_typel);
        iret = WRT_TOF_LINE (com_ptr, NULL);
        if (!psl_test_status (iret)) goto wrt_tof_thr_end;
    }

    /*
     *  Write node record, if specified:
     *
     *  node <string>
     */
    if ((tdf_ptr->tdf_default_flags1l & TDF_THRDF1_LTNODE) == 0)
    {
        (void) sprintf (lin_ptr, "node %s", tdf_ptr->tdf_ltnodec8);
        iret = WRT_TOF_LINE (com_ptr, NULL);
        if (!psl_test_status (iret)) goto wrt_tof_thr_end;
    }

    /*
     *  Write page size record, if specified:
     *
     *  pagesize <size>
     */
    if ((tdf_ptr->tdf_default_flags1l & TDF_THRDF1_PAGE) == 0)
    {
        (void) sprintf (lin_ptr, "pagesize %lu", tdf_ptr->tdf_pagel);
        iret = WRT_TOF_LINE (com_ptr, NULL);
        if (!psl_test_status (iret)) goto wrt_tof_thr_end;
    }

    /*
     *  Write physical device record, if specified:
     *
     *  phydevice <string>
     */
    if ((tdf_ptr->tdf_default_flags1l & TDF_THRDF1_PHYSICAL_DEVICE) == 0)
    {
        (void) sprintf (lin_ptr, "phydevice %s", tdf_ptr->tdf_physical_devicec36);
        iret = WRT_TOF_LINE (com_ptr, NULL);
        if (!psl_test_status (iret)) goto wrt_tof_thr_end;
    }

    /*
     *  Write port number record, if specified:
     *
     *  port <value>
     */
    if ((tdf_ptr->tdf_default_flags2l & TDF_THRDF2_TCPIP_PORT) == 0)
    {
        (void) sprintf (lin_ptr, "port %lu", tdf_ptr->tdf_tcpip_portl);
        iret = WRT_TOF_LINE (com_ptr, NULL);
        if (!psl_test_status (iret)) goto wrt_tof_thr_end;
    }

    /*
     *  Write recording state record, if specified:
     *
     *  recordstate <state>
     */
    if ((tdf_ptr->tdf_default_flags1l & TDF_THRDF1_RECORD) == 0)
    {
        (void) sprintf (lin_ptr, "recordstate %lu", tdf_ptr->tdf_recordl);
        iret = WRT_TOF_LINE (com_ptr, NULL);
        if (!psl_test_status (iret)) goto wrt_tof_thr_end;
    }

    /*
     *  Write recording file record, if specified:
     *
     *  recordfile <name>
     */
    if ((tdf_ptr->tdf_default_flags1l & TDF_THRDF1_RECORDFILE) == 0)
    {
        (void) sprintf (lin_ptr, "recordfile %s", tdf_ptr->tdf_recordfilec52);
        iret = WRT_TOF_LINE (com_ptr, NULL);
        if (!psl_test_status (iret)) goto wrt_tof_thr_end;
    }

    /*
     *  Write recording options/attributes record, if specified:
     *
     *  recordatt <attributes_mask>
     */
    if ((tdf_ptr->tdf_default_flags2l & TDF_THRDF2_RECORD_OPTS) == 0)
    {
        (void) sprintf (lin_ptr, "recordatt %lu", tdf_ptr->tdf_record_typel);
        iret = WRT_TOF_LINE (com_ptr, NULL);
        if (!psl_test_status (iret)) goto wrt_tof_thr_end;
    }

    /*
     *  Write recording stop on detach record, if specified:
     *
     *  recordstopondet <state>
     */
    if ((tdf_ptr->tdf_default_flags1l & TDF_THRDF1_STOPRECDET) == 0)
    {
        (void) sprintf (lin_ptr, "recordstopondet %lu", tdf_ptr->tdf_stoprecdetl);
        iret = WRT_TOF_LINE (com_ptr, NULL);
        if (!psl_test_status (iret)) goto wrt_tof_thr_end;
    }

    /*
     *  Write LAT service name record, if specified:
     *
     *  servicename <name>
     */
    if ((tdf_ptr->tdf_default_flags1l & TDF_THRDF1_LTSERVICE) == 0)
    {
        (void) sprintf (lin_ptr, "servicename %s", tdf_ptr->tdf_ltservicec20);
        iret = WRT_TOF_LINE (com_ptr, NULL);
        if (!psl_test_status (iret)) goto wrt_tof_thr_end;
    }

    /*
     *  Write TCP/IP socket record, if specified:
     *
     *  socketuse
     */
    if ((tdf_ptr->tdf_default_flags2l & TDF_THRDF2_USE_TCP_SOCKET) == 0)
    {
        (void) sprintf (lin_ptr, "socketuse %lu", tdf_ptr->tdf_use_tcp_socketl);
        iret = WRT_TOF_LINE (com_ptr, NULL);
        if (!psl_test_status (iret)) goto wrt_tof_thr_end;
    }

    /*
     *  Write thread ID record, if specified:
     *
     *  threadid <string>
     */
    if ((tdf_ptr->tdf_default_flags1l & TDF_THRDF1_TERMINAL_ID) == 0)
    {
        (void) sprintf (lin_ptr, "threadid %lu", tdf_ptr->tdf_terminal_idl);
        iret = WRT_TOF_LINE (com_ptr, NULL);
        if (!psl_test_status (iret)) goto wrt_tof_thr_end;
    }

    /*
     *  Write thread name record, if specified:
     *
     *  threadname <name>
     */
    if ((tdf_ptr->tdf_default_flags1l & TDF_THRDF1_TRM_NAME) == 0)
    {
        (void) sprintf (lin_ptr, "threadname %s", tdf_ptr->tdf_trm_namec20);
        iret = WRT_TOF_LINE (com_ptr, NULL);
        if (!psl_test_status (iret)) goto wrt_tof_thr_end;
    }

    /*
     *  Write terminfo name record, if specified:
     *
     *  terminfoname <name>
     */
    if ((tdf_ptr->tdf_default_flags2l & TDF_THRDF2_TRMINFO) == 0)
    {
        (void) sprintf (lin_ptr, "terminfoname %s", tdf_ptr->tdf_terminfo_namec36);
        iret = WRT_TOF_LINE (com_ptr, NULL);
        if (!psl_test_status (iret)) goto wrt_tof_thr_end;
    }

    /*
     *  Write tracing flag record, if specified:
     *
     *  tracing <state>
     */
    if ((tdf_ptr->tdf_default_flags1l & TDF_THRDF1_TRACEFLAG) == 0)
    {
        (void) sprintf (lin_ptr, "tracing %lu", tdf_ptr->tdf_traceflagl);
        iret = WRT_TOF_LINE (com_ptr, NULL);
        if (!psl_test_status (iret)) goto wrt_tof_thr_end;
    }

    /*
     *  Write tracing file record, if specified:
     *
     *  tracefile <name>
     */
    if ((tdf_ptr->tdf_default_flags1l & TDF_THRDF1_TRACEFILE) == 0)
    {
        (void) sprintf (lin_ptr, "tracefile %s", tdf_ptr->tdf_tracefilec52);
        iret = WRT_TOF_LINE (com_ptr, NULL);
        if (!psl_test_status (iret)) goto wrt_tof_thr_end;
    }

    /*
     *  Write thread type record, if specified:
     *
     *  threadtype <type>
     */
    if ((tdf_ptr->tdf_default_flags1l & TDF_THRDF1_TERMINAL_TYPE) == 0)
    {
        (void) sprintf (lin_ptr, "threadtype %lu", tdf_ptr->tdf_terminal_typel);
        iret = WRT_TOF_LINE (com_ptr, NULL);
        if (!psl_test_status (iret)) goto wrt_tof_thr_end;
    }

    /*
     *  Write type-ahead record, if specified:
     *
     *  typeahead <state>
     */
    if ((tdf_ptr->tdf_default_flags1l & TDF_THRDF1_TYPEAHEAD) == 0)
    {
        (void) sprintf (lin_ptr, "typeahead %lu", tdf_ptr->tdf_typeaheadl);
        iret = WRT_TOF_LINE (com_ptr, NULL);
        if (!psl_test_status (iret)) goto wrt_tof_thr_end;
    }

    /*
     *  Write username record, if specified:
     *
     *  username <name>
     */
    if ((tdf_ptr->tdf_default_flags1l & TDF_THRDF1_USERNAME) == 0)
    {
        (void) sprintf (lin_ptr, "username %s", tdf_ptr->tdf_usernamec36);
        iret = WRT_TOF_LINE (com_ptr, NULL);
        if (!psl_test_status (iret)) goto wrt_tof_thr_end;
    }

    /*
     *  Write terminal page width record, if specified:
     *
     *  pagewidth <value>
     */
    if ((tdf_ptr->tdf_default_flags1l & TDF_THRDF1_WIDTH) == 0)
    {
        (void) sprintf (lin_ptr, "pagewidth %lu", tdf_ptr->tdf_widthl);
        iret = WRT_TOF_LINE (com_ptr, NULL);
        if (!psl_test_status (iret)) goto wrt_tof_thr_end;
    }

    /*
     *  Now that we have finished with this table entry, free the space associated
     *  with it and clear the pointer to it.
     */
    SCL_FREE_TDF (tdf_ptr);
    FREE (tdf_ptr);
    thr_ptr->thr_tdf_ptrps = NULL;

    iret = PSL_NORMAL;

wrt_tof_thr_end:
    ;
    return iret;
}

/******************************************************************************
 * @func   This routine writes the 'tof' timer definition records to the
 *      script object ('tof') file.
 *
 * @rdesc   Unsigned long PSL routine return status.
 * @parm    Pointer to the scl common data structure.
 */
static unsigned long WRT_TOF_TIMS (struct scl_com *com_ptr)
{
    /*
     *  Local data.
     */
    struct scl_tpa   *tpa_ptr;      /* Tparse data */
    struct timtbl    *tim_ptr;      /* Timers symbol table entry */
    unsigned long    iret;          /* PSL routine return status */
    unsigned long    tim_cnt;       /* Timer count */

    /*
     *  Copy pointers for ease of reference.
     */
    tpa_ptr = com_ptr->tpa_ptrps;   /* Tparse data pointer */

    /*
     *  Loop through the timers symbol table writing a timer definition
     *  record for each entry.
     */
    for (tim_ptr = tpa_ptr->timer_begps,
            tim_cnt = 1;
            tim_cnt <= tpa_ptr->tim_cntl;
            tim_ptr++, tim_cnt++)
    {
        iret = WRT_TOF_TIM (com_ptr, tim_ptr);
        if (!psl_test_status (iret)) goto wrt_tof_tims_end;
    }

    iret = PSL_NORMAL;

    /*
     *  All done - return with status.
     */
wrt_tof_tims_end:
    ;
    return iret;
}

/******************************************************************************
 * @func   This routine writes a single Timdef record to the script object
 *      ('tof') file. A Timdef record has the following format:
 *
 *      Timdef
 *      name <timer_name>
 *
 * @rdesc   Unsigned long PSL routine return status.
 * @parm    Pointer to the scl common data structure.
 */
static unsigned long WRT_TOF_TIM (struct scl_com *com_ptr,
                                  struct timtbl *tim_ptr)
{
    struct scl_tpa   *tpa_ptr;      /* Tparse data */
    unsigned long    iret;          /* PSL routine return status */
    char    *lin_ptr;      /* Pointer to line buffer */

    /*
     *  Copy pointers for ease of reference.
     */
    tpa_ptr = com_ptr->tpa_ptrps;   /* Tparse data pointer */
    lin_ptr = com_ptr->toflinpc; /* 'tof' line buffer pointer */

    /*
     *  Write timer definition header:
     *
     *  Timdef
     */
    iret = WRT_TOF_RECHDR (com_ptr, "Timdef", tim_ptr->tim_tof_srclinl);
    if (!psl_test_status (iret)) goto wrt_tof_tim_end;

    /*
     *  Write name record:
     *
     *  name <timer_name>
     */
    (void) sprintf (lin_ptr, "name %s", tim_ptr->tim_namec68);
    iret = WRT_TOF_LINE (com_ptr, NULL);
    if (!psl_test_status (iret)) goto wrt_tof_tim_end;

    iret = PSL_NORMAL;

wrt_tof_tim_end:
    ;
    return iret;
}

/******************************************************************************
 * @func    This routine writes the 'tof' test and sub-test definition records
 *      to the script object ('tof') file. It calls WRT_TOF_TST and
 *      WRT_TOF_SUBTST to write each test and sub-test record respectively.
 *
 * @rdesc   Unsigned long PSL routine return status.
 * @parm    Pointer to the scl common data structure.
 */
static unsigned long WRT_TOF_TSTS (struct scl_com *com_ptr)
{
    /*
     *  Local data.
     */
    struct scl_tpa   *tpa_ptr;      /* Tparse data */
    struct tsttbl    *tst_ptr;      /* Test symbol table entry */
    unsigned long    iret;          /* PSL routine return status */
    unsigned long    tst_cnt;       /* Test count */

    /*
     *  Copy pointers for ease of reference.
     */
    tpa_ptr = com_ptr->tpa_ptrps;   /* Tparse data pointer */

    /*
     *  Loop through the test symbol table writing a definition
     *  record for each external test declaration entry.
     */
    for (tst_ptr = tpa_ptr->tst_begps,
            tst_cnt = 1;
            tst_cnt <= tpa_ptr->tst_cntl;
            tst_ptr++, tst_cnt++)
    {
        if (!(tst_ptr->tst_subtstn))    /* If an EXTERNAL TEST declaration */
        {
            iret = WRT_TOF_TST (com_ptr, tst_ptr);
            if (!psl_test_status (iret)) goto wrt_tof_tsts_end;
        }
    }

    /*
     *  Loop through the test symbol table writing a definition
     *  record for each sub-test definition entry.
     */
    for (tst_ptr = tpa_ptr->tst_begps,
            tst_cnt = 1;
            tst_cnt <= tpa_ptr->tst_cntl;
            tst_ptr++, tst_cnt++)
    {
        if (tst_ptr->tst_subtstn)   /* If a SUB-TEST definition */
        {
            iret = WRT_TOF_SUBTST (com_ptr, tst_ptr);
            if (!psl_test_status (iret)) goto wrt_tof_tsts_end;
        }
    }

    iret = PSL_NORMAL;

    /*
     *  All done - return with status.
     */
wrt_tof_tsts_end:
    ;
    return iret;
}

/******************************************************************************
 * @func    This routine writes a single Tstdef record to the script object
 *      ('tof') file. A Tstdef record has the following format:
 *
 *      <PRE>
 *      Tstdef
 *      name <test_name>
 *      </PRE>
 *
 * @rdesc   Unsigned long PSL routine return status.
 * @parm    Pointer to the scl common data structure.
 */
static unsigned long WRT_TOF_TST (struct scl_com *com_ptr,
                                  struct tsttbl *tst_ptr)
{
    struct scl_tpa   *tpa_ptr;      /* Tparse data */
    struct prm   *prm_ptr;      /* Pointer to test parameter data */
    struct tlh   *tlh_ptr;      /* Pointer to test layout header data */
    unsigned long    iret;          /* PSL routine return status */
    char    *lin_ptr;      /* Pointer to line buffer */

    /*
     *  Copy pointers for ease of reference.
     */
    tpa_ptr = com_ptr->tpa_ptrps;   /* Tparse data pointer */
    lin_ptr = com_ptr->toflinpc; /* 'tof' line buffer pointer */
    prm_ptr = tst_ptr->tst_prm_ptrps;   /* Test parameter data pointer */
    tlh_ptr = tst_ptr->tst_tlh_ptrps;   /* Test layout header data pointer */

    /*
     *  Write test definition header:
     *
     *  Tstdef
     */
    iret = WRT_TOF_RECHDR (com_ptr, "Tstdef", tst_ptr->tst_tof_srclinl);
    if (!psl_test_status (iret)) goto wrt_tof_tst_end;

    /*
     *  Write name record:
     *
     *  name <test_name>
     */
    (void) sprintf (lin_ptr, "name %s", tst_ptr->tst_namec68);
    iret = WRT_TOF_LINE (com_ptr, NULL);
    if (!psl_test_status (iret)) goto wrt_tof_tst_end;

    iret = PSL_NORMAL;

wrt_tof_tst_end:
    ;
    return iret;
}

/******************************************************************************
 * @func    This routine writes a single Ststdef record to the script object
 *      ('tof') file. An Ststdef record has the following format:
 *
 *      <PRE>
 *      Ststdef
 *      name <test_name>
 *      {parameters...}
 *      </PRE>
 *
 *      Where, the optional parameter records are written by the
 *      WRT_TOF_TSTPRMS routine.
 *
 * @rdesc   Unsigned long PSL routine return status.
 * @parm    Pointer to the scl common data structure.
 */
static unsigned long WRT_TOF_SUBTST (struct scl_com *com_ptr,
                                     struct tsttbl *tst_ptr)
{
    struct scl_tpa   *tpa_ptr;      /* Tparse data */
    unsigned long    iret;          /* PSL routine return status */
    char    *lin_ptr;      /* Pointer to line buffer */

    /*
     *  Copy pointers for ease of reference.
     */
    tpa_ptr = com_ptr->tpa_ptrps;   /* Tparse data pointer */
    lin_ptr = com_ptr->toflinpc; /* 'tof' line buffer pointer */

    /*
     *  Write test definition header:
     *
     *  Ststdef
     */
    iret = WRT_TOF_RECHDR (com_ptr, "Ststdef", tst_ptr->tst_tof_srclinl);
    if (!psl_test_status (iret)) goto wrt_tof_subtst_end;

    /*
     *  Write name record:
     *
     *  name <test_name>
     */
    (void) sprintf (lin_ptr, "name %s", tst_ptr->tst_namec68);
    iret = WRT_TOF_LINE (com_ptr, NULL);
    if (!psl_test_status (iret)) goto wrt_tof_subtst_end;

    /*
     *  Write optional parameter records.
     */
    iret = WRT_TOF_TSTPRMS (com_ptr, tst_ptr);
    if (!psl_test_status (iret)) goto wrt_tof_subtst_end;

    iret = PSL_NORMAL;

wrt_tof_subtst_end:
    ;
    return iret;
}

/******************************************************************************
 * @func    This routine writes the user specified test parameter records to
 *      the script object ('tof') file. It writes the folloeing parameter
 *      records:
 *
 *      <PRE>
 *      {abortonfail <state>}
 *      {abortmsg <string>}
 *      {completemsg <string>}
 *      {concthreads <number>}
 *      {description <string>}
 *      {dumponexc <state>}
 *      {tcfailmsg <string>}
 *      {hcdevice <device>}
 *      {ignoreinf <state>}
 *      {inprate <string>}
 *      {tcpassmsg <string>}
 *      {wksq <quota>}
 *      </PRE>
 *
 *      This routine writes the test parameter records common to the
 *      test file and individual sub-test definitions.
 *
 * @rdesc   Unsigned long PSL routine return status.
 * @parm    Pointer to the scl common data structure.
 */
static unsigned long WRT_TOF_TSTPRMS (struct scl_com *com_ptr,
                                      struct tsttbl *tst_ptr)
{
    struct scl_tpa   *tpa_ptr;      /* Tparse data */
    struct prm   *prm_ptr;      /* Pointer to test parameter data */
    struct tlh   *tlh_ptr;      /* Pointer to test layout header data */
    unsigned long    iret;          /* PSL routine return status */
    char    *lin_ptr;      /* Pointer to line buffer */

    /*
     *  Copy pointers for ease of reference.
     */
    tpa_ptr = com_ptr->tpa_ptrps;   /* Tparse data pointer */
    lin_ptr = com_ptr->toflinpc; /* 'tof' line buffer pointer */
    prm_ptr = tst_ptr->tst_prm_ptrps;   /* Test parameter data pointer */
    tlh_ptr = tst_ptr->tst_tlh_ptrps;   /* Test layout header data pointer */

    /*
     *  Write description record:
     *
     *  description <string>
     */
    if ((prm_ptr->prm_default_flagsl & PRM_TSTDF_DSC) == 0)
    {
        (void) sprintf (lin_ptr, "description %s", tlh_ptr->tlh_testdescc50);
        iret = WRT_TOF_LINE (com_ptr, NULL);
        if (!psl_test_status (iret)) goto wrt_tof_tstprms_end;
    }

    /*
     *  Write abort on failure record:
     *
     *  abortonfail <state>
     */
    if ((prm_ptr->prm_default_flagsl & PRM_TSTDF_ABORTFAIL) == 0)
    {
        (void) sprintf (lin_ptr, "abortonfail %lu", prm_ptr->prm_abortfaill);
        iret = WRT_TOF_LINE (com_ptr, NULL);
        if (!psl_test_status (iret)) goto wrt_tof_tstprms_end;
    }

    /*
     *  Write abort test message record:
     *
     *  abortmsg <message>
     */
    if ((prm_ptr->prm_default_flagsl & PRM_TSTDF_ABRTTEST) == 0)
    {
        (void) sprintf (lin_ptr, "abortmsg %s", prm_ptr->prm_abrttestc84);
        iret = WRT_TOF_LINE (com_ptr, NULL);
        if (!psl_test_status (iret)) goto wrt_tof_tstprms_end;
    }

    /*
     *  Write test completion message record:
     *
     *  completemsg <message>
     */
    if ((prm_ptr->prm_default_flagsl & PRM_TSTDF_COMPTEST) == 0)
    {
        (void) sprintf (lin_ptr, "completemsg %s", prm_ptr->prm_comptestc84);
        iret = WRT_TOF_LINE (com_ptr, NULL);
        if (!psl_test_status (iret)) goto wrt_tof_tstprms_end;
    }

    /*
     *  Write concurrent threads record:
     *
     *  concthreads <count>
     */
    if ((prm_ptr->prm_default_flagsl & PRM_TSTDF_MAX_CONCURRENT) == 0)
    {
        (void) sprintf (lin_ptr, "concthreads %lu", prm_ptr->prm_max_concurrentl);
        iret = WRT_TOF_LINE (com_ptr, NULL);
        if (!psl_test_status (iret)) goto wrt_tof_tstprms_end;
    }

    /*
     *  Write dump on exception record:
     *
     *  dumponexc <state>
     */
    if ((prm_ptr->prm_default_flagsl & PRM_TSTDF_DUMP_STYLE) == 0)
    {
        (void) sprintf (lin_ptr, "dumponexc %lu", prm_ptr->prm_dump_stylel);
        iret = WRT_TOF_LINE (com_ptr, NULL);
        if (!psl_test_status (iret)) goto wrt_tof_tstprms_end;
    }

    /*
     *  Write fail test-case message record:
     *
     *  tcfailmsg <string>
     */
    if ((prm_ptr->prm_default_flagsl & PRM_TSTDF_FAILTC) == 0)
    {
        (void) sprintf (lin_ptr, "tcfailmsg %s", prm_ptr->prm_failtcc84);
        iret = WRT_TOF_LINE (com_ptr, NULL);
        if (!psl_test_status (iret)) goto wrt_tof_tstprms_end;
    }

    /*
     *  Write hardcopy device record:
     *
     *  hcdevice <device>
     */
    if ((prm_ptr->prm_default_flagsl & PRM_TSTDF_HARDCOPY_LOG) == 0)
    {
        (void) sprintf (lin_ptr, "hcdevice %s", prm_ptr->prm_hardcopy_logc52);
        iret = WRT_TOF_LINE (com_ptr, NULL);
        if (!psl_test_status (iret)) goto wrt_tof_tstprms_end;
    }

    /*
     *  Write ignore informational record:
     *
     *  ignoreinf <state>
     */
    if ((prm_ptr->prm_default_flagsl & PRM_TSTDF_IGNRINF) == 0)
    {
        (void) sprintf (lin_ptr, "ignoreinf %lu", prm_ptr->prm_ignrinfl);
        iret = WRT_TOF_LINE (com_ptr, NULL);
        if (!psl_test_status (iret)) goto wrt_tof_tstprms_end;
    }

    /*
     *  Write input rate record:
     *
     *  inprate <string>
     */
    if ((prm_ptr->prm_default_flagsl & PRM_TSTDF_INPUT_RATE) == 0)
    {
        (void) sprintf (lin_ptr, "inprate %s", prm_ptr->prm_input_ratec8);
        iret = WRT_TOF_LINE (com_ptr, NULL);
        if (!psl_test_status (iret)) goto wrt_tof_tstprms_end;
    }

    /*
     *  Write pass test-case message record:
     *
     *  tcpassmsg <string>
     */
    if ((prm_ptr->prm_default_flagsl & PRM_TSTDF_PASSTC) == 0)
    {
        (void) sprintf (lin_ptr, "tcpassmsg %s", prm_ptr->prm_passtcc84);
        iret = WRT_TOF_LINE (com_ptr, NULL);
        if (!psl_test_status (iret)) goto wrt_tof_tstprms_end;
    }

    /*
     *  Write working set quota record:
     *
     *  wksq <quota>
     */
    if ((prm_ptr->prm_default_flagsl & PRM_TSTDF_WSQUOTA) == 0)
    {
        (void) sprintf (lin_ptr, "wksq %lu", prm_ptr->prm_wsquotal);
        iret = WRT_TOF_LINE (com_ptr, NULL);
        if (!psl_test_status (iret)) goto wrt_tof_tstprms_end;
    }

    iret = PSL_NORMAL;

wrt_tof_tstprms_end:
    ;
    return iret;
}

/******************************************************************************
 * @func   This routine writes the 'tof' test thread definition records to the
 *      script object ('tof') file.
 *
 * @rdesc   Unsigned long PSL routine return status.
 * @parm    Pointer to the scl common data structure.
 */
static unsigned long WRT_TOF_TSTTHRS (struct scl_com *com_ptr)
{
    /*
     *  Local data.
     */
    struct scl_tpa   *tpa_ptr;      /* Tparse data */
    struct tstthrtbl *tstthr_ptr;       /* Test thread symbol table entry */
    unsigned long    iret;          /* PSL routine return status */
    unsigned long    tstthr_cnt;        /* Test thread count */

    /*
     *  Copy pointers for ease of reference.
     */
    tpa_ptr = com_ptr->tpa_ptrps;   /* Tparse data pointer */

    /*
     *  Loop through the test thread symbol table writing a test thread definition
     *  record for each entry.
     */
    for (tstthr_ptr = tpa_ptr->tstthr_begps,
            tstthr_cnt = 1;
            tstthr_cnt <= tpa_ptr->tstthr_cntl;
            tstthr_ptr++, tstthr_cnt++)
    {
        iret = WRT_TOF_TSTTHR (com_ptr, tstthr_ptr);
        if (!psl_test_status (iret)) goto wrt_tof_tstthrs_end;
    }

    iret = PSL_NORMAL;

    /*
     *  All done - return with status.
     */
wrt_tof_tstthrs_end:
    ;
    return iret;
}

/******************************************************************************
 * @func    This routine writes a single Thrdef record to the script object
 *      ('tof') file from a test thread definition. A Thrdef record has the
 *      following format:
 *
 *      Thrdef
 *      name <thread_name>
 *      target <target_id>
 *
 * @rdesc   Unsigned long PSL routine return status.
 * @parm    Pointer to the scl common data structure.
 */
static unsigned long WRT_TOF_TSTTHR (struct scl_com *com_ptr,
                                     struct tstthrtbl *tstthr_ptr)
{
    struct scl_tpa   *tpa_ptr;      /* Tparse data */
    struct tthr  *tthr_ptr;     /* Pointer to test thread data */
    struct tdf   *tdf_ptr;      /* Pointer to test definition data */
    unsigned long    iret;          /* PSL routine return status */
    char    *lin_ptr;      /* Pointer to line buffer */

    /*
     *  Copy pointers for ease of reference.
     */
    tpa_ptr = com_ptr->tpa_ptrps;   /* Tparse data pointer */
    lin_ptr = com_ptr->toflinpc; /* 'tof' line buffer pointer */
    tdf_ptr = tstthr_ptr->tstthr_tdf_ptrps;
    /* Pointer to thread data */
    tthr_ptr = tstthr_ptr->tstthr_tthr_ptrps;
    /* Test thread data pointer */
    /*
     *  Write thread definition header:
     *
     *  Thrdef
     */
    iret = WRT_TOF_RECHDR (com_ptr, "Thrdef", tstthr_ptr->tstthr_tof_srclinl);
    if (!psl_test_status (iret)) goto wrt_tof_tstthr_end;

    /*
     *  Write name record:
     *
     *  name <thread_name>
     */
    (void) sprintf (lin_ptr, "name %s", tstthr_ptr->tstthr_namec68);
    iret = WRT_TOF_LINE (com_ptr, NULL);
    if (!psl_test_status (iret)) goto wrt_tof_tstthr_end;

    /*
     *  Write target record:
     *
     *  target <target_id>
     */
    (void) sprintf (lin_ptr, "target %lu", tdf_ptr->tdf_targetl);
    iret = WRT_TOF_LINE (com_ptr, NULL);
    if (!psl_test_status (iret)) goto wrt_tof_tstthr_end;

    iret = PSL_NORMAL;

wrt_tof_tstthr_end:
    ;
    return iret;
}

/******************************************************************************
 * @func   This routine writes the 'tof' variable definition records to the
 *      script object ('tof') file.
 *
 * @rdesc   Unsigned long PSL routine return status.
 * @parm    Pointer to the scl common data structure.
 */
static unsigned long WRT_TOF_VARS (struct scl_com *com_ptr)
{
    /*
     *  Local data.
     */
    struct scl_tpa   *tpa_ptr;      /* Tparse data */
    struct symtbl    *var_ptr;      /* Variables symbol table entry */
    unsigned long    iret;          /* PSL routine return status */
    unsigned long    var_cnt;       /* Variable count */

    /*
     *  Copy pointers for ease of reference.
     */
    tpa_ptr = (*com_ptr).tpa_ptrps; /* Tparse data pointer */

    /*
     *  Loop through the variables symbol table writing a variable definition
     *  record for each entry.
     */
    for (var_ptr = (*tpa_ptr).variable_begps,
            var_cnt = 1;
            var_cnt <= (*tpa_ptr).var_cntl;
            var_ptr++, var_cnt++)
    {
        iret = WRT_TOF_VAR (com_ptr, var_ptr);
        if (!psl_test_status (iret)) goto wrt_tof_vars_end;
    }

    iret = PSL_NORMAL;

    /*
     *  All done - return with status.
     */
wrt_tof_vars_end:
    ;
    return iret;
}

/******************************************************************************
 * @func    This routine writes a single variable definition record to the
 *      script object ('tof') file. The variable may be a record, a field
 *      within a record or a 'regular' variable'. This routine simply calls
 *      the appropriate routine to create the required definition record.
 *
 * @rdesc   Unsigned long PSL routine return status.
 * @parm    Pointer to the scl common data structure.
 * @parm    Pointer to variable's entry in Variables Symbol Table.
 */
static unsigned long WRT_TOF_VAR (struct scl_com *com_ptr,
                                  struct symtbl *var_ptr)
{
    unsigned long    iret;          /* PSL routine return status */

    /*
     *  Write the appropriate definition record to the TOF file.
     */
    if (var_ptr->var_typel == DTYP_RECORD)      /* If its a record definition */
    {
        iret = WRT_TOF_RECDEF (com_ptr, var_ptr);   /* Write a Recdef record */
    }
    else if (var_ptr->var_recidxl != 0)         /* Else if its a field within a record */
    {
        iret = WRT_TOF_FLDDEF (com_ptr, var_ptr);   /* Write a Flddef record */
    }
    else if (var_ptr->var_addrl == ADDR_CONSTANT)   /* Else if it's a Constant */
    {
        iret = PSL_NORMAL;              /* No entry required in TOF */
    }
    else                        /* Else must be a 'regular variable */
    {
        iret = WRT_TOF_VARDEF (com_ptr, var_ptr);   /* Write a Vardef record */
    }

    return iret;
}

/******************************************************************************
 * @func    This routine writes a single vardef record to the script object
 *      ('tof') file. A Vardef record has the following format:
 *
 *      <PRE>
 *      Vardef
 *      name <variable_name>
 *      options <option_value>
 *      size <variable_size>
 *      {dimensions <dimensions>}   // Dimensions for arrays
 *      {lValues...}        // Optional list or range of values
 *      {ivalue <seed>}     // Optional seed for repeatable random
 *      {file <filename>}       // Filename for file variables
 *      </PRE>
 *
 *      Vardef sub-records not generated by SCL are not listed.
 *
 * @rdesc   Unsigned long PSL routine return status.
 * @parm    Pointer to the scl common data structure.
 * @parm    Pointer to variable's entry in Variables Symbol Table.
 */
static unsigned long WRT_TOF_VARDEF (struct scl_com *com_ptr,
                                     struct symtbl *var_ptr)
{
    unsigned long    iret;          /* PSL routine return status */
    char    *lin_ptr;      /* Pointer to line buffer */

    /*
     *  Copy pointers for ease of reference.
     */
    lin_ptr = com_ptr->toflinpc; /* 'tof' line buffer pointer */

    /*
     *  Write variable definition header:
     *
     *  Vardef
     */
    iret = WRT_TOF_RECHDR (com_ptr, "Vardef", var_ptr->var_tof_srclinl);
    if (!psl_test_status (iret)) goto wrt_tof_vardef_end;

    /*
     *  Write variable name record.
     */
    iret = WRT_TOF_VARNAM (com_ptr, var_ptr);
    if (!psl_test_status (iret)) goto wrt_tof_vardef_end;

    /*
     *  Write options record.
     */
    iret = WRT_TOF_VAROPT (com_ptr, var_ptr);
    if (!psl_test_status (iret)) goto wrt_tof_vardef_end;

    /*
     *  Write size record.
     */
    iret = WRT_TOF_VARSIZ (com_ptr, var_ptr);
    if (!psl_test_status (iret)) goto wrt_tof_vardef_end;

    /*
     *  Write dimensions record, if variable is an array.
     */
    if (var_ptr->var_array_dimcntl != 0)
    {
        iret = WRT_TOF_VARDEFDIM (com_ptr, var_ptr);
        if (!psl_test_status (iret)) goto wrt_tof_vardef_end;
    }

    /*
     *  Write values record, if variable has an associated list or range of values.
     */
    if ((var_ptr->var_optl & (VOPT_LIST|VOPT_RANGE)) != 0)
    {
        iret = WRT_TOF_VARDEFVAL (com_ptr, var_ptr);
        if (!psl_test_status (iret)) goto wrt_tof_vardef_end;
    }

    /*
     *  Write seed record, if seed has been specified.
     */
    if ((var_ptr->var_optl & VOPT_SEED) != 0)
    {
        (void) sprintf (lin_ptr, "ivalue %ld", var_ptr->var_seedl);
        iret = WRT_TOF_LINE (com_ptr, NULL);
        if (!psl_test_status (iret)) goto wrt_tof_vardef_end;
    }

    /*
     *  Write file record, if this is a file variable.
     */
    if ((var_ptr->var_optl & VOPT_FILE) != 0)
    {
        (void) sprintf (lin_ptr, "file %s", var_ptr->var_filnam_pc);
        iret = WRT_TOF_LINE (com_ptr, NULL);
        if (!psl_test_status (iret)) goto wrt_tof_vardef_end;
    }

    iret = PSL_NORMAL;
wrt_tof_vardef_end:
    ;
    return iret;
}

/******************************************************************************
 * @func    This routine writes a Vardef dimensions record to the script object
 *      ('tof') file. A Vardef dimensions record has the following format:
 *
 *      <PRE>
 *      dimensions <dimensions>
 *      </PRE>
 *
 *      where, <dimensions> is a comma separated list of array bounds, e.g.
 *      100,200,5-9. The lower bound may be omitted if its value is one.
 *
 * @rdesc   Unsigned long PSL routine return status.
 * @parm    Pointer to the scl common data structure.
 * @parm    Pointer to variable's symbol table entry.
 */
static unsigned long WRT_TOF_VARDEFDIM (struct scl_com *com_ptr,
                                        struct symtbl *var_ptr)
{
    struct scl_tpa   *tpa_ptr;      /* Tparse data */
    char    valbuf [TOFBUFLEN];    /* Bound value buffer */
    unsigned long    iret;          /* PSL routine return status */
    long    isub;          /* Loop control */
    char    *lin_ptr;      /* Pointer to line buffer */

    /*
     *  Copy pointers for ease of reference.
     */
    tpa_ptr = com_ptr->tpa_ptrps;   /* Tparse data pointer */
    lin_ptr = com_ptr->toflinpc; /* 'tof' line buffer pointer */

    /*
     *  If this variable is not an array, all done.
     */
    if (var_ptr->var_array_dimcntl == 0) goto wrt_tof_vardefdim_end;

    /*
     *  Write the dimensions record. This contains a comma separated list of array
     *  bounds in which the bounds are separated by a separator string and the
     *  lower bound can be omitted if it is one, e.g.:
     *
     *  dimensions 100,200,5-9  // Array with 3 dimensions, 1:100,1:200 and 5:9.
     */
    (void) strcpy (lin_ptr, "dimensions ");

    for (isub = 0; isub < var_ptr->var_array_dimcntl; isub++)
    {
        if (var_ptr->var_array_bounds1l10[isub] != 1)
        {
            (void) sprintf (valbuf, "%ld", var_ptr->var_array_bounds1l10[isub]);
            (void) strcat (lin_ptr, valbuf);
            (void) strcat (lin_ptr, TOF_OPR_RNGSEP);
        }

        (void) sprintf (valbuf, "%ld", var_ptr->var_array_bounds2l10[isub]);
        (void) strcat (lin_ptr, valbuf);

        if (isub < (var_ptr->var_array_dimcntl - 1))
        {
            (void) strcat (lin_ptr, ",");
        }
    }

    iret = WRT_TOF_LINE (com_ptr, NULL);
    if (!psl_test_status (iret)) goto wrt_tof_vardefdim_end;

wrt_tof_vardefdim_end:
    ;
    return iret;
}

/******************************************************************************
 * @func    This routine writes a Vardef values record to the script object
 *      ('tof') file. A Vardef values record has the following format:
 *
 *      <PRE>
 *      lValues <ltxt_delimiter>
 *      <values>
 *      <ltxt_delimiter>
 *      </PRE>
 *
 *      where, <values> is the list or range of values. A list may be split
 *      over multiple lines.
 *
 * @rdesc   Unsigned long PSL routine return status.
 * @parm    Pointer to the scl common data structure.
 * @parm    Pointer to variable's symbol table entry.
 */
static unsigned long WRT_TOF_VARDEFVAL (struct scl_com *com_ptr,
                                        struct symtbl *var_ptr)
{
    struct scl_tpa   *tpa_ptr;      /* Tparse data */
    unsigned long    iret;          /* PSL routine return status */
    char    *lin_ptr;      /* Pointer to line buffer */

    /*
     *  Copy pointers for ease of reference.
     */
    tpa_ptr = com_ptr->tpa_ptrps;   /* Tparse data pointer */
    lin_ptr = com_ptr->toflinpc; /* 'tof' line buffer pointer */

    /*
     *  If this variable has no associated list or range, all done.
     */
    if ((var_ptr->var_optl & (VOPT_LIST|VOPT_RANGE)) == 0) goto wrt_tof_vardefval_end;

    /*
     *  Write 'lValues <ltxt_delimiter>'
     */
    (void) sprintf (lin_ptr, "lValues %s", LTXT_DELIMITER);
    iret = WRT_TOF_LINE (com_ptr, NULL);
    if (!psl_test_status (iret)) goto wrt_tof_vardefval_end;

    /*
     *  Write '<values>'
     */
    if (((var_ptr->var_optl & VOPT_LIST) != 0) &&
            (var_ptr->var_typel == DTYP_INTEGER))
    {
        iret = WRT_TOF_INTLIS (com_ptr, var_ptr);
    }
    else if (((var_ptr->var_optl & VOPT_LIST) != 0) &&
             (var_ptr->var_typel == DTYP_CHARACTER))
    {
        iret = WRT_TOF_CHRLIS (com_ptr, var_ptr);
    }
    else if (((var_ptr->var_optl & VOPT_RANGE) != 0) &&
             (var_ptr->var_typel == DTYP_INTEGER))
    {
        iret = WRT_TOF_INTRNG (com_ptr, var_ptr);
    }

    if (!psl_test_status (iret)) goto wrt_tof_vardefval_end;

    /*
     *  Write '<ltxt_delimiter>'.
     */
    iret = WRT_TOF_LINE (com_ptr, LTXT_DELIMITER);
    if (!psl_test_status (iret)) goto wrt_tof_vardefval_end;

wrt_tof_vardefval_end:
    ;
    return iret;
}

/******************************************************************************
 * @func    This routine writes a variable (Vardef/Recdef/Flddef) name
 *      record to the script object ('tof') file. A variable name record
 *      has the following format:
 *
 *      <PRE>
 *      name <variable_name>
 *      </PRE>
 *
 * @rdesc   Unsigned long PSL routine return status.
 * @parm    Pointer to the scl common data structure.
 * @parm    Pointer to variable's symbol table entry.
 */
static unsigned long WRT_TOF_VARNAM (struct scl_com *com_ptr,
                                     struct symtbl *var_ptr)
{
    struct scl_tpa   *tpa_ptr;      /* Tparse data */
    unsigned long    iret;          /* PSL routine return status */
    char    varnam[TOFBUFLEN]; /* Variable name */
    char    *lin_ptr;      /* Pointer to line buffer */

    /*
     *  Copy pointers for ease of reference.
     */
    tpa_ptr = (*com_ptr).tpa_ptrps; /* Tparse data pointer */
    lin_ptr = (*com_ptr).toflinpc;   /* 'tof' line buffer pointer */

    /*
     *  Write variable name record:
     *
     *  name <variable_name>
     */
    (void) memcpy (varnam, var_ptr->var_namec16, var_ptr->var_nlengthl);
    varnam[var_ptr->var_nlengthl] = '\0';
    (void) sprintf (lin_ptr, "name %s", varnam);
    iret = WRT_TOF_LINE (com_ptr, NULL);
    if (!psl_test_status (iret)) goto wrt_tof_varnam_end;

wrt_tof_varnam_end:
    ;
    return iret;
}

/******************************************************************************
 * @func    This routine writes a variable (Vardef/Recdef/Flddef) options
 *      record to the script object ('tof') file. A variable options record
 *      has the following format:
 *
 *      <PRE>
 *      options <option_value>
 *      </PRE>
 *
 *      where, <option_value> is a numeric (decimal) option mask.
 *
 * @rdesc   Unsigned long PSL routine return status.
 * @parm    Pointer to the scl common data structure.
 * @parm    Pointer to variable's symbol table entry.
 */
static unsigned long WRT_TOF_VAROPT (struct scl_com *com_ptr,
                                     struct symtbl *var_ptr)
{
    struct scl_tpa   *tpa_ptr;      /* Tparse data */
    unsigned long    iret;          /* PSL routine return status */
    unsigned long    optval;        /* Option value */
    char    *lin_ptr;      /* Pointer to line buffer */

    /*
     *  Copy pointers for ease of reference.
     */
    tpa_ptr = (*com_ptr).tpa_ptrps; /* Tparse data pointer */
    lin_ptr = (*com_ptr).toflinpc;   /* 'tof' line buffer pointer */

    /*
     *  Write options record:
     *
     *  option <options_value>
     */
    GET_VAROPT (com_ptr, var_ptr, &optval);
    (void) sprintf (lin_ptr, "options %lu", optval);
    iret = WRT_TOF_LINE (com_ptr, NULL);
    if (!psl_test_status (iret)) goto wrt_tof_varopt_end;

wrt_tof_varopt_end:
    ;
    return iret;
}

/******************************************************************************
 * @func    This routine writes a variable (Vardef/Recdef/Flddef) size
 *      record to the script object ('tof') file. A variable size record
 *      has the following format:
 *
 *      <PRE>
 *      size <size>
 *      </PRE>
 *
 *      where, <option_value> is a numeric (decimal) option mask.
 *
 * @rdesc   Unsigned long PSL routine return status.
 * @parm    Pointer to the scl common data structure.
 * @parm    Pointer to variable's symbol table entry.
 */
static unsigned long WRT_TOF_VARSIZ (struct scl_com *com_ptr,
                                     struct symtbl *var_ptr)
{
    struct scl_tpa   *tpa_ptr;      /* Tparse data */
    unsigned long    iret;          /* PSL routine return status */
    unsigned long    varsiz;        /* Variable size */
    char    *lin_ptr;      /* Pointer to line buffer */

    /*
     *  Copy pointers for ease of reference.
     */
    tpa_ptr = (*com_ptr).tpa_ptrps; /* Tparse data pointer */
    lin_ptr = (*com_ptr).toflinpc;   /* 'tof' line buffer pointer */

    /*
     *  Write size record:
     *
     *  size <size>
     *
     *  Integer variables should always be written as size 1 (ideally no size
     *  record should be written, but this is not currently supported by the
     *  run-time component). Character variables should specify the variable size.
     */
    if (var_ptr->var_typel == DTYP_INTEGER)
    {
        varsiz = 1;
    }
    else
    {
        varsiz = var_ptr->var_sizel;
    }

    (void) sprintf (lin_ptr, "size %ld", varsiz);
    iret = WRT_TOF_LINE (com_ptr, NULL);
    if (!psl_test_status (iret)) goto wrt_tof_varsiz_end;

wrt_tof_varsiz_end:
    ;
    return iret;
}
