/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/


#ifndef _CYR_ATVFINPUT_H
#define _CYR_ATVFINPUT_H
/**
 * Class within which to head the input stream and give us the
 * extra functionality we require for reading Atvf files.
 **/

#include <string>
#ifdef HPUX
#include <fstream.h>
#else
#include <fstream>
#endif

#ifndef CYR_NO_STD_NAMESPACE
using namespace std;
#endif
#if defined (WIN32)
#pragma warning(disable:4786)
#endif // unknown

// class definition
class AtvfInput
{
    enum {BUFFLEN = 65535};
    char buff[BUFFLEN];
    ifstream* _ifP;
    istream* _inP;
    bool _moreOnLine;
    string _strbuf;
    const bool _ignoreComments;
    // Helper to getMatch()
    bool getMatchHelper(const string& type);
public:
    //Constructors and destructor
    AtvfInput(const char* fname, const bool ignore_comments = false );
    AtvfInput(istream* inP, const bool ignore_comments = false );
    ~AtvfInput();

    //Accessors
    bool moreOnLine() const;
    bool ready() const;
    bool eof() const;

    //Mutators
    void open(const char* fname);
    string* getLine();
    string* getToken();
    string* getUntil(const string& delim);
    bool getMatch(const string& type);
private:
    //// Forbidden.
    AtvfInput();
    AtvfInput(const AtvfInput&);
    operator=(const AtvfInput&);
}
; // AtvfInput

#endif // _CYR_ATVFINPUT_H
