/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

// CacheEntry.cpp
/**
 * Method definitions for the CacheEntry class
 **/

// local includes
#include "CacheEntry.h"

// Init static class variables
unsigned int CacheEntry::_zeroRefCnts = 0;

/**
 * Constructor, create a cache entry object
 **/
CacheEntry::CacheEntry()
        :   _refCnt(1)
{ // Assume one use on construction
}

/**
 * Destructor
 **/
CacheEntry::~CacheEntry()
{
    if((_refCnt == 0) && (_zeroRefCnts > 0)) _zeroRefCnts--;
}

/**
 * Static accessor to find out if any of Cache entries have dropped
 * to zero usage.
 **/
bool CacheEntry::someUnused()
{
    return (_zeroRefCnts > 0) ? true : false;
}

/**
 * Accessor to check on the reference count of this cache entry
 **/
bool CacheEntry::inUse() const
{
    return (_refCnt > 0) ? true : false;
}

/**
 * Mutator to increment the reference count of this cache entry
 **/
void CacheEntry::addUse()
{
    if((_refCnt == 0) && (_zeroRefCnts > 0)) _zeroRefCnts--;
    _refCnt++;
    return;
}

/**
 * Mutator to decrement the reference count of this cache entry
 **/
void CacheEntry::release()
{
    if(_refCnt > 0)
    {  // If this not true something broke.
        _refCnt--;
        if(_refCnt == 0) _zeroRefCnts++;
    }
    return;
}

/**
 * Define operator << to allow a CacheEntry to be output.
 * Mostly for debugging
 **/
ostream& operator<<(ostream& os, const CacheEntry& entry)
{
    os << " entry with " << entry._refCnt << " uses";
    return os;
}

