/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

///////////////////////////////////////////////////////////////////////////////
// @doc     TInterRecords.inl
///////////////////////////////////////////////////////////////////////////////

#include "TInterRecords.h"

// Define RTTI keys for each record type ...

// VARIABLE MANIPULATION

// TIMER
const AtvfKey TRcrdStimer::			KEY("Stimer");
const AtvfKey TRcrdEtimer::			KEY("Etimer");

#ifndef __DLLSA__
const AtvfKey TRcrdAcmutex::			KEY("Acmutex");
const AtvfKey TRcrdRlmutex::			KEY("Rlmutex");
const AtvfKey TRcrdStsemaphore::		KEY("Stsemaphore");
const AtvfKey TRcrdWtsemaphore::		KEY("Wtsemaphore");
const AtvfKey TRcrdClsemaphore::		KEY("Clsemaphore");
#endif

// *****************************************************************************
// * TIMERS
// *****************************************************************************
bool TRcrdStimer::_registeredCreator = false;
const bool TRcrdStimer::_RegisterCreator()
{
    if (!recCreatorMap) recCreatorMap = new RecCreator_map_t;
    (*recCreatorMap)[TRcrdStimer::KEY] = SharedPtr<TRecCreator>(new TRecMake<TRcrdStimer>());
    TRcrdStimer::_registeredCreator = true;
    return true;
}

bool TRcrdEtimer::_registeredCreator = false;
const bool TRcrdEtimer::_RegisterCreator()
{
    if (!recCreatorMap) recCreatorMap = new RecCreator_map_t;
    (*recCreatorMap)[TRcrdEtimer::KEY] = SharedPtr<TRecCreator>(new TRecMake<TRcrdEtimer>());
    TRcrdEtimer::_registeredCreator = true;
    return true;
}


#ifndef __DLLSA__

// *****************************************************************************
// * SYNCHRONISATION
// *****************************************************************************

bool TRcrdAcmutex::_registeredCreator = false;
const bool TRcrdAcmutex::_RegisterCreator()
{
    if (!recCreatorMap) recCreatorMap = new RecCreator_map_t;
    (*recCreatorMap)[TRcrdAcmutex::KEY] = SharedPtr<TRecCreator>(new TRecMake<TRcrdAcmutex>());
    TRcrdAcmutex::_registeredCreator = true;
    return true;
}

bool TRcrdRlmutex::_registeredCreator = false;
const bool TRcrdRlmutex::_RegisterCreator()
{
    if (!recCreatorMap) recCreatorMap = new RecCreator_map_t;
    (*recCreatorMap)[TRcrdRlmutex::KEY] = SharedPtr<TRecCreator>(new TRecMake<TRcrdRlmutex>());
    TRcrdRlmutex::_registeredCreator = true;
    return true;
}

bool TRcrdStsemaphore::_registeredCreator = false;
const bool TRcrdStsemaphore::_RegisterCreator()
{
    if (!recCreatorMap) recCreatorMap = new RecCreator_map_t;
    (*recCreatorMap)[TRcrdStsemaphore::KEY] = SharedPtr<TRecCreator>(new TRecMake<TRcrdStsemaphore>());
    TRcrdStsemaphore::_registeredCreator = true;
    return true;
}

bool TRcrdWtsemaphore::_registeredCreator = false;
const bool TRcrdWtsemaphore::_RegisterCreator()
{
    if (!recCreatorMap) recCreatorMap = new RecCreator_map_t;
    (*recCreatorMap)[TRcrdWtsemaphore::KEY] = SharedPtr<TRecCreator>(new TRecMake<TRcrdWtsemaphore>());
    TRcrdWtsemaphore::_registeredCreator = true;
    return true;
}

bool TRcrdClsemaphore::_registeredCreator = false;
const bool TRcrdClsemaphore::_RegisterCreator()
{
    if (!recCreatorMap) recCreatorMap = new RecCreator_map_t;
    (*recCreatorMap)[TRcrdClsemaphore::KEY] = SharedPtr<TRecCreator>(new TRecMake<TRcrdClsemaphore>());
    TRcrdClsemaphore::_registeredCreator = true;
    return true;
}

#endif
