/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

#include <stdlib.h>
#include <string.h>
#include "mdbtypes2.h"

char * mfc_getenvd(const char *, const char *);

/* Return the value of an Environment Variable */
/*
 * This routine returns a pointer to static data which gets overwritten
 * by each call. Where getenv() fails, a pointer to a NULL string is
 * returned. Buffer is shared with mdb_getenvd() !.
 */

static	char	mdbenv[MAX_PATH_LEN+1];

char * mdb_getenv(const char * envar)
{

    char	*tenv;

#if defined(WIN32) && defined(WINGUI)

    tenv = mfc_getenvd(envar, NULL);

#else

    tenv = getenv(envar);

#endif

    if(tenv != NULL)
    {
        strncpy(mdbenv, tenv, MAX_PATH_LEN);
        mdbenv[MAX_PATH_LEN]='\0';
    }
    else
    {
        mdbenv[0]='\0';
    }
    return(mdbenv);
}

/* Return the value of an Environment Variable with default */
/*
 * This routine returns a pointer to static data which gets overwritten
 * by each call. Where getenv() fails, a pointer to the default string is
 * returned. Buffer is shared with mdb_getenv() !.
 */



char * mdb_getenvd(const char * envar, const char * edefval)
{
    char	*tenv;

#if defined(WIN32) && defined(WINGUI)

    tenv = mfc_getenvd(envar, edefval);

#else

    tenv = getenv(envar);

#endif

    if(tenv != NULL)
    {
        strncpy(mdbenv, tenv, MAX_PATH_LEN);
    }
    else
    {
        strncpy(mdbenv, edefval, MAX_PATH_LEN);
    }
    mdbenv[MAX_PATH_LEN]='\0';
    return(mdbenv);
}
