/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

// mdbmtmacros.h. Generic multithread utility macros
//

#if defined(USE_POSIX_THREADS)
#include <pthread.h>

#if defined(HPV10) || defined(__VAX)
#define DFLT_PTHREAD_MUTEX_ATTR	pthread_mutexattr_default

#else
#define DFLT_PTHREAD_MUTEX_ATTR	NULL

#endif

typedef pthread_mutex_t MDBMUTEX;
#define INIT_MUTEX(m)   pthread_mutex_init(&m, DFLT_PTHREAD_MUTEX_ATTR)
#define LOCK_MUTEX(m)   pthread_mutex_lock(&m)
#define UNLOCK_MUTEX(m) pthread_mutex_unlock(&m)
#define CLOSE_MUTEX(m)

#elif defined(WIN32)
#include <windows.h>
typedef HANDLE          MDBMUTEX;
#define INIT_MUTEX(m)   m = CreateMutex(NULL, FALSE, NULL)
#define LOCK_MUTEX(m)   WaitForSingleObject(m, INFINITE)
#define CLOSE_MUTEX(m)  CloseHandle(m)

#define UNLOCK_MUTEX(m) ReleaseMutex(m)

#elif defined(UNIX)
#include <thread.h>
typedef mutex_t         MDBMUTEX;
#define INIT_MUTEX(m)   mutex_init(&m, USYNC_THREAD, 0)
#define LOCK_MUTEX(m)   mutex_lock(&m)
#define UNLOCK_MUTEX(m) mutex_unlock(&m)
#define CLOSE_MUTEX(m)

#endif
