/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

#ifdef WIN32
#pragma warning(disable:4786)
#pragma warning(disable:4503)
#endif

#include "TPLog.h"
#include "threadcontrol.h"
#include "mdbthread.h"
#ifndef CYRANO_TP_STANDALONE
#include "TestProcess.h"
#include <Objects/SpawnProc.h>
#include <GlobalVariable.h>
#endif
#include <assert.h>
void Cmdbthread_start(Cmdbthread* thread)
{
    thread->start();
}
Cmdbthread::Cmdbthread(Cthreadcontrol* threadcontrol,string filename)
        : taskP(NULL),m_threadcontrol(threadcontrol),abort_message("No exception")
{
    m_running_thread = NULL;
    set_file(filename);
}

Cmdbthread::~Cmdbthread()
{
    end_thread();

    if(taskP != NULL)
        delete taskP;
}
void Cmdbthread::ActivateThread(void)
{
    logr->AddTrace(DBGSTD,DEBUGLEVEL,"START THREAD");
    //Set thread going and return
    m_running_thread=new CTThread(makeFunctorA1(Cmdbthread_start,Cmdbthread*,this));
}
//Run actual thread and catch exceptions we can deal with
void* Cmdbthread::start(void)
{
    bool result=false;//not successfull
    try
    {
        logr->AddTrace(DBGSTD,DETAILLEVEL,"Retrieve Task to run");
        try
        {
            taskP=m_threadcontrol->RetrieveTask(get_file());
            assert(taskP);
        }
        catch(const mdbRuntimeError &e)
        {
            string tmsg = "Task creation failed,";
            tmsg += e.what();
            logr->AddTrace(DBGSTD,DEBUGLEVEL,tmsg.c_str());
            throw;
        }
        logr->AddTrace(DBGSTD,DETAILLEVEL,"Run OS Thread");
        result=run();
    }
    // Catch exceptions during run. Add additional exception types
    // as required.
    catch(const string& es)
    {
        // If not already aborting, try to report error
        if(!Cthreadcontrol::is_aborting())
            exception_abort(es);
        result=false;
    }
    catch(const char * es)
    {
        // If not already aborting, try to report error
        if(!Cthreadcontrol::is_aborting())
            exception_abort(es);
        result=false;
    }
    catch(const bad_alloc&)
    {
        // If not already aborting, try to report error
        if(!Cthreadcontrol::is_aborting())
            exception_abort("memory allocation failure");
        result=false;
    }
    catch(const mdbIOException &e)
    {
        // If not already aborting, try to report error
        if(!Cthreadcontrol::is_aborting())
        {
            string err=e.what();
            err += " "+e.ioname();
            exception_abort(err);
        }
        result=false;
    }
    catch(const DetachThreadException &e)
    {
        // If not already aborting, Report Nioe message to say we are closing down
        if(!Cthreadcontrol::is_aborting())
        {
            log(e.what());
            CleanUp();
            // Attempt cleanup. True flag causes any connection/command
            // specific cleanup methods to be called.
            cleanup(true);
        }
        result=false;
    }
    catch(const mdbRuntimeError &e)
    {
        // If not already aborting, try to report error
        if(!Cthreadcontrol::is_aborting())
            exception_abort(e.what());
        result=false;
    }
    catch (const CyrProcessException& e)
    {
        // If not already aborting, try to report error
        if(!Cthreadcontrol::is_aborting())
        {
#ifndef CYRANO_TP_STANDALONE
            string err=string(e.what) + "," + string(e.where);
#else
            string err=string(e.what()) + "," + string(e.where());
#endif
            exception_abort(err);
        }
        result=false;
    }
    catch(const CyrRepositoryException& e)
    {
        // If not already aborting, try to report error
        if(!Cthreadcontrol::is_aborting())
        {
            string err=string(e.what()) + "," + string(e.where());
            exception_abort(err);
        }
        result=false;
    }
#ifndef CYRANO_TP_STANDALONE
    catch(const CyrGlobalVariableException& e)
    {
        // If not already aborting, try to report error
        if(!Cthreadcontrol::is_aborting())
        {
            exception_abort((const char*)e.what);
        }
        result=false;
    }
#endif
    catch(const CyrException& e)
    {
        // If not already aborting, try to report error
        if(!Cthreadcontrol::is_aborting())
        {
            string err=string(e.what()) + "," + string(e.where());
            exception_abort(err);
        }
        result=false;
    }
#ifndef CYRANO_TP_STANDALONE
    catch(const CyrSpawnException& e)
    {
        // If not already aborting, try to report error
        if(!Cthreadcontrol::is_aborting())
        {
            string err=string(e.what) + "," + string(e.where);
            exception_abort(err);
        }
        result=false;
    }
#endif
    catch(const exception& e)
    {
        // If not already aborting, try to report error
        if(!Cthreadcontrol::is_aborting())
            exception_abort(e.what());
        result=false;
    }
    //catch CORBA exceptions
#ifndef CYRANO_TP_STANDALONE
    catch(const CORBA::Exception& e)
    {
        // Some other unexpected exception caught
        if(!Cthreadcontrol::is_aborting())
        {
            string msg="Caught CORBA::"+CORBAExString(e);
            exception_abort(msg.c_str());
        }
        result=false;
    }
#endif
    // Other exception types or C signals (signals only caught on WIN32).
    catch( ... )
    {
        // If not already aborting, try to report error
        if(!Cthreadcontrol::is_aborting())
            exception_abort("signal or unknown type");
        result=false;
    }
    if(result!=true)//something happened throw to be caught by thread Handling
    {
        logr->AddTrace(DBGSTD,DETAILLEVEL,"Finished Running OS Thread Abnormally");
        string ex_msg("Thread ");
        ex_msg+=GetThreadID();
        ex_msg+=" ended abnormally with ";
        ex_msg+=abort_message;
        throw mdbParseException(ex_msg);
    }
    logr->AddTrace(DBGSTD,DETAILLEVEL,"Finished Running OS Thread Normally");
    return NULL;
}
// Run thread task(s)
// Can be called by kernel thread start routine, must be mt safe
bool Cmdbthread::run(void)
{
    bool bret=false;
    logr->AddTrace(DBGSTD,DETAILLEVEL,"Started Running OS Thread, enter switch-loop");
    LogStartMessage();//log a message dependant on Thread type
    logr->AddTrace(DBGSTD,DEBUGLEVEL,string("Run a task "+GetThreadID()).c_str());
    if((taskP->run(*this)) == -1)
    {
#ifndef __DLLSA__
        logr->AddTrace(DBGSTD,ERRORLEVEL,"Script Failed");
#else
        logr->AddTrace(DBGSTD,10000,"Script Failed");
#endif
        bret=false;
    }
    else
    {
#ifndef __DLLSA__
        logr->AddTrace(DBGSTD,ERRORLEVEL,"Script Completed Successfully");
#else
        logr->AddTrace(DBGSTD,10000,"Script Completed Successfully");
#endif
        bret=true;
    }
    logr->AddTrace(DBGSTD,DEBUGLEVEL,"Have Run a task");
    LogEndMessage();//log a message dependant on Thread type
    CleanUp();
    logr->AddTrace(DBGSTD,DETAILLEVEL,"Leave OS Thread Run");
    return bret;
}

// Wait for the attached OS thread to complete.
void Cmdbthread::join(void)
{
    CTCurrentThread::join(*m_running_thread);
}
// Thread caught an exception/signal, try to report and terminate.
// NOTE: Doing stuff in here is risky, this method can be called
// at interrupt level in response to a caught signal. Any resulting
// system calls may abort or hang.
void Cmdbthread::exception_abort(string reason)
{
    //don't catch execptions, rely on thread handling to close down thread
    static string exmsg("Thread aborted on exception - ");
    // Attempt to report error
    abort_message=string(exmsg + reason
                         + " (File: " + filename_base(get_file())
                         + " Phase: " + get_phase_text() + ")");
    log(abort_message,ERRMSG);
    CleanUp();
    // Attempt cleanup. True flag causes any connection/command
    // specific cleanup methods to be called.
#ifdef __DLLSA__
    logr->AddTrace(DBGSTD,10000,"Script Failed");
#endif
    cleanup(true);
}

// Notify thread it should complete
void Cmdbthread::end_thread(void)
{
    abort();//tell script to abort/complete if it hasn't yet
    m_running_thread->kill(NULL);
}

// Terminate a thread
void Cmdbthread::Terminate(void)
{
    string msg="Terminate thread: "+GetThreadID();
    logr->AddTrace(THREADING,DEBUGLEVEL,msg.c_str());
    end_thread();
}

bool Cmdbthread::ReportStatus()
{
    string reportstr="Thread "+GetThreadID()+" Priority: ";
    enumThreadPriority comp_state=m_running_thread->getPriority();
    string str_comp;
    bool has_passed=true;
    switch(comp_state)
    {
    case PRIORITY_ERROR:
        str_comp="Error";
        break;
    case PRIORITY_LOW:
        str_comp="Low";
        break;
    case PRIORITY_NORMAL:
        str_comp="Normal";
        break;
    case PRIORITY_HIGH:
        str_comp="High";
        has_passed=false;
        break;
    default:
        str_comp="Unknown!!!";
        has_passed=false;
        break;
    }
    reportstr=reportstr+str_comp+" Execution Status: ";
    enumThreadState ex_state=m_running_thread->getState();
    string str_ex;
    switch(ex_state)
    {
    case THREAD_CREATED:
        str_ex="Created";
        break;
    case THREAD_RUNNING:
        str_ex="Running";
        break;
    case THREAD_SUSPENDED:
        str_ex="Suspened";
        break;
    case THREAD_KILLED:
        str_ex="Killed";
        break;
    case THREAD_TERMINATED:
        str_ex="Terminated";
        break;
    default:
        str_ex="Unknown!!!";
        break;
    }
    reportstr=reportstr+str_ex;
    logr->AddTrace(THREADING,DEBUGLEVEL,reportstr.c_str());
    return has_passed;
}
enumThreadState	Cmdbthread::get_state(void) const
{
    return m_running_thread->getState();
}
