/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

// Cthreadcontrol controlling class for a bunch of threads.

#if defined(WIN32)
#pragma warning(disable:4786)
#endif

#include <string>
#include <map>
#include <vector>

#include "mdbthread.h"

#include "TPLog.h"

#include "ScriptCache.h"

typedef map<int, int, less<int> > IDMAP;
//////////////////////////////////////////////////////////////////////
// Cthreadcontrol class
class	Cthreadcontrol
{
public:
    // Constructor/Destructor
    Cthreadcontrol(TScriptRule* rule, int max_threads=DEFAULT_THREAD_MAX);
    virtual	~Cthreadcontrol();
    // Public member functions
    //wait for completion of threads
    bool WaitForCompletion();
    //repotr on current status of threads
    bool ReportOnThreads();
    // Set delay time scale factor
    void    delay_percent(int d);
    // Return active thread count
    int     get_nactive(void)
    {
        return threads.size();
    }
    ThreadTask* RetrieveTask(const string& fileame);
    bool    doesTaskExist(const string& filename);
    // Return pointer to thread at index
    Cmdbthread * get_threadp(int index)
    {
        if(index < (int)threads.size())
            return threads[index];
        else
            return NULL;
    }
    // Return abort in progess flag
    static bool is_aborting(void)
    {
        return aborting;
    }

protected:

public:
    // Number of threads with ok status
    int         okcount;
    // Number of threads with fail status
    int         failcount;
protected:
    // Protected variables
    // Array of thread object pointers
    vector<Cmdbthread*>  threads;


    // Maximum threads allowed
    int         maxthrd;
    // Directory for .tof files (should have a trailing path
    // separator character, / or \)
    string      tofdir;
    // Flag to indicate abort in progress
    static bool aborting;
private:
    ScriptCache scriptCache;
    CTMutex m_cachemutex;//for accessing threadcontrol cache


}

;

