/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

#include <winsock2.h>
#include "CECInterProcessVar.h"
#include "TEMainStructure.hpp"


extern CMainExecuterStructure *g_pTExecuter;

CECInterProcessVar::CECInterProcessVar(const string& name, int size, const string& file,
    const string& fileid, const string& dimensions, const string& values, int options,
    unsigned long seed, const string& scriptname)
        : CECVar(name),_options(options),_size(size)
{
	
	//set it to not found condition -1
	m_lVarId=-1;
	//keep it for later needed for connecting again by the web
	m_strVarName = name;

	m_bOnWeb = false;
	m_bOnLan = false;


	// Connect to the Correct Variable Factory depending on scope
	// try and connect to var factory on repository via LAN
	try
	{
		CECInetInfo inet;
		CORBA::Object_var obj = pCServer->FindObjectWithName(VarFactory::CLASS_NAME,
        inet.GetIPAddressAsString(g_pTExecuter->GetRepositoryHost()).c_str());
		VarFactory_var VF=VarFactory::_narrow(obj.inout());
		if(!(CORBA::is_nil(VF)))
		{
			obj=VF->GetVariable(name.c_str(),size,file.c_str(),fileid.c_str(),dimensions.c_str(),values.c_str(),options,seed,scriptname.c_str());
			m_variable=GlobalVariable::_narrow(obj.inout());
			m_bOnLan = true;
			return;
		}
	}
	catch(...)
	{
	}
	m_bOnLan = false;

	//the remote version must do a getvariable 
	//keep the varId returned

	try
	{
		CORBA::Object_var obj2 = pCServer->FindObjectWithName(XMLRelayVarFactory::CLASS_NAME,"WebRelay");
		XMLRelayVarFactory_var VFonWeb = XMLRelayVarFactory::_narrow(obj2);
		long lRet= VFonWeb->GetVariable((g_pTExecuter->GetRepositoryHost()).c_str(),name.c_str(),size,file.c_str(),fileid.c_str(),dimensions.c_str(),values.c_str(),options,seed,scriptname.c_str());
		if(lRet>-1)
		{
			//successful -1 is failure we now have a variable id
			m_lVarId = lRet;
			m_bOnWeb = true;
			return;
		}
	}
	catch(...)
	{
	}
	
	//shouldnt have got here!!!
	m_lVarId=-1;
	m_bOnLan = false;
	m_bOnWeb = false;
}
CECInterProcessVar::~CECInterProcessVar()
{
    //Released Reference to Global Variable
}


string  CECInterProcessVar::get_value(const string& expression)
{
    //Return the value stored in CORBA member variable
    CORBA::String_var cb_expression=CORBA::string_dup(expression.c_str());
	
	if(m_bOnLan)
	{
		CORBA::String_var Cstr=m_variable->get_value(cb_expression.in());
		string retstr(Cstr);
		return retstr;
	}
	else if(m_bOnWeb)
	{
		try
		{
			CORBA::Object_var obj2 = pCServer->FindObjectWithName(XMLRelayVarFactory::CLASS_NAME,"WebRelay");
			XMLRelayVarFactory_var VFonWeb = XMLRelayVarFactory::_narrow(obj2);
			CORBA::String_var Cstr=VFonWeb->GlobalVariable_get_value((g_pTExecuter->GetRepositoryHost()).c_str(),m_strVarName.c_str(),cb_expression.in(),m_lVarId);
			string retstr(Cstr);
			return retstr;
		}
		catch(...)
		{
		}
	}
	return "";
}
int CECInterProcessVar::set_value(const string& newval,const string& expression)
{
    // Should Set Value be allowed on a range variable!!!!!!
    //Set the value in the CORBA member variable
	if(m_bOnLan)
	{
		m_variable->set_value(newval.c_str(),expression.c_str());
	}
	else if(m_bOnWeb)
	{
		try
		{
			CORBA::Object_var obj2 = pCServer->FindObjectWithName(XMLRelayVarFactory::CLASS_NAME,"WebRelay");
			XMLRelayVarFactory_var VFonWeb = XMLRelayVarFactory::_narrow(obj2);
			return (VFonWeb->GlobalVariable_set_value((g_pTExecuter->GetRepositoryHost()).c_str(),m_strVarName.c_str(),newval.c_str(),expression.c_str(),m_lVarId));
		}
		catch(...)
		{
		}
	}
    return 0;
}
void    CECInterProcessVar::next_value()
{
	if(m_bOnLan)
	{
		m_variable->next_value();
	}
	else if(m_bOnWeb)
	{
		try
		{
			CORBA::Object_var obj2 = pCServer->FindObjectWithName(XMLRelayVarFactory::CLASS_NAME,"WebRelay");
			XMLRelayVarFactory_var VFonWeb = XMLRelayVarFactory::_narrow(obj2);
			VFonWeb->GlobalVariable_next_value((g_pTExecuter->GetRepositoryHost()).c_str(),m_strVarName.c_str(),m_lVarId);
		}
		catch(...)
		{
		}
	}
}
void    CECInterProcessVar::generate_value()
{
	if(m_bOnLan)
	{
		m_variable->generate_value();
	}
	else if(m_bOnWeb)
	{
		try
		{
			CORBA::Object_var obj2 = pCServer->FindObjectWithName(XMLRelayVarFactory::CLASS_NAME,"WebRelay");
			XMLRelayVarFactory_var VFonWeb = XMLRelayVarFactory::_narrow(obj2);
			VFonWeb->GlobalVariable_generate_value((g_pTExecuter->GetRepositoryHost()).c_str(),m_strVarName.c_str(),m_lVarId);
		}
		catch(...)
		{
		}
	}
}
void    CECInterProcessVar::reset_seed(unsigned long seed)
{
	if(m_bOnLan)
	{
		m_variable->reset_seed();
	}
	else if(m_bOnWeb)
	{
		try
		{
			CORBA::Object_var obj2 = pCServer->FindObjectWithName(XMLRelayVarFactory::CLASS_NAME,"WebRelay");
			XMLRelayVarFactory_var VFonWeb = XMLRelayVarFactory::_narrow(obj2);
			VFonWeb->GlobalVariable_reset_seed((g_pTExecuter->GetRepositoryHost()).c_str(),m_strVarName.c_str(),m_lVarId);
		}
		catch(...)
		{
		}
	}
}
