/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/
#ifndef _CECInterProcessVar_H_
#define _CECInterProcessVar_H_
#include "CORBARepository.h"
#include "OSTAORB/OSTAORB.h"
#include "CECVar.h"
#include "VarFactory.h"
#include "..\XMLRelay\XMLVDK011Lib\XMLRelayVarFactory.h"
#include "..\XMLRelay\XMLVDK010Lib\XMLFileTransferNode.h"
#include <GlobalVariable.h>
extern CORBAServer*     pCServer;//This has become a Global variable due to
//compilation problems if included in some headers
//They could all be replaced by local CORBAServers created with no arguments if this causes any problems
//CORBA variable need names from the repository
//@class Base Class for wrappers to Variables that are available from more than 1 process


class CECInterProcessVar : public CECVar
{
public:
    CECInterProcessVar(const string& name,int size,const string& file,
                       const string& fileid,const string& dimensions,
                       const string& values,int options,
                       unsigned long seed,const string& scriptname);
    virtual ~CECInterProcessVar();
    virtual string  get_value(const string& expression);
    virtual int     set_value(const string& newval,const string& expression);
    virtual void    next_value();//generate next value
    virtual void    generate_value();//generate random value
    virtual void    reset_seed(unsigned long=0);//reset random seed

    //things required by Cvarlist
    virtual int     get_options()
    {
        return _options;
    };
    virtual string  get_name()
    {
        return _name;
    };
    virtual bool    is_volatile()
    {
        return true;
    };
    virtual bool    is_temp(void)
    {
        return false;
    };
    virtual bool    is_private(void)
    {
        return false;
    };
    virtual void    clear_values(void)
    {
        return;
    };
    virtual int     get_size(void)
    {
        return _size;
    };
protected:
    int     _options;
    int     _size;
    GlobalVariable_var  m_variable;
	string m_strVarName;
	bool m_bOnLan;
	bool m_bOnWeb;

	long m_lVarId;

private:
    //// Forbidden.
    CECInterProcessVar();
    CECInterProcessVar(const CECInterProcessVar&);
    operator=(const CECInterProcessVar&);
};
//These following classes are not really required but here for clarity of code
// in class varlist.
class CECInterRangeVar : public CECInterProcessVar
{
public:
    CECInterRangeVar(const string& name,int options,const string& scriptname,
                     const string& values,unsigned long seed)
        :CECInterProcessVar(name,1,"","","",values,options,seed,scriptname)
    {}
    virtual ~CECInterRangeVar()
    {}
private:
    //// Forbidden.
    CECInterRangeVar();
    CECInterRangeVar(const CECInterRangeVar&);
    operator=(const CECInterRangeVar&);
};

class CECInterListVar : public CECInterProcessVar
{
public:
    CECInterListVar(const string& name,int size,int options,const string& scriptname,
                    const string& values,unsigned long seed)
        :CECInterProcessVar(name,size,"","","",values,options,seed,scriptname)
    {}
    virtual ~CECInterListVar()
    {}
private:
    //// Forbidden.
    CECInterListVar();
    CECInterListVar(const CECInterListVar&);
    operator=(const CECInterListVar&);
};

class CECInterFileVar : public CECInterProcessVar
{
public:
    CECInterFileVar(const string& name,int size,int options,const string& fileid,
                    const string& file,const string& scriptname)
        :CECInterProcessVar(name,size,file,fileid,"","",options,0,scriptname)
    {}
    virtual ~CECInterFileVar()
    {}
private:
    //// Forbidden.
    CECInterFileVar();
    CECInterFileVar(const CECInterFileVar&);
    operator=(const CECInterFileVar&);
};

class CECInterArrayVar : public CECInterProcessVar
{
public:
    CECInterArrayVar(const string& name,int size,int options,
                     const string& scriptname,const string& dimensions)
        :CECInterProcessVar(name,size,"","",dimensions,"",options,0,scriptname)
    {}
    virtual ~CECInterArrayVar()
    {}
private:
    //// Forbidden.
    CECInterArrayVar();
    CECInterArrayVar(const CECInterArrayVar&);
    operator=(const CECInterArrayVar&);
};

class CECInterSimpleVar : public CECInterProcessVar
{
public:
    CECInterSimpleVar(const string& name,int size,int options,const string& scriptname)
        :CECInterProcessVar(name,size,"","","","",options,0,scriptname)
    {
    }
    virtual ~CECInterSimpleVar()
    {
    }
private:
    //// Forbidden.
    CECInterSimpleVar();
    CECInterSimpleVar(const CECInterSimpleVar&);
    operator=(const CECInterSimpleVar&);
};
#endif
