/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

#include <winsock2.h>
#include "CECSemaphore.h"
#include "TEMainStructure.hpp"

extern CORBAServer*		pCServer;
extern CMainExecuterStructure *g_pTExecuter;

CECSemaphore::CECSemaphore(const string& name,int scope) {
    m_bOnLan = false;
    m_bOnWeb = false;

    switch(scope) {
      case TOF_SEMSCP_TESTWIDE:
        m_strName = g_pTExecuter->GetTestRun()+"."+name+".TEST-WIDE";
        break;
      case TOF_SEMSCP_LOCAL:
      default:
        m_strName = g_pTExecuter->GetTestRun()+"."+name+".LOCAL";
        break;
    }

    //Does the semaphore already exist on LAN?
    try {
	CORBA::Object_var obj;
        obj = pCServer->FindObjectWithName(TPSemaphore::CLASS_NAME,m_strName.c_str());
	if(!(CORBA::is_nil(obj))) {
	    m_Semaphore = TPSemaphore::_narrow(obj);
	    m_bOnLan = true;
	    return;
	}
    } catch(...) { }

    //can we connect to the repository var factory on tihs LAN?
    try {
	CECInetInfo inet;
	CORBA::Object_var Ofac = pCServer->FindObjectWithName(VarFactory::CLASS_NAME,
        inet.GetIPAddressAsString(g_pTExecuter->GetRepositoryHost()).c_str());
	if(!(CORBA::is_nil(Ofac))) {
	    m_bOnLan = true;
	    try {
		VarFactory_var VF = VarFactory::_narrow(Ofac.inout());
		if(!CORBA::is_nil(VF)) {
		    //CREATING NEW SEMAPHORE ON LAN
		    Ofac=VF->CreateSimpleSemaphore(m_strName.c_str());
		    m_Semaphore=TPSemaphore::_narrow(Ofac);
		    return;
		}
	    } catch(...) {
		return;
	    }
	}
    } catch(...) {
	m_bOnLan = false;
    }

    //Does it already exist on WEB ?
    try {
	CORBA::Object_var obj;
	obj=pCServer->FindObjectWithName(XMLRelayVarFactory::CLASS_NAME,"WebRelay");
	if(!(CORBA::is_nil(obj))) {
	    //narrowed to XML relay var factory
	    XMLRelayVarFactory_var VFonWEB = XMLRelayVarFactory::_narrow(obj);
	    if(!(CORBA::is_nil(VFonWEB))) {
		m_bOnWeb = true;
		try {
		    if(VFonWEB->DoesSimpleSemaphoreExist((g_pTExecuter->GetRepositoryHost()).c_str(),
							 m_strName.c_str())) {
			//FOUND EXISTING SEMAPHORE ON WEB
			return;
		    }
		} catch(...) { }
	    }
	}
    } catch(...) {
	m_bOnWeb = false;
    }
		
    //create a new one on the WEB
    try {
	CORBA::Object_var obj;
	obj = pCServer->FindObjectWithName(XMLRelayVarFactory::CLASS_NAME,"WebRelay");
			
	//narrowed to XML relay var factory
	XMLRelayVarFactory_var VFonWEB=XMLRelayVarFactory::_narrow(obj);
	if(!(CORBA::is_nil(VFonWEB))) {
	    m_bOnWeb=VFonWEB->CreateSimpleSemaphore((g_pTExecuter->GetRepositoryHost()).c_str(),m_strName.c_str());//Create CYRANO Test type semaphore
	    return;
	}
    } catch(...) {
	m_bOnWeb = false;
    }
}

CECSemaphore::~CECSemaphore() { }

bool
CECSemaphore::acquire() {
    if(m_bOnLan) {
	m_Semaphore->acquire();
	return true;
    } else if(m_bOnWeb) {
	try {
	    CORBA::Object_var obj2 = pCServer->FindObjectWithName(XMLRelayVarFactory::CLASS_NAME,"WebRelay");
	    XMLRelayVarFactory_var VFonWeb = XMLRelayVarFactory::_narrow(obj2);
		
	    if(VFonWeb->SimpleSemaphore_acquire((g_pTExecuter->GetRepositoryHost()).c_str(),m_strName.c_str())) {
		return true;
	    }
	} catch(...) { }
    }
    return false;
}

bool
CECSemaphore::release() {
    if(m_bOnLan) {
	m_Semaphore->release();
	return true;
    } else if(m_bOnWeb) {
	try {
	    CORBA::Object_var obj2 = pCServer->FindObjectWithName(XMLRelayVarFactory::CLASS_NAME,"WebRelay");
	    XMLRelayVarFactory_var VFonWeb = XMLRelayVarFactory::_narrow(obj2);
		
	    if(VFonWeb->SimpleSemaphore_release((g_pTExecuter->GetRepositoryHost()).c_str(),m_strName.c_str())) {
		return true;
	    }
	} catch(...) { }
    }
    return false;
}

bool
CECSemaphore::tryacquire() {
    bool bRet = false;
    if(m_bOnLan) {
	bRet = m_Semaphore->tryacquire();
    } else if(m_bOnWeb) {
	try {
	    CORBA::Object_var obj2 = pCServer->FindObjectWithName(XMLRelayVarFactory::CLASS_NAME,"WebRelay");
	    XMLRelayVarFactory_var VFonWeb = XMLRelayVarFactory::_narrow(obj2);
		
	    bRet = VFonWeb->SimpleSemaphore_tryacquire((g_pTExecuter->GetRepositoryHost()).c_str(),m_strName.c_str());
	} catch(...) {
	    bRet = false;
	}
    }
    return bRet;
}