/* Copyright (C) 2000-2001 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/
#include <InjectorControl.h>
#include <OnExecuterStop.hpp>
#include <mutex.hpp>


executerEventList gExecuterEventList ;
CTMutex	gEventListMutex ;
CTMutex	gEventStopConditionMutex ;
CTCondition eventStopCondition(gEventStopConditionMutex) ;

void onExecuterStop(InjectorControl_ptr pControl, const string & registeredName)
{
	executerEventList evList;
	CExecuterEvent ev ;
	TaskGroupStopReason tgsr = tgstop_stopped;
	bool bContinue = true ;

	gEventStopConditionMutex.lock();

	while (bContinue)
	{
		eventStopCondition.wait();
		{
			CTLockGuard<CTMutex> guard(gEventListMutex) ;
			if (gExecuterEventList.empty())
				continue ;
			evList = gExecuterEventList;
			gExecuterEventList.clear();
		}
		while ( !evList.empty() )
		{
			ev = evList.front() ;
			evList.pop_front() ;
			switch(ev.eExecuterEventType_)
			{
				case EEE_ONSTOPTASKGROUP	:	pControl->OnTaskGroupStopped(ev.lTaskGroupId_, tgsr);
												break ;

				case EEE_ONSTOPEXECUTER		:	pControl->OnExecuterStopping(registeredName.c_str());
												break ;

				case EEE_FINISHED			:	bContinue = false ;
												break ;
			}			
		}
	}
}
