/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

#ifndef _TESTEXECUTER_I_H_
#define _TESTEXECUTER_I_H_

#include <string>

#ifndef CYRANO_TP_STANDALONE
#   include <OSTAORB/OSTAORB.h>
#   include "TestExecuter.h"
#   include "StepByStep_TestExecuter_i.h"
#else
#	include <CECSACORBA.h>
#endif


class TestExecuterImpl
#ifndef CYRANO_TP_STANDALONE
    : public POA_HTTP_TestExecuter, public PortableServer::RefCountServantBase
{
public:
    TestExecuterImpl(CORBAServer* pserver, StepByStepScriptExecuterImpl* pSBS);
#else
{
public:
    TestExecuterImpl(CORBAServer* pserver);
#endif
    virtual ~TestExecuterImpl();

#ifndef CYRANO_TP_STANDALONE
    // TestExecuter methods:
    virtual CORBA::Boolean StartTaskGroup(TaskGroupExecInfo& tgExec);
    virtual CORBA::Boolean StopTaskGroup(CORBA::Long lTaskGroupId);
    virtual CORBA::Boolean Stop();
    virtual CORBA::Boolean GetStatsSnapshot(TestExecuterSnapshot& teSnapshot);

    // Script_TestExecuter methods:
	virtual CORBA::Boolean GetTaskSnapshot(const TaskIdList& taskIDs, TaskSnap2List& taskSnapshots);
    virtual CORBA::Long    GetTEVUs(CORBA::Long lPreviousTestVUCount);
	virtual void           GetTETotalAndActiveVUs(CORBA::Long lPreviousTestActiveVUCount, CORBA::Long& lActiveVUs, CORBA::Long lPreviousTestTotalVUCount, CORBA::Long& lTotalVUs);


    // HTTP_TestExecuter methods:
    virtual HTTP_TestExecuterSnapshot* HTTP_GetStatsSnapshot();

#endif

    // Helpers.
    void                   Deactivate();
#ifndef CYRANO_TP_STANDALONE
    InjectorControl_ptr    GetInjectorControl() const {return m_pControl;}
    StepByStepScriptExecuterImpl* GetStepByStep() {return m_pSBS;}
    const StepByStepScriptExecuterImpl* GetStepByStep() const {return m_pSBS;}
#endif
    const char*            GetRegisterName() const {return m_strRegisterName.c_str();}

private:

#ifndef CYRANO_TP_STANDALONE
    BOOL                      m_bOpen;      // TRUE if the Test Executer is open for incoming calls.
    PortableServer::ObjectId* m_pObjid;
	InjectorControl_ptr       m_pControl;
    StepByStepScriptExecuterImpl* m_pSBS;
#endif
    CORBAServer*              m_pServer;
    std::string               m_strRegisterName;

    //// Forbidden.
    TestExecuterImpl();
    TestExecuterImpl(const TestExecuterImpl&);
    operator=(const TestExecuterImpl&);
};

#endif  // _TESTEXECUTER_I_H_
