/* Copyright (C) 2000-2001 Cyrano SA 
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/
// HTTP_TestManagerImpl.cpp: implementation of the HTTP_TestManagerImpl class.
//
//////////////////////////////////////////////////////////////////////
#include "stdafx.h"
#include "HTTP_TestManagerImpl.h"
#include "TestManager_i.h"

const char* strNoValidHTTPExecuter = "NO_HTTP_EXECUTER_RUNNING";
//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

HTTP_TestManagerImpl::HTTP_TestManagerImpl(CORBAServer* pServer) : TestManagerImpl(pServer,FALSE)
{

	//register new http class
	m_pServer->ActivateObject(this);
    m_pServer->NameObject(this->POA_HTTP_TestManager::_this(),TestManager::CLASS_NAME,m_strReg.c_str());

	string strMsg = "HTTP_TestManager Interface registered with name: ";
	strMsg += m_strReg;
	LOG_MESSAGE(TRCCAT_STARTUP, TRCLEV_MEDIUM, strMsg);
}

HTTP_TestManagerImpl::~HTTP_TestManagerImpl()
{

}

HTTPTEsSnapshot* HTTP_TestManagerImpl::HTTP_GetStatsSnapshot()
{
	CTLockGuard<CTMutex> lock(m_InjListMutex);

	//it's new so must be initialised
	HTTPTEsSnapshot* pResult =new HTTPTEsSnapshot();
	pResult->length(0);	

	try
	{
	    for (int i = 0; i < (int)m_injectorsInUse.size(); i++)
		{	    
			InjectorHost inj = m_injectorsInUse[i];

			if(inj.bOnLocalLAN==TRUE)
			{
				//local LAN
				LOG_MESSAGE(TRCCAT_STARTUP, TRCLEV_MEDIUM, "HTTP_GetStatsSnapshot on the local LAN");
				if(inj.bNarrowedHTTP)
				{
					//its been narrowed to the HTTP type before so it can be done
					CORBA::Object_ptr isObj = m_pServer->FindObjectWithName(inj.strClassName.c_str(),inj.strRegName.c_str());
					if (!CORBA::is_nil(isObj))
					{
						HTTP_InjectorStatus_var HTTPpIS;
						HTTPpIS = HTTP_InjectorStatus::_narrow(isObj);
						if (!(CORBA::is_nil(HTTPpIS)))
						{
							/*
							HTTP_TestExecuterSnapshot_var answer;	//this is var for memory
							answer=HTTPpIS->HTTP_GetStatsSnapshot();
							if(!(CORBA::is_nil(answer)))
							{
								int curLen = pResult->length();
								pResult->length(curLen + 1);
								pResult->operator [] (curLen) = answer;
							}
							*/
						}
					}
				}
				else
				{
					LOG_MESSAGE(TRCCAT_STARTUP, TRCLEV_MEDIUM, "This IS cannot do HTTP stats on the local LAN");
				}
			}
			else if(inj.bOnWEB==TRUE)
			{
				//via the web
				LOG_MESSAGE(TRCCAT_STARTUP, TRCLEV_MEDIUM, "HTTP_GetStatsSnapshot: via the WEB");
				
				//this function isn't called so it doesn't need implementing?

			}
			else
			{
				//no IS on WEB or LAN
				LOG_MESSAGE(TRCCAT_STARTUP, TRCLEV_MEDIUM, "No connection to IS on local LAN or WEB");			
			}
		}
	}
	catch(...)
	{
		LOG_MESSAGE(TRCCAT_STARTUP, TRCLEV_NONE, "Exception: HTTP_GetStatsSnapshot");
	}
	
	return pResult;	
}


CORBA::Boolean HTTP_TestManagerImpl::HTTP_GetBothStatsSnapshots(HTTPTEsSnapshot_out HTTP_InjectorSnapshot, TESnapList_out InjectorSnapList)
{	

	HTTP_InjectorSnapshot = new HTTPTEsSnapshot();
	InjectorSnapList = new TESnapList();

	CTLockGuard<CTMutex> lock(m_InjListMutex);

	try
	{
	    for (int i = 0; i < (int)m_injectorsInUse.size(); i++)
		{	    
			InjectorHost inj = m_injectorsInUse[i];

			if(inj.bOnLocalLAN==TRUE)
			{
				LOG_MESSAGE(TRCCAT_STARTUP, TRCLEV_MEDIUM, "HTTP_GetBothStatsSnapshots: on local LAN");
				if(inj.bNarrowedHTTP)
				{
					//its been narrowed to the HTTP type before so it can be done
					CORBA::Object_ptr isObj = m_pServer->FindObjectWithName(inj.strClassName.c_str(),inj.strRegName.c_str());
					if (!CORBA::is_nil(isObj))
					{
						try
						{
							HTTP_InjectorStatus_var HTTPpIS;
							HTTPpIS = HTTP_InjectorStatus::_narrow(isObj);
							if (!(CORBA::is_nil(HTTPpIS)))
							{
								HTTP_TestExecuterSnapshot_var result;
								TESnapList_var result2;

								if(HTTPpIS->HTTP_GetBothStatsSnapshots(result,result2)==true)
								{
									//copy the answers

									//check HTTP executer name is not the special case "NO_HTTP_EXECUTER_RUNNING" 
									if(strcmp(strNoValidHTTPExecuter,result->strTestExecuter)!=0)
									{
										int currlength = HTTP_InjectorSnapshot->length();
										HTTP_InjectorSnapshot->length(currlength+1);
										HTTP_InjectorSnapshot->operator [] (currlength)= result;
									}

									int curLen3 = InjectorSnapList->length();
									InjectorSnapList->length(curLen3 + result2->length());
									for (int snap2 = 0; snap2 < (int)result2->length(); snap2++)
									{
										InjectorSnapList->operator [] (curLen3 + snap2) = result2->operator [] (snap2);
									}
								}
							}
						}
						catch(...)
						{
							LOG_MESSAGE(TRCCAT_STARTUP, TRCLEV_NONE, "Exception: HTTP_GetBothStatsSnapshots(): via LAN");
						}
					}
				}
				else
				{
					//dont use this if the result 2 is ok!

					LOG_MESSAGE(TRCCAT_STARTUP, TRCLEV_MEDIUM, "This IS cannot do HTTP stats, getting basic stats on LAN");
					try
					{
						//try for original stats

						TESnapList* temp3= new TESnapList();
						InjectorStatus_var PISOriginal = InjectorStatus::_narrow(inj.pIS);
						PISOriginal->GetStatsSnapshot(*temp3);
	
						int curLen3 = InjectorSnapList->length();
						InjectorSnapList->length(curLen3 + (temp3->length()));
		
						for (int snap2 = 0; snap2 < (int)temp3->length(); snap2++)
						{
							InjectorSnapList->operator [] (curLen3 + snap2) = temp3->operator [] (snap2);
						}
						delete temp3; //after deep copy
					}
					catch(...)
					{
						LOG_MESSAGE(TRCCAT_STARTUP, TRCLEV_NONE, "Exception: HTTP_GetBothStatsSnapshots(): Getting basic stats only via LAN");	
					}
				}
			}
			else if(inj.bOnWEB==TRUE)
			{
				LOG_MESSAGE(TRCCAT_STARTUP, TRCLEV_MEDIUM, "HTTP_GetBothStatsSnapshots: via the WEB");
				if (!(CORBA::is_nil(inj.pWebIS)))
				{
					try
					{
						HTTP_TestExecuterSnapshot result;
						TESnapList result2;

						if(inj.pWebIS->HTTP_GetBothStatsSnapshots(inj.strHostName.c_str(),result,result2)==true)
						{
							//copy the answers
							//check HTTP executer name is not the special case "NO_HTTP_EXECUTER_RUNNING" 
							if(strcmp(strNoValidHTTPExecuter,result.strTestExecuter)!=0)
							{
								int currlength = HTTP_InjectorSnapshot->length();
								HTTP_InjectorSnapshot->length(currlength+1);
								HTTP_InjectorSnapshot->operator [] (currlength)= result;
							}
		
							int curLen3 = InjectorSnapList->length();
							InjectorSnapList->length(curLen3 + result2.length());
							for (int snap2 = 0; snap2 < (int)result2.length(); snap2++)
							{
								InjectorSnapList->operator [] (curLen3 + snap2) = result2.operator [] (snap2);
							}
						}
					}
					catch(...)
					{
						LOG_MESSAGE(TRCCAT_STARTUP, TRCLEV_NONE, "Exception: HTTP_GetBothStatsSnapshots(): via the web");
					}
				}
				else
				{
					//the web relay has gone
					LOG_MESSAGE(TRCCAT_STARTUP, TRCLEV_MEDIUM, "No connection to XML IS Relay");	
				}
			}
			else
			{
				//no connection to IS on LAN or WEB
				LOG_MESSAGE(TRCCAT_STARTUP, TRCLEV_MEDIUM, "No connection to IS on local LAN or WEB");			
			}
		}
	}
	catch(...)
	{
		LOG_MESSAGE(TRCCAT_STARTUP, TRCLEV_NONE, "Exception: HTTP_GetBothStatsSnapshots()");
	}

	return true;
}

