/* Copyright (C) 2000-2001 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/
// TestManager.cpp : Defines the entry point for the console application.
//

#include "stdafx.h"
#include "ManagerMain.h"
#include "HTTP_TestManagerImpl.h"
#include "OstaTraceDef.h"
#include <OSTAORB\OSTAORB.h>
#include "TestManager_i.h"
#include "TestFile.h"
#include "RegFunctions.h"
#include "TaskGroupDefData.h"
#include "ScriptDefData.h"
#include "process.h"
#include <ctime.hpp>
#include "mutex.hpp"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

const int MINUTE = 60;
const int HOUR = 60*MINUTE;
const int DAY = 24*HOUR;
/////////////////////////////////////////////////////////////////////////////
// The one and only application object

CWinApp theApp;
using namespace std;

CStdioFile* m_pLogFile = NULL;
CORBAServer* m_pServer = NULL;
HTTP_TestManagerImpl* MANAGER = NULL;
CString m_strTest = "";
long	m_lSampleRate = -1;
int m_nTraceLevel;
extern CTSemaphore blocking_semaphore;
extern CTMutex gStopTestMutex ;
string gOpenSTAPath = "";

void usage();
void cleanup();
void run();
void ExecuteFromTestFile();
void InitLogging();
void CloseLogging(CString strRunFolder);
long ConvertToTime(CString strScheduledTime);

//
// Handlers for CORBA connection Exceptions.
// When number of exceptions = max number of TCP connections --> exit.
//


CORBA::Boolean comFailureExceptionHandler(void* cookie, CORBA::ULong n_retries, const CORBA::COMM_FAILURE& ex)
{
	LOG_MESSAGE(TRCCAT_STARTUP, TRCLEV_MEDIUM, "comFailureExceptionHandler.");
	return false;
}

CORBA::Boolean systemExceptionHandler(void* cookie, CORBA::ULong n_retries, const CORBA::SystemException& ex)
{
	LOG_MESSAGE(TRCCAT_STARTUP, TRCLEV_MEDIUM, "systemExceptionHandler.");
	return false;
}

CORBA::Boolean transientExceptionHandler(void* cookie, CORBA::ULong n_retries, const CORBA::TRANSIENT& ex)
{
	LOG_MESSAGE(TRCCAT_STARTUP, TRCLEV_MEDIUM, "transientExceptionHandler.");
	return false;
}


int _tmain(int argc, TCHAR* argv[], TCHAR* envp[])
{
#ifdef _DEBUG
_asm { int 3 };   // ### L.M. LM TO REMOVED!!
#endif
	int nRetCode = 0;

	// initialize MFC and print and error on failure
	if (!AfxWinInit(::GetModuleHandle(NULL), NULL, ::GetCommandLine(), 0))
	{
		cerr << _T("Fatal Error: MFC initialization failed") << endl;
		nRetCode = 1;
		return nRetCode;
	}

	// Read command line parameters. Only one parameter is required, the session
	// name which is used as the folder name for storing the results.
    for(int i=1;i<argc;i++)
    {
        if(strcmp(argv[i],"-T")==0)
        {
            if(i+1<argc)m_strTest=argv[++i];
        }
		else if(strcmp(argv[i],"-S")==0)
		{
			 if(i+1<argc)
			 {
				CString strRefreshInterval=argv[++i];
				m_lSampleRate=atol(strRefreshInterval);
				if((m_lSampleRate==0)&&(strRefreshInterval!="0"))
				{
					//atol error
					usage();
					return 1;
				}
			 }
		}
        else
        {
            usage();
            return 1;
        }
	}

	if((m_strTest == "")||(m_lSampleRate==-1))
	{
        usage();
        return 1;
	}

	char * pOpenSTA = getenv("OPENSTA_HOME");
	if (pOpenSTA != NULL)
	{
		gOpenSTAPath = pOpenSTA;
		if (pOpenSTA[strlen(pOpenSTA) - 1] != '\\')
			gOpenSTAPath += '\\' ;
	}
	// Command line parameters are valid so lets continue...
	InitLogging();
	string strMsg = "Starting Test ";
	strMsg += m_strTest.GetBuffer(0);
	LOG_MESSAGE(TRCCAT_STARTUP, TRCLEV_MEDIUM, strMsg);

	// Initialise CORBA
    try
    {
		LOG_MESSAGE(TRCCAT_STARTUP, TRCLEV_MEDIUM, "Initailizing CORBA Services.");
        m_pServer=new CORBAServer(argc,argv);
		omniORB::installTransientExceptionHandler(NULL, transientExceptionHandler);
		omniORB::installCommFailureExceptionHandler(NULL, comFailureExceptionHandler);
		omniORB::installSystemExceptionHandler(NULL, systemExceptionHandler);

		unsigned int uiTimeOut = 15 ;
		string IniFile = gOpenSTAPath + "Server\\Architecture.ini" ;
		uiTimeOut = GetPrivateProfileInt("TESTMANAGER", "ClientCallTimeOutPerio", 15, IniFile.c_str());

		char zBuf[10];
		sprintf(zBuf, "%d", uiTimeOut);
		LOG_MESSAGE(TRCCAT_STARTUP, TRCLEV_MEDIUM, "Setting call timeout to " + string(zBuf) + "...");
		omniORB::callTimeOutPeriod(omniORB::clientSide, uiTimeOut);
    }
    catch(...)
    {
        LOG_MESSAGE(TRCCAT_STARTUP, TRCLEV_NONE, "Caught Exception on startup, is Nameservice Running?");
        return 1;
    }

	CyrtoolsInitialize();
	// Run the main function which will block until the TestManager is ready to shut down
	run();

	// The run cycle has finished so clean up the memory usage and close down.
	cleanup();

	CyrtoolsUninitialize();

	return nRetCode;
}

//
// Show command line usage information
void usage()
{
	CString strUsage;
	strUsage.LoadString(IDS_USAGE);
	cout << (LPCTSTR)strUsage << endl;
}

//
// Make sure all CORBA interfaces have been de-registered, tidy memory usage and close the log file
void cleanup()
{
	CString strRunFolder = "";

	if (MANAGER)
	{
		strRunFolder = MANAGER->GetTestRunFolderName();
		LOG_MESSAGE(TRCCAT_SHUTDOWN, TRCLEV_MEDIUM, "Closing TestManager Interface.");
		MANAGER->CloseDown();
		delete MANAGER;
	}

	if (m_pServer)
	{
		LOG_MESSAGE(TRCCAT_SHUTDOWN, TRCLEV_MEDIUM, "Closing CORBA Services.");
		delete m_pServer;
	}
	CloseLogging(strRunFolder);
}

void LogVUsers()
{
	try
	{
		int	 nIntervalMonitoring;
		long lActiveVUs = 0 ;
		long lTotalVUs = 0 ;
		CyrStTestRun_ptr pTR = MANAGER->GetRepository()->m_pTR;

		nIntervalMonitoring = 10000;
		HKEY hKey;
		if(RegOpenKeyEx(HKEY_CURRENT_USER,"SOFTWARE\\OpenSTA\\TestPlugin\\Settings",0,KEY_READ,&hKey)==ERROR_SUCCESS)
		{
			DWORD dwInterval;
			DWORD dwDataSize = sizeof(DWORD);
			if(RegQueryValueEx(hKey,"Sample Interval",NULL,NULL,(LPBYTE)&dwInterval,&dwDataSize)== ERROR_SUCCESS)
			{
				nIntervalMonitoring = dwInterval*1000;
			}
			RegCloseKey(hKey);
		}
		while (true)
		{
			lActiveVUs = lTotalVUs = 0 ;
			MANAGER->RefreshTestVUs(lActiveVUs, lTotalVUs);
			CECVULogOutput vulog(0, lActiveVUs, lTotalVUs);
			pTR->AddTestOutput(CECVULogOutput::OUTPUT_TYPE, vulog.ToString().c_str());
			::Sleep(nIntervalMonitoring);
		}
	}
	catch(...)
	{
	}
}


//
// The main run function... This blocks using a semaphore which will be released by the TestManager
// CORBA interface when all stop conditions have been met. Any exceptions thrown on initializtion
// are fatal.
void run()
{
// Initialization
	try
	{
		MANAGER = new HTTP_TestManagerImpl(m_pServer);

		if (!MANAGER->Initialize(m_strTest.GetBuffer(0)))
		    LOG_MESSAGE(TRCCAT_STARTUP, TRCLEV_NONE, "Failed to initialize the TestManager Interface.");
	}
	catch(CyrException e)
	{
		string strMsg = e.what();
		strMsg += " in ";
		strMsg += e.where();
		LOG_MESSAGE(TRCCAT_STARTUP, TRCLEV_NONE, strMsg);

		// If this has been caught it is a fatal error!
		return;
	}

	// Start logging number of virtual users
    CTThread thr (makeFunctorA0 (LogVUsers));

	// Start existing task groups from tst file
	ExecuteFromTestFile();

	// Block until all stop conditions have been met
	blocking_semaphore.lock();
	CTLockGuard<CTMutex> guard(gStopTestMutex);
	// Stop logging the virtual users
	thr.kill();
}

void ExecuteFromTestFile()
{
	// CL : 04/06/2002
	// Bug fix : Trace Settings were not sent to TestExecuters
#ifdef _DEBUG
	_asm int 3h
#endif
	LOG_MESSAGE(TRCCAT_STARTUP, TRCLEV_MEDIUM, "Reading trace file.");

	std::map<std::string, std::string> mapTraceSettingsPerTargetName;
	CString strRepPath = GetRepositoryPath() + "\\TraceSettings.txt";

	// Check if file is here
	CFileFind fileFind;
	BOOL bFound = fileFind.FindFile(strRepPath);
	if ( bFound != FALSE )
	{
		CStdioFile fileTrace(strRepPath, CFile::modeRead);
		CString strLine;
		int iDotPos = -1;
		while ( fileTrace.ReadString(strLine) )
		{
			iDotPos = strLine.Find(",");
			if ( iDotPos == -1 )
				continue;
			std::string strKey = (LPCTSTR)(strLine.Left(iDotPos));
			std::string strValue = (LPCTSTR)(strLine.Right(strLine.GetLength() - iDotPos - 1));
			mapTraceSettingsPerTargetName.insert(std::map<std::string, std::string>::value_type(strKey, strValue));
		}
	}
	LOG_MESSAGE(TRCCAT_STARTUP, TRCLEV_MEDIUM, "End Reading trace file.");

	// Open the Test file
	CTestFile TF;
	CString strTestFile = GetRepositoryPath() + "Tests\\" + m_strTest + "\\" + m_strTest + ".tst";
	try
	{
		CFile f(strTestFile,CFile::modeRead);
		CArchive ar(&f,CArchive::load);

		TF.Serialize(ar);
		ar.Close();
		f.Close();
	}
	catch(CFileException *e)
	{
		e->Delete();
	}

	int nTaskGroup = TF.GetStepByStepTaskGroup();
	BOOL bSBSMode = (nTaskGroup != -1);
	BOOL bAdvancedSBSMode = TF.IsAdvancedStepByStepMode();

	// read task groups
	for (int g = 0; g<TF.GetNumTaskGroups(); g++)
	{
		LPBYTE pData;
		DWORD dwBufSize;
		TF.GetTaskGroupData(g,&pData,dwBufSize);
		CTaskGroupDefData TGData;
		TGData.LoadFromBuffer(pData);
		
		if(bSBSMode)
		{
			if((TGData.GetTaskGroupEnabled()&& (TF.GetTaskType(g,0) == CTestFile::enProfile)) || (g == nTaskGroup))
			{
				TaskGroupStartupInfo tgStartup;
				tgStartup.strHost = TF.GetTaskGroupHost(g);
				tgStartup.lStartTime = 0;
	
				tgStartup.bStopOnComplete = (TF.GetTaskType(g,0) != CTestFile::enProfile);
				
				TaskGroupExecInfo execInfo;
				execInfo.tgtType = (TF.GetTaskType(g,0) == CTestFile::enProfile) ? tgt_profile: tgt_script;
				execInfo.bIsDuration = FALSE;
				execInfo.lTaskGroupDuration = 0;
				execInfo.lTaskGroupIterations = TGData.GetTaskIterations();
				execInfo.lNumVUs = (bAdvancedSBSMode ? TGData.GetNumVUs() : 1);
				execInfo.lInterBatchDelay=0;   
				execInfo.lVUsPerBatch=(bAdvancedSBSMode ? TGData.GetNumVUs() : 1);
				execInfo.lBatchRampUpTime=0;
				
				execInfo.bUsePageTimers = TGData.GetUsePageTimers() != FALSE;
				execInfo.lNumVUsHttpRes = TGData.GetNumVUsHttpRes();
				execInfo.lNumVUsTimRes = TGData.GetNumVUsTimRes();

				execInfo.seqTasks.length(TF.GetNumTasks(g));
				for (int t =0; t<TF.GetNumTasks(g); t++)
				{
					LPBYTE pTaskData = NULL;
					DWORD dwTaskBufSize;
					TF.GetTaskData(g,t, &pTaskData, dwTaskBufSize);

					TaskExecInfo task;
					task.strScriptName = TF.GetTaskName(g,t);
					if (pTaskData != NULL)
					{
						CScriptDefData TaskData;
						TaskData.LoadFromBuffer(pTaskData);
		
						task.bIsDuration   = FALSE;
						task.lDuration     = 0;    
						task.lIterations   = TaskData.GetNumIterations();
						task.bIsFixedDelay = TaskData.GetUseFixedIterDelay() != FALSE;
						task.lFixedDelay   = TaskData.GetFixedIterDelay();
						task.lMinDelay     = TaskData.GetMinIterDelay();	
						task.lMaxDelay     = TaskData.GetMaxIterDelay();
					}
					else
					{
						task.bIsDuration = 0;
						task.lDuration = 0;    
						task.lIterations = 0;
						task.bIsFixedDelay = 0;
						task.lFixedDelay = 0;
						task.lMinDelay = 0;	
						task.lMaxDelay = 0;
					}

					execInfo.seqTasks[t] = task; 
				}

				execInfo.strTarget = TF.GetTaskTargetType(g,0);

				// CL : 04/06/2002
				// Bug fix : Trace Settings were not sent to TestExecuters
				std::map<std::string, std::string>::iterator itr;
				if ( (itr = mapTraceSettingsPerTargetName.find((LPCTSTR)(TF.GetTaskTargetType(g,0)))) != mapTraceSettingsPerTargetName.end() )
				{
					execInfo.lTraceLevel = atoi(itr->second.c_str());
				}
				else
				{
					execInfo.lTraceLevel = 10;
				}
				// CL

				execInfo.lTaskGroupId = g;
				tgStartup.execInfo = execInfo;

				MANAGER->StartTaskGroup(tgStartup);
			}
		}
		else
		{
			if (TGData.GetTaskGroupEnabled())
			{
				TaskGroupStartupInfo tgStartup;
				tgStartup.strHost = TF.GetTaskGroupHost(g);
				if (TGData.GetIsScheduled())
					tgStartup.lStartTime = ConvertToTime(TGData.GetScheduledTime());
				else if (TGData.GetIsDelayed())
				{
					int nDelay = TGData.GetStartDelay();
					CTTime tm;
					tm.now();
					tgStartup.lStartTime = tm.getSeconds() + nDelay;
				}
				else
					tgStartup.lStartTime = 0;

				if (TF.GetTaskType(g,0) == CTestFile::enProfile)
				{
					tgStartup.bStopOnComplete = FALSE;
				}
				else
				{
					tgStartup.bStopOnComplete = TGData.GetStopOnComplete() != FALSE;
				}

				TaskGroupExecInfo execInfo;
				if (TF.GetTaskType(g,0) == CTestFile::enProfile)
				{
					execInfo.tgtType = tgt_profile;
				}
				else
				{
					execInfo.tgtType = tgt_script;
				}
				execInfo.bIsDuration = TGData.GetbIsDuration() != FALSE;
				execInfo.lTaskGroupDuration = TGData.GetTaskDuration();
				execInfo.lTaskGroupIterations = TGData.GetTaskIterations();
				execInfo.lNumVUs = TGData.GetNumVUs();  
				if(TGData.GetUseAdvancedVUInfo()==TRUE)
				{
					execInfo.lInterBatchDelay=TGData.GetInterBatchDelay();    
					execInfo.lVUsPerBatch=TGData.GetVUsPerBatch();        
					execInfo.lBatchRampUpTime=TGData.GetBatchRampUpTime();
				}
				else
				{
					execInfo.lInterBatchDelay=0;   
					execInfo.lVUsPerBatch=TGData.GetNumVUs();
					execInfo.lBatchRampUpTime=0;
				}
				execInfo.bUsePageTimers = TGData.GetUsePageTimers() != FALSE;

				execInfo.lNumVUsHttpRes = TGData.GetNumVUsHttpRes();
				execInfo.lNumVUsTimRes = TGData.GetNumVUsTimRes();

				execInfo.seqTasks.length(TF.GetNumTasks(g));
				for (int t =0; t<TF.GetNumTasks(g); t++)
				{
					LPBYTE pTaskData = NULL;
					DWORD dwTaskBufSize;
					TF.GetTaskData(g,t, &pTaskData, dwTaskBufSize);

					TaskExecInfo task;
					task.strScriptName = TF.GetTaskName(g,t);
					if (pTaskData != NULL)
					{
						CScriptDefData TaskData;
						TaskData.LoadFromBuffer(pTaskData);
		
						task.bIsDuration   = TaskData.GetIsDuration() != FALSE;
						task.lDuration     = TaskData.GetDuration();    
						task.lIterations   = TaskData.GetNumIterations();
						task.bIsFixedDelay = TaskData.GetUseFixedIterDelay() != FALSE;
						task.lFixedDelay   = TaskData.GetFixedIterDelay();
						task.lMinDelay     = TaskData.GetMinIterDelay();	
						task.lMaxDelay     = TaskData.GetMaxIterDelay();
					}
					else
					{
						task.bIsDuration = 0;
						task.lDuration = 0;    
						task.lIterations = 0;
						task.bIsFixedDelay = 0;
						task.lFixedDelay = 0;
						task.lMinDelay = 0;	
						task.lMaxDelay = 0;
					}

					execInfo.seqTasks[t] = task; 
				}

				execInfo.strTarget = TF.GetTaskTargetType(g,0);

				// CL : 04/06/2002
				// Bug fix : Trace Settings were not sent to TestExecuters
				std::map<std::string, std::string>::iterator itr;
				if ( (itr = mapTraceSettingsPerTargetName.find((LPCTSTR)(TF.GetTaskTargetType(g,0)))) != mapTraceSettingsPerTargetName.end() )
				{
					execInfo.lTraceLevel = atoi(itr->second.c_str());
				}
				else
				{
					execInfo.lTraceLevel = 10;
				}
				// CL

				execInfo.lTaskGroupId = g;
				tgStartup.execInfo = execInfo;

				MANAGER->StartTaskGroup(tgStartup);
			}
		}
	}
}

void LOG_MESSAGE(const char* category, int nLevel, string strMsg)
{
	if (nLevel>m_nTraceLevel)
		return;

	CTTime time;
	time.now();

	CString msg;
	msg.Format("%s,%s,%d,%s\r\n", time.Format("%H:%M:%S %d-%m-%Y").c_str(),category, nLevel, strMsg.c_str());
	m_pLogFile->WriteString(msg);
	m_pLogFile->Flush();
}

void InitLogging()
{
	// Find Trace Level
	m_nTraceLevel = 10;

	try
	{
		CString strTraceFilePath = GetRepositoryPath();
		strTraceFilePath += "TraceSettings.txt";
		CStdioFile fSettings(strTraceFilePath, CFile::modeRead | CFile::shareDenyNone);
		CString strLine;
		while(fSettings.ReadString(strLine))
		{
			int nPos = strLine.Find(",");
			if (strLine.Left(nPos) == "Test Manager")
			{
				strLine.Delete(0,nPos+1);
				m_nTraceLevel = atoi(strLine);
				break;
			}
		}
	}
	catch(...)
	{
	}

	CString strName;
	strName.Format("%s\\Engines\\Temp\\TestManager_%d.log", gOpenSTAPath.c_str(),_getpid());
	m_pLogFile = new CStdioFile(strName,CFile::modeCreate | CFile::modeWrite | CFile::shareDenyWrite);
	LOG_MESSAGE(TRCCAT_STARTUP, TRCLEV_MEDIUM, "TestManager started.");

	CString strMsg;
	strMsg.Format("Trace Level = %d", m_nTraceLevel);
	LOG_MESSAGE(TRCCAT_STARTUP, TRCLEV_MEDIUM, strMsg.GetBuffer(0));

}

void CloseLogging(CString strRunFolder)
{
	CString strDestPath = GetRepositoryPath();
	strDestPath += "Tests\\" + m_strTest + "\\" + strRunFolder + "\\" + m_pLogFile->GetFileName();
	if (m_pLogFile)
	{
		LOG_MESSAGE(TRCCAT_SHUTDOWN, TRCLEV_MEDIUM, "Closing log file.");
		CString fullPath = m_pLogFile->GetFilePath();
		m_pLogFile->Close();
		delete m_pLogFile;

		if(!strRunFolder.IsEmpty() && ::CopyFile(fullPath, strDestPath, FALSE))
		{
			::DeleteFile(fullPath);
		}
	}
}

long ConvertToTime(CString strScheduledTime)
{
	//Parse string: format "+days hours:minutes:seconds"
	strScheduledTime.Remove('+');

	char* token;
	token = strtok( strScheduledTime.GetBuffer(0), " ");
	int nOffsetDays = atoi(token);

	token = strtok(NULL, ":");
	int nOffsetHours = atoi(token);

	token = strtok(NULL, ":");
	int nOffsetMins = atoi(token);

	token = strtok( NULL, "\0");
	int nOffsetSecs = atoi(token);

	// No delay so return start time as 0 (now!)
	if (nOffsetDays == 0 && nOffsetHours == 0 && nOffsetMins == 0 && nOffsetSecs == 0)
		return 0;

	// Get the time in seconds for today's date with no time
	CTTime tm;
	tm.now();
	long lTime = tm.getSeconds();

	string strTime = tm.Format("%H:%M:%S");
    char * buf = new char[strTime.size() + 1] ;
    strcpy(buf, strTime.c_str());
	token = strtok( buf, ":");
	lTime -= (atoi(token) * HOUR);
	token = strtok( NULL, ":");
	lTime -= (atoi(token) * MINUTE);
	token = strtok( NULL, "\0");
	lTime -= atoi(token);

	// Add the time offsets
	lTime += nOffsetDays * DAY;
	lTime += nOffsetHours * HOUR;
	lTime += nOffsetMins * MINUTE;
	lTime += nOffsetSecs;

	return lTime;
}
