/* Copyright (C) 2000-2001 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/
// TestManager_i.h: interface for the TestManager_i class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_TESTMANAGER_I_H__2AF23798_4853_4B21_8A99_E6342BC6FFE6__INCLUDED_)
#define AFX_TESTMANAGER_I_H__2AF23798_4853_4B21_8A99_E6342BC6FFE6__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include "TestManager.h"
#include "TestExecuter.h"
#include "ManagerMain.h"
#include "HTTPInjectorStatus.h" // new
#include <OSTAORB\OSTAORB.h>
#include "LocalRepository.h"
#include "mutex.hpp"
#include "..\..\XMLRelay\XMLVDK010Lib\XMLRelayInjectorControl.h"
#include "..\..\XMLRelay\XMLVDK010Lib\XMLRelayInjectorStatus.h"


class TestManagerImpl : public POA_TestManager, public PortableServer::RefCountServantBase  
{
public:
	XMLRelayInjectorControl_ptr ConnectToInjectorViaWeb(const char *strHost);


	TestManagerImpl(CORBAServer* pServer);
	TestManagerImpl(CORBAServer* pServer, BOOL registerServer);
	virtual ~TestManagerImpl();

	char * TestName();
	char * GetTestRunFolderName();
	CORBA::Boolean IsStopping();

	
    // Methods for starting and stopping a Task Group.
	CORBA::Boolean StartTaskGroup(TaskGroupStartupInfo& tgInfo);
    TestExecuter_ptr GetTaskGroupExecuter(long lTaskGroupId);
    CORBA::Boolean StopTaskGroup(long lTaskGroupId);

    // Methods for terminating a Test.
    CORBA::Boolean StopTest(CORBA::Boolean bForce, long lTimeout);

    CORBA::Boolean GetStatsSnapshot(TESnapList& TestSnapList);
    long    GetTestVUs();
	void	GetTotalAndActiveVUs(CORBA::Long& lActiveVUs, CORBA::Long& lTotalVUs) ;
    void    OnTaskGroupStopped(long lTaskGroupId, TaskGroupStopReason& eReason);
    void    OnInjectorStopped(const char* strInjectorHost);
	TGExecStatus GetTaskGroupStatus(long lTaskGroupId);
	CORBA::Boolean GetTaskSnapshot(const TaskIdList& taskIDs, TaskSnap2List& taskSnapshots);
	BOOL Initialize(string strTest);
	void CloseDown();

	struct TGExecInfo
	{
		TaskGroupStartupInfo tgStartup;
		TGExecStatus status;
		InjectorControl_ptr pIC;
		XMLRelayInjectorControl_ptr pWebIC;
	};

//	vector <TGExecInfo> m_execInfo;
	CMap <long,long,TGExecInfo,TGExecInfo> m_execInfo;
	string m_strReg;
	string m_strTestRunString;
	string m_strTestName;

	struct InjectorHost
	{
		BOOL bOnLocalLAN;
		BOOL bOnWEB;
		BOOL bComplete;
		string strHostName;
		InjectorStatus_ptr pIS;
		XMLRelayInjectorStatus_ptr pWebIS;
		BOOL bNarrowedHTTP;
		string strClassName;
		string strRegName;
	};
	vector<InjectorHost> m_injectorsInUse;

	InjectorControl_ptr ConnectToInjector(const char* strHost, bool bForce = false);
	int m_nDelayedCount;

	CLocalRepository* GetRepository() { return m_pRepos; };
    void RefreshTestVUs(long & lActiveVUs, long & lTotalVUs);

	// Stop On Complete
	BOOL m_bUseStopOnComplete;
	int m_nStopOnCompleteRemaining;

	CORBAServer* m_pServer;
	string m_strXMLreg;
	
protected:

	//allow access for derived classes
	CTMutex m_InjListMutex;


private:
	int m_nPID;
	CLocalRepository* m_pRepos;
	omni_thread* m_pScheduler;
    long m_lCachedTotalVUs;
    long m_lCachedActiveVUs;
	bool m_bStopping ;
};

#endif // !defined(AFX_TESTMANAGER_I_H__2AF23798_4853_4B21_8A99_E6342BC6FFE6__INCLUDED_)
