// TofOptimizer.cpp : Defines the entry point for the console application.

#include "stdafx.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// The one and only application object

CWinApp theApp;

using namespace std;

BOOL Optimize(CStdioFile& inFile, CStdioFile& outFile);
CString MergeConsecutiveStrings(LPCTSTR zString);
void BreakLine(LPCTSTR zLine, CStringList& strList);
int FindChar(const CString& str, TCHAR c);

int _tmain(int argc, TCHAR* argv[], TCHAR* envp[])
{
	int nRetCode = 0;

	// initialize MFC and print and error on failure
	if (!AfxWinInit(::GetModuleHandle(NULL), NULL, ::GetCommandLine(), 0))
	{
		// TODO: change error code to suit your needs
		cerr << _T("Fatal Error: MFC initialization failed") << endl;
		nRetCode = 1;
	}
	else
	{
        if(argc < 2)
        {
            cerr << "Usage: TofOptimizer <tof_file_path>" << endl;
		    nRetCode = 1;
        }
        else
        {
            try
            {
                CStdioFile inFile(argv[1], CFile::modeRead|CFile::typeText);
                CString strInFile = inFile.GetFilePath();
                int iLast;
                if((iLast = strInFile.ReverseFind('\\')) != -1)
                {
                    CString strDir(strInFile.Left(iLast));
                    char zOutFile[MAX_PATH];
                    if(GetTempFileName(strDir, "tof", 0, zOutFile))
                    {
                        CStdioFile outFile(zOutFile, CFile::modeWrite|CFile::modeCreate);
                        if(Optimize(inFile, outFile))
                        {
                            // Delete the original TOF file, and rename
                            // the output file:
                            inFile.Close();
#ifdef NDEBUG
                            CFile::Remove(strInFile);
#endif
                            outFile.Close();
#ifdef NDEBUG
                            CFile::Rename(zOutFile, strInFile);
#endif
                        }
                        else
                        {
                            // Delete the output file:
                            outFile.Close();
                            CFile::Remove(zOutFile);
                        }
                    }
                }
            }
            catch(CFileException* pE)
            {
                char zErrMess[2048];
                CString strMess;
                if(pE->GetErrorMessage(zErrMess, sizeof(zErrMess)/sizeof(zErrMess[0])))
                    cerr << zErrMess << endl;
                pE->Delete();
            }
        }
	}

	return nRetCode;
}

// Copies the original TOF file to the output file, optimizing the lines
// along the way.
BOOL Optimize(CStdioFile& inFile, CStdioFile& outFile)
{
    BOOL bRet = TRUE;
    CString strLine;

    CString strValue("value ");
    CString strHeader("header ");
    int iHeaderLen = strHeader.GetLength();
    int iSepLen = 0;
    CString strSeparator;
    CString strWith("with ");
    BOOL bAccumulate = FALSE;
    CString strAccum;
    CString strHdrs, strWithHdrs;

    while(inFile.ReadString(strLine))
    {
        BOOL bWrite = TRUE;

        // Look if this is a value line:
        if(strLine.Find(strValue) == 0)
        {
            // Merge all series of consecutive string literals.
            strLine = strValue + MergeConsecutiveStrings(strLine.Mid(strValue.GetLength())) + "\n";
        }
        else if(strLine.Find(strHeader) == 0)
        {
            // Get the special separator:
            if(strSeparator.IsEmpty())
            {
                strSeparator = strLine.Mid(iHeaderLen);
                iSepLen = strSeparator.GetLength();
                strWith += strSeparator;
            }

            // Get the next line, where the first series of headers are:
            CString strFirstHeaders;
            if(!inFile.ReadString(strFirstHeaders))
            {
                bRet = FALSE;
                break;
            }
            if(strFirstHeaders.Right(iSepLen) != strSeparator)
            {
                bRet = FALSE;
                break;
            }
            strHdrs = strFirstHeaders.Left(strFirstHeaders.GetLength() - iSepLen);

            strLine += "\n";

            bAccumulate = TRUE; // Start accumulating instead of writing...
            strAccum.Empty();
        }
        else if(strLine.Find(strWith) == 0)
        {
            strWithHdrs.Empty();
            CString strWithLine(strLine + "\n");

            // Read the next line, where the actual with values are:
            if(!inFile.ReadString(strLine))
            {
                bRet = FALSE;
                break;
            }
            if(strLine.Right(iSepLen) != strSeparator)
            {
                bRet = FALSE;
                break;
            }
            strLine = strLine.Left(strLine.GetLength() - iSepLen);

            // Break the line into individual string expressions:
            CStringList strListExpr;
            BreakLine(strLine, strListExpr);
            CString strWithClause;

            // Append each expression to the headers.
            // Each expression defines a header and its value. A header
            // of the same name in the "header" section is superceded by
            // the header in the "with" section.
            for(POSITION pos = strListExpr.GetHeadPosition(); pos != NULL; )
            {
                CString strHdr(strListExpr.GetNext(pos));
                BOOL bSupercede = FALSE;

                if(strHdr[0] == '\"')
                {
                    // This expression is a string literal.

                    // Get the header name:
                    int iColonPos = strHdr.Find(':');
                    if(iColonPos != -1)
                    {
                        CString strHdrName(strHdr.Mid(1, iColonPos));

                        // Look for the same header in the "header" section.
                        int iHdrPos = strHdrs.Find(strHdrName);
                        if(iHdrPos != -1)
                        {
                            // The header already exists in the "header" section.
                            // Replace its value.

                            // Look for the end of the value:
                            int iHdrEnd = strHdrs.Find("^J", iHdrPos + strHdrName.GetLength());
                            if(iHdrEnd == -1)
                            {
                                // Take all characters to the end:
                                iHdrEnd = strHdrs.GetLength();
                            }
                            else
                                iHdrEnd;

                            // Rebuild the "header" section:
                            strHdrs = strHdrs.Left(iHdrPos)
                                + strHdr.Mid(1, strHdr.GetLength() - 2) // Remove double quotes if necessary.
                                + strHdrs.Mid(iHdrEnd);
                            bSupercede = TRUE;
                        }
                    }
                }
                else
                {
                    // This expression is a variable name: keep it in the with clause.
                    if(!strWithClause.IsEmpty())
                        strWithClause += ',';
                    strWithClause += strHdr;
                    bSupercede = TRUE;
                }

                if(!bSupercede)
                    strWithHdrs += "+\"^J\"+" + strHdr;
            }

            // Merge all series of consecutive string literals.
            strHdrs = MergeConsecutiveStrings(strHdrs + strWithHdrs);

            // Prepare the whole line for writing:
            strLine = strHdrs + strSeparator + "\n";

            bAccumulate = FALSE;        // Stop accumulating.
            outFile.WriteString(strLine);

            strLine = strAccum;

            // If there is a with clause, then add it:
            if(!strWithClause.IsEmpty())
                strLine += strWithLine + strWithClause + strSeparator + "\n";
        }
        else if(!strLine.IsEmpty())
        {
            // Non-blank line.

            strLine += "\n";
            if(bAccumulate)
            {
                strAccum += strLine;
                bWrite = FALSE;         // Inhibit writing.
            }
        }
        else
        {
            // Blank line.

            if(bAccumulate)
            {
                bAccumulate = FALSE;    // Stop accumulating.
                strLine = strAccum;
            }
            else
                strLine = "\n";
        }

        if(bWrite)
            outFile.WriteString(strLine);
    }

    return bRet;
}

// This function merges all series of consecutive string literals in the input string.
// The input string is supposed to be a string expression written as a concatenation of terms
// (the concatenation operator is the + sign).
// For example, the following:
//      "toto" + tata + "tutu" + "titi" + tete
// must be transformed into:
//      "toto" + tata + "tutuiti" + tete
CString MergeConsecutiveStrings(LPCTSTR zString)
{
    CString strNewExpr;
    CString strCandidate(zString);

    strCandidate.TrimLeft(); strCandidate.TrimRight();
    BOOL bOK = TRUE;
    BOOL bInString = FALSE;
    while(!strCandidate.IsEmpty())
    {
        if(strCandidate[0] == '\"')
        {
            // The next term is a string literal.

            if(!bInString)
            {
                bInString = TRUE;
                strNewExpr += '\"'; // Opening double quote.
            }

            // Extract the string:
            int iPos = strCandidate.Find('\"', 1);
            if(iPos == -1)
            {
                bOK = FALSE;
                break;
            }

            // Take all characters except the opening and closing double quotes:
            strNewExpr += strCandidate.Mid(1, iPos - 1);

            // Update the candidate line:
            strCandidate = strCandidate.Mid(iPos + 1);
            strCandidate.TrimLeft();
        }
        else
        {
            // The next term is NOT a string literal.

            if(bInString)
            {
                bInString = FALSE;
                strNewExpr += "\"+";
            }

            int i = 0;

            if(strCandidate.Find("~LENGTH(") == 0
                || strCandidate.Find("~LEFTSTR(") == 0
                || strCandidate.Find("~RIGHTSTR(") == 0
                || strCandidate.Find("~LTRIM(") == 0
                || strCandidate.Find("~RTRIM(") == 0
                || strCandidate.Find("~LOCATE(") == 0
                || strCandidate.Find("~EXTRACT(") == 0)
            {
                // The next term is a built-in function.

                // Skip to the matching right parenthesis.
                // Watch for string literal argument values!
                i = FindChar(strCandidate, ')');
                if(i == -1)
                {
                    // Error: right parenthesis not found!
                    bOK = FALSE;
                    break;
                }

                // Take all characters up to i:
                strNewExpr += strCandidate.Left(i + 1);

                // Update the candidate line:
                strCandidate = strCandidate.Mid(i + 1);
                strCandidate.TrimLeft();
            }
            else
            {
                // The next term is a variable name.

                // Look for the next + sign:
                i = strCandidate.Find('+');
                if(i != -1)
                {
                    // Take all characters up to (but not including) i:
                    CString strTerm(strCandidate.Left(i));
                    strTerm.TrimRight();
                    strNewExpr += strTerm;

                    // Update the candidate line:
                    strCandidate = strCandidate.Mid(i);
                }
                else
                {
                    // This is the last term.
                    break;
                }
            }
        }

        // Look for a + sign:
        if(!strCandidate.IsEmpty())
        {
            if(strCandidate[0] == '+')
            {
                // Skip the + sign:
                strCandidate = strCandidate.Mid(1);
                strCandidate.TrimLeft();
                if(!bInString)
                    strNewExpr += '+';
            }
            else
            {
                // A + sign should be present before the next term! Error.
                bOK = FALSE;
                break;
            }
        }
    }

    if(bOK)
    {
        if(!strCandidate.IsEmpty())
        {
            if(!strNewExpr.IsEmpty() && strNewExpr.Right(1) != "+")
                strNewExpr += '+';
            strNewExpr += strCandidate;
        }
        else if(bInString)
            strNewExpr += '\"';
    }
    else
        strNewExpr = zString;

    return strNewExpr;
}

// Break a line into individual string expressions:
// Expressions are separated by commas.
// Watch for string literals.
void BreakLine(LPCTSTR zLine, CStringList& strList)
{
    strList.RemoveAll();
    CString strLine(zLine);
    int iPos;
    CString strElement;

    while(!strLine.IsEmpty())
    {
        iPos = FindChar(strLine, ',');
        if(iPos != -1)
        {
            // Extract the next expression:
            strElement = strLine.Left(iPos);
            strElement.TrimLeft();
            strList.AddTail(strElement);
            strLine = strLine.Mid(iPos + 1);    // Skip the comma.
        }
        else
        {
            // Take the remainder.
            strList.AddTail(strLine);
            strLine.Empty();
        }
    }
}

// Find the next character c in the string str, and return
// its position (zero based).
// Skip string literals (surrounded by double quotes) in the process.
// Returns -1 if not found.
int FindChar(const CString& str, TCHAR c)
{
    BOOL bInStrLit = FALSE;
    int iLen = str.GetLength();
    int i;
    for(i = 0; i < iLen; ++i)
    {
        TCHAR cNext = str[i];

        if(!bInStrLit)
        {
            // NOT in a string literal:

            if(cNext == c)
                break;  // Found!
            else if(cNext == '\"')
                bInStrLit = TRUE;
        }
        else
        {
            // Inside a string literal!

            if(cNext == '\"')
                bInStrLit = FALSE;
        }
    }

    if(i == iLen)
        i = -1;     // Not found!

    return i;
}
