/* Copyright (C) 2000-2001 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

// RelayDmn.cpp : Defines the entry point for the console application.
//
///////////////////////////////////////////////////////////////////////////////
#include "stdafx.h"

///////////////////////////////////////////////////////////////////////////////
// Archtecture Includes
#include "OSTAORB.h"
#include "CECInetInfo.h"
#include "..\include\Logger.h"
#include "OstaTraceDef.h"

///////////////////////////////////////////////////////////////////////////////
// Local Includes
#include "RelayInit.h"

///////////////////////////////////////////////////////////////////////////////
// Global variables
CRelayInit* relay = NULL;
CORBAServer* server = NULL;
CLogger* g_Logger = NULL;

///////////////////////////////////////////////////////////////////////////////
// Function declarations
void AbortDaemon( int );
void Shutdown();

int main(int argc, char* argv[])
{
	int nTraceLevel = 0;
    for(int i=1;i<argc;i++)
    {
        if(strcmp(argv[i],"-T")==0)             // Test name
        {
            if(i+1<argc)
			{
				nTraceLevel=atoi(argv[++i]);
			}
        }
	}

	g_Logger = new CLogger("RelayDmn.txt");
	g_Logger->SetLevel(nTraceLevel);
	g_Logger->WriteLogMsg(TRCLEV_NONE, "Relay Daemon Started.");

	char buf[4];
	sprintf(buf,"%d",nTraceLevel);
	g_Logger->WriteLogMsg(TRCLEV_NONE, "Trace Level = "+string(buf));


    CyrtoolsInitialize();

    // Tell application what to do in case daemon is aborted.
    signal( SIGINT, AbortDaemon );
    signal( SIGABRT, AbortDaemon );

    // Calculate name for this daemon.  This will be the host address.
    try
    {
        // Start ORB and POA
        server = new CORBAServer( argc, argv );

        relay = new CRelayInit( server, nTraceLevel );
        relay->Run();
    }
    catch( ... )
    {
        cout << "Exception: Initialization failed." << endl;
    }

    // Call shutdown procedure
    Shutdown();

    CyrtoolsUninitialize();

    return 0;
}

void AbortDaemon( int )
{
    Shutdown();
    CyrtoolsUninitialize();

    // Output message and quit application
	cout << "Relay daemon application aborted." << endl;
    exit( 0 );
}

void Shutdown()
{
    try
    {
        // Tidy memory
        if ( relay != NULL )
        {
            delete relay;
            relay = NULL;
        }

        // Shut down ORB and BOA
        if ( server != NULL )
        {
            delete server;
            server = NULL;
        }
    }
    catch(...)
    {}

}