/* Copyright (C) 2000-2001 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

#include "stdafx.h"
#include "..\CallProcessor\XMLCallProcessor.h"
#include "XMLCallProcessor_i.h"
#include <string>
#include <iostream>
#include <fstream>
using namespace std;
#include <ipmutex.hpp>
#include "xmlrpc.h"
#include "..\include\Logger.h"
#include "OstaTraceDef.h"

extern CCallManager* g_pCallManager;
extern CLogger* g_Logger;

XMLCallProcessorImpl::XMLCallProcessorImpl(CORBAServer* corba)
{
    // Get IP address (with dots replaced by underscores) of daemon host.
    m_registered_name="WebRelay";
    m_corba=corba;

	m_corba->ActivateObject(this);
	g_Logger->WriteLogMsg(TRCLEV_LOW, "XMLCallProcessorImpl: Registering XMLCallProcessorImpl interface");
    m_corba->NameObject( this->_this(),"XMLCallProcessor",m_registered_name.c_str());

}

XMLCallProcessorImpl::~XMLCallProcessorImpl()
{
	g_Logger->WriteLogMsg(TRCLEV_LOW, "XMLCallProcessorImpl: DeRegistering XMLCallProcessorImpl interface");
    m_corba->UnNameObject( "XMLCallProcessor",m_registered_name.c_str());
}

char* XMLCallProcessorImpl::ProcessCall(const char* strXMLCall)
{
//    static CTMutex mutex;
//    CTLockGuard<CTMutex> lock(mutex);

	string strRet = "";
	g_Logger->WriteLogMsg(TRCLEV_LOW, "XMLCallProcessorImpl: ProcessCall: \r\n"+string(strXMLCall));
	CXMLRequest* pRequest = NULL ;
	CXMLResponse* pResponse = NULL ;
	try
	{
		if (strstr(strXMLCall, "AddTestOutput") == NULL)
		{
			g_Logger->WriteLogMsg(TRCLEV_LOW, "XMLCallProcessorImpl: ProcessCall: Construct Request");
			pRequest = new CXMLRequest(strXMLCall);
			g_Logger->WriteLogMsg(TRCLEV_LOW, "XMLCallProcessorImpl: ProcessCall: GetCallHandler");
			CCallHandler* pCallHandler = g_pCallManager->GetHandlerForName(pRequest->m_strHandlerName);

			g_Logger->WriteLogMsg(TRCLEV_LOW, "XMLCallProcessorImpl: ProcessCall: ProcessMethodCall");
			pResponse = pCallHandler->ProcessMethodCall(pRequest);
		}
		else
		{
			g_Logger->WriteLogMsg(TRCLEV_LOW, "XMLCallProcessorImpl: ProcessCall: Construct Request");
			pRequest = new CXMLRequest(strXMLCall);
			g_Logger->WriteLogMsg(TRCLEV_LOW, "XMLCallProcessorImpl: ProcessCall: GetCallHandler");
			CCallHandler* pCallHandler = g_pCallManager->GetHandlerForName(pRequest->m_strHandlerName);

			g_Logger->WriteLogMsg(TRCLEV_LOW, "XMLCallProcessorImpl: ProcessCall: ProcessMethodCall");
			pResponse = pCallHandler->ProcessMethodCall(pRequest);
		}
		strRet = pResponse->ToXML();


		if (pRequest)
			delete pRequest;
		if (pResponse)
			delete pResponse;

	}
	catch(...)
	{
		if (pRequest)
			delete pRequest ;
		if (pResponse)
			delete pResponse;
		g_Logger->WriteLogMsg(TRCLEV_LOW, "XMLCallProcessorImpl: ProcessCall: EXCEPTION");
		CXMLResponse resp;
		resp.SetError(CXMLResponse::UNKNOWN_ERROR);
		strRet = resp.ToXML();
	}

	g_Logger->WriteLogMsg(TRCLEV_LOW, "XMLCallProcessorImpl: ProcessCall: Return reply : " + strRet);
	return CORBA::string_dup(strRet.c_str());
}
