/* Copyright (C) 2000-2001 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

// XML_IC_TaskGroupExecInfo.cpp: implementation of the CXML_IC_TaskGroupExecInfo class.
//
//////////////////////////////////////////////////////////////////////

#include "XML_IC_TaskGroupExecInfo.h"
#include "XML_IC_TaskExecInfo.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CXML_IC_TaskGroupExecInfo::CXML_IC_TaskGroupExecInfo()
{
	m_bSet = false;
}

CXML_IC_TaskGroupExecInfo::CXML_IC_TaskGroupExecInfo(TaskGroupExecInfo tgInfo)
{
	SetValue(tgInfo);
}

CXML_IC_TaskGroupExecInfo::~CXML_IC_TaskGroupExecInfo()
{

}

void CXML_IC_TaskGroupExecInfo::SetValue(TaskGroupExecInfo tgInfo)
{
	m_tgInfo = tgInfo;
	m_bSet = true;
}

const TaskGroupExecInfo & CXML_IC_TaskGroupExecInfo::GetValue()
{
	return m_tgInfo;
}

string CXML_IC_TaskGroupExecInfo::GetType()
{
	return "TaskGroupExecInfo";
}

bool CXML_IC_TaskGroupExecInfo::ToXML(string& strXML)
{
	if (!m_bSet)
		return false;


	strXML += "<value>";
	strXML += "<TaskGroupExecInfo>";
	strXML += AddMember("lTaskGroupId",&CXMLLong(m_tgInfo.lTaskGroupId));
	strXML += AddMember("strTarget",&CXMLString(string(m_tgInfo.strTarget)));
	strXML += AddMember("tgtType",&CXMLInt(m_tgInfo.tgtType));
	strXML += AddMember("bIsDuration",&CXMLBool(m_tgInfo.bIsDuration));
	strXML += AddMember("lTaskGroupDuration",&CXMLLong(m_tgInfo.lTaskGroupDuration));
	strXML += AddMember("lTaskGroupIterations",&CXMLLong(m_tgInfo.lTaskGroupIterations));
	strXML += AddMember("lNumVUs",&CXMLLong(m_tgInfo.lNumVUs));
	strXML += AddMember("lInterBatchDelay",&CXMLLong(m_tgInfo.lInterBatchDelay));
	strXML += AddMember("lVUsPerBatch",&CXMLLong(m_tgInfo.lVUsPerBatch));
	strXML += AddMember("lBatchRampUpTime",&CXMLLong(m_tgInfo.lBatchRampUpTime));
	strXML += AddMember("lNumVUsHttpRes",&CXMLInt(m_tgInfo.lNumVUsHttpRes));
	strXML += AddMember("lNumVUsTimRes",&CXMLInt(m_tgInfo.lNumVUsTimRes));
	strXML += AddMember("bUsePageTimers",&CXMLBool(m_tgInfo.bUsePageTimers));


	CXMLArray arrTasks;
	for (int i = 0; i<(int)m_tgInfo.seqTasks.length(); i++)
	{
		TaskExecInfo inf = m_tgInfo.seqTasks[i];
		CXML_IC_TaskExecInfo* task = new CXML_IC_TaskExecInfo(inf);
		arrTasks.AddElement(task);
	}

	strXML += AddMember("seqTasks",&arrTasks);

	strXML += "</TaskGroupExecInfo>";
	strXML += "</value>";

	return true;
}

bool CXML_IC_TaskGroupExecInfo::FromXML(CXMLNodeElement* pNode)
{
/* Kyle Harrison 11/8/01 : Modified to use new CXMLParser Wrapper Class */ 

    CXMLNodeElement* pMember = pNode->GetFirstChild();
	while(pMember)
	{
		CXMLNodeElement* pName = pMember->GetChildElementByName("name");
		string strName = pName->GetText();

		CXMLNodeElement* pValue = pMember->GetChildElementByName("value");
		CXMLNodeElement* pType = pValue->GetFirstChild();

		if (strName == "lTaskGroupId")
		{
			CXMLLong* pVal = new CXMLLong();
			pVal->FromXML(pType);
			m_tgInfo.lTaskGroupId=pVal->GetValue();
		}
		else if (strName == "strTarget")
		{
			CXMLString* pVal = new CXMLString();
			pVal->FromXML(pType);

			m_tgInfo.strTarget=pVal->GetValue().c_str();
		}
		else if (strName == "tgtType")
		{
			CXMLInt* pVal = new CXMLInt();
			pVal->FromXML(pType);
			m_tgInfo.tgtType=TargetType(pVal->GetValue());
		}
		else if (strName == "bIsDuration")
		{
			CXMLBool* pVal = new CXMLBool();
			pVal->FromXML(pType);
			m_tgInfo.bIsDuration=pVal->GetValue();
		}
		else if (strName == "lTaskGroupDuration")
		{
			CXMLLong* pVal = new CXMLLong();
			pVal->FromXML(pType);
			m_tgInfo.lTaskGroupDuration=pVal->GetValue();
		}
		else if (strName == "lTaskGroupIterations")
		{
			CXMLLong* pVal = new CXMLLong();
			pVal->FromXML(pType);
			m_tgInfo.lTaskGroupIterations=pVal->GetValue();
		}
		else if (strName == "lNumVUs")
		{
			CXMLLong* pVal = new CXMLLong();
			pVal->FromXML(pType);
			m_tgInfo.lNumVUs=pVal->GetValue();
		}
		else if (strName == "lInterBatchDelay")
		{
			CXMLLong* pVal = new CXMLLong();
			pVal->FromXML(pType);
			m_tgInfo.lInterBatchDelay=pVal->GetValue();
		}
		else if (strName == "lVUsPerBatch")
		{
			CXMLLong* pVal = new CXMLLong();
			pVal->FromXML(pType);
			m_tgInfo.lVUsPerBatch=pVal->GetValue();
		}
		else if (strName == "lBatchRampUpTime")
		{
			CXMLLong* pVal = new CXMLLong();
			pVal->FromXML(pType);
			m_tgInfo.lBatchRampUpTime=pVal->GetValue();
		}
		else if (strName == "lNumVUsHttpRes")
		{
			CXMLLong* pVal = new CXMLLong();
			pVal->FromXML(pType);
			m_tgInfo.lNumVUsHttpRes=pVal->GetValue();
		}
		else if (strName == "lNumVUsTimRes")
		{
			CXMLLong* pVal = new CXMLLong();
			pVal->FromXML(pType);
			m_tgInfo.lNumVUsTimRes=pVal->GetValue();
		}
		else if (strName == "bUsePageTimers")
		{
			CXMLBool* pVal = new CXMLBool();
			pVal->FromXML(pType);
			m_tgInfo.bUsePageTimers=pVal->GetValue();
		}
		else if (strName == "seqTasks")
		{
			CXMLArray* pVal = new CXMLArray();
			pVal->SetCallManager(m_pCallManager);
			pVal->FromXML(pType);
			m_tgInfo.seqTasks.length(pVal->GetSize());
			for (int i =0; i<pVal->GetSize(); i++)
			{
				CXML_IC_TaskExecInfo* pExec = (CXML_IC_TaskExecInfo*)pVal->GetAt(i);
				m_tgInfo.seqTasks[i] = pExec->GetValue();
			}
		}
		pMember = pMember->GetNextSibling();
	}

	return true;
}

string CXML_IC_TaskGroupExecInfo::AddMember(string name, CXMLParameter* param)
{
	string strXML = "";
	strXML += "<member>";
	strXML += "<name>"+name+"</name>";
	string strTmp;
	param->ToXML(strTmp);
	strXML += strTmp;
	strXML += "</member>";
	return strXML;
}
