/* Copyright (C) 2000-2001 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

// XML_IS_TaskSnap2.cpp: implementation of the CXML_IS_TaskSnap2 class.
//
//////////////////////////////////////////////////////////////////////

#include "XML_IS_TaskSnap2.h"
#include "XML_IS_TaskId.h"
#include "XML_IS_VUSnap.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CXML_IS_TaskSnap2::CXML_IS_TaskSnap2()
{
	m_bSet = false;
}

CXML_IS_TaskSnap2::CXML_IS_TaskSnap2(TaskSnap2 tsnap2)
{
	SetValue(tsnap2);
}

CXML_IS_TaskSnap2::~CXML_IS_TaskSnap2()
{

}

void CXML_IS_TaskSnap2::SetValue(TaskSnap2 tsnap2)
{
	m_taskSnap2 = tsnap2;
	m_bSet = true;
}

const TaskSnap2 & CXML_IS_TaskSnap2::GetValue()
{
	return m_taskSnap2;
}

string CXML_IS_TaskSnap2::GetType()
{
	return "TaskSnap2";
}

bool CXML_IS_TaskSnap2::ToXML(string& strXML)
{
	if (!m_bSet)
		return false;

	strXML += "<value>";
	strXML += "<TaskSnap2>";
	strXML += AddMember("task",&CXML_IS_TaskId(m_taskSnap2.task));
	CXMLArray arrTasks;
	for (int i = 0; i<(int)m_taskSnap2.VUSnaps.length(); i++)
	{
		VUSnap snap = m_taskSnap2.VUSnaps[i];
		CXML_IS_VUSnap* tSnap = new CXML_IS_VUSnap(snap);
		arrTasks.AddElement(tSnap);
	}
	strXML += AddMember("VUSnaps",&arrTasks);
	strXML += "</TaskSnap2>";
	strXML += "</value>";

	return true;
}

bool CXML_IS_TaskSnap2::FromXML(CXMLNodeElement* pNode)
{
/* Kyle Harrison 11/8/01 : Modified to use new CXMLParser Wrapper Class */ 

    CXMLNodeElement* pMember = pNode->GetFirstChild();
	while(pMember)
	{
		CXMLNodeElement* pName = pMember->GetChildElementByName("name");
		string strName = pName->GetText();

		CXMLNodeElement* pValue = pMember->GetChildElementByName("value");
		CXMLNodeElement* pType = NULL ;
		if (pValue)
			pType = pValue->GetFirstChild();

		if (strName == "task" && pType != NULL)
		{
			CXML_IS_TaskId* pVal = new CXML_IS_TaskId();
			pVal->FromXML(pType);
			m_taskSnap2.task=pVal->GetValue();
		}
		else if (strName == "VUSnaps" && pType != NULL)
		{
			CXMLArray* pVal = new CXMLArray();
			pVal->FromXML(pType);
			m_taskSnap2.VUSnaps.length(pVal->GetSize());
			for (int i =0; i<pVal->GetSize(); i++)
			{
				CXML_IS_VUSnap* pSnap = (CXML_IS_VUSnap*)pVal->GetAt(i);
				m_taskSnap2.VUSnaps[i] = pSnap->GetValue();
			}
		}
		pMember = pMember->GetNextSibling();
	}

	return true;
}

string CXML_IS_TaskSnap2::AddMember(string name, CXMLParameter* param)
{
	string strXML = "";
	strXML += "<member>";
	strXML += "<name>"+name+"</name>";
	string strTmp;
	param->ToXML(strTmp);
	strXML += strTmp;
	strXML += "</member>";
	return strXML;
}
