/* Copyright (C) 2000-2001 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

// InjectorControlHandler.cpp: implementation of the CInjectorControlHandler class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "InjectorControlHandler.h"
#include "InjectorControl.h"
#include "CECInetInfo.h"
#include "XML_IC_TaskGroupExecInfo.h"
#include "..\include\Logger.h"
#include "OstaTraceDef.h"
#include "XMLRelayTestRun_i.h"

extern CLogger* g_Logger;
extern XMLRelayTestRunImpl* TRRelay ;
//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CInjectorControlHandler::CInjectorControlHandler(CORBAServer* pServer)
{
	m_pCorba = pServer;
}

CInjectorControlHandler::~CInjectorControlHandler()
{

}

CXMLResponse* CInjectorControlHandler::ProcessMethodCall(CXMLRequest* request)
{
	g_Logger->WriteLogMsg(TRCLEV_MEDIUM, "IC: ProcessMethodCall "+request->m_strFullMethodName);

	if (request->m_strMethod == "Activate")
	{
		if (request->GetNumParams() == 5)
		{
			CXMLString* pParam1 = (CXMLString*)request->GetParam(0);
			CXMLString* pParam2 = (CXMLString*)request->GetParam(1);
			CXMLString* pParam3 = (CXMLString*)request->GetParam(2);
			CXMLString* pParam4 = (CXMLString*)request->GetParam(3);
			CXMLString* pParam5 = (CXMLString*)request->GetParam(4);

			return Activate(pParam1->GetValue(), pParam2->GetValue(), pParam3->GetValue(),
					 pParam4->GetValue(), pParam5->GetValue());
		}
		else
		{
			CXMLResponse* response = new CXMLResponse();
			if (request->GetNumParams() < 5)
				response->SetError(CXMLResponse::INSUFFICIENT_PARAMS);
			if (request->GetNumParams() > 5)
				response->SetError(CXMLResponse::TOO_MANY_PARAMS);
			return response;
		}

	}
	else if (request->m_strMethod == "IsActivated")
	{
		// A new test is started, init the Flush parameter for AddTestOutput
		TRRelay->SetFlush(false);
		if (request->GetNumParams() == 2)
		{
			CXMLString* pParam1 = (CXMLString*)request->GetParam(0);
			CXMLString* pParam2 = (CXMLString*)request->GetParam(1);

			return IsActivated(pParam1->GetValue(), pParam2->GetValue());
		}
		else
		{
			CXMLResponse* response = new CXMLResponse();
			if (request->GetNumParams() < 2)
				response->SetError(CXMLResponse::INSUFFICIENT_PARAMS);
			if (request->GetNumParams() > 2)
				response->SetError(CXMLResponse::TOO_MANY_PARAMS);
			return response;
		}
	}
	else if (request->m_strMethod == "StartTaskGroup")
	{
		TRRelay->SetFlush(false);
		if (request->GetNumParams() == 2)
		{
			CXMLString* pParam1 = (CXMLString*)request->GetParam(0);
			CXML_IC_TaskGroupExecInfo* pParam2 = (CXML_IC_TaskGroupExecInfo*)request->GetParam(1);
			return StartTaskGroup(pParam1->GetValue(), pParam2->GetValue());
		}
		else
		{
			CXMLResponse* response = new CXMLResponse();
			if (request->GetNumParams() < 2)
				response->SetError(CXMLResponse::INSUFFICIENT_PARAMS);
			if (request->GetNumParams() > 2)
				response->SetError(CXMLResponse::TOO_MANY_PARAMS);
			return response;
		}
	}
	else if (request->m_strMethod == "StopTaskGroup")
	{
		// FLush messages on Stop Task group
		TRRelay->flush();
		if (request->GetNumParams() == 2)
		{
			CXMLString* pParam1 = (CXMLString*)request->GetParam(0);
			CXMLLong* pParam2 = (CXMLLong*)request->GetParam(1);
			return StopTaskGroup(pParam1->GetValue(), pParam2->GetValue());
		}
		else
		{
			CXMLResponse* response = new CXMLResponse();
			if (request->GetNumParams() < 2)
				response->SetError(CXMLResponse::INSUFFICIENT_PARAMS);
			if (request->GetNumParams() > 2)
				response->SetError(CXMLResponse::TOO_MANY_PARAMS);
			return response;
		}
	}
	else if (request->m_strMethod == "StopTest")
	{
		TRRelay->flush();
		if (request->GetNumParams() == 3)
		{
			CXMLString* pParam1 = (CXMLString*)request->GetParam(0);
			CXMLBool* pParam2 = (CXMLBool*)request->GetParam(1);
			CXMLLong* pParam3 = (CXMLLong*)request->GetParam(2);
			return StopTest(pParam1->GetValue(), pParam2->GetValue(), pParam3->GetValue());
		}
		else
		{
			CXMLResponse* response = new CXMLResponse();
			if (request->GetNumParams() < 3)
				response->SetError(CXMLResponse::INSUFFICIENT_PARAMS);
			if (request->GetNumParams() > 3)
				response->SetError(CXMLResponse::TOO_MANY_PARAMS);
			return response;
		}
	}
	else if (request->m_strMethod == "TransferResults")
	{
		if (request->GetNumParams() == 4)
		{
			CXMLString* pParam1 = (CXMLString*)request->GetParam(0);
			CXMLString* pParam2 = (CXMLString*)request->GetParam(1);
			CXMLString* pParam3 = (CXMLString*)request->GetParam(2);
			CXMLString* pParam4 = (CXMLString*)request->GetParam(3);
			return TransferResults(pParam1->GetValue(), pParam2->GetValue(), pParam3->GetValue(), pParam4->GetValue());
		}
		else
		{
			CXMLResponse* response = new CXMLResponse();
			if (request->GetNumParams() < 4)
				response->SetError(CXMLResponse::INSUFFICIENT_PARAMS);
			if (request->GetNumParams() > 4)
				response->SetError(CXMLResponse::TOO_MANY_PARAMS);
			return response;
		}
	}

	g_Logger->WriteLogMsg(TRCLEV_LOW, "IC: ProcessMethodCall: Error: Method Not Found");
	CXMLResponse* response = new CXMLResponse();
	response->SetError(CXMLResponse::FUNC_NOT_FOUND);
	return response;
}

CXMLResponse* CInjectorControlHandler::Activate(string strTargetMachine, string strManagerRegString,
												string strReposHost, string strTestRunString, string strTestName)
{
	g_Logger->WriteLogMsg(TRCLEV_MEDIUM, "IC: Activate");
	try
	{
		// Narrow the time service corba object on the target machine
	    CECInetInfo inet;
	    string strIP = inet.GetIPAddressAsString(strTargetMachine);
		g_Logger->WriteLogMsg(TRCLEV_HIGH, "IC: Activate: Narrowing InjectorControl on "+strIP);
		CORBA::Object_var obj = m_pCorba->FindObjectWithName(InjectorControl::CLASS_NAME,strIP.c_str());
		InjectorControl_var IC = InjectorControl::_narrow(obj);
		g_Logger->WriteLogMsg(TRCLEV_HIGH, "IC: Activate: Calling Activate");
		bool bActivate = IC->Activate(strManagerRegString.c_str(), strReposHost.c_str(),
									  strTestRunString.c_str(), strTestName.c_str());

		CXMLResponse* response = new CXMLResponse();
		response->m_params.Add(new CXMLBool(bActivate));
		return response;
	}
	catch(...)
	{
		g_Logger->WriteLogMsg(TRCLEV_LOW, "IC: Activate: Exception calling Activate on InjectorControl");
	}


	CXMLResponse* response = new CXMLResponse();
	response->SetError(CXMLResponse::UNKNOWN_ERROR);
	return response;
}

CXMLResponse* CInjectorControlHandler::IsActivated(string strTargetMachine, string strManagerRegString)
{
	g_Logger->WriteLogMsg(TRCLEV_MEDIUM, "IC: IsActivated");
	g_Logger->WriteLogMsg(TRCLEV_HIGH, "IC: IsActivated: TargetMachine = "+strTargetMachine);
	g_Logger->WriteLogMsg(TRCLEV_HIGH, "IC: IsActivated: strManagerRegString = "+strManagerRegString);
	try
	{
		// Narrow the time service corba object on the target machine
	    CECInetInfo inet;
	    string strIP = inet.GetIPAddressAsString(strTargetMachine);
		CORBA::Object_var obj = m_pCorba->FindObjectWithName(InjectorControl::CLASS_NAME,strIP.c_str());

		g_Logger->WriteLogMsg(TRCLEV_HIGH, "IC: IsActivated: Narrow IC");
		InjectorControl_var IC = InjectorControl::_narrow(obj);
		g_Logger->WriteLogMsg(TRCLEV_HIGH, "IC: IsActivated: Call IsActivated");
		bool bActivated = IC->IsActivated(strManagerRegString.c_str());
		if (bActivated)
		{
			g_Logger->WriteLogMsg(TRCLEV_HIGH, "IC: IsActivated: IsActivated = TRUE");
		}
		else
		{
			g_Logger->WriteLogMsg(TRCLEV_HIGH, "IC: IsActivated: IsActivated = FALSE");
		}

		CXMLResponse* response = new CXMLResponse();
		response->m_params.Add(new CXMLBool(bActivated));
		return response;
	}
	catch(...)
	{
		g_Logger->WriteLogMsg(TRCLEV_LOW, "IC: IsActivated: Exception calling IsActivated on InjectorControl");
	}


	CXMLResponse* response = new CXMLResponse();
	response->SetError(CXMLResponse::UNKNOWN_ERROR);
	return response;
}

CXMLResponse* CInjectorControlHandler::StartTaskGroup(string strTargetMachine, TaskGroupExecInfo tgExec)
{
	g_Logger->WriteLogMsg(TRCLEV_MEDIUM, "IC: StartTaskGroup");
	try
	{
		// Narrow the time service corba object on the target machine
	    CECInetInfo inet;
	    string strIP = inet.GetIPAddressAsString(strTargetMachine);
		g_Logger->WriteLogMsg(TRCLEV_HIGH, "IC: StartTaskGroup: Narrow IC");
		CORBA::Object_var obj = m_pCorba->FindObjectWithName(InjectorControl::CLASS_NAME,strIP.c_str());
		InjectorControl_var IC = InjectorControl::_narrow(obj);

		g_Logger->WriteLogMsg(TRCLEV_HIGH, "IC: StartTaskGroup: Call StartTaskGroup");
		bool bStarted = IC->StartTaskGroup(tgExec);

		CXMLResponse* response = new CXMLResponse();
		response->m_params.Add(new CXMLBool(bStarted));
		return response;
	}
	catch(...)
	{
		g_Logger->WriteLogMsg(TRCLEV_LOW, "IC: StartTaskGroup: Exception calling StartTaskGroup on InjectorControl");
	}

	CXMLResponse* response = new CXMLResponse();
	response->SetError(CXMLResponse::UNKNOWN_ERROR);
	return response;
}

CXMLResponse* CInjectorControlHandler::StopTaskGroup(string strTargetMachine, long lTaskGroupId)
{
	g_Logger->WriteLogMsg(TRCLEV_MEDIUM, "IC: StopTaskGroup");
	try
	{
		// Narrow the time service corba object on the target machine
	    CECInetInfo inet;
	    string strIP = inet.GetIPAddressAsString(strTargetMachine);
		g_Logger->WriteLogMsg(TRCLEV_HIGH, "IC: StopTaskGroup: Narrow IC");
		CORBA::Object_var obj = m_pCorba->FindObjectWithName(InjectorControl::CLASS_NAME,strIP.c_str());
		InjectorControl_var IC = InjectorControl::_narrow(obj);
		g_Logger->WriteLogMsg(TRCLEV_HIGH, "IC: StopTaskGroup: Call StopTaskGroup");
		bool bStopped = IC->StopTaskGroup(lTaskGroupId);

		CXMLResponse* response = new CXMLResponse();
		response->m_params.Add(new CXMLBool(bStopped));
		return response;
	}
	catch(...)
	{
		g_Logger->WriteLogMsg(TRCLEV_LOW, "IC: StopTaskGroup: Exception calling StopTaskGroup on InjectorControl");
	}


	CXMLResponse* response = new CXMLResponse();
	response->SetError(CXMLResponse::UNKNOWN_ERROR);
	return response;

}

CXMLResponse* CInjectorControlHandler::StopTest(string strTargetMachine, bool bForce, long lTimeout)
{
	g_Logger->WriteLogMsg(TRCLEV_MEDIUM, "IC: StopTest");
	try
	{
		// Narrow the time service corba object on the target machine
	    CECInetInfo inet;
	    string strIP = inet.GetIPAddressAsString(strTargetMachine);
		g_Logger->WriteLogMsg(TRCLEV_HIGH, "IC: StopTest: Narrow IC");
		CORBA::Object_var obj = m_pCorba->FindObjectWithName(InjectorControl::CLASS_NAME,strIP.c_str());
		InjectorControl_var IC = InjectorControl::_narrow(obj);
		g_Logger->WriteLogMsg(TRCLEV_HIGH, "IC: StopTest: Call StopTest");
		bool bStopped = IC->StopTest(bForce, lTimeout);

		CXMLResponse* response = new CXMLResponse();
		response->m_params.Add(new CXMLBool(bStopped));
		return response;
	}
	catch(...)
	{
		g_Logger->WriteLogMsg(TRCLEV_LOW, "IC: StopTest: Exception calling StopTest on InjectorControl");
	}


	CXMLResponse* response = new CXMLResponse();
	response->SetError(CXMLResponse::UNKNOWN_ERROR);
	return response;
}


CXMLResponse* CInjectorControlHandler::TransferResults(string strTargetMachine, string strTestName, string strReposHost,
									string strRunFolder)
{
	g_Logger->WriteLogMsg(TRCLEV_MEDIUM, "IC: TransferResults");
	try
	{
		// Narrow the time service corba object on the target machine
	    CECInetInfo inet;
	    string strIP = inet.GetIPAddressAsString(strTargetMachine);
		g_Logger->WriteLogMsg(TRCLEV_HIGH, "IC: TransferResults: Narrow IC");
		CORBA::Object_var obj = m_pCorba->FindObjectWithName(InjectorControl::CLASS_NAME,strIP.c_str());
		InjectorControl_var IC = InjectorControl::_narrow(obj);
		g_Logger->WriteLogMsg(TRCLEV_HIGH, "IC: TransferResults: Call TransferResults");
		long nTransfered = IC->TransferResults(strTestName.c_str(), strReposHost.c_str(), strRunFolder.c_str());

		CXMLResponse* response = new CXMLResponse();
		response->m_params.Add(new CXMLLong(nTransfered));
		return response;
	}
	catch(...)
	{
		g_Logger->WriteLogMsg(TRCLEV_LOW, "IC: TransferResults: Exception calling TransferResults on InjectorControl");
	}


	CXMLResponse* response = new CXMLResponse();
	response->SetError(CXMLResponse::UNKNOWN_ERROR);
	return response;

}
