/* Copyright (C) 2000-2001 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

// TimeServiceHandler.cpp: implementation of the CTimeServiceHandler class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "TestManagerHandler.h"
#include "CECInetInfo.h"
#include "TestManager.h"
#include "..\include\Logger.h"
#include "OstaTraceDef.h"

extern CLogger* g_Logger;
//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CTestManagerHandler::CTestManagerHandler(CORBAServer* pServer)
{
	m_pCorba = pServer;
}

CTestManagerHandler::~CTestManagerHandler()
{

}

CXMLResponse* CTestManagerHandler::ProcessMethodCall(CXMLRequest* request)
{
	g_Logger->WriteLogMsg(TRCLEV_MEDIUM, "TM: ProcessMethodCall "+request->m_strFullMethodName);
	if (request->m_strMethod == "OnTaskGroupStopped")
	{
		if (request->GetNumParams() == 3)
		{
			CXMLString* pParam1 = (CXMLString*)request->GetParam(0);
			CXMLLong* pParam2 = (CXMLLong*)request->GetParam(1);
			CXMLInt* pParam3 = (CXMLInt*)request->GetParam(2);
			return OnTaskGroupStopped(pParam1->GetValue(), pParam2->GetValue(),
									  (TaskGroupStopReason)pParam3->GetValue());
		}
		else
		{
			CXMLResponse* response = new CXMLResponse();
			if (request->GetNumParams() < 3)
				response->SetError(CXMLResponse::INSUFFICIENT_PARAMS);
			if (request->GetNumParams() > 3)
				response->SetError(CXMLResponse::TOO_MANY_PARAMS);
			return response;
		}

	}
	else if (request->m_strMethod == "OnInjectorStopped")
	{
		if (request->GetNumParams() == 2)
		{
			CXMLString* pParam1 = (CXMLString*)request->GetParam(0);
			CXMLString* pParam2 = (CXMLString*)request->GetParam(1);
			return OnInjectorStopped(pParam1->GetValue(), pParam2->GetValue());
		}
		else
		{
			CXMLResponse* response = new CXMLResponse();
			if (request->GetNumParams() < 2)
				response->SetError(CXMLResponse::INSUFFICIENT_PARAMS);
			if (request->GetNumParams() > 2)
				response->SetError(CXMLResponse::TOO_MANY_PARAMS);
			return response;
		}

	}
	else if (request->m_strMethod == "GetTestVUs")
	{
		if (request->GetNumParams() == 1)
		{
			CXMLString* pParam1 = (CXMLString*)request->GetParam(0);
			return GetTestVUs(pParam1->GetValue());
		}
		else
		{
			CXMLResponse* response = new CXMLResponse();
			if (request->GetNumParams() < 1)
				response->SetError(CXMLResponse::INSUFFICIENT_PARAMS);
			if (request->GetNumParams() > 1)
				response->SetError(CXMLResponse::TOO_MANY_PARAMS);
			return response;
		}
	}
	else if (request->m_strMethod == "GetTestTotalAndActiveVUs")
	{
		if (request->GetNumParams() == 3)
		{
			CXMLString* pParam1 = (CXMLString*)request->GetParam(0);
			CXMLLong * pParam2 = (CXMLLong *)request->GetParam(1);
			CXMLLong * pParam3 = (CXMLLong *)request->GetParam(2);
			return GetTestTotalAndActiveVUs(pParam1->GetValue(), pParam2->GetValue(), pParam3->GetValue());
		}
		else
		{
			CXMLResponse* response = new CXMLResponse();
			if (request->GetNumParams() < 3)
				response->SetError(CXMLResponse::INSUFFICIENT_PARAMS);
			if (request->GetNumParams() > 3)
				response->SetError(CXMLResponse::TOO_MANY_PARAMS);
			return response;
		}
	}

	g_Logger->WriteLogMsg(TRCLEV_LOW, "TM: ProcessMethodCall: Error: Method Not Found");
	CXMLResponse* response = new CXMLResponse();
	response->SetError(CXMLResponse::FUNC_NOT_FOUND);
	return response;
}

CXMLResponse* CTestManagerHandler::OnTaskGroupStopped(string strManagerReg, 
		long lTaskGroupId, TaskGroupStopReason eReason)
{
	g_Logger->WriteLogMsg(TRCLEV_MEDIUM, "TM: OnTaskGroupStopped");
	try
	{
		// Narrow the time service corba object on the target machine
		g_Logger->WriteLogMsg(TRCLEV_HIGH, "TM: OnTaskGroupStopped: Narrowing TestManager "+strManagerReg);
		CORBA::Object_var obj = m_pCorba->FindObjectWithName(TestManager::CLASS_NAME,strManagerReg.c_str());
		TestManager_var TM = TestManager::_narrow(obj);
		g_Logger->WriteLogMsg(TRCLEV_HIGH, "TM: OnTaskGroupStopped: Calling OnTaskGroupStopped");
		TM->OnTaskGroupStopped(lTaskGroupId, eReason);

		CXMLResponse* response = new CXMLResponse();
		return response;
	}
	catch(...)
	{
		g_Logger->WriteLogMsg(TRCLEV_LOW, "TM: OnTaskGroupStopped: Exception calling OnTaskGroupStopped on TestManager");
	}


	CXMLResponse* response = new CXMLResponse();
	response->SetError(CXMLResponse::UNKNOWN_ERROR);
	return response;

}

CXMLResponse* CTestManagerHandler::OnInjectorStopped(string strManagerReg,
		string strInjectorHost)
{
	g_Logger->WriteLogMsg(TRCLEV_MEDIUM, "TM: OnInjectorStopped");
	try
	{
		// Narrow the TestManager service corba object on the target machine
		g_Logger->WriteLogMsg(TRCLEV_HIGH, "TM: OnInjectorStopped: Narrowing TestManager "+strManagerReg);
		CORBA::Object_var obj = m_pCorba->FindObjectWithName(TestManager::CLASS_NAME,strManagerReg.c_str());
		TestManager_var TM = TestManager::_narrow(obj);
		g_Logger->WriteLogMsg(TRCLEV_HIGH, "TM: OnInjectorStopped: Calling OnInjectorStopped");
		TM->OnInjectorStopped(strInjectorHost.c_str());

		CXMLResponse* response = new CXMLResponse();
		return response;
	}
	catch(...)
	{
		g_Logger->WriteLogMsg(TRCLEV_LOW, "TM: OnInjectorStopped: Exception calling OnInjectorStopped on TestManager");
	}


	CXMLResponse* response = new CXMLResponse();
	response->SetError(CXMLResponse::UNKNOWN_ERROR);
	return response;
}

CXMLResponse* CTestManagerHandler::GetTestVUs(string strManagerReg)
{
	g_Logger->WriteLogMsg(TRCLEV_MEDIUM, "TM: GetTestVUs");
	try
	{
		// Narrow the time service corba object on the target machine
		g_Logger->WriteLogMsg(TRCLEV_HIGH, "TM: GetTestVUs: Narrowing TestManager "+strManagerReg);
		CORBA::Object_var obj = m_pCorba->FindObjectWithName(TestManager::CLASS_NAME,strManagerReg.c_str());
		TestManager_var TM = TestManager::_narrow(obj);
		g_Logger->WriteLogMsg(TRCLEV_HIGH, "TM: GetTestVUs: Calling GetTestVUs");
		long lTestVUs = TM->GetTestVUs();

		CXMLResponse* response = new CXMLResponse();
		response->m_params.Add(new CXMLLong(lTestVUs));
		return response;
	}
	catch(...)
	{
		g_Logger->WriteLogMsg(TRCLEV_LOW, "TM: GetTestVUs: Exception calling GetTestVUs on TestManager");
	}


	CXMLResponse* response = new CXMLResponse();
	response->SetError(CXMLResponse::UNKNOWN_ERROR);
	return response;
}

CXMLResponse* CTestManagerHandler::GetTestTotalAndActiveVUs(string strManagerReg, long lActiveVUs, long lTotalVUs)
{
	g_Logger->WriteLogMsg(TRCLEV_MEDIUM, "TM: GetTestVUs");
	try
	{
		// Narrow the time service corba object on the target machine
		g_Logger->WriteLogMsg(TRCLEV_HIGH, "TM: GetTestVUs: Narrowing TestManager "+strManagerReg);
		CORBA::Object_var obj = m_pCorba->FindObjectWithName(TestManager::CLASS_NAME,strManagerReg.c_str());
		TestManager_var TM = TestManager::_narrow(obj);
		g_Logger->WriteLogMsg(TRCLEV_HIGH, "TM: GetTestVUs: Calling GetTestVUs");
		long lActiveVus = 0;
		long lTotalVUs = 0 ;
		TM->GetTotalAndActiveVUs(lActiveVus, lTotalVUs);

		CXMLResponse* response = new CXMLResponse();
		response->m_params.Add(new CXMLLong(lActiveVus));
		response->m_params.Add(new CXMLLong(lTotalVUs));
		return response;
	}
	catch(...)
	{
		g_Logger->WriteLogMsg(TRCLEV_LOW, "TM: GetTestVUs: Exception calling GetTestVUs on TestManager");
	}


	CXMLResponse* response = new CXMLResponse();
	response->SetError(CXMLResponse::UNKNOWN_ERROR);
	return response;
}