/* Copyright (C) 2000-2001 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

#include "stdafx.h"
#include "TestRunHandler.h"
#include "CECInetInfo.h"
#include "..\include\Logger.h"
#include "OstaTraceDef.h"
#include "CyrStRepository.h"

extern CLogger* g_Logger;
//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CTestRunHandler::CTestRunHandler(CORBAServer* pServer)
{
	m_pCorba = pServer;
}

CTestRunHandler::~CTestRunHandler()
{

}

CXMLResponse* CTestRunHandler::ProcessMethodCall(CXMLRequest* request)
{
	g_Logger->WriteLogMsg(TRCLEV_MEDIUM, "TR: ProcessMethodCall "+request->m_strFullMethodName);
	if (request->m_strMethod == "AddTestOutput")
	{
		if (request->GetNumParams() % 3 == 0)
		{
			CXMLResponse * response = new CXMLResponse() ;
			for (int i = 0; i<request->GetNumParams()/3; i++)
			{
				CXMLString* pParam1 = (CXMLString*)request->GetParam(i*3);
				CXMLString* pParam2 = (CXMLString*)request->GetParam(1+(i*3));
				CXMLBase64* pParam3 = (CXMLBase64*)request->GetParam(2 +(i*3));
				bool bRet = AddTestOutput(pParam1->GetValue(), pParam2->GetValue(), pParam3->GetValue());
				response->m_params.Add(new CXMLBool(bRet)) ;
			}
			return response ;
		}
		else
		{
			CXMLResponse* response = new CXMLResponse();
			if (request->GetNumParams() < 3)
				response->SetError(CXMLResponse::INSUFFICIENT_PARAMS);
			if (request->GetNumParams() > 3)
				response->SetError(CXMLResponse::TOO_MANY_PARAMS);
			return response;
		}

	}

	g_Logger->WriteLogMsg(TRCLEV_LOW, "TM: ProcessMethodCall: Error: Method Not Found");
	CXMLResponse* response = new CXMLResponse();
	response->SetError(CXMLResponse::FUNC_NOT_FOUND);
	return response;
}

bool CTestRunHandler::AddTestOutput(string strTestRunReg, string strType, string strMessage)
{
	g_Logger->WriteLogMsg(TRCLEV_MEDIUM, "TR: AddTestOutput");
	try
	{
		// Narrow the time service corba object on the target machine
		g_Logger->WriteLogMsg(TRCLEV_HIGH, "TR: AddTestOutput: Narrowing TestRun "+strTestRunReg);
		CORBA::Object_var obj = m_pCorba->FindObjectWithName(CyrStTestRun::CLASS_NAME,strTestRunReg.c_str());
		CyrStTestRun_var TR = CyrStTestRun::_narrow(obj);
		g_Logger->WriteLogMsg(TRCLEV_HIGH, "TR: AddTestOutput: Calling AddTestOutput");
		TR->AddTestOutput(strType.c_str(), strMessage.c_str());
		g_Logger->WriteLogMsg(TRCLEV_HIGH, "TR: AddTestOutput: AddTestOutput Called");

		return true ;
//		CXMLResponse* response = new CXMLResponse();
//		return response;
	}
	catch(...)
	{
		g_Logger->WriteLogMsg(TRCLEV_LOW, "TR: AddTestOutput: Exception calling AddTestOutput on TestRun");
	}

	return false ;
/*
	CXMLResponse* response = new CXMLResponse();
	response->SetError(CXMLResponse::UNKNOWN_ERROR);
	return response;
*/
}

