/* Copyright (C) 2000-2001 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

#include "stdafx.h"
#include "XMLFileTransferNode_i.h"
#include <CECInetInfo.h>
#include "xmlrpc.h"
#include <string>
#include <iostream>
#include <fstream>
using namespace std;
#include "..\include\Logger.h"
#include "tofprmdef.h"
#include "OstaTraceDef.h"


#include <string>
#include <iostream>
#include <fstream>
using namespace std;

extern CLogger* g_Logger;

XMLFileTransferNodeImpl::XMLFileTransferNodeImpl(CORBAServer* corba)
{
    m_registered_name="WebRelay";

    m_corba=corba;
	g_Logger->WriteLogMsg(TRCLEV_LOW, "RelayFTN: Registering XMLFileTransferNode interface");
	m_corba->ActivateObject(this);
    m_corba->NameObject( this->_this(),XMLFileTransferNode::CLASS_NAME,m_registered_name.c_str());

	m_RelayMap = XMLRelayMap::_nil();
}

XMLFileTransferNodeImpl::~XMLFileTransferNodeImpl()
{
    m_corba->UnNameObject( XMLFileTransferNode::CLASS_NAME,m_registered_name.c_str());
}

bool XMLFileTransferNodeImpl::TransferFileToLocal(const string & strURL, const string & strFile)
{
	g_Logger->WriteLogMsg(TRCLEV_LOW, "RelayFTN : TransferFileToLocal: " + strURL);
	if (strURL.substr(0,3) == "ftp")
	{
		string strServer, strUser, strPass, strRemotePath;
		int nPort;

		if(ParseFTPURL(strURL, strServer, nPort, strUser, strPass, strRemotePath))
		{
			g_Logger->WriteLogMsg(TRCLEV_LOW, "TransferFileToLocal: Open FTP connection");
			CFTPFileTransfer ftp;
			ftp.OpenConnection(strServer, nPort, strUser, strPass);
			g_Logger->WriteLogMsg(TRCLEV_LOW, "TransferFileToLocal: Get file by FTP");
			return ftp.GetFile(strRemotePath, strFile);
		}
		else
		{
			g_Logger->WriteLogMsg(TRCLEV_LOW, "TransferFileToLocal: Parse FTP URL failed");
		}
	}
	else if (strURL.substr(0,4) == "http")
	{
		g_Logger->WriteLogMsg(TRCLEV_LOW, "TransferFileToLocal: Use HTTP");
		CHTTPTransport t(strURL);

		g_Logger->WriteLogMsg(TRCLEV_LOW, "TransferFileToLocal: CALL GetRemoteFile");
		bool bRet = t.GetRemoteFile(strFile);
		if (bRet)
		{
			g_Logger->WriteLogMsg(TRCLEV_LOW, "TransferFileToLocal: TRANSFERRED");
		}
		else
		{
			g_Logger->WriteLogMsg(TRCLEV_LOW, "TransferFileToLocal: FAILED");
		}

		return bRet;
	}

	return false;
}


CORBA::Boolean XMLFileTransferNodeImpl::GetRemoteFileURL(const char* strReposHost, CORBA::Long nType, const char* strFile)
{
	try
	{
		g_Logger->WriteLogMsg(TRCLEV_LOW, "GetRemoteFileURL: "+string(strFile));
		if (CORBA::is_nil(m_RelayMap))
				Connect();
		else
		{
			// If the relay daemon has been stopped, must get a new reference to it.
			try
			{
				m_RelayMap->ping();
			}
			catch(...)
			{
				Connect();
			}
		}


		RelayDef def;	
		if (!m_RelayMap->GetRelayForAlias(strReposHost, def))
		{
			g_Logger->WriteLogMsg(TRCLEV_LOW, "No entry in relay map for "+string(strReposHost));
			return false;
		}
		g_Logger->WriteLogMsg(TRCLEV_LOW, "Relay = "+string(def.strHost));

		CHTTPTransport transport(string(def.strHost),string(def.strExtPath),def.lPort,string(def.strUser),string(def.strPass),def.bUseSSL);
		CXMLRequest req ;
		string strFullFile ;
		char *pOp = getenv("OPENSTA_HOME") ;

		if (!pOp)
		{
			g_Logger->WriteLogMsg(TRCLEV_LOW, "OPENSTA_HOME should not be NULL.");
			return false;
		}
		strFullFile = pOp ;

		switch (nType)
		{
			case REPOS_ROOT:
			{
				strFullFile += "Repository\\" + string(strFile);
				req.SetMethodName("FILECONTROL.CheckOutReposTraceSettings");
				req.AddParameter(new CXMLString(strReposHost));
				g_Logger->WriteLogMsg(TRCLEV_LOW, "Create FILECONTROL.CheckOutReposTraceSettings request");
				break;
			}
			case REPOS_COLLECTOR:
			{
				strFullFile += "Repository\\Profiles\\" + string(strFile);
				req.SetMethodName("FILECONTROL.CheckOutReposProfile");
				req.AddParameter(new CXMLString(strReposHost));
				req.AddParameter(new CXMLString(strFile));
				g_Logger->WriteLogMsg(TRCLEV_LOW, "Create FILECONTROL.CheckOutReposProfile request");
				break;
			}
			case REPOS_SCRIPT:
			{
				strFullFile += "Repository\\ObjectCode\\" + string(strFile);
				req.SetMethodName("FILECONTROL.CheckOutReposScript");
				req.AddParameter(new CXMLString(strReposHost));
				req.AddParameter(new CXMLString(strFile));
				break;
			}
			case REPOS_DATA:
			{
				strFullFile += "Repository\\Data\\" + string(strFile);
				req.SetMethodName("FILECONTROL.CheckOutReposData");
				req.AddParameter(new CXMLString(strReposHost));
				req.AddParameter(new CXMLString(strFile));
				break;
			}
		}
		req.SetTransport(transport);
		g_Logger->WriteLogMsg(TRCLEV_LOW, "Send request");
		CXMLResponse* resp = NULL;
		try
		{
			resp = req.SendRequest();
		}
		catch(...)
		{
			g_Logger->WriteLogMsg(TRCLEV_LOW, "EXCEPTION");
			if (resp)
				delete resp ;
			return false;
		}
		g_Logger->WriteLogMsg(TRCLEV_LOW, "Got a response!!");
		if (!resp || resp->IsError())
		{
			g_Logger->WriteLogMsg(TRCLEV_LOW, resp->PrintError());
			if (resp)
				delete resp;
			return false;
		}
	

		string strURL = "";
		if (resp->m_params.GetSize() == 1 && resp->m_params.GetAt(0)->GetType() == "base64")
		{
			CXMLBase64* pVal = (CXMLBase64*)resp->m_params.GetAt(0);
			strURL = pVal->GetValue();
		}
	
		g_Logger->WriteLogMsg(TRCLEV_LOW, "URL = "+strURL);
		delete resp;

		// retreive the file by ftp or http
		if (!TransferFileToLocal(strURL, strFullFile))
		{
			g_Logger->WriteLogMsg(TRCLEV_LOW, "Transfer File to " + strFullFile + " failed.");
			return false ;
		}
		return true;
	}
	catch(...)
	{
		g_Logger->WriteLogMsg(TRCLEV_LOW, "ERROR: GetRemoteFileURL: Exception.");
	}
	return false;
}


bool XMLFileTransferNodeImpl::ParseFTPURL(string strURL, string& strServer, int& nPort, string& strUser, string& strPass, string& strRemotePath)
{
	// FTP URL in the form ftp://user:pass@server:port/path

	string strTmp = strURL;

	// Remove ftp://
	if (strURL.substr(0,6) != "ftp://")
		return false;	
	strTmp.erase(0,6);


	// search for '@' and parse username/password
	int nPos = strTmp.find_first_of('@');
	if (nPos > 0)
	{
		int nColonPos = strTmp.find_first_of(':');
		if (nColonPos <= 0)
			return false;

		strUser = strTmp.substr(0,nColonPos);
		strPass = strTmp.substr(nColonPos+1, nPos-nColonPos-1);

		strTmp.erase(0, nPos+1);
	}

	// Next find the server address and port
	nPos = strTmp.find_first_of('/');
	if (nPos <= 0)
		return false;

	int nColonPos = strTmp.find_first_of(':');
	if (nColonPos > 0)
	{
		strServer = strTmp.substr(0, nColonPos);
		nPort = atoi(strTmp.substr(nColonPos+1, nPos).c_str());
	}
	else
	{
		strServer = strTmp.substr(0,nPos);
		nPort = 21;
	}
	strTmp.erase(0,nPos);

	if (strTmp.size() > 0 && strTmp[0] == '/')
		strTmp.erase(0, 1);
	// The rest is the remote path.
	strRemotePath = strTmp;

	return true;
}

void XMLFileTransferNodeImpl::Connect()
{
	try
	{
		CORBA::Object_ptr obj = m_corba->FindObjectWithName(XMLRelayMap::CLASS_NAME, "WebRelay");
		m_RelayMap = XMLRelayMap::_narrow(obj);
	}
	catch(...)
	{
	}
}
