/* Copyright (C) 2000-2001 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

#include "stdafx.h"
#include "XMLRelayInjectorControl_i.h"
#include <CECInetInfo.h>
#include "xmlrpc.h"
#include "..\VDK10DataTypes\XML_IC_TaskGroupExecInfo.h"
#include "..\include\Logger.h"
#include "OstaTraceDef.h"


#include <string>
#include <iostream>
#include <fstream>
using namespace std;

extern CLogger* g_Logger;

XMLRelayInjectorControlImpl::XMLRelayInjectorControlImpl(CORBAServer* corba)
{
    m_registered_name="WebRelay";

    m_corba=corba;
	g_Logger->WriteLogMsg(TRCLEV_LOW, "RelayIC: Registering XMLRelayInjectorControl interface");
	m_corba->ActivateObject(this);
    m_corba->NameObject( this->_this(),XMLRelayInjectorControl::CLASS_NAME,m_registered_name.c_str());

	m_RelayMap = XMLRelayMap::_nil();
}

XMLRelayInjectorControlImpl::~XMLRelayInjectorControlImpl()
{
    m_corba->UnNameObject( XMLRelayInjectorControl::CLASS_NAME,m_registered_name.c_str());
}

CORBA::Boolean XMLRelayInjectorControlImpl::Activate(const char* strTargetMachine,
								   const char* strManagerRegString, const char* strReposHost,
								   const char* strTestRunString, const char* strTestName)
{
	try
	{
		g_Logger->WriteLogMsg(TRCLEV_LOW, "RelayIC: Activate: Relaying Activate request for host "+string(strTargetMachine));
	
		if (CORBA::is_nil(m_RelayMap))
		{
			CORBA::Object_ptr obj = m_corba->FindObjectWithName(XMLRelayMap::CLASS_NAME, m_registered_name.c_str());
			m_RelayMap = XMLRelayMap::_narrow(obj);
		}
	
		RelayDef def;	
		if (!m_RelayMap->GetRelayForAlias(strTargetMachine, def))
		{
			g_Logger->WriteLogMsg(TRCLEV_LOW, "RelayIC: Activate: No relay server defined for this host");
			return false;
		}
		else
		{
			g_Logger->WriteLogMsg(TRCLEV_HIGH, "RelayIC: Activate: Relay server is "+string(def.strHost));
		}
	
		
		CHTTPTransport transport(string(def.strHost),string(def.strExtPath),def.lPort,string(def.strUser),string(def.strPass), def.bUseSSL);
	
		CXMLRequest req ;
		req.SetMethodName("INJECTORCONTROL.Activate");
		req.SetTransport(transport);
		req.AddParameter(new CXMLString(string(strTargetMachine)));
		req.AddParameter(new CXMLString(strManagerRegString));
		req.AddParameter(new CXMLString(strReposHost));
		req.AddParameter(new CXMLString(strTestRunString));
		req.AddParameter(new CXMLString(strTestName));
		CXMLResponse* resp = req.SendRequest();
		if (resp->IsError())
		{
			g_Logger->WriteLogMsg(TRCLEV_LOW, "RelayIC: Activate: Relay Error:"+resp->PrintError());
			delete resp ;
			return false;
		}
	
	
		bool bVal = false;
		if (resp->m_params.GetSize() == 1 && resp->m_params.GetAt(0)->GetType() == "boolean")
		{
			CXMLBool* pVal = (CXMLBool*)resp->m_params.GetAt(0);
			bVal = pVal->GetValue();
		}
	
		
		delete resp;
	
	    return bVal;
	}
	catch(...)
	{
		g_Logger->WriteLogMsg(TRCLEV_LOW, "RelayIC: Activate: EXCEPTION");
	}
	return false;
}


CORBA::Boolean XMLRelayInjectorControlImpl::IsActivated(const char* strTargetMachine, 
													const char* strManagerRegString)
{
	g_Logger->WriteLogMsg(TRCLEV_LOW, "RelayIC: IsActivated: Relaying IsActivated request for host "+string(strTargetMachine));
	if (CORBA::is_nil(m_RelayMap))
	{
		CORBA::Object_ptr obj = m_corba->FindObjectWithName(XMLRelayMap::CLASS_NAME, m_registered_name.c_str());
		m_RelayMap = XMLRelayMap::_narrow(obj);
	}

	RelayDef def;	
	if (!m_RelayMap->GetRelayForAlias(strTargetMachine, def))
	{
		g_Logger->WriteLogMsg(TRCLEV_LOW, "RelayIC: IsActivated: No relay server defined for this host");
		return false;
	}
	else
	{
		g_Logger->WriteLogMsg(TRCLEV_HIGH, "RelayIC: IsActivated: Relay server is "+string(def.strHost));
	}

	
	CHTTPTransport transport(string(def.strHost),string(def.strExtPath),def.lPort,string(def.strUser),string(def.strPass), def.bUseSSL);

	CXMLRequest req ;
	req.SetMethodName("INJECTORCONTROL.IsActivated");
	req.SetTransport(transport);
	req.AddParameter(new CXMLString(string(strTargetMachine)));
	req.AddParameter(new CXMLString(strManagerRegString));
	CXMLResponse* resp = req.SendRequest();
	if (resp->IsError())
	{
		g_Logger->WriteLogMsg(TRCLEV_LOW, "RelayIC: IsActivated: Relay Error:"+resp->PrintError());
		delete resp ;
		return false;
	}


	bool bVal = false;
	if (resp->m_params.GetSize() == 1 && resp->m_params.GetAt(0)->GetType() == "boolean")
	{
		CXMLBool* pVal = (CXMLBool*)resp->m_params.GetAt(0);
		bVal = pVal->GetValue();
	}

	
	delete resp;

    return bVal;
}

CORBA::Boolean XMLRelayInjectorControlImpl::StartTaskGroup(const char* strTargetMachine, TaskGroupExecInfo& tgExec)
{
	g_Logger->WriteLogMsg(TRCLEV_LOW, "RelayIC: StartTaskGroup: Relaying StartTaskGroup request for host "+string(strTargetMachine));
	if (CORBA::is_nil(m_RelayMap))
	{
		CORBA::Object_ptr obj = m_corba->FindObjectWithName(XMLRelayMap::CLASS_NAME, m_registered_name.c_str());
		m_RelayMap = XMLRelayMap::_narrow(obj);
	}

	RelayDef def;	
	if (!m_RelayMap->GetRelayForAlias(strTargetMachine, def))
	{
		g_Logger->WriteLogMsg(TRCLEV_LOW, "RelayIC: StartTaskGroup: No relay server defined for this host");
		return false;
	}
	else
	{
		g_Logger->WriteLogMsg(TRCLEV_HIGH, "RelayIC: StartTaskGroup: Relay server is "+string(def.strHost));
	}

	
	CHTTPTransport transport(string(def.strHost),string(def.strExtPath),def.lPort,string(def.strUser),string(def.strPass), def.bUseSSL);

	CXMLRequest req ;
	req.SetMethodName("INJECTORCONTROL.StartTaskGroup");
	req.SetTransport(transport);
	req.AddParameter(new CXMLString(string(strTargetMachine)));
	req.AddParameter(new CXML_IC_TaskGroupExecInfo(tgExec));
	CXMLResponse* resp = req.SendRequest();
	if (resp->IsError())
	{
		g_Logger->WriteLogMsg(TRCLEV_LOW, "RelayIC: StartTaskGroup: Relay Error:"+resp->PrintError());
		delete resp ;
		return false;
	}


	bool bVal = false;
	if (resp->m_params.GetSize() == 1 && resp->m_params.GetAt(0)->GetType() == "boolean")
	{
		CXMLBool* pVal = (CXMLBool*)resp->m_params.GetAt(0);
		bVal = pVal->GetValue();
	}

	
	delete resp;

    return bVal;
}

CORBA::Boolean XMLRelayInjectorControlImpl::StopTaskGroup(const char* strTargetMachine, CORBA::Long lTaskGroupId)
{
	g_Logger->WriteLogMsg(TRCLEV_LOW, "RelayIC: StopTaskGroup: Relaying StopTaskGroup request for host "+string(strTargetMachine));
	if (CORBA::is_nil(m_RelayMap))
	{
		CORBA::Object_ptr obj = m_corba->FindObjectWithName(XMLRelayMap::CLASS_NAME, m_registered_name.c_str());
		m_RelayMap = XMLRelayMap::_narrow(obj);
	}

	RelayDef def;	
	if (!m_RelayMap->GetRelayForAlias(strTargetMachine, def))
	{
		g_Logger->WriteLogMsg(TRCLEV_LOW, "RelayIC: StopTaskGroup: No relay server defined for this host");
		return false;
	}
	else
	{
		g_Logger->WriteLogMsg(TRCLEV_HIGH, "RelayIC: StopTaskGroup: Relay server is "+string(def.strHost));
	}

	
	CHTTPTransport transport(string(def.strHost),string(def.strExtPath),def.lPort,string(def.strUser),string(def.strPass), def.bUseSSL);

	CXMLRequest req ;
	req.SetMethodName("INJECTORCONTROL.StopTaskGroup");
	req.SetTransport(transport);
	req.AddParameter(new CXMLString(string(strTargetMachine)));
	req.AddParameter(new CXMLLong(lTaskGroupId));
	CXMLResponse* resp = req.SendRequest();
	if (resp->IsError())
	{
		g_Logger->WriteLogMsg(TRCLEV_LOW, "RelayIC: StopTaskGroup: Relay Error:"+resp->PrintError());
		delete resp ;
		return false;
	}


	bool bVal = false;
	if (resp->m_params.GetSize() == 1 && resp->m_params.GetAt(0)->GetType() == "boolean")
	{
		CXMLBool* pVal = (CXMLBool*)resp->m_params.GetAt(0);
		bVal = pVal->GetValue();
	}

	
	delete resp;

    return bVal;
}

CORBA::Boolean XMLRelayInjectorControlImpl::StopTest(const char* strTargetMachine, CORBA::Boolean bForce, CORBA::Long lTimeout)
{
	g_Logger->WriteLogMsg(TRCLEV_LOW, "RelayIC: StopTest: Relaying StopTest request for host "+string(strTargetMachine));
	if (CORBA::is_nil(m_RelayMap))
	{
		CORBA::Object_ptr obj = m_corba->FindObjectWithName(XMLRelayMap::CLASS_NAME, m_registered_name.c_str());
		m_RelayMap = XMLRelayMap::_narrow(obj);
	}

	RelayDef def;	
	if (!m_RelayMap->GetRelayForAlias(strTargetMachine, def))
	{
		g_Logger->WriteLogMsg(TRCLEV_LOW, "RelayIC: StopTest: No relay server defined for this host");
		return false;
	}
	else
	{
		g_Logger->WriteLogMsg(TRCLEV_HIGH, "RelayIC: StopTest: Relay server is "+string(def.strHost));
	}

	
	CHTTPTransport transport(string(def.strHost),string(def.strExtPath),def.lPort,string(def.strUser),string(def.strPass), def.bUseSSL);

	CXMLRequest req ;
	req.SetMethodName("INJECTORCONTROL.StopTest");
	req.SetTransport(transport);
	req.AddParameter(new CXMLString(string(strTargetMachine)));
	req.AddParameter(new CXMLBool(bForce));
	req.AddParameter(new CXMLLong(lTimeout));
	CXMLResponse* resp = req.SendRequest();
	if (resp->IsError())
	{
		g_Logger->WriteLogMsg(TRCLEV_LOW, "RelayIC: StopTest: Relay Error:"+resp->PrintError());
		delete resp ;
		return false;
	}

	bool bVal = false;
	if (resp->m_params.GetSize() == 1 && resp->m_params.GetAt(0)->GetType() == "boolean")
	{
		CXMLBool* pVal = (CXMLBool*)resp->m_params.GetAt(0);
		bVal = pVal->GetValue();
	}

	delete resp;

    return bVal;
}

CORBA::Long XMLRelayInjectorControlImpl::TransferResults(const char* strTargetMachine, const char* strTestName, const char* strReposHost, const char* strRunFolder)
{
	g_Logger->WriteLogMsg(TRCLEV_LOW, "RelayIC: TransferResults: Relaying TransferResults request for host "+string(strTargetMachine));
	if (CORBA::is_nil(m_RelayMap))
	{
		CORBA::Object_ptr obj = m_corba->FindObjectWithName(XMLRelayMap::CLASS_NAME, m_registered_name.c_str());
		m_RelayMap = XMLRelayMap::_narrow(obj);
	}

	RelayDef def;	
	if (!m_RelayMap->GetRelayForAlias(strTargetMachine, def))
	{
		g_Logger->WriteLogMsg(TRCLEV_LOW, "RelayIC: TransferResults: No relay server defined for this host");
		return 0;
	}
	else
	{
		g_Logger->WriteLogMsg(TRCLEV_HIGH, "RelayIC: TransferResults: Relay server is "+string(def.strHost));
	}

	
	CHTTPTransport transport(string(def.strHost),string(def.strExtPath),def.lPort,string(def.strUser),string(def.strPass), def.bUseSSL);

	CXMLRequest req ;
	req.SetMethodName("INJECTORCONTROL.TransferResults");
	req.SetTransport(transport);
	req.AddParameter(new CXMLString(string(strTargetMachine)));
	req.AddParameter(new CXMLString(strTestName));
	req.AddParameter(new CXMLString(strReposHost));
	req.AddParameter(new CXMLString(strRunFolder));
	CXMLResponse* resp = req.SendRequest();
	if (resp->IsError())
	{
		g_Logger->WriteLogMsg(TRCLEV_LOW, "RelayIC: TransferResults: Relay Error:"+resp->PrintError());
		delete resp ;
		return 0;
	}

	long lVal = 0;
	if (resp->m_params.GetSize() == 1 && resp->m_params.GetAt(0)->GetType() == "boolean")
	{
		CXMLLong* pVal = (CXMLLong*)resp->m_params.GetAt(0);
		lVal = pVal->GetValue();
	}

	delete resp;

    return lVal;
}
