/* Copyright (C) 2000-2001 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

// XMLRelayInjectorStatus_i.cpp: implementation of the XMLRelayInjectorStatusImpl class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "XMLRelayInjectorStatus_i.h"
#include <CECInetInfo.h>
#include "xmlrpc.h"
#include "..\include\Logger.h"
#include "OstaTraceDef.h"
#include "..\VDK10DataTypes\XML_IS_TestExecuterSnapshot.h"
#include "..\VDK10DataTypes\XML_IS_TaskSnap2.h"
#include "..\VDK10DataTypes\XML_IS_HTTPExecuterSnap.h"
#include "..\VDK10DataTypes\XML_IS_TaskId.h"

#include <string>
#include <iostream>
#include <fstream>
using namespace std;

extern CLogger* g_Logger;

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

XMLRelayInjectorStatusImpl::XMLRelayInjectorStatusImpl(CORBAServer* corba)
{
	 m_registered_name="WebRelay";

    m_corba=corba;
	g_Logger->WriteLogMsg(TRCLEV_LOW, "RelayIS: Registering XMLRelayInjectorStatus interface");
	m_corba->ActivateObject(this);
    m_corba->NameObject( this->_this(),XMLRelayInjectorStatus::CLASS_NAME,m_registered_name.c_str());

	m_RelayMap = XMLRelayMap::_nil();
}

XMLRelayInjectorStatusImpl::~XMLRelayInjectorStatusImpl()
{
	m_corba->UnNameObject( XMLRelayInjectorStatus::CLASS_NAME,m_registered_name.c_str());
}

CORBA::Boolean XMLRelayInjectorStatusImpl::GetStatsSnapshot(const char* strTargetMachine, TESnapList& InjectorSnapList)
{	
	g_Logger->WriteLogMsg(TRCLEV_LOW, "RelayIS: GetStatsSnapshot: Relaying GetStatsSnapshot request for host "+string(strTargetMachine));
	if (CORBA::is_nil(m_RelayMap))
	{
		CORBA::Object_ptr obj = m_corba->FindObjectWithName(XMLRelayMap::CLASS_NAME, m_registered_name.c_str());
		m_RelayMap = XMLRelayMap::_narrow(obj);
	}

	RelayDef def;	
	if (!m_RelayMap->GetRelayForAlias(strTargetMachine, def))
	{
		g_Logger->WriteLogMsg(TRCLEV_LOW, "RelayIS: GetStatsSnapshot: No relay server defined for this host");
		return false;
	}
	else
	{
		g_Logger->WriteLogMsg(TRCLEV_HIGH, "RelayIS: GetStatsSnapshot: Relay server is "+string(def.strHost));
	}

	
	CHTTPTransport transport(string(def.strHost),string(def.strExtPath),def.lPort,string(def.strUser),string(def.strPass), def.bUseSSL);

	CXMLRequest req ;
	req.SetMethodName("INJECTORSTATUS.GetStatsSnapshot");
	req.SetTransport(transport);
	req.AddParameter(new CXMLString(string(strTargetMachine)));

	CXMLResponse* resp = req.SendRequest();
	if (resp->IsError())
	{
		g_Logger->WriteLogMsg(TRCLEV_LOW, "RelayIS: GetStatsSnapshot: Relay Error:"+resp->PrintError());
		delete resp ;
		
		return 0;
	}

	//	typedef sequence <TestExecuterSnapshot> TESnapList;
	
	if (resp->m_params.GetSize() >0 && resp->m_params.GetAt(0)->GetType() == "boolean")
	{
		CXMLBool* pVal = (CXMLBool*)resp->m_params.GetAt(0);
		if((pVal->GetValue())==true)
		{
			//it's a valid response and there is a snapshot

			if (resp->m_params.GetSize() >1 && resp->m_params.GetAt(1)->GetType() == "i4")
			{
				long lSnaps=0;
				CXMLLong* pVal = (CXMLLong*)resp->m_params.GetAt(1);
				lSnaps = pVal->GetValue();
				if(lSnaps>0)
				{
					InjectorSnapList.length(lSnaps);			
					for(long a=0 ; a<lSnaps ; a++)
					{
						if(resp->m_params.GetSize() >a+2 && resp->m_params.GetAt(a+2)->GetType() == "TestExecuterSnapshot")
						{
							//then its a valid snapshot! add it to the list
							CXML_IS_TestExecuterSnapshot* pVal = (CXML_IS_TestExecuterSnapshot*)resp->m_params.GetAt(a+2);
							InjectorSnapList[a] = pVal->GetValue();
						}
						else
						{
							g_Logger->WriteLogMsg(TRCLEV_LOW, "RelayIS: GetStatsSnapshot: Error unexpected type in compound struct type");
							delete resp;
							return false;	
						}
					}
				}
				else
				{
					//ensure it's empty
					InjectorSnapList.length(0);
				}
				
				delete resp;
				return true;
			}
		}
	}

	
	delete resp;
	return false;
}

CORBA::Long XMLRelayInjectorStatusImpl::GetInjectorVUs(const char* strTargetMachine, CORBA::Long lPreviousTestVUCount)
{
	long lNumVUs=0;
	try
	{
		g_Logger->WriteLogMsg(TRCLEV_LOW, "RelayIS: GetInjectorVUs: Relaying GetInjectorVUs request for host "+string(strTargetMachine));
		if (CORBA::is_nil(m_RelayMap))
		{
			CORBA::Object_ptr obj = m_corba->FindObjectWithName(XMLRelayMap::CLASS_NAME, m_registered_name.c_str());
			m_RelayMap = XMLRelayMap::_narrow(obj);
		}

		RelayDef def;	
		if (!m_RelayMap->GetRelayForAlias(strTargetMachine, def))
		{
			g_Logger->WriteLogMsg(TRCLEV_LOW, "RelayIS: GetInjectorVUs: No relay server defined for this host");
			return false;
		}
		else
		{
			g_Logger->WriteLogMsg(TRCLEV_HIGH, "RelayIS: GetInjectorVUs: Relay server is "+string(def.strHost));
		}

	
		CHTTPTransport transport(string(def.strHost),string(def.strExtPath),def.lPort,string(def.strUser),string(def.strPass), def.bUseSSL);

		CXMLRequest req ;
		req.SetMethodName("INJECTORSTATUS.GetInjectorVUs");
		req.SetTransport(transport);
		req.AddParameter(new CXMLString(string(strTargetMachine)));
		req.AddParameter(new CXMLLong(lPreviousTestVUCount));
		CXMLResponse* resp = req.SendRequest();
		if (resp->IsError())
		{
			g_Logger->WriteLogMsg(TRCLEV_LOW, "RelayIS: GetInjectorVUs: Relay Error:"+resp->PrintError());
			delete resp ;
			
			return 0;
		}
	

		if (resp->m_params.GetSize() == 1 && resp->m_params.GetAt(0)->GetType() == "i4")
		{
			CXMLLong* pVal = (CXMLLong*)resp->m_params.GetAt(0);
			lNumVUs = pVal->GetValue();
		}
	
		
		delete resp;
	}
	catch(...)
	{
		g_Logger->WriteLogMsg(TRCLEV_NONE, "RelayIS: GetInjectorVUs: EXCEPTION");
	}
    return lNumVUs;

}

void XMLRelayInjectorStatusImpl::GetInjectorTotalAndActiveVUs(const char* strTargetMachine, CORBA::Long lPreviousTestActiveVUCount, CORBA::Long& lActiveVUCount, CORBA::Long lPreviousTestTotalVUCount, CORBA::Long& lTotalVUCount)
{
	long lActiveVUs = 0;
	long lTotalVUs = 0 ;
	try
	{
		g_Logger->WriteLogMsg(TRCLEV_LOW, "RelayIS: GetInjectorTotalAndActiveVUs: Relaying GetInjectorVUs request for host "+string(strTargetMachine));
		if (CORBA::is_nil(m_RelayMap))
		{
			CORBA::Object_ptr obj = m_corba->FindObjectWithName(XMLRelayMap::CLASS_NAME, m_registered_name.c_str());
			m_RelayMap = XMLRelayMap::_narrow(obj);
		}

		RelayDef def;	
		if (!m_RelayMap->GetRelayForAlias(strTargetMachine, def))
		{
			g_Logger->WriteLogMsg(TRCLEV_LOW, "RelayIS: GetInjectorTotalAndActiveVUs: No relay server defined for this host");
			return ;
		}
		else
		{
			g_Logger->WriteLogMsg(TRCLEV_HIGH, "RelayIS: GetInjectorTotalAndActiveVUs: Relay server is "+string(def.strHost));
		}

	
		CHTTPTransport transport(string(def.strHost),string(def.strExtPath),def.lPort,string(def.strUser),string(def.strPass), def.bUseSSL);

		CXMLRequest req ;
		req.SetMethodName("INJECTORSTATUS.GetInjectorTotalAndActiveVUs");
		req.SetTransport(transport);
		req.AddParameter(new CXMLString(string(strTargetMachine)));
		req.AddParameter(new CXMLLong(lPreviousTestActiveVUCount));
		req.AddParameter(new CXMLLong(lActiveVUs));
		req.AddParameter(new CXMLLong(lPreviousTestTotalVUCount));
		req.AddParameter(new CXMLLong(lTotalVUs));
		CXMLResponse* resp = req.SendRequest();
		if (resp->IsError())
		{
			g_Logger->WriteLogMsg(TRCLEV_LOW, "RelayIS: GetInjectorTotalAndActiveVUs: Relay Error:"+resp->PrintError());
			delete resp ;
			
			return ;
		}
	

		if (resp->m_params.GetSize() == 2 && 
			resp->m_params.GetAt(0)->GetType() == "i4" &&
			resp->m_params.GetAt(1)->GetType() == "i4")
		{
			CXMLLong* pVal = (CXMLLong*)resp->m_params.GetAt(0);
			lActiveVUCount = pVal->GetValue();
			pVal = (CXMLLong*)resp->m_params.GetAt(1);
			lTotalVUCount = pVal->GetValue();
		}
	
		
		delete resp;
	}
	catch(...)
	{
			g_Logger->WriteLogMsg(TRCLEV_NONE, "RelayIS: GetInjectorTotalAndActiveVUs: EXCEPTION");
	}
}


CORBA::Boolean XMLRelayInjectorStatusImpl::GetTaskSnapshot(const char* strTargetMachine, const TaskIdList& taskIDs, TaskSnap2List& taskSnapshots)
{
	g_Logger->WriteLogMsg(TRCLEV_LOW, "RelayIS: GetTaskSnapshot: Relaying GetTaskSnapshot request for host "+string(strTargetMachine));
	if (CORBA::is_nil(m_RelayMap))
	{
		CORBA::Object_ptr obj = m_corba->FindObjectWithName(XMLRelayMap::CLASS_NAME, m_registered_name.c_str());
		m_RelayMap = XMLRelayMap::_narrow(obj);
	}

	RelayDef def;	
	if (!m_RelayMap->GetRelayForAlias(strTargetMachine, def))
	{
		g_Logger->WriteLogMsg(TRCLEV_LOW, "RelayIS: GetTaskSnapshot: No relay server defined for this host");
		return false;
	}
	else
	{
		g_Logger->WriteLogMsg(TRCLEV_HIGH, "RelayIS: GetTaskSnapshot: Relay server is "+string(def.strHost));
	}

	
	CHTTPTransport transport(string(def.strHost),string(def.strExtPath),def.lPort,string(def.strUser),string(def.strPass), def.bUseSSL);

	CXMLRequest req ;
	req.SetMethodName("INJECTORSTATUS.GetTaskSnapshot");
	req.SetTransport(transport);
	req.AddParameter(new CXMLString(string(strTargetMachine)));

	CXMLArray * arrTaskIds = new CXMLArray;
	for (int i = 0; i<(int)taskIDs.length(); i++)
	{
		CXML_IS_TaskId* task = new CXML_IS_TaskId(taskIDs[i]);
		arrTaskIds->AddElement(task);
	}
	req.AddParameter(arrTaskIds);

	CXMLResponse* resp = req.SendRequest();
	if (resp->IsError())
	{
		g_Logger->WriteLogMsg(TRCLEV_LOW, "RelayIS: GetTaskSnapshot: Relay Error:"+resp->PrintError());
		delete resp ;
		
		return 0;
	}

	if (resp->m_params.GetSize() >0 && resp->m_params.GetAt(0)->GetType() == "boolean")
	{
		CXMLBool* pVal = (CXMLBool*)resp->m_params.GetAt(0);
		if((pVal->GetValue())==true)
		{
			//it's a valid response and there is a snapshot

			if (resp->m_params.GetSize() >1 && resp->m_params.GetAt(1)->GetType() == "i4")
			{
				long lSnaps=0;
				CXMLLong* pVal = (CXMLLong*)resp->m_params.GetAt(1);
				lSnaps = pVal->GetValue();
				if(lSnaps>0)
				{
					taskSnapshots.length(lSnaps);			
					for(long a=0 ; a<lSnaps ; a++)
					{
						if(resp->m_params.GetSize() >a+2 && resp->m_params.GetAt(a+2)->GetType() == "TaskSnap2")
						{
							//then its a valid snapshot! add it to the list
							CXML_IS_TaskSnap2* pVal = (CXML_IS_TaskSnap2*)resp->m_params.GetAt(a+2);
							const TaskSnap2 & ts2 = pVal->GetValue() ;
							taskSnapshots[a].task.lTaskGroupId =  ts2.task.lTaskGroupId;
							taskSnapshots[a].task.lTaskId =  ts2.task.lTaskId;
							if (ts2.VUSnaps.length() > 0)
							{
								taskSnapshots[a].VUSnaps.length(ts2.VUSnaps.length());
								for (unsigned int ii=0; ii<ts2.VUSnaps.length();ii++)
								{
									taskSnapshots[a].VUSnaps[ii].lVUId = ts2.VUSnaps[ii].lVUId ;
									taskSnapshots[a].VUSnaps[ii].lStartTime = ts2.VUSnaps[ii].lStartTime ;
									taskSnapshots[a].VUSnaps[ii].lRunTime = ts2.VUSnaps[ii].lRunTime ;
									taskSnapshots[a].VUSnaps[ii].lCompletedIterations = ts2.VUSnaps[ii].lCompletedIterations ;
									taskSnapshots[a].VUSnaps[ii].strNoteText = ts2.VUSnaps[ii].strNoteText ;
								}
							}
						}
						else
						{
							g_Logger->WriteLogMsg(TRCLEV_LOW, "RelayIS: GetTaskSnapshot: Error unexpected type in compound struct type");
							
							delete resp;
							return false;	
						}
					}
				}
				else
				{
					//ensure it's empty
					taskSnapshots.length(0);
				}
				
				delete resp;
				return true;
			}
		}
	}

	
	delete resp;
	return false;
}

CORBA::Boolean XMLRelayInjectorStatusImpl::StartMonitoring(const char* strTargetMachine,const char* strFilename,long lInterval)
{
	g_Logger->WriteLogMsg(TRCLEV_LOW, "RelayIS: StartMonitoring: Relaying StartMonitoring request for host "+string(strTargetMachine));
	if (CORBA::is_nil(m_RelayMap))
	{
		CORBA::Object_ptr obj = m_corba->FindObjectWithName(XMLRelayMap::CLASS_NAME, m_registered_name.c_str());
		m_RelayMap = XMLRelayMap::_narrow(obj);
	}

	RelayDef def;	
	if (!m_RelayMap->GetRelayForAlias(strTargetMachine, def))
	{
		g_Logger->WriteLogMsg(TRCLEV_LOW, "RelayIS: StartMonitoring: No relay server defined for this host");
		return false;
	}
	else
	{
		g_Logger->WriteLogMsg(TRCLEV_HIGH, "RelayIS: StartMonitoring: Relay server is "+string(def.strHost));
	}

	
	CHTTPTransport transport(string(def.strHost),string(def.strExtPath),def.lPort,string(def.strUser),string(def.strPass), def.bUseSSL);

	CXMLRequest req ;
	req.SetMethodName("INJECTORSTATUS.StartMonitoring");
	req.SetTransport(transport);
	req.AddParameter(new CXMLString(string(strTargetMachine)));
	req.AddParameter(new CXMLString(strFilename));
	req.AddParameter(new CXMLLong(lInterval));
	CXMLResponse* resp = req.SendRequest();
	if (resp->IsError())
	{
		g_Logger->WriteLogMsg(TRCLEV_LOW, "RelayIS: StartMonitoring: Relay Error:"+resp->PrintError());
		delete resp ;
		
		return 0;
	}

	bool bRetVal=false;
	if (resp->m_params.GetSize() == 1 && resp->m_params.GetAt(0)->GetType() == "boolean")
	{
		CXMLBool* pVal = (CXMLBool*)resp->m_params.GetAt(0);
		bRetVal = pVal->GetValue();
	}

	
	delete resp;

    return bRetVal;
}

CORBA::Boolean XMLRelayInjectorStatusImpl::HTTP_GetBothStatsSnapshots(const char* strTargetMachine, HTTP_TestExecuterSnapshot& HTTP_InjectorSnapshot, TESnapList& InjectorSnapList)
{

	g_Logger->WriteLogMsg(TRCLEV_LOW, "RelayIS: HTTP_GetBothStatsSnapshots: Relaying HTTP_GetBothStatsSnapshots request for host "+string(strTargetMachine));
	if (CORBA::is_nil(m_RelayMap))
	{
		CORBA::Object_ptr obj = m_corba->FindObjectWithName(XMLRelayMap::CLASS_NAME, m_registered_name.c_str());
		m_RelayMap = XMLRelayMap::_narrow(obj);
	}

	RelayDef def;	
	if (!m_RelayMap->GetRelayForAlias(strTargetMachine, def))
	{
		g_Logger->WriteLogMsg(TRCLEV_LOW, "RelayIS: HTTP_GetBothStatsSnapshots: No relay server defined for this host");
		return false;
	}
	else
	{
		g_Logger->WriteLogMsg(TRCLEV_HIGH, "RelayIS: HTTP_GetBothStatsSnapshots: Relay server is "+string(def.strHost));
	}

	
	CHTTPTransport transport(string(def.strHost),string(def.strExtPath),def.lPort,string(def.strUser),string(def.strPass), def.bUseSSL);

	CXMLRequest req ;
	req.SetMethodName("INJECTORSTATUS.HTTP_GetBothStatsSnapshots");
	req.SetTransport(transport);
	req.AddParameter(new CXMLString(string(strTargetMachine)));

	CXMLResponse* resp = req.SendRequest();
	if (resp->IsError())
	{
		g_Logger->WriteLogMsg(TRCLEV_LOW, "RelayIS: HTTP_GetBothStatsSnapshots: Relay Error:"+resp->PrintError());
		
		delete resp;
		return false;
	}
	if (resp->m_params.GetSize() >0 && resp->m_params.GetAt(0)->GetType() == "boolean")
	{
		CXMLBool* pVal = (CXMLBool*)resp->m_params.GetAt(0);
		if((pVal->GetValue())==true)
		{
			//it's a valid response and there is a snapshot

			if (resp->m_params.GetSize() >1 && resp->m_params.GetAt(1)->GetType() == "i4")
			{
				long lHTTPSnaps=0;
				CXMLLong* pVal = (CXMLLong*)resp->m_params.GetAt(1);
				lHTTPSnaps = pVal->GetValue();
				char zBuf[150];
				sprintf(zBuf,"%d", lHTTPSnaps);
				g_Logger->WriteLogMsg(TRCLEV_LOW, "RelayIS: HTTP_GetBothStatsSnapshots: lHTTPSnaps = " + string(zBuf));
				if(lHTTPSnaps != 0)
				{
					sprintf(zBuf,"%d, %s", resp->m_params.GetSize(), resp->m_params.GetAt(2)->GetType().c_str ());
					g_Logger->WriteLogMsg(TRCLEV_LOW, "RelayIS: HTTP_GetBothStatsSnapshots: " + string(zBuf));
					//there is one HTTP executer always - it has a dummy record if there is not one present
					if(resp->m_params.GetSize() >2 && resp->m_params.GetAt(2)->GetType() == "HTTPExecuterSnapshot")
					{
						//then its a valid HTTP snapshot
			
						CXML_IS_HTTPExecuterSnap* pVal = (CXML_IS_HTTPExecuterSnap*)resp->m_params.GetAt(2);

						const HTTP_TestExecuterSnapshot & hh = pVal->GetValue();
						HTTP_InjectorSnapshot.lAvgConnectionTime = hh.lAvgConnectionTime;
						HTTP_InjectorSnapshot.strTestExecuter = CORBA::string_dup(hh.strTestExecuter);
						HTTP_InjectorSnapshot.seqTaskGroupSnaps.length(hh.seqTaskGroupSnaps.length());

						sprintf(zBuf, "%s, %d, %d", HTTP_InjectorSnapshot.strTestExecuter.in(), HTTP_InjectorSnapshot.lAvgConnectionTime, HTTP_InjectorSnapshot.seqTaskGroupSnaps.length());
						g_Logger->WriteLogMsg(TRCLEV_LOW, "RelayIS: HTTP_GetBothStatsSnapshots: " + string(zBuf));
						for (unsigned int ii = 0; ii<HTTP_InjectorSnapshot.seqTaskGroupSnaps.length(); ii++)
						{
							HTTP_InjectorSnapshot.seqTaskGroupSnaps[ii].lTaskGroupId = hh.seqTaskGroupSnaps[ii].lTaskGroupId ;
							HTTP_InjectorSnapshot.seqTaskGroupSnaps[ii].lCompletedIterations = hh.seqTaskGroupSnaps[ii].lCompletedIterations ;
							HTTP_InjectorSnapshot.seqTaskGroupSnaps[ii].lRunTime = hh.seqTaskGroupSnaps[ii].lRunTime ;
							HTTP_InjectorSnapshot.seqTaskGroupSnaps[ii].lTotalUsers = hh.seqTaskGroupSnaps[ii].lTotalUsers ;
							HTTP_InjectorSnapshot.seqTaskGroupSnaps[ii].lTotalErrorUsers = hh.seqTaskGroupSnaps[ii].lTotalErrorUsers ;
							HTTP_InjectorSnapshot.seqTaskGroupSnaps[ii].lActiveErrorUsers = hh.seqTaskGroupSnaps[ii].lActiveErrorUsers ;
							HTTP_InjectorSnapshot.seqTaskGroupSnaps[ii].lFinishedUsers = hh.seqTaskGroupSnaps[ii].lFinishedUsers ;
							HTTP_InjectorSnapshot.seqTaskGroupSnaps[ii].lRequests = hh.seqTaskGroupSnaps[ii].lRequests ;
							HTTP_InjectorSnapshot.seqTaskGroupSnaps[ii].lFailedRequests = hh.seqTaskGroupSnaps[ii].lFailedRequests ;
							HTTP_InjectorSnapshot.seqTaskGroupSnaps[ii].l3XXRequests = hh.seqTaskGroupSnaps[ii].l3XXRequests ;
							HTTP_InjectorSnapshot.seqTaskGroupSnaps[ii].l4XXRequests = hh.seqTaskGroupSnaps[ii].l4XXRequests ;
							HTTP_InjectorSnapshot.seqTaskGroupSnaps[ii].l5XXRequests = hh.seqTaskGroupSnaps[ii].l5XXRequests ;
							HTTP_InjectorSnapshot.seqTaskGroupSnaps[ii].dBytesIn = hh.seqTaskGroupSnaps[ii].dBytesIn ;
							HTTP_InjectorSnapshot.seqTaskGroupSnaps[ii].dBytesOut = hh.seqTaskGroupSnaps[ii].dBytesOut ;
							HTTP_InjectorSnapshot.seqTaskGroupSnaps[ii].lMinRequestLatency = hh.seqTaskGroupSnaps[ii].lMinRequestLatency ;
							HTTP_InjectorSnapshot.seqTaskGroupSnaps[ii].lAvgRequestLatency = hh.seqTaskGroupSnaps[ii].lAvgRequestLatency ;
							HTTP_InjectorSnapshot.seqTaskGroupSnaps[ii].lMaxRequestLatency = hh.seqTaskGroupSnaps[ii].lMaxRequestLatency ;
							HTTP_InjectorSnapshot.seqTaskGroupSnaps[ii].seqTaskSnaps.length(hh.seqTaskGroupSnaps[ii].seqTaskSnaps.length());

							sprintf(zBuf, "%d, %d, %d, %d, %d, %d, %d, %d",
								HTTP_InjectorSnapshot.seqTaskGroupSnaps[ii].lTaskGroupId, 
								HTTP_InjectorSnapshot.seqTaskGroupSnaps[ii].lCompletedIterations, 
								HTTP_InjectorSnapshot.seqTaskGroupSnaps[ii].lRunTime,
								HTTP_InjectorSnapshot.seqTaskGroupSnaps[ii].lTotalUsers, 
								HTTP_InjectorSnapshot.seqTaskGroupSnaps[ii].lTotalErrorUsers, 
								HTTP_InjectorSnapshot.seqTaskGroupSnaps[ii].lActiveErrorUsers,
								HTTP_InjectorSnapshot.seqTaskGroupSnaps[ii].lFinishedUsers, 
								HTTP_InjectorSnapshot.seqTaskGroupSnaps[ii].lRequests);
							g_Logger->WriteLogMsg(TRCLEV_LOW, "RelayIS: HTTP_GetBothStatsSnapshots: " + string(zBuf));

							for (unsigned int jj = 0; jj<HTTP_InjectorSnapshot.seqTaskGroupSnaps[ii].seqTaskSnaps.length(); jj++)
							{
								HTTP_InjectorSnapshot.seqTaskGroupSnaps[ii].seqTaskSnaps[jj].lTaskId = hh.seqTaskGroupSnaps[ii].seqTaskSnaps[jj].lTaskId;
								HTTP_InjectorSnapshot.seqTaskGroupSnaps[ii].seqTaskSnaps[jj].lActiveUsers = hh.seqTaskGroupSnaps[ii].seqTaskSnaps[jj].lActiveUsers;
								HTTP_InjectorSnapshot.seqTaskGroupSnaps[ii].seqTaskSnaps[jj].lBlockingErrorUsers = hh.seqTaskGroupSnaps[ii].seqTaskSnaps[jj].lBlockingErrorUsers;
								HTTP_InjectorSnapshot.seqTaskGroupSnaps[ii].seqTaskSnaps[jj].lNonBlockingErrorUsers = hh.seqTaskGroupSnaps[ii].seqTaskSnaps[jj].lNonBlockingErrorUsers;
								HTTP_InjectorSnapshot.seqTaskGroupSnaps[ii].seqTaskSnaps[jj].lIterations = hh.seqTaskGroupSnaps[ii].seqTaskSnaps[jj].lIterations;
								HTTP_InjectorSnapshot.seqTaskGroupSnaps[ii].seqTaskSnaps[jj].lRuntime = hh.seqTaskGroupSnaps[ii].seqTaskSnaps[jj].lRuntime;
							}
						}

						sprintf(zBuf,"%d, %s", resp->m_params.GetSize(), resp->m_params.GetAt(3)->GetType().c_str ());
						g_Logger->WriteLogMsg(TRCLEV_LOW, "RelayIS: HTTP_GetBothStatsSnapshots: " + string(zBuf));
						//now the original stats
						if (resp->m_params.GetSize() >3 && resp->m_params.GetAt(3)->GetType() == "i4")
						{
							long lOriginalSnaps=0;
							CXMLLong* pVal = (CXMLLong*)resp->m_params.GetAt(3);
							lOriginalSnaps = pVal->GetValue();

							TESnapList tempInjectorSnapList;

							sprintf(zBuf,"lOriginalSnaps = %d", lOriginalSnaps);
							g_Logger->WriteLogMsg(TRCLEV_LOW, "RelayIS: HTTP_GetBothStatsSnapshots: " + string(zBuf));
							if(lOriginalSnaps>0)
							{
								//get each of the original stats

								InjectorSnapList.length(lOriginalSnaps);			
								for(long a=0 ; a<lOriginalSnaps ; a++)
								{
									if(resp->m_params.GetSize() >(a+4) && resp->m_params.GetAt(a+4)->GetType() == "TestExecuterSnapshot")
									{
										//then its a valid snapshot! add it to the list
										CXML_IS_TestExecuterSnapshot* pVal = (CXML_IS_TestExecuterSnapshot*)resp->m_params.GetAt(a+4);
										InjectorSnapList[a] = pVal->GetValue();
									}
									else
									{
										g_Logger->WriteLogMsg(TRCLEV_LOW, "RelayIS: HTTP_GetBothStatsSnapshots: Error unexpected type in compound struct type");
										
										delete resp;
										return false;
									}
								}
							
							}
								
							
							delete resp;
							return true;						
						}
					}
				}
			}
		}
	}

	
	delete resp;
	return false;
}
