/* Copyright (C) 2000-2001 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

///////////////////////////////////////////////////////////////////////////////
// Cyrano includes
#include "stdafx.h"
#include "..\RelayDmn\XMLVDK.h"
#include "XMLRelayInjectorControl_i.h"
#include "XMLRelayInjectorStatus_i.h"
#include "XMLRelayTestManager_i.h"
#include "XMLRelayTestRun_i.h"
#include "XMLFileTransferNode_i.h"
#include <CECInetInfo.h>
#include "..\include\Logger.h"
#include "InjectorControlHandler.h"
#include "InjectorStatusHandler.h"
#include "TestManagerHandler.h"
#include "TestRunHandler.h"
#include "VDK10DataProviders.h"
#include "FileControl.h"
#include "OstaTraceDef.h"

///////////////////////////////////////////////////////////////////////////////
// Global variables
CLogger* g_Logger = NULL;
XMLRelayInjectorControlImpl* ICRelay = NULL;
XMLRelayInjectorStatusImpl* ISRelay = NULL;
XMLRelayTestManagerImpl* TMRelay = NULL;
XMLRelayTestRunImpl* TRRelay = NULL;
XMLFileTransferNodeImpl* FTNRelay = NULL ;

CInjectorControlHandler* pICHandler = NULL;
CInjectorStatusHandler* pISHandler = NULL;
CTestManagerHandler* pTMHandler = NULL;
CTaskExecInfoProvider* pTEProv = NULL;
CTaskGroupExecInfoProvider* pTGProv = NULL;
CHTTPTaskGroupSnapProvider*	pHttpTGProv = NULL;
CHTTPTaskSnapProvider* pHttpTSProv = NULL;
CHTTPExecuterSnapProvider*	pHttpESProv = NULL;
CTaskGroupSnapProvider*	pTGSProv = NULL;
CTaskIdProvider	*pTIdProv = NULL;
CTaskSnapProvider*	pTSProv = NULL;
CTaskSnap2Provider*	pTS2Prov = NULL;
CTestExecuterSnapshotProvider*	pTESProv = NULL;
CVUSnapProvider* pVUSProv = NULL;
CFileControl* pFCHandler = NULL;
CTestRunHandler* pTRHandler = NULL;

unsigned long gulAddTestOutput = 1 ; // Number of lines sent at a time (AddTestOutput)

extern CCallManager* g_pCallManager ;
///////////////////////////////////////////////////////////////////////////////
// @func   This function initialises Name service cleaner
// @rdesc A standard Cyrano error code.
XMLVDK_EXP XMLVDKInit( CORBAServer* pServer, CCallManager* pCallManager, int nTraceLevel )
{
	char * pOpenSTA = getenv("OPENSTA_HOME");
	string OpenSTAPath ;
	if (pOpenSTA != NULL)
	{
		OpenSTAPath = pOpenSTA;
		if (pOpenSTA[strlen(pOpenSTA) - 1] != '\\')
			OpenSTAPath += '\\' ;
		string IniFile = OpenSTAPath + "Server\\Architecture.ini" ;
		gulAddTestOutput = GetPrivateProfileInt("XMLVDK010", "ADDTESTOUTPUT_LINES", 10, IniFile.c_str());
	}

	g_pCallManager = pCallManager ;
	g_Logger = new CLogger("XMLVDK010_Log.txt");
	g_Logger->SetLevel(nTraceLevel);
	g_Logger->WriteLogMsg(TRCLEV_NONE, "XMLVDK010 Loaded.");
	
	char buf[10];
	sprintf(buf,"%d",gulAddTestOutput);
	g_Logger->WriteLogMsg(TRCLEV_NONE, "AddTestOutput will be sent by batch of " + string(buf) + " lines.");

	sprintf(buf,"%d",nTraceLevel);
	g_Logger->WriteLogMsg(TRCLEV_NONE, "Trace Level = "+string(buf));

	ICRelay = new XMLRelayInjectorControlImpl(pServer);
	ISRelay = new XMLRelayInjectorStatusImpl(pServer);
	TMRelay = new XMLRelayTestManagerImpl(pServer);
	TRRelay = new XMLRelayTestRunImpl(pServer);
	FTNRelay = new XMLFileTransferNodeImpl(pServer);

	pICHandler = new CInjectorControlHandler(pServer);
	pISHandler = new CInjectorStatusHandler(pServer);
	pTMHandler = new CTestManagerHandler(pServer);
	pTRHandler = new CTestRunHandler(pServer);
	pFCHandler = new CFileControl(pServer, pCallManager->m_settings);

	pTEProv = new CTaskExecInfoProvider();
	pTGProv = new CTaskGroupExecInfoProvider();
	pHttpTGProv = new CHTTPTaskGroupSnapProvider();
	pHttpTSProv = new CHTTPTaskSnapProvider();
	pHttpESProv = new CHTTPExecuterSnapProvider();
	pTGSProv = new CTaskGroupSnapProvider();
	pTIdProv = new CTaskIdProvider();
	pTSProv = new CTaskSnapProvider();
	pTS2Prov = new CTaskSnap2Provider();
	pTESProv = new CTestExecuterSnapshotProvider();
	pVUSProv = new CVUSnapProvider();

	pCallManager->AddHandler("INJECTORCONTROL",pICHandler);
	pCallManager->AddHandler("INJECTORSTATUS",pISHandler);
	pCallManager->AddHandler("TESTMANAGER",pTMHandler);
	pCallManager->AddHandler("TESTRUN",pTRHandler);
	pCallManager->AddHandler("FILECONTROL",pFCHandler);
	pCallManager->AddDataProvider(pTEProv);
	pCallManager->AddDataProvider(pTGProv);
	pCallManager->AddDataProvider(pHttpTGProv);
	pCallManager->AddDataProvider(pHttpTSProv);
	pCallManager->AddDataProvider(pHttpESProv);
	pCallManager->AddDataProvider(pTGSProv);
	pCallManager->AddDataProvider(pTIdProv);
	pCallManager->AddDataProvider(pTSProv);
	pCallManager->AddDataProvider(pTS2Prov);
	pCallManager->AddDataProvider(pTESProv);
	pCallManager->AddDataProvider(pVUSProv);

	return 0;
}

///////////////////////////////////////////////////////////////////////////////
// @func  This function stops the  Name service cleaner
// @rdesc A standard Cyrano error code.
XMLVDK_EXP XMLVDKShutdown()
{
	delete pTEProv;
	delete pTGProv;
	delete pFCHandler;
	delete pTMHandler;
	delete pTRHandler;
	delete pICHandler;
	delete pISHandler;
	delete g_Logger;
	delete ICRelay;
	delete ISRelay;
	delete TMRelay;
	delete TRRelay;
	return 0;
}

extern "C"
BOOL WINAPI DllMain(HINSTANCE hInstance, DWORD dwReason, LPVOID /*lpReserved*/)
{
    if (dwReason == DLL_PROCESS_ATTACH)
    {
        CyrtoolsInitialize();
    }
    else if (dwReason == DLL_PROCESS_DETACH)
    {
        CyrtoolsUninitialize();
    }
    return TRUE;
}
