/* Copyright (C) 2000-2001 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

#include "stdafx.h"
#include "XMLRelayVarFactory_i.h"
#include "..\XMLVDK010Lib\XMLFileTransferNode.h"
#include <CECInetInfo.h>
#include "xmlrpc.h"
#include <string>
#include <iostream>
#include <fstream>
using namespace std;
#include "..\include\Logger.h"
#include "VarFactoryHandler.h"
#include "tofprmdef.h"
#include "OstaTraceDef.h"
#include "ReposEnum.h"

extern CLogger* g_Logger;
extern CVarFactoryHandler* VFHandler ;

const string EMPTY = "FAKEEMPTYSTRING";

XMLRelayVarFactoryImpl::XMLRelayVarFactoryImpl(CORBAServer* corba)
{
    // Get IP address (with dots replaced by underscores) of daemon host.
    m_registered_name="WebRelay";
    m_corba=corba;

	g_Logger->WriteLogMsg(TRCLEV_LOW, "RelayVF: Registering XMLRelayVarFactory interface");
	m_corba->ActivateObject(this);
    m_corba->NameObject( this->_this(),XMLRelayVarFactory::CLASS_NAME,m_registered_name.c_str());

	m_RelayMap = XMLRelayMap::_nil();
}

XMLRelayVarFactoryImpl::~XMLRelayVarFactoryImpl()
{
	g_Logger->WriteLogMsg(TRCLEV_LOW, "RelayVF: Unregistering XMLRelayVarFactory interface - destructor");
    m_corba->UnNameObject( XMLRelayVarFactory::CLASS_NAME,m_registered_name.c_str());
}

CORBA::Long  XMLRelayVarFactoryImpl::RegisterScript(const char* strTargetMachine,const char* strScriptName)
{
	g_Logger->WriteLogMsg(TRCLEV_LOW, "RelayVF: RegisterScript: Relaying RegisterScript request for host "+string(strTargetMachine));

	if (CORBA::is_nil(m_RelayMap))
	{
		CORBA::Object_ptr obj = m_corba->FindObjectWithName(XMLRelayMap::CLASS_NAME, m_registered_name.c_str());
		m_RelayMap = XMLRelayMap::_narrow(obj);
	}

	RelayDef def;	
	if (!m_RelayMap->GetRelayForAlias(strTargetMachine, def))
	{
		g_Logger->WriteLogMsg(TRCLEV_LOW, "RelayVF: RegisterScript: No relay server defined for this host");
		return -1;
	}
	else
	{
		g_Logger->WriteLogMsg(TRCLEV_HIGH, "RelayVF: RegisterScript: Relay server is "+string(def.strHost));
	}

	
	CHTTPTransport transport(string(def.strHost),string(def.strExtPath),def.lPort,string(def.strUser),string(def.strPass), def.bUseSSL);

	CXMLRequest req ;
	req.SetMethodName("VARFACTORY.RegisterScript");
	req.SetTransport(transport);
	req.AddParameter(new CXMLString(strTargetMachine)); 
	req.AddParameter(new CXMLString(strScriptName));
	CXMLResponse* resp = req.SendRequest();
	if (resp->IsError())
	{
		g_Logger->WriteLogMsg(TRCLEV_LOW, "RelayVF: Relay Error: "+resp->PrintError());
		delete resp;
		return -1;
	}

	long lReply =-1;
	if (resp->m_params.GetSize() == 1 && resp->m_params.GetAt(0)->GetType() == "i4")
	{
		CXMLLong* pLongVal = (CXMLLong*)resp->m_params.GetAt(0);
		lReply = pLongVal->GetValue();
	}

	delete resp;

    return lReply;
}

CORBA::Long  XMLRelayVarFactoryImpl::DeregisterScript(const char* strTargetMachine,const char* strScriptName, CORBA::Boolean bKeepAlive)
{
	g_Logger->WriteLogMsg(TRCLEV_LOW, "RelayVF: DeregisterScript: Relaying DeregisterScript request for host "+string(strTargetMachine));

	if (CORBA::is_nil(m_RelayMap))
	{
		CORBA::Object_ptr obj = m_corba->FindObjectWithName(XMLRelayMap::CLASS_NAME, m_registered_name.c_str());
		m_RelayMap = XMLRelayMap::_narrow(obj);
	}

	RelayDef def;	
	if (!m_RelayMap->GetRelayForAlias(strTargetMachine, def))
	{
		g_Logger->WriteLogMsg(TRCLEV_LOW, "RelayVF: DeregisterScript: No relay server defined for this host");
		return -1;
	}
	else
	{
		g_Logger->WriteLogMsg(TRCLEV_HIGH, "RelayVF: DeregisterScript: Relay server is "+string(def.strHost));
	}

	
	CHTTPTransport transport(string(def.strHost),string(def.strExtPath),def.lPort,string(def.strUser),string(def.strPass), def.bUseSSL);

	CXMLRequest req;
	req.SetMethodName("VARFACTORY.DeregisterScript");
	req.SetTransport(transport);
	req.AddParameter(new CXMLString(strTargetMachine));
	req.AddParameter(new CXMLString(strScriptName));
	req.AddParameter(new CXMLBool(bKeepAlive));
	CXMLResponse* resp = req.SendRequest();
	if (resp->IsError())
	{
		g_Logger->WriteLogMsg(TRCLEV_LOW, "RelayVF: Relay Error: "+resp->PrintError());
		delete resp;
		return -1;
	}

	long lReply =-1;
	if (resp->m_params.GetSize() == 1 && resp->m_params.GetAt(0)->GetType() == "i4")
	{
		CXMLLong* pLongVal = (CXMLLong*)resp->m_params.GetAt(0);
		lReply = pLongVal->GetValue();
	}

	delete resp;

    return lReply;
}

CORBA::Boolean  XMLRelayVarFactoryImpl::ClearAll(const char* strTargetMachine)
{
	// Clear the local variables
	// This function is called from the XMLVDL010, function OnInjectorStopped on the same
	// machine.
	// Indeed, the TestManager is not able to call this remote function.
	if (strTargetMachine[0] == '\0')
	{
		VFHandler->ClearAll("");
		return true ;
	}

	g_Logger->WriteLogMsg(TRCLEV_LOW, "RelayVF: ClearAll: Relaying ClearAll request for host "+string(strTargetMachine));

	if (CORBA::is_nil(m_RelayMap))
	{
		CORBA::Object_ptr obj = m_corba->FindObjectWithName(XMLRelayMap::CLASS_NAME, m_registered_name.c_str());
		m_RelayMap = XMLRelayMap::_narrow(obj);
	}

	RelayDef def;	
	if (!m_RelayMap->GetRelayForAlias(strTargetMachine, def))
	{
		g_Logger->WriteLogMsg(TRCLEV_LOW, "RelayVF: ClearAll: No relay server defined for this host");
		return false;
	}
	else
	{
		g_Logger->WriteLogMsg(TRCLEV_HIGH, "RelayVF: ClearAll: Relay server is "+string(def.strHost));
	}

	
	CHTTPTransport transport(string(def.strHost),string(def.strExtPath),def.lPort,string(def.strUser),string(def.strPass), def.bUseSSL);

	CXMLRequest req ;
	req.SetMethodName("VARFACTORY.ClearAll");
	req.SetTransport(transport);
	req.AddParameter(new CXMLString(strTargetMachine));
	
	CXMLResponse* resp = req.SendRequest();
	if (resp->IsError())
	{
		g_Logger->WriteLogMsg(TRCLEV_LOW, "RelayVF: Relay Error: "+resp->PrintError());
		delete resp;
		return false;
	}

	CORBA::Boolean bReply =false;
	if (resp->m_params.GetSize() == 1 && resp->m_params.GetAt(0)->GetType() == "boolean")
	{
		CXMLBool* pVal = (CXMLBool*)resp->m_params.GetAt(0);
		bReply = pVal->GetValue();
	}

	delete resp;

    return bReply;
}

CORBA::Boolean XMLRelayVarFactoryImpl::CreateMutex(const char* strTargetMachine,const char* strName)
{
	CXMLResponse* resp = NULL ;
	
	char *pEndMutexName = (char *)strName + strlen(strName) - strlen(".LOCAL");
	if (strcmp(pEndMutexName , ".LOCAL") == 0)
	{
		g_Logger->WriteLogMsg(TRCLEV_LOW, "RelayVF: CreateMutex: create a LOCAL mutex.");
		resp = VFHandler->CreateMutex("",strName);
	}
	else
	{
		g_Logger->WriteLogMsg(TRCLEV_LOW, "RelayVF: CreateMutex: Relaying CreateMutex request for host "+string(strTargetMachine));

		if (CORBA::is_nil(m_RelayMap))
		{
			CORBA::Object_ptr obj = m_corba->FindObjectWithName(XMLRelayMap::CLASS_NAME, m_registered_name.c_str());
			m_RelayMap = XMLRelayMap::_narrow(obj);
		}

		RelayDef def;	
		if (!m_RelayMap->GetRelayForAlias(strTargetMachine, def))
		{
			g_Logger->WriteLogMsg(TRCLEV_LOW, "RelayVF: CreateMutex: No relay server defined for this host");
			return false;
		}
		else
		{
			g_Logger->WriteLogMsg(TRCLEV_HIGH, "RelayVF: CreateMutex: Relay server is "+string(def.strHost));
		}

		
		CHTTPTransport transport(string(def.strHost),string(def.strExtPath),def.lPort,string(def.strUser),string(def.strPass), def.bUseSSL);

		CXMLRequest req ;
		req.SetMethodName("VARFACTORY.CreateMutex");
		req.SetTransport(transport);
		req.AddParameter(new CXMLString(strTargetMachine));
		req.AddParameter(new CXMLString(strName));
		
		resp = req.SendRequest();
	}
	if (resp->IsError())
	{
		g_Logger->WriteLogMsg(TRCLEV_LOW, "RelayVF: Relay Error: "+resp->PrintError());
		delete resp;
		return false;
	}

	CORBA::Boolean bReply =false;
	if (resp->m_params.GetSize() == 1 && resp->m_params.GetAt(0)->GetType() == "boolean")
	{
		CXMLBool* pVal = (CXMLBool*)resp->m_params.GetAt(0);
		bReply = pVal->GetValue();
	}

	delete resp;

    return bReply;
}

CORBA::Boolean XMLRelayVarFactoryImpl::CreateSimpleSemaphore(const char* strTargetMachine,const char* strName)
{
	CXMLResponse* resp = NULL ;
	
	char *pEndSemName = (char *)strName + strlen(strName) - strlen(".LOCAL");
	if (strcmp(pEndSemName , ".LOCAL") == 0)
	{
		g_Logger->WriteLogMsg(TRCLEV_LOW, "RelayVF: CreateMutex: create a LOCAL semaphore.");
		resp = VFHandler->CreateSimpleSemaphore("",strName);
	}
	else
	{
		g_Logger->WriteLogMsg(TRCLEV_LOW, "RelayVF: CreateSimpleSemaphore: Relaying CreateSimpleSemaphore request for host "+string(strTargetMachine));

		if (CORBA::is_nil(m_RelayMap))
		{
			CORBA::Object_ptr obj = m_corba->FindObjectWithName(XMLRelayMap::CLASS_NAME, m_registered_name.c_str());
			m_RelayMap = XMLRelayMap::_narrow(obj);
		}

		RelayDef def;	
		if (!m_RelayMap->GetRelayForAlias(strTargetMachine, def))
		{
			g_Logger->WriteLogMsg(TRCLEV_LOW, "RelayVF: CreateSimpleSemaphore: No relay server defined for this host");
			return false;
		}
		else
		{
			g_Logger->WriteLogMsg(TRCLEV_HIGH, "RelayVF: CreateSimpleSemaphore: Relay server is "+string(def.strHost));
		}

		
		CHTTPTransport transport(string(def.strHost),string(def.strExtPath),def.lPort,string(def.strUser),string(def.strPass), def.bUseSSL);

		CXMLRequest req ;
		req.SetMethodName("VARFACTORY.CreateSimpleSemaphore");
		req.SetTransport(transport);
		req.AddParameter(new CXMLString(strTargetMachine));
		req.AddParameter(new CXMLString(strName));

		resp = req.SendRequest();
	}
	if (resp->IsError())
	{
		g_Logger->WriteLogMsg(TRCLEV_LOW, "RelayVF: Relay Error: "+resp->PrintError());
		delete resp;
		return false;
	}

	CORBA::Boolean bReply =false;
	if (resp->m_params.GetSize() == 1 && resp->m_params.GetAt(0)->GetType() == "boolean")
	{
		CXMLBool* pVal = (CXMLBool*)resp->m_params.GetAt(0);
		bReply = pVal->GetValue();
	}

	delete resp;

    return bReply;
}

CORBA::Long XMLRelayVarFactoryImpl::GetVariable(const char* strTargetMachine, const char* strName, CORBA::Long size, const char* strFile, const char* strFileid,
					  const char* strDimensions, const char* strValues, CORBA::Long options, CORBA::Long seed, const char* strScriptname)
{
	CXMLResponse* resp = NULL ;
	
	if (!(options & VOTESTWIDE))
	{
		if (!VFHandler->IsAlreadyDefined(strName, strScriptname))
		{
			// If not a TEST-WIDE variable, must get the file from the remote
			// repository and save it in the local LAN repository.
			if (options &  VSFILE)
			{
				CORBA::Object_var objFile = m_corba->FindObjectWithName(XMLFileTransferNode::CLASS_NAME,"WebRelay");
				XMLFileTransferNode_var varFile = XMLFileTransferNode::_narrow(objFile);
				string file = strFile ;
				file += ".fvr";
				varFile->GetRemoteFileURL(strTargetMachine, REPOS_DATA, file.c_str());
			}
		}
		resp = VFHandler->GetVariable("",strName,size,strFile, strFileid,strDimensions, strValues, options, seed, strScriptname);
	}
	else
	{
		g_Logger->WriteLogMsg(TRCLEV_LOW, "RelayVF: GetVariable: Relaying GetVariable request for host "+string(strTargetMachine));

		if (CORBA::is_nil(m_RelayMap))
		{
			CORBA::Object_ptr obj = m_corba->FindObjectWithName(XMLRelayMap::CLASS_NAME, m_registered_name.c_str());
			m_RelayMap = XMLRelayMap::_narrow(obj);
		}

		RelayDef def;	
		if (!m_RelayMap->GetRelayForAlias(strTargetMachine, def))
		{
			g_Logger->WriteLogMsg(TRCLEV_LOW, "RelayVF: GetVariable: No relay server defined for this host");
			return -1;
		}
		else
		{
			g_Logger->WriteLogMsg(TRCLEV_HIGH, "RelayVF: GetVariable: Relay server is "+string(def.strHost));
		}

		//check for empty strings and replace them with the special fake string as the xml parse can't cope with ""
		CString temp;
		temp=strFile;
		if(temp.IsEmpty())
		{
			strFile=EMPTY.c_str();
		}
		temp=strFileid;
		if(temp.IsEmpty())
		{
			strFileid=EMPTY.c_str();
		}
		temp=strDimensions;
		if(temp.IsEmpty())
		{
			strDimensions=EMPTY.c_str();
		}
		temp=strValues;
		if(temp.IsEmpty())
		{
			strValues=EMPTY.c_str();
		}
		temp=strScriptname;
		if(temp.IsEmpty())
		{
			strScriptname=EMPTY.c_str();
		}
		// end of conversion

		CHTTPTransport transport(string(def.strHost),string(def.strExtPath),def.lPort,string(def.strUser),string(def.strPass), def.bUseSSL);


		CXMLRequest req ;
		req.SetMethodName("VARFACTORY.GetVariable");
		req.SetTransport(transport);
		req.AddParameter(new CXMLString(strTargetMachine));
		req.AddParameter(new CXMLString(strName));
		req.AddParameter(new CXMLLong(size));
		req.AddParameter(new CXMLString(strFile));
		req.AddParameter(new CXMLString(strFileid));
		req.AddParameter(new CXMLString(strDimensions));
		req.AddParameter(new CXMLBase64(strValues));
		req.AddParameter(new CXMLLong(options));
		req.AddParameter(new CXMLLong(seed));
		req.AddParameter(new CXMLString(strScriptname));
		resp = req.SendRequest();
	}
	if (resp->IsError())
	{
		g_Logger->WriteLogMsg(TRCLEV_LOW, "RelayVF: Relay Error: "+resp->PrintError());
		delete resp;
		return -1;
	}

	CORBA::Long lReply = -1;
	if (resp->m_params.GetSize() == 1 && resp->m_params.GetAt(0)->GetType() == "i4")
	{
		CXMLLong* pVal = (CXMLLong*)resp->m_params.GetAt(0);
		lReply = pVal->GetValue();
	}

	delete resp;
	return lReply;
}
		
CORBA::Long XMLRelayVarFactoryImpl::TPMutex_lock(const char* strTargetMachine, const char* mutexName)
{
	CXMLResponse * resp = NULL ;

	char *pEndMutexName = (char *)mutexName + strlen(mutexName) - strlen(".LOCAL");
	if (strcmp(pEndMutexName , ".LOCAL") == 0)
	{
		g_Logger->WriteLogMsg(TRCLEV_LOW, "RelayVF: TPMutex_lock: lock a LOCAL mutex.");
		resp = VFHandler->TPMutex_lock("",mutexName);
	}
	else
	{
		g_Logger->WriteLogMsg(TRCLEV_LOW, "RelayVF: TPMutex_lock: Relaying TPMutex_lock request for host "+string(strTargetMachine));

		if (CORBA::is_nil(m_RelayMap))
		{
			CORBA::Object_ptr obj = m_corba->FindObjectWithName(XMLRelayMap::CLASS_NAME, m_registered_name.c_str());
			m_RelayMap = XMLRelayMap::_narrow(obj);
		}

		RelayDef def;	
		if (!m_RelayMap->GetRelayForAlias(strTargetMachine, def))
		{
			g_Logger->WriteLogMsg(TRCLEV_LOW, "RelayVF: TPMutex_lock: No relay server defined for this host");
			return -1;
		}
		else
		{
			g_Logger->WriteLogMsg(TRCLEV_HIGH, "RelayVF: TPMutex_lock: Relay server is "+string(def.strHost));
		}

		CHTTPTransport transport(string(def.strHost),string(def.strExtPath),def.lPort,string(def.strUser),string(def.strPass), def.bUseSSL);

		CXMLRequest req ;
		req.SetMethodName("VARFACTORY.TPMutex_lock");
		req.SetTransport(transport);
		req.AddParameter(new CXMLString(strTargetMachine));
		req.AddParameter(new CXMLString(mutexName));
		resp = req.SendRequest();
	}
	if (resp->IsError())
	{
		g_Logger->WriteLogMsg(TRCLEV_LOW, "RelayVF: Relay Error: "+resp->PrintError());
		delete resp;
		return -1;
	}

	long lReply =-1;
	if (resp->m_params.GetSize() == 1 && resp->m_params.GetAt(0)->GetType() == "i4")
	{
		CXMLLong* pLongVal = (CXMLLong*)resp->m_params.GetAt(0);
		lReply = pLongVal->GetValue();
	}

	delete resp;

    return lReply;
}

CORBA::Boolean XMLRelayVarFactoryImpl::TPMutex_unlock(const char* strTargetMachine, const char* mutexName, CORBA::Long mutexid)
{
	CXMLResponse * resp = NULL ;

	char *pEndMutexName = (char *)mutexName + strlen(mutexName) - strlen(".LOCAL");
	if (strcmp(pEndMutexName , ".LOCAL") == 0)
	{
		g_Logger->WriteLogMsg(TRCLEV_LOW, "RelayVF: TPMutex_unlock: unlock a LOCAL mutex.");
		resp = VFHandler->TPMutex_unlock("",mutexName, mutexid);
	}
	else
	{
		g_Logger->WriteLogMsg(TRCLEV_LOW, "RelayVF: TPMutex_unlock: Relaying TPMutex_unlock request for host "+string(strTargetMachine));

		if (CORBA::is_nil(m_RelayMap))
		{
			CORBA::Object_ptr obj = m_corba->FindObjectWithName(XMLRelayMap::CLASS_NAME, m_registered_name.c_str());
			m_RelayMap = XMLRelayMap::_narrow(obj);
		}

		RelayDef def;	
		if (!m_RelayMap->GetRelayForAlias(strTargetMachine, def))
		{
			g_Logger->WriteLogMsg(TRCLEV_LOW, "RelayVF: TPMutex_unlock: No relay server defined for this host");
			return false;
		}
		else
		{
			g_Logger->WriteLogMsg(TRCLEV_HIGH, "RelayVF: TPMutex_unlock: Relay server is "+string(def.strHost));
		}

		
		CHTTPTransport transport(string(def.strHost),string(def.strExtPath),def.lPort,string(def.strUser),string(def.strPass), def.bUseSSL);

		CXMLRequest req ;
		req.SetMethodName("VARFACTORY.TPMutex_unlock");
		req.SetTransport(transport);
		req.AddParameter(new CXMLString(strTargetMachine));
		req.AddParameter(new CXMLString(mutexName));
		req.AddParameter(new CXMLLong(mutexid));
		
		resp = req.SendRequest();
	}
	if (resp->IsError())
	{
		g_Logger->WriteLogMsg(TRCLEV_LOW, "RelayVF: Relay Error: "+resp->PrintError());
		delete resp;
		return false;
	}

	CORBA::Boolean bReply =false;
	if (resp->m_params.GetSize() == 1 && resp->m_params.GetAt(0)->GetType() == "boolean")
	{
		CXMLBool* pVal = (CXMLBool*)resp->m_params.GetAt(0);
		bReply = pVal->GetValue();
	}

	delete resp;

    return bReply;
}

CORBA::Long XMLRelayVarFactoryImpl::TPMutex_trylock(const char* strTargetMachine, const char* mutexName)
{
	CXMLResponse * resp = NULL ;

	char *pEndMutexName = (char *)mutexName + strlen(mutexName) - strlen(".LOCAL");
	if (strcmp(pEndMutexName , ".LOCAL") == 0)
	{
		g_Logger->WriteLogMsg(TRCLEV_LOW, "RelayVF: TPMutex_trylock: trylock a LOCAL mutex.");
		resp = VFHandler->TPMutex_trylock("",mutexName);
	}
	else
	{
		g_Logger->WriteLogMsg(TRCLEV_LOW, "RelayVF: TPMutex_trylock: Relaying TPMutex_trylock request for host "+string(strTargetMachine));

		if (CORBA::is_nil(m_RelayMap))
		{
			CORBA::Object_ptr obj = m_corba->FindObjectWithName(XMLRelayMap::CLASS_NAME, m_registered_name.c_str());
			m_RelayMap = XMLRelayMap::_narrow(obj);
		}

		RelayDef def;	
		if (!m_RelayMap->GetRelayForAlias(strTargetMachine, def))
		{
			g_Logger->WriteLogMsg(TRCLEV_LOW, "RelayVF: TPMutex_trylock: No relay server defined for this host");
			return -1;
		}
		else
		{
			g_Logger->WriteLogMsg(TRCLEV_HIGH, "RelayVF: TPMutex_trylock: Relay server is "+string(def.strHost));
		}

		CHTTPTransport transport(string(def.strHost),string(def.strExtPath),def.lPort,string(def.strUser),string(def.strPass), def.bUseSSL);

		CXMLRequest req ;
		req.SetMethodName("VARFACTORY.TPMutex_trylock");
		req.SetTransport(transport);
		req.AddParameter(new CXMLString(strTargetMachine));
		req.AddParameter(new CXMLString(mutexName));
		resp = req.SendRequest();
	}
	if (resp->IsError())
	{
		g_Logger->WriteLogMsg(TRCLEV_LOW, "RelayVF: Relay Error: "+resp->PrintError());
		delete resp;
		return -1;
	}

	long lReply =-1;
	if (resp->m_params.GetSize() == 1 && resp->m_params.GetAt(0)->GetType() == "i4")
	{
		CXMLLong* pLongVal = (CXMLLong*)resp->m_params.GetAt(0);
		lReply = pLongVal->GetValue();
	}

	delete resp;

    return lReply;
}

CORBA::Boolean XMLRelayVarFactoryImpl::SimpleSemaphore_acquire(const char* strTargetMachine, const char* semName)
{
	CXMLResponse * resp = NULL ;

	char *pEndSemName = (char *)semName + strlen(semName) - strlen(".LOCAL");
	if (strcmp(pEndSemName , ".LOCAL") == 0)
	{
		g_Logger->WriteLogMsg(TRCLEV_LOW, "RelayVF: SimpleSemaphore_acquire: acquire a LOCAL semaphore.");
		resp = VFHandler->TPSemaphore_acquire("",semName);
	}
	else
	{
		g_Logger->WriteLogMsg(TRCLEV_LOW, "RelayVF: SimpleSemaphore_acquire: Relaying SimpleSemaphore_acquire request for host "+string(strTargetMachine));

		if (CORBA::is_nil(m_RelayMap))
		{
			CORBA::Object_ptr obj = m_corba->FindObjectWithName(XMLRelayMap::CLASS_NAME, m_registered_name.c_str());
			m_RelayMap = XMLRelayMap::_narrow(obj);
		}

		RelayDef def;	
		if (!m_RelayMap->GetRelayForAlias(strTargetMachine, def))
		{
			g_Logger->WriteLogMsg(TRCLEV_LOW, "RelayVF: SimpleSemaphore_acquire: No relay server defined for this host");
			return false;
		}
		else
		{
			g_Logger->WriteLogMsg(TRCLEV_HIGH, "RelayVF: SimpleSemaphore_acquire: Relay server is "+string(def.strHost));
		}

		
		CHTTPTransport transport(string(def.strHost),string(def.strExtPath),def.lPort,string(def.strUser),string(def.strPass), def.bUseSSL);

		CXMLRequest req ;
		req.SetMethodName("VARFACTORY.SimpleSemaphore_acquire");
		req.SetTransport(transport);
		req.AddParameter(new CXMLString(strTargetMachine));
		req.AddParameter(new CXMLString(semName));
		
		resp = req.SendRequest();
	}
	if (resp->IsError())
	{
		g_Logger->WriteLogMsg(TRCLEV_LOW, "RelayVF: Relay Error: "+resp->PrintError());
		delete resp;
		return false;
	}

	CORBA::Boolean bReply =false;
	if (resp->m_params.GetSize() == 1 && resp->m_params.GetAt(0)->GetType() == "boolean")
	{
		CXMLBool* pVal = (CXMLBool*)resp->m_params.GetAt(0);
		bReply = pVal->GetValue();
	}

	delete resp;

    return bReply;
}

CORBA::Boolean XMLRelayVarFactoryImpl::SimpleSemaphore_release(const char* strTargetMachine, const char* semName)
{
	CXMLResponse * resp = NULL ;

	char *pEndSemName = (char *)semName + strlen(semName) - strlen(".LOCAL");
	if (strcmp(pEndSemName , ".LOCAL") == 0)
	{
		g_Logger->WriteLogMsg(TRCLEV_LOW, "RelayVF: SimpleSemaphore_acquire: release a LOCAL semaphore.");
		resp = VFHandler->TPSemaphore_release("",semName);
	}
	else
	{
		g_Logger->WriteLogMsg(TRCLEV_LOW, "RelayVF: SimpleSemaphore_release: Relaying SimpleSemaphore_release request for host "+string(strTargetMachine));

		if (CORBA::is_nil(m_RelayMap))
		{
			CORBA::Object_ptr obj = m_corba->FindObjectWithName(XMLRelayMap::CLASS_NAME, m_registered_name.c_str());
			m_RelayMap = XMLRelayMap::_narrow(obj);
		}

		RelayDef def;	
		if (!m_RelayMap->GetRelayForAlias(strTargetMachine, def))
		{
			g_Logger->WriteLogMsg(TRCLEV_LOW, "RelayVF: SimpleSemaphore_release: No relay server defined for this host");
			return false;
		}
		else
		{
			g_Logger->WriteLogMsg(TRCLEV_HIGH, "RelayVF: SimpleSemaphore_release: Relay server is "+string(def.strHost));
		}
		
		CHTTPTransport transport(string(def.strHost),string(def.strExtPath),def.lPort,string(def.strUser),string(def.strPass), def.bUseSSL);

		CXMLRequest req ;
		req.SetMethodName("VARFACTORY.SimpleSemaphore_release");
		req.SetTransport(transport);
		req.AddParameter(new CXMLString(strTargetMachine));
		req.AddParameter(new CXMLString(semName));
		
		resp = req.SendRequest();
	}
	if (resp->IsError())
	{
		g_Logger->WriteLogMsg(TRCLEV_LOW, "RelayVF: Relay Error: "+resp->PrintError());
		delete resp;
		return false;
	}

	CORBA::Boolean bReply =false;
	if (resp->m_params.GetSize() == 1 && resp->m_params.GetAt(0)->GetType() == "boolean")
	{
		CXMLBool* pVal = (CXMLBool*)resp->m_params.GetAt(0);
		bReply = pVal->GetValue();
	}

	delete resp;

    return bReply;
}

CORBA::Boolean XMLRelayVarFactoryImpl::SimpleSemaphore_tryacquire(const char* strTargetMachine, const char* semName)
{
	CXMLResponse * resp = NULL ;

	char *pEndSemName = (char *)semName + strlen(semName) - strlen(".LOCAL");
	if (strcmp(pEndSemName , ".LOCAL") == 0)
	{
		g_Logger->WriteLogMsg(TRCLEV_LOW, "RelayVF: SimpleSemaphore_acquire: try to acquire a LOCAL semaphore.");
		resp = VFHandler->TPSemaphore_tryacquire("",semName);
	}
	else
	{
		g_Logger->WriteLogMsg(TRCLEV_LOW, "RelayVF: SimpleSemaphore_tryacquire: Relaying SimpleSemaphore_tryacquire request for host "+string(strTargetMachine));

		if (CORBA::is_nil(m_RelayMap))
		{
			CORBA::Object_ptr obj = m_corba->FindObjectWithName(XMLRelayMap::CLASS_NAME, m_registered_name.c_str());
			m_RelayMap = XMLRelayMap::_narrow(obj);
		}

		RelayDef def;	
		if (!m_RelayMap->GetRelayForAlias(strTargetMachine, def))
		{
			g_Logger->WriteLogMsg(TRCLEV_LOW, "RelayVF: SimpleSemaphore_tryacquire: No relay server defined for this host");
			return false;
		}
		else
		{
			g_Logger->WriteLogMsg(TRCLEV_HIGH, "RelayVF: SimpleSemaphore_tryacquire: Relay server is "+string(def.strHost));
		}

		
		CHTTPTransport transport(string(def.strHost),string(def.strExtPath),def.lPort,string(def.strUser),string(def.strPass), def.bUseSSL);

		CXMLRequest req ;
		req.SetMethodName("VARFACTORY.SimpleSemaphore_tryacquire");
		req.SetTransport(transport);
		req.AddParameter(new CXMLString(strTargetMachine));
		req.AddParameter(new CXMLString(semName));
		
		resp = req.SendRequest();
	}
	if (resp->IsError())
	{
		g_Logger->WriteLogMsg(TRCLEV_LOW, "RelayVF: Relay Error: "+resp->PrintError());
		delete resp;
		return false;
	}

	CORBA::Boolean bReply =false;
	if (resp->m_params.GetSize() == 1 && resp->m_params.GetAt(0)->GetType() == "boolean")
	{
		CXMLBool* pVal = (CXMLBool*)resp->m_params.GetAt(0);
		bReply = pVal->GetValue();
	}

	delete resp;

    return bReply;
}

char* XMLRelayVarFactoryImpl::GlobalVariable_get_value(const char* strTargetMachine, const char* strVarName, const char* strExpression, long lVarId)
{
	CXMLResponse * resp = NULL ;
	resp = VFHandler->GlobalVariable_get_value("", strVarName, strExpression, lVarId);

	// If error, the variable is not local, should be TESTWIDE
	if (resp->IsError())
	{
		delete resp ;
		g_Logger->WriteLogMsg(TRCLEV_LOW, "RelayVF: GlobalVariable_get_value: Relaying GlobalVariable_get_value request for host "+string(strTargetMachine));

		if (CORBA::is_nil(m_RelayMap))
		{
			CORBA::Object_ptr obj = m_corba->FindObjectWithName(XMLRelayMap::CLASS_NAME, m_registered_name.c_str());
			m_RelayMap = XMLRelayMap::_narrow(obj);
		}

		RelayDef def;	
		if (!m_RelayMap->GetRelayForAlias(strTargetMachine, def))
		{
			g_Logger->WriteLogMsg(TRCLEV_LOW, "RelayVF: GlobalVariable_get_value: No relay server defined for this host");
			return CORBA::string_dup("");
		}
		else
		{
			g_Logger->WriteLogMsg(TRCLEV_HIGH, "RelayVF: GlobalVariable_get_value: Relay server is "+string(def.strHost));
		}

		//check for empty strings and replace them with the special fake string as the xml parser can't cope with ""
		CString temp;
		temp=strExpression;
		if(temp.IsEmpty())
		{
			strExpression=EMPTY.c_str();
		}
		//end of conversion

		CHTTPTransport transport(string(def.strHost),string(def.strExtPath),def.lPort,string(def.strUser),string(def.strPass), def.bUseSSL);

		CXMLRequest req ;
		req.SetMethodName("VARFACTORY.GlobalVariable_get_value");
		req.SetTransport(transport);
		req.AddParameter(new CXMLString(strTargetMachine));
		req.AddParameter(new CXMLString(strVarName));
		req.AddParameter(new CXMLString(strExpression));
		req.AddParameter(new CXMLLong(lVarId));
		resp = req.SendRequest();
		if (resp->IsError())
		{
			g_Logger->WriteLogMsg(TRCLEV_LOW, "RelayVF: Relay Error: "+resp->PrintError());
			delete resp;
			return CORBA::string_dup("");
		}
	}

	string strReply ="";
	if (resp->m_params.GetSize() == 1 && resp->m_params.GetAt(0)->GetType() == "base64")
	{
		CXMLBase64* pStrVal = (CXMLBase64*)resp->m_params.GetAt(0);
		strReply = pStrVal->GetValue();

		// conversion as XML parser cannot do ""
		if(strReply == EMPTY)
		{
			strReply="";
		}
		//end conversion
	}

	delete resp;

	g_Logger->WriteLogMsg(TRCLEV_HIGH, string("RelayVF: GlobalVariable_get_value: value of variable = \""+strReply+"\""));

    return CORBA::string_dup(strReply.c_str());
}
CORBA::Long XMLRelayVarFactoryImpl::GlobalVariable_set_value(const char* strTargetMachine, const char* strVarName, const char* strNewval, const char* strExpression, long lVarId)
{
	CXMLResponse * resp = NULL ;
	resp = VFHandler->GlobalVariable_set_value("", strVarName, strNewval, strExpression, lVarId);

	// If error, the variable is not local, should be TESTWIDE
	if (resp->IsError())
	{
		delete resp ;
		g_Logger->WriteLogMsg(TRCLEV_LOW, "RelayVF: GlobalVariable_set_value: Relaying GlobalVariable_set_value request for host "+string(strTargetMachine));

		if (CORBA::is_nil(m_RelayMap))
		{
			CORBA::Object_ptr obj = m_corba->FindObjectWithName(XMLRelayMap::CLASS_NAME, m_registered_name.c_str());
			m_RelayMap = XMLRelayMap::_narrow(obj);
		}

		RelayDef def;	
		if (!m_RelayMap->GetRelayForAlias(strTargetMachine, def))
		{
			g_Logger->WriteLogMsg(TRCLEV_LOW, "RelayVF: GlobalVariable_set_value: No relay server defined for this host");
			return -1;
		}
		else
		{
			g_Logger->WriteLogMsg(TRCLEV_HIGH, "RelayVF: GlobalVariable_set_value: Relay server is "+string(def.strHost));
		}

		//check for empty strings and replace them with the special fake string as the xml parse can't cope with ""
		CString temp;
		temp=strNewval;
		if(temp.IsEmpty())
		{
			strNewval=EMPTY.c_str();
		}
		temp=strExpression;
		if(temp.IsEmpty())
		{
			strExpression=EMPTY.c_str();
		}
		// end of conversion

		CHTTPTransport transport(string(def.strHost),string(def.strExtPath),def.lPort,string(def.strUser),string(def.strPass), def.bUseSSL);

		CXMLRequest req ;
		req.SetMethodName("VARFACTORY.GlobalVariable_set_value");
		req.SetTransport(transport);
		req.AddParameter(new CXMLString(strTargetMachine));
		req.AddParameter(new CXMLString(strVarName));
		req.AddParameter(new CXMLString(strNewval));
		req.AddParameter(new CXMLString(strExpression));
		req.AddParameter(new CXMLLong(lVarId));
		resp = req.SendRequest();
		if (resp->IsError())
		{
			g_Logger->WriteLogMsg(TRCLEV_LOW, "RelayVF: Relay Error: "+resp->PrintError());
			delete resp;
			return -1;
		}
	}

	long lReply =-1;
	if (resp->m_params.GetSize() == 1 && resp->m_params.GetAt(0)->GetType() == "i4")
	{
		CXMLLong* pLongVal = (CXMLLong*)resp->m_params.GetAt(0);
		lReply = pLongVal->GetValue();
	}

	delete resp;

    return lReply;
}
CORBA::Boolean XMLRelayVarFactoryImpl::GlobalVariable_next_value(const char* strTargetMachine, const char* strVarName, long lVarId)
{
	CXMLResponse * resp = NULL ;
	resp = VFHandler->GlobalVariable_next_value("", strVarName, lVarId);

	// If error, the variable is not local, should be TESTWIDE
	if (resp->IsError())
	{
		delete resp ;
		g_Logger->WriteLogMsg(TRCLEV_LOW, "RelayVF: GlobalVariable_next_value: Relaying GlobalVariable_next_value request for host "+string(strTargetMachine));

		if (CORBA::is_nil(m_RelayMap))
		{
			CORBA::Object_ptr obj = m_corba->FindObjectWithName(XMLRelayMap::CLASS_NAME, m_registered_name.c_str());
			m_RelayMap = XMLRelayMap::_narrow(obj);
		}

		RelayDef def;	
		if (!m_RelayMap->GetRelayForAlias(strTargetMachine, def))
		{
			g_Logger->WriteLogMsg(TRCLEV_LOW, "RelayVF: GlobalVariable_next_value: No relay server defined for this host");
			return false;
		}
		else
		{
			g_Logger->WriteLogMsg(TRCLEV_HIGH, "RelayVF: GlobalVariable_next_value: Relay server is "+string(def.strHost));
		}

		
		CHTTPTransport transport(string(def.strHost),string(def.strExtPath),def.lPort,string(def.strUser),string(def.strPass), def.bUseSSL);

		CXMLRequest req ;
		req.SetMethodName("VARFACTORY.GlobalVariable_next_value");
		req.SetTransport(transport);
		req.AddParameter(new CXMLString(strTargetMachine));
		req.AddParameter(new CXMLString(strVarName));
		req.AddParameter(new CXMLLong(lVarId));
		
		resp = req.SendRequest();
		if (resp->IsError())
		{
			g_Logger->WriteLogMsg(TRCLEV_LOW, "RelayVF: Relay Error: "+resp->PrintError());
			delete resp;
			return false;
		}
	}

	CORBA::Boolean bReply =false;
	if (resp->m_params.GetSize() == 1 && resp->m_params.GetAt(0)->GetType() == "boolean")
	{
		CXMLBool* pVal = (CXMLBool*)resp->m_params.GetAt(0);
		bReply = pVal->GetValue();
	}

	delete resp;

    return bReply;
}
CORBA::Boolean XMLRelayVarFactoryImpl::GlobalVariable_generate_value(const char* strTargetMachine, const char* strVarName, long lVarId)
{
	CXMLResponse * resp = NULL ;
	resp = VFHandler->GlobalVariable_generate_value("", strVarName, lVarId);

	// If error, the variable is not local, should be TESTWIDE
	if (resp->IsError())
	{
		delete resp ;
		g_Logger->WriteLogMsg(TRCLEV_LOW, "RelayVF: GlobalVariable_generate_value: Relaying GlobalVariable_generate_value request for host "+string(strTargetMachine));

		if (CORBA::is_nil(m_RelayMap))
		{
			CORBA::Object_ptr obj = m_corba->FindObjectWithName(XMLRelayMap::CLASS_NAME, m_registered_name.c_str());
			m_RelayMap = XMLRelayMap::_narrow(obj);
		}

		RelayDef def;	
		if (!m_RelayMap->GetRelayForAlias(strTargetMachine, def))
		{
			g_Logger->WriteLogMsg(TRCLEV_LOW, "RelayVF: GlobalVariable_generate_value: No relay server defined for this host");
			return false;
		}
		else
		{
			g_Logger->WriteLogMsg(TRCLEV_HIGH, "RelayVF: GlobalVariable_generate_value: Relay server is "+string(def.strHost));
		}

		
		CHTTPTransport transport(string(def.strHost),string(def.strExtPath),def.lPort,string(def.strUser),string(def.strPass), def.bUseSSL);

		CXMLRequest req ;
		req.SetMethodName("VARFACTORY.GlobalVariable_generate_value");
		req.SetTransport(transport);
		req.AddParameter(new CXMLString(strTargetMachine));
		req.AddParameter(new CXMLString(strVarName));
		req.AddParameter(new CXMLLong(lVarId));
		
		resp = req.SendRequest();
		if (resp->IsError())
		{
			g_Logger->WriteLogMsg(TRCLEV_LOW, "RelayVF: Relay Error: "+resp->PrintError());
			delete resp;
			return false;
		}
	}

	CORBA::Boolean bReply =false;
	if (resp->m_params.GetSize() == 1 && resp->m_params.GetAt(0)->GetType() == "boolean")
	{
		CXMLBool* pVal = (CXMLBool*)resp->m_params.GetAt(0);
		bReply = pVal->GetValue();
	}

	delete resp;

    return bReply;
}
CORBA::Boolean XMLRelayVarFactoryImpl::GlobalVariable_reset_seed(const char* strTargetMachine, const char* strVarName, long lVarId)
{
	CXMLResponse * resp = NULL ;
	resp = VFHandler->GlobalVariable_reset_seed("", strVarName, lVarId);

	// If error, the variable is not local, should be TESTWIDE
	if (resp->IsError())
	{
		delete resp ;
		g_Logger->WriteLogMsg(TRCLEV_LOW, "RelayVF: GlobalVariable_reset_seed: Relaying GlobalVariable_reset_seed request for host "+string(strTargetMachine));

		if (CORBA::is_nil(m_RelayMap))
		{
			CORBA::Object_ptr obj = m_corba->FindObjectWithName(XMLRelayMap::CLASS_NAME, m_registered_name.c_str());
			m_RelayMap = XMLRelayMap::_narrow(obj);
		}

		RelayDef def;	
		if (!m_RelayMap->GetRelayForAlias(strTargetMachine, def))
		{
			g_Logger->WriteLogMsg(TRCLEV_LOW, "RelayVF: GlobalVariable_reset_seed: No relay server defined for this host");
			return false;
		}
		else
		{
			g_Logger->WriteLogMsg(TRCLEV_HIGH, "RelayVF: GlobalVariable_reset_seed: Relay server is "+string(def.strHost));
		}

		
		CHTTPTransport transport(string(def.strHost),string(def.strExtPath),def.lPort,string(def.strUser),string(def.strPass), def.bUseSSL);

		CXMLRequest req ;
		req.SetMethodName("VARFACTORY.GlobalVariable_reset_seed");
		req.SetTransport(transport);
		req.AddParameter(new CXMLString(strTargetMachine));
		req.AddParameter(new CXMLString(strVarName));
		req.AddParameter(new CXMLLong(lVarId));
		
		resp = req.SendRequest();
		if (resp->IsError())
		{
			g_Logger->WriteLogMsg(TRCLEV_LOW, "RelayVF: Relay Error: "+resp->PrintError());
			delete resp;
			return false;
		}
	}

	CORBA::Boolean bReply =false;
	if (resp->m_params.GetSize() == 1 && resp->m_params.GetAt(0)->GetType() == "boolean")
	{
		CXMLBool* pVal = (CXMLBool*)resp->m_params.GetAt(0);
		bReply = pVal->GetValue();
	}

	delete resp;

    return bReply;
}

CORBA::Boolean XMLRelayVarFactoryImpl::DoesSimpleSemaphoreExist(const char* strTargetMachine, const char* strVarName)
{
	CXMLResponse * resp = NULL ;

	char *pEndSemName = (char *)strVarName + strlen(strVarName) - strlen(".LOCAL");
	if (strcmp(strVarName , ".LOCAL") == 0)
	{
		g_Logger->WriteLogMsg(TRCLEV_LOW, "RelayVF: DoesSimpleSemaphoreExist: LOCAL semaphore.");
		resp = VFHandler->DoesSimpleSemaphoreExist("",strVarName);
	}
	else
	{
		g_Logger->WriteLogMsg(TRCLEV_LOW, "RelayVF: DoesSimpleSemaphoreExist: Relaying DoesSimpleSemaphoreExist request for host "+string(strTargetMachine));

		if (CORBA::is_nil(m_RelayMap))
		{
			CORBA::Object_ptr obj = m_corba->FindObjectWithName(XMLRelayMap::CLASS_NAME, m_registered_name.c_str());
			m_RelayMap = XMLRelayMap::_narrow(obj);
		}

		RelayDef def;	
		if (!m_RelayMap->GetRelayForAlias(strTargetMachine, def))
		{
			g_Logger->WriteLogMsg(TRCLEV_LOW, "RelayVF: DoesSimpleSemaphoreExist: No relay server defined for this host");
			return false;
		}
		else
		{
			g_Logger->WriteLogMsg(TRCLEV_HIGH, "RelayVF: DoesSimpleSemaphoreExist: Relay server is "+string(def.strHost));
		}

		
		CHTTPTransport transport(string(def.strHost),string(def.strExtPath),def.lPort,string(def.strUser),string(def.strPass), def.bUseSSL);

		CXMLRequest req ;
		req.SetMethodName("VARFACTORY.DoesSimpleSemaphoreExist");
		req.SetTransport(transport);
		req.AddParameter(new CXMLString(strTargetMachine));
		req.AddParameter(new CXMLString(strVarName));
		
		resp = req.SendRequest();
	}
	if (resp->IsError())
	{
		g_Logger->WriteLogMsg(TRCLEV_LOW, "RelayVF: Relay Error: "+resp->PrintError());
		delete resp;
		return false;
	}

	CORBA::Boolean bReply =false;
	if (resp->m_params.GetSize() == 1 && resp->m_params.GetAt(0)->GetType() == "boolean")
	{
		CXMLBool* pVal = (CXMLBool*)resp->m_params.GetAt(0);
		bReply = pVal->GetValue();
	}

	delete resp;

    return bReply;
}

CORBA::Boolean XMLRelayVarFactoryImpl::DoesMutexExist(const char* strTargetMachine, const char* strVarName)
{
	CXMLResponse * resp = NULL ;

	char *pEndMutexName = (char *)strVarName + strlen(strVarName) - strlen(".LOCAL");
	if (strcmp(pEndMutexName , ".LOCAL") == 0)
	{
		g_Logger->WriteLogMsg(TRCLEV_LOW, "RelayVF: DoesMutexExist: DoesMutexExist on a LOCAL mutex.");
		resp = VFHandler->DoesMutexExist("",strVarName);
	}
	else
	{
		g_Logger->WriteLogMsg(TRCLEV_LOW, "RelayVF: DoesMutexExist: Relaying DoesMutexExist request for host "+string(strTargetMachine));

		if (CORBA::is_nil(m_RelayMap))
		{
			CORBA::Object_ptr obj = m_corba->FindObjectWithName(XMLRelayMap::CLASS_NAME, m_registered_name.c_str());
			m_RelayMap = XMLRelayMap::_narrow(obj);
		}

		RelayDef def;	
		if (!m_RelayMap->GetRelayForAlias(strTargetMachine, def))
		{
			g_Logger->WriteLogMsg(TRCLEV_LOW, "RelayVF: DoesMutexExist: No relay server defined for this host");
			return false;
		}
		else
		{
			g_Logger->WriteLogMsg(TRCLEV_HIGH, "RelayVF: DoesMutexExist: Relay server is "+string(def.strHost));
		}

		
		CHTTPTransport transport(string(def.strHost),string(def.strExtPath),def.lPort,string(def.strUser),string(def.strPass), def.bUseSSL);

		CXMLRequest req ;
		req.SetMethodName("VARFACTORY.DoesMutexExist");
		req.SetTransport(transport);
		req.AddParameter(new CXMLString(strTargetMachine));
		req.AddParameter(new CXMLString(strVarName));
		
		resp = req.SendRequest();
	}
	if (resp->IsError())
	{
		g_Logger->WriteLogMsg(TRCLEV_LOW, "RelayVF: Relay Error: "+resp->PrintError());
		delete resp;
		return false;
	}

	CORBA::Boolean bReply =false;
	if (resp->m_params.GetSize() == 1 && resp->m_params.GetAt(0)->GetType() == "boolean")
	{
		CXMLBool* pVal = (CXMLBool*)resp->m_params.GetAt(0);
		bReply = pVal->GetValue();
	}

	delete resp;

    return bReply;
}
