/* Copyright (C) 2000-2001 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

#include "stdafx.h"
#include "xmlpost.h"
#include "..\CallProcessor\XMLCallProcessor.h"
#include <sys/timeb.h>
#include <string>
using namespace std;

///////////////////////////////////////////////////////////////////////
// The one and only CWinApp object
// NOTE: You may remove this object if you alter your project to no
// longer use MFC in a DLL.

CWinApp theApp;

///////////////////////////////////////////////////////////////////////
// command-parsing map

BEGIN_PARSE_MAP(CXMLRPCExtension, CHttpServer)
	// TODO: insert your ON_PARSE_COMMAND() and 
	// ON_PARSE_COMMAND_PARAMS() here to hook up your commands.
	// For example:

	ON_PARSE_COMMAND(Default, CXMLRPCExtension, ITS_EMPTY)
	DEFAULT_PARSE_COMMAND(Default, CXMLRPCExtension)
END_PARSE_MAP(CXMLRPCExtension)


///////////////////////////////////////////////////////////////////////
// The one and only CXMLRPCExtension object

CXMLRPCExtension	theExtension;

CORBAServer	*		CXMLRPCExtension::m_pServer = NULL ;
FILE *				CXMLRPCExtension::m_pLogFile = NULL;
long                CXMLRPCExtension::m_lTraceLevel = 0 ;

///////////////////////////////////////////////////////////////////////
// CXMLRPCExtension implementation

CXMLRPCExtension::CXMLRPCExtension()
{
	if (m_pLogFile == NULL)
	{
		char * pOpenSTA = getenv("OPENSTA_HOME");
		string OpenSTAPath ;
		if (pOpenSTA != NULL)
		{
			OpenSTAPath = pOpenSTA;
			if (pOpenSTA[strlen(pOpenSTA) - 1] != '\\')
				OpenSTAPath += '\\' ;
			string IniFile = OpenSTAPath + "Server\\Architecture.ini" ;
			m_lTraceLevel = GetPrivateProfileInt("XMLPOST", "TRACELEVEL", 0, IniFile.c_str());

			string LogFile = OpenSTAPath + "Server\\Logs\\XmlPost.log";

			if ((m_pLogFile = fopen(LogFile.c_str(), "w+")) != NULL)
			{
				char zBuf[120] ;
				sprintf(zBuf, "Log file created : TraceLevel: %d", m_lTraceLevel);
				WriteLog(0, zBuf);
			}
		}
	}
	if (!m_pServer)
	{
		m_pServer = new CORBAServer( 0, NULL);

		if(m_lTraceLevel>0)
		{
			if(m_pServer)
			{
				WriteLog(0, "Server created successfully.");
			}
			else
			{
				WriteLog(0, "ERROR: Failed to create server."); 
			}
		}
	}
}

CXMLRPCExtension::~CXMLRPCExtension()
{
}

void CXMLRPCExtension::WriteLog(unsigned int uiLevel, const string & msg)
{
	if (m_pLogFile == NULL || uiLevel > (unsigned int)m_lTraceLevel)
		return ;

    struct timeb time ;
    ftime (&time) ;
    unsigned long ulSeconds = (unsigned long) time.time ;
    char time_buffer[256];
    size_t len;
    len=strftime(time_buffer,256,"%c",localtime((time_t*) &ulSeconds));
	fprintf(m_pLogFile, "%s, %s\r\n", time_buffer, msg.c_str()) ;
	fflush(m_pLogFile);
}

BOOL CXMLRPCExtension::GetExtensionVersion(HSE_VERSION_INFO* pVer)
{
	WriteLog(10, "CXMLRPCExtension::GetExtensionVersion");

	// Call default implementation for initialization
	CHttpServer::GetExtensionVersion(pVer);

	// Load description string
	TCHAR sz[HSE_MAX_EXT_DLL_NAME_LEN+1];
	ISAPIVERIFY(::LoadString(AfxGetResourceHandle(),
			IDS_SERVER, sz, HSE_MAX_EXT_DLL_NAME_LEN));
	_tcscpy(pVer->lpszExtensionDesc, sz);
	return TRUE;
}

BOOL CXMLRPCExtension::TerminateExtension(DWORD dwFlags)
{
	// extension is being terminated
	//TODO: Clean up any per-instance resources
	return TRUE;
}

DWORD CXMLRPCExtension::HttpExtensionProc( EXTENSION_CONTROL_BLOCK *pECB )
{
	// get the data from the form 
	if( 0 == stricmp(pECB->lpszMethod, "post") )
	{
		char zBuf[200];
		// Start processing of input information here. 
		CHAR* lpszTemp = (CHAR *)LocalAlloc( LPTR, pECB->cbTotalBytes + 1); 
		if( NULL == lpszTemp ) 
			return HSE_STATUS_ERROR; 

		// do not always trust the compiler to zero the memory 
		memset(lpszTemp, '\0', pECB->cbTotalBytes + 1); 
		strncpy(lpszTemp, (CHAR *)pECB->lpbData, pECB->cbAvailable ); 
		unsigned long cbQuery = pECB->cbTotalBytes - pECB->cbAvailable; 

		WriteLog(10, "=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-="); 
		sprintf(zBuf, "Got a POST with size : %d, %d.", pECB->cbTotalBytes, pECB->cbAvailable);
		WriteLog(10, zBuf);

		unsigned long cbQueryTmp = 0 ;
		if( cbQuery > 0 ) 
		{ 
			unsigned long ulAlreadyRead = 0 ;
			do
			{
				cbQueryTmp = cbQuery ;
				pECB->ReadClient(pECB->ConnID, (LPVOID)(lpszTemp + pECB->cbAvailable + ulAlreadyRead), &cbQuery); 
				if (m_lTraceLevel > 40)
				{
					sprintf(zBuf, "Read : %d/%d.", cbQuery, cbQueryTmp);
					WriteLog(10, zBuf);
				}
				if (cbQuery < cbQueryTmp)
				{ 
					ulAlreadyRead += cbQuery ;
					cbQuery = cbQueryTmp - cbQuery ;
				}
				else
					break ;
			}
			while (1);

		} 
	
		string strCall = lpszTemp;
		LocalFree(lpszTemp);
		strCall.resize(pECB->cbTotalBytes);

		WriteLog(40, strCall.c_str());

		try
		{
			CORBA::Object_var obj = m_pServer->FindObjectWithName(XMLCallProcessor::CLASS_NAME, "WebRelay");
			
			WriteLog(10, "Narrow CallProcessor.");
			
			XMLCallProcessor_ptr CP = XMLCallProcessor::_narrow(obj);
			if (CORBA::is_nil(CP))
			{
				WriteLog(0, "ERROR: CallProcessor is NULL.");
			}

			WriteLog(10, "Forwarding Call...");

			string strTmp = CP->ProcessCall(strCall.c_str());
			sprintf(zBuf, "Content-Type: text/xml\r\nContent-Length: %d\r\n\r\n", strTmp.size());
			string strReply = zBuf ;
	
			DWORD size=strReply.length();
	
			WriteLog(10, "Send Reply.");
			WriteLog(40, (strReply + strTmp).c_str());
			if ( !pECB->ServerSupportFunction( 
					pECB->ConnID,
		            HSE_REQ_SEND_RESPONSE_HEADER,
		             "200 OK",
		            &size,
		            (LPDWORD) strReply.c_str() ))
			{
				WriteLog(0, "ERROR: Send Reply Failed "+ string(strReply.c_str()));
				return HSE_STATUS_ERROR;
			}

			unsigned long lSize = strTmp.size() ;
			if ( !pECB->WriteClient(
					pECB->ConnID,
					(LPVOID)strTmp.c_str(),
					&lSize,
					HSE_IO_SYNC))
			{
				WriteLog(0, "ERROR: Send Reply Failed "+ string(strReply.c_str()));
				return HSE_STATUS_ERROR;
			}

		}
		catch(...)
		{
			WriteLog(0, "ERROR : Exception.");
		}
	}

	// ready to return to IIS
    pECB->dwHttpStatusCode=200; // 200 OK


    return HSE_STATUS_SUCCESS;
}
///////////////////////////////////////////////////////////////////////
// CXMLRPCExtension command handlers

void CXMLRPCExtension::Default(CHttpServerContext* pCtxt)
{
	StartContent(pCtxt);
	WriteTitle(pCtxt);

	*pCtxt << _T("This default message was produced by the Internet");
	*pCtxt << _T(" Server DLL Wizard. Edit your CXMLRPCExtension::Default()");
	*pCtxt << _T(" implementation to change it.\r\n");

	EndContent(pCtxt);


}

// Do not edit the following lines, which are needed by ClassWizard.
#if 0
BEGIN_MESSAGE_MAP(CXMLRPCExtension, CHttpServer)
	//{{AFX_MSG_MAP(CXMLRPCExtension)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()
#endif	// 0




///////////////////////////////////////////////////////////////////////
// If your extension will not use MFC, you'll need this code to make
// sure the extension objects can find the resource handle for the
// module.  If you convert your extension to not be dependent on MFC,
// remove the comments arounn the following AfxGetResourceHandle()
// and DllMain() functions, as well as the g_hInstance global.

/****

static HINSTANCE g_hInstance;

HINSTANCE AFXISAPI AfxGetResourceHandle()
{
	return g_hInstance;
}

BOOL WINAPI DllMain(HINSTANCE hInst, ULONG ulReason,
					LPVOID lpReserved)
{
	if (ulReason == DLL_PROCESS_ATTACH)
	{
		g_hInstance = hInst;
	}

	return TRUE;
}

****/
