/* Copyright (C) 2000-2001 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

// HTTPTransport.cpp: implementation of the CHTTPTransport class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "HTTPTransport.h"
#include "Base64Coder.h"
#include "HTTPSocket.h"

#include <iostream>
using namespace std;
//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////
CHTTPTransport::CHTTPTransport()
{
	// Start Winsock
	WSADATA wsd;
	WSAStartup(0x0101, &wsd);

	m_strHost = "";
	m_strObject = "";
	m_nPort = 80;
	m_strUserName = "";
	m_strPassword = "";
	m_bUseSSL = false;
}

CHTTPTransport::CHTTPTransport(string strHost, string strObject, int nPort, string strUserName, string strPassword, bool bUseSSL)
{
	// Start Winsock
	WSADATA wsd;
	WSAStartup(0x0101, &wsd);

	m_strHost = strHost;
	m_strObject = strObject;
	m_nPort = nPort;
	m_strUserName = strUserName;
	m_strPassword = strPassword;
	m_bUseSSL = bUseSSL;
}

CHTTPTransport::CHTTPTransport(string strFullURL)
{
	// Start Winsock
	WSADATA wsd;
	WSAStartup(0x0101, &wsd);

	m_strHost = "";
	m_strObject = "";
	m_nPort = 80;
	m_strUserName = "";
	m_strPassword = "";
	m_bUseSSL = false;
	m_strFullURL = strFullURL;
	ParseURL(strFullURL);
}

CHTTPTransport::~CHTTPTransport()
{
	WSACleanup();
}


string CHTTPTransport::SendXMLCall(string strXML)
{
    TCHAR szErrMsg[200];
    HINTERNET hOpen = NULL, hConnect  = NULL, hRequest = NULL;
	hOpen = InternetOpen(_T("Mozilla/4.0 (compatible; MSIE 5.0; Windows NT 5.0)"),INTERNET_OPEN_TYPE_PRECONFIG, NULL, 0, 0);

	if (!(hConnect = InternetConnect (
						 hOpen,
						 m_strHost.c_str(),
						 m_nPort,
						 NULL, NULL,
						 INTERNET_SERVICE_HTTP,
						 0, 0)))
	{
		sprintf (szErrMsg, TEXT("%s: %x"), TEXT("InternetConnect Error"),
				 GetLastError());
		return szErrMsg;
	}
	DWORD flag = INTERNET_FLAG_NO_AUTO_REDIRECT|
                 INTERNET_FLAG_IGNORE_REDIRECT_TO_HTTPS|
                 INTERNET_FLAG_IGNORE_REDIRECT_TO_HTTP|
				 INTERNET_FLAG_IGNORE_CERT_CN_INVALID |
				 INTERNET_FLAG_IGNORE_CERT_DATE_INVALID |
                 INTERNET_FLAG_RELOAD|
				 INTERNET_FLAG_NO_AUTH |
                 INTERNET_FLAG_NO_COOKIES ; // to allow to send the real cookie
	
	if (m_bUseSSL)
		flag |= INTERNET_FLAG_SECURE ;

	LPTSTR AcceptTypes[2] = {TEXT("text/xml"), NULL};
	if (!(hRequest = HttpOpenRequest (
                         hConnect,
                         TEXT("POST"),
                         m_strObject.c_str(),
                         NULL,
                         NULL,
                         (LPCTSTR*)AcceptTypes,
                         flag,
                         0)))
    {
        sprintf (szErrMsg, TEXT("%s: %x"), TEXT("HttpOpenRequest Error"),
                 GetLastError());
        return szErrMsg;
    }

	if (!HttpSendRequest (hRequest, NULL, 0, (void *)strXML.c_str() , strXML.size()))
	{
        sprintf (szErrMsg, TEXT("%s: %x"), TEXT("HttpOpenRequest Error"),
                 GetLastError());
        return szErrMsg;
	}

	DWORD dwBytesRead = 0;
	string strResp ;
	do
	{
		DWORD dwBytes;
		if(InternetQueryDataAvailable(hRequest, &dwBytes, 0, 0))
		{
            LPSTR pData = new char[dwBytes+1];
            if(InternetReadFile(hRequest, pData, dwBytes, &dwBytesRead))
            {
				pData[dwBytesRead] = 0 ;
				strResp += pData ;
			}
			delete [] pData ;
		}
	}
    while(dwBytesRead);
    InternetCloseHandle(hRequest);
    InternetCloseHandle(hConnect);
    InternetCloseHandle(hOpen);

	return strResp ;
}

bool CHTTPTransport::GetRemoteFile(string strLocalPath)
{
    #define HTTPBUFLEN    512 // Size of HTTP Buffer...
     char httpbuff[HTTPBUFLEN];
     TCHAR   szCause[255];
     CString Cause;
     Cause.Format("YES");
 
     TRY
     {
          CInternetSession mysession;
          CHttpFile *remotefile = (CHttpFile *)mysession.OpenURL(m_strFullURL.c_str(),1,INTERNET_FLAG_TRANSFER_BINARY|INTERNET_FLAG_RELOAD);
 
          CFile myfile(strLocalPath.c_str(), CFile::modeCreate|CFile::modeWrite|CFile::typeBinary);
          int numbytes;
 
		  
	      while (numbytes = remotefile->Read(httpbuff, HTTPBUFLEN))
               myfile.Write(httpbuff, numbytes);

		  myfile.Close();
		  remotefile->Close() ;
		  delete remotefile;
		  mysession.Close();
     }
 
     CATCH_ALL(error)
     {
		 FILE * fd = fopen("c:\\temp\\ftp.log","a+");
          error->GetErrorMessage(szCause,254,NULL);
          Cause.Format("%s",szCause);
		 fprintf(fd, "%s", szCause);
		 fclose(fd);
		  return false;
     }
     END_CATCH_ALL;

	 return true;
}

void CHTTPTransport::ParseURL(string strURL)
{
	if (strURL.substr(0,7) != "http://")
	{
		cout<<"URL Parse Error!: "<<strURL.c_str()<<endl;;
		return ;
	}
	strURL.erase(0,7);
	int nPos = strURL.find_first_of('/');
	if(nPos > 0)
	{
		m_strHost = strURL.substr(0,nPos).c_str();
		m_strObject = strURL.substr(nPos).c_str();
	}
	else
	{
		m_strHost = strURL.c_str();
		m_strObject = "/";
	}

	int pos = m_strHost.find_first_of(':');
	if (pos > 0)
	{
		int pos = m_strHost.find_first_of(':');
		string strPort = m_strHost.substr(pos);
		m_nPort = atoi(strPort.c_str());
		m_strHost = m_strHost.substr(0,pos);
	}
	else
	{
		m_nPort = 80;
	}
}