/* Copyright (C) 2000-2001 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

// ServerSettings.cpp: implementation of the CServerSettings class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "ServerSettings.h"
#include "Base64Coder.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CServerSettings::CServerSettings()
{
	// Web Server Defaults
	m_nWebPort = 80;
	m_strWebRoot = ".\\";
	m_strExtensionPath = "/xmlrpc/xmlpost.dll";
	m_strCachePath = "/xmlrpc/cache/";
	m_bUseSSL = false;

	// FTP Server Defaults
	m_bUseFTP = false;
	m_nFtpPort = 21;
	m_strFtpRoot = ".\\";
	m_strFtpUser = "";
	m_strFtpPass = "";
}

CServerSettings::~CServerSettings()
{

}

bool CServerSettings::ReadSettings(string strFile)
{
	CFileStatus fs;
	if (!CFile::GetStatus(strFile.c_str(), fs))
		return false;

	CFile f(strFile.c_str(), CFile::modeRead);
	string strPath = f.GetFilePath();
	f.Close();

	// Read Web Server Settings
	m_nWebPort = GetPrivateProfileInt("WEBSERVER","PORT",999,strPath.c_str());

	char buf[256];
	DWORD dwLen = GetPrivateProfileString("WEBSERVER","ROOT",".\\",buf,128,strPath.c_str());
	m_strWebRoot = buf;

	dwLen = GetPrivateProfileString("WEBSERVER","EXT","/xmlrpc/xmlpost.dll",buf,128,strPath.c_str());
	m_strExtensionPath = buf;

	// OC 10/06/2002 add a slash at the beginning of m_strExtensionPath
	if (m_strExtensionPath[0] != '/')
		m_strExtensionPath = "/" + m_strExtensionPath ;

	dwLen = GetPrivateProfileString("WEBSERVER","CACHE","/xmlrpc/cache/",buf,128,strPath.c_str());
	m_strCachePath = buf;

	m_bUseSSL = GetPrivateProfileInt("WEBSERVER","USESSL",0,strPath.c_str()) != FALSE;

	// Read FTP Server Settings
	m_bUseFTP = GetPrivateProfileInt("FTPSERVER","USEFTP",1,strPath.c_str()) != FALSE;
	m_nFtpPort = GetPrivateProfileInt("FTPSERVER","PORT",999,strPath.c_str());

	char ftpBuf[256];
	dwLen = GetPrivateProfileString("FTPSERVER","ROOT",".\\",ftpBuf,128,strPath.c_str());
	m_strFtpRoot = ftpBuf;
	// OC 10/06/2002 add a back slash at the end of m_strFtpRoot
	if (ftpBuf[strlen(ftpBuf) - 1] != '\\')
		m_strFtpRoot += '\\' ;

	dwLen = GetPrivateProfileString("FTPSERVER","USER","",ftpBuf,128,strPath.c_str());

	Base64Coder coder;
	string strTmp = ftpBuf;
	if (strTmp != "")
	{
		coder.Decode(strTmp.c_str());
		m_strFtpUser = coder.DecodedMessage();
	}

	dwLen = GetPrivateProfileString("FTPSERVER","PASS","",ftpBuf,128,strPath.c_str());
	strTmp = ftpBuf;
	if (strTmp != "")
	{
		coder.Decode(strTmp.c_str());
		m_strFtpPass = coder.DecodedMessage();
	}

	return true;
}

CServerSettings& CServerSettings::operator = (CServerSettings& src)
{
	m_nWebPort = src.m_nWebPort;
	m_strWebRoot = src.m_strWebRoot;
	m_strExtensionPath = src.m_strExtensionPath;
	m_strCachePath = src.m_strCachePath;
	m_bUseSSL = src.m_bUseSSL;

	m_bUseFTP = src.m_bUseFTP;
	m_nFtpPort = src.m_nFtpPort;
	m_strFtpRoot = src.m_strFtpRoot;
	m_strFtpUser = src.m_strFtpUser;
	m_strFtpPass = src.m_strFtpPass;

	return *this;
}

bool CServerSettings::SaveSettings(string strFile)
{
	char buf[16];

	sprintf(buf,"%d", m_nWebPort);
	WritePrivateProfileString("WEBSERVER", "PORT", buf, strFile.c_str());
	WritePrivateProfileString("WEBSERVER", "ROOT", m_strWebRoot.c_str(), strFile.c_str());
	WritePrivateProfileString("WEBSERVER", "EXT", m_strExtensionPath.c_str(), strFile.c_str());
	WritePrivateProfileString("WEBSERVER", "CACHE", m_strCachePath.c_str(), strFile.c_str());
	sprintf(buf,"%d", m_bUseSSL);
	WritePrivateProfileString("WEBSERVER", "USESSL", buf, strFile.c_str());

	sprintf(buf,"%d",m_bUseFTP);
	WritePrivateProfileString("FTPSERVER", "USEFTP", buf, strFile.c_str());
	WritePrivateProfileString("FTPSERVER", "ROOT", m_strFtpRoot.c_str(), strFile.c_str());
	sprintf(buf,"%d", m_nFtpPort);
	WritePrivateProfileString("FTPSERVER", "PORT", buf, strFile.c_str());

	// Encode the username and password
	Base64Coder coder;
	coder.Encode(m_strFtpUser.c_str());
	WritePrivateProfileString("FTPSERVER", "USER", coder.EncodedMessage(), strFile.c_str());
	coder.Encode(m_strFtpPass.c_str());
	WritePrivateProfileString("FTPSERVER", "PASS", coder.EncodedMessage(), strFile.c_str());


	return true;
}
