/* Copyright (C) 2001 Kyle Harrison
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

// XMLParser.cpp: implementation of the CXMLParser class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "XMLParser.h"
#include <atlbase.h>
#include <comdef.h>

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////
CXMLNodeElement::CXMLNodeElement()
{
	m_pXMLNode = NULL;
}

CXMLNodeElement::~CXMLNodeElement()
{
	if (m_pXMLNode)
	{
		m_pXMLNode->Release();
//		delete m_pXMLNode ;
	}
}

void CXMLNodeElement::SetXMLNode(IXMLDOMNode *pXMLNode)
{
	m_pXMLNode = pXMLNode;
}

CString CXMLNodeElement::GetNodeName()
{
	BSTR bstrNodeName;
	m_pXMLNode->get_nodeName(&bstrNodeName);
	CString strName;
	strName.Format(_T("%S"),bstrNodeName);
	SysFreeString(bstrNodeName);
	return strName;
}

BOOL CXMLNodeElement::HasChildNodes()
{
	VARIANT_BOOL bVal;	
	m_pXMLNode->hasChildNodes(&bVal);
	return (BOOL)bVal;
}

CString CXMLNodeElement::GetText()
{
	BSTR bstrNodeText;
	m_pXMLNode->get_text(&bstrNodeText);
	CString strText;
	strText.Format(_T("%S"),bstrNodeText);
	SysFreeString(bstrNodeText);
	return strText;

}

CXMLNodeElement* CXMLNodeElement::GetFirstChild()
{
	IXMLDOMNode *pXMLNode = NULL;
	m_pXMLNode->get_firstChild(&pXMLNode);
	if(pXMLNode)
	{
		CXMLNodeElement* pChild = new CXMLNodeElement();
		pChild->SetXMLNode(pXMLNode);
		return pChild;
	}
	return NULL;
}

CXMLNodeElement* CXMLNodeElement::GetNextSibling()
{
	IXMLDOMNode *pXMLNode = NULL;
	m_pXMLNode->get_nextSibling(&pXMLNode);
	if(pXMLNode)
	{
		CXMLNodeElement* pSibling = new CXMLNodeElement();
		pSibling->SetXMLNode(pXMLNode);
		return pSibling;
	}
	return NULL;
}

CXMLNodeElement* CXMLNodeElement::GetChildElementByName(CString strName)
{
	// Iterate through the child items of this note to find a matching tag
	CXMLNodeElement* pChild = GetFirstChild();
	CXMLNodeElement* pChildTmp = NULL;
	while (pChild)
	{
		if (pChild->GetNodeName() == strName)
			return pChild;

		pChildTmp = pChild ;
		pChild = pChild->GetNextSibling();
		delete pChildTmp ;
	}

	return NULL;
}

CXMLParser::CXMLParser()
{
	m_pXMLDoc = NULL;
	m_strXML = "";


}

CXMLParser::~CXMLParser()
{
	if (m_pXMLDoc)
	{
		m_pXMLDoc->Release() ;
//		delete m_pXMLDoc ;
	}
}

BOOL CXMLParser::Parse(CString strXML)
{
	if ( SUCCEEDED(CoInitialize(NULL)) )
	{
		if ( SUCCEEDED (CoCreateInstance(CLSID_DOMDocument,
												NULL,
												CLSCTX_INPROC_SERVER,
												IID_IXMLDOMDocument,
												reinterpret_cast<void**>(&m_pXMLDoc))))
		{
			//
			// Tell the "doc" that we're not going to load asynchronously.
			//
			if ( SUCCEEDED(m_pXMLDoc->put_async(VARIANT_FALSE)) )
			{

				// Load the XML from the XML Text string
				_bstr_t xml = strXML;
				VARIANT_BOOL vBool;
				m_pXMLDoc->loadXML(xml,&vBool);
				SysFreeString(xml);
				if ( vBool != VARIANT_TRUE )
				{
					return FALSE;
				}
				return TRUE;
			}
		}
	}

	return FALSE;
}

CXMLNodeElement* CXMLParser::GetRootElement()
{
	IXMLDOMNode *pDocRoot = NULL;
	IXMLDOMNode *pXMLVer = NULL;
	IXMLDOMNode *pFirstEl = NULL;
	CXMLNodeElement* pFirstElement = NULL ;
	// Query the XMLDocument to obtain the documnet root
	if ( SUCCEEDED(m_pXMLDoc->QueryInterface(IID_IXMLDOMNode, 
											   reinterpret_cast<void**>(&pDocRoot))))
	{
		// First element is the the xml version tag
		pDocRoot->get_firstChild(&pXMLVer);

		// Next sibling to the xml version tag is the data element
		pXMLVer->get_nextSibling(&pFirstEl);
		if(pFirstEl)
		{
			// Construct the Element wrapper class for this node
			pFirstElement = new CXMLNodeElement();
			pFirstElement->SetXMLNode(pFirstEl);
		}
	}
	if (pDocRoot)
	{
		pDocRoot->Release();
//		delete pDocRoot ;
	}
	if (pXMLVer)
	{
		pXMLVer->Release();
//		delete pXMLVer ;
	}
	return pFirstElement;
}

