#ifndef PSL__ERRTRAPDEF_LOADED
#define PSL__ERRTRAPDEF_LOADED 1
/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/


/*  C Include File Written by SDL on 19-MAR-1999 20:05:16.47 */

/*===========================================================================*/
/*                                       */
/*               **** C O M M O N ****                   */
/*                                       */
/*===========================================================================*/
/*                                       */
/*          Module Name: ERRTRAPDEF                  */
/*          ============                         */
/*                                       */
/*  Description:                                 */
/*  ============                                 */
/*                                       */
/*  This module defines the Error Trapping data structure            */
/*  used by the Error Trapping routines.                     */
/*                                       */
/*                                       */
/*===========================================================================*/

/*** MODULE ERRTRAPDEF ***/

struct errtrap
{

    /*  Class of Error to Trap */

    /*  This defines the class of error that is to be trapped. */
    /*  These have symbolic names of the form PSLERRCLS_*. */

    PSL_U_LONGWORD_T        err_classl;

    /*  Number of Error to Trap */

    /*  This defines the number of error that is to be trapped. */

    PSL_U_LONGWORD_T        err_msgnuml;

    /*  Response to be taken */

    /*  This defines the response to be taken when the error is trapped. */
    /*  These have symbolic names of the form PSLERRACT_*. */

    PSL_U_LONGWORD_T        err_responsel;

    /*  Pointer to C jmp_buf structure */

    /*  If the response to be taken includes a request to make a */
    /*  longjmp, then this structure contains a pointer to the */
    /*  longjmp structure. */

    jmp_buf                 *err_jmp_ptrpj;

    /*  Pointer to Error Handling Function */

    /*  If the response to be taken is "Call User Routine" then this */
    /*  contrains a pointer to the routine to be called. */

    PSL_U_LONGWORD_T        (*err_handlerpf)();

    /*  Diagnostics Stack Context */

    /*  This contains the diagnostics stack context associated with */
    /*  this entry and is used to ensure that obsolete entries are */
    /*  deleted from the stack. */

    PSL_U_LONGWORD_T        err_contextl;

    /*  Pointer to Previous Entry */

    /*  This contains a pointer to the previous entry in the error */
    /*  trap list (or NULL if there aren't any more). */

    struct errtrap          *err_prev_ptrps;

    /*  Error Trap Number */

    /*  This contains a "sequence number" for the error trap, relating */
    /*  to its position on the error trap list.  It is used to establish */
    /*  an error trap "context" for the user to play with. */

    PSL_U_LONGWORD_T        err_numberl;

} ;


/*  End of C Include File */

#endif  /* PSL__ERRTRAPDEF_LOADED */
