#ifndef PSL__TDBDEF_LOADED
#define PSL__TDBDEF_LOADED 1
/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

/*===========================================================================*/
/*                                       */
/*               **** C O M M O N  ****                  */
/*                                       */
/*===========================================================================*/
/*                                       */
/*          Module Name: TDBDEF                  */
/*          ============                         */
/*                                       */
/* Description:                                  */
/* ============                                  */
/*                                       */
/*  TPA Routines Data Structure.                         */
/*                                       */
/*  This structure defines the Tparse Definition Block structure         */
/*  used by the generic Tparse routines.                     */
/*                                       */
/*===========================================================================*/

/*** MODULE TDBDEF ***/

/*  Conditional Compilation stack structure */

struct ifs
{

    /*  Variant name. */

    char                    ifs_namec20[20];

    /*  Action type. */

    PSL_U_LONGWORD_T        ifs_actionl;
#define            TDB_IFAC_INC 1
    /* INCLUDE text if defined */
#define            TDB_IFAC_EXC 2
    /* EXCLUDE text if defined */

    /*  Flags. */

    PSL_U_LONGWORD_T        ifs_flagsl;
#define            TDB_IFFL_ELSE 0X1
    /* Got #ELSE directive */

    /*  Padding for expansion. */

    char                    ifs_paddingb4[4];

} ;

/*  Main TDB structure */

struct tdb
{

    /*  Structure Type. */

    /*  This is a text string that identifies what type of structure this is. */

    PSL_U_LONGWORD_T        tdb_strtypl;

    /*  The previous field contains this constant value.  Note that */
    /*  this is held as a hexadecimal number to improve performance, */
    /*  but the precise value varies from platform to platform as */
    /*  necessary to represent the ASCII string "TDB ". */

#ifdef LITTLE_ENDIAN
#define            TDB_STRNAM 0X20424454    /* = "TDB " */
#else  /* LITTLE_ENDIAN */
#define            TDB_STRNAM 0X54444220    /* = "TDB " */
#endif /* LITTLE_ENDIAN */

    /*  Diagnostic Stack pointer. */

    struct diagstk          *tdb_diag_ptrps;

    /*  User data pointer. */

    void                    *tdb_usrdat_ptrp;

    /*  Maximum length of source lines. */

    PSL_LONGWORD_T          tdb_max_linsizl;

    /*  Pointer to Source Line. */

    /*  This is the line as read from the source file. */

    char                    *tdb_srclin_ptrpc;

    /*  Length of Source Line. */

    PSL_LONGWORD_T          tdb_srclin_lenl;

    /*  Pointer to Parsable Line. */

    /*  This is the line with comments removed and converted to upper case. */

    char                    *tdb_prslin_ptrpc;

    /*  Length of Parsable Line. */

    PSL_LONGWORD_T          tdb_prslin_lenl;

    /*  Pointer to Error Line. */

    /*  This is used for underlining errors. */

    char                    *tdb_errlin_ptrpc;

    /*  End of File flag. */

    Boolean                 tdb_eof_flagn;

    /*  Underline Line Required flag. */

    Boolean                 tdb_undlin_flagn;

    /*  Array or Errors in Source Line. */

    PSL_U_LONGWORD_T        tdb_err_arrayl5[5];
#define            TDB_MAX_ERRS_IN_LINE 5

    /*  Number of Errors in Source Line */

    PSL_U_LONGWORD_T        tdb_errs_in_linel;

    /*  Line Count. */

    PSL_U_LONGWORD_T        tdb_line_countl;

    /*  Count of the number of errors encountered during parsing. */

    PSL_U_LONGWORD_T        tdb_err_countl;

    /*  Listing required flag. */

    Boolean                 tdb_listn;

    /*  Logging of messages required flag. */

    Boolean                 tdb_logn;

    /*  Object required flag. */

    Boolean                 tdb_objectn;

    /*  Listing Context Block pointer. */

    struct list_ctx         *tdb_list_ctx_ptrps;

    /*  Pointer to Source File. */

    FILE                    *tdb_srcfil_ptrpf;

    /*  Expanded Source File name. */

    char                    tdb_expsrcnamc256[256];

    /*  Default source file extension. */

    char                    tdb_srcextc16[16];

    /*  Comment string array. */

    char                    tdb_comment_strc4_4[4][4];
#define            TDB_MAX_COMMENT_STR 4

    /*  Continuation string length array. */

    PSL_LONGWORD_T          tdb_comment_lenl4[4];

    /*  Number of comment strings specified. */

    PSL_LONGWORD_T          tdb_comment_numl;

    /*  Continuation character array. */

    char                    tdb_cont_charb4[4];
#define            TDB_MAX_CONT_CHAR 4

    /*  Number of continuation characters specified. */

    PSL_LONGWORD_T          tdb_cont_numl;

    /*  Extra token terminator characters. */

    char                    tdb_sep_charb4[4];
#define            TDB_MAX_SEP_CHAR 4

    /*  Include nesting level. */

    PSL_U_LONGWORD_T        tdb_inc_levell;
#define            TDB_MAX_INC_LEVEL 8

    /*  File pointers stack. */

    FILE                    *tdb_file_ptr_stkpf8[8];

    /*  File name stack. */

    char                    tdb_file_name_stkc256_8[8][256];

    /*  Line count stack. */

    PSL_U_LONGWORD_T        tdb_line_count_stkl8[8];

    /*  Flags. */

    PSL_U_LONGWORD_T        tdb_flagsl;
#define            TDFL_ENBINC 0X1  /* Enable #INCLUDE handling */
#define            TDFL_ENBCND 0X2  /* Enable #IFDEF handling */
#define            TDFL_KEPCMT 0X4  /* Keep comments    */
#define            TDFL_ENBERR 0X8  /* Enable #ERROR handling */


    /*  List of defined variants. */

    char                    **tdb_variant_listppc;

    /*  Number of values in the variant list. */

    PSL_U_LONGWORD_T        tdb_variant_countl;

    /*  Conditional Compilation nesting level. */

    PSL_LONGWORD_T          tdb_if_levell;
#define            TDB_MAX_IF_LEVEL 10

    /*  Conditional Compilation stack. */

    struct ifs              tdb_if_stkr10[10];


    /*  This field indicates if there was any platform specific */
    /*  code in the parsed code. */

    Boolean                 tdb_plat_spec_coden;

    /*  This field indicates if there was any operating system specific */
    /*  code in the parsed code. */

    Boolean                 tdb_opsys_spec_coden;


    /*  Padding for expansion. */

    char                    tdb_paddingb12[12];

} ;


/*  End of C Include File */

#endif  /* PSL__TDBDEF_LOADED */
