/* Copyright (C) 2001 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

// ArchMgrSystem.cpp : implementation file
//

#include "stdafx.h"
#include "commander.h"
#include "ArchitectureManager.h"
#include "ArchMgrSystem.h"
#include "ArchMgrHostFile.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CArchMgrSystem dialog


CArchMgrSystem::CArchMgrSystem(CWnd* pParent /*=NULL*/)
	: CDialog(CArchMgrSystem::IDD, pParent)
{
	//{{AFX_DATA_INIT(CArchMgrSystem)
		// NOTE: the ClassWizard will add member initialization here
	//}}AFX_DATA_INIT
    m_pArchitectureManager = (CArchitectureManager *) pParent;
}


void CArchMgrSystem::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CArchMgrSystem)
	DDX_Control(pDX, IDC_ARCHMGR_SYSTEM_LIST, m_lcSystemInfo);
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(CArchMgrSystem, CDialog)
	//{{AFX_MSG_MAP(CArchMgrSystem)
	ON_WM_SHOWWINDOW()
	ON_COMMAND(ID_REFRESH, OnRefresh)
	ON_WM_SIZE()
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CArchMgrSystem message handlers

BOOL CArchMgrSystem::OnInitDialog() 
{
	CDialog::OnInitDialog();
	
	// TODO: Add extra initialization here
	
    // Initialize System Information list control columns.
    m_lcSystemInfo.InsertColumn(0, "Item", LVCFMT_LEFT, 120);
    m_lcSystemInfo.InsertColumn(1, "Value", LVCFMT_LEFT, 300);

	return TRUE;  // return TRUE unless you set the focus to a control
	              // EXCEPTION: OCX Property Pages should return FALSE
}

///////////////////////////////////////////////////////////////////////////////
// @mfunc   Update the System Information list box from file. The list box
//          has 2 columns - an item header and an item value. These are stored
//          in the Data File as Key Name/Key Value pairs under the System
//          Information data section, e.g.:
//
//          [SYSTEM-MAIN-LIST]
//          Computer Name=MYCOMPUTER
//          ...
//
// @rdesc   TRUE=Success; FALSE=Failure.
//
BOOL CArchMgrSystem::UpdateDialog()
{
    // Delete all items from list control.
    m_lcSystemInfo.DeleteAllItems();

    // Read data from file and populate list control.
    CArchMgrHostFile sysfil;
    std::string strHostFileName (m_pArchitectureManager->m_selOstaHost.strHostFileName.GetBuffer(0));
    sysfil.SetFile(strHostFileName);

    TRY
    {
        // Get list of Item (Key) Names in target file section.
        const DWORD MAXBUFSIZ = 1024;
        char szSectionName[MAXBUFSIZ] = "SYSTEM-MAIN-LIST";
        char szKeyNames[MAXBUFSIZ];
        sysfil.GetKeyNames(szKeyNames, MAXBUFSIZ, szSectionName);

        // If there are some Item Names, loop through the Data file getting the
        // corresponding Key Values and populating the list control.
        if (strlen(szKeyNames) > 0)
        {
            std::string strKeyValue;
            int nItem = -1;
            for (char *pKeyName = &szKeyNames[0];
                 strlen(pKeyName) != 0;
                 pKeyName = pKeyName + strlen(pKeyName) + 1)
            {
                std::string strKeyName(pKeyName);
                sysfil.Get(strKeyName, strKeyValue, NULL, szSectionName);
                m_lcSystemInfo.InsertItem(++nItem, strKeyName.c_str());
                m_lcSystemInfo.SetItemText(nItem, 1, strKeyValue.c_str());
            }
        }
    }
    CATCH( CFileException, e )
    {
        #ifdef _DEBUG
            afxDump << "File could not be opened "
                << e->m_cause << "\n";
        #endif
        return FALSE;
    }
    END_CATCH

    // Update the contents of the controls to display.
    UpdateData(FALSE);

    return TRUE;
}

void CArchMgrSystem::OnShowWindow(BOOL bShow, UINT nStatus) 
{
	CDialog::OnShowWindow(bShow, nStatus);
	
	// TODO: Add your message handler code here
    if (bShow == TRUE) UpdateDialog();	
}

void CArchMgrSystem::OnRefresh() 
{
	// TODO: Add your command handler code here
    UpdateDialog();	
}

void CArchMgrSystem::OnSize(UINT nType, int cx, int cy) 
{
	CDialog::OnSize(nType, cx, cy);
	
	// TODO: Add your message handler code here

    CRect myrect;

    // Resize the System Information List Control Window:
    //   Adjust Width with main window.
    //   Adjust height with main window.
    if (m_lcSystemInfo.GetSafeHwnd())
    {
        m_lcSystemInfo.GetWindowRect(myrect);
        ScreenToClient(myrect);
        myrect.NormalizeRect();
        m_lcSystemInfo.MoveWindow(myrect.left ,myrect.top, (cx - myrect.left), (cy - (myrect.top * 2)));
    }	
}
