/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

// CECAboutDlg.cpp : implementation file

#include "stdafx.h"
#include "Commander.h"
#include "CECAboutDlg.h"
#include "CECAboutPlusDlg.h"
#include "LicenceDlg.h"
#include "htmlhelp.h"
#include "BaseUIHelpID.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CECAboutDlg dialog
CECAboutDlg::CECAboutDlg(CWnd* pParent /*=NULL*/)
        : CDialog(CECAboutDlg::IDD, pParent) {
    //{{AFX_DATA_INIT(CECAboutDlg)
    //}}AFX_DATA_INIT
}

void
CECAboutDlg::DoDataExchange(CDataExchange* pDX) {
    CDialog::DoDataExchange(pDX);
    //{{AFX_DATA_MAP(CECAboutDlg)
    //}}AFX_DATA_MAP
}

BEGIN_MESSAGE_MAP(CECAboutDlg, CDialog)
//{{AFX_MSG_MAP(CECAboutDlg)
ON_BN_CLICKED(IDC_ADVANCED, OnAdvanced)
ON_BN_CLICKED(IDC_LICENCE, OnLicence)
	ON_WM_HELPINFO()
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CECAboutDlg message handlers
BOOL
CECAboutDlg::OnInitDialog() {
    CDialog::OnInitDialog();
    CWaitCursor WaitCursor;
    LoadBuildInfo();
    CenterWindow();
    return TRUE;
}

void
CECAboutDlg::OnOK() {
    CDialog::OnOK();
}

void
CECAboutDlg::OnAdvanced() {
    CECAboutPlusDlg AdvancedDlg;
    AdvancedDlg.DoModal();
}

//    LoadBuildInfo
//    =============
//    Fetch and load the build information for this executable.
void
CECAboutDlg::LoadBuildInfo () {
    char strFileName[MAX_PATH];
    CString strProdInfo;
    CString strFileInfo;
    CString strCopyInfo;
    ::GetModuleFileName(NULL, strFileName, sizeof(strFileName));
    DWORD dwHandle;
    DWORD dwLen = ::GetFileVersionInfoSize(strFileName, &dwHandle);
    LPVOID lpInfoBuf = malloc(dwLen);
    if(::GetFileVersionInfo(strFileName, dwHandle, dwLen, lpInfoBuf)) {
        VS_FIXEDFILEINFO* fixedfileinfo;
	LPVOID lpBuffer;
	UINT uiBytes;
        if(::VerQueryValue(lpInfoBuf, TEXT("\\"), (LPVOID*)&fixedfileinfo, &uiBytes)) {
	    if(::VerQueryValue(lpInfoBuf,
		               TEXT("\\StringFileInfo\\000004b0\\ProductName"),
		    	       &lpBuffer, &uiBytes) &&
	       (uiBytes > 0)) {
		strProdInfo.Format("%s %d.%d.%d.%d",
			           (LPCTSTR)lpBuffer,
				   fixedfileinfo->dwProductVersionMS/0x10000,
				   fixedfileinfo->dwProductVersionMS%0x10000,
				   fixedfileinfo->dwProductVersionLS/0x10000,
				   fixedfileinfo->dwProductVersionLS%0x10000);
	    } else {
		strProdInfo.Format("OpenSTA %d.%d.%d.%d",
				   fixedfileinfo->dwProductVersionMS/0x10000,
                                   fixedfileinfo->dwProductVersionMS%0x10000,
                                   fixedfileinfo->dwProductVersionLS/0x10000,
                                   fixedfileinfo->dwProductVersionLS%0x10000);
	    }
	    if(::VerQueryValue(lpInfoBuf,
		               TEXT("\\StringFileInfo\\000004b0\\InternalName"),
		    	       &lpBuffer, &uiBytes) &&
	       (uiBytes > 0)) {
		strFileInfo.Format("%s %d.%d.%d.%d, built %s %s",
			           (LPCTSTR)lpBuffer,
				   fixedfileinfo->dwFileVersionMS/0x10000,
				   fixedfileinfo->dwFileVersionMS%0x10000,
				   fixedfileinfo->dwFileVersionLS/0x10000,
				   fixedfileinfo->dwFileVersionLS%0x10000,
				   __DATE__, __TIME__);
	    } else {
		strFileInfo.Format("Utility %d.%d.%d.%d, built %s %s",
				   fixedfileinfo->dwFileVersionMS/0x10000,
                                   fixedfileinfo->dwFileVersionMS%0x10000,
                                   fixedfileinfo->dwFileVersionLS/0x10000,
                                   fixedfileinfo->dwFileVersionLS%0x10000,
                                   __DATE__, __TIME__);
	    }
	    if(::VerQueryValue(lpInfoBuf,
		               TEXT("\\StringFileInfo\\000004b0\\LegalCopyright"),
		    	       &lpBuffer, &uiBytes) &&
	       (uiBytes > 0)) {
		strCopyInfo.Format("%s", (LPCTSTR)lpBuffer);
	    }
	}
    }
    free(lpInfoBuf);

    if(strFileInfo.IsEmpty()) {
	strFileInfo.Format("Built %s %s", __DATE__, __TIME__);
    }
    SetDlgItemText(IDC_BUILD, strFileInfo);
    
    if(strProdInfo.IsEmpty()) {
	strProdInfo.Format("OpenSTA XXX");
    }
    SetDlgItemText(IDC_PRODVER, strProdInfo);

    if(strCopyInfo.IsEmpty()) {
	strCopyInfo.Format("Copyrighted to Various Entities");
    }
    SetDlgItemText(IDC_COPYRIGHT, strCopyInfo);
}

void
CECAboutDlg::OnLicence() {
    CLicenceDlg dlg;
    dlg.DoModal();
}

BOOL
CECAboutDlg::OnHelpInfo(HELPINFO* pHelpInfo) {
    pHelpInfo->dwContextId = HIDD_ABOUT;
    return CDialog::OnHelpInfo(pHelpInfo);
}
