/* Copyright (C) 2001 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

// ObjectTree.cpp : implementation file
//

#include "stdafx.h"
#include "commander.h"
#include "ObjectTree.h"
#include "RegFunctions.h"
#include "MainFrm.h"
#include "ScriptCommentDlg.h"
#include "TestFile.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

IMPLEMENT_DYNAMIC(CObjectTree, CTreeCtrl)

/////////////////////////////////////////////////////////////////////////////
// CObjectTree
#define REFRESH_TIMER 1

#define REFRESH_DURATION 5000
extern CLIPFORMAT g_ClipFormat;

CObjectTree::CObjectTree():
	m_hTestFolder(NULL),
	m_hScriptFolder(NULL),
	m_hProfileFolder(NULL),
	m_hRepositoryRoot(NULL),
	m_dwPrevRefreshTime(0)
{ 
}

CObjectTree::~CObjectTree()
{

}

BEGIN_MESSAGE_MAP(CObjectTree, CTreeCtrl)
	//{{AFX_MSG_MAP(CObjectTree)
	ON_WM_CREATE()
	ON_NOTIFY_REFLECT(TVN_BEGINLABELEDIT, OnBeginlabeledit)
	ON_NOTIFY_REFLECT(TVN_ENDLABELEDIT, OnEndlabeledit)
	ON_NOTIFY_REFLECT(TVN_BEGINDRAG, OnBegindrag)
	ON_NOTIFY_REFLECT(TVN_KEYDOWN, OnKeydown)
	ON_NOTIFY_REFLECT(NM_RCLICK, OnRclick)
	ON_WM_CONTEXTMENU()
	ON_COMMAND(ID_NEW_FOLDER, OnNewFolder)
	ON_COMMAND(ID_DELETE, OnDelete)
	ON_COMMAND(ID_RENAME, OnRename)
	ON_WM_TIMER()
	ON_WM_ERASEBKGND()
	ON_WM_PAINT()
	ON_COMMAND(ID_COMMENT, OnComment)
	ON_COMMAND(ID_DUPLICATE, OnDuplicate)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CObjectTree message handlers

int CObjectTree::OnCreate(LPCREATESTRUCT lpCreateStruct) 
{
	if (CTreeCtrl::OnCreate(lpCreateStruct) == -1)
		return -1;
	
	m_DropTarget.Register(this);

	m_TreeImages.Create(IDB_TREEIMAGES_BMP,16,1,RGB(0,128,128));
	SetImageList(&m_TreeImages,TVSIL_NORMAL);
	SetImageList(&m_TreeImages,TVSIL_STATE);

	m_TreeImages.SetOverlayImage(enEmptyFileOverlay,1);
	m_TreeImages.SetOverlayImage(enLockedFileOverlay,2);
	m_TreeImages.SetOverlayImage(enRunningFileOverlay,3);

//	SetTimer(REFRESH_TIMER,REFRESH_DURATION,NULL);

	RefreshTree();

	return 0;
}

CString CObjectTree::GetItemPath(HTREEITEM hItem)
{
	CString strPath;
	if(m_mapItemToPath.Lookup(hItem,strPath))
		return strPath;

	return "";
}

BOOL CObjectTree::LoadProfilesFromDisk()
{
	CString strPath = GetRepositoryPath();
	
	if(!m_hProfileFolder)
		m_hProfileFolder = InsertItem("Collectors"/*strProfileType*/,enProfilesFold,enSelProfilesFold,m_hRepositoryRoot);
		
	CString strProfilePath(strPath + strProfileType);
	m_mapItemToPath.SetAt(m_hProfileFolder,strProfilePath);

	return LoadProfilesInPath(strProfilePath+"\\",m_hProfileFolder);
}

BOOL CObjectTree::LoadProfilesInPath(const CString& strPath,HTREEITEM hParent)
{
	BOOL bMoreProfiles;
	CFileFind FF;
	int nImage,nSelImage;
	//Look for profiles in the repository
	if(FF.FindFile(strPath + "*.*"))
	{
		do
		{
			bMoreProfiles = FF.FindNextFile();
			if(FF.IsDots() || FF.IsHidden())
				continue;
			
			HTREEITEM hItem;
			if(FF.IsDirectory())
			{
				hItem = InsertItem(FF.GetFileName(),enFolder,enSelFolder,hParent);
				LoadProfilesInPath(strPath + FF.GetFileName() + "\\",hItem);
				m_mapItemToPath.SetAt(hItem,FF.GetFilePath());
			}
			else
			if(GetFileItemImage(FF.GetFilePath(),nImage,nSelImage))
			{
				hItem = InsertItem(FF.GetFileTitle(),nImage,nSelImage,hParent);
				m_mapItemToPath.SetAt(hItem,FF.GetFilePath());
				if(!FF.GetLength())
					SetItemState(hItem,INDEXTOOVERLAYMASK(1),TVIS_OVERLAYMASK);

				DWORD dwStates;
				if(m_mapFileToState.Lookup(FF.GetFilePath(),dwStates))
				{
					if(dwStates & RFF_LOCKED)
						SetItemState(hItem,INDEXTOOVERLAYMASK(2),TVIS_OVERLAYMASK);
					else
					if(dwStates & RFF_RUNNING)
						SetItemState(hItem,INDEXTOOVERLAYMASK(3),TVIS_OVERLAYMASK);
				}
			}
		}while(bMoreProfiles);
	}
	return true;
}

BOOL CObjectTree::LoadScriptsFromDisk()
{
	CString strPath = GetRepositoryPath();
	
	if(!m_hScriptFolder)
		m_hScriptFolder = InsertItem(strScriptType,enScriptsFold,enSelScriptsFold,m_hRepositoryRoot);
		
	CString strScriptPath(strPath + strScriptType);
	m_mapItemToPath.SetAt(m_hScriptFolder,strScriptPath);

	return LoadScriptsInPath(strScriptPath+"\\",m_hScriptFolder);
}

BOOL CObjectTree::LoadScriptsInPath(const CString& strPath,HTREEITEM hParent)
{
	BOOL bMoreScripts;
	CFileFind FF;
	int nImage,nSelImage;
	//Look for Scripts in the repository
	if(FF.FindFile(strPath + "*.*"))
	{
		do
		{
			bMoreScripts = FF.FindNextFile();
			if(FF.IsDots() || FF.IsHidden())
				continue;

			HTREEITEM hItem;
			if(FF.IsDirectory())
			{
				hItem = InsertItem(FF.GetFileName(),enFolder,enSelFolder,hParent);
				LoadScriptsInPath(strPath + FF.GetFileName() + "\\",hItem);
				m_mapItemToPath.SetAt(hItem,FF.GetFilePath());
			}
			else
			if(GetFileItemImage(FF.GetFilePath(),nImage,nSelImage))
			{
				hItem = InsertItem(FF.GetFileTitle(),nImage,nSelImage,hParent);
				m_mapItemToPath.SetAt(hItem,FF.GetFilePath());
				if(!FF.GetLength())
					SetItemState(hItem,INDEXTOOVERLAYMASK(1),TVIS_OVERLAYMASK);

				DWORD dwStates;
				if(m_mapFileToState.Lookup(FF.GetFilePath(),dwStates))
				{
					if(dwStates & RFF_LOCKED)
						SetItemState(hItem,INDEXTOOVERLAYMASK(2),TVIS_OVERLAYMASK);
					else
					if(dwStates & RFF_RUNNING)
						SetItemState(hItem,INDEXTOOVERLAYMASK(3),TVIS_OVERLAYMASK);
				}
			}
		}while(bMoreScripts);
	}
	return true;
}

BOOL CObjectTree::GetFileItemImage(const CString& strFile,int& nImage,int& nSelImage)
{
	nImage = nSelImage = RegisterFileExtension(strFile);
	
	return true;
}

BOOL CObjectTree::LoadTestsFromDisk()
{
	CString strPath = GetRepositoryPath();
	
	if(!m_hTestFolder)
		m_hTestFolder = InsertItem(strTestType,enTestsFold,enSelTestsFold,m_hRepositoryRoot);
		
	CString strTestsPath(strPath + strTestType);
	m_mapItemToPath.SetAt(m_hTestFolder,strTestsPath);

	return LoadTestsInPath(strTestsPath+"\\",m_hTestFolder);
}

BOOL CObjectTree::LoadTestsInPath(const CString& strPath,HTREEITEM hParent)
{
	BOOL bMoreTests;
	CFileFind FF;
	int nImage,nSelImage;
	//Look for Tests in the repository
	if(FF.FindFile(strPath + "*.*"))
	{
		do
		{
			bMoreTests = FF.FindNextFile();
			if(FF.IsDots() || FF.IsHidden())
				continue;

			HTREEITEM hItem;
			if(FF.IsDirectory())
			{
				//Is this a test folder (check if it contains the test file)
				CString strTestFile = FF.GetFilePath();
				strTestFile += "\\" + FF.GetFileTitle() + ".";

				//Found a test folder					
				HTREEITEM hItem;
				CFileStatus fs;

				BOOL bTest = false;

				CStringArray arstrTestExtensions;
				GetTestExtensions(arstrTestExtensions);
				for(int nExt=0;nExt<arstrTestExtensions.GetSize();nExt++)
				{
					if(CFile::GetStatus(strTestFile+arstrTestExtensions[nExt],fs))
					{
						strTestFile += arstrTestExtensions[nExt];

						GetFileItemImage(strTestFile,nImage,nSelImage);
						hItem = InsertItem(FF.GetFileTitle(),nImage,nSelImage,hParent);

						if(!fs.m_size)
							SetItemState(hItem,INDEXTOOVERLAYMASK(1),TVIS_OVERLAYMASK);

						DWORD dwStates;
						if(m_mapFileToState.Lookup(strTestFile,dwStates))
						{
							if(dwStates & RFF_LOCKED)
								SetItemState(hItem,INDEXTOOVERLAYMASK(2),TVIS_OVERLAYMASK);
							else
							if(dwStates & RFF_RUNNING)
								SetItemState(hItem,INDEXTOOVERLAYMASK(3),TVIS_OVERLAYMASK);
						}

						m_mapItemToPath.SetAt(hItem,strTestFile);
						bTest = true;
						break;
					}
				}

				if(!bTest)
				{
					//just a normal folder
					hItem = InsertItem(FF.GetFileName(),enFolder,enSelFolder,hParent);
					m_mapItemToPath.SetAt(hItem,FF.GetFilePath());
					LoadTestsInPath(strPath + FF.GetFileName() + "\\",hItem);
				} 
			}
			else
			if(GetFileItemImage(FF.GetFilePath(),nImage,nSelImage))
			{
				hItem = InsertItem(FF.GetFileTitle(),nImage,nSelImage,hParent);
				m_mapItemToPath.SetAt(hItem,FF.GetFilePath());
				if(FF.GetLength())
					SetItemState(hItem,INDEXTOOVERLAYMASK(1),TVIS_OVERLAYMASK);

				DWORD dwStates;
				if(m_mapFileToState.Lookup(FF.GetFilePath(),dwStates))
				{
					if(dwStates & RFF_LOCKED)
						SetItemState(hItem,INDEXTOOVERLAYMASK(2),TVIS_OVERLAYMASK);
					else
					if(dwStates & RFF_RUNNING)
						SetItemState(hItem,INDEXTOOVERLAYMASK(3),TVIS_OVERLAYMASK);
				}
			}
		}while(bMoreTests);
	}
	return true;
}

void CObjectTree::SaveBranchState(HTREEITEM hBranch,CMap <CString,LPCTSTR,UINT,UINT>& mapPathToState)
{
	if(ItemHasChildren(hBranch))
	{
		CString strPath;
		m_mapItemToPath.Lookup(hBranch,strPath);
		UINT uiState = GetItemState(hBranch,TVIS_EXPANDED|TVIS_SELECTED);
		mapPathToState.SetAt(strPath,uiState);

		HTREEITEM hNextItem;
		HTREEITEM hChildItem = GetChildItem(hBranch);
		while (hChildItem != NULL)
		{
			hNextItem = GetNextItem(hChildItem, TVGN_NEXT);

			m_mapItemToPath.Lookup(hChildItem,strPath);
			uiState = GetItemState(hChildItem,TVIS_EXPANDED|TVIS_SELECTED);
			mapPathToState.SetAt(strPath,uiState);
	
			SaveBranchState(hChildItem,mapPathToState);
			
			hChildItem = hNextItem;
		}
	}
}

void CObjectTree::RestoreBranchState(HTREEITEM hBranch,CMap <CString,LPCTSTR,UINT,UINT>& mapPathToState)
{
	if(ItemHasChildren(hBranch))
	{
		UINT uiState;
		CString strPath;

		m_mapItemToPath.Lookup(hBranch,strPath);
		if(mapPathToState.Lookup(strPath,uiState))
		{
			if(uiState & TVIS_EXPANDED)
				Expand(hBranch,TVE_EXPAND);
			if(uiState & TVIS_SELECTED)
				SelectItem(hBranch);
		}

		HTREEITEM hNextItem;
		HTREEITEM hChildItem = GetChildItem(hBranch);
		while (hChildItem != NULL)
		{
			hNextItem = GetNextItem(hChildItem, TVGN_NEXT);

			m_mapItemToPath.Lookup(hChildItem,strPath);
			if(mapPathToState.Lookup(strPath,uiState))
			{
				if(uiState & TVIS_EXPANDED)
					Expand(hChildItem,TVE_EXPAND);
				if(uiState & TVIS_SELECTED)
					SelectItem(hChildItem);
			}
				
			RestoreBranchState(hChildItem,mapPathToState);
			
			hChildItem = hNextItem ;
		}
	}
}

BOOL CObjectTree::RefreshTree()
{
	SetRedraw(FALSE);
	CMap <CString,LPCTSTR,UINT,UINT> mapPathToState;

	//Store tree state
	if(m_hRepositoryRoot)
		SaveBranchState(m_hRepositoryRoot,mapPathToState);

	GetAllRepositoryFileFlags(m_mapFileToState);

	int nXScrollPos = GetScrollPos(SB_HORZ);
	int nYScrollPos = GetScrollPos(SB_VERT);
	
	BOOL bResult = DeleteAllItems();
	m_mapItemToPath.RemoveAll();

	m_hRepositoryRoot = InsertItem("Repository",enRepository,enSelRepository);
	m_hProfileFolder = NULL;
	m_hTestFolder = NULL;
	m_hScriptFolder = NULL;	
	bResult &= LoadProfilesFromDisk();
	bResult &= LoadScriptsFromDisk();
	bResult &= LoadTestsFromDisk();

	Expand(m_hRepositoryRoot,TVE_EXPAND);

	//Restore tree state
	RestoreBranchState(m_hRepositoryRoot,mapPathToState);

	SetScrollPos(SB_HORZ,nXScrollPos);
	SetScrollPos(SB_VERT,nYScrollPos);

	SetRedraw();
	return bResult;
}

HTREEITEM CObjectTree::AddNewScript(const CString& strType)
{
	CString strPath = GetRepositoryPath() + strScriptType + "\\";
	HTREEITEM hParent = m_hScriptFolder;

	CFileStatus fs;
	
/*	//TODO: Folders should are only one level deep (name conflicts)
	//If the selected item is a directory within 
	//the scripts folder add the new script to this folder
	HTREEITEM hItem = GetSelectedItem();
	CString strSelPath = GetItemPath(hItem);
	if(CFile::GetStatus(strSelPath,fs) && (fs.m_attribute & CFile::Attribute::directory))
	{
		if(strSelPath.Left(strPath.GetLength()).CompareNoCase(strPath)==0)
		{
			strPath = strSelPath;
			hParent = hItem;
		}	
	}
*/
	CString strFile = "NEWSCRIPT." + strType;		//Path to script
	
	int nIndex=1;
	while(CFile::GetStatus(strPath + strFile,fs))
	{
		//File exists, so increase index
		strFile.Format("NEWSCRIPT_%d.%s",nIndex++,strType);
	}

	CString strNewScript(strPath + strFile);
	CFile f;
	f.Open(strNewScript,CFile::modeCreate);
	f.Close();

	int nImage,nSelImage;
	if(GetFileItemImage(strNewScript,nImage,nSelImage))
	{
		CString strName = strFile.SpanExcluding(".");
		HTREEITEM hItem = InsertItem(strName,nImage,nSelImage,hParent);
		m_mapItemToPath.SetAt(hItem,strNewScript);
		SetItemState(hItem,INDEXTOOVERLAYMASK(1),TVIS_OVERLAYMASK);
		EnsureVisible(hItem);
		EditLabel(hItem);
		return hItem;
	}
	return NULL;
}

HTREEITEM CObjectTree::AddNewTest(const CString& strExtension)
{
	CString strPath = GetRepositoryPath() + strTestType + "\\";
	HTREEITEM hParent = m_hTestFolder;

	CFileStatus fs;
	//If the selected item is a directory within 
	//the tests folder add the new Test to this folder
	HTREEITEM hItem = GetSelectedItem();
	CString strSelPath = GetItemPath(hItem);
	if(CFile::GetStatus(strSelPath,fs) && (fs.m_attribute & CFile::Attribute::directory))
	{
		if(strSelPath.Left(strPath.GetLength()).CompareNoCase(strPath)==0)
		{
			strPath = strSelPath;
			hParent = hItem;
		}	
	}
	
	CString strFile = "NEWTEST";		//Path to test folder

	CStringArray arstrExts;
	GetTestExtensions(arstrExts);

	BOOL bFound = true;
	int nIndex=1;
	while(bFound)
	{
		bFound = false;
		for(int nExt=0;nExt<arstrExts.GetSize();nExt++)
		{
			while(CFile::GetStatus(strPath + strFile + "\\" + strFile + "." + arstrExts[nExt],fs))
			{
				//File exists, so increase index
				strFile.Format("NEWTEST_%d",nIndex++);
				bFound = true;
			}
		}
	}
	
	CString strDir(strPath + strFile);
	CreateDirectory(strDir,NULL);
	
	CString strNewTest(strDir + "\\" + strFile + "." + strExtension);
	CFile f;
	f.Open(strNewTest,CFile::modeCreate);
	f.Close();

	int nImage,nSelImage;
	if(GetFileItemImage(strNewTest,nImage,nSelImage))
	{
		CString strNam = strFile.SpanExcluding(".");
		HTREEITEM hItem = InsertItem(strNam,nImage,nSelImage,hParent);
		m_mapItemToPath.SetAt(hItem,strNewTest);
		SetItemState(hItem,INDEXTOOVERLAYMASK(1),TVIS_OVERLAYMASK);
		EnsureVisible(hItem);
		EditLabel(hItem);
		return hItem;
	}
	return NULL;
}

HTREEITEM CObjectTree::DuplicateTest(const CString& strExtension, const CString & strName)
{
	HTREEITEM hParent = m_hTestFolder;

	CStringArray arstrExts;
	GetTestExtensions(arstrExts);

	// Get the Path 
	CString strPath = GetRepositoryPath() + strTestType + "\\";

	// Build the original test file name to duplicate
	CString strOriginalFile = strPath + strName + '\\' + strName + '.' + strExtension ;

	// Find a name for the new test
	CFileStatus fs;
	int nIndex=1;
	CString strFile ;
	strFile.Format("%s_%d",strName, nIndex++);
	while(CFile::GetStatus(strPath + strFile + '\\' + strFile + "." + strExtension,fs))
	{
		//File exists, so increase index
		strFile.Format("%s_%d",strName, nIndex++);
	}
	
	CString strDir(strPath + strFile);
	CreateDirectory(strDir,NULL);
	
	CString strNewTest(strDir + "\\" + strFile + "." + strExtension);

	CTestFile testFile ;
	testFile.SetTestFilePath(strOriginalFile);

	CFile f;
	CFileException e;
	if(f.Open(strOriginalFile,CFile::modeRead,&e))
	{
		CArchive arTst(&f,CArchive::load);
		testFile.Serialize(arTst);
		testFile.SetTestFilePath(strNewTest);
		arTst.Close();
	}

	CFile fOut;
	if(fOut.Open(strNewTest,CFile::modeWrite|CFile::modeCreate))
	{
		CArchive arTst(&fOut,CArchive::store);
		testFile.Serialize(arTst);
		arTst.Close();
	}

	int nImage,nSelImage;
	if(GetFileItemImage(strNewTest,nImage,nSelImage))
	{
		CString strNam = strFile.SpanExcluding(".");
		HTREEITEM hItem = InsertItem(strNam,nImage,nSelImage,hParent);
		m_mapItemToPath.SetAt(hItem,strNewTest);
		EnsureVisible(hItem);
		return hItem;
	}
	return NULL;
}

HTREEITEM CObjectTree::AddNewProfile(const CString& strType)
{
	CString strPath = GetRepositoryPath() + strProfileType + "\\";
	HTREEITEM hParent = m_hProfileFolder;
	
	CFileStatus fs;
/*	//TODO: Folders should are only one level deep (name conflicts)
	//If the selected item is a directory within 
	//the profiles folder add the new profile to this folder
	HTREEITEM hItem = GetSelectedItem();
	CString strSelPath = GetItemPath(hItem);
	if(CFile::GetStatus(strSelPath,fs) && (fs.m_attribute & CFile::Attribute::directory))
	{
		if(strSelPath.Left(strPath.GetLength()).CompareNoCase(strPath)==0)
		{
			strPath = strSelPath;
			hParent = hItem;
		}	
	}
*/	
	CString strFile = "NEWCOLLECTOR.";		//Path to profile
	
	CFileFind FF;
	int nIndex=1;
	while(FF.FindFile(strPath + strFile + "*"))
	{
		//Filename exists, so increase index
		strFile.Format("NEWCOLLECTOR_%d.",nIndex++);
	}

	strFile += strType;
	
	CString strNewProfile(strPath + strFile);
	CFile f;
	f.Open(strNewProfile,CFile::modeCreate);
	f.Close();

	int nImage,nSelImage;
	if(GetFileItemImage(strNewProfile,nImage,nSelImage))
	{
		CString strName = strFile.SpanExcluding(".");
		HTREEITEM hItem = InsertItem(strName,nImage,nSelImage,hParent);
		m_mapItemToPath.SetAt(hItem,strNewProfile);
		SetItemState(hItem,INDEXTOOVERLAYMASK(1),TVIS_OVERLAYMASK);
		EnsureVisible(hItem);
		EditLabel(hItem);
		return hItem;
	}
	return NULL;
}

void CObjectTree::OnBeginlabeledit(NMHDR* pNMHDR, LRESULT* pResult) 
{
	TV_DISPINFO* pTVDispInfo = (TV_DISPINFO*)pNMHDR;
	
	int nImage,nSelImage;
	GetItemImage(pTVDispInfo->item.hItem,nImage,nSelImage);

	if((pTVDispInfo->item.hItem == m_hProfileFolder) || 
		(pTVDispInfo->item.hItem == m_hScriptFolder) ||
		(pTVDispInfo->item.hItem == m_hTestFolder))
	{
		*pResult = 1;
		return;
	}

	CString strPath = GetItemPath(pTVDispInfo->item.hItem);

	DWORD dwState;
	if(m_mapFileToState.Lookup(strPath,dwState) && ((dwState & RFF_RUNNING) || (dwState & RFF_LOCKED)))
	{
		AfxMessageBox("You cannot rename the current file or a running file");
		*pResult = 1;
		return;
	}

	
	BOOL bIsProfile = IsProfile(strPath);
	if(IsTest(strPath)||IsScript(strPath)||bIsProfile)
	{
		CEdit *pEdit = GetEditControl();		//Use data name if one of these types
		HWND hwnd = pEdit->GetSafeHwnd();
		m_ctlLabelEdit.SubclassWindow(hwnd);
		m_ctlLabelEdit.SetMaxLength(bIsProfile ? MAX_PATH : 60);
	}

	*pResult = 0;
}

void CObjectTree::OnEndlabeledit(NMHDR* pNMHDR, LRESULT* pResult) 
{
	TV_DISPINFO* pTVDispInfo = (TV_DISPINFO*)pNMHDR;
	
	*pResult = 0;

	CString strOldText(GetItemText(pTVDispInfo->item.hItem));

	CString strNewText(pTVDispInfo->item.pszText);
	strNewText.TrimRight();
	strNewText.TrimLeft();
	
	if(strNewText.IsEmpty() || (strOldText.CompareNoCase(strNewText)==0))
		return;
	
	TCHAR cLast = strNewText[strNewText.GetLength()-1];
	if((cLast == '-') || (cLast == '_'))
	{
	    AfxMessageBox(CString("Sorry, \"") + cLast +
		          "\" cannot be used as the final character in the name.");
	    return;
	}
	
	CString strPath = GetItemPath(pTVDispInfo->item.hItem);
	
	int nSlashPos = strPath.ReverseFind('\\');
	
	CString strNewPath(strPath.Left(nSlashPos+1));
	strNewPath += pTVDispInfo->item.pszText;
	
	CFileStatus fs;
	if(!CFile::GetStatus(strPath,fs))
		return;

	CFileFind FF;
	int nIndex=1;
	if(FF.FindFile(strNewPath + ".*"))
	{
		FF.FindNextFile();

		if(!FF.IsHidden())
		{
			AfxMessageBox(CString("The file name \"") + pTVDispInfo->item.pszText + "\" is already in use.");
			return;
		}
	}
	FF.Close();

	if(!(fs.m_attribute & CFile::Attribute::directory))
	{
		int nDotPos = strPath.ReverseFind('.');
		if(nDotPos != -1)
			strNewPath += strPath.Mid(nDotPos);
	}

	try
	{
		if(IsTest(strPath))
		{
			//If this is a test rename the folder as well
			CString strOldFolder(strPath.Left(nSlashPos));
			int nSlash = strOldFolder.ReverseFind('\\');
			CString strNewFolder(strOldFolder.Left(nSlash+1));
			strNewFolder += pTVDispInfo->item.pszText;
			
			nSlash = strNewPath.ReverseFind('\\');
			CString strNewTest = strNewFolder + strNewPath.Mid(nSlash);

			if(CFile::GetStatus(strNewFolder,fs))
			{
				AfxMessageBox(CString("The test \"") + pTVDispInfo->item.pszText + "\" already exists");
				return;
			}

			//Rename the file
			RenameSingleFile(strPath,strNewPath);
			
			//Rename the folder
			RenameSingleFile(strOldFolder,strNewFolder);
			
			strNewPath = strNewTest;
		}
		else
			//Rename the file
			RenameSingleFile(strPath,strNewPath);
			
		m_mapItemToPath.SetAt(pTVDispInfo->item.hItem,strNewPath);
	}
	catch(CFileException *e)
	{
		CString strError;
		e->GetErrorMessage(strError.GetBuffer(256), 256);
		strError.ReleaseBuffer();
		strError.Replace("an unnamed file", "\"" + strPath + "\"");

		AfxMessageBox(strError);
		e->Delete();
	}

	*pResult = 1;
}

int CObjectTree::RegisterFileExtension(const CString& strFile)
{
	int nPos = strFile.ReverseFind('.');
	if(nPos == -1)
		return -1;

	CString strExtension(strFile.Mid(nPos));

	int nIndex=0;
	if(m_mapExtToImage.Lookup(strExtension,nIndex))
		return nIndex;

	SHFILEINFO shfi;
	SHGetFileInfo( strFile, 0, &shfi, sizeof( SHFILEINFO ), 
               SHGFI_ICON | SHGFI_SHELLICONSIZE | SHGFI_SMALLICON);

	if(!shfi.hIcon || !(nIndex = (WORD)m_TreeImages.Add(shfi.hIcon)))
		return -1;

	DestroyIcon(shfi.hIcon);

	m_mapExtToImage.SetAt(strExtension,nIndex);	
	return nIndex;
}

void CObjectTree::OnBegindrag(NMHDR* pNMHDR, LRESULT* pResult) 
{
	NM_TREEVIEW* pNMTreeView = (NM_TREEVIEW*)pNMHDR;

	HTREEITEM hItem = pNMTreeView->itemNew.hItem;

	CString strPath = GetItemPath(hItem);
	int nPathLen = strPath.GetLength() + 1;
	int nDataSize = ClipboardData::GetSize() + nPathLen;

	HGLOBAL hClipData = GlobalAlloc(GMEM_FIXED,nDataSize);
	ClipboardData *pClipData = (ClipboardData *)GlobalLock(hClipData);
	
	if(IsScript(strPath))
		pClipData->nDataType = ClipboardData::enScript;
	else
	if(IsProfile(strPath))
		pClipData->nDataType = ClipboardData::enProfile;
	else
		pClipData->nDataType = ClipboardData::enTest;
	
	strcpy((char*)pClipData->GetData(),strPath);
	GlobalUnlock(hClipData);

	COleDataSource DataSource;
	DataSource.CacheGlobalData(g_ClipFormat,hClipData);
	DROPEFFECT DE = DataSource.DoDragDrop();

	*pResult = 0;
}

void CObjectTree::OnKeydown(NMHDR* pNMHDR, LRESULT* pResult) 
{
	TV_KEYDOWN* pTVKeyDown = (TV_KEYDOWN*)pNMHDR;

	switch(pTVKeyDown->wVKey)
	{
	case VK_F5:
		RefreshTree();
	break;
	case VK_DELETE:
		m_hMenuSel = GetSelectedItem();
		OnDelete();
	break;
	}

	*pResult = 0;
}

void CObjectTree::OnRclick(NMHDR* pNMHDR, LRESULT* pResult) 
{
	SendMessage(WM_CONTEXTMENU, (WPARAM) m_hWnd, GetMessagePos());
	
	*pResult = 1;
}

void CObjectTree::OnContextMenu(CWnd* pWnd, CPoint point) 
{
//	KillTimer(REFRESH_TIMER);
	if (point.x == -1 && point.y == -1)
    {
        //keystroke invocation
        CRect rect;
        GetClientRect(rect);
        ClientToScreen(rect);

        point = rect.TopLeft();
        point.Offset(5, 5);
    }

	UINT uFlags;
	ScreenToClient(&point);
	HTREEITEM hItem = HitTest(point,&uFlags);
	if(hItem)
	{
		m_hMenuSel = hItem;

	    CMenu menu;
		menu.LoadMenu(IDR_TREE_MENU);

		CString strPath = GetItemPath(hItem);
	
		CFileStatus fs;
		if(!CFile::GetStatus(strPath,fs))
		{
//			SetTimer(REFRESH_TIMER,REFRESH_DURATION,NULL);
			return;
		}
		
		//For the time being can only create new folders in test folder
		BOOL bTestFolder = FALSE;
		BOOL bProfileFolder = FALSE;
		BOOL bScriptFolder = FALSE;
		HTREEITEM hParent=hItem;
		do
		{
			if(hParent == m_hTestFolder)
			{
				bTestFolder = true;
				break;
			}
			if(hParent == m_hProfileFolder)
			{
				bProfileFolder = true;
				break;
			}
			if(hParent == m_hScriptFolder)
			{
				bScriptFolder = true;
				break;
			}
		}
		while(hParent = GetParentItem(hParent));

		if (bTestFolder || bProfileFolder)
			menu.RemoveMenu(ID_COMMENT,MF_BYCOMMAND);			
		if (bScriptFolder || bProfileFolder)
			menu.RemoveMenu(ID_DUPLICATE,MF_BYCOMMAND);			


		/*
		if(!(fs.m_attribute & CFile::Attribute::directory) || !bTestFolder)
		{
			menu.RemoveMenu(ID_NEW_FOLDER,MF_BYCOMMAND);
		}
		menu.EnableMenuItem(ID_NEW_FOLDER,MF_GRAYED);
		*/

		// Never show the new folder menu
		menu.RemoveMenu(ID_NEW_FOLDER,MF_BYCOMMAND);

		if((hItem==m_hProfileFolder) ||
			(hItem==m_hScriptFolder) ||
			(hItem==m_hTestFolder))
		{
			menu.RemoveMenu(ID_DELETE,MF_BYCOMMAND);			
			menu.RemoveMenu(ID_RENAME,MF_BYCOMMAND);			
			menu.RemoveMenu(ID_COMMENT,MF_BYCOMMAND);			
			menu.RemoveMenu(ID_DUPLICATE,MF_BYCOMMAND);			

			//Copy new menu items from mainmenu
			CMenu menuPopup;
			CString strMenu;
			CMenu *pMenu = AfxGetMainWnd()->GetMenu();
			CMenu *pFileMenu = pMenu->GetSubMenu(0);
			CMenu *pNewMenu = NULL;
			if(hItem==m_hProfileFolder)
			{
				pNewMenu = pFileMenu->GetSubMenu(CMainFrame::enNewProfileMenuPos);
				strMenu = "New &Collector";
			}
			else
			if(hItem==m_hScriptFolder)
			{
				pNewMenu = pFileMenu->GetSubMenu(CMainFrame::enNewScriptMenuPos);
				strMenu = "New &Script";
			}
			else
			if(hItem==m_hTestFolder)
			{
				pNewMenu = pFileMenu->GetSubMenu(CMainFrame::enNewTestMenuPos);
				strMenu = "New &Test";
			}
			
			if(pNewMenu)
			{
				// Create a pop-up menu.
				menuPopup.CreatePopupMenu();

				int nMenu=0;
				UINT uMenuState;
				CString str;
				while((uMenuState = pNewMenu->GetMenuState(nMenu,MF_BYPOSITION)) != -1)
				{
					// Get the menu string.
					pNewMenu->GetMenuString(nMenu,str,MF_BYPOSITION);
					if(LOBYTE(uMenuState) & MF_POPUP) // Pop-up menu.
					{
						CMenu *pSubMenu = pNewMenu->GetSubMenu(nMenu);
						menuPopup.AppendMenu(LOBYTE(uMenuState),(UINT)pSubMenu->GetSafeHmenu(),str);
					}
					else  // Is a menu item, get the ID.
					{
						int nID = pNewMenu->GetMenuItemID(nMenu);
						menuPopup.AppendMenu(LOBYTE(uMenuState),nID,str);
					}
					nMenu++;  // Get the next item.
				}
				menu.GetSubMenu(0)->AppendMenu(MF_POPUP,(UINT)menuPopup.GetSafeHmenu(),strMenu);
			}	
		}
			
		ClientToScreen(&point);
		menu.GetSubMenu(0)->TrackPopupMenu(TPM_LEFTALIGN|TPM_LEFTBUTTON|TPM_RIGHTBUTTON,point.x+5,point.y+5,this);
	}

//	SetTimer(REFRESH_TIMER,REFRESH_DURATION,NULL);
}


void CObjectTree::OnNewFolder() 
{
	CString strPath = GetItemPath(m_hMenuSel) + "\\";
	CString strFolder("NewFolder");
	
	int nIndex=1;
	CFileStatus fs;
	while(CFile::GetStatus(strPath + strFolder,fs))
		//Folder exists, so increase index
		strFolder.Format("NewFolder(%d)",nIndex++);

	CString strNewFolder(strPath + strFolder);
	if(!CreateDirectory(strNewFolder,NULL))
	{
		CString strMsg;
		strMsg.Format("Failed to create a new folder. (Error %d)",GetLastError());
		AfxMessageBox(strMsg,MB_OK);
		return;
	}

	HTREEITEM hItem = InsertItem(strFolder,enFolder,enSelFolder,m_hMenuSel);
	m_mapItemToPath.SetAt(hItem,strNewFolder);
	EnsureVisible(hItem);
	EditLabel(hItem);
}

void CObjectTree::OnDelete() 
{
	CString strPath = GetItemPath(m_hMenuSel);

	//Can't delete group folders
	if((m_hMenuSel==m_hProfileFolder) || (m_hMenuSel==m_hScriptFolder) || (m_hMenuSel==m_hTestFolder))
		return;
	
	DWORD dwState;
	if(m_mapFileToState.Lookup(strPath,dwState) && ((dwState & RFF_RUNNING) || (dwState & RFF_LOCKED)))
	{
		AfxMessageBox("You cannot delete the current file or a running file");
		return;
	}

	if(IsTest(strPath))
	{
		int nSlashPos = strPath.ReverseFind('\\');
		
		CString strFolder(strPath.Left(nSlashPos));
		CString strTest(strPath.Mid(nSlashPos+1));

		int nDotPos= strTest.ReverseFind('.');
		strTest = strTest.Left(nDotPos);

		if(AfxMessageBox("Are you sure you want to delete test \"" + strTest + "\" and all its results?",MB_YESNO)==IDNO)
			return;

		//Delete Files from disk
		DeleteFolder(strFolder);
		RefreshTree();
		return;
	}

	CFileStatus fs;
	if(!CFile::GetStatus(strPath,fs))
		return;

	if(AfxMessageBox("Are you sure you want to delete \"" + strPath + "\"?",MB_YESNO)==IDNO)
		return;
	
	if(fs.m_attribute & CFile::Attribute::directory)
	{
		//Delete Files from disk
		DeleteFolder(strPath);
		RefreshTree();
	}
	else
	{
		DeleteSingleFile(strPath);
		RefreshTree();
	}
}

BOOL CObjectTree::DeleteFolder(const CString& strFolder)
{
	CFileFind FF;
	if(!FF.FindFile(strFolder + "\\*.*"))
		return FALSE;

	BOOL bContinue=true;
	while(bContinue)
	{
		bContinue = FF.FindNextFile();
		if(FF.IsDots())
			continue;

		if(FF.IsDirectory())
		{
			if(!DeleteFolder(FF.GetFilePath()))
				return false;
		}
		else
		{
			if(!DeleteSingleFile(FF.GetFilePath()))
			{
				CString str;
				str.Format("Could not delete \"%s\"\n\n%s\nContinue with remaining files?",FF.GetFilePath(),GetErrorMessage(GetLastError()));
				if(AfxMessageBox(str,MB_YESNO)==IDNO)
					return false;
			}
		}
	}
	FF.Close();
	
	if(!RemoveDirectory(strFolder))
	{
		CString str;
		str.Format("Could not delete \"%s\"\n\n%s\nContinue with remaining files?",strFolder,GetErrorMessage(GetLastError()));
		if(AfxMessageBox(str,MB_YESNO)==IDNO)
			return false;
	}

	return true;
}

CString CObjectTree::GetErrorMessage(DWORD dwError)
{
	LPVOID lpMsgBuf;
	FormatMessage( 
		FORMAT_MESSAGE_ALLOCATE_BUFFER | 
		FORMAT_MESSAGE_FROM_SYSTEM | 
		FORMAT_MESSAGE_IGNORE_INSERTS,
		NULL,
		GetLastError(),
		MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT), // Default language
		(LPTSTR) &lpMsgBuf,
		0,
		NULL 
	);
	
	CString str((LPCTSTR)lpMsgBuf);
	str.Replace("\n"," ");
	str.Remove('\r');

	// Free the buffer.
	LocalFree( lpMsgBuf );

	return str;
}

void CObjectTree::OnRename() 
{
	EditLabel(m_hMenuSel);
}

BOOL CObjectTree::PreTranslateMessage(MSG* pMsg) 
{
	// If edit control is visible in tree view control, when you send a
	// WM_KEYDOWN message to the edit control it will dismiss the edit
	// control. When the ENTER key was sent to the edit control, the
	// parent window of the tree view control is responsible for updating
	// the item's label in TVN_ENDLABELEDIT notification code.
	if (pMsg->message == WM_KEYDOWN &&	pMsg->wParam == VK_RETURN 
			|| pMsg->wParam == VK_ESCAPE)
	{
		CEdit* pEdit = GetEditControl();
		if (pEdit)
		{
			pEdit->SendMessage(WM_KEYDOWN, pMsg->wParam, pMsg->lParam);
			return TRUE;
		}
	}

	return CTreeCtrl::PreTranslateMessage(pMsg);
}

DROPEFFECT CObjectTree::OnDragOver(COleDataObject* pDataObject, DWORD dwKeyState, CPoint point) 
{
	//Is the right sort of data available
	if(pDataObject->IsDataAvailable(g_ClipFormat))
	{
		//Find the target item
		UINT uFlags;
		HTREEITEM hItem = HitTest(point,&uFlags);
		if(hItem)
		{
			//Get path of target item
			CString strTargetPath = GetItemPath(hItem);
			
			CFileStatus fs;
			if(CFile::GetStatus(strTargetPath,fs) &&	//Check target is a directory...
				(fs.m_attribute & CFile::Attribute::directory) &&
				!IsTest(strTargetPath))	//...and that it is not a test directory
			{
				//Can only drop to a folder in test directory
				int nReposPathLen = GetRepositoryPath().GetLength();
				if(strTargetPath.Mid(nReposPathLen,strTestType.GetLength())==strTestType)
				{
					HGLOBAL hClipData = pDataObject->GetGlobalData(g_ClipFormat);
					ClipboardData *pClipData = (ClipboardData*)GlobalLock(hClipData);
					int nType = pClipData->nDataType;
					GlobalUnlock(hClipData);
					if(nType == ClipboardData::enTest)
					{
						SelectDropTarget(hItem);
						return DROPEFFECT_MOVE;
					}
				}
			}
		}
	}
	
	SelectDropTarget(NULL);
	return DROPEFFECT_NONE;
}

BOOL CObjectTree::OnDrop(COleDataObject* pDataObject, DROPEFFECT dropEffect, CPoint point) 
{
	CWaitCursor wc;

	HTREEITEM hItem = GetDropHilightItem();
	SelectDropTarget(NULL);

	HGLOBAL hClipData = pDataObject->GetGlobalData(g_ClipFormat);
	ClipboardData *pClipData = (ClipboardData*)GlobalLock(hClipData);
	int nType = pClipData->nDataType;
	
	if((nType != ClipboardData::enScript) && 
		(nType != ClipboardData::enProfile) && 
		(nType != ClipboardData::enTest))
	{
		GlobalUnlock(hClipData);
		return false;
	}

	CString strSource = (char*)pClipData->GetData();
	GlobalUnlock(hClipData);

	CString strTarget = GetItemPath(hItem);
	if(nType == ClipboardData::enTest)
	{
		int nSlashPos = strSource.ReverseFind('\\');
		strSource = strSource.Left(nSlashPos);
	}

	int nSrcLen = strSource.GetLength();	//need a /0/0 terminated string
	char *szSrc = new char[nSrcLen+2];
	strcpy(szSrc,strSource);
	szSrc[nSrcLen+1] = NULL;

	int nTgtLen = strTarget.GetLength();	//need a /0/0 terminated string
	char *szTgt = new char[nTgtLen+2];
	strcpy(szTgt,strTarget);
	szTgt[nTgtLen+1] = NULL;

	SHFILEOPSTRUCT FileOp;
	memset(&FileOp,0,sizeof(SHFILEOPSTRUCT));
	FileOp.wFunc = FO_MOVE;
	FileOp.pFrom = szSrc;
	FileOp.pTo = szTgt;
	BOOL bResult = (SHFileOperation(&FileOp)==0);

	delete []szSrc;
	delete []szTgt;
	
	RefreshTree();
	
	return bResult;
}

void CObjectTree::OnTimer(UINT nIDEvent) 
{
	// Remove the refresh on timer
	return ;

	switch(nIDEvent)
	{
	case REFRESH_TIMER:
		if(GetEditControl())
			return;

		CString strReposPath = GetRepositoryPath();

		const DWORD dwFileSysLen=100;
		TCHAR szFileSystem[dwFileSysLen];
		GetVolumeInformation(strReposPath.Left(3),NULL,0,NULL,NULL,NULL,szFileSystem,dwFileSysLen);
		BOOL bNTFS = !strcmp(szFileSystem,"NTFS");
		
		ULARGE_INTEGER uliFree,uliTotal,uliUser;
		GetDiskFreeSpaceEx(strReposPath,&uliUser,&uliTotal,&uliFree);

		BOOL bFlagsChanged = AreRepositoryFileFlagsModified();
		SetRepositoryFileFlagsModified(FALSE);

		//Refresh the tree if something has changed on the disk,
		//unless this is NTFS where small files do not change
		//the amount of free space, so refresh more often
		if(bFlagsChanged || (m_uliFreeDiskSpace.QuadPart != uliFree.QuadPart) || (bNTFS && (GetTickCount()-m_dwPrevRefreshTime >= 20000)))
		{
			m_uliFreeDiskSpace = uliFree;
			RefreshTree();
			if(bNTFS)
				m_dwPrevRefreshTime = GetTickCount();
		}
	break;
	}

	CTreeCtrl::OnTimer(nIDEvent);
}


BOOL CObjectTree::OnCommand(WPARAM wParam, LPARAM lParam) 
{
	if(!CTreeCtrl::OnCommand(wParam, lParam))
	{
		if(!AfxGetMainWnd()->SendMessage(WM_COMMAND,wParam,lParam))
			return false;
	}
	return true;
}

BOOL CObjectTree::OnEraseBkgnd(CDC* pDC) 
{
	return true;
}

void CObjectTree::OnPaint() 
{
	CPaintDC dc(this); // device context for painting
	
	CRect rc;
	GetClientRect(rc);

	CBitmap bmp;
	bmp.CreateCompatibleBitmap(&dc,rc.Width(),rc.Height());

	CDC dcMem; dcMem.CreateCompatibleDC(&dc);
	CBitmap *pOldBmp = (CBitmap*)dcMem.SelectObject(&bmp);

	Print(&dcMem,PRF_ERASEBKGND|PRF_CLIENT);
	
	dc.BitBlt(0,0,rc.Width(),rc.Height(),&dcMem,0,0,SRCCOPY);
	dcMem.SelectObject(pOldBmp);
		
	// Do not call CTreeCtrl::OnPaint() for painting messages
}

BOOL CObjectTree::DeleteSingleFile(const CString& strPath)
{
	try
	{
		//bodge to delete capture files and SBS files
		if(strPath.Right(4).CompareNoCase(".htp")==0)
		{
			int nSlashPos = strPath.ReverseFind('\\');
			int nDotPos = strPath.ReverseFind('.');
			CString strName = strPath.Left(nDotPos).Mid(nSlashPos+1);

			CFileStatus fs;
			CString strALLPath = GetRepositoryPath() + strCaptureFolder + "\\" + strName + ".ALL";
			if(CFile::GetStatus(strALLPath,fs))
				DeleteFile(strALLPath);

			CString strDatPath = strPath.Left(nDotPos) + ".DAT";
			if(CFile::GetStatus(strDatPath,fs))
				DeleteFile(strDatPath);
		}
	}
	catch(...)
	{
	}

	return DeleteFile(strPath);
}

void CObjectTree::RenameSingleFile(const CString& strOldPath,const CString& strNewPath)
{
	try
	{
		//bodge to rename capture files and SBS files
		if(strOldPath.Right(4).CompareNoCase(".htp")==0)
		{
			int nOldSlashPos = strOldPath.ReverseFind('\\');
			int nOldDotPos = strOldPath.ReverseFind('.');
			CString strOldName = strOldPath.Left(nOldDotPos).Mid(nOldSlashPos+1);

			int nNewSlashPos = strNewPath.ReverseFind('\\');
			int nNewDotPos = strNewPath.ReverseFind('.');
			CString strNewName = strNewPath.Left(nNewDotPos).Mid(nNewSlashPos+1);

			CFileStatus fs;
			CString strAllPath = GetRepositoryPath() + strCaptureFolder + "\\";
			CString strOldALLPath = strAllPath + strOldName + ".ALL";
			CString strNewALLPath = strAllPath + strNewName + ".ALL";
			if(CFile::GetStatus(strOldALLPath,fs))
				CFile::Rename(strOldALLPath,strNewALLPath);

			CString strOldDatPath = strOldPath.Left(nOldDotPos) + ".DAT";
			CString strNewDatPath = strNewPath.Left(nNewDotPos) + ".DAT";
			if(CFile::GetStatus(strOldDatPath,fs))
				CFile::Rename(strOldDatPath,strNewDatPath);
		}
	}
	catch(...)
	{
	}

	CFile::Rename(strOldPath,strNewPath);	
}

void CObjectTree::OnComment() 
{
	CString strPath = GetItemPath(m_hMenuSel);
	int nDotPos = strPath.ReverseFind('.');
	
	CString strNewPath(strPath.Left(nDotPos+1));

	CScriptCommentDlg commentDlg ;
	commentDlg.SetScriptFilePath (strNewPath + "cmt");
	if (commentDlg.DoModal() != IDOK)
		return ;	
}

void CObjectTree::OnDuplicate()  
{
	CMainFrame * pMainFrame = (CMainFrame *)AfxGetMainWnd() ;

	POSITION pos = pMainFrame->m_listPlugins.GetHeadPosition();
	while(pos)
	{
		CMainFrame::PluginInfo& PI = pMainFrame->m_listPlugins.GetNext(pos);
		if(PI.nType != CMainFrame::enTest)
			continue;
		CString strPath = GetItemPath(m_hMenuSel);
		int nPos = strPath.ReverseFind('.');
		strPath = strPath.Left(nPos) ; 
		nPos = strPath.ReverseFind('\\');
		strPath = strPath.Mid(nPos + 1) ;

		DuplicateTest(PI.strExtension, strPath);
	}
}
