/* Copyright (C) 2001 Cyrano SA 
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

// TaskGroupDefData.cpp: implementation of the TaskGroupDefData class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "TaskGroupDefData.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CTaskGroupDefData::CTaskGroupDefData()
{
	m_nStartInterval = 0;
	m_nNumVUs = 1;
	m_nNumVUsTimRes = 1;
	m_nNumVUsHttpRes = 1;
	m_nVUPerInterval = 1;
	m_bGenPageTimers = FALSE;
	m_nStartDelay = 1;
	m_bIsDuration = TRUE;		//initially manually
	m_lTaskGroupIterations = 1;	
	m_lTaskGroupDuration = -1;	//initially manually
	m_bIsScheduled = FALSE;
	strcpy(m_iScheduledTime,"+0 00:00:00");
	m_bShowAdvancedVUInfo=FALSE;
	m_bTaskGroupEnabled=FALSE;
	m_bStopOnComplete=TRUE;
	m_bIsDelayed=FALSE;

	m_lVUsPerBatch=1;
	m_iInterBatchDelay=1;
	m_iBatchRampUpTime=1;
}

CTaskGroupDefData::~CTaskGroupDefData()
{

}

BOOL CTaskGroupDefData::SaveToBuffer(LPBYTE pData,DWORD &dwBufferSize)
{
	const int nVersion=4;
	
	//Calculate required buffer size
	DWORD dwRequiredSize = sizeof(int);	//version

	dwRequiredSize += sizeof(int);		//start interval
	dwRequiredSize += sizeof(int);		//Num VUs
	dwRequiredSize += sizeof(int);		//VUs per interval
	dwRequiredSize += sizeof(int);		//VUs for HTTP results
	dwRequiredSize += sizeof(BOOL);		//Gen Page Timers
	dwRequiredSize += sizeof(int);		//Start Delay

	// modify char by int for version 4 
	dwRequiredSize += sizeof(int);		//VUs for Timer results
	dwRequiredSize += sizeof(BOOL);		//b is duration
	dwRequiredSize += sizeof(long);		//l task froup iteration
	dwRequiredSize += sizeof(long);		//l task group durations
	dwRequiredSize += sizeof(BOOL);		//b is scheduled
	dwRequiredSize += sizeof(BOOL);		//b show advanced VU info
	dwRequiredSize += sizeof(long);		//l VUs per batch
	dwRequiredSize += sizeof(int);		//inter batch dealy
	dwRequiredSize += sizeof(int);		//batch ramp up time
	dwRequiredSize += sizeof(BOOL);		//b task group enabled
	dwRequiredSize += sizeof(char[MAXSCHEDULEDTIME+1]);		//scheduled time string
	dwRequiredSize += sizeof(BOOL);		//b Stop On Complete
	dwRequiredSize += sizeof(BOOL);		//b is delayed

	

	if((dwBufferSize < dwRequiredSize)||(!pData))
	{
		dwBufferSize = dwRequiredSize;		//buffer is too small
		return false;
		
	}

	//AfxMessageBox("Saving to buffer");
	
	LPBYTE pDataPos = pData;
	
	memcpy(pDataPos,&nVersion,sizeof(int));
	pDataPos+=sizeof(int);

	memcpy(pDataPos,&m_nStartInterval,sizeof(int));
	pDataPos+=sizeof(int);

	memcpy(pDataPos,&m_nNumVUs,sizeof(int));
	pDataPos+=sizeof(int);

	memcpy(pDataPos,&m_nVUPerInterval,sizeof(int));
	pDataPos+=sizeof(int);

	memcpy(pDataPos,&m_nNumVUsHttpRes,sizeof(int));
	pDataPos+=sizeof(int);	

	memcpy(pDataPos,&m_bGenPageTimers,sizeof(BOOL));
	pDataPos+=sizeof(BOOL);

	memcpy(pDataPos,&m_nStartDelay,sizeof(int));
	pDataPos+=sizeof(int);

	memcpy(pDataPos,&m_nNumVUsTimRes,sizeof(int));
	pDataPos+=sizeof(int);

    memcpy(pDataPos,&m_bIsDuration,sizeof(BOOL));
	pDataPos+=sizeof(BOOL);
	memcpy(pDataPos,&m_lTaskGroupIterations,sizeof(long));
	pDataPos+=sizeof(long);
	memcpy(pDataPos,&m_lTaskGroupDuration,sizeof(long));
	pDataPos+=sizeof(long);

	memcpy(pDataPos,&m_bIsScheduled,sizeof(BOOL));
	pDataPos+=sizeof(BOOL);

	memcpy(pDataPos,&m_bShowAdvancedVUInfo,sizeof(BOOL));
	pDataPos+=sizeof(BOOL);
	memcpy(pDataPos,&m_lVUsPerBatch,sizeof(long));
	pDataPos+=sizeof(long);
	memcpy(pDataPos,&m_iInterBatchDelay,sizeof(int));
	pDataPos+=sizeof(int);
	memcpy(pDataPos,&m_iBatchRampUpTime,sizeof(int));
	pDataPos+=sizeof(int);
	memcpy(pDataPos,&m_bTaskGroupEnabled,sizeof(BOOL));
	pDataPos+=sizeof(BOOL);
	//check the length of the string if the following data is corrupted
	memcpy(pDataPos,&m_iScheduledTime,sizeof(char[MAXSCHEDULEDTIME+1]));
	pDataPos+=sizeof(char[MAXSCHEDULEDTIME+1]);
	memcpy(pDataPos,&m_bStopOnComplete,sizeof(BOOL));
	pDataPos+=sizeof(BOOL);
	memcpy(pDataPos,&m_bIsDelayed,sizeof(BOOL));
	pDataPos+=sizeof(BOOL);

	return true;
}

BOOL CTaskGroupDefData::LoadFromBuffer(LPBYTE pData)
{
	if(!pData)
	{
		//if no pointer use defaults, as first time ever used there is no data
		m_nStartInterval = 0;
		m_nNumVUs = 1;
		m_nNumVUsTimRes = 1;
		m_nNumVUsHttpRes = 1;
		m_nVUPerInterval = 1;
		m_bGenPageTimers = FALSE;
		m_nStartDelay = 1;
		m_bIsDuration = TRUE;		//initially manually
		m_lTaskGroupIterations = 1;	
		m_lTaskGroupDuration = -1;	//initially manually
		m_bIsScheduled = FALSE;
		strcpy(m_iScheduledTime,"+0 00:00:00");
		m_bShowAdvancedVUInfo=FALSE;
		m_bTaskGroupEnabled=FALSE;
		m_bStopOnComplete=TRUE;
		
		m_lVUsPerBatch=1;
		m_iInterBatchDelay=1;
		m_iBatchRampUpTime=1;

		m_bIsDelayed=FALSE;
		return false;
	}


	LPBYTE pDataPos = pData;

	int nVersion;
	
	memcpy(&nVersion,pDataPos,sizeof(int));
	pDataPos+=sizeof(int);

	if(nVersion <= 4)
	{
		memcpy(&m_nStartInterval,pDataPos,sizeof(int));
		pDataPos+=sizeof(int);

		memcpy(&m_nNumVUs,pDataPos,sizeof(int));
		pDataPos+=sizeof(int);

		memcpy(&m_nVUPerInterval,pDataPos,sizeof(int));
		pDataPos+=sizeof(int);

		memcpy(&m_nNumVUsHttpRes,pDataPos,sizeof(int));
		pDataPos+=sizeof(int);
		// Convert the 'H/L/N' of version to corresponding integer values
		if (nVersion == 3)
		{
			switch(m_nNumVUsHttpRes)
			{
				case (int)'H' : m_nNumVUsHttpRes = m_nNumVUs ; break ;
				case (int)'L' : m_nNumVUsHttpRes = m_nNumVUs>10?10:m_nNumVUs ; break ;
				case (int)'N' : m_nNumVUsHttpRes = 0 ; break ;
				default : m_nNumVUsHttpRes = 10 ; break ;
			}
		}

		memcpy(&m_bGenPageTimers,pDataPos,sizeof(BOOL));
		pDataPos+=sizeof(BOOL);
	}

	if((nVersion==1)||(nVersion==2)||(nVersion==3)||(nVersion==4))
	{
		memcpy(&m_nStartDelay,pDataPos,sizeof(int));
		pDataPos+=sizeof(int);
	}
	
	if((nVersion==2)||(nVersion==3))
	{
		char cStatLevel ;
		memcpy(&cStatLevel,pDataPos,sizeof(char));
		pDataPos+=sizeof(char);
		switch(cStatLevel)
		{
			case 'H' : m_nNumVUsTimRes = m_nNumVUs ; break ;
			case 'L' : m_nNumVUsTimRes = m_nNumVUs>10?10:m_nNumVUs  ; break ;
			case 'N' : m_nNumVUsTimRes = 0 ; break ;
			default : m_nNumVUsTimRes = 10 ; break ;
		}
	}
	else if (nVersion == 4)
	{
		memcpy(&m_nNumVUsTimRes,pDataPos,sizeof(int));
		pDataPos+=sizeof(int);
	}
	if(nVersion >= 3)
	{

		memcpy(&m_bIsDuration,pDataPos,sizeof(BOOL));
		pDataPos+=sizeof(BOOL);
		memcpy(&m_lTaskGroupIterations,pDataPos,sizeof(long));
		pDataPos+=sizeof(long);
		memcpy(&m_lTaskGroupDuration,pDataPos,sizeof(long));
		pDataPos+=sizeof(long);
		memcpy(&m_bIsScheduled,pDataPos,sizeof(BOOL));
		pDataPos+=sizeof(BOOL);

		memcpy(&m_bShowAdvancedVUInfo,pDataPos,sizeof(BOOL));
		pDataPos+=sizeof(BOOL);
		memcpy(&m_lVUsPerBatch,pDataPos,sizeof(long));
		pDataPos+=sizeof(long);
		memcpy(&m_iInterBatchDelay,pDataPos,sizeof(int));
		pDataPos+=sizeof(int);
		memcpy(&m_iBatchRampUpTime,pDataPos,sizeof(int));
		pDataPos+=sizeof(int);
		memcpy(&m_bTaskGroupEnabled,pDataPos,sizeof(BOOL));
		pDataPos+=sizeof(BOOL);
		memcpy(&m_iScheduledTime,pDataPos,sizeof(char[MAXSCHEDULEDTIME+1]));
		pDataPos+=sizeof(char[MAXSCHEDULEDTIME+1]);
		memcpy(&m_bStopOnComplete,pDataPos,sizeof(BOOL));
		pDataPos+=sizeof(BOOL);
		memcpy(&m_bIsDelayed,pDataPos,sizeof(BOOL));
		pDataPos+=sizeof(BOOL);
	}
	return true;
}

long CTaskGroupDefData::GetTaskIterations()
{
	return m_lTaskGroupIterations;
}

void CTaskGroupDefData::SetTaskIterations(long iterations)
{
	m_lTaskGroupIterations=iterations;
	return;
}

long CTaskGroupDefData::GetTaskDuration()
{
	return m_lTaskGroupDuration;
}

void CTaskGroupDefData::SetTaskDuration(long duration)
{
	m_lTaskGroupDuration=duration;
	return;
}

void CTaskGroupDefData::SetbIsDuration(BOOL d)
{
	m_bIsDuration=d;
	return;
}

BOOL CTaskGroupDefData::GetbIsDuration()
{
  return m_bIsDuration;
}

CString CTaskGroupDefData::GetScheduledTime()
{
	return (CString)m_iScheduledTime;
}

void CTaskGroupDefData::SetScheduledTime(CString time)
{
	
	char temp[MAXSCHEDULEDTIME+1];
	strcpy(temp,(char*)time.GetBuffer(0));
	strcpy(m_iScheduledTime,temp);
	return;
}



void CTaskGroupDefData::SetUseAdvancedVUInfo(BOOL b)
{
	m_bShowAdvancedVUInfo=b;
}

BOOL CTaskGroupDefData::GetUseAdvancedVUInfo()
{
	return m_bShowAdvancedVUInfo;
}

BOOL CTaskGroupDefData::GetTaskGroupEnabled()
{
	return m_bTaskGroupEnabled;
}

void CTaskGroupDefData::SetTaskGroupEnabled(BOOL b)
{
	m_bTaskGroupEnabled=b;
	return;
}

BOOL CTaskGroupDefData::GetStopOnComplete()
{
	return m_bStopOnComplete;
}

void CTaskGroupDefData::SetStopOnComplete(BOOL b)
{
	m_bStopOnComplete=b;
}

BOOL CTaskGroupDefData::GetIsDelayed()
{
	return m_bIsDelayed;
}

void CTaskGroupDefData::SetIsDelayed(BOOL b)
{
	m_bIsDelayed=b;
}
