/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

//////////////////////////////////////////////////////////////////////////////
//                                                                          //
//    CECCommandProcess.h                                                   //
//    ============                                                          //
//                                                                          //
//    This is the header file for the CECCommandProcess Class.              //
//                                                                          //
//////////////////////////////////////////////////////////////////////////////

//
//    Only load if not already loaded
//
#ifndef CECCommandProcess_Loaded
#define CECCommandProcess_Loaded

//
//    Include any referenced header files.
//
#include "FileSys.h"

//
//    CPSRepository Class Definition
//


//
// NOTE: ALL classes in common are assumed to be required to be compiled into the DLL, which
// your program should use at runtime. However, situations do arise where we wish to use
// a rather useful class but without wanting to use the DLL.
//
// In such cases the client program should use the 'define' "_CYRANO_NOT_DLL_" and also
// ensure that the MFC macro "AFX_EXT_CLASS" is enclosed within "#ifndef _CYRANO_NOT_DLL_".
//
//

class

#ifndef _CYRANO_NOT_DLL_
            AFX_EXT_CLASS
#endif


            CECCommandProcess : public CObject
{

    //
    //    Public Member Functions
    //
public:
    virtual BOOL DidCommandSucceed      (int SuccessId) const;

    virtual BOOL DidCommandSucceed      (CString& SuccessText) const;

    int         OpenRemoteConnection    (CString* pRemoteHostName
                                         ,CString* pRemoteUsername
                                         ,CString* pRemotePassword
                                         ,CString* pRemoteDirectory
                                         ,CEdit*   pOutputWindow
                                         ,BOOL     ClearOutputWindow);

    void        FlushRemoteConnection   ();

    BOOL        CheckForCYRRSH          (CString* pRemoteHostName, CString& strErrorMessage);

    int         CloseRemoteConnection   (BOOL FlushFirst = TRUE);

    int         SendRemoteCommand       (char* pCommandLine ,BOOL Flush = TRUE);

    //
    //    Constructor
    //
    CECCommandProcess   ();

    virtual ~CECCommandProcess();               // Destructor

    //
    //    Implementation funtions.
    //
    int RunImage        (char *pImagePath);

    int ExecuteCommand  (char *pCommandLine
                         ,CEdit* pEditBox
                         ,BOOL bClearOutput = TRUE
                                              ,CString* pRemoteNodeName = NULL
                                                                          ,CString* pRemoteUsername = NULL
                                                                                                      ,CString* pRemotePassword = NULL
                                                                                                                                  ,CString* pRemoteDirectory = NULL);
    int ExecuteCommandStr   (char *pCommandLine
                             ,CString* pstrOutput
                             ,BOOL bClearOutput = TRUE
                                                  ,CString* pRemoteNodeName = NULL
                                                                              ,CString* pRemoteUsername = NULL
                                                                                                          ,CString* pRemotePassword = NULL
                                                                                                                                      ,CString* pRemoteDirectory = NULL);


    //
    //    Protected data members and functions
    //
protected:

    //
    //    Private data members and functions
    //
private:
    void        Trace           (char* pMsg1 ,char* pMsg2);

    void        Trace           (char* pMsg1 ,CString* pMsg2);

    void        Trace           (CString& pTraceMsg);

    CEdit* m_pOutputWindow;
    BOOL m_bClearOutputWindow;
    int         InvokeImage     (char*  pImageName ,BOOL DOSCommand);

    void        ReadFromChild   () ;

    int         GoMultiThread   (char*    pCommandLine,
                                 BOOL     DosCommand,
                                 CString* pRemoteNodeName = NULL,
                                 CString* pRemoteUsername = NULL,
                                 CString* pRemotePassword = NULL,
                                 CString* pRemoteDirectory = NULL);

    void        ErrorExit       (LPTSTR lpszMessage);

    int         ExecuteRemoteCommand        (char*    pCommandLine
            ,CString* pRemoteHostName
            ,CString* pRemoteUsername
            ,CString* pRemotePassword
            ,CString* pRemoteDirectory);


    void        FormatOutput            (CString& Buffer);

    void        RecoverRemoteOutput     (BOOL bOutToAWindow = TRUE);


    //
    // Test whether the component exists
    BOOL        DoesVTESTComponentExist(const CString& strFullCmdLine );

    HANDLE      m_hChildStdoutRd;
    HANDLE      m_hChildStdoutWr;
    HANDLE      m_hChildStdinRd;

    int         m_RemoteChannel;


    //
    // Flag used to determine whether output is to be sent to a window (TRUE)
    // or a string (FALSE)
    BOOL m_bOutToAWindow;

    //
    // Used to store the output if the above flag indicates output is to a string
    CString m_strOutput;
};
#endif

