/* Copyright (C) 2001 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

// TestTabPlugin.cpp: implementation of the CTestKitPlugin class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "TestPlugin.h"
#include "TestTabPlugin.h"
#include "PluginTypes.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CTestTabPlugin::CTestTabPlugin(CString szTitle,CPluginManager* pPluginManager)
{
	CTestPluginApp* pApp = (CTestPluginApp*)AfxGetApp();
	m_pPlugin = pPluginManager->GetPlugin(TEST_PLUGIN_TAB, szTitle);
}

CTestTabPlugin::~CTestTabPlugin()
{

}

void CTestTabPlugin::SetTestFile(CTestFile *pTestFile)
{
	if(!m_pPlugin)
		return;

	typedef void	 ( * LPSETTESTFILEFUNC)     (CTestFile*);
	LPSETTESTFILEFUNC func = (LPSETTESTFILEFUNC)m_pPlugin->GetFunction("SetTestFile");  
	if(!func)
		return;
	
	func(pTestFile);
}

void CTestTabPlugin::SetDocument(CDocument *pDoc)
{
	if(!m_pPlugin)
		return;

	typedef void	 ( * LPSETDOCUMENTFUNC)     (CDocument*);
	LPSETDOCUMENTFUNC func = (LPSETDOCUMENTFUNC)m_pPlugin->GetFunction("SetDocument");
	if(!func)
		return;
	
	func(pDoc);
}

int CTestTabPlugin::GetNumTabs()
{
	if(!m_pPlugin)
		return 0;

	typedef int		 ( * LPGETNUMTABSFUNC)		(void);
	LPGETNUMTABSFUNC func = (LPGETNUMTABSFUNC)m_pPlugin->GetFunction("GetNumTabs");  
	if(!func)
		return 0;
	
	return func();	
}

CDialog* CTestTabPlugin::CreateTab(int nTab,CWnd *pParent,CString& strTabName)
{
	if(!m_pPlugin)
		return NULL;

	typedef CDialog* ( * LPCREATETABFUNC)		(int,CWnd*,CString&);
	LPCREATETABFUNC func = (LPCREATETABFUNC)m_pPlugin->GetFunction("CreateTab");  
	if(!func)
		return NULL;

	return func(nTab,pParent,strTabName);
}

void CTestTabPlugin::GetTabImages(CBitmap& bmpTabs)
{
	if(!m_pPlugin)
		return;

	typedef void	 ( * LPGETTABIMAGESFUNC)	(CBitmap&);
	LPGETTABIMAGESFUNC func = (LPGETTABIMAGESFUNC)m_pPlugin->GetFunction("GetTabImages");
	if(!func)
		return;

	func(bmpTabs);
}

BOOL CTestTabPlugin::LoadToolBars(CFrameWnd* pWndFrame,CFrameWnd *pOwner)
{
	if(!m_pPlugin)
		return FALSE;

	typedef BOOL	 ( * LPLOADTOOLBARSFUNC)	(CFrameWnd* pFrame, CFrameWnd *pOwner);
	LPLOADTOOLBARSFUNC func = (LPLOADTOOLBARSFUNC)m_pPlugin->GetFunction("LoadToolbars");
	if(!func)
		return FALSE;

	return func(pWndFrame,pOwner);
}

void CTestTabPlugin::SetCorbaInfo(CCORBAInfo *pCorbaInfo)
{
	if(!m_pPlugin)
		return;

	typedef void	 ( * LPSETCORBAINFOFUNC)    (CCORBAInfo*);
	LPSETCORBAINFOFUNC func = (LPSETCORBAINFOFUNC)m_pPlugin->GetFunction("SetCorbaInfo");
	if(!func)
		return;

	func(pCorbaInfo);
}

void CTestTabPlugin::SetPluginManager(CPluginManager* pManager)
{
	if(!m_pPlugin)
		return;

	typedef void	 ( * LPSETPLUGINMANFUNC)    (CPluginManager*);
	LPSETPLUGINMANFUNC func = (LPSETPLUGINMANFUNC)m_pPlugin->GetFunction("SetPluginManager");
	if(!func)
		return;

	func(pManager);
}

void CTestTabPlugin::SetTestActive(BOOL bActive,const CString& strReg)
{
	if(!m_pPlugin)
		return;

	typedef void	 ( * LPSETTESTACTIVEFUNC)   (BOOL bActive,const CString& strReg);
	LPSETTESTACTIVEFUNC func = (LPSETTESTACTIVEFUNC)m_pPlugin->GetFunction("SetTestActive");
	if(!func)
		return;

	func(bActive,strReg);
}

void CTestTabPlugin::LoadMenus(HMENU hMenu)
{
	if(!m_pPlugin)
		return;

	typedef void	 ( * LPLOADMENUSFUNC)		(HMENU hMenu);
	LPLOADMENUSFUNC func = (LPLOADMENUSFUNC)m_pPlugin->GetFunction("LoadMenus");
	if(!func)
		return;

	func(hMenu);
}

void CTestTabPlugin::SetNotifyWindow(HWND hWnd)
{
	if(!m_pPlugin)
		return;

	typedef void	 ( * LPSETNOTIFYWNDFUNC)	(HWND hWnd);
	LPSETNOTIFYWNDFUNC func = (LPSETNOTIFYWNDFUNC)m_pPlugin->GetFunction("SetNotifyWindow");
	if(!func)
		return;

	func(hWnd);
}

BOOL CTestTabPlugin::DisplayInStepByStepMode(int nTab)
{
	if(!m_pPlugin)
		return false;

	typedef BOOL	 ( * LPFUNC)	(int);

	LPFUNC func = (LPFUNC)m_pPlugin->GetFunction("DisplayInStepByStepMode");
	if(!func)
		return false;

	return func(nTab);
}