/* Copyright (C) 2000-2001 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/
// ScriptTaskDlg.cpp : implementation file
//

#include "stdafx.h"
#include "ScriptTaskDlg.h"
#include "TestFile.h"
#include "TestCtrl.h"
#include "BaseUIHelpID.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CScriptTaskDlg dialog


CScriptTaskDlg::CScriptTaskDlg(CWnd* pParent /*=NULL*/)
	: CDialog(CScriptTaskDlg::IDD, pParent),
	m_pTestFile(NULL),
	m_nTaskGroup(-1),
	m_nTask(-1)
{
	//{{AFX_DATA_INIT(CScriptTaskDlg)
	m_nFixedDelay = 0;
	m_nIterations = 1;
	m_nMaxDelay = 0;
	m_nMinDelay = 0;
	m_nDuration = 0;
	m_WarningString = _T("");
	//}}AFX_DATA_INIT
}


void CScriptTaskDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CScriptTaskDlg)
	DDX_Control(pDX, IDC_DURATION_SPIN, m_ctlDurationSpin);
	DDX_Control(pDX, IDC_TERMINATION_TYPE, m_ctlTerminationType);
	DDX_Control(pDX, IDC_MIN_DELAY_SPIN, m_ctlMinDelaySpin);
	DDX_Control(pDX, IDC_MAX_DELAY_SPIN, m_ctlMaxDelaySpin);
	DDX_Control(pDX, IDC_ITER_SPIN, m_ctlIterSpin);
	DDX_Control(pDX, IDC_FIXED_DELAY_SPIN, m_ctlFixedDelaySpin);
	DDX_Control(pDX, IDC_DELAY_TYPE, m_ctlDelayType);
	DDX_Text(pDX, IDC_FIXED_DELAY, m_nFixedDelay);
	DDX_Text(pDX, IDC_ITERATIONS, m_nIterations);
	DDX_Text(pDX, IDC_MAX_DELAY, m_nMaxDelay);
	DDX_Text(pDX, IDC_MIN_DELAY, m_nMinDelay);
	DDX_Text(pDX, IDC_DURATION, m_nDuration);
	DDX_Text(pDX, IDC_NO_SCRIPT_EDIT, m_WarningString);
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(CScriptTaskDlg, CDialog)
	//{{AFX_MSG_MAP(CScriptTaskDlg)
	ON_CBN_SELCHANGE(IDC_TERMINATION_TYPE, OnSelchangeTerminationType)
	ON_CBN_SELCHANGE(IDC_DELAY_TYPE, OnSelchangeDelayType)
	ON_EN_CHANGE(IDC_ITERATIONS, OnChangeIterations)
	ON_EN_CHANGE(IDC_DURATION, OnChangeDuration)
	ON_EN_CHANGE(IDC_FIXED_DELAY, OnChangeFixedDelay)
	ON_EN_CHANGE(IDC_MAX_DELAY, OnChangeMaxDelay)
	ON_EN_CHANGE(IDC_MIN_DELAY, OnChangeMinDelay)
	ON_WM_HELPINFO()
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CScriptTaskDlg message handlers
void CScriptTaskDlg::SetTestFile(CTestFile* pTestFile,int nGroup,int nTask,HWND hwndNotify)
{
	m_pTestFile = pTestFile;
	m_nTask = nTask;
	m_nTaskGroup = nGroup;
	m_hwndNotify = hwndNotify;

	DWORD dwBufSize=0;
	LPBYTE pLoadData = NULL;
	m_pTestFile->GetTaskData(nGroup,nTask,&pLoadData,dwBufSize);
	
	m_SDData.LoadFromBuffer(pLoadData);

	m_ctlTerminationType.ResetContent();
	m_ctlTerminationType.AddString("On Completion");
	m_ctlTerminationType.AddString("After Fixed Time");
	
	if(m_SDData.GetIsDuration())
		m_ctlTerminationType.SetCurSel(1);
	else
		m_ctlTerminationType.SetCurSel(0);

	m_ctlDelayType.ResetContent();
	m_ctlDelayType.AddString("Fixed Delay");
	m_ctlDelayType.AddString("Variable Delay");
	
	if(m_SDData.GetUseFixedIterDelay())
		m_ctlDelayType.SetCurSel(0);
	else
		m_ctlDelayType.SetCurSel(1);

	OnSelchangeDelayType();
	OnSelchangeTerminationType();

    const int nNumAccel = 3;
    UDACCEL arAccel[nNumAccel];
    arAccel[0].nInc = 1;
    arAccel[0].nSec = 2;
    arAccel[1].nInc = 10;
    arAccel[1].nSec = 3;
    arAccel[2].nInc = 100;
    arAccel[2].nSec = 4;

	m_ctlIterSpin.SetAccel(nNumAccel,arAccel);
	m_ctlFixedDelaySpin.SetAccel(nNumAccel,arAccel);
	m_ctlMaxDelaySpin.SetAccel(nNumAccel,arAccel);
	m_ctlMinDelaySpin.SetAccel(nNumAccel,arAccel);
	m_ctlDurationSpin.SetAccel(nNumAccel,arAccel);

	m_ctlFixedDelaySpin.SetRange(0,UD_MAXVAL);
	m_ctlMaxDelaySpin.SetRange(m_nMinDelay,UD_MAXVAL);
	m_ctlMinDelaySpin.SetRange(0,m_nMaxDelay);
	m_ctlIterSpin.SetRange(1,UD_MAXVAL);
	m_ctlDurationSpin.SetRange(0,UD_MAXVAL);

	UpdateData(FALSE);
}
void CScriptTaskDlg::OnSelchangeTerminationType() 
{
	BOOL bAfterTime = (m_ctlTerminationType.GetCurSel() == 1);
	
	GetDlgItem(IDC_ITERATIONS)->ShowWindow(bAfterTime ? SW_HIDE:SW_SHOW);
	GetDlgItem(IDC_ITER_CAPTION)->ShowWindow(bAfterTime ? SW_HIDE:SW_SHOW);
	GetDlgItem(IDC_ITER_SPIN)->ShowWindow(bAfterTime ? SW_HIDE:SW_SHOW);
	GetDlgItem(IDC_DURATION_CAPTION)->ShowWindow(bAfterTime ? SW_SHOW:SW_HIDE);
	GetDlgItem(IDC_DURATION_SPIN)->ShowWindow(bAfterTime ? SW_SHOW:SW_HIDE);
	GetDlgItem(IDC_DURATION)->ShowWindow(bAfterTime ? SW_SHOW:SW_HIDE);

	if(bAfterTime)
		m_nDuration = m_SDData.GetDuration();
	else
		m_nIterations = m_SDData.GetNumIterations();

	UpdateData(FALSE);

	if(m_SDData.GetIsDuration() != bAfterTime)
	{
		m_SDData.SetIsDuration(bAfterTime);
		SaveDataToTestFile();
	}
}

void CScriptTaskDlg::OnSelchangeDelayType() 
{
	BOOL bVarDelay = (m_ctlDelayType.GetCurSel() == 1);// 0 is fixed, 1 is variable
	
	GetDlgItem(IDC_MIN_DELAY_CAPTION)->ShowWindow(bVarDelay ? SW_SHOW:SW_HIDE);
	GetDlgItem(IDC_MIN_DELAY)->ShowWindow(bVarDelay ? SW_SHOW:SW_HIDE);
	GetDlgItem(IDC_MIN_DELAY_SPIN)->ShowWindow(bVarDelay ? SW_SHOW:SW_HIDE);
	GetDlgItem(IDC_MAX_DELAY)->ShowWindow(bVarDelay ? SW_SHOW:SW_HIDE);
	GetDlgItem(IDC_MAX_DELAY_CAPTION)->ShowWindow(bVarDelay ? SW_SHOW:SW_HIDE);
	GetDlgItem(IDC_MAX_DELAY_SPIN)->ShowWindow(bVarDelay ? SW_SHOW:SW_HIDE);

	GetDlgItem(IDC_DELAY_CAPTION)->ShowWindow(bVarDelay ? SW_HIDE:SW_SHOW);
	GetDlgItem(IDC_FIXED_DELAY)->ShowWindow(bVarDelay ? SW_HIDE:SW_SHOW);
	GetDlgItem(IDC_FIXED_DELAY_SPIN)->ShowWindow(bVarDelay ? SW_HIDE:SW_SHOW);

	if(bVarDelay)
	{
		//AfxMessageBox("min and max");
		m_nMinDelay = m_SDData.GetMinIterDelay();
		m_nMaxDelay = m_SDData.GetMaxIterDelay();
	}
	else
	{
		//AfxMessageBox("fixed");
		m_nFixedDelay = m_SDData.GetFixedIterDelay();
	}
	UpdateData(FALSE);

	if(m_SDData.GetUseFixedIterDelay() == bVarDelay)
	{
		m_SDData.SetUseFixedIterDelay(!bVarDelay);
		SaveDataToTestFile();
	}
	return;
}


void CScriptTaskDlg::SaveDataToTestFile()
{
	if((m_nTaskGroup == -1) || (m_nTask == -1))
		return;

	//Delete the old script data from the testfile
	DWORD dwBufSize;
	LPBYTE pData;
	m_pTestFile->GetTaskData(m_nTaskGroup,m_nTask,&pData,dwBufSize);
	delete []pData;

	//Get size of buffer required
	m_SDData.SaveToBuffer(NULL,dwBufSize);

	//create a buffer and save data to it
	pData = new BYTE[dwBufSize];
	m_SDData.SaveToBuffer(pData,dwBufSize);

	//set the data in the test file
	m_pTestFile->SetTaskData(m_nTaskGroup,m_nTask,pData,dwBufSize);

	//Tell test ctrl data has changed
	::SendMessage(m_hwndNotify,WMU_REFRESH_LIST,0,MAKELPARAM(m_nTaskGroup+1,m_nTask+CTestCtrl::enFirstTaskCol));
}

void CScriptTaskDlg::OnChangeIterations() 
{
	if(m_nTaskGroup == -1)
		return;

	int		nPrevNum = m_nIterations;

	UpdateData();
	if(m_nIterations<1)
	{
		m_nIterations=1;
		UpdateData(FALSE);
	}
	if (m_nIterations > UD_MAXVAL)
	{
		m_nIterations = nPrevNum;
		UpdateData(FALSE);
	}

	if(m_SDData.GetNumIterations() != m_nIterations)
	{
		m_SDData.SetNumIterations(m_nIterations);
		SaveDataToTestFile();
	}
}

void CScriptTaskDlg::OnChangeDuration() 
{
	if(m_nTaskGroup == -1)
		return;

	int		nPrevNum = m_nDuration;

	UpdateData();
	if (m_nDuration > UD_MAXVAL)
	{
		m_nDuration = nPrevNum;
		UpdateData(FALSE);
	}

	if(m_SDData.GetDuration() != m_nDuration)
	{
		m_SDData.SetDuration(m_nDuration);
		SaveDataToTestFile();
	}
}

void CScriptTaskDlg::OnChangeFixedDelay() 
{

	if(m_nTaskGroup == -1)
		return;

	int		nPrevNum = m_nFixedDelay;

	//	AfxMessageBox("changed fixed delay");
	UpdateData();
	if (m_nFixedDelay > UD_MAXVAL)
	{
		m_nFixedDelay = nPrevNum;
		UpdateData(FALSE);
	}

	if(m_SDData.GetFixedIterDelay() != m_nFixedDelay)
	{
		m_SDData.SetFixedIterDelay(m_nFixedDelay);
		SaveDataToTestFile();
	}
}

void CScriptTaskDlg::OnChangeMaxDelay() 
{
	if(m_nTaskGroup == -1)
		return;

	int		nPrevNum = m_nMaxDelay;

	UpdateData();
	if (m_nMaxDelay > UD_MAXVAL)
	{
		m_nMaxDelay = nPrevNum;
		UpdateData(FALSE);
	}
	if(m_nMinDelay>m_nMaxDelay)
	{
		m_nMaxDelay = m_nMinDelay;
		UpdateData(FALSE);
	}

	m_ctlMaxDelaySpin.SetRange(m_nMinDelay,UD_MAXVAL);
	m_ctlMinDelaySpin.SetRange(0,m_nMaxDelay);

	m_SDData.SetMaxIterDelay(m_nMaxDelay);
	SaveDataToTestFile();
}

void CScriptTaskDlg::OnChangeMinDelay() 
{
	
	if(m_nTaskGroup == -1)
		return;

	//AfxMessageBox("changed min delay");
	UpdateData();

	if(m_nMinDelay>m_nMaxDelay)
	{
		m_nMinDelay = m_nMaxDelay;
		UpdateData(FALSE);
	}

	m_ctlMaxDelaySpin.SetRange(m_nMinDelay,UD_MAXVAL);
	m_ctlMinDelaySpin.SetRange(0,m_nMaxDelay);
	
	m_SDData.SetMinIterDelay(m_nMinDelay);
	SaveDataToTestFile();
}

BOOL CScriptTaskDlg::SetWindowEnabled()
{	
	
	//get the cwnd of this dialog
	if(IsWindowEnabled()==FALSE)
	{
		m_WarningString="";//These fields cannot be edited because the Task group is in use, if a test is running disable the Task group before attempting to edit it";
		GetDlgItem(IDC_TERMINATION_TYPE)->EnableWindow(FALSE);
		GetDlgItem(IDC_ITERATIONS)->EnableWindow(FALSE);
		GetDlgItem(IDC_ITER_SPIN)->EnableWindow(FALSE);
		GetDlgItem(	IDC_DURATION_SPIN)->EnableWindow(FALSE);
		GetDlgItem(IDC_DURATION)->EnableWindow(FALSE);
		GetDlgItem(IDC_DELAY_TYPE)->EnableWindow(FALSE);
		GetDlgItem(IDC_MIN_DELAY)->EnableWindow(FALSE);
		GetDlgItem(IDC_FIXED_DELAY_SPIN)->EnableWindow(FALSE);
		GetDlgItem(IDC_MIN_DELAY_SPIN)->EnableWindow(FALSE);
		GetDlgItem(IDC_FIXED_DELAY)->EnableWindow(FALSE);
		GetDlgItem(IDC_MAX_DELAY)->EnableWindow(FALSE);
		GetDlgItem(IDC_MAX_DELAY_SPIN)->EnableWindow(FALSE);
	
	}
	else
	{
		m_WarningString="";//These fields can be edited";
		GetDlgItem(IDC_TERMINATION_TYPE)->EnableWindow(TRUE);
		GetDlgItem(IDC_ITERATIONS)->EnableWindow(TRUE);
		GetDlgItem(IDC_ITER_SPIN)->EnableWindow(TRUE);
		GetDlgItem(	IDC_DURATION_SPIN)->EnableWindow(TRUE);
		GetDlgItem(IDC_DURATION)->EnableWindow(TRUE);
		GetDlgItem(IDC_DELAY_TYPE)->EnableWindow(TRUE);
		GetDlgItem(IDC_MIN_DELAY)->EnableWindow(TRUE);
		GetDlgItem(IDC_FIXED_DELAY_SPIN)->EnableWindow(TRUE);
		GetDlgItem(IDC_MIN_DELAY_SPIN)->EnableWindow(TRUE);
		GetDlgItem(IDC_FIXED_DELAY)->EnableWindow(TRUE);
		GetDlgItem(IDC_MAX_DELAY)->EnableWindow(TRUE);
		GetDlgItem(IDC_MAX_DELAY_SPIN)->EnableWindow(TRUE);
	}
	UpdateData(FALSE);
	return true;
}

BOOL CScriptTaskDlg::OnHelpInfo(HELPINFO* pHelpInfo) 
{
	pHelpInfo->dwContextId = HIDD_SCRIPTTASK_DLG;
	return CDialog::OnHelpInfo(pHelpInfo);
}
