/* Copyright (C) 2001 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

#if !defined(AFX_TESTCTRL_H__81F44676_D4E6_11D4_94C7_00C04F89E392__INCLUDED_)
#define AFX_TESTCTRL_H__81F44676_D4E6_11D4_94C7_00C04F89E392__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000
// TestCtrl.h : header file
//
#define IDT_UPDATE_STATUS 2
/////////////////////////////////////////////////////////////////////////////
// CTestCtrl window
#include "CECGridCtrl.h"
#include "TestFile.h"
#include "mthread.hpp"
#include "TestManager.h"
class CDocument;

struct TaskGroupTuple
{
	int TaskGroupId;
	TGExecStatus TaskGroupStatus;
};


class CTestCtrl : public CECGridCtrl
{
// Construction
public:
	CTestCtrl();

// Attributes
public:
	CArray<TaskGroupTuple,TaskGroupTuple> m_TaskGroupStatuses;

// Operations
public:

	void SetDocument(CDocument* pDoc);
	void SetTestFile(CTestFile *pTestFile);

	void StartStopButtonPressed(int nRow);
	void DisableButtonPressed(int nRow);
	void Refresh();
	
	void DeleteSelection();
	BOOL StartTG(int nGroup);
	BOOL StopTG(int nGroup);

	DROPEFFECT OnDragOver(COleDataObject* pDataObject, DWORD dwKeyState, CPoint point);
	BOOL OnDrop(COleDataObject* pDataObject, DROPEFFECT dropEffect, CPoint point);

// Overrides
	// ClassWizard generated virtual function overrides
	//{{AFX_VIRTUAL(CTestCtrl)
	protected:
	virtual void PreSubclassWindow();
	//}}AFX_VIRTUAL

// Implementation
public:

	void DeleteItem(int nGroup,int nTask);
	virtual ~CTestCtrl();
	BOOL DisplayStartStopButton(int nRow);
	long ConvertToTime(CString strScheduledTime);

	HANDLE m_hCloseThread;
	// Generated message map functions
protected:
	
	//{{AFX_MSG(CTestCtrl)
	afx_msg void OnLButtonDown(UINT nFlags, CPoint point);
	afx_msg void OnTimer(UINT nIDEvent);
	afx_msg void OnContextMenu(CWnd* pWnd, CPoint point);
	afx_msg void OnDuplicatetaskgroup();
	afx_msg void OnShowControl();
	afx_msg void OnShowEnabled();
	afx_msg void OnShowHost();
	afx_msg void OnShowScheduling();
	afx_msg void OnShowStatus();
	afx_msg void OnShowTaskgroup();
	afx_msg void OnShowVus();
	afx_msg void OnDestroy();
	afx_msg void OnDeleteselection();
	afx_msg	void OnDeleteTaskGroup();
	afx_msg void OnSinglestepTaskgroup();
	//}}AFX_MSG
	afx_msg LRESULT OnRefreshList(WPARAM,LPARAM);
	DECLARE_MESSAGE_MAP()
public:
	void OutputTestFileAsText();
	CString ConvertToTime(long seconds);
	BOOL DeleteTGStatus(int nTaskGroup);
	TGExecStatus GetTGStatus(int nTaskGroup);
	BOOL SetTestInactive();
	BOOL SetTestActive(CString StrReg);
	CString GetStrReg();

	typedef enum{enNameCol,enScheduleCol,enStartStopStatusCol,enHostCol,enVUCol,enFirstTaskCol} ListColumns;
private:
	BOOL m_bTestActive;
	BOOL DisplayScheduleInfo(int nRow);
	BOOL DisplayDisableButton(int nRow);
	CTestFile *m_pTestFile;
	CString m_strTestName;
	CString m_strTestFolder;
	CImageList m_ListImages;
	CMap <CString,LPCTSTR,int,int> m_mapExtToImage;
	CDocument *m_pDoc;
	CLIPFORMAT m_ClipFormat;
	CPoint m_ptContextClick;
	CWinThread* m_pUpdateThreadctrl;
	const int m_nMaxTasks,m_nMaxGroups;
	const COLORREF m_colMissing,m_colNormal;
	typedef enum{enHostImage,enVUImage,enNameImage,enACTIVEImage,enINACTIVEImage, enSCHEDULEDImage, enDISABLEDImage,enWatchImage} ConfigImages;

	void SetupList();
	void LoadTestFile();
	void ToggleColumn(int nCol);
	void SaveColumnState();

	BOOL HitTest(CPoint pt,int& nRow,int& nCol);

	CString GetScriptOutputPath();
	CString GetTargetType(const CString& strPath);

	void InsertScript(int nItem,int nSubItem,const CString& strItemPath);
	void InsertProfile(int nItem,int nSubItem,const CString& strItemPath);

	int RegisterFileExtension(const CString& strFile);

	// property window

	BOOL GetTestActive();
	CString m_strReg;
};

/////////////////////////////////////////////////////////////////////////////

//{{AFX_INSERT_LOCATION}}
// Microsoft Visual C++ will insert additional declarations immediately before the previous line.

#endif // !defined(AFX_TESTCTRL_H__81F44676_D4E6_11D4_94C7_00C04F89E392__INCLUDED_)
