/* Copyright (C) 2001 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

// MonitorSummary.cpp : implementation file
//

#include "stdafx.h"
#include "MonitorSummary.h"
#include "TaskGroupDefData.h"
#include "CORBAInfo.h"	//access to the struct
#include "BaseUIHelpID.h"
#include <ctime.hpp>

extern CCORBAInfo *m_pCorbaInfo;

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

#define IDT_DATA 9
const int COL_WIDTH=60;
const int TASKVUCOL_WIDTH=80;
const int TASKITCOL_WIDTH=105;

/////////////////////////////////////////////////////////////////////////////
// CMonitorSummary dialog

UINT CMonitorSummary::GetUpdate(LPVOID pParam)
{
	CMonitorSummary* pWnd = (CMonitorSummary*)pParam;
	try
	{
		DWORD  dwBufSize=0;
		LPBYTE pData = NULL;

		if (!m_pCorbaInfo->HasServerBeenCreated())
			return 0;

		// initally both the the cached values are empty
		pWnd->m_nLatestSnapshot=0;
		
		do
		{
			CSingleLock lock(&(pWnd->m_ArrayLocked));
			lock.Lock();

		
			pWnd->m_TaskStatus.RemoveAll();
			for(int a=0;a<pWnd->m_pTestFile->GetNumTaskGroups();a++)
			{
				CMonitorSummary::taskStatus newitem;
				newitem.lTaskGroupId=a;

				pWnd->m_pTestFile->GetTaskGroupData(a,&pData,dwBufSize);
				CTaskGroupDefData data;
				data.LoadFromBuffer(pData);
				newitem.bEnabled=data.GetTaskGroupEnabled();
				newitem.status=m_pCorbaInfo->GetTaskGroupStatus(a,pWnd->m_strManagerReg);
				pWnd->m_TaskStatus.Add(newitem);
			}

			//STORE THE STATS IN ONE OF THE CACHES
	
			if(m_pCorbaInfo->HTTP_GetBothStatsSnapshots(pWnd->m_strManagerReg,pWnd->m_HTTPTEsSnapList1,pWnd->m_snapList1))
			{
				// the TM can do the new HTTP stats
				if(pWnd->m_nLatestSnapshot==0)
				{
					//set 1 empty 
					//set data into 2
					pWnd->m_nLatestSnapshot=1;
				}
			}
			else
			{
				//Get Original Basis stats
				m_pCorbaInfo->GetTestSummary(pWnd->m_strManagerReg,pWnd->m_snapList1);
				pWnd->m_HTTPTEsSnapList1.length(0);	//set it empty
				pWnd->m_nLatestSnapshot=0;
			}

			lock.Unlock();
		}
		while(WaitForSingleObject(pWnd->m_hCloseThread,pWnd->m_nInterval)==WAIT_TIMEOUT);
	}
	catch(...)
	{
	}
	return 0;
}

CMonitorSummary::CMonitorSummary(CWnd* pParent /*=NULL*/)
	: CDialog(CMonitorSummary::IDD, pParent)
{
	//{{AFX_DATA_INIT(CMonitorSummary)
	//}}AFX_DATA_INIT

	m_nLatestSnapshot=0;
	m_pTestFile = NULL;
	m_nTotalIndex = 0;
	m_pUpdateThread = NULL;
	m_hCloseThread = CreateEvent(NULL,FALSE,FALSE,NULL);
	m_nMaxUsedTaskCol=-1;//no tasks defined
	m_nTaskGroups=0;	//number of task groups
	m_TaskStatus.RemoveAll();

	//empty them in case they are outputted before being filled
	m_snapList1.length(0);
	m_HTTPTEsSnapList1.length(0);
	
	//get the monitoring interval from the registry
	m_nInterval = 10000;//summary updated every ten seconds as default
	HKEY hKey;
	if(RegOpenKeyEx(HKEY_CURRENT_USER,"SOFTWARE\\OpenSTA\\TestPlugin\\Settings",0,KEY_READ,&hKey)==ERROR_SUCCESS)
	{
		DWORD dwInterval;
		DWORD dwDataSize = sizeof(DWORD);
		if(RegQueryValueEx(hKey,"Task Monitoring Interval",NULL,NULL,(LPBYTE)&dwInterval,&dwDataSize)== ERROR_SUCCESS)
		{
			m_nInterval =dwInterval*1000;
		}
		RegCloseKey(hKey);
	}
}

CMonitorSummary::~CMonitorSummary()
{
	if(m_pUpdateThread)
	{
		SetEvent(m_hCloseThread);
		WaitForSingleObject(m_pUpdateThread->m_hThread,m_nInterval);
	}
	CloseHandle(m_hCloseThread);
}

void CMonitorSummary::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CMonitorSummary)
	DDX_Control(pDX, IDC_TEST_SUMMARY_TEXT, m_ctlTestText);
	DDX_Control(pDX, IDC_TASK_GROUP_SUMMARY, m_ctlTGText);
	DDX_Control(pDX, IDC_TEST_SUMMARY_LIST, m_ctlTestSummary);
	DDX_Control(pDX, IDC_SUMMARY_LIST, m_ctlSummary);
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(CMonitorSummary, CDialog)
	//{{AFX_MSG_MAP(CMonitorSummary)
	ON_WM_CONTEXTMENU()
	ON_WM_TIMER()
	ON_WM_SIZE()
	ON_WM_DESTROY()
	ON_WM_SHOWWINDOW()
	ON_WM_HELPINFO()
	ON_COMMAND(ID_MON_SHOW_TASKGROUP, OnMonShowTaskGroup )
	ON_COMMAND(ID_MON_SHOW_STATUS, OnMonShowStatus )
	ON_COMMAND(ID_MON_SHOW_DURATION, OnMonShowDuration )
	ON_COMMAND(ID_MON_SHOW_HTTPR, OnMonShowHttpR )
	ON_COMMAND(ID_MON_SHOW_HTTPB, OnMonShowHttpB )
	ON_COMMAND(ID_MON_SHOW_BYTESIN, OnMonShowBytesIn )
	ON_COMMAND(ID_MON_SHOW_BYTESOUT, OnMonShowBytesOut )
	ON_COMMAND(ID_MON_SHOW_MIN, OnMonShowMin )
	ON_COMMAND(ID_MON_SHOW_MAX, OnMonShowMax )
	ON_COMMAND(ID_MON_SHOW_AVG, OnMonShowAvg )
	ON_COMMAND(ID_MON_SHOW_VUS, OnMonShowVus )
	ON_COMMAND(ID_MON_SHOW_TASKVUS, OnMonShowTaskVus )
	ON_COMMAND(ID_MON_SHOW_TASKITERATIONS, OnMonShowTaskIterations )
	ON_COMMAND(ID_MON_SHOW_TASKPERIOD, OnMonShowTaskPeriod )
	ON_COMMAND(ID_MON_SHOW_ACTIVEVUS, OnMonShowActivevus)
	ON_COMMAND(ID_MON_SHOW_BYTESINPS, OnMonShowBytesinps)
	ON_COMMAND(ID_MON_SHOW_BYTESOUTPS, OnMonShowBytesoutps)
	ON_COMMAND(ID_MON_SHOW_HTTPBPS, OnMonShowHttpbps)
	ON_COMMAND(ID_MON_SHOW_HTTPRPS, OnMonShowHttprps)
	ON_COMMAND(ID_MON_SHOW_INACTIVEVUS, OnMonShowInactivevus)
	ON_COMMAND(ID_MON_SHOW_TASKTOTITERATIONS, OnMonShowTasktotiterations)
	ON_COMMAND(ID_MON_SHOWTEST_ACTIVEVUS, OnMonShowtestActivevus)
	ON_COMMAND(ID_MON_SHOWTEST_AVGCON, OnMonShowtestAvgcon)
	ON_COMMAND(ID_MON_SHOWTEST_AVGLAT, OnMonShowtestAvglat)
	ON_COMMAND(ID_MON_SHOWTEST_BYTESINPS, OnMonShowtestBytesinps)
	ON_COMMAND(ID_MON_SHOWTEST_BYTESOUTPS, OnMonShowtestBytesoutps)
	ON_COMMAND(ID_MON_SHOWTEST_HTTPBPS, OnMonShowtestHttpbps)
	ON_COMMAND(ID_MON_SHOWTEST_HTTPRPS, OnMonShowtestHttprps)
	ON_COMMAND(ID_MON_SHOWTEST_INACTIVEVUS, OnMonShowtestInactivevus)
	ON_COMMAND(ID_MON_SHOWTEST_MAXCON, OnMonShowtestMaxcon)
	ON_COMMAND(ID_MON_SHOWTEST_MAXLAT, OnMonShowtestMaxlat)
	ON_COMMAND(ID_MON_SHOWTEST_MINCON, OnMonShowtestMincon)
	ON_COMMAND(ID_MON_SHOWTEST_MINLAT, OnMonShowtestMinlat)
	ON_COMMAND(ID_MON_SHOWTEST_TASKITERATIONS, OnMonShowtestTaskiterations)
	ON_COMMAND(ID_MON_SHOWTEST_VUS, OnMonShowtestVus)
	ON_COMMAND(ID_MON_SHOW_ERRORVUS, OnMonShowErrorvus)
	ON_COMMAND(ID_MON_SHOW_FAILEDR, OnMonShowFailedr)
	ON_COMMAND(ID_MON_SHOW_FAILEDRPS, OnMonShowFailedrps)
	ON_COMMAND(ID_MON_SHOW_FINISHEDVUS, OnMonShowFinishedvus)
	ON_COMMAND(ID_MON_SHOW_HTTP3XXR, OnMonShowHttp3xxr)
	ON_COMMAND(ID_MON_SHOW_HTTP3XXRPS, OnMonShowHttp3xxrps)
	ON_COMMAND(ID_MON_SHOW_HTTP4XXR, OnMonShowHttp4xxr)
	ON_COMMAND(ID_MON_SHOW_HTTP4XXRPS, OnMonShowHttp4xxrps)
	ON_COMMAND(ID_MON_SHOW_HTTP5XXR, OnMonShowHttp5xxr)
	ON_COMMAND(ID_MON_SHOW_HTTP5XXRPS, OnMonShowHttp5xxrps)
	ON_COMMAND(ID_MON_SHOWTEST_FAILEDRPS, OnMonShowtestFailedrps)
	ON_COMMAND(ID_MON_SHOWTEST_HTTP3XXRPS, OnMonShowtestHttp3xxrps)
	ON_COMMAND(ID_MON_SHOWTEST_HTTP4XXRPS, OnMonShowtestHttp4xxrps)
	ON_COMMAND(ID_MON_SHOWTEST_HTTP5XXRPS, OnMonShowtestHttp5xxrps)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CMonitorSummary message handlers

BOOL CMonitorSummary::OnInitDialog() 
{
	CDialog::OnInitDialog();
	
	LoadProfileTypes();

	CWinApp *pApp = AfxGetApp();

	//if the monitoring has never been used call the initialisation routine
	if(!(pApp->GetProfileInt("Monitoring","UserDefined",0)))
	{
		DisplaySimpleColumns();
	} 

	//the test window
	m_ctlTestSummary.InsertColumn(0,"HTTP Bytes In\nper Second",LVCFMT_LEFT,100);
	if(!pApp->GetProfileInt("Test Monitor Enabled Cols","Bytes In per Second",1))
	{
		m_ctlTestSummary.SetColumnWidth(0,0);
	}
	m_ctlTestSummary.InsertColumn(1,"HTTP Bytes Out\nper Second",LVCFMT_LEFT,100);
	if(!pApp->GetProfileInt("Test Monitor Enabled Cols","Bytes Out per Second",1))
	{
		m_ctlTestSummary.SetColumnWidth(1,0);
	}
	m_ctlTestSummary.InsertColumn(2,"HTTP Total Bytes\nper Second",LVCFMT_LEFT,100);
	if(!pApp->GetProfileInt("Test Monitor Enabled Cols","Total Bytes per Second",1))
	{
		m_ctlTestSummary.SetColumnWidth(2,0);
	}
	m_ctlTestSummary.InsertColumn(3,"Started\nVUs",LVCFMT_LEFT,100);
	if(!pApp->GetProfileInt("Test Monitor Enabled Cols","Started VUs",1))
	{
		m_ctlTestSummary.SetColumnWidth(3,0);
	}
	m_ctlTestSummary.InsertColumn(4,"Active\nVUs",LVCFMT_LEFT,100);
	if(!pApp->GetProfileInt("Test Monitor Enabled Cols","Active VUs",1))
	{
		m_ctlTestSummary.SetColumnWidth(4,0);
	}
	m_ctlTestSummary.InsertColumn(5,"Inactive\nVUs",LVCFMT_LEFT,100);
	if(!pApp->GetProfileInt("Test Monitor Enabled Cols","Inactive VUs",1))
	{
		m_ctlTestSummary.SetColumnWidth(5,0);
	}
	m_ctlTestSummary.InsertColumn(6,"HTTP 3xx Requests\nper Second",LVCFMT_LEFT,100);
	if(!pApp->GetProfileInt("Test Monitor Enabled Cols","HTTP 3xx per Second",1))
	{
		m_ctlTestSummary.SetColumnWidth(6,0);
	}
	m_ctlTestSummary.InsertColumn(7,"HTTP 4xx Requests\nper Second",LVCFMT_LEFT,100);
	if(!pApp->GetProfileInt("Test Monitor Enabled Cols","HTTP 4xx Requests per Second",1))
	{
		m_ctlTestSummary.SetColumnWidth(7,0);
	}
	m_ctlTestSummary.InsertColumn(8,"HTTP 5xx Requests\nper Second",LVCFMT_LEFT,100);
	if(!pApp->GetProfileInt("Test Monitor Enabled Cols","HTTP 5xx Requests per Second",1))
	{
		m_ctlTestSummary.SetColumnWidth(8,0);
	}
	m_ctlTestSummary.InsertColumn(9,"HTTP Failed Requests\nper Second",LVCFMT_LEFT,100);
	if(!pApp->GetProfileInt("Test Monitor Enabled Cols","Failed Requests per Second",1))
	{
		m_ctlTestSummary.SetColumnWidth(9,0);
	}
	m_ctlTestSummary.InsertColumn(10,"HTTP Requests\nper Second",LVCFMT_LEFT,100);
	if(!pApp->GetProfileInt("Test Monitor Enabled Cols","Requests per Second",1))
	{
		m_ctlTestSummary.SetColumnWidth(10,0);
	}
	m_ctlTestSummary.InsertColumn(11,"Task Iterations\nper Second",LVCFMT_LEFT,100);
	if(!pApp->GetProfileInt("Test Monitor Enabled Cols","Task Iterations per Second",1))
	{
		m_ctlTestSummary.SetColumnWidth(11,0);
	}
	m_ctlTestSummary.InsertColumn(12,"Minimum\nConnection Time",LVCFMT_LEFT,100);
	if(!pApp->GetProfileInt("Test Monitor Enabled Cols","Min Connection Time",1))
	{
		m_ctlTestSummary.SetColumnWidth(12,0);
	}
	m_ctlTestSummary.InsertColumn(13,"Average\nConnection Time",LVCFMT_LEFT,100);
	if(!pApp->GetProfileInt("Test Monitor Enabled Cols","Average Connection Time",1))
	{
		m_ctlTestSummary.SetColumnWidth(13,0);
	}
	m_ctlTestSummary.InsertColumn(14,"Maximum\nConnection Time",LVCFMT_LEFT,100);
	if(!pApp->GetProfileInt("Test Monitor Enabled Cols","Max Connection Time",1))
	{
		m_ctlTestSummary.SetColumnWidth(14,0);
	}
	m_ctlTestSummary.InsertColumn(15,"Minimum Request\nLatency",LVCFMT_LEFT,100);
	if(!pApp->GetProfileInt("Test Monitor Enabled Cols","Min Request latency",1))
	{
		m_ctlTestSummary.SetColumnWidth(15,0);
	}
	m_ctlTestSummary.InsertColumn(16,"Average Request\nLatency",LVCFMT_LEFT,100);
	if(!pApp->GetProfileInt("Test Monitor Enabled Cols","Average Request latency",1))
	{
		m_ctlTestSummary.SetColumnWidth(16,0);
	}
	m_ctlTestSummary.InsertColumn(17,"Maximum Request\nLatency",LVCFMT_LEFT,100);
	if(!pApp->GetProfileInt("Test Monitor Enabled Cols","Max Request latency",1))
	{
		m_ctlTestSummary.SetColumnWidth(17,0);
	}

	m_ctlTestSummary.InsertItem(0,"-");
	m_ctlTestSummary.SetItemText(0,1,"-");
	m_ctlTestSummary.SetItemText(0,2,"-");
	m_ctlTestSummary.SetItemText(0,3,"-");
	m_ctlTestSummary.SetItemText(0,4,"-");
	m_ctlTestSummary.SetItemText(0,5,"-");
	m_ctlTestSummary.SetItemText(0,6,"-");
	m_ctlTestSummary.SetItemText(0,7,"-");
	m_ctlTestSummary.SetItemText(0,8,"-");
	m_ctlTestSummary.SetItemText(0,9,"-");
	m_ctlTestSummary.SetItemText(0,10,"-");
	m_ctlTestSummary.SetItemText(0,11,"-");
	m_ctlTestSummary.SetItemText(0,12,"-");
	m_ctlTestSummary.SetItemText(0,13,"-");
	m_ctlTestSummary.SetItemText(0,14,"-");
	m_ctlTestSummary.SetItemText(0,15,"-");
	m_ctlTestSummary.SetItemText(0,16,"-");
	m_ctlTestSummary.SetItemText(0,17,"-");

	//the summary window
	
	m_ctlSummary.InsertColumn(enTaskGroupCol,"Task\nGroup",LVCFMT_LEFT,100);
	if(!pApp->GetProfileInt("Monitor Enabled Cols","TaskGroup",1))
	{
		m_ctlSummary.SetColumnWidth(enTaskGroupCol,0);
	}
	m_ctlSummary.InsertColumn(enStatusCol,"Status",LVCFMT_LEFT,100);
	if(!pApp->GetProfileInt("Monitor Enabled Cols","Status",1))
	{
		m_ctlSummary.SetColumnWidth(enStatusCol,0);
	}
	m_ctlSummary.InsertColumn(enDurationCol,"Duration",LVCFMT_LEFT,75);
	if(!pApp->GetProfileInt("Monitor Enabled Cols","Duration",1))
	{
		m_ctlSummary.SetColumnWidth(enDurationCol,0);
	}
	m_ctlSummary.InsertColumn(enHTTPrCol,"HTTP Requests",LVCFMT_LEFT,100);
	if(!pApp->GetProfileInt("Monitor Enabled Cols","HTTPr",1))
	{
		m_ctlSummary.SetColumnWidth(enHTTPrCol,0);
	}
	m_ctlSummary.InsertColumn(enHTTPrperSecondCol,"HTTP Requests\nper Second",LVCFMT_LEFT,100);
	if(!pApp->GetProfileInt("Monitor Enabled Cols","HTTPrperSecond",1))
	{
		m_ctlSummary.SetColumnWidth(enHTTPrperSecondCol,0);
	}
	m_ctlSummary.InsertColumn(enHTTPrFailedCol,"Failed Requests",LVCFMT_LEFT,100);
	if(!pApp->GetProfileInt("Monitor Enabled Cols","HTTPrFailed",1))
	{
		m_ctlSummary.SetColumnWidth(enHTTPrFailedCol,0);
	}
	m_ctlSummary.InsertColumn(enHTTPrFailedperSecondCol,"Failed Requests\nper Second",LVCFMT_LEFT,120);
	if(!pApp->GetProfileInt("Monitor Enabled Cols","HTTPrFailedperSecond",1))
	{
		m_ctlSummary.SetColumnWidth(enHTTPrFailedperSecondCol,0);
	}
	m_ctlSummary.InsertColumn(enHTTP3xxCol,"HTTP 3xx Requests",LVCFMT_LEFT,100);
	if(!pApp->GetProfileInt("Monitor Enabled Cols","HTTP3xx",1))
	{
		m_ctlSummary.SetColumnWidth(enHTTP3xxCol,0);
	}
	m_ctlSummary.InsertColumn(enHTTP3xxColPerSecond,"HTTP 3xx Requests\nPer Second",LVCFMT_LEFT,100);
	if(!pApp->GetProfileInt("Monitor Enabled Cols","HTTP3xxPerSecond",1))
	{
		m_ctlSummary.SetColumnWidth(enHTTP3xxColPerSecond,0);
	}
	m_ctlSummary.InsertColumn(enHTTP4xxCol,"HTTP 4xx Requests",LVCFMT_LEFT,100);
	if(!pApp->GetProfileInt("Monitor Enabled Cols","HTTP4xx",1))
	{
		m_ctlSummary.SetColumnWidth(enHTTP4xxCol,0);
	}
	m_ctlSummary.InsertColumn(enHTTP4xxColPerSecond,"HTTP 4xx Requests\nPer Second",LVCFMT_LEFT,100);
	if(!pApp->GetProfileInt("Monitor Enabled Cols","HTTP4xxPerSecond",1))
	{
		m_ctlSummary.SetColumnWidth(enHTTP4xxColPerSecond,0);
	}
	m_ctlSummary.InsertColumn(enHTTP5xxCol,"HTTP 5xx Requests",LVCFMT_LEFT,100);
	if(!pApp->GetProfileInt("Monitor Enabled Cols","HTTP5xx",1))
	{
		m_ctlSummary.SetColumnWidth(enHTTP5xxCol,0);
	}
	m_ctlSummary.InsertColumn(enHTTP5xxColPerSecond,"HTTP 5xx Requests\nPer Second",LVCFMT_LEFT,100);
	if(!pApp->GetProfileInt("Monitor Enabled Cols","HTTP5xxPerSecond",1))
	{
		m_ctlSummary.SetColumnWidth(enHTTP5xxColPerSecond,0);
	}
	m_ctlSummary.InsertColumn(enHTTPbCol,"HTTP Total Bytes",LVCFMT_LEFT,100);
	if(!pApp->GetProfileInt("Monitor Enabled Cols","HTTPb",1))
	{
		m_ctlSummary.SetColumnWidth(enHTTPbCol,0);
	}

	m_ctlSummary.InsertColumn(enHTTPbperSecondCol,"HTTP Total Bytes\nPer Second",LVCFMT_LEFT,100);
	if(!pApp->GetProfileInt("Monitor Enabled Cols","HTTPbperSecond",1))
	{
		m_ctlSummary.SetColumnWidth(enHTTPbperSecondCol,0);
	}

	m_ctlSummary.InsertColumn(enBytesInCol,"HTTP Bytes In",LVCFMT_LEFT,85);
	if(!pApp->GetProfileInt("Monitor Enabled Cols","BytesIn",1))
	{
		m_ctlSummary.SetColumnWidth(enBytesInCol,0);
	}

	m_ctlSummary.InsertColumn(enBytesInperSecondCol,"HTTP Bytes In\nPer Second",LVCFMT_LEFT,85);
	if(!pApp->GetProfileInt("Monitor Enabled Cols","BytesInperSecond",1))
	{
		m_ctlSummary.SetColumnWidth(enBytesInperSecondCol,0);
	}
	m_ctlSummary.InsertColumn(enBytesOutCol,"HTTP Bytes Out",LVCFMT_LEFT,85);
	if(!pApp->GetProfileInt("Monitor Enabled Cols","BytesOut",1))
	{
		m_ctlSummary.SetColumnWidth(enBytesOutCol,0);
	}

	m_ctlSummary.InsertColumn(enBytesOutperSecondCol,"HTTP Bytes Out\nPer Second",LVCFMT_LEFT,85);
	if(!pApp->GetProfileInt("Monitor Enabled Cols","BytesOutperSecond",1))
	{
		m_ctlSummary.SetColumnWidth(enBytesOutperSecondCol,0);
	}

	m_ctlSummary.InsertColumn(enMinCol,"Minimum Request\nLatency",LVCFMT_LEFT,100);
	if(!pApp->GetProfileInt("Monitor Enabled Cols","Min",1))
	{
		m_ctlSummary.SetColumnWidth(enMinCol,0);
	}
	
	m_ctlSummary.InsertColumn(enAvgCol,"Average Request\nLatency",LVCFMT_LEFT,100);
	if(!pApp->GetProfileInt("Monitor Enabled Cols","Avg",1))
	{
		m_ctlSummary.SetColumnWidth(enAvgCol,0);
	}

	m_ctlSummary.InsertColumn(enMaxCol,"Maximum Request\nLatency",LVCFMT_LEFT,100);
	if(!pApp->GetProfileInt("Monitor Enabled Cols","Max",1))
	{
		m_ctlSummary.SetColumnWidth(enMaxCol,0);
	}
	
	m_ctlSummary.InsertColumn(enTaskIterationsperSecondCol,"Total Task Iterations\nPer Second",LVCFMT_LEFT,120);
	if(!pApp->GetProfileInt("Monitor Enabled Cols","TaskIterationsperSecond",1))
	{
		m_ctlSummary.SetColumnWidth(enTaskIterationsperSecondCol,0);
	}

	m_ctlSummary.InsertColumn(enTotalVuCol,"Total VUs",LVCFMT_LEFT,75);
	if(!pApp->GetProfileInt("Monitor Enabled Cols","VUs",1))
	{
		m_ctlSummary.SetColumnWidth(enTotalVuCol,0);
	}

	m_ctlSummary.InsertColumn(enActiveVuCol,"Active VUs",LVCFMT_LEFT,75);
	if(!pApp->GetProfileInt("Monitor Enabled Cols","ActiveVu",1))
	{
		m_ctlSummary.SetColumnWidth(enActiveVuCol,0);
	}
	
	m_ctlSummary.InsertColumn(enInactiveVuCol,"Inactive VUs",LVCFMT_LEFT,75);
	if(!pApp->GetProfileInt("Monitor Enabled Cols","InactiveVu",1))
	{
		m_ctlSummary.SetColumnWidth(enInactiveVuCol,0);
	}

	m_ctlSummary.InsertColumn(enOnErrorVuCol,"Aborted VUs",LVCFMT_LEFT,75);
	if(!pApp->GetProfileInt("Monitor Enabled Cols","AbortedVu",1))
	{
		m_ctlSummary.SetColumnWidth(enOnErrorVuCol,0);
	}
	m_ctlSummary.InsertColumn(enFinishedVuCol,"Finished VUs",LVCFMT_LEFT,75);
	if(!pApp->GetProfileInt("Monitor Enabled Cols","FinishedVu",1))
	{
		m_ctlSummary.SetColumnWidth(enFinishedVuCol,0);
	}
	ASSERT(m_pTestFile != NULL);


	for (int group = 0; group<m_pTestFile->GetNumTaskGroups(); group++)
	{
		m_ctlSummary.InsertItem(group,m_pTestFile->GetTaskGroupName(group));
		m_ctlSummary.SetItemText(group,enStatusCol,"-");
		m_ctlSummary.SetItemText(group,enDurationCol,"-");
		m_ctlSummary.SetItemText(group,enHTTPrCol,"-");		
		m_ctlSummary.SetItemText(group,enHTTPrFailedCol,"-");
		m_ctlSummary.SetItemText(group,enHTTP3xxCol,"-");
		m_ctlSummary.SetItemText(group,enHTTP4xxCol,"-");
		m_ctlSummary.SetItemText(group,enHTTP5xxCol,"-");
		m_ctlSummary.SetItemText(group,enHTTPbCol,"-");
		m_ctlSummary.SetItemText(group,enBytesInCol,"-");
		m_ctlSummary.SetItemText(group,enBytesOutCol,"-");
		m_ctlSummary.SetItemText(group,enMinCol,"-");		
		m_ctlSummary.SetItemText(group,enMaxCol,"-");
		m_ctlSummary.SetItemText(group,enAvgCol,"-");
		m_ctlSummary.SetItemText(group,enTotalVuCol,"-");
		m_ctlSummary.SetItemText(group,enHTTPbperSecondCol,"-");
		m_ctlSummary.SetItemText(group,enBytesInperSecondCol,"-");
		m_ctlSummary.SetItemText(group,enBytesOutperSecondCol,"-");
		m_ctlSummary.SetItemText(group,enHTTPrperSecondCol,"-");
		m_ctlSummary.SetItemText(group,enHTTPrFailedperSecondCol,"-");
		m_ctlSummary.SetItemText(group,enHTTP3xxColPerSecond,"-");
		m_ctlSummary.SetItemText(group,enHTTP4xxColPerSecond,"-");
		m_ctlSummary.SetItemText(group,enHTTP5xxColPerSecond,"-");
		m_ctlSummary.SetItemText(group,enTaskIterationsperSecondCol,"-");
		m_ctlSummary.SetItemText(group,enActiveVuCol,"-");
		m_ctlSummary.SetItemText(group,enInactiveVuCol,"-");
		m_ctlSummary.SetItemText(group,enOnErrorVuCol,"-");
		m_ctlSummary.SetItemText(group,enFinishedVuCol,"-");


		for(int tasks=0;tasks<m_pTestFile->GetNumTasks(group);tasks++)
		{
			if(tasks>m_nMaxUsedTaskCol)
			{
				//create new columns
				CString heading;
				heading.Format("Task %d\n(VU's)",tasks+1);
				m_ctlSummary.InsertColumn((tasks*3)+enFirstTaskCol,heading,LVCFMT_LEFT,TASKVUCOL_WIDTH);
				if(!pApp->GetProfileInt("Monitor Enabled Cols","TaskVUs",1))
				{
					m_ctlSummary.SetColumnWidth((tasks*3)+enFirstTaskCol,0);
				}
				heading.Format("Task %d\n(Iterations)",tasks+1);
				m_ctlSummary.InsertColumn((tasks*3)+enFirstTaskCol+1,heading,LVCFMT_LEFT,TASKITCOL_WIDTH);
				heading.Format("Task %d\n(Period)",tasks+1);
				if(!pApp->GetProfileInt("Monitor Enabled Cols","TaskIterations",1))
				{
					m_ctlSummary.SetColumnWidth((tasks*3)+enFirstTaskCol+1,0);
				}
				m_ctlSummary.InsertColumn((tasks*3)+enFirstTaskCol+2,heading,LVCFMT_LEFT,TASKITCOL_WIDTH);
				if(!pApp->GetProfileInt("Monitor Enabled Cols","TaskPeriod",1))
				{
					m_ctlSummary.SetColumnWidth((tasks*3)+enFirstTaskCol+2,0);
				}
				m_nMaxUsedTaskCol=tasks;
			}
			m_ctlSummary.SetItemText(group,(tasks*3)+enFirstTaskCol,"-");
			m_ctlSummary.SetItemText(group,(tasks*3)+enFirstTaskCol+1,"-");
			m_ctlSummary.SetItemText(group,(tasks*3)+enFirstTaskCol+2,"-");
		}
	}

	///////////////////////SET UP THE MULTILINE HEADER CONTROL for the TEST SUMMARY////////////
	
	CFont NewHeaderFont;
	NewHeaderFont.CreatePointFont(190,"MS Serif");

	CHeaderCtrl* pTestHeader = NULL;
	pTestHeader=m_ctlTestSummary.GetHeaderCtrl();

	if(pTestHeader!=NULL)
	{
		VERIFY(m_TestHeaderCtrl.SubclassWindow(pTestHeader->m_hWnd));	

		//A BIGGER FONT MAKES THE CONTROL BIGGER
		m_TestHeaderCtrl.SetFont(&NewHeaderFont);

		HDITEM hdItem;
		hdItem.mask = HDI_FORMAT;

		for(int i=0; i<m_TestHeaderCtrl.GetItemCount(); i++)
		{
			m_TestHeaderCtrl.GetItem(i,&hdItem);
			hdItem.fmt|= HDF_OWNERDRAW;
			m_TestHeaderCtrl.SetItem(i,&hdItem);
		}
	}

	///////////////////////SET UP THE MULTILINE HEADER CONTROL for the Task Group SUMMARY//////
	
	CHeaderCtrl* pTGHeader = NULL;
	pTGHeader=m_ctlSummary.GetHeaderCtrl();

	if(pTGHeader!=NULL)
	{
		VERIFY(m_TGHeaderCtrl.SubclassWindow(pTGHeader->m_hWnd));	

		//A BIGGER FONT MAKES THE CONTROL BIGGER
		m_TGHeaderCtrl.SetFont(&NewHeaderFont);

		HDITEM hdItem;
		hdItem.mask = HDI_FORMAT;

		for(int i=0; i<m_TGHeaderCtrl.GetItemCount(); i++)
		{
			m_TGHeaderCtrl.GetItem(i,&hdItem);
			hdItem.fmt|= HDF_OWNERDRAW;
			m_TGHeaderCtrl.SetItem(i,&hdItem);
		}
	}

	//////////////////////////////////////////////////////////////////////

	m_nTaskGroups=group+1;
	m_pUpdateThread = AfxBeginThread(GetUpdate,this);
	SetTimer(IDT_DATA, m_nInterval, NULL);

	//pretend to move it to force a resize
	CRect r;
	GetClientRect(&r);
	SetWindowPos(NULL,0,0,r.Width(),r.Height(),NULL);

	return TRUE;  // return TRUE unless you set the focus to a control
	              // EXCEPTION: OCX Property Pages should return FALSE
}

BOOL CMonitorSummary::Init(CString strManagerReg, CTestFile* pTestFile)
{
	m_pTestFile = pTestFile;
	m_strManagerReg = strManagerReg;

	return true;
}

void CMonitorSummary::OnTimer(UINT nIDEvent) 
{
	UpdateSummary();
	CDialog::OnTimer(nIDEvent);
}

void CMonitorSummary::UpdateSummary()
{
	CSingleLock lock(&m_ArrayLocked);
	lock.Lock();

	//also locked in Update Member function + OnShowWindow

	for(int a=0;a<m_TaskStatus.GetSize();a++)
	{
		long ltgId=m_TaskStatus[a].lTaskGroupId;
		if(m_TaskStatus[a].bEnabled)
		{
			switch(m_TaskStatus[a].status)
			{
				case ACTIVE: m_ctlSummary.SetItemText(ltgId,enStatusCol,"Running");
				break;
				case INACTIVE: m_ctlSummary.SetItemText(ltgId,enStatusCol,"Waiting enabled");
				break;
				case DELAYED: m_ctlSummary.SetItemText(ltgId,enStatusCol,"Scheduled");
				break;
				case FAILED: m_ctlSummary.SetItemText(ltgId,enStatusCol,"Failed");
				break;
				case STOPPED: m_ctlSummary.SetItemText(ltgId,enStatusCol,"Stopped");
				break;
				case STOPPING: m_ctlSummary.SetItemText(ltgId,enStatusCol,"Stopping");
				break;
				case COMPLETED: m_ctlSummary.SetItemText(ltgId,enStatusCol,"Completed");
				break;
				default:m_ctlSummary.SetItemText(ltgId,enStatusCol,"No status");
			}
		}
		else
		{
			m_ctlSummary.SetItemText(ltgId,enStatusCol,"Disabled");
		}
	}

	
	
	/////////////////////////////////////////
	//
	// original stats - absolute values at current time
	//

	TaskGroupSnapList TGSnap;
	CString temp;

	for (int i = 0; i<(int)m_snapList1.length(); i++)
	{
		TGSnap=m_snapList1[i].seqTaskGroupSnaps;
		for(int j = 0; j<(int)TGSnap.length(); j++)
		{
			long lTaskGroupId=TGSnap[j].lTaskGroupId;
			m_ctlSummary.SetItemText(lTaskGroupId,enTaskGroupCol,m_pTestFile->GetTaskGroupName(lTaskGroupId));

			if((TGSnap[j].lRunTime>0)&&
				(m_TaskStatus[lTaskGroupId].status==ACTIVE ||m_TaskStatus[lTaskGroupId].status==STOPPING))
			{
			
				m_ctlSummary.SetItemText(lTaskGroupId,enDurationCol,ConvertTimeToString(TGSnap[j].lRunTime));

				CTestFile::TaskType taskType=m_pTestFile->GetTaskType(lTaskGroupId,enTaskGroupCol);

				if(taskType==CTestFile::enScript)
				{	
					temp.Format("%d", TGSnap[j].lRequests);
					m_ctlSummary.SetItemText(lTaskGroupId,enHTTPrCol,temp);
					temp.Format("%f",TGSnap[j].dBytes);
					m_ctlSummary.SetItemText(lTaskGroupId,enHTTPbCol,temp);

					DWORD  dwBufSize=0;
					LPBYTE pData = NULL;
					m_pTestFile->GetTaskGroupData(lTaskGroupId,&pData,dwBufSize);
					CTaskGroupDefData data;
					data.LoadFromBuffer(pData);
					temp.Format("%d/%d",TGSnap[j].lActiveUsers,data.GetNumVUs());
					m_ctlSummary.SetItemText(lTaskGroupId,enTotalVuCol,temp);
				}
				else if(taskType==CTestFile::enProfile)
				{
					m_ctlSummary.SetItemText(lTaskGroupId,enHTTPrCol,"-");
					m_ctlSummary.SetItemText(lTaskGroupId,enHTTPbCol,"-");
					m_ctlSummary.SetItemText(lTaskGroupId,enTotalVuCol,"-");
				}

				TaskSnapList seqTasks = TGSnap[j].seqTaskSnaps;
				for (int k = 0; k<(int)seqTasks.length(); k++)
				{
					if(seqTasks[k].lTaskId>m_nMaxUsedTaskCol)
					{
						temp.Format("Task %d\n(VU's)",seqTasks[k].lTaskId+1);
						m_ctlSummary.InsertColumn((seqTasks[k].lTaskId*3)+enFirstTaskCol,temp,LVCFMT_LEFT,	m_ctlSummary.GetColumnWidth(enFirstTaskCol));
						temp.Format("Task %d\n(Iterations)",seqTasks[k].lTaskId+1);
						m_ctlSummary.InsertColumn((seqTasks[k].lTaskId*3)+enFirstTaskCol+1,temp,LVCFMT_LEFT,	m_ctlSummary.GetColumnWidth(enFirstTaskCol+1));
						temp.Format("Task %d\n(Period)",seqTasks[k].lTaskId+1);
						m_ctlSummary.InsertColumn((seqTasks[k].lTaskId*3)+enFirstTaskCol+2,temp,LVCFMT_LEFT,	m_ctlSummary.GetColumnWidth(enFirstTaskCol+2));
						m_nMaxUsedTaskCol=seqTasks[k].lTaskId;

						//make the extra columns have the two level heading
						CHeaderCtrl* pTGHeader = NULL;
						pTGHeader=m_ctlSummary.GetHeaderCtrl();

						if(pTGHeader!=NULL)
						{
							VERIFY(m_TGHeaderCtrl.SubclassWindow(pTGHeader->m_hWnd));	

							CFont NewHeaderFont;
							NewHeaderFont.CreatePointFont(190,"MS Serif");
							m_TGHeaderCtrl.SetFont(&NewHeaderFont);

							HDITEM hdItem;
							hdItem.mask = HDI_FORMAT;

							for(int i=0; i<m_TGHeaderCtrl.GetItemCount(); i++)
							{
								m_TGHeaderCtrl.GetItem(i,&hdItem);
								hdItem.fmt|= HDF_OWNERDRAW;
								m_TGHeaderCtrl.SetItem(i,&hdItem);
							}
						}
					}
					if(taskType==CTestFile::enScript)
					{
						temp.Format("%d",seqTasks[k].lActiveUsers);
						m_ctlSummary.SetItemText(lTaskGroupId,(seqTasks[k].lTaskId*3)+enFirstTaskCol,temp);
					}
					else if(taskType==CTestFile::enProfile)
					{
						m_ctlSummary.SetItemText(lTaskGroupId,(seqTasks[k].lTaskId*3)+enFirstTaskCol,"-");
					}
				}
						
				for(int j=seqTasks.length();j<enFirstTaskCol+m_nMaxUsedTaskCol;j++)
				{
					//incase some tasks have been removed
					m_ctlSummary.SetItemText(lTaskGroupId,(j*3)+enFirstTaskCol,"");
					m_ctlSummary.SetItemText(lTaskGroupId,(j*3)+enFirstTaskCol+1,"");
					m_ctlSummary.SetItemText(lTaskGroupId,(j*3)+enFirstTaskCol+2,"");
				}
			}
			else
			{
				//duration is 0
				m_ctlSummary.SetItemText(lTaskGroupId,enDurationCol,"-");
				m_ctlSummary.SetItemText(lTaskGroupId,enHTTPrCol,"-");
				m_ctlSummary.SetItemText(lTaskGroupId,enHTTPbCol,"-");
				m_ctlSummary.SetItemText(lTaskGroupId,enTotalVuCol,"-");
				for(int task2=0;task2<m_pTestFile->GetNumTasks(lTaskGroupId); task2++)
				{
					if(task2>m_nMaxUsedTaskCol)
					{
						temp.Format("Task %d\n(VU's)",(task2*3)+1);
						m_ctlSummary.InsertColumn((task2*3)+enFirstTaskCol,temp,LVCFMT_LEFT,m_ctlSummary.GetColumnWidth(enFirstTaskCol));
						temp.Format("Task %d\n(Iterations)",(task2*3)+1);
						m_ctlSummary.InsertColumn((task2*3)+enFirstTaskCol+1,temp,LVCFMT_LEFT,m_ctlSummary.GetColumnWidth(enFirstTaskCol+1));
						temp.Format("Task %d\n(Period)",(task2*3)+2);
						m_ctlSummary.InsertColumn((task2*3)+enFirstTaskCol+2,temp,LVCFMT_LEFT,m_ctlSummary.GetColumnWidth(enFirstTaskCol+2));
						m_nMaxUsedTaskCol=task2;

						//make the extra columns have the two level heading
						CHeaderCtrl* pTGHeader = NULL;
						pTGHeader=m_ctlSummary.GetHeaderCtrl();
	
						if(pTGHeader!=NULL)
						{
							VERIFY(m_TGHeaderCtrl.SubclassWindow(pTGHeader->m_hWnd));	
	
							CFont NewHeaderFont;
							NewHeaderFont.CreatePointFont(190,"MS Serif");
							m_TGHeaderCtrl.SetFont(&NewHeaderFont);
	
							HDITEM hdItem;
							hdItem.mask = HDI_FORMAT;

							for(int i=0; i<m_TGHeaderCtrl.GetItemCount(); i++)
							{
								m_TGHeaderCtrl.GetItem(i,&hdItem);
								hdItem.fmt|= HDF_OWNERDRAW;
								m_TGHeaderCtrl.SetItem(i,&hdItem);
							}
						}
					}
					m_ctlSummary.SetItemText(lTaskGroupId,(task2*3)+enFirstTaskCol,"-");
					m_ctlSummary.SetItemText(lTaskGroupId,(task2*3)+enFirstTaskCol+1,"-");
					m_ctlSummary.SetItemText(lTaskGroupId,(task2*3)+enFirstTaskCol+2,"-");
				}
			}
		
		}
	}

  
	////////////////////////
	// http stats - absolute values at current time
	//
	//	If a daemon doesnt support HTTP stats then the "-" will remain in all values place
	//
	// m_HTTPTEsSnapList contains a sequence of test executer snapshots
	// columns 6 to 11 are per task group HTTP stats
	//
	//	6	Bytes In
	//	7	Bytes Out
	//	8	Min Request
	//	9	Max Request
	//	10	Avg Request
	//	11	HTTP Errors
	//
	//
	////////////////////////

	//empty the snapshot array
	if(m_nLatestSnapshot==1)
	{
		m_DataTime2.RemoveAll();
	}
	else if(m_nLatestSnapshot==2)
	{
		m_DataTime1.RemoveAll();
	}

	testSnap testData;
	testData.lActiveVUs=0;
	testData.lInactiveVUs=0;
	testData.dTotalBytesIn=0.;
	testData.dTotalBytesOut=0.;
	testData.lTotalFailedRequests=0;
	testData.lTotalRequests=0;
	testData.l3xxRequests = 0;
	testData.l4xxRequests = 0;
	testData.l5xxRequests = 0;
	testData.lTotalTaskIterations=0;
	testData.nTaskGroups=0;
	testData.nExecuters=0;

	testData.lAvgConnectionTime=0;
	testData.lAvgRequestLatency=0;
	testData.lMaxConnectionTime=-1;
	testData.lMaxRequestLatency=-1;
	testData.lMinConnectionTime=-1;
	testData.lMinRequestLatency=-1;


	for(int nTE =0 ; nTE<(int)m_HTTPTEsSnapList1.length();nTE++)
	{
		testData.nExecuters++;
		CString temp2;

		HTTP_TestExecuterSnapshot HTTPTESnap;
		HTTPTESnap=m_HTTPTEsSnapList1[nTE];

		if((testData.lMinConnectionTime=-1)||(testData.lMinConnectionTime>HTTPTESnap.lAvgConnectionTime))
		{
			testData.lMinConnectionTime=HTTPTESnap.lAvgConnectionTime;
		}
		if((testData.lMaxConnectionTime=-1)||(testData.lMaxConnectionTime<HTTPTESnap.lAvgConnectionTime))
		{
			testData.lMaxConnectionTime=HTTPTESnap.lAvgConnectionTime;
		}
	
		testData.lAvgConnectionTime+=HTTPTESnap.lAvgConnectionTime;

		for (int nTG=0; nTG<(int)HTTPTESnap.seqTaskGroupSnaps.length();nTG++)
		{
			bool bHTTP=false;
			testData.nTaskGroups++;

			HTTP_TaskGroupSnap HTTPTGSnap;
			HTTPTGSnap=HTTPTESnap.seqTaskGroupSnaps[nTG];
			
			taskGroupData thisData;//this is the result structure
			
			long lTaskGroupId = HTTPTGSnap.lTaskGroupId;
			thisData.lTaskGroupId=lTaskGroupId;

			m_ctlSummary.SetItemText(lTaskGroupId,enDurationCol,ConvertTimeToString(HTTPTGSnap.lRunTime));
			thisData.lDuration=HTTPTGSnap.lRunTime;

			CTestFile::TaskType taskType=m_pTestFile->GetTaskType(lTaskGroupId,0);

			if(taskType==CTestFile::enScript)
			{
				CString temp;
				// Requests
				temp.Format("%d", HTTPTGSnap.lRequests);
				m_ctlSummary.SetItemText(lTaskGroupId,enHTTPrCol,temp);
				thisData.lRequests = HTTPTGSnap.lRequests;
				testData.lTotalRequests +=HTTPTGSnap.lRequests;

				// Failed Requests
				temp.Format("%d", HTTPTGSnap.lFailedRequests);
				m_ctlSummary.SetItemText(lTaskGroupId,enHTTPrFailedCol,temp);
				thisData.lFailedRequests = HTTPTGSnap.lFailedRequests ;
				testData.lTotalFailedRequests += HTTPTGSnap.lFailedRequests ;

				// 3xx Requests
				temp.Format("%d", HTTPTGSnap.l3XXRequests);
				m_ctlSummary.SetItemText(lTaskGroupId,enHTTP3xxCol,temp);
				thisData.l3xxRequests = HTTPTGSnap.l3XXRequests ;
				testData.l3xxRequests += HTTPTGSnap.l3XXRequests ;

				// 4xx Requests
				temp.Format("%d", HTTPTGSnap.l4XXRequests);
				m_ctlSummary.SetItemText(lTaskGroupId,enHTTP4xxCol,temp);
				thisData.l4xxRequests = HTTPTGSnap.l4XXRequests ;
				testData.l4xxRequests += HTTPTGSnap.l4XXRequests ;

				// 5xx Requests
				temp.Format("%d", HTTPTGSnap.l5XXRequests);
				m_ctlSummary.SetItemText(lTaskGroupId,enHTTP5xxCol,temp);
				thisData.l5xxRequests = HTTPTGSnap.l5XXRequests ;
				testData.l5xxRequests += HTTPTGSnap.l5XXRequests ;

				temp.Format("%f",((HTTPTGSnap.dBytesIn)+(HTTPTGSnap.dBytesOut)));
				m_ctlSummary.SetItemText(lTaskGroupId,enHTTPbCol,temp);

				DWORD  dwBufSize=0;
				LPBYTE pData = NULL;
				m_pTestFile->GetTaskGroupData(lTaskGroupId,&pData,dwBufSize);
				CTaskGroupDefData data;
				data.LoadFromBuffer(pData);
				temp.Format("%d/%d",HTTPTGSnap.lTotalUsers,data.GetNumVUs());
				m_ctlSummary.SetItemText(lTaskGroupId,enTotalVuCol,temp);
			}
			else if(taskType==CTestFile::enProfile)
			{
				thisData.lRequests=0;
				m_ctlSummary.SetItemText(lTaskGroupId,enHTTPrCol,"-");
				m_ctlSummary.SetItemText(lTaskGroupId,enHTTPbCol,"-");
				m_ctlSummary.SetItemText(lTaskGroupId,enTotalVuCol,"-");
			}

			long taskGroup = HTTPTGSnap.lTaskGroupId;

			// Byte In
			temp2.Format("%f",HTTPTGSnap.dBytesIn);
			m_ctlSummary.SetItemText(taskGroup,enBytesInCol,temp2);
			thisData.dBytesIn = HTTPTGSnap.dBytesIn;
			testData.dTotalBytesIn += HTTPTGSnap.dBytesIn;

			// Byte Out
			temp2.Format("%f",HTTPTGSnap.dBytesOut);
			m_ctlSummary.SetItemText(taskGroup,enBytesOutCol,temp2);
			thisData.dBytesOut = HTTPTGSnap.dBytesOut;
			testData.dTotalBytesOut += HTTPTGSnap.dBytesOut;

			// Min Request Latency
			temp2=ConvertToSeconds(HTTPTGSnap.lMinRequestLatency);
			m_ctlSummary.SetItemText(taskGroup,enMinCol,temp2);

			// store min test latency?
			if((testData.lMinRequestLatency==-1) || (testData.lMinRequestLatency>HTTPTGSnap.lMinRequestLatency))
			{
				testData.lMinRequestLatency=HTTPTGSnap.lMinRequestLatency;
			}

			// Max Request Latency
			temp2=ConvertToSeconds(HTTPTGSnap.lMaxRequestLatency);
			m_ctlSummary.SetItemText(taskGroup,enMaxCol,temp2);
			// store max test latency?
			if((testData.lMaxRequestLatency==-1) || (testData.lMaxRequestLatency<HTTPTGSnap.lMaxRequestLatency))
			{
				testData.lMaxRequestLatency=HTTPTGSnap.lMaxRequestLatency;
			}
			temp2=ConvertToSeconds(HTTPTGSnap.lAvgRequestLatency);
			m_ctlSummary.SetItemText(taskGroup,enAvgCol,temp2);

			//the average of averages!
			testData.lAvgRequestLatency +=HTTPTGSnap.lAvgRequestLatency;

			long numberOfHTTPTaskSnapShots=HTTPTGSnap.seqTaskSnaps.length();

			int nTotalVUsForAllTasks=0;
			int nTotalVUsErrorForAllTasks = 0 ;
			int nTotalIterationsForAllTasks=0;

			if(numberOfHTTPTaskSnapShots)
			{
				//iterate through each task snap
				HTTP_TaskSnap ts;
				for(int t=0; t<numberOfHTTPTaskSnapShots; t++)
				{
					ts=HTTPTGSnap.seqTaskSnaps[t];

					if(ts.lTaskId>m_nMaxUsedTaskCol)
					{
						CString temp;
						temp.Format("Task %d\n(VU's)",ts.lTaskId+1);
						m_ctlSummary.InsertColumn((ts.lTaskId*3)+enFirstTaskCol,temp,LVCFMT_LEFT,m_ctlSummary.GetColumnWidth(enFirstTaskCol));
						temp.Format("Task %d\n(Iterations)",ts.lTaskId+1);
						m_ctlSummary.InsertColumn((ts.lTaskId*3)+enFirstTaskCol+1,temp,LVCFMT_LEFT,m_ctlSummary.GetColumnWidth(enFirstTaskCol+1));
						temp.Format("Task %d\n(Period)",(ts.lTaskId)+1);
						m_ctlSummary.InsertColumn(((ts.lTaskId)*3)+enFirstTaskCol+2,temp,LVCFMT_LEFT,m_ctlSummary.GetColumnWidth(enFirstTaskCol+2));
						m_nMaxUsedTaskCol=ts.lTaskId;
					
						//make the extra columns have the two level heading
						CHeaderCtrl* pTGHeader = NULL;
						pTGHeader=m_ctlSummary.GetHeaderCtrl();

						if(pTGHeader!=NULL)
						{
							VERIFY(m_TGHeaderCtrl.SubclassWindow(pTGHeader->m_hWnd));	

							CFont NewHeaderFont;
							NewHeaderFont.CreatePointFont(190,"MS Serif");
							m_TGHeaderCtrl.SetFont(&NewHeaderFont);

							HDITEM hdItem;
							hdItem.mask = HDI_FORMAT;
		
							for(int i=0; i<m_TGHeaderCtrl.GetItemCount(); i++)
							{
								m_TGHeaderCtrl.GetItem(i,&hdItem);
								hdItem.fmt|= HDF_OWNERDRAW;
								m_TGHeaderCtrl.SetItem(i,&hdItem);
							}
						}
					}
		
					nTotalVUsForAllTasks += ts.lActiveUsers;
					nTotalVUsErrorForAllTasks += ts.lBlockingErrorUsers ;

					nTotalIterationsForAllTasks+= ts.lIterations;
					CString temp;
					temp.Format("%d",ts.lActiveUsers);
					m_ctlSummary.SetItemText(taskGroup,((ts.lTaskId)*3)+enFirstTaskCol,temp);
					temp.Format("%d",ts.lIterations);
					m_ctlSummary.SetItemText(taskGroup,((ts.lTaskId)*3)+enFirstTaskCol+1,temp);
					temp=ConvertToTimeMilliseconds(ts.lRuntime);
					m_ctlSummary.SetItemText(taskGroup,((ts.lTaskId)*3)+enFirstTaskCol+2,temp);
				}
				
				for(int j=numberOfHTTPTaskSnapShots;j<enFirstTaskCol+m_nMaxUsedTaskCol;j++)
				{
					//incase some tasks have been removed
					m_ctlSummary.SetItemText(lTaskGroupId,(j*3)+enFirstTaskCol,"");
					m_ctlSummary.SetItemText(lTaskGroupId,(j*3)+enFirstTaskCol+1,"");
					m_ctlSummary.SetItemText(lTaskGroupId,(j*3)+enFirstTaskCol+2,"");
				}
			}
			//set the number of Active virtal users in the column
			CString temp;
			temp.Format("%d",nTotalVUsForAllTasks);
			m_ctlSummary.SetItemText(taskGroup,enActiveVuCol,temp);
			//number of inactive = total - active
			temp.Format("%d",((HTTPTGSnap.lTotalUsers)-(nTotalVUsForAllTasks)));
			m_ctlSummary.SetItemText(taskGroup,enInactiveVuCol,temp);

			thisData.lSumActiveUsers =  nTotalVUsForAllTasks;
			testData.lActiveVUs		 += nTotalVUsForAllTasks;
			testData.lInactiveVUs    += ((HTTPTGSnap.lTotalUsers)-(nTotalVUsForAllTasks));
			thisData.lSumTaskIterations   =  nTotalIterationsForAllTasks;
			testData.lTotalTaskIterations += nTotalIterationsForAllTasks;

			// Finished OK
			thisData.lSumFinishedVUs = HTTPTGSnap.lFinishedUsers - HTTPTGSnap.lTotalErrorUsers;
			temp.Format("%d",HTTPTGSnap.lFinishedUsers - HTTPTGSnap.lTotalErrorUsers);
			m_ctlSummary.SetItemText(taskGroup,enFinishedVuCol,temp);

			// Finished Error
			thisData.lSumFinishedOnErrorVUs = HTTPTGSnap.lTotalErrorUsers ;
			temp.Format("%d",HTTPTGSnap.lTotalErrorUsers);
			m_ctlSummary.SetItemText(taskGroup,enOnErrorVuCol,temp);

			//Add the taskGroupData thisData; to the array of results

			if(m_nLatestSnapshot==1)
			{
				m_DataTime2.Add(thisData);
			}
			else if(m_nLatestSnapshot==2)
			{
				m_DataTime1.Add(thisData);
			}
		}
	}

	//divide the average latency by the number of contributing taskgroups
	if(testData.nTaskGroups<1)
	{
		testData.lAvgRequestLatency=0;
	}
	else
	{
		testData.lAvgRequestLatency = (testData.lAvgRequestLatency / testData.nTaskGroups);
	}

	//and the connection time avg by the number of executers contributing

	if(testData.nExecuters<1)
	{
		testData.lAvgConnectionTime=0;
	}
	else
	{
		testData.lAvgConnectionTime = (testData.lAvgConnectionTime / testData.nExecuters);
	}

	//store the testData record
	if(m_nLatestSnapshot==1)
	{
		m_TestWideStat2=testData;
	}
	else if(m_nLatestSnapshot==2)
	{
		m_TestWideStat1=testData;
	}

	

	//////////////////////////////////////////////////////////////////////////////////
	//
	// Now calculate the rate stats as a difference between the two cached values over the time difference 

	int nTestTime=0;  //is 0 unless a valid snapshot is found when it is updated to = the time gap between snapshots

	if(m_nLatestSnapshot==1)
	{
		//CString abcdef;
		//abcdef.Format("%d task groups in snapshot 2",m_DataTime2.GetSize());
		//MessageBox("",abcdef,MB_OK);

		//do data from 2 - data from 1
		for (int tg=0; tg<m_nTaskGroups ; tg++)
		{
			bool bmatch = false;
			for(int a=0;a<m_DataTime2.GetSize();a++)
			{
				if(m_DataTime2[a].lTaskGroupId==tg && 				
					(m_TaskStatus[tg].status==ACTIVE ||m_TaskStatus[tg].status==STOPPING))

				{
					//now look for the previous snap
					for(int b=0 ; b<m_DataTime1.GetSize();b++)
					{
						if(m_DataTime1[b].lTaskGroupId==tg)
						{
							if(m_DataTime2[a].lDuration > m_DataTime1[b].lDuration)
							{
								int nTime= m_DataTime2[a].lDuration - m_DataTime1[b].lDuration;
								nTestTime=nTime;
								CString temp;
								bmatch=true;
								temp.Format("%.3f",GetRate(m_DataTime2[a].dBytesIn +m_DataTime2[a].dBytesOut, m_DataTime1[b].dBytesIn+m_DataTime1[b].dBytesOut, nTime));
								m_ctlSummary.SetItemText(tg,enHTTPbperSecondCol,temp);
								temp.Format("%.3f",GetRate(m_DataTime2[a].dBytesIn , m_DataTime1[b].dBytesIn, nTime));
								m_ctlSummary.SetItemText(tg,enBytesInperSecondCol,temp);
								temp.Format("%.3f",GetRate(m_DataTime2[a].dBytesOut, m_DataTime1[b].dBytesOut, nTime));
								m_ctlSummary.SetItemText(tg,enBytesOutperSecondCol,temp);
								temp.Format("%.3f",GetRate(m_DataTime2[a].lRequests, m_DataTime1[b].lRequests, nTime));
								m_ctlSummary.SetItemText(tg,enHTTPrperSecondCol,temp);
								temp.Format("%.3f",GetRate(m_DataTime2[a].lFailedRequests, m_DataTime1[b].lFailedRequests, nTime));
								m_ctlSummary.SetItemText(tg,enHTTPrFailedperSecondCol,temp);
								temp.Format("%.3f",GetRate(m_DataTime2[a].l3xxRequests, m_DataTime1[b].l3xxRequests, nTime));
								m_ctlSummary.SetItemText(tg,enHTTP3xxColPerSecond,temp);
								temp.Format("%.3f",GetRate(m_DataTime2[a].l4xxRequests, m_DataTime1[b].l4xxRequests, nTime));
								m_ctlSummary.SetItemText(tg,enHTTP4xxColPerSecond,temp);
								temp.Format("%.3f",GetRate(m_DataTime2[a].l5xxRequests, m_DataTime1[b].l5xxRequests, nTime));
								m_ctlSummary.SetItemText(tg,enHTTP5xxColPerSecond,temp);
								temp.Format("%.3f",GetRate(m_DataTime2[a].lSumTaskIterations, m_DataTime1[b].lSumTaskIterations, nTime));
								m_ctlSummary.SetItemText(tg,enTaskIterationsperSecondCol,temp);
								break;
							}
						}
					}
				}
		
			}
			if(!bmatch)
			{
				//no entry in the current data for this Task Group
				m_ctlSummary.SetItemText(tg,enHTTPbperSecondCol,"-");
				m_ctlSummary.SetItemText(tg,enBytesInperSecondCol,"-");
				m_ctlSummary.SetItemText(tg,enBytesOutperSecondCol,"-");
				m_ctlSummary.SetItemText(tg,enHTTPrperSecondCol,"-");
				m_ctlSummary.SetItemText(tg,enHTTPrFailedperSecondCol,"-");
				m_ctlSummary.SetItemText(tg,enHTTP3xxColPerSecond,"-");
				m_ctlSummary.SetItemText(tg,enHTTP4xxColPerSecond,"-");
				m_ctlSummary.SetItemText(tg,enHTTP5xxColPerSecond,"-");
				m_ctlSummary.SetItemText(tg,enTaskIterationsperSecondCol,"-");
			}
		}
		// UPDATE TEST WIDE STATS

		if(nTestTime>0)
		{
			//it's a new record
			CString temp2;

			if(m_TestWideStat2.nTaskGroups==m_TestWideStat1.nTaskGroups)
			{
				temp2.Format("%.3f",GetRate(m_TestWideStat2.dTotalBytesIn,m_TestWideStat1.dTotalBytesIn,nTestTime));
				m_ctlTestSummary.SetItemText(0,0,temp2);
				temp2.Format("%.3f",GetRate(m_TestWideStat2.dTotalBytesOut,m_TestWideStat1.dTotalBytesOut,nTestTime));
				m_ctlTestSummary.SetItemText(0,1,temp2);
				temp2.Format("%.3f",GetRate(m_TestWideStat2.dTotalBytesIn+m_TestWideStat2.dTotalBytesOut,m_TestWideStat1.dTotalBytesIn+m_TestWideStat1.dTotalBytesOut,nTestTime));
				m_ctlTestSummary.SetItemText(0,2,temp2);

				temp2.Format("%.3f",GetRate(m_TestWideStat2.l3xxRequests,m_TestWideStat1.l3xxRequests,nTestTime));
				m_ctlTestSummary.SetItemText(0,6,temp2);
				temp2.Format("%.3f",GetRate(m_TestWideStat2.l4xxRequests,m_TestWideStat1.l4xxRequests,nTestTime));
				m_ctlTestSummary.SetItemText(0,7,temp2);
				temp2.Format("%.3f",GetRate(m_TestWideStat2.l5xxRequests,m_TestWideStat1.l5xxRequests,nTestTime));
				m_ctlTestSummary.SetItemText(0,8,temp2);
				temp2.Format("%.3f",GetRate(m_TestWideStat2.lTotalFailedRequests,m_TestWideStat1.lTotalFailedRequests,nTestTime));
				m_ctlTestSummary.SetItemText(0,9,temp2);
				temp2.Format("%.3f",GetRate(m_TestWideStat2.lTotalRequests,m_TestWideStat1.lTotalRequests,nTestTime));
				m_ctlTestSummary.SetItemText(0,10,temp2);
				temp2.Format("%.3f",GetRate(m_TestWideStat2.lTotalTaskIterations,m_TestWideStat1.lTotalTaskIterations,nTestTime));
				m_ctlTestSummary.SetItemText(0,11,temp2);
			}
			else
			{
				//if the number of task groups has changed it's not a fair comparison
				m_ctlTestSummary.SetItemText(0,0,"-");
				m_ctlTestSummary.SetItemText(0,1,"-");
				m_ctlTestSummary.SetItemText(0,2,"-");
				m_ctlTestSummary.SetItemText(0,6,"-");
				m_ctlTestSummary.SetItemText(0,7,"-");
				m_ctlTestSummary.SetItemText(0,8,"-");
				m_ctlTestSummary.SetItemText(0,9,"-");
				m_ctlTestSummary.SetItemText(0,10,"-");
				m_ctlTestSummary.SetItemText(0,11,"-");
			}

			temp2.Format("%d",m_TestWideStat2.lActiveVUs+m_TestWideStat2.lInactiveVUs);
			m_ctlTestSummary.SetItemText(0,3,temp2);
			temp2.Format("%d",m_TestWideStat2.lActiveVUs);
			m_ctlTestSummary.SetItemText(0,4,temp2);
			temp2.Format("%d",m_TestWideStat2.lInactiveVUs);
			m_ctlTestSummary.SetItemText(0,5,temp2);

			temp2=ConvertToSeconds(m_TestWideStat2.lMinConnectionTime);
			m_ctlTestSummary.SetItemText(0,12,temp2);
			temp2=ConvertToSeconds(m_TestWideStat2.lAvgConnectionTime);
			m_ctlTestSummary.SetItemText(0,13,temp2);
			temp2=ConvertToSeconds(m_TestWideStat2.lMaxConnectionTime);
			m_ctlTestSummary.SetItemText(0,14,temp2);
			temp2=ConvertToSeconds(m_TestWideStat2.lMinRequestLatency);
			m_ctlTestSummary.SetItemText(0,15,temp2);
			temp2=ConvertToSeconds(m_TestWideStat2.lAvgRequestLatency);
			m_ctlTestSummary.SetItemText(0,16,temp2);
			temp2=ConvertToSeconds(m_TestWideStat2.lMaxRequestLatency);
			m_ctlTestSummary.SetItemText(0,17,temp2);

		}
		else
		{
			//empty the test stats table
			m_ctlTestSummary.SetItemText(0,0,"-");
			m_ctlTestSummary.SetItemText(0,1,"-");
			m_ctlTestSummary.SetItemText(0,2,"-");
			m_ctlTestSummary.SetItemText(0,3,"-");
			m_ctlTestSummary.SetItemText(0,4,"-");
			m_ctlTestSummary.SetItemText(0,5,"-");
			m_ctlTestSummary.SetItemText(0,6,"-");
			m_ctlTestSummary.SetItemText(0,7,"-");
			m_ctlTestSummary.SetItemText(0,8,"-");
			m_ctlTestSummary.SetItemText(0,9,"-");
			m_ctlTestSummary.SetItemText(0,10,"-");
			m_ctlTestSummary.SetItemText(0,11,"-");
			m_ctlTestSummary.SetItemText(0,12,"-");
			m_ctlTestSummary.SetItemText(0,13,"-");
			m_ctlTestSummary.SetItemText(0,14,"-");
			m_ctlTestSummary.SetItemText(0,15,"-");
			m_ctlTestSummary.SetItemText(0,16,"-");
			m_ctlTestSummary.SetItemText(0,17,"-");

		}
		m_nLatestSnapshot=2;
	}
	else if(m_nLatestSnapshot==2)
	{
		//CString abcdef;
		//abcdef.Format("%d task groups in snapshot 1",m_DataTime1.GetSize());
		//MessageBox("",abcdef,MB_OK);

		//do data from 1 - data from 2
		for (int tg=0; tg<m_nTaskGroups ; tg++)
		{
			bool bmatch = false;
			for(int a=0;a<m_DataTime1.GetSize();a++)
			{
				if(m_DataTime1[a].lTaskGroupId==tg &&
					(m_TaskStatus[tg].status==ACTIVE ||m_TaskStatus[tg].status==STOPPING))
				{
					//now look for the previous snap
					for(int b=0 ; b<m_DataTime2.GetSize();b++)
					{
						if(m_DataTime2[b].lTaskGroupId==tg)
						{
							if(m_DataTime1[a].lDuration > m_DataTime2[b].lDuration)
							{
								bmatch=true;
								int nTime= m_DataTime1[a].lDuration - m_DataTime2[b].lDuration;
								nTestTime=nTime;
								CString temp;
								bmatch=true;
								temp.Format("%.3f",GetRate(m_DataTime1[a].dBytesIn +m_DataTime1[a].dBytesOut, m_DataTime2[b].dBytesIn+m_DataTime2[b].dBytesOut, nTime));
								m_ctlSummary.SetItemText(tg,enHTTPbperSecondCol,temp);
								temp.Format("%.3f",GetRate(m_DataTime1[a].dBytesIn , m_DataTime2[b].dBytesIn, nTime));
								m_ctlSummary.SetItemText(tg,enBytesInperSecondCol,temp);
								temp.Format("%.3f",GetRate(m_DataTime1[a].dBytesOut, m_DataTime2[b].dBytesOut, nTime));
								m_ctlSummary.SetItemText(tg,enBytesOutperSecondCol,temp);
								temp.Format("%.3f",GetRate(m_DataTime1[a].lRequests, m_DataTime2[b].lRequests, nTime));
								m_ctlSummary.SetItemText(tg,enHTTPrperSecondCol,temp);
								temp.Format("%.3f",GetRate(m_DataTime1[a].lFailedRequests, m_DataTime2[b].lFailedRequests, nTime));
								m_ctlSummary.SetItemText(tg,enHTTPrFailedperSecondCol,temp);
								temp.Format("%.3f",GetRate(m_DataTime1[a].l3xxRequests, m_DataTime2[b].l3xxRequests, nTime));
								m_ctlSummary.SetItemText(tg,enHTTP3xxColPerSecond,temp);
								temp.Format("%.3f",GetRate(m_DataTime1[a].l4xxRequests, m_DataTime2[b].l4xxRequests, nTime));
								m_ctlSummary.SetItemText(tg,enHTTP4xxColPerSecond,temp);
								temp.Format("%.3f",GetRate(m_DataTime1[a].l5xxRequests, m_DataTime2[b].l5xxRequests, nTime));
								m_ctlSummary.SetItemText(tg,enHTTP5xxColPerSecond,temp);
								temp.Format("%.3f",GetRate(m_DataTime1[a].lSumTaskIterations, m_DataTime2[b].lSumTaskIterations, nTime));
								m_ctlSummary.SetItemText(tg,enTaskIterationsperSecondCol,temp);
							
								break;
							}
						}
					}
				}
			}
			if(!bmatch)
			{
				//no entry in the current data for this Task Group
				m_ctlSummary.SetItemText(tg,enHTTPbperSecondCol,"-");
				m_ctlSummary.SetItemText(tg,enBytesInperSecondCol,"-");
				m_ctlSummary.SetItemText(tg,enBytesOutperSecondCol,"-");
				m_ctlSummary.SetItemText(tg,enHTTPrperSecondCol,"-");
				m_ctlSummary.SetItemText(tg,enHTTPrFailedperSecondCol,"-");
				m_ctlSummary.SetItemText(tg,enHTTP3xxColPerSecond,"-");
				m_ctlSummary.SetItemText(tg,enHTTP4xxColPerSecond,"-");
				m_ctlSummary.SetItemText(tg,enHTTP5xxColPerSecond,"-");
				m_ctlSummary.SetItemText(tg,enTaskIterationsperSecondCol,"-");
			}
		}
		// UPDATE TEST WIDE STATS
		if(nTestTime>0)
		{
			//it's a new record
			CString temp2;
			
			if(m_TestWideStat1.nTaskGroups==m_TestWideStat2.nTaskGroups)
			{
				temp2.Format("%.3f",GetRate(m_TestWideStat1.dTotalBytesIn,m_TestWideStat2.dTotalBytesIn,nTestTime));
				m_ctlTestSummary.SetItemText(0,0,temp2);
				temp2.Format("%.3f",GetRate(m_TestWideStat1.dTotalBytesOut,m_TestWideStat2.dTotalBytesOut,nTestTime));
				m_ctlTestSummary.SetItemText(0,1,temp2);
				temp2.Format("%.3f",GetRate(m_TestWideStat1.dTotalBytesIn+m_TestWideStat1.dTotalBytesOut,m_TestWideStat2.dTotalBytesIn+m_TestWideStat2.dTotalBytesOut,nTestTime));
				m_ctlTestSummary.SetItemText(0,2,temp2);

				temp2.Format("%.3f",GetRate(m_TestWideStat1.l3xxRequests,m_TestWideStat2.l3xxRequests,nTestTime));
				m_ctlTestSummary.SetItemText(0,6,temp2);
				temp2.Format("%.3f",GetRate(m_TestWideStat1.l4xxRequests,m_TestWideStat2.l4xxRequests,nTestTime));
				m_ctlTestSummary.SetItemText(0,7,temp2);
				temp2.Format("%.3f",GetRate(m_TestWideStat1.l5xxRequests,m_TestWideStat2.l5xxRequests,nTestTime));
				m_ctlTestSummary.SetItemText(0,8,temp2);
				temp2.Format("%.3f",GetRate(m_TestWideStat1.lTotalFailedRequests,m_TestWideStat2.lTotalFailedRequests,nTestTime));
				m_ctlTestSummary.SetItemText(0,9,temp2);
				temp2.Format("%.3f",GetRate(m_TestWideStat1.lTotalRequests,m_TestWideStat2.lTotalRequests,nTestTime));
				m_ctlTestSummary.SetItemText(0,10,temp2);
				temp2.Format("%.3f",GetRate(m_TestWideStat1.lTotalTaskIterations,m_TestWideStat2.lTotalTaskIterations,nTestTime));
				m_ctlTestSummary.SetItemText(0,11,temp2);
			}
			else
			{
				//if the number of task groups has changed it's not a fair comparison
				m_ctlTestSummary.SetItemText(0,0,"-");
				m_ctlTestSummary.SetItemText(0,1,"-");
				m_ctlTestSummary.SetItemText(0,2,"-");
				m_ctlTestSummary.SetItemText(0,6,"-");
				m_ctlTestSummary.SetItemText(0,7,"-");
				m_ctlTestSummary.SetItemText(0,8,"-");
				m_ctlTestSummary.SetItemText(0,9,"-");
				m_ctlTestSummary.SetItemText(0,10,"-");
				m_ctlTestSummary.SetItemText(0,11,"-");

			}
			temp2.Format("%d",m_TestWideStat1.lActiveVUs+m_TestWideStat1.lInactiveVUs);
			m_ctlTestSummary.SetItemText(0,3,temp2);
			temp2.Format("%d",m_TestWideStat1.lActiveVUs);
			m_ctlTestSummary.SetItemText(0,4,temp2);
			temp2.Format("%d",m_TestWideStat1.lInactiveVUs);
			m_ctlTestSummary.SetItemText(0,5,temp2);

			temp2=ConvertToSeconds(m_TestWideStat1.lMinConnectionTime);
			m_ctlTestSummary.SetItemText(0,12,temp2);
			temp2=ConvertToSeconds(m_TestWideStat1.lAvgConnectionTime);
			m_ctlTestSummary.SetItemText(0,13,temp2);
			temp2=ConvertToSeconds(m_TestWideStat1.lMaxConnectionTime);
			m_ctlTestSummary.SetItemText(0,14,temp2);
			temp2=ConvertToSeconds(m_TestWideStat1.lMinRequestLatency);
			m_ctlTestSummary.SetItemText(0,15,temp2);
			temp2=ConvertToSeconds(m_TestWideStat1.lAvgRequestLatency);
			m_ctlTestSummary.SetItemText(0,16,temp2);
			temp2=ConvertToSeconds(m_TestWideStat1.lMaxRequestLatency);
			m_ctlTestSummary.SetItemText(0,17,temp2);
		}
		else
		{
			//empty the test stats table
			m_ctlTestSummary.SetItemText(0,0,"-");
			m_ctlTestSummary.SetItemText(0,1,"-");
			m_ctlTestSummary.SetItemText(0,2,"-");
			m_ctlTestSummary.SetItemText(0,3,"-");
			m_ctlTestSummary.SetItemText(0,4,"-");
			m_ctlTestSummary.SetItemText(0,5,"-");
			m_ctlTestSummary.SetItemText(0,6,"-");
			m_ctlTestSummary.SetItemText(0,7,"-");
			m_ctlTestSummary.SetItemText(0,8,"-");
			m_ctlTestSummary.SetItemText(0,9,"-");
			m_ctlTestSummary.SetItemText(0,10,"-");
			m_ctlTestSummary.SetItemText(0,11,"-");
			m_ctlTestSummary.SetItemText(0,12,"-");
			m_ctlTestSummary.SetItemText(0,13,"-");
			m_ctlTestSummary.SetItemText(0,14,"-");
			m_ctlTestSummary.SetItemText(0,15,"-");
			m_ctlTestSummary.SetItemText(0,16,"-");
			m_ctlTestSummary.SetItemText(0,17,"-");
		}

		m_nLatestSnapshot=1;
	}
	else
	{
		//it's an error blank out the lot - may be < OpenSTA 1.3 so no http data
		for (int tg=0; tg<m_nTaskGroups ; tg++)
		{
			m_ctlSummary.SetItemText(tg,enHTTPbperSecondCol,"-");
			m_ctlSummary.SetItemText(tg,enBytesInperSecondCol,"-");
			m_ctlSummary.SetItemText(tg,enBytesOutperSecondCol,"-");
			m_ctlSummary.SetItemText(tg,enHTTPrperSecondCol,"-");
			m_ctlSummary.SetItemText(tg,enHTTPrFailedperSecondCol,"-");
			m_ctlSummary.SetItemText(tg,enHTTP3xxColPerSecond,"-");
			m_ctlSummary.SetItemText(tg,enHTTP4xxColPerSecond,"-");
			m_ctlSummary.SetItemText(tg,enHTTP5xxColPerSecond,"-");
			m_ctlSummary.SetItemText(tg,enTaskIterationsperSecondCol,"-");
			m_ctlSummary.SetItemText(tg,enActiveVuCol,"-");
			m_ctlSummary.SetItemText(tg,enInactiveVuCol,"-");
			m_ctlSummary.SetItemText(tg,enOnErrorVuCol,"-");
			m_ctlSummary.SetItemText(tg,enFinishedVuCol,"-");

		}

		// UPDATE TEST WIDE STATS - clear them out
		m_ctlTestSummary.SetItemText(0,0,"-");
		m_ctlTestSummary.SetItemText(0,1,"-");
		m_ctlTestSummary.SetItemText(0,2,"-");
		m_ctlTestSummary.SetItemText(0,3,"-");
		m_ctlTestSummary.SetItemText(0,4,"-");
		m_ctlTestSummary.SetItemText(0,5,"-");
		m_ctlTestSummary.SetItemText(0,6,"-");
		m_ctlTestSummary.SetItemText(0,7,"-");
		m_ctlTestSummary.SetItemText(0,8,"-");
		m_ctlTestSummary.SetItemText(0,9,"-");
		m_ctlTestSummary.SetItemText(0,10,"-");
		m_ctlTestSummary.SetItemText(0,11,"-");
		m_ctlTestSummary.SetItemText(0,12,"-");
		m_ctlTestSummary.SetItemText(0,13,"-");
		m_ctlTestSummary.SetItemText(0,14,"-");
		m_ctlTestSummary.SetItemText(0,15,"-");
		m_ctlTestSummary.SetItemText(0,16,"-");
		m_ctlTestSummary.SetItemText(0,17,"-");
	}

	lock.Unlock();
	return;
}

void CMonitorSummary::OnSize(UINT nType, int cx, int cy) 
{
	CDialog::OnSize(nType, cx, cy);
		
	int nHeight=85;
	int nSeperator = 4;
	int nTextHeight = 15;

	if(m_ctlTestText.m_hWnd != NULL)
	{
		m_ctlTestText.MoveWindow(nSeperator,nSeperator,cx-nSeperator,nTextHeight);
	}
	if(m_ctlTestSummary.m_hWnd != NULL)
	{
		m_ctlTestSummary.MoveWindow(0,nSeperator+nSeperator+nTextHeight,cx,nHeight);
	}

	if(m_ctlTGText.m_hWnd != NULL)
	{
		m_ctlTGText.MoveWindow(nSeperator,nSeperator+nSeperator+nTextHeight+nSeperator+nHeight,cx-nSeperator,nTextHeight);
	}

	if(m_ctlSummary.m_hWnd != NULL)
	{
		m_ctlSummary.MoveWindow(0,nHeight+(4*nSeperator)+(2*nTextHeight),cx,cy-nHeight-(4*nSeperator)-(2*nTextHeight));
	}
}

void CMonitorSummary::OnDestroy() 
{
	KillTimer(IDT_DATA);

	if(m_pUpdateThread)
	{
		SetEvent(m_hCloseThread);
		WaitForSingleObject(m_pUpdateThread->m_hThread,5000);
		m_pUpdateThread = NULL;
	}

	CDialog::OnDestroy();
}

void CMonitorSummary::OnShowWindow(BOOL bShow, UINT nStatus) 
{
	CDialog::OnShowWindow(bShow, nStatus);

	CWinApp *pApp = AfxGetApp();

	int highestusedcol=0;
	
	//check that the number of tasks is correct, added/removed?
	for(int group=0;group<m_pTestFile->GetNumTaskGroups();group++)
	{
		if(m_pTestFile->GetNumTasks(group)>highestusedcol)
		{
			highestusedcol=m_pTestFile->GetNumTasks(group)-1;
		}

	}

	//delete the extra task columns
	if(highestusedcol<m_nMaxUsedTaskCol)
	{
		for(int b=highestusedcol;b<m_nMaxUsedTaskCol;b++)
		{
			m_ctlSummary.DeleteColumn(((highestusedcol+1)*3)+enFirstTaskCol);	//vu's
			m_ctlSummary.DeleteColumn(((highestusedcol+1)*3)+enFirstTaskCol);	//iterations
			m_ctlSummary.DeleteColumn(((highestusedcol+1)*3)+enFirstTaskCol);	//last period
		}
	}

	//delete all info from deleted tasks, add task columns if number of tasks increased
	for(int b=0;b<m_pTestFile->GetNumTaskGroups();b++)
	{
		for(int a=0;a<m_pTestFile->GetNumTasks(b);a++)
		{
			if(a>m_nMaxUsedTaskCol)
			{
				CString temp;
				temp.Format("Task %d\n(VU's)",a+1);
				m_ctlSummary.InsertColumn((a*3)+enFirstTaskCol,temp,LVCFMT_LEFT,m_ctlSummary.GetColumnWidth(enFirstTaskCol));
				temp.Format("Task %d\n(Iterations)",a+1);
				m_ctlSummary.InsertColumn((a*3)+enFirstTaskCol+1,temp,LVCFMT_LEFT,m_ctlSummary.GetColumnWidth(enFirstTaskCol+1));
				temp.Format("Task %d\n(Period)",a+1);
				m_ctlSummary.InsertColumn((a*3)+enFirstTaskCol+2,temp,LVCFMT_LEFT,m_ctlSummary.GetColumnWidth(enFirstTaskCol+2));
				m_nMaxUsedTaskCol=a;

				//make the extra columns have the two level heading
				CHeaderCtrl* pTGHeader = NULL;
				pTGHeader=m_ctlSummary.GetHeaderCtrl();

				if(pTGHeader!=NULL)
				{
					VERIFY(m_TGHeaderCtrl.SubclassWindow(pTGHeader->m_hWnd));	

					CFont NewHeaderFont;
					NewHeaderFont.CreatePointFont(190,"MS Serif");
					m_TGHeaderCtrl.SetFont(&NewHeaderFont);

					HDITEM hdItem;
					hdItem.mask = HDI_FORMAT;

					for(int i=0; i<m_TGHeaderCtrl.GetItemCount(); i++)
					{
						m_TGHeaderCtrl.GetItem(i,&hdItem);
						hdItem.fmt|= HDF_OWNERDRAW;
						m_TGHeaderCtrl.SetItem(i,&hdItem);
					}
				}
			}
			m_ctlSummary.SetItemText(b,enFirstTaskCol+(a*3),"-");
			m_ctlSummary.SetItemText(b,enFirstTaskCol+(a*3)+1,"-");
			m_ctlSummary.SetItemText(b,enFirstTaskCol+(a*3)+2,"-");
		}
		for(int c=m_pTestFile->GetNumTasks(b);c<highestusedcol+enFirstTaskCol;c++)
		{
			m_ctlSummary.SetItemText(b,enFirstTaskCol+(c*3),"");
			m_ctlSummary.SetItemText(b,enFirstTaskCol+1+(c*3),"");
			m_ctlSummary.SetItemText(b,enFirstTaskCol+2+(c*3),"");
		}
	}


	//add any extra task groups
	for(int extraTG=m_nTaskGroups-1; extraTG<m_pTestFile->GetNumTaskGroups(); extraTG++)
	{
		m_ctlSummary.InsertItem(extraTG,m_pTestFile->GetTaskGroupName(extraTG));
		m_ctlSummary.SetItemText(extraTG,enStatusCol,"-");
		m_ctlSummary.SetItemText(extraTG,enDurationCol,"-");
		m_ctlSummary.SetItemText(extraTG,enHTTPrCol,"-");		
		m_ctlSummary.SetItemText(extraTG,enHTTPrFailedCol,"-");
		m_ctlSummary.SetItemText(extraTG,enHTTP3xxCol,"-");
		m_ctlSummary.SetItemText(extraTG,enHTTP4xxCol,"-");
		m_ctlSummary.SetItemText(extraTG,enHTTP5xxCol,"-");
		m_ctlSummary.SetItemText(extraTG,enHTTPbCol,"-");
		m_ctlSummary.SetItemText(extraTG,enBytesInCol,"-");
		m_ctlSummary.SetItemText(extraTG,enBytesOutCol,"-");
		m_ctlSummary.SetItemText(extraTG,enMinCol,"-");		
		m_ctlSummary.SetItemText(extraTG,enMaxCol,"-");
		m_ctlSummary.SetItemText(extraTG,enAvgCol,"-");
		m_ctlSummary.SetItemText(extraTG,enTotalVuCol,"-");
		m_ctlSummary.SetItemText(extraTG,enHTTPbperSecondCol,"-");
		m_ctlSummary.SetItemText(extraTG,enBytesInperSecondCol,"-");
		m_ctlSummary.SetItemText(extraTG,enBytesOutperSecondCol,"-");
		m_ctlSummary.SetItemText(extraTG,enHTTPrperSecondCol,"-");
		m_ctlSummary.SetItemText(extraTG,enHTTPrFailedperSecondCol,"-");
		m_ctlSummary.SetItemText(extraTG,enHTTP3xxColPerSecond,"-");
		m_ctlSummary.SetItemText(extraTG,enHTTP4xxColPerSecond,"-");
		m_ctlSummary.SetItemText(extraTG,enHTTP5xxColPerSecond,"-");
		m_ctlSummary.SetItemText(extraTG,enTaskIterationsperSecondCol,"-");
		m_ctlSummary.SetItemText(extraTG,enActiveVuCol,"-");
		m_ctlSummary.SetItemText(extraTG,enInactiveVuCol,"-");
		m_ctlSummary.SetItemText(extraTG,enOnErrorVuCol,"-");
		m_ctlSummary.SetItemText(extraTG,enFinishedVuCol,"-");
		for(int tasks=0;tasks<m_pTestFile->GetNumTasks(extraTG);tasks++)
		{
			if(tasks>m_nMaxUsedTaskCol)
			{
				CString heading;
				heading.Format("Task %d\n(VU's)",tasks+1);
				m_ctlSummary.InsertColumn((tasks*3)+enFirstTaskCol,heading,LVCFMT_LEFT,m_ctlSummary.GetColumnWidth(enFirstTaskCol));
				heading.Format("Task %d\n(Iterations)",tasks+1);
				m_ctlSummary.InsertColumn((tasks*3)+enFirstTaskCol+1,heading,LVCFMT_LEFT,m_ctlSummary.GetColumnWidth(enFirstTaskCol+1));
				heading.Format("Task %d\n(Period)",tasks+1);
				m_ctlSummary.InsertColumn((tasks*3)+enFirstTaskCol+2,heading,LVCFMT_LEFT,m_ctlSummary.GetColumnWidth(enFirstTaskCol+2));
				m_nMaxUsedTaskCol=tasks;

				//make the extra columns have the two level heading
				CHeaderCtrl* pTGHeader = NULL;
				pTGHeader=m_ctlSummary.GetHeaderCtrl();

				if(pTGHeader!=NULL)
				{
					VERIFY(m_TGHeaderCtrl.SubclassWindow(pTGHeader->m_hWnd));	

					CFont NewHeaderFont;
					NewHeaderFont.CreatePointFont(190,"MS Serif");
					m_TGHeaderCtrl.SetFont(&NewHeaderFont);

					HDITEM hdItem;
					hdItem.mask = HDI_FORMAT;

					for(int i=0; i<m_TGHeaderCtrl.GetItemCount(); i++)
					{
						m_TGHeaderCtrl.GetItem(i,&hdItem);
						hdItem.fmt|= HDF_OWNERDRAW;
						m_TGHeaderCtrl.SetItem(i,&hdItem);
					}
				}
			}
			m_ctlSummary.SetItemText(extraTG,(tasks*3)+enFirstTaskCol,"-");
			m_ctlSummary.SetItemText(extraTG,(tasks*3)+enFirstTaskCol+1,"-");
			m_ctlSummary.SetItemText(extraTG,(tasks*3)+enFirstTaskCol+2,"-");
		}
	}
	m_nTaskGroups=extraTG+1;

	UpdateSummary();
}

CString CMonitorSummary::LookupTargetType(CString strTarget)
{
	CString strDesc = "";
	if (strTarget.CompareNoCase("HTP")==0)
	{
		strDesc = "HTTP Load";
		return strDesc;
	}

	POSITION pos = m_listPlugins.GetHeadPosition();
	while(pos)
	{
		PluginInfo &PI = m_listPlugins.GetNext(pos);
		CString strPIExts(PI.strExtension);
		if (strPIExts.CompareNoCase(strTarget) == 0)
		{
			strDesc = PI.strName;
			strDesc += " Collector";
			return strDesc;
		}
	}
	return strTarget;
}

int CMonitorSummary::LoadProfileTypes()
{
	int nCount = 0;
	CString strRegSubKey("SOFTWARE\\OpenSTA\\PlugIns\\Profiles");

	HKEY hKey;
	if(RegOpenKeyEx(HKEY_LOCAL_MACHINE,strRegSubKey,0,KEY_READ,&hKey)!=ERROR_SUCCESS)
		return nCount;

	DWORD dwIndex=0;
	DWORD dwNameSize=64;
	char szName[64];
	while(RegEnumKeyEx(hKey,dwIndex,szName,&dwNameSize,0,NULL,NULL,NULL)==ERROR_SUCCESS)
	{
		dwNameSize = 64;
		HKEY hKeyPlugin;
		if(RegOpenKeyEx(hKey,szName,0,KEY_READ,&hKeyPlugin) == ERROR_SUCCESS)
		{
			PluginInfo PI;
			PI.strName = szName;

			DWORD dwType;
			DWORD dwDataSize = 256;
			BYTE byData[256];
			//Get Extension for this type
			if(RegQueryValueEx(hKeyPlugin,"Extension",0,&dwType,byData,&dwDataSize)==ERROR_SUCCESS)
				PI.strExtension = (char*)byData;
			dwDataSize = 256;

			RegCloseKey(hKeyPlugin);
			m_listPlugins.AddTail(PI);
			nCount++;
		}
		dwIndex++;
	}
	return nCount;
}

CString CMonitorSummary::ConvertTimeToString(long duration)
{

	CString answer;
	int remainder;
	div_t temp;
	int h=0;
	int m=0;
	int s=0;
	int days=0;

	temp=div(duration,3600*24);
	days=temp.quot;
	remainder=temp.rem;

	temp=div(remainder,3600);
	h=temp.quot;
	remainder=temp.rem;

	temp=div(remainder,60);
	m=temp.quot;
	s=temp.rem;

	if(days>0)
	{
		answer.Format("%d %02d:%02d:%02d",days,h,m,s);
	}
	else
	{
		answer.Format("%02d:%02d:%02d",h,m,s);
	}
	return answer;
}

BOOL CMonitorSummary::OnHelpInfo(HELPINFO* pHelpInfo) 
{
	pHelpInfo->dwContextId = HIDD_MON_SUMMARY;
	return CDialog::OnHelpInfo(pHelpInfo);
}


void CMonitorSummary::OnContextMenu(CWnd* pWnd, CPoint point) 
{
	if(HitTaskGroupSummary(point))
	{
		//it's in the tg area
		if (point.x == -1 && point.y == -1)
		{
		    //keystroke invocation
		    CRect rect;
		    GetClientRect(rect);
		    ClientToScreen(rect);

		    point = rect.TopLeft();
		    point.Offset(5, 5);
		}
   
		if(1)//limit to fixed header row?
		{
	
			CMenu menu;
			menu.LoadMenu(IDR_SHOWHIDE_MONSUMMARY_COLS);

			menu.CheckMenuItem(ID_MON_SHOW_TASKGROUP,MF_BYCOMMAND|(m_ctlSummary.GetColumnWidth(enTaskGroupCol) ? MF_CHECKED:MF_UNCHECKED));
			menu.CheckMenuItem(ID_MON_SHOW_STATUS,MF_BYCOMMAND|(m_ctlSummary.GetColumnWidth(enStatusCol) ? MF_CHECKED:MF_UNCHECKED));
			menu.CheckMenuItem(ID_MON_SHOW_DURATION,MF_BYCOMMAND|(m_ctlSummary.GetColumnWidth(enDurationCol) ? MF_CHECKED:MF_UNCHECKED));
			menu.CheckMenuItem(ID_MON_SHOW_HTTPR,MF_BYCOMMAND|(m_ctlSummary.GetColumnWidth(enHTTPrCol) ? MF_CHECKED:MF_UNCHECKED));
			menu.CheckMenuItem(ID_MON_SHOW_FAILEDR,MF_BYCOMMAND|(m_ctlSummary.GetColumnWidth(enHTTPrFailedCol) ? MF_CHECKED:MF_UNCHECKED));
			menu.CheckMenuItem(ID_MON_SHOW_HTTPB,MF_BYCOMMAND|(m_ctlSummary.GetColumnWidth(enHTTPbCol) ? MF_CHECKED:MF_UNCHECKED));
			menu.CheckMenuItem(ID_MON_SHOW_BYTESIN,MF_BYCOMMAND|(m_ctlSummary.GetColumnWidth(enBytesInCol) ? MF_CHECKED:MF_UNCHECKED));
			menu.CheckMenuItem(ID_MON_SHOW_BYTESOUT,MF_BYCOMMAND|(m_ctlSummary.GetColumnWidth(enBytesOutCol) ? MF_CHECKED:MF_UNCHECKED));
			menu.CheckMenuItem(ID_MON_SHOW_MIN,MF_BYCOMMAND|(m_ctlSummary.GetColumnWidth(enMinCol) ? MF_CHECKED:MF_UNCHECKED));
			menu.CheckMenuItem(ID_MON_SHOW_MAX,MF_BYCOMMAND|(m_ctlSummary.GetColumnWidth(enMaxCol) ? MF_CHECKED:MF_UNCHECKED));
			menu.CheckMenuItem(ID_MON_SHOW_AVG,MF_BYCOMMAND|(m_ctlSummary.GetColumnWidth(enAvgCol) ? MF_CHECKED:MF_UNCHECKED));
			menu.CheckMenuItem(ID_MON_SHOW_VUS,MF_BYCOMMAND|(m_ctlSummary.GetColumnWidth(enTotalVuCol) ? MF_CHECKED:MF_UNCHECKED));
			menu.CheckMenuItem(ID_MON_SHOW_HTTPBPS,MF_BYCOMMAND|(m_ctlSummary.GetColumnWidth(enHTTPbperSecondCol) ? MF_CHECKED:MF_UNCHECKED));
			menu.CheckMenuItem(ID_MON_SHOW_FAILEDRPS,MF_BYCOMMAND|(m_ctlSummary.GetColumnWidth(enHTTPrFailedperSecondCol) ? MF_CHECKED:MF_UNCHECKED));
			menu.CheckMenuItem(ID_MON_SHOW_HTTPRPS,MF_BYCOMMAND|(m_ctlSummary.GetColumnWidth(enHTTPrperSecondCol) ? MF_CHECKED:MF_UNCHECKED));
			menu.CheckMenuItem(ID_MON_SHOW_HTTP3XXR,MF_BYCOMMAND|(m_ctlSummary.GetColumnWidth(enHTTP3xxCol) ? MF_CHECKED:MF_UNCHECKED));
			menu.CheckMenuItem(ID_MON_SHOW_HTTP4XXR,MF_BYCOMMAND|(m_ctlSummary.GetColumnWidth(enHTTP4xxCol) ? MF_CHECKED:MF_UNCHECKED));
			menu.CheckMenuItem(ID_MON_SHOW_HTTP5XXR,MF_BYCOMMAND|(m_ctlSummary.GetColumnWidth(enHTTP5xxCol) ? MF_CHECKED:MF_UNCHECKED));
			menu.CheckMenuItem(ID_MON_SHOW_HTTP3XXRPS,MF_BYCOMMAND|(m_ctlSummary.GetColumnWidth(enHTTP3xxColPerSecond) ? MF_CHECKED:MF_UNCHECKED));
			menu.CheckMenuItem(ID_MON_SHOW_HTTP4XXRPS,MF_BYCOMMAND|(m_ctlSummary.GetColumnWidth(enHTTP4xxColPerSecond) ? MF_CHECKED:MF_UNCHECKED));
			menu.CheckMenuItem(ID_MON_SHOW_HTTP5XXRPS,MF_BYCOMMAND|(m_ctlSummary.GetColumnWidth(enHTTP5xxColPerSecond) ? MF_CHECKED:MF_UNCHECKED));
			menu.CheckMenuItem(ID_MON_SHOW_BYTESINPS,MF_BYCOMMAND|(m_ctlSummary.GetColumnWidth(enBytesInperSecondCol) ? MF_CHECKED:MF_UNCHECKED));
			menu.CheckMenuItem(ID_MON_SHOW_BYTESOUTPS,MF_BYCOMMAND|	(m_ctlSummary.GetColumnWidth(enBytesOutperSecondCol) ? MF_CHECKED:MF_UNCHECKED));
			menu.CheckMenuItem(ID_MON_SHOW_TASKTOTITERATIONS,MF_BYCOMMAND|(m_ctlSummary.GetColumnWidth(enTaskIterationsperSecondCol) ? MF_CHECKED:MF_UNCHECKED));
			menu.CheckMenuItem(ID_MON_SHOW_ACTIVEVUS,MF_BYCOMMAND|(m_ctlSummary.GetColumnWidth(enActiveVuCol) ? MF_CHECKED:MF_UNCHECKED));
			menu.CheckMenuItem(ID_MON_SHOW_INACTIVEVUS,MF_BYCOMMAND|(m_ctlSummary.GetColumnWidth(enInactiveVuCol) ? MF_CHECKED:MF_UNCHECKED));
			menu.CheckMenuItem(ID_MON_SHOW_ERRORVUS,MF_BYCOMMAND|(m_ctlSummary.GetColumnWidth(enOnErrorVuCol) ? MF_CHECKED:MF_UNCHECKED));
			menu.CheckMenuItem(ID_MON_SHOW_FINISHEDVUS,MF_BYCOMMAND|(m_ctlSummary.GetColumnWidth(enFinishedVuCol) ? MF_CHECKED:MF_UNCHECKED));
			menu.CheckMenuItem(ID_MON_SHOW_TASKVUS,MF_BYCOMMAND|(m_ctlSummary.GetColumnWidth(enFirstTaskCol) ? MF_CHECKED:MF_UNCHECKED));
			menu.CheckMenuItem(ID_MON_SHOW_TASKITERATIONS,MF_BYCOMMAND|(m_ctlSummary.GetColumnWidth(enFirstTaskCol+1) ? MF_CHECKED:MF_UNCHECKED));
			menu.CheckMenuItem(ID_MON_SHOW_TASKPERIOD,MF_BYCOMMAND|(m_ctlSummary.GetColumnWidth(enFirstTaskCol+2) ? MF_CHECKED:MF_UNCHECKED));

			CMenu* pPopup = menu.GetSubMenu(0);
			ASSERT(pPopup != NULL);

			pPopup->TrackPopupMenu(TPM_LEFTALIGN | TPM_RIGHTBUTTON, point.x, point.y,this);
		}
	}
	else if(HitTestSummary(point))
	{
		//context menu in test area
		if (point.x == -1 && point.y == -1)
		{
		    //keystroke invocation
		    CRect rect;
		    GetClientRect(rect);
		    ClientToScreen(rect);

		    point = rect.TopLeft();
		    point.Offset(5, 5);
		}
		if(1)//limit to fixed header row?
		{
	
			CMenu menu;
			menu.LoadMenu(IDR_SHOWHIDE_MONSUMMARYTEST_COLS);

			menu.CheckMenuItem(ID_MON_SHOWTEST_BYTESINPS,MF_BYCOMMAND|(m_ctlTestSummary.GetColumnWidth(0) ? MF_CHECKED:MF_UNCHECKED));
			menu.CheckMenuItem(ID_MON_SHOWTEST_BYTESOUTPS,MF_BYCOMMAND|(m_ctlTestSummary.GetColumnWidth(1) ? MF_CHECKED:MF_UNCHECKED));
			menu.CheckMenuItem(ID_MON_SHOWTEST_HTTPBPS,MF_BYCOMMAND|(m_ctlTestSummary.GetColumnWidth(2) ? MF_CHECKED:MF_UNCHECKED));
			menu.CheckMenuItem(ID_MON_SHOWTEST_VUS,MF_BYCOMMAND|(m_ctlTestSummary.GetColumnWidth(3) ? MF_CHECKED:MF_UNCHECKED));
			menu.CheckMenuItem(ID_MON_SHOWTEST_ACTIVEVUS,MF_BYCOMMAND|(m_ctlTestSummary.GetColumnWidth(4) ? MF_CHECKED:MF_UNCHECKED));
			menu.CheckMenuItem(ID_MON_SHOWTEST_INACTIVEVUS,MF_BYCOMMAND|(m_ctlTestSummary.GetColumnWidth(5) ? MF_CHECKED:MF_UNCHECKED));
			menu.CheckMenuItem(ID_MON_SHOWTEST_HTTP3XXRPS,MF_BYCOMMAND|(m_ctlTestSummary.GetColumnWidth(6) ? MF_CHECKED:MF_UNCHECKED));
			menu.CheckMenuItem(ID_MON_SHOWTEST_HTTP4XXRPS,MF_BYCOMMAND|(m_ctlTestSummary.GetColumnWidth(7) ? MF_CHECKED:MF_UNCHECKED));
			menu.CheckMenuItem(ID_MON_SHOWTEST_HTTP5XXRPS,MF_BYCOMMAND|(m_ctlTestSummary.GetColumnWidth(8) ? MF_CHECKED:MF_UNCHECKED));
			menu.CheckMenuItem(ID_MON_SHOWTEST_HTTPRPS,MF_BYCOMMAND|(m_ctlTestSummary.GetColumnWidth(9) ? MF_CHECKED:MF_UNCHECKED));
			menu.CheckMenuItem(ID_MON_SHOWTEST_FAILEDRPS,MF_BYCOMMAND|(m_ctlTestSummary.GetColumnWidth(10) ? MF_CHECKED:MF_UNCHECKED));
			menu.CheckMenuItem(ID_MON_SHOWTEST_TASKITERATIONS,MF_BYCOMMAND|(m_ctlTestSummary.GetColumnWidth(11) ? MF_CHECKED:MF_UNCHECKED));
			menu.CheckMenuItem(ID_MON_SHOWTEST_MINCON,MF_BYCOMMAND|(m_ctlTestSummary.GetColumnWidth(12) ? MF_CHECKED:MF_UNCHECKED));
			menu.CheckMenuItem(ID_MON_SHOWTEST_AVGCON,MF_BYCOMMAND|(m_ctlTestSummary.GetColumnWidth(13) ? MF_CHECKED:MF_UNCHECKED));
			menu.CheckMenuItem(ID_MON_SHOWTEST_MAXCON,MF_BYCOMMAND|(m_ctlTestSummary.GetColumnWidth(14) ? MF_CHECKED:MF_UNCHECKED));
			menu.CheckMenuItem(ID_MON_SHOWTEST_MINLAT,MF_BYCOMMAND|(m_ctlTestSummary.GetColumnWidth(15) ? MF_CHECKED:MF_UNCHECKED));
			menu.CheckMenuItem(ID_MON_SHOWTEST_AVGLAT,MF_BYCOMMAND|(m_ctlTestSummary.GetColumnWidth(16) ? MF_CHECKED:MF_UNCHECKED));
			menu.CheckMenuItem(ID_MON_SHOWTEST_MAXLAT,MF_BYCOMMAND|(m_ctlTestSummary.GetColumnWidth(17) ? MF_CHECKED:MF_UNCHECKED));

			CMenu* pPopup = menu.GetSubMenu(0);
			ASSERT(pPopup != NULL);

			pPopup->TrackPopupMenu(TPM_LEFTALIGN | TPM_RIGHTBUTTON, point.x, point.y,this);
		}
	}
	return;
}

void CMonitorSummary::ToggleColumn(int nCol, int nWidth)
{
	if(m_ctlSummary.GetColumnWidth(nCol))
	{
		m_ctlSummary.SetColumnWidth(nCol,0);
	}
	else
	{	
		m_ctlSummary.SetColumnWidth(nCol,nWidth);
	}	
	SaveColumnState();
	m_ctlSummary.Invalidate();

}

void CMonitorSummary::ToggleTestColumn(int nCol, int nWidth)
{
	if(m_ctlTestSummary.GetColumnWidth(nCol))
	{
		m_ctlTestSummary.SetColumnWidth(nCol,0);
	}
	else
	{	
		m_ctlTestSummary.SetColumnWidth(nCol,nWidth);
	}	
	SaveColumnState();
	m_ctlTestSummary.Invalidate();

}

void CMonitorSummary::SaveColumnState()
{
	CWinApp *pApp = AfxGetApp();

	//the task group summary

	pApp->WriteProfileInt("Monitor Enabled Cols","TaskGroup",(m_ctlSummary.GetColumnWidth(enTaskGroupCol) ? TRUE:FALSE));
	pApp->WriteProfileInt("Monitor Enabled Cols","Status",(m_ctlSummary.GetColumnWidth(enStatusCol) ? TRUE:FALSE));
	pApp->WriteProfileInt("Monitor Enabled Cols","Duration",(m_ctlSummary.GetColumnWidth(enDurationCol) ? TRUE:FALSE));
	pApp->WriteProfileInt("Monitor Enabled Cols","HTTPr",(m_ctlSummary.GetColumnWidth(enHTTPrCol) ? TRUE:FALSE));
	pApp->WriteProfileInt("Monitor Enabled Cols","HTTP3xx",(m_ctlSummary.GetColumnWidth(enHTTP3xxCol) ? TRUE:FALSE));
	pApp->WriteProfileInt("Monitor Enabled Cols","HTTP4xx",(m_ctlSummary.GetColumnWidth(enHTTP4xxCol) ? TRUE:FALSE));
	pApp->WriteProfileInt("Monitor Enabled Cols","HTTP5xx",(m_ctlSummary.GetColumnWidth(enHTTP5xxCol) ? TRUE:FALSE));
	pApp->WriteProfileInt("Monitor Enabled Cols","HTTPb",(m_ctlSummary.GetColumnWidth(enHTTPbCol) ? TRUE:FALSE));
	pApp->WriteProfileInt("Monitor Enabled Cols","VUs",(m_ctlSummary.GetColumnWidth(enTotalVuCol) ? TRUE:FALSE));
	pApp->WriteProfileInt("Monitor Enabled Cols","BytesIn",(m_ctlSummary.GetColumnWidth(enBytesInCol) ? TRUE:FALSE));
	pApp->WriteProfileInt("Monitor Enabled Cols","BytesOut",(m_ctlSummary.GetColumnWidth(enBytesOutCol) ? TRUE:FALSE));
	pApp->WriteProfileInt("Monitor Enabled Cols","Min",(m_ctlSummary.GetColumnWidth(enMinCol) ? TRUE:FALSE));
	pApp->WriteProfileInt("Monitor Enabled Cols","Max",(m_ctlSummary.GetColumnWidth(enMaxCol) ? TRUE:FALSE));
	pApp->WriteProfileInt("Monitor Enabled Cols","Avg",(m_ctlSummary.GetColumnWidth(enAvgCol) ? TRUE:FALSE));
	pApp->WriteProfileInt("Monitor Enabled Cols","HTTPrFailed",(m_ctlSummary.GetColumnWidth(enHTTPrFailedCol) ? TRUE:FALSE));
	pApp->WriteProfileInt("Monitor Enabled Cols","TaskVUs",(m_ctlSummary.GetColumnWidth(enFirstTaskCol) ? TRUE:FALSE));
	pApp->WriteProfileInt("Monitor Enabled Cols","TaskIterations",(m_ctlSummary.GetColumnWidth(enFirstTaskCol+1) ? TRUE:FALSE));
	pApp->WriteProfileInt("Monitor Enabled Cols","TaskPeriod",(m_ctlSummary.GetColumnWidth(enFirstTaskCol+2) ? TRUE:FALSE));

	pApp->WriteProfileInt("Monitor Enabled Cols","HTTPrperSecond",(m_ctlSummary.GetColumnWidth(enHTTPrperSecondCol) ? TRUE:FALSE));
	pApp->WriteProfileInt("Monitor Enabled Cols","HTTPrFailedperSecond",(m_ctlSummary.GetColumnWidth(enHTTPrFailedperSecondCol) ? TRUE:FALSE));
	pApp->WriteProfileInt("Monitor Enabled Cols","HTTP3xxPerSecond",(m_ctlSummary.GetColumnWidth(enHTTP3xxColPerSecond) ? TRUE:FALSE));
	pApp->WriteProfileInt("Monitor Enabled Cols","HTTP4xxPerSecond",(m_ctlSummary.GetColumnWidth(enHTTP4xxColPerSecond) ? TRUE:FALSE));
	pApp->WriteProfileInt("Monitor Enabled Cols","HTTP5xxPerSecond",(m_ctlSummary.GetColumnWidth(enHTTP5xxColPerSecond) ? TRUE:FALSE));
	pApp->WriteProfileInt("Monitor Enabled Cols","HTTPbperSecond",(m_ctlSummary.GetColumnWidth(enHTTPbperSecondCol) ? TRUE:FALSE));
	pApp->WriteProfileInt("Monitor Enabled Cols","BytesInperSecond",(m_ctlSummary.GetColumnWidth(enBytesInperSecondCol) ? TRUE:FALSE));
	pApp->WriteProfileInt("Monitor Enabled Cols","BytesOutperSecond",(m_ctlSummary.GetColumnWidth(enBytesOutperSecondCol) ? TRUE:FALSE));
	pApp->WriteProfileInt("Monitor Enabled Cols","TaskIterationsperSecond",(m_ctlSummary.GetColumnWidth(enTaskIterationsperSecondCol) ? TRUE:FALSE));
	pApp->WriteProfileInt("Monitor Enabled Cols","ActiveVu",(m_ctlSummary.GetColumnWidth(enActiveVuCol) ? TRUE:FALSE));
	pApp->WriteProfileInt("Monitor Enabled Cols","InactiveVu",(m_ctlSummary.GetColumnWidth(enInactiveVuCol) ? TRUE:FALSE));
	pApp->WriteProfileInt("Monitor Enabled Cols","AbortedVu",(m_ctlSummary.GetColumnWidth(enOnErrorVuCol) ? TRUE:FALSE));
	pApp->WriteProfileInt("Monitor Enabled Cols","FinishedVu",(m_ctlSummary.GetColumnWidth(enFinishedVuCol) ? TRUE:FALSE));

	//the test summary

	pApp->WriteProfileInt("Test Monitor Enabled Cols","Bytes In per Second",(m_ctlTestSummary.GetColumnWidth(0) ? TRUE:FALSE));
	pApp->WriteProfileInt("Test Monitor Enabled Cols","Bytes Out per Second",(m_ctlTestSummary.GetColumnWidth(1) ? TRUE:FALSE));
	pApp->WriteProfileInt("Test Monitor Enabled Cols","Total Bytes per Second",(m_ctlTestSummary.GetColumnWidth(2) ? TRUE:FALSE));
	pApp->WriteProfileInt("Test Monitor Enabled Cols","Started VUs",(m_ctlTestSummary.GetColumnWidth(3) ? TRUE:FALSE));
	pApp->WriteProfileInt("Test Monitor Enabled Cols","Active VUs",(m_ctlTestSummary.GetColumnWidth(4) ? TRUE:FALSE));
	pApp->WriteProfileInt("Test Monitor Enabled Cols","Inactive VUs",(m_ctlTestSummary.GetColumnWidth(5) ? TRUE:FALSE));
	pApp->WriteProfileInt("Test Monitor Enabled Cols","HTTP 3xx per Second",(m_ctlTestSummary.GetColumnWidth(6) ? TRUE:FALSE));
	pApp->WriteProfileInt("Test Monitor Enabled Cols","HTTP 4xx per Second",(m_ctlTestSummary.GetColumnWidth(7) ? TRUE:FALSE));
	pApp->WriteProfileInt("Test Monitor Enabled Cols","HTTP 5xx per Second",(m_ctlTestSummary.GetColumnWidth(8) ? TRUE:FALSE));
	pApp->WriteProfileInt("Test Monitor Enabled Cols","Failed Requests per Second",(m_ctlTestSummary.GetColumnWidth(9) ? TRUE:FALSE));
	pApp->WriteProfileInt("Test Monitor Enabled Cols","Requests per Second",(m_ctlTestSummary.GetColumnWidth(10) ? TRUE:FALSE));
	pApp->WriteProfileInt("Test Monitor Enabled Cols","Task Iterations per Second",(m_ctlTestSummary.GetColumnWidth(11) ? TRUE:FALSE));
	pApp->WriteProfileInt("Test Monitor Enabled Cols","Min Connection Time",(m_ctlTestSummary.GetColumnWidth(12) ? TRUE:FALSE));
	pApp->WriteProfileInt("Test Monitor Enabled Cols","Average Connection Time",(m_ctlTestSummary.GetColumnWidth(13) ? TRUE:FALSE));
	pApp->WriteProfileInt("Test Monitor Enabled Cols","Max Connection Time",(m_ctlTestSummary.GetColumnWidth(14) ? TRUE:FALSE));
	pApp->WriteProfileInt("Test Monitor Enabled Cols","Min Request latency",(m_ctlTestSummary.GetColumnWidth(15) ? TRUE:FALSE));
	pApp->WriteProfileInt("Test Monitor Enabled Cols","Average Request latency",(m_ctlTestSummary.GetColumnWidth(16) ? TRUE:FALSE));
	pApp->WriteProfileInt("Test Monitor Enabled Cols","Max Request latency",(m_ctlTestSummary.GetColumnWidth(17) ? TRUE:FALSE));
}


CString CMonitorSummary::ConvertToTimeMilliseconds(long milliseconds)
{
	CString answer;
	int remainder;
	div_t temp;
	int h=0;
	int m=0;
	int s=0;
	int ms=0;
	int days=0;
	

	temp=div(milliseconds,3600*24*1000);
	days=temp.quot;
	remainder=temp.rem;

	temp=div(remainder,3600*1000);
	h=temp.quot;
	remainder=temp.rem;

	temp=div(remainder,60*1000);
	m=temp.quot;
	remainder=temp.rem;

	temp=div(remainder,1000);
	s=temp.quot;
	ms=temp.rem;


	if(days>0)
	{
		answer.Format("%d days %02d:%02d:%02d.%03d",days,h,m,s,ms);
	}
	else
	{
		answer.Format("%02d:%02d:%02d.%03d",h,m,s,ms);
	}
	return answer;
}

CString CMonitorSummary::ConvertToSeconds(long milliseconds)
{
	if(milliseconds == -1)
	{
		return "-";
	}
	CString answer;
	div_t temp;

	int s=0;
	int ms=0;
	temp=div(milliseconds,1000);
	s=temp.quot;
	ms=temp.rem;

	answer.Format("%d.%03d",s,ms);

	return answer;
}

double CMonitorSummary::GetRate(double dCurrent, double dOld, int nSeconds)
{
	//divide by zero errors
	if(nSeconds<=0)
	{
		return 0.0;
	}

	return (dCurrent - dOld)/((double)nSeconds);
}

BOOL CMonitorSummary::HitTestSummary(CPoint point)
{
	CRect rc;
	m_ctlTestSummary.GetClientRect(&rc);
	m_ctlTestSummary.ClientToScreen(&rc);

	if((rc.bottom>=point.y) && (rc.top<=point.y) && (rc.left<=point.x) && (rc.right>=point.x))
	{
		return true;
	}
	return false;
}

BOOL CMonitorSummary::HitTaskGroupSummary(CPoint point)
{
	CRect rc;
	m_ctlSummary.GetClientRect(&rc);
	m_ctlSummary.ClientToScreen(&rc);

	if((rc.bottom>=point.y) && (rc.top<=point.y) && (rc.left<=point.x) && (rc.right>=point.x))
	{
		return true;
	}
	return false;
}

// message handlers for the context menus
// ON MON SHOW TEST FUNCTIONS

void CMonitorSummary::OnMonShowtestBytesinps() 
{
	ToggleTestColumn(0,100);
}

void CMonitorSummary::OnMonShowtestBytesoutps() 
{
	ToggleTestColumn(1,100);
}

void CMonitorSummary::OnMonShowtestHttpbps() 
{
	ToggleTestColumn(2,100);
}

void CMonitorSummary::OnMonShowtestVus() 
{
	ToggleTestColumn(3,100);
}

void CMonitorSummary::OnMonShowtestActivevus() 
{
	ToggleTestColumn(4,100);
}

void CMonitorSummary::OnMonShowtestInactivevus() 
{
	ToggleTestColumn(5,100);
}

void CMonitorSummary::OnMonShowtestHttp3xxrps() 
{
	ToggleTestColumn(6,100);
}

void CMonitorSummary::OnMonShowtestHttp4xxrps() 
{
	ToggleTestColumn(7,100);
}

void CMonitorSummary::OnMonShowtestHttp5xxrps() 
{
	ToggleTestColumn(8,100);
}

void CMonitorSummary::OnMonShowtestFailedrps() 
{
	ToggleTestColumn(9,120);
}

void CMonitorSummary::OnMonShowtestHttprps() 
{
	ToggleTestColumn(10,100);
}

void CMonitorSummary::OnMonShowtestTaskiterations() 
{
	ToggleTestColumn(11,100);	
}

void CMonitorSummary::OnMonShowtestMincon() 
{
	ToggleTestColumn(12,100);
}

void CMonitorSummary::OnMonShowtestAvgcon() 
{
	ToggleTestColumn(13,100);
}

void CMonitorSummary::OnMonShowtestMaxcon() 
{
	ToggleTestColumn(14,100);
}

void CMonitorSummary::OnMonShowtestMinlat() 
{
	ToggleTestColumn(15,100);
}

void CMonitorSummary::OnMonShowtestAvglat() 
{
	ToggleTestColumn(16,100);
}

void CMonitorSummary::OnMonShowtestMaxlat() 
{
	ToggleTestColumn(17,100);
}


// ON MON SHOW FUNCTIONS
void CMonitorSummary::OnMonShowActivevus() 
{
	ToggleColumn(enActiveVuCol,75);
}

void CMonitorSummary::OnMonShowBytesinps() 
{
	ToggleColumn(enBytesInperSecondCol,85);
}

void CMonitorSummary::OnMonShowBytesoutps() 
{
	ToggleColumn(enBytesOutperSecondCol,85);
}

void CMonitorSummary::OnMonShowHttpbps() 
{
	ToggleColumn(enHTTPbperSecondCol,100);
}

void CMonitorSummary::OnMonShowHttprps() 
{
	ToggleColumn(enHTTPrperSecondCol,100);	
}

void CMonitorSummary::OnMonShowInactivevus() 
{
	ToggleColumn(enInactiveVuCol,75);
}

void CMonitorSummary::OnMonShowTasktotiterations() 
{
	ToggleColumn(enTaskIterationsperSecondCol,120);
}

void CMonitorSummary::OnMonShowTaskGroup()
{
	ToggleColumn(enTaskGroupCol,100);
}
void CMonitorSummary::OnMonShowStatus()
{
	ToggleColumn(enStatusCol,100);
}
void CMonitorSummary::OnMonShowDuration()
{
	ToggleColumn(enDurationCol,75);
}
void CMonitorSummary::OnMonShowHttpR()
{
	ToggleColumn(enHTTPrCol,100);
}
void CMonitorSummary::OnMonShowHttpB()
{
	ToggleColumn(enHTTPbCol,100);
}
void CMonitorSummary::OnMonShowBytesIn()
{
	ToggleColumn(enBytesInCol,85);
}
void CMonitorSummary::OnMonShowBytesOut()
{
	ToggleColumn(enBytesOutCol,85);
}
void CMonitorSummary::OnMonShowMin()
{
	ToggleColumn(enMinCol,100);
}
void CMonitorSummary::OnMonShowMax()
{
	ToggleColumn(enMaxCol,100);
}
void CMonitorSummary::OnMonShowAvg()
{
	ToggleColumn(enAvgCol,100);
}
void CMonitorSummary::OnMonShowVus()
{
	ToggleColumn(enTotalVuCol,75);
}
void CMonitorSummary::OnMonShowTaskVus()
{
	//iterate for all tasks
	for (int a=0; a<=m_nMaxUsedTaskCol; a++)
	{
		ToggleColumn((enFirstTaskCol+(3*a)),TASKVUCOL_WIDTH);
	}
}
void CMonitorSummary::OnMonShowTaskIterations()
{
	//iterate for all tasks
	for (int a=0; a<=m_nMaxUsedTaskCol; a++)
	{
		ToggleColumn((enFirstTaskCol+1+(3*a)),TASKITCOL_WIDTH);
	}

}
void CMonitorSummary::OnMonShowTaskPeriod()
{
	//iterate for all tasks
	for (int a=0; a<=m_nMaxUsedTaskCol; a++)
	{
		ToggleColumn((enFirstTaskCol+2+(3*a)),TASKITCOL_WIDTH);
	}
}

void CMonitorSummary::OnMonShowErrorvus() 
{
	ToggleColumn(enOnErrorVuCol,100);
}

void CMonitorSummary::OnMonShowFailedr() 
{
	ToggleColumn(enHTTPrFailedCol,100);
}

void CMonitorSummary::OnMonShowFailedrps() 
{
	ToggleColumn(enHTTPrFailedperSecondCol,100);
}

void CMonitorSummary::OnMonShowFinishedvus() 
{
	ToggleColumn(enFinishedVuCol,100);
}

void CMonitorSummary::OnMonShowHttp3xxr() 
{
	ToggleColumn(enHTTP3xxCol,100);
}

void CMonitorSummary::OnMonShowHttp3xxrps() 
{
	ToggleColumn(enHTTP3xxColPerSecond,100);
}

void CMonitorSummary::OnMonShowHttp4xxr() 
{
	ToggleColumn(enHTTP4xxCol,100);
}

void CMonitorSummary::OnMonShowHttp4xxrps() 
{
	ToggleColumn(enHTTP4xxColPerSecond,100);
}

void CMonitorSummary::OnMonShowHttp5xxr() 
{
	ToggleColumn(enHTTP5xxCol,100);
}

void CMonitorSummary::OnMonShowHttp5xxrps() 
{
	ToggleColumn(enHTTP5xxColPerSecond,100);
}

void CMonitorSummary::DisplaySimpleColumns()
{
	//this function should be called the first time that the monitoring is used so the user is not scared
	//MessageBox("","first time displaying simple columns only",MB_OK);

	CWinApp *pApp = AfxGetApp();

	//the task group summary

	pApp->WriteProfileInt("Monitor Enabled Cols","TaskGroup",TRUE);
	pApp->WriteProfileInt("Monitor Enabled Cols","Status",TRUE);
	pApp->WriteProfileInt("Monitor Enabled Cols","Duration",TRUE);
	pApp->WriteProfileInt("Monitor Enabled Cols","HTTPr",TRUE);
	pApp->WriteProfileInt("Monitor Enabled Cols","HTTP3xx",TRUE);
	pApp->WriteProfileInt("Monitor Enabled Cols","HTTP4xx",TRUE);
	pApp->WriteProfileInt("Monitor Enabled Cols","HTTP5xx",TRUE);
	pApp->WriteProfileInt("Monitor Enabled Cols","HTTPb",TRUE);
	pApp->WriteProfileInt("Monitor Enabled Cols","HTTPrFailedperSecond",FALSE);
	pApp->WriteProfileInt("Monitor Enabled Cols","HTTP3xxPerSecond",FALSE);
	pApp->WriteProfileInt("Monitor Enabled Cols","HTTP4xxPerSecond",FALSE);
	pApp->WriteProfileInt("Monitor Enabled Cols","HTTP5xxPerSecond",FALSE);
	pApp->WriteProfileInt("Monitor Enabled Cols","VUs",TRUE);
	pApp->WriteProfileInt("Monitor Enabled Cols","BytesIn",FALSE);
	pApp->WriteProfileInt("Monitor Enabled Cols","BytesOut",FALSE);
	pApp->WriteProfileInt("Monitor Enabled Cols","Min",FALSE);
	pApp->WriteProfileInt("Monitor Enabled Cols","Max",FALSE);
	pApp->WriteProfileInt("Monitor Enabled Cols","Avg",FALSE);
	pApp->WriteProfileInt("Monitor Enabled Cols","HTTPe",TRUE);
	pApp->WriteProfileInt("Monitor Enabled Cols","TaskVUs",TRUE);
	pApp->WriteProfileInt("Monitor Enabled Cols","TaskIterations",FALSE);
	pApp->WriteProfileInt("Monitor Enabled Cols","TaskPeriod",FALSE);
	pApp->WriteProfileInt("Monitor Enabled Cols","HTTPrperSecond",FALSE);
	pApp->WriteProfileInt("Monitor Enabled Cols","HTTPbperSecond",FALSE);
	pApp->WriteProfileInt("Monitor Enabled Cols","BytesInperSecond",FALSE);
	pApp->WriteProfileInt("Monitor Enabled Cols","BytesOutperSecond",FALSE);
	pApp->WriteProfileInt("Monitor Enabled Cols","TaskIterationsperSecond",FALSE);
	pApp->WriteProfileInt("Monitor Enabled Cols","ActiveVu",FALSE);
	pApp->WriteProfileInt("Monitor Enabled Cols","InactiveVu",FALSE);
	pApp->WriteProfileInt("Monitor Enabled Cols","AbortedVu",FALSE);
	pApp->WriteProfileInt("Monitor Enabled Cols","FinishedVu",FALSE);

	//the test summary

	pApp->WriteProfileInt("Test Monitor Enabled Cols","Bytes In per Second",FALSE);
	pApp->WriteProfileInt("Test Monitor Enabled Cols","Bytes Out per Second",FALSE);
	pApp->WriteProfileInt("Test Monitor Enabled Cols","Total Bytes per Second",TRUE);
	pApp->WriteProfileInt("Test Monitor Enabled Cols","Started VUs",TRUE);
	pApp->WriteProfileInt("Test Monitor Enabled Cols","Active VUs",TRUE);
	pApp->WriteProfileInt("Test Monitor Enabled Cols","Inactive VUs",TRUE);
	pApp->WriteProfileInt("Test Monitor Enabled Cols","HTTP 3xx per Second",FALSE);
	pApp->WriteProfileInt("Test Monitor Enabled Cols","HTTP 4xx per Second",FALSE);
	pApp->WriteProfileInt("Test Monitor Enabled Cols","HTTP 5xx per Second",FALSE);
	pApp->WriteProfileInt("Test Monitor Enabled Cols","Failed Requests per Second",FALSE);
	pApp->WriteProfileInt("Test Monitor Enabled Cols","Requests per Second",FALSE);
	pApp->WriteProfileInt("Test Monitor Enabled Cols","Task Iterations per Second",TRUE);
	pApp->WriteProfileInt("Test Monitor Enabled Cols","Min Connection Time",FALSE);
	pApp->WriteProfileInt("Test Monitor Enabled Cols","Average Connection Time",FALSE);
	pApp->WriteProfileInt("Test Monitor Enabled Cols","Max Connection Time",FALSE);
	pApp->WriteProfileInt("Test Monitor Enabled Cols","Min Request latency",FALSE);
	pApp->WriteProfileInt("Test Monitor Enabled Cols","Average Request latency",TRUE);
	pApp->WriteProfileInt("Test Monitor Enabled Cols","Max Request latency",FALSE);


	//set the dont force the column selection if not the first time = 1
	//so this is only done once, after the first time the user chooses
	pApp->WriteProfileInt("Monitoring","UserDefined",TRUE);
}

