/* Copyright (C) 2000-2001 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/
// MonitorVU.cpp : implementation file
//

#include "stdafx.h"
#include "resource.h"
#include "MonitorVU.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

#define IDT_POLL_DATA 1
static int ID_REFRESH_RATE = 2000;
const int VtChSeriesType2dLine = 6;
/////////////////////////////////////////////////////////////////////////////
// CMonitorVU

CMonitorVU::CMonitorVU()
{
	m_nIntervalDisplaying = ID_REFRESH_RATE;
	HKEY hKey;
	if(RegOpenKeyEx(HKEY_CURRENT_USER,"SOFTWARE\\OpenSTA\\TestPlugin\\Settings",0,KEY_READ,&hKey)==ERROR_SUCCESS)
	{
		DWORD dwInterval;
		DWORD dwDataSize = sizeof(DWORD);
		if(RegQueryValueEx(hKey,"Task Monitoring Interval",NULL,NULL,(LPBYTE)&dwInterval,&dwDataSize)== ERROR_SUCCESS)
		{
			m_nIntervalDisplaying = dwInterval*1000;
		}
		RegCloseKey(hKey);
	}

	m_bDataFileOpen = false;
	m_nFilePos = 0;

	m_pChart=NULL;
	m_colPlotBk = RGB(0xff,0xff,0xff);
    m_strLabelFont = "Verdana";
    m_colGrid = RGB(0xf0,0xca,0xa6);
    m_colLabel = RGB(0,0,0);
    m_colAxisText = RGB(0,0,0);
	m_nRollingChecked=0;
	LoadGraphStyles();
}

CMonitorVU::~CMonitorVU()
{
}


BEGIN_MESSAGE_MAP(CMonitorVU, CWnd)
	//{{AFX_MSG_MAP(CMonitorVU)
	ON_WM_SIZE()
	ON_WM_CREATE()
	ON_WM_DESTROY()
	ON_WM_TIMER()
	ON_WM_CONTEXTMENU()
	ON_COMMAND(ID_ROLLING_GRAPH_VU, ToggleRollingGraphs)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()


/////////////////////////////////////////////////////////////////////////////
// CMonitorVU message handlers

void CMonitorVU::OnSize(UINT nType, int cx, int cy) 
{
	CWnd::OnSize(nType, cx, cy);
	if(m_pChart->GetSafeHwnd())
	{
		m_pChart->MoveWindow(0,0,cx,cy);
	}
	
}

CXChartCtrl* CMonitorVU::CreateDisplay()
{
    CXChartCtrl *pChart = new CXChartCtrl;

    if(!pChart)
        return NULL;

	CRect rcGraph;
	GetClientRect(rcGraph);

    pChart->Create(NULL,WS_CHILD|WS_VISIBLE,rcGraph,this,0);

    pChart->SetPlotBkColor(m_colPlotBk);

    pChart->SetLabelColor(m_colLabel);

    pChart->SetAxisTextColor(m_colAxisText);

    pChart->SetGridColor(m_colGrid);
    pChart->SetLabelFont(m_strLabelFont);

    pChart->AddDataPoint(1,0.0);
    pChart->SetSeriesStyle(1,VtChSeriesType2dLine);
    pChart->EnableZoom(FALSE);

	//Setup data colors
	COleSafeArray saData;
	saData.CreateOneDim(VT_I4,m_arDefLineColours.GetSize(),m_arDefLineColours.GetData());
    pChart->SetDataColors(saData);

	pChart->SetYAxisLabel("VUs");
    return pChart;
}

int CMonitorVU::OnCreate(LPCREATESTRUCT lpCreateStruct) 
{
	if (CWnd::OnCreate(lpCreateStruct) == -1)
	{
		return -1;
	}
	m_pChart=CreateDisplay();
	UpdateGraph();
	SetTimer(IDT_POLL_DATA, m_nIntervalDisplaying, NULL);
	
	return 0;
}

void CMonitorVU::LoadGraphStyles()
{
	CWinApp *pApp = AfxGetApp();

    m_colPlotBk = pApp->GetProfileInt("GraphStyles","BGCol",(int)m_colPlotBk);
    m_strLabelFont = pApp->GetProfileString("GraphStyles","LabelFont", m_strLabelFont);
    m_colGrid = pApp->GetProfileInt("GraphStyles","GridCol",(int)m_colGrid);
    m_colLabel = pApp->GetProfileInt("GraphStyles","LabelCol",(int)m_colLabel);
    m_colAxisText = pApp->GetProfileInt("GraphStyles","ATCol",(int)m_colAxisText);

	CString strKey;
	int nNumLineColors = m_arDefLineColours.GetSize();
	for(int nCol=0;nCol<nNumLineColors;nCol++)
	{
		strKey.Format("LineCol%d",nCol);
		m_arDefLineColours[nCol] = pApp->GetProfileInt("GraphStyles",strKey,(int)m_arDefLineColours[nCol]);
	}
}


BOOL CMonitorVU::Init(CString path)
{
	 m_strTestRunPath=path;
	 return true;
}

BOOL CMonitorVU::UpdateGraph()
{
	try
    {
		if (!m_bDataFileOpen)
		{
			//open the file
			CFileStatus fs;
			CString Path = 	m_strTestRunPath+"\\VUserslog.txt";
			if (!CFile::GetStatus(Path, fs))
			{
				return false;
			}
			m_bDataFileOpen = m_dataFile.Open(Path, CFile::modeRead | CFile::typeText | CFile::shareDenyNone );
			if (!m_bDataFileOpen)
			{
				return false;
			}
		}
		m_dataFile.Seek(m_nFilePos, CFile::begin);

        CString strData;

		while(m_dataFile.ReadString(strData))
		{
			CString time, number;
			GetNextField(',', strData, time);
			GetNextField(',', strData, number);

			//draw a point on the graph
			m_pChart->AddDataPoint(1,atof(number));
		
		}
		m_nFilePos = m_dataFile.GetPosition();
	
    }
    catch(...)
    {
		return false;
    }	

	return true;
}

void CMonitorVU::OnDestroy() 
{
	CWnd::OnDestroy();
	KillTimer(IDT_POLL_DATA);
	return;
}

void CMonitorVU::OnTimer(UINT nIDEvent) 
{	
	CWnd::OnTimer(nIDEvent);
	UpdateGraph();
}


int CMonitorVU::GetNextField(char sep, CString &inStr, CString &field)
{
    if (inStr == "")
    {
        return -1;
    }

    int len = inStr.GetLength();
    int nPos = inStr.Find(sep, 0);

    if (nPos == -1)
    {
        field = inStr;
        inStr = "";
        return 0;
    }
    else
    {
        field = inStr.Left(nPos);
        inStr = inStr.Right(len - nPos - 1);
        return 0;
    }
}


void CMonitorVU::OnContextMenu(CWnd* pWnd, CPoint point) 
{
    if (point.x == -1 && point.y == -1)
    {
        //keystroke invocation
        CRect rect;
        GetClientRect(rect);
        ClientToScreen(rect);

        point = rect.TopLeft();
        point.Offset(5, 5);
    }
    CMenu menu;	
	menu.LoadMenu(IDR_ROLLING_GRAPHS_VU);

	menu.CheckMenuItem(ID_ROLLING_GRAPH_VU,MF_BYCOMMAND|(m_nRollingChecked ? MF_CHECKED:MF_UNCHECKED));
	
    CMenu* pPopup = menu.GetSubMenu(0);
    ASSERT(pPopup != NULL);
	pPopup->TrackPopupMenu(TPM_LEFTALIGN | TPM_RIGHTBUTTON, point.x, point.y,this);

}

void CMonitorVU::ToggleRollingGraphs()
{
	if(m_nRollingChecked==0)
	{
		m_pChart->SetRollingGraph(20);
		m_nRollingChecked=1;
	}
	else
	{
		m_nRollingChecked=0;
		KillTimer(IDT_POLL_DATA);
		delete m_pChart;
		m_pChart=CreateDisplay();
		m_nFilePos=0;
		UpdateGraph();
		SetTimer(IDT_POLL_DATA, m_nIntervalDisplaying, NULL);
	}

}

