/* Copyright (C) 2001 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

// MonitoringDlg.cpp : implementation file
//

#include "stdafx.h"
#include "MonitoringDlg.h"
#include "PluginTypes.h"
#include "TestMonitorPlugin.h"
#include "CmdExec.h"
#include "ostadefs.h"
#include "TaskMonitorIntervalDlg.h"
#include "StepByStepDlg.h"
#include "BaseUIHelpID.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CMonitoringDlg dialog
extern CToolBar* m_pDebugToolbar;
extern CToolBar* m_pToolbar;
extern CPluginManager* m_pPluginManager;
extern HINSTANCE m_hInstance;
extern HWND m_hWndNotify;

#define IDT_SHOW_CONTROLBARS 1

CMonitoringDlg::CMonitoringDlg(CWnd* pParent /*=NULL*/)
	: CDialog(CMonitoringDlg::IDD, pParent)
{
	//{{AFX_DATA_INIT(CMonitoringDlg)
	//}}AFX_DATA_INIT
	m_pSBSWnd = NULL;
	m_pMonitorBar = NULL;
	m_bMonBarVisible = true;
	m_strTestName = "";
	m_strManagerReg = "";
	m_bTestActive = false;
	m_pFrame = NULL;
	m_bFirstSize = false;
}


void CMonitoringDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CMonitoringDlg)
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(CMonitoringDlg, CDialog)
	//{{AFX_MSG_MAP(CMonitoringDlg)
	ON_WM_SHOWWINDOW()
	ON_WM_SIZE()
	ON_COMMAND(ID_TILEHORIZONTALLY, OnTilehorizontally)
	ON_COMMAND(ID_TILEVERTICALLY, OnTilevertically)
	ON_COMMAND(ID_CASCADE, OnCascade)
	ON_COMMAND(ID_TOGGLE_SHOWRUNS, OnToggleShowMonBar)
	ON_UPDATE_COMMAND_UI(ID_TOGGLE_SHOWRUNS, OnUpdateToggleShowMonBar)
	ON_COMMAND(ID_CLOSEALL,OnCloseAll)
	ON_COMMAND(ID_TASKMON_INTERVAL, OnTaskmonInterval)
	ON_WM_HELPINFO()
	ON_COMMAND(ID_END_STEPBYSTEP, OnEndStepbystep)
	ON_UPDATE_COMMAND_UI(ID_END_STEPBYSTEP, OnUpdateEndStepbystep)
	ON_WM_TIMER()
	//}}AFX_MSG_MAP
	ON_MESSAGE(WMU_MDISTATECHANGED,OnMDIStateChanged)
	ON_MESSAGE(WMU_MONITOR_SELCHANGED,OnMonitorSelChanged)
	ON_WM_ERASEBKGND()
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CMonitoringDlg message handlers

void CMonitoringDlg::OnShowWindow(BOOL bShow, UINT nStatus) 
{
	CDialog::OnShowWindow(bShow, nStatus);

	int nGroup = m_pTestFile->GetStepByStepTaskGroup();
	
	ShowControlBars(bShow);
	if(bShow)
	{
		if(nGroup!=-1)
		{
			ShowStepByStepScreen();
			m_bFirstSize = true;
		}
		else
		if(m_pSBSWnd)
		{
			m_pSBSWnd->MDIDestroy();
			m_pSBSWnd = NULL;
		}

		for (int i = 0; i<m_arrOpenWnds.GetSize(); i++)
		{
			OpenWnd OW = m_arrOpenWnds.GetAt(i);
			OW.pChildWnd->GetGraph()->SendMessage(WM_SHOWWINDOW,TRUE);
		}
	}
	else
	{
		if(m_pSBSWnd)
			m_pSBSWnd->GetGraph()->SendMessage(WM_SHOWWINDOW,FALSE);

		for (int i = 0; i<m_arrOpenWnds.GetSize(); i++)
		{
			OpenWnd OW = m_arrOpenWnds.GetAt(i);
			OW.pChildWnd->GetGraph()->SendMessage(WM_SHOWWINDOW,FALSE);
		}
	}
}

void CMonitoringDlg::ShowControlBars(BOOL bShow)
{
	if(bShow)
	{
		int nGroup = m_pTestFile->GetStepByStepTaskGroup();

		if(m_bMonBarVisible && m_pMonitorBar)
		{
			m_pMonitorBar->Refresh();
			m_pMonitorBar->m_pDockSite->ShowControlBar(m_pMonitorBar,TRUE,TRUE);
		}
		
		if(m_pToolbar)
			m_pToolbar->m_pDockSite->ShowControlBar(m_pToolbar,TRUE,TRUE);

		if(m_pDebugToolbar)
			m_pDebugToolbar->m_pDockSite->ShowControlBar(m_pDebugToolbar,(nGroup!=-1),TRUE);
	}
	else
	{
		if(m_pMonitorBar)
		{
			m_bMonBarVisible = m_pMonitorBar->IsWindowVisible();
			m_pMonitorBar->m_pDockSite->ShowControlBar(m_pMonitorBar,FALSE,TRUE);
		}

		if(m_pToolbar)
			m_pToolbar->m_pDockSite->ShowControlBar(m_pToolbar,FALSE,TRUE);
		
		if(m_pDebugToolbar)
			m_pDebugToolbar->m_pDockSite->ShowControlBar(m_pDebugToolbar,FALSE,TRUE);
	}

	((CFrameWnd*)AfxGetMainWnd())->RecalcLayout(FALSE);

}

void CMonitoringDlg::SetTestFile(CTestFile* pTestFile)
{
	m_pTestFile = pTestFile;
	CString strTFPath = m_pTestFile->GetTestFilePath();
	if(m_pMonitorBar)
	{
		m_pMonitorBar->SetTestFile(m_pTestFile);
		m_pMonitorBar->SetNotifyWindow(m_hWnd);
	}

	int nLen = strTFPath.GetLength();
	int nPos = strTFPath.ReverseFind('\\');
	m_strTestPath = strTFPath.Left(nPos+1);

	m_strTestName = strTFPath.Right(nLen-nPos-1);
	nPos = m_strTestName.ReverseFind('.');
	m_strTestName = m_strTestName.Left(nPos);
}


BOOL CMonitoringDlg::OnInitDialog() 
{
	CDialog::OnInitDialog();
	
	CRect rcClient;
	GetClientRect(rcClient);

	m_pFrame = new CMDIFrameWnd;
	m_pFrame->LoadFrame(IDR_MAINFRAME,FWS_ADDTOTITLE|WS_CHILD|WS_VISIBLE ,this);
	m_pFrame->UpdateWindow();
	m_pFrame->ModifyStyle(FWS_ADDTOTITLE,0);
	m_pFrame->MoveWindow(rcClient);

	if(m_pMonitorBar)
	{
		m_pMonitorBar->EnableTree(m_bTestActive);
		m_pMonitorBar->SetNotifyWindow(m_hWnd);
	}

	SetTimer(IDT_SHOW_CONTROLBARS,2000,NULL);
	
	return TRUE;  // return TRUE unless you set the focus to a control
	              // EXCEPTION: OCX Property Pages should return FALSE
}

void CMonitoringDlg::OnSize(UINT nType, int cx, int cy) 
{
	CDialog::OnSize(nType, cx, cy);
	
	if(m_pFrame && cx && cy)
	{
		m_pFrame->MoveWindow(0,0,cx,cy);
		
		//Dirty way to ensure SBS window is within client area at startup
		if(m_pSBSWnd && m_bFirstSize && m_pMonitorBar && m_pMonitorBar->IsWindowVisible())	
		{
			m_pSBSWnd->MoveWindow(0,0,cx,cy);
			m_bFirstSize = false;
		}
	}
}

BOOL CMonitoringDlg::FindTestRunFolder()
{
	CFileFind FF;
	if(!FF.FindFile(m_strTestPath + "*.*"))
		return false;

	BOOL bContinue=true;
	CTime tmLast(0);
	while(bContinue)
	{
		bContinue = FF.FindNextFile();

		if(FF.IsDirectory() && !FF.IsDots() && !FF.IsHidden())
		{
			CTime tmFolder;
			FF.GetCreationTime(tmFolder);
			if (tmFolder > tmLast)
			{
				m_strRunFolder = FF.GetFilePath();
				tmLast = tmFolder;
			}
		}
	}

	return !m_strRunFolder.IsEmpty();
}

void CMonitoringDlg::ShowErrorLog(const CString& strTitle)
{
	OpenWnd OW;
	OW.monInfo.nTaskGroup = -1;
	OW.monInfo.strScreenName = strTitle;
	OW.strTitle = strTitle;
	CRect rcChild;
	OW.pChildWnd = CreateNewChild(OW.strTitle);
	OW.pChildWnd->GetClientRect(rcChild);

	AfxSetResourceHandle(m_hInstance);

	CMonitoringErrorLog* pErrorLog = new CMonitoringErrorLog();
	pErrorLog->Init(m_strRunFolder);
	pErrorLog->Create(IDD_MONITORING_ERROR_LOG, OW.pChildWnd);
	pErrorLog->ShowWindow(SW_SHOW);
	pErrorLog->MoveWindow(0,0,rcChild.Width(), rcChild.Height());
	OW.pChildWnd->SetGraph(pErrorLog);
	OW.pChildWnd->SetNotifyWindow(m_hWnd);

	AfxSetResourceHandle(AfxGetInstanceHandle());
	m_arrOpenWnds.Add(OW);
}

void CMonitoringDlg::ShowSummary(const CString& strTitle)
{
	OpenWnd OW;
	OW.monInfo.nTaskGroup = -1;
	OW.monInfo.strScreenName = strTitle;
	OW.strTitle = strTitle;
	CRect rcChild;
	OW.pChildWnd = CreateNewChild(OW.strTitle);
	OW.pChildWnd->GetClientRect(rcChild);

	AfxSetResourceHandle(m_hInstance);

	CMonitorSummary* pSummary = new CMonitorSummary();
	pSummary->Init(m_strManagerReg, m_pTestFile);
	pSummary->Create(IDD_MON_SUMMARY, OW.pChildWnd);
	pSummary->ShowWindow(SW_SHOW);
	pSummary->MoveWindow(0,0,rcChild.Width(), rcChild.Height());
	OW.pChildWnd->SetGraph(pSummary);
	OW.pChildWnd->SetNotifyWindow(m_hWnd);

	AfxSetResourceHandle(AfxGetInstanceHandle());

	m_arrOpenWnds.Add(OW);
}

void CMonitoringDlg::OnOK() 
{
}

void CMonitoringDlg::OnCancel() 
{
}

void CMonitoringDlg::SetTestActive(CString strReg)
{
	if (strReg.IsEmpty())
		return;

	if(m_pMonitorBar)
	{
		m_pMonitorBar->EnableTree(true);
		m_pMonitorBar->SetTestStatus("ACTIVE");
	}

	m_strManagerReg = strReg;
	m_strManagerReg.TrimRight(" \r\n");
	FindTestRunFolder();

	if(m_pSBSWnd && (m_pTestFile->GetStepByStepTaskGroup()!=-1))
		((CStepByStepDlg*)m_pSBSWnd->GetGraph())->SetTestActive(strReg);
				
	m_bTestActive = true;
}

void CMonitoringDlg::StopActiveTest()
{
	if(m_pMonitorBar)
	{
		m_pMonitorBar->EnableTree(false);
		m_pMonitorBar->ClearAllChecks();
		m_pMonitorBar->SetTestStatus("INACTIVE");
	}

	if(m_pSBSWnd)
		((CStepByStepDlg*)m_pSBSWnd->GetGraph())->StopActiveTest();

	m_bTestActive = false;
	m_strManagerReg.Empty();

	while (m_arrOpenWnds.GetSize() > 0)
	{
		OpenWnd OW = m_arrOpenWnds.GetAt(0);
		OW.pChildWnd->MDIDestroy();
	}
}

CGraphWnd *CMonitoringDlg::CreateNewChild(const CString& strTitle)
{
	HINSTANCE hInst = AfxGetResourceHandle();
	HMENU hMDIMenu  = ::LoadMenu(hInst, MAKEINTRESOURCE(IDR_MDITESTYPE));
	HACCEL hMDIAccel = ::LoadAccelerators(hInst, MAKEINTRESOURCE(IDR_MDITESTYPE));

	CRuntimeClass *pClass = RUNTIME_CLASS(CGraphWnd);
	ASSERT(pClass != NULL);
	CGraphWnd *pFrame = (CGraphWnd*) pClass->CreateObject();
	ASSERT_KINDOF(CGraphWnd, pFrame);

	// load the frame
	CCreateContext context;
	context.m_pCurrentFrame = m_pFrame;

	if(!pFrame->Create(NULL,strTitle,WS_CHILD|WS_VISIBLE|WS_OVERLAPPEDWINDOW,CFrameWnd::rectDefault,m_pFrame,&context))
	{
		TRACE0("Couldn't load frame window.\n");
		delete (LPVOID)pFrame;
		return NULL;
	}
	pFrame->SetHandles(hMDIMenu, hMDIAccel);
	pFrame->SetNotifyWindow(m_hWnd);

	return pFrame;
}

LRESULT CMonitoringDlg::OnMDIStateChanged(WPARAM wParam,LPARAM lParam)
{
	HWND hWnd = (HWND)wParam;
	for(int nNdx=0;nNdx<m_arrOpenWnds.GetSize();nNdx++)
	{
		if(m_arrOpenWnds[nNdx].pChildWnd->m_hWnd == hWnd)
		{
			switch(lParam)
			{
			case CGraphWnd::enActive:
				UpdateDialogControls(this,TRUE);
			return 0;
			case CGraphWnd::enInactive:
			return 0;
			case CGraphWnd::enDestroyed:
				if(m_pMonitorBar)
					m_pMonitorBar->SetCheck(m_arrOpenWnds.GetAt(nNdx).monInfo,false);
				
				CWnd *pWnd = m_arrOpenWnds[nNdx].pChildWnd;
				m_arrOpenWnds.RemoveAt(nNdx);
			return 0;
			}
		}
	}
	
	return 0;
}

void CMonitoringDlg::OnTilehorizontally() 
{
	m_pFrame->MDITile(MDITILE_HORIZONTAL);	
}

void CMonitoringDlg::OnTilevertically() 
{
	m_pFrame->MDITile(MDITILE_VERTICAL);	
}

void CMonitoringDlg::OnCascade() 
{
	m_pFrame->MDICascade();	
}

void CMonitoringDlg::SetMonitoringBar(CMonitoringBar* pBar)
{
	m_pMonitorBar = pBar;
	m_pMonitorBar->SetNotifyWindow(m_hWnd);
	m_pMonitorBar->SetTestFile(m_pTestFile);
}

BOOL CMonitoringDlg::PreTranslateMessage(MSG* pMsg) 
{
	switch(pMsg->message)
	{
	case WM_KEYDOWN:
		switch(pMsg->wParam)
		{
		case VK_F5:
			if(m_pMonitorBar)
				m_pMonitorBar->Refresh();
		break;
		}
	break;
	}
	
	return CDialog::PreTranslateMessage(pMsg);
}

LRESULT CMonitoringDlg::OnMonitorSelChanged(WPARAM wParam,LPARAM lParam)
{
	CMonitoringBar::NM_MONITOR *pMon = (CMonitoringBar::NM_MONITOR*)wParam;

//	if (!m_bTestActive)
//		return 0;

	if(pMon->nTaskGroup < 0)
	{
		BOOL bShow = (BOOL)lParam;
		// Check to see if the window is already open
		for (int i = 0; i<m_arrOpenWnds.GetSize(); i++)
		{
			OpenWnd OW = m_arrOpenWnds.GetAt(i);
			if (OW.strTitle == pMon->strScreenName)
			{
				if(!bShow)
					OW.pChildWnd->MDIDestroy();
				return 0;
			}
		}

		if(pMon->strScreenName == "Summary")
		{
			ShowSummary(pMon->strScreenName);
		}
		else
		if(pMon->strScreenName == "Error Log")
		{
			ShowErrorLog(pMon->strScreenName);
		}
		else
		if(pMon->strScreenName == "Total Active Users")
		{
			ShowTotalActiveUsers(pMon->strScreenName);
		}
		return 0;
	}
	

	CStringArray arrScreens;
	CString strTaskGroup = m_pTestFile->GetTaskGroupName(pMon->nTaskGroup);

	// Find the plugin responsible for the screen and activate it
	CStringArray arrPlugs;
	m_pPluginManager->ListPluginsByType(TEST_MONITOR_PLUGIN, arrPlugs);

	arrScreens.Add(pMon->strScreenName);
	for(int j =0; j<arrPlugs.GetSize(); j++)
	{
		CTestMonitorPlugin plug(arrPlugs.GetAt(j), m_pPluginManager);
		if(plug.IsSupportedExtension(pMon->strTargetType))
		{
			// Check to see if the window is already open
			for (int i = 0; i<m_arrOpenWnds.GetSize(); i++)
			{
				OpenWnd OW = m_arrOpenWnds.GetAt(i);
				if(OW.monInfo == *pMon)
				{
					//Close the window
					if(!lParam)	
						OW.pChildWnd->MDIDestroy();

					return 0;
				}
			}
			if(!lParam)	
				return 0;

			CStringArray arScreens;
			plug.GetAvailableScreens(m_pTestFile,pMon->nTaskGroup,arScreens);

			BOOL bFound=false;
			for(int nScr=0;nScr<arScreens.GetSize()&& !bFound;nScr++)
			{
				bFound = (arScreens[nScr] == pMon->strScreenName);
			}
			if(!bFound)
				continue;

			OpenWnd OW;
			OW.monInfo = *pMon;
			OW.strTitle = strTaskGroup + " - " + pMon->strScreenName;
			CRect rcChild;
			OW.pChildWnd = CreateNewChild(OW.strTitle);
			OW.pChildWnd->GetClientRect(rcChild);

			CWnd* pMonWnd = plug.ViewScreen(m_strManagerReg, m_strRunFolder, arrScreens, OW.pChildWnd);
			pMonWnd->ShowWindow(SW_SHOW);
			pMonWnd->MoveWindow(0,0,rcChild.Width(), rcChild.Height());
			OW.pChildWnd->SetGraph(pMonWnd);
			OW.pChildWnd->SetNotifyWindow(m_hWnd);

			m_arrOpenWnds.Add(OW);
		}
	}	

	return 0;
}

void CMonitoringDlg::OnToggleShowMonBar() 
{
	if(!m_pMonitorBar)
		return;

	BOOL bShow = !m_pMonitorBar->IsVisible();
	if(m_pMonitorBar)
		m_pMonitorBar->m_pDockSite->ShowControlBar(m_pMonitorBar,bShow,TRUE);
	
	((CFrameWnd*)AfxGetMainWnd())->RecalcLayout();//was false
}

void CMonitoringDlg::OnUpdateToggleShowMonBar(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable( IsWindowVisible());
	if(m_pMonitorBar)
		pCmdUI->SetCheck(m_pMonitorBar->IsWindowVisible() );
}

BOOL CMonitoringDlg::OnEraseBkgnd(CDC *pDC)
{
	return true;
}

void CMonitoringDlg::ShowTotalActiveUsers(const CString& strTitle)
{
	OpenWnd OW;
	OW.monInfo.nTaskGroup = -1;
	OW.monInfo.strScreenName = strTitle;
	OW.strTitle = strTitle;
	CRect rcChild;
	OW.pChildWnd = CreateNewChild(OW.strTitle);
	OW.pChildWnd->GetClientRect(rcChild);

	AfxSetResourceHandle(m_hInstance);
	
	CMonitorVU* pVUs = new CMonitorVU();
	pVUs->Init(m_strRunFolder);
	pVUs->Create(NULL,NULL,WS_CHILD|WS_VISIBLE,rcChild,OW.pChildWnd,0);
	pVUs->ShowWindow(SW_SHOW);
	pVUs->MoveWindow(0,0,rcChild.Width(), rcChild.Height());
	OW.pChildWnd->SetGraph(pVUs);
	OW.pChildWnd->SetNotifyWindow(m_hWnd);

	AfxSetResourceHandle(AfxGetInstanceHandle());

	m_arrOpenWnds.Add(OW);
}

void CMonitoringDlg::OnCloseAll()
{
	if(m_pMonitorBar)
		m_pMonitorBar->ClearAllChecks();

	while (m_arrOpenWnds.GetSize() > 0)
	{
		OpenWnd OW = m_arrOpenWnds.GetAt(0);
		OW.pChildWnd->MDIDestroy();
	}
}
void CMonitoringDlg::OnTaskmonInterval() 
{
	AfxSetResourceHandle(m_hInstance);
	
	CTaskMonitorIntervalDlg dlg;
	if(dlg.DoModal()==IDOK)
	{
	}

	AfxSetResourceHandle(AfxGetInstanceHandle());
}

void CMonitoringDlg::ShowStepByStepScreen()
{
	if(m_pSBSWnd)
	{
		m_pSBSWnd->ShowWindow(SW_SHOW);
		return;
	}

	CRect rcChild;
	CGraphWnd *pChildWnd = CreateNewChild("Single Step");

	pChildWnd->EnableCloseButton(FALSE);
//	pChildWnd->GetClientRect(rcChild);

	AfxSetResourceHandle(m_hInstance);
	
	CStepByStepDlg *pDlg = new CStepByStepDlg;
	pDlg->Create(IDD_SBS_DLG,pChildWnd);
	pDlg->ShowWindow(SW_SHOW);
//	pDlg->MoveWindow(0,0,rcChild.Width(), rcChild.Height());
	
	pChildWnd->SetGraph(pDlg);
	pChildWnd->SetNotifyWindow(m_hWnd);
	
	CRect rcFrame;
	m_pFrame->GetClientRect(rcFrame);
	pChildWnd->MoveWindow(rcFrame);

	AfxSetResourceHandle(AfxGetInstanceHandle());
	m_pSBSWnd = pChildWnd;

	if(m_pSBSWnd && !m_strManagerReg.IsEmpty())
		((CStepByStepDlg*)m_pSBSWnd->GetGraph())->SetTestActive(m_strManagerReg);

	m_bFirstSize = true;
}

BOOL CMonitoringDlg::OnCmdMsg(UINT nID, int nCode, void* pExtra, AFX_CMDHANDLERINFO* pHandlerInfo) 
{
	if(m_pSBSWnd && m_pSBSWnd->m_bActive && 
		m_pSBSWnd->GetGraph()->OnCmdMsg(nID,nCode,pExtra,pHandlerInfo))
			return true;

	return CDialog::OnCmdMsg(nID, nCode, pExtra, pHandlerInfo);
}

BOOL CMonitoringDlg::OnHelpInfo(HELPINFO* pHelpInfo) 
{
	pHelpInfo->dwContextId = HIDD_MONITORING;
	return CDialog::OnHelpInfo(pHelpInfo);
}

void CMonitoringDlg::OnEndStepbystep() 
{
	if(m_pSBSWnd)
	{
		m_pSBSWnd->MDIDestroy();
		m_pSBSWnd = NULL;
	}
	
	if(m_pDebugToolbar)
		m_pDebugToolbar->m_pDockSite->ShowControlBar(m_pDebugToolbar,FALSE,TRUE);
	
	::SendMessage(m_hWndNotify,WMU_SETSTEPBYSTEPMODE,-1,FALSE);
}

void CMonitoringDlg::OnUpdateEndStepbystep(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable(m_pSBSWnd && !((CStepByStepDlg*)m_pSBSWnd->GetGraph())->IsTestRunning());
}


void CMonitoringDlg::OnTimer(UINT nIDEvent) 
{
	if(IsWindowVisible())
		ShowControlBars(true);
	KillTimer(IDT_SHOW_CONTROLBARS);
	
	CDialog::OnTimer(nIDEvent);
}
