/* Copyright (C) 2001 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/
// TestMonitorDebugPlugin.cpp: implementation of the CStepByStepPlugin class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "Plugin.h"
#include "PluginManager.h"
#include "PluginTypes.h"
#include "TestMonitorDebugPlugin.h"
#include "TestFile.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////
CStepByStepPlugin::CStepByStepPlugin(const CString& strTitle,CPluginManager* pPluginManager):
	m_fncNextRequest(NULL),
	m_fncRequestComplete(NULL),
	m_fncGetConfigStates(NULL),
	m_fncSetActiveUser(NULL)
{
	m_pPlugin = pPluginManager->GetPlugin(TEST_MONITOR_DEBUG_PLUGIN, strTitle);
}

CStepByStepPlugin::~CStepByStepPlugin()
{

}

void CStepByStepPlugin::GetAvailableViews(CTestFile* pTestFile,CStringArray& arstrViews)
{
	LPGETVIEWS func = (LPGETVIEWS)m_pPlugin->GetFunction("GetAvailableViews");
	if(!func)
		return ;

	func(pTestFile, arstrViews);
}

CWnd* CStepByStepPlugin::DisplayView(const CString& strView,const CString& strTestRunPath,CWnd *pParent)
{
	LPDISPLAYVIEWS func = (LPDISPLAYVIEWS)m_pPlugin->GetFunction("DisplayViews");
	if (!func)
		return NULL;

	return func(strView,strTestRunPath,pParent);
}

void CStepByStepPlugin::NextRequest(const CString& strProfile,const CNextRequestData& NRD,DWORD dwMonitoringState)
{
	if(m_fncNextRequest == (LPNEXTREQUEST)0xffffffff)
		return;
	else
	if(!m_fncNextRequest)
	{
		m_fncNextRequest = (LPNEXTREQUEST)m_pPlugin->GetFunction("NextRequest");
		if (!m_fncNextRequest)
		{
			m_fncNextRequest = (LPNEXTREQUEST)0xffffffff;
			return;
		}
	}
	
	m_fncNextRequest(strProfile,NRD,dwMonitoringState);
}

void CStepByStepPlugin::RequestComplete(const CString& strProfile,const CRequestCompleteData& RCD)
{
	if(m_fncRequestComplete == (LPREQUESTCOMPLETE)0xffffffff)
		return;
	else
	if(!m_fncRequestComplete)
	{
		m_fncRequestComplete = (LPREQUESTCOMPLETE)m_pPlugin->GetFunction("RequestComplete");
		if (!m_fncRequestComplete)
		{
			m_fncRequestComplete = (LPREQUESTCOMPLETE)0xffffffff;
			return;
		}
	}
	
	m_fncRequestComplete(strProfile,RCD);
}

void CStepByStepPlugin::SetRunFolder(const CString& strRunFolder)
{
	LPSETRUNFOLDER func = (LPSETRUNFOLDER)m_pPlugin->GetFunction("SetRunFolder");
	if (!func)
		return;

	func(strRunFolder);
}

BOOL CStepByStepPlugin::IsSupportedExtension(const CString& strExtension)
{

	LPISSUPPORTEDEXT func = (LPISSUPPORTEDEXT)m_pPlugin->GetFunction("IsSupportedExtension");
	if (!func)
		return FALSE;

	return func(strExtension);
}

void CStepByStepPlugin::GetDefaultConfigStates(const CString& strGateway,CStringArray& arTypes,DWORD& dwDefStates)
{
	if(m_fncGetConfigStates == (LPGETCONFIGSTATES)0xffffffff)
		return;
	else
	if(!m_fncGetConfigStates)
	{
		m_fncGetConfigStates = (LPGETCONFIGSTATES)m_pPlugin->GetFunction("GetDefaultConfigStates");
		if (!m_fncGetConfigStates)
		{
			m_fncGetConfigStates = (LPGETCONFIGSTATES)0xffffffff;
			return;
		}
	}

	m_fncGetConfigStates(strGateway,arTypes,dwDefStates);
}

void CStepByStepPlugin::SetActiveUser(int nVUId)
{
	if(m_fncSetActiveUser == (LPSETACTIVEUSER)0xffffffff)
		return;
	else
	if(!m_fncSetActiveUser)
	{
		m_fncSetActiveUser = (LPSETACTIVEUSER)m_pPlugin->GetFunction("SetActiveUser");
		if (!m_fncSetActiveUser)
		{
			m_fncSetActiveUser = (LPSETACTIVEUSER)0xffffffff;
			return;
		}
	}

	m_fncSetActiveUser(nVUId);
}

void CStepByStepPlugin::SetTestActive(const CString& strReg)
{
	LPSETTESTACTIVE func = (LPSETTESTACTIVE)m_pPlugin->GetFunction("SetTestActive");
	if (!func)
		return;

	func(strReg);
}

void CStepByStepPlugin::StopActiveTest()
{
	LPSTOPACTIVETEST func = (LPSTOPACTIVETEST)m_pPlugin->GetFunction("StopActiveTest");
	if (!func)
		return;

	func();
}
