/* Copyright (C) 2000-2001 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/
// DeleteRunDlg.cpp : implementation file
//

#include "stdafx.h"
#include "DeleteRunDlg.h"
#include "BaseUIHelpID.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CDeleteRunDlg dialog


CDeleteRunDlg::CDeleteRunDlg(const CString& strTestFolder,CWnd* pParent /*=NULL*/)
	: CDialog(CDeleteRunDlg::IDD, pParent),
	m_strTestFolder(strTestFolder)
{
	//{{AFX_DATA_INIT(CDeleteRunDlg)
		// NOTE: the ClassWizard will add member initialization here
	//}}AFX_DATA_INIT
}


void CDeleteRunDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CDeleteRunDlg)
	DDX_Control(pDX, IDC_RUN_LIST, m_ctlRunList);
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(CDeleteRunDlg, CDialog)
	//{{AFX_MSG_MAP(CDeleteRunDlg)
	ON_WM_HELPINFO()
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CDeleteRunDlg message handlers

BOOL CDeleteRunDlg::OnInitDialog() 
{
	CDialog::OnInitDialog();
	
	CFileFind FF;
	if(!FF.FindFile(m_strTestFolder + "*.00?"))
		return FALSE;

	BOOL bContinue=true;
	while(bContinue)
	{
		bContinue = FF.FindNextFile();

		if(FF.IsDirectory() && !FF.IsDots() && !FF.IsHidden())
		{
			m_ctlRunList.AddString(FF.GetFileName());
		}
	}

	
	return TRUE;  // return TRUE unless you set the focus to a control
	              // EXCEPTION: OCX Property Pages should return FALSE
}

void CDeleteRunDlg::OnOK() 
{
	int nSelCount = m_ctlRunList.GetSelCount();
	int *pSel = new int[nSelCount];
	m_ctlRunList.GetSelItems(nSelCount,pSel);

	CString strRun;
	for(int nRun=0;nRun<nSelCount;nRun++)
	{
		m_ctlRunList.GetText(pSel[nRun],strRun);
		if(!DeleteFolder(m_strTestFolder + "\\" + strRun))
			return;
	}
	
	CDialog::OnOK();
}

BOOL CDeleteRunDlg::DeleteFolder(const CString& strFolder)
{
	CFileFind FF;
	if(!FF.FindFile(strFolder + "\\*.*"))
		return FALSE;

	BOOL bContinue=true;
	while(bContinue)
	{
		bContinue = FF.FindNextFile();
		if(FF.IsDots())
			continue;

		if(FF.IsDirectory())
		{
			if(!DeleteFolder(FF.GetFilePath()))
				return false;
		}
		else
		{
			if(!DeleteFile(FF.GetFilePath()))
			{
				CString str;
				str.Format("Could not delete \"%s\"\n\n%s\nContinue with remaining files?",FF.GetFilePath(),GetErrorMessage(GetLastError()));
				if(AfxMessageBox(str,MB_YESNO)==IDNO)
					return false;
			}
		}
	}
	FF.Close();
	
	if(!RemoveDirectory(strFolder))
	{
		CString str;
		str.Format("Could not delete \"%s\"\n\n%s\nContinue with remaining files?",strFolder,GetErrorMessage(GetLastError()));
		if(AfxMessageBox(str,MB_YESNO)==IDNO)
			return false;
	}

	return true;
}

CString CDeleteRunDlg::GetErrorMessage(DWORD dwError)
{
	LPVOID lpMsgBuf;
	FormatMessage( 
		FORMAT_MESSAGE_ALLOCATE_BUFFER | 
		FORMAT_MESSAGE_FROM_SYSTEM | 
		FORMAT_MESSAGE_IGNORE_INSERTS,
		NULL,
		GetLastError(),
		MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT), // Default language
		(LPTSTR) &lpMsgBuf,
		0,
		NULL 
	);
	
	CString str((LPCTSTR)lpMsgBuf);
	str.Replace("\n"," ");
	str.Remove('\r');

	// Free the buffer.
	LocalFree( lpMsgBuf );

	return str;
}

BOOL CDeleteRunDlg::OnHelpInfo(HELPINFO* pHelpInfo) 
{
	pHelpInfo->dwContextId = HIDD_DELETE_RUNS_DLG;
	return CDialog::OnHelpInfo(pHelpInfo);
}
