/* Copyright (C) 2001 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

// ResultsDlg.cpp : implementation file
//

#include "stdafx.h"
#include "XAuditViewer.h"
#include "ResultsDlg.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CResultsDlg dialog


CResultsDlg::CResultsDlg(CWnd* pParent /*=NULL*/)
	: CDialog(CResultsDlg::IDD, pParent)
{
	//{{AFX_DATA_INIT(CResultsDlg)
	//}}AFX_DATA_INIT
	m_nCurrentType = enNone;
}


void CResultsDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CResultsDlg)
	DDX_Control(pDX, IDC_RESULTSLIST, m_ctlResultsList);
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(CResultsDlg, CDialog)
	//{{AFX_MSG_MAP(CResultsDlg)
	ON_WM_SIZE()
	ON_WM_CONTEXTMENU()
	ON_COMMAND(ID_EXPORT, OnExport)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CResultsDlg message handlers

BOOL CResultsDlg::OnInitDialog() 
{
	CDialog::OnInitDialog();

	ShowType(m_nCurrentType);
	
	return TRUE;  // return TRUE unless you set the focus to a control
	              // EXCEPTION: OCX Property Pages should return FALSE
}

void CResultsDlg::OnSize(UINT nType, int cx, int cy) 
{
	CDialog::OnSize(nType, cx, cy);

	if(!m_ctlResultsList.GetSafeHwnd())
		return;

	CRect rcList;
	m_ctlResultsList.GetWindowRect(&rcList);
	ScreenToClient(&rcList);

	m_ctlResultsList.SetWindowPos(NULL,0,0,cx-(rcList.left*2),cy-rcList.top-rcList.left,SWP_NOZORDER|SWP_NOMOVE);
}

BOOL CResultsDlg::SetRunFolder(const CString& strPath)
{
	m_strRunFolder = strPath;
	return ShowType(m_nCurrentType);
}

BOOL CResultsDlg::ShowType(AuditType nType)
{
	if((nType == enNone) || m_strRunFolder.IsEmpty())
		return false;

	m_ctlResultsList.DeleteAllItems();

	switch(nType)
	{
	case enAudit:
		return ShowAudit();
	case enReport:
		return ShowReport();
	case enHistory:
		return ShowHistory();
	case enError:
		return ShowError();
	case enHTTPSnapShot:
		return ShowHTTPSnapShot();
	default:
		ASSERT(FALSE);
	break;
	}
	return false;
}

BOOL CResultsDlg::ShowError()
{
	m_ctlResultsList.SetRedraw(FALSE);
	m_ctlResultsList.SetFixedRowCount(1);
	m_ctlResultsList.InsertColumn("Time Stamp",DT_LEFT|DT_VCENTER|DT_SINGLELINE);
	m_ctlResultsList.InsertColumn("Test Name",DT_LEFT|DT_VCENTER|DT_SINGLELINE);
	m_ctlResultsList.InsertColumn("Location",DT_LEFT|DT_VCENTER|DT_SINGLELINE);
	m_ctlResultsList.InsertColumn("Message",DT_LEFT|DT_VCENTER|DT_SINGLELINE);
	
    int nRow = 1;
    try
    {
        CString strErrorPath(m_strRunFolder  + "\\ErrLog.txt");
        CStdioFile ErrorFile(strErrorPath,CFile::modeRead);
        
		CString strTime;
		CString strName;
		CString strLocation;
		CString strMessage;
        CString strLine;
        while(ErrorFile.ReadString(strLine))
        {
			int nStartPos = 0;
			int nEndPos = strLine.Find(',',0);
			if(nEndPos == -1)
				continue;

			if(strLine.IsEmpty() || !isdigit(strLine[0]))
				continue;
			
			CTime time(atol( strLine.Mid(nStartPos,nEndPos-nStartPos) ));
			strTime=( time.Format("%d/%m/%y %H:%M:%S"));
			nStartPos = nEndPos+1;

            //    ...parse ',' separated message

			CString strThread,strScript,strSrclin,strMsg;//,strExe,strPID;
			if((nEndPos = strLine.Find(',',nStartPos))!=-1)
			{
				strName = strLine.Mid(nStartPos,nEndPos-nStartPos);
				nStartPos = nEndPos+1;
			}

			if((nEndPos = strLine.Find(',',nStartPos))!=-1)
			{
				strLocation= strLine.Mid(nStartPos,nEndPos-nStartPos);
				nStartPos = nEndPos+1;
			}
			
			//to end of line in case message contains ","
			strMessage = strLine.Mid(nStartPos);
			

			m_ctlResultsList.InsertRow("");
			m_ctlResultsList.SetItemText(nRow,0,strTime);
			m_ctlResultsList.SetItemText(nRow,1,strName);
			m_ctlResultsList.SetItemText(nRow,2,strLocation);
			m_ctlResultsList.SetItemText(nRow,3,strMessage);
			
			m_ctlResultsList.SetCellEnabled(nRow,0,FALSE);
			m_ctlResultsList.SetCellEnabled(nRow,1,FALSE);
			m_ctlResultsList.SetCellEnabled(nRow,2,FALSE);
			m_ctlResultsList.SetCellEnabled(nRow,3,FALSE);
		
            //    ...and move onto the next entry
            nRow++;
        }
		m_ctlResultsList.AutoSizeColumns();
	}
    catch(...)
    {
	}

	m_ctlResultsList.SetRedraw(TRUE);
    return true;
}

BOOL CResultsDlg::ShowAudit()
{
	typedef enum { enTimeCol,enThreadCol,enScriptCol,enSrclinCol,enMsgCol} ListColumns;
    
	m_ctlResultsList.SetRedraw(FALSE);
	m_ctlResultsList.SetFixedRowCount(1);
	m_ctlResultsList.InsertColumn("Time Stamp",DT_LEFT|DT_VCENTER|DT_SINGLELINE);
	m_ctlResultsList.InsertColumn("User ID",DT_LEFT|DT_VCENTER|DT_SINGLELINE);
	m_ctlResultsList.InsertColumn("Script",DT_LEFT|DT_VCENTER|DT_SINGLELINE);
	m_ctlResultsList.InsertColumn("Line #",DT_LEFT|DT_VCENTER|DT_SINGLELINE);
	m_ctlResultsList.InsertColumn("Message",DT_LEFT|DT_VCENTER|DT_SINGLELINE);
	
    int nRow = 1;
    try
    {
        CString strAuditPath(m_strRunFolder  + "\\TestLog.txt");
        CStdioFile AuditFile(strAuditPath,CFile::modeRead);
        
        CString strLine;
        while(AuditFile.ReadString(strLine))
        {
			int nStartPos = 0;
			int nEndPos = strLine.Find(',',0);
			if(nEndPos == -1)
				continue;

			if(strLine.IsEmpty() || !isdigit(strLine[0]))
				continue;
			
			CTime time(atol( strLine.Mid(nStartPos,nEndPos-nStartPos) ));
			CString strTime( time.Format("%d/%m/%y %H:%M:%S"));
			nStartPos = nEndPos+1;

            //    ...parse '|' separated message
			CString strThread,strScript,strSrclin,strMsg;//,strExe,strPID;
			if((nEndPos = strLine.Find('|',nStartPos))!=-1)
			{
//				strExe = strLine.Mid(nStartPos,nEndPos-nStartPos);
				nStartPos = nEndPos+1;
			}

			if((nEndPos = strLine.Find('|',nStartPos))!=-1)
			{
//				strPID = strLine.Mid(nStartPos,nEndPos-nStartPos);
				nStartPos = nEndPos+1;
			}
			
			if((nEndPos = strLine.Find('|',nStartPos))!=-1)
			{
				strThread = strLine.Mid(nStartPos,nEndPos-nStartPos);
				nStartPos = nEndPos+1;
			}
			
			if((nEndPos = strLine.Find('|',nStartPos))!=-1)
			{
				strScript = strLine.Mid(nStartPos,nEndPos-nStartPos);
				nStartPos = nEndPos+1;
			}
			
			if((nEndPos = strLine.Find('|',nStartPos))!=-1)
			{
				strSrclin = strLine.Mid(nStartPos,nEndPos-nStartPos);
				nStartPos = nEndPos+1;
			}
			
//			if((nEndPos = strLine.Find(',',nStartPos))!=-1)
//			{
				strMsg = strLine.Mid(nStartPos);//,nEndPos-nStartPos);
//				nStartPos = nEndPos+1;
//			}

			m_ctlResultsList.InsertRow("");
            m_ctlResultsList.SetItemText(nRow,enTimeCol,strTime);
			m_ctlResultsList.SetCellEnabled(nRow,enTimeCol,FALSE);
			m_ctlResultsList.SetItemText(nRow,enThreadCol,strThread);
			m_ctlResultsList.SetCellEnabled(nRow,enThreadCol,FALSE);
			m_ctlResultsList.SetItemText(nRow,enScriptCol,strScript);
			m_ctlResultsList.SetCellEnabled(nRow,enScriptCol,FALSE);
			m_ctlResultsList.SetItemText(nRow,enSrclinCol,strSrclin);
			m_ctlResultsList.SetCellEnabled(nRow,enSrclinCol,FALSE);
            m_ctlResultsList.SetItemText(nRow,enMsgCol,strMsg);
			m_ctlResultsList.SetCellEnabled(nRow,enMsgCol,FALSE);

            //    ...and move onto the next entry
            nRow++;
        }
		m_ctlResultsList.AutoSizeColumns();
	}
    catch(...)
    {
	}

	m_ctlResultsList.SetRedraw(TRUE);
    return true;
}

BOOL CResultsDlg::ShowReport()
{
	typedef enum { enTimeCol,enThreadCol,enScriptCol,enSrclinCol,enMsgCol} ListColumns;
    
	m_ctlResultsList.SetRedraw(FALSE);
	m_ctlResultsList.SetFixedRowCount(1);
	m_ctlResultsList.InsertColumn("Time Stamp",DT_LEFT|DT_VCENTER|DT_SINGLELINE);
	m_ctlResultsList.InsertColumn("User ID",DT_LEFT|DT_VCENTER|DT_SINGLELINE);
	m_ctlResultsList.InsertColumn("Script",DT_LEFT|DT_VCENTER|DT_SINGLELINE);
	m_ctlResultsList.InsertColumn("Line #",DT_LEFT|DT_VCENTER|DT_SINGLELINE);
	m_ctlResultsList.InsertColumn("Message",DT_LEFT|DT_VCENTER|DT_SINGLELINE);
	
    int nRow = 1;
    try
    {
        CString strAuditPath(m_strRunFolder  + "\\TestRep.txt");
        CStdioFile AuditFile(strAuditPath,CFile::modeRead);
        
        CString strLine;
        while(AuditFile.ReadString(strLine))
        {
			int nStartPos = 0;
			int nEndPos = strLine.Find(',',0);
			if(nEndPos == -1)
				continue;
			
			if(strLine.IsEmpty() || !isdigit(strLine[0]))
				continue;

			CTime time(atol( strLine.Mid(nStartPos,nEndPos-nStartPos) ));
			CString strTime( time.Format("%d/%m/%y %H:%M:%S"));
			nStartPos = nEndPos+1;

            //    ...parse '|' separated message
			CString strThread,strScript,strSrclin,strMsg;//,strExe,strPID;
			if((nEndPos = strLine.Find('|',nStartPos))!=-1)
			{
//				strExe = strLine.Mid(nStartPos,nEndPos-nStartPos);
				nStartPos = nEndPos+1;
			}

			if((nEndPos = strLine.Find('|',nStartPos))!=-1)
			{
//				strPID = strLine.Mid(nStartPos,nEndPos-nStartPos);
				nStartPos = nEndPos+1;
			}
			
			if((nEndPos = strLine.Find('|',nStartPos))!=-1)
			{
				strThread = strLine.Mid(nStartPos,nEndPos-nStartPos);
				nStartPos = nEndPos+1;
			}
			
			if((nEndPos = strLine.Find('|',nStartPos))!=-1)
			{
				strScript = strLine.Mid(nStartPos,nEndPos-nStartPos);
				nStartPos = nEndPos+1;
			}
			
			if((nEndPos = strLine.Find('|',nStartPos))!=-1)
			{
				strSrclin = strLine.Mid(nStartPos,nEndPos-nStartPos);
				nStartPos = nEndPos+1;
			}
			
//			if((nEndPos = strLine.Find(',',nStartPos))!=-1)
//			{
				strMsg = strLine.Mid(nStartPos);//,nEndPos-nStartPos);
//				nStartPos = nEndPos+1;
//			}

			m_ctlResultsList.InsertRow("");
            m_ctlResultsList.SetItemText(nRow,enTimeCol,strTime);
			m_ctlResultsList.SetCellEnabled(nRow,enTimeCol,FALSE);
			m_ctlResultsList.SetItemText(nRow,enThreadCol,strThread);
			m_ctlResultsList.SetCellEnabled(nRow,enThreadCol,FALSE);
			m_ctlResultsList.SetItemText(nRow,enScriptCol,strScript);
			m_ctlResultsList.SetCellEnabled(nRow,enScriptCol,FALSE);
			m_ctlResultsList.SetItemText(nRow,enSrclinCol,strSrclin);
			m_ctlResultsList.SetCellEnabled(nRow,enSrclinCol,FALSE);
            m_ctlResultsList.SetItemText(nRow,enMsgCol,strMsg);
			m_ctlResultsList.SetCellEnabled(nRow,enMsgCol,FALSE);
            
            //    ...and move onto the next entry
            nRow++;
        }

		m_ctlResultsList.AutoSizeColumns();
    }
    catch(...)
    {
	}

	m_ctlResultsList.SetRedraw(TRUE);
    return true;
}

BOOL CResultsDlg::ShowHistory()
{
	m_ctlResultsList.SetRedraw(FALSE);
	m_ctlResultsList.SetFixedRowCount(1);
	m_ctlResultsList.InsertColumn("Time Stamp",DT_LEFT|DT_VCENTER|DT_SINGLELINE);
	m_ctlResultsList.InsertColumn("User ID",DT_LEFT|DT_VCENTER|DT_SINGLELINE);
	m_ctlResultsList.InsertColumn("Script",DT_LEFT|DT_VCENTER|DT_SINGLELINE);
	m_ctlResultsList.InsertColumn("Line #",DT_LEFT|DT_VCENTER|DT_SINGLELINE);
	m_ctlResultsList.InsertColumn("Message",DT_LEFT|DT_VCENTER|DT_SINGLELINE);

	int nSlashPos = m_strRunFolder.ReverseFind('\\');
	CString strTestFolder(m_strRunFolder.Left(nSlashPos));

	BOOL bResult = true;
	CFileFind FF;
	if(FF.FindFile(strTestFolder + "\\*.*"))
	{
		BOOL bContinue = true;
		while(bContinue)
		{
			bContinue = FF.FindNextFile();
			if(FF.IsDirectory() && !FF.IsDots() && !FF.IsHidden())
				bResult &= ShowHistoryInPath(FF.GetFilePath());
		}
	}
	m_ctlResultsList.SetRedraw(TRUE);

	return bResult;
}

BOOL CResultsDlg::ShowHistoryInPath(const CString& strRunFolder)
{
	typedef enum { enTimeCol,enThreadCol,enScriptCol,enSrclinCol,enMsgCol} ListColumns;
	
    int nRow = m_ctlResultsList.GetRowCount();
    try
    {
        CString strAuditPath(strRunFolder  + "\\TestHis.txt");
        CStdioFile AuditFile(strAuditPath,CFile::modeRead);
        
        CString strLine;
        while(AuditFile.ReadString(strLine))
        {
			int nStartPos = 0;
			int nEndPos = strLine.Find(',',0);
			if(nEndPos == -1)
				continue;
			
			if(strLine.IsEmpty() || !isdigit(strLine[0]))
				continue;

			CTime time(atol( strLine.Mid(nStartPos,nEndPos-nStartPos) ));
			CString strTime( time.Format("%d/%m/%y %H:%M:%S"));
			nStartPos = nEndPos+1;

            //    ...parse '|' separated message
			CString strThread,strScript,strSrclin,strMsg;//,strExe,strPID;
			if((nEndPos = strLine.Find('|',nStartPos))!=-1)
			{
//				strExe = strLine.Mid(nStartPos,nEndPos-nStartPos);
				nStartPos = nEndPos+1;
			}

			if((nEndPos = strLine.Find('|',nStartPos))!=-1)
			{
//				strPID = strLine.Mid(nStartPos,nEndPos-nStartPos);
				nStartPos = nEndPos+1;
			}
			
			if((nEndPos = strLine.Find('|',nStartPos))!=-1)
			{
				strThread = strLine.Mid(nStartPos,nEndPos-nStartPos);
				nStartPos = nEndPos+1;
			}
			
			if((nEndPos = strLine.Find('|',nStartPos))!=-1)
			{
				strScript = strLine.Mid(nStartPos,nEndPos-nStartPos);
				nStartPos = nEndPos+1;
			}
			
			if((nEndPos = strLine.Find('|',nStartPos))!=-1)
			{
				strSrclin = strLine.Mid(nStartPos,nEndPos-nStartPos);
				nStartPos = nEndPos+1;
			}
			
//			if((nEndPos = strLine.Find(',',nStartPos))!=-1)
//			{
				strMsg = strLine.Mid(nStartPos);//,nEndPos-nStartPos);
//				nStartPos = nEndPos+1;
//			}

			m_ctlResultsList.InsertRow("");
            m_ctlResultsList.SetItemText(nRow,enTimeCol,strTime);
			m_ctlResultsList.SetCellEnabled(nRow,enTimeCol,FALSE);
			m_ctlResultsList.SetItemText(nRow,enThreadCol,strThread);
			m_ctlResultsList.SetCellEnabled(nRow,enThreadCol,FALSE);
			m_ctlResultsList.SetItemText(nRow,enScriptCol,strScript);
			m_ctlResultsList.SetCellEnabled(nRow,enScriptCol,FALSE);
			m_ctlResultsList.SetItemText(nRow,enSrclinCol,strSrclin);
			m_ctlResultsList.SetCellEnabled(nRow,enSrclinCol,FALSE);
			m_ctlResultsList.SetItemText(nRow,enMsgCol,strMsg);
			m_ctlResultsList.SetCellEnabled(nRow,enMsgCol,FALSE);

            //    ...and move onto the next entry
            nRow++;
        }

		m_ctlResultsList.AutoSizeColumns();
    }
    catch(...)
    {
	}

    return true;
}

BOOL CResultsDlg::ShowHTTPSnapShot()
{

	CString temp;
	m_nMaxUsedTaskCol=0;
	const int enFirstTaskCol=14;

	m_ctlResultsList.SetRedraw(FALSE);
	m_ctlResultsList.SetFixedRowCount(1);

	//check if the final file has been created already
	if(CreateSingleHTTPDataFile()==false)
	{
		//some error
		m_ctlResultsList.InsertColumn("Error",DT_LEFT|DT_VCENTER|DT_SINGLELINE);
		m_ctlResultsList.InsertRow("");
		m_ctlResultsList.SetItemText(1,0,"Error loading files");
		m_ctlResultsList.AutoSizeColumns();
		return true;
	}

	// now load the stats from the file, all these results are valid
		

	CFile fInput(m_strRunFolder + "\\Combined_HTTPDataFile.dat",CFile::modeRead|CFile::typeBinary);

	long lTemp;
	int nRow=1;
	CString strTemp;
	BOOL bContinue=true;
	const int buffSize=200;
	char* buffer = new char[buffSize];
	try
	{
		//get version number
		int nVersion=-1;
		fInput.Read(&nVersion,sizeof(int));

		if(nVersion==0)
		{
			m_ctlResultsList.InsertColumn("TimeStamp",DT_LEFT|DT_VCENTER|DT_SINGLELINE);
			m_ctlResultsList.InsertColumn("Executer Name",DT_LEFT|DT_VCENTER|DT_SINGLELINE);
			m_ctlResultsList.InsertColumn("Avg Connection Time",DT_LEFT|DT_VCENTER|DT_SINGLELINE);
			m_ctlResultsList.InsertColumn("Task Group Id",DT_LEFT|DT_VCENTER|DT_SINGLELINE);
			m_ctlResultsList.InsertColumn("Completed iterations",DT_LEFT|DT_VCENTER|DT_SINGLELINE);
			m_ctlResultsList.InsertColumn("Run Time",DT_LEFT|DT_VCENTER|DT_SINGLELINE);
			m_ctlResultsList.InsertColumn("Total Users",DT_LEFT|DT_VCENTER|DT_SINGLELINE);
			m_ctlResultsList.InsertColumn("HTTP Requests",DT_LEFT|DT_VCENTER|DT_SINGLELINE);
			m_ctlResultsList.InsertColumn("HTTP Errors",DT_LEFT|DT_VCENTER|DT_SINGLELINE);
			m_ctlResultsList.InsertColumn("Bytes In",DT_LEFT|DT_VCENTER|DT_SINGLELINE);
			m_ctlResultsList.InsertColumn("Bytes Out",DT_LEFT|DT_VCENTER|DT_SINGLELINE);
			m_ctlResultsList.InsertColumn("Min Request Latency",DT_LEFT|DT_VCENTER|DT_SINGLELINE);
			m_ctlResultsList.InsertColumn("Max Request Latency",DT_LEFT|DT_VCENTER|DT_SINGLELINE);
			m_ctlResultsList.InsertColumn("Average Request Latency",DT_LEFT|DT_VCENTER|DT_SINGLELINE);

			m_ctlResultsList.InsertColumn("Task 1 (VU's)",DT_LEFT|DT_VCENTER|DT_SINGLELINE);
			m_ctlResultsList.InsertColumn("Task 1 (Iterations)",DT_LEFT|DT_VCENTER|DT_SINGLELINE);
			m_ctlResultsList.InsertColumn("Task 1 (Period)",DT_LEFT|DT_VCENTER|DT_SINGLELINE);
			m_ctlResultsList.InsertRow("");
			while(1)
			{
				
				//time
				if(fInput.Read(&lTemp,sizeof(long)))
				{
					CTime time(lTemp);
					strTemp = (time.Format("%d/%m/%y %H:%M:%S"));
					m_ctlResultsList.SetItemText(nRow,0,strTemp);
					m_ctlResultsList.SetCellEnabled(nRow,0,FALSE);
				}
				else
				{
					break;
				}

				//executer name
				if(fInput.Read(&lTemp,sizeof(long)))
				{
				
					if(lTemp>buffSize)
					{
						lTemp=buffSize;
					}
					if(fInput.Read(buffer,lTemp))
					{
						strTemp.Format("%s",buffer);
						m_ctlResultsList.SetItemText(nRow,1,strTemp);
						m_ctlResultsList.SetCellEnabled(nRow,1,FALSE);
					}
				}
				
				//avg connection time
				if(fInput.Read(&lTemp,sizeof(long)))
				{
					strTemp = ConvertToTimeMilliseconds(lTemp);
					m_ctlResultsList.SetItemText(nRow,2,strTemp);
					m_ctlResultsList.SetCellEnabled(nRow,2,FALSE);
				}

				if(fInput.Read(&lTemp,sizeof(long)))
				{
					long numberOfTaskGroupSnaps=lTemp;
					for (long a=0; a<numberOfTaskGroupSnaps; a++)
					{
						//task group id +1
						if(fInput.Read(&lTemp,sizeof(long)))
						{
								strTemp.Format("%d",lTemp+1);
								m_ctlResultsList.SetItemText(nRow,3,strTemp);
								m_ctlResultsList.SetCellEnabled(nRow,3,FALSE);
						}

						//completed iterations
						if(fInput.Read(&lTemp,sizeof(long)))
						{
							strTemp.Format("%d",lTemp);
							m_ctlResultsList.SetItemText(nRow,4,strTemp);
							m_ctlResultsList.SetCellEnabled(nRow,4,FALSE);
						}
							//run time
						if(fInput.Read(&lTemp,sizeof(long)))
						{
							strTemp = ConvertToTime(lTemp);
							m_ctlResultsList.SetItemText(nRow,5,strTemp);
							m_ctlResultsList.SetCellEnabled(nRow,5,FALSE);
						}
						for (int count= 6 ; count <11 ; count++)
						{
							if(fInput.Read(&lTemp,sizeof(long)))
							{
								strTemp.Format("%d",lTemp);
								m_ctlResultsList.SetItemText(nRow,count,strTemp);
								m_ctlResultsList.SetCellEnabled(nRow,count,FALSE);
							}
						}
						for (int count2= 11 ; count2 <14; count2++)
						{
							if(fInput.Read(&lTemp,sizeof(long)))
							{
								strTemp=ConvertToSeconds(lTemp);
								m_ctlResultsList.SetItemText(nRow,count2,strTemp);
								m_ctlResultsList.SetCellEnabled(nRow,count2,FALSE);
							}
						}

						long numberOfTaskSnaps=0;
						if(fInput.Read(&lTemp,sizeof(long)))
						{
							numberOfTaskSnaps=lTemp;

							for(long b = 0; b < numberOfTaskSnaps; b++)
							{
								long lTaskId=-1;
								//task id - 0 based
								if(fInput.Read(&lTemp,sizeof(long)))
								{
									lTaskId=lTemp;
								}
								//check if enough columns for this task
								if(lTaskId>m_nMaxUsedTaskCol)
								{
									for (int counter= m_nMaxUsedTaskCol ; counter<lTaskId ; counter++)
									{
										//need to add columns
										CString header;
										header.Format("Task %d (VU's)",lTaskId+1);
										m_ctlResultsList.InsertColumn(header,DT_LEFT|DT_VCENTER|DT_SINGLELINE);
										header.Format("Task %d (Iterations)",lTaskId+1);
										m_ctlResultsList.InsertColumn(header,DT_LEFT|DT_VCENTER|DT_SINGLELINE);
										header.Format("Task %d (Period)",lTaskId+1);
										m_ctlResultsList.InsertColumn(header,DT_LEFT|DT_VCENTER|DT_SINGLELINE);
									}
									m_nMaxUsedTaskCol=lTaskId;
								}
								//vu count
								if(fInput.Read(&lTemp,sizeof(long)))
								{
									strTemp.Format("%d",lTemp);
									m_ctlResultsList.SetItemText(nRow,14+(lTaskId*3),strTemp);
									m_ctlResultsList.SetCellEnabled(nRow,14+(lTaskId*3),FALSE);
								}
								//completed
								if(fInput.Read(&lTemp,sizeof(long)))
								{
									strTemp.Format("%d",lTemp);
									m_ctlResultsList.SetItemText(nRow,14+(lTaskId*3)+1,strTemp);
									m_ctlResultsList.SetCellEnabled(nRow,14+(lTaskId*3)+1,FALSE);
								}
								//period
								if(fInput.Read(&lTemp,sizeof(long)))
								{
									strTemp = ConvertToTimeMilliseconds(lTemp);
									m_ctlResultsList.SetItemText(nRow,14+(lTaskId*3)+2,strTemp);
									m_ctlResultsList.SetCellEnabled(nRow,14+(lTaskId*3)+2,FALSE);
								}
							}
							m_ctlResultsList.InsertRow("");
							nRow++;
						}
					}
				}
				
				if(fInput.Read(buffer,2))
				{
					//this should be "|Y" terminator
				}	
			}
		
		}
//
// Version 1
//
		else if (nVersion == 1)
		{
			m_ctlResultsList.InsertColumn("TimeStamp",DT_LEFT|DT_VCENTER|DT_SINGLELINE);
			m_ctlResultsList.InsertColumn("Executer Name",DT_LEFT|DT_VCENTER|DT_SINGLELINE);
			m_ctlResultsList.InsertColumn("Avg Connection Time",DT_LEFT|DT_VCENTER|DT_SINGLELINE);
			m_ctlResultsList.InsertColumn("Task Group Id",DT_LEFT|DT_VCENTER|DT_SINGLELINE);
			m_ctlResultsList.InsertColumn("Completed iterations",DT_LEFT|DT_VCENTER|DT_SINGLELINE);
			m_ctlResultsList.InsertColumn("Run Time",DT_LEFT|DT_VCENTER|DT_SINGLELINE);
			m_ctlResultsList.InsertColumn("Total Users",DT_LEFT|DT_VCENTER|DT_SINGLELINE);
			m_ctlResultsList.InsertColumn("Finished Users",DT_LEFT|DT_VCENTER|DT_SINGLELINE);
			m_ctlResultsList.InsertColumn("Aborted Users",DT_LEFT|DT_VCENTER|DT_SINGLELINE);
			m_ctlResultsList.InsertColumn("HTTP Requests",DT_LEFT|DT_VCENTER|DT_SINGLELINE);
			m_ctlResultsList.InsertColumn("Failed Requests",DT_LEFT|DT_VCENTER|DT_SINGLELINE);
			m_ctlResultsList.InsertColumn("HTTP 3xx Requests",DT_LEFT|DT_VCENTER|DT_SINGLELINE);
			m_ctlResultsList.InsertColumn("HTTP 4xx Requests",DT_LEFT|DT_VCENTER|DT_SINGLELINE);
			m_ctlResultsList.InsertColumn("HTTP 5xx Requests",DT_LEFT|DT_VCENTER|DT_SINGLELINE);
			m_ctlResultsList.InsertColumn("Bytes In",DT_LEFT|DT_VCENTER|DT_SINGLELINE);
			m_ctlResultsList.InsertColumn("Bytes Out",DT_LEFT|DT_VCENTER|DT_SINGLELINE);
			m_ctlResultsList.InsertColumn("Min Request Latency",DT_LEFT|DT_VCENTER|DT_SINGLELINE);
			m_ctlResultsList.InsertColumn("Max Request Latency",DT_LEFT|DT_VCENTER|DT_SINGLELINE);
			m_ctlResultsList.InsertColumn("Average Request Latency",DT_LEFT|DT_VCENTER|DT_SINGLELINE);

			m_ctlResultsList.InsertColumn("Task 1 (VU's)",DT_LEFT|DT_VCENTER|DT_SINGLELINE);
			m_ctlResultsList.InsertColumn("Task 1 (Iterations)",DT_LEFT|DT_VCENTER|DT_SINGLELINE);
			m_ctlResultsList.InsertColumn("Task 1 (Period)",DT_LEFT|DT_VCENTER|DT_SINGLELINE);
			m_ctlResultsList.InsertRow("");
			while(1)
			{
				
				//time
				if(fInput.Read(&lTemp,sizeof(long)))
				{
					CTime time(lTemp);
					strTemp = (time.Format("%d/%m/%y %H:%M:%S"));
					m_ctlResultsList.SetItemText(nRow,0,strTemp);
					m_ctlResultsList.SetCellEnabled(nRow,0,FALSE);
				}
				else
				{
					break;
				}

				//executer name
				if(fInput.Read(&lTemp,sizeof(long)))
				{
				
					if(lTemp>buffSize)
					{
						lTemp=buffSize;
					}
					if(fInput.Read(buffer,lTemp))
					{
						strTemp.Format("%s",buffer);
						m_ctlResultsList.SetItemText(nRow,1,strTemp);
						m_ctlResultsList.SetCellEnabled(nRow,1,FALSE);
					}
				}
				
				//avg connection time
				if(fInput.Read(&lTemp,sizeof(long)))
				{
					strTemp = ConvertToTimeMilliseconds(lTemp);
					m_ctlResultsList.SetItemText(nRow,2,strTemp);
					m_ctlResultsList.SetCellEnabled(nRow,2,FALSE);
				}

				if(fInput.Read(&lTemp,sizeof(long)))
				{
					long numberOfTaskGroupSnaps=lTemp;
					for (long a=0; a<numberOfTaskGroupSnaps; a++)
					{
						//task group id +1
						if(fInput.Read(&lTemp,sizeof(long)))
						{
								strTemp.Format("%d",lTemp+1);
								m_ctlResultsList.SetItemText(nRow,3,strTemp);
								m_ctlResultsList.SetCellEnabled(nRow,3,FALSE);
						}

						//completed iterations
						if(fInput.Read(&lTemp,sizeof(long)))
						{
							strTemp.Format("%d",lTemp);
							m_ctlResultsList.SetItemText(nRow,4,strTemp);
							m_ctlResultsList.SetCellEnabled(nRow,4,FALSE);
						}
							//run time
						if(fInput.Read(&lTemp,sizeof(long)))
						{
							strTemp = ConvertToTime(lTemp);
							m_ctlResultsList.SetItemText(nRow,5,strTemp);
							m_ctlResultsList.SetCellEnabled(nRow,5,FALSE);
						}
						// Total Users
						if(fInput.Read(&lTemp,sizeof(long)))
						{
							strTemp.Format("%d",lTemp);
							m_ctlResultsList.SetItemText(nRow,6,strTemp);
							m_ctlResultsList.SetCellEnabled(nRow,6,FALSE);
						}
						// Finished Users and Abort Users
						long lFinishedUsers = 0 ;
						long lAbortUsers = 0 ;
						if(fInput.Read(&lFinishedUsers,sizeof(long)))
						{
							if(fInput.Read(&lAbortUsers,sizeof(long)))
							{
								strTemp.Format("%d",lFinishedUsers - lAbortUsers);
								m_ctlResultsList.SetItemText(nRow,7,strTemp);
								m_ctlResultsList.SetCellEnabled(nRow,7,FALSE);
								strTemp.Format("%d",lAbortUsers);
								m_ctlResultsList.SetItemText(nRow,8,strTemp);
								m_ctlResultsList.SetCellEnabled(nRow,8,FALSE);
							}
						}

						for (int count= 9 ; count <14 ; count++)
						{
							if(fInput.Read(&lTemp,sizeof(long)))
							{
								strTemp.Format("%d",lTemp);
								m_ctlResultsList.SetItemText(nRow,count,strTemp);
								m_ctlResultsList.SetCellEnabled(nRow,count,FALSE);
							}
						}
						double dTemp ;
						for (count= 14 ; count <16 ; count++)
						{
							if(fInput.Read(&dTemp,sizeof(double)))
							{
								strTemp.Format("%f",dTemp);
								m_ctlResultsList.SetItemText(nRow,count,strTemp);
								m_ctlResultsList.SetCellEnabled(nRow,count,FALSE);
							}
						}
						for (int count2= 16 ; count2 <19; count2++)
						{
							if(fInput.Read(&lTemp,sizeof(long)))
							{
								strTemp=ConvertToSeconds(lTemp);
								m_ctlResultsList.SetItemText(nRow,count2,strTemp);
								m_ctlResultsList.SetCellEnabled(nRow,count2,FALSE);
							}
						}

						long numberOfTaskSnaps=0;
						if(fInput.Read(&lTemp,sizeof(long)))
						{
							numberOfTaskSnaps=lTemp;

							for(long b = 0; b < numberOfTaskSnaps; b++)
							{
								long lTaskId=-1;
								//task id - 0 based
								if(fInput.Read(&lTemp,sizeof(long)))
								{
									lTaskId=lTemp;
								}
								//check if enough columns for this task
								if(lTaskId>m_nMaxUsedTaskCol)
								{
									for (int counter= m_nMaxUsedTaskCol ; counter<lTaskId ; counter++)
									{
										//need to add columns
										CString header;
										header.Format("Task %d (VU's)",lTaskId+1);
										m_ctlResultsList.InsertColumn(header,DT_LEFT|DT_VCENTER|DT_SINGLELINE);
										header.Format("Task %d (Iterations)",lTaskId+1);
										m_ctlResultsList.InsertColumn(header,DT_LEFT|DT_VCENTER|DT_SINGLELINE);
										header.Format("Task %d (Period)",lTaskId+1);
										m_ctlResultsList.InsertColumn(header,DT_LEFT|DT_VCENTER|DT_SINGLELINE);
									}
									m_nMaxUsedTaskCol=lTaskId;
								}
								//vu count
								if(fInput.Read(&lTemp,sizeof(long)))
								{
									strTemp.Format("%d",lTemp);
									m_ctlResultsList.SetItemText(nRow,19+(lTaskId*3),strTemp);
									m_ctlResultsList.SetCellEnabled(nRow,19+(lTaskId*3),FALSE);
								}
								//completed
								if(fInput.Read(&lTemp,sizeof(long)))
								{
									strTemp.Format("%d",lTemp);
									m_ctlResultsList.SetItemText(nRow,19+(lTaskId*3)+1,strTemp);
									m_ctlResultsList.SetCellEnabled(nRow,19+(lTaskId*3)+1,FALSE);
								}
								//period
								if(fInput.Read(&lTemp,sizeof(long)))
								{
									strTemp = ConvertToTimeMilliseconds(lTemp);
									m_ctlResultsList.SetItemText(nRow,19+(lTaskId*3)+2,strTemp);
									m_ctlResultsList.SetCellEnabled(nRow,19+(lTaskId*3)+2,FALSE);
								}
							}
							m_ctlResultsList.InsertRow("");
							nRow++;
						}
					}
				}
				
				if(fInput.Read(buffer,2))
				{
					//this should be "|Y" terminator
				}	
			}
		}
		else if (nVersion > 1)
		{
			//not supported yet
		}
		
		m_ctlResultsList.AutoSizeColumns();
			
	}
	catch(...)
	{
	}

	delete []buffer;

	//delete last row as it will be empty
	if(m_ctlResultsList.GetRowCount()>m_ctlResultsList.GetFixedRowCount())
	{
		//there are same data columns so delete the last one
		m_ctlResultsList.DeleteRow(m_ctlResultsList.GetRowCount()-1);
	}

    return true;
}

BOOL CResultsDlg::CreateSingleHTTPDataFile()
{
	//change this so that creating a combined file is only done once
	const BOOL alwaysCreateNew=FALSE;

	CFileFind FF1;
	if((FF1.FindFile(m_strRunFolder + "\\Combined_HTTPDataFile.dat"))&&(alwaysCreateNew==FALSE))
	{
		//it's already been combined
		return true;
	}

	//Concatenate all the files in this test run directory
	CFileFind FF;
	if(!FF.FindFile(m_strRunFolder + "\\HTTPDataFile_*.dat"))
	{
		//no files to concatenate
		return false;
	}

	
	BOOL bContinue=true;
	CStringArray filesToMerge;
	filesToMerge.RemoveAll();
	int nTemp=0;
	CString strOutputFileName;
	try
	{
		while(bContinue)
		{
			bContinue = FF.FindNextFile();
			CString strInputFile = FF.GetFilePath();
			filesToMerge.Add(strInputFile);
		}

		while(filesToMerge.GetSize()>0)
		{
			if(filesToMerge.GetSize()==1)
			{
				MergeFiles(filesToMerge[0],"",m_strRunFolder + "\\Combined_HTTPDataFile.dat");
				//delete the temporary files
				remove(m_strRunFolder + "\\tempA.tmp");
				remove(m_strRunFolder + "\\tempB.tmp");
				return true;
			}
			else if(filesToMerge.GetSize()==2)
			{
				MergeFiles(filesToMerge[0],filesToMerge[1],m_strRunFolder + "\\Combined_HTTPDataFile.dat");
				//delete the temporary files
				remove(m_strRunFolder + "\\tempA.tmp");
				remove(m_strRunFolder + "\\tempB.tmp");
				return true;
			}
			else
			{
				//number of files is >2 so merge these back into the list

				if(nTemp==0)
				{
					strOutputFileName=m_strRunFolder + "\\tempA.tmp";
					nTemp=1;
				}
				else if(nTemp==1)
				{
						strOutputFileName=m_strRunFolder + "\\tempB.tmp";
					nTemp=0;
				}

				// merge the last two files in the list
				MergeFiles(filesToMerge[filesToMerge.GetSize()-1],filesToMerge[filesToMerge.GetSize()-2],strOutputFileName);

				// remove those from the list
				// and then insert the temp file 
				filesToMerge.RemoveAt(filesToMerge.GetSize()-1);
				filesToMerge.RemoveAt(filesToMerge.GetSize()-1);
				filesToMerge.Add(strOutputFileName);
			}
		}
	}
	catch(...)
	{

	}

	//delete all temp files
	remove(m_strRunFolder + "\\tempA.tmp");
	remove(m_strRunFolder + "\\tempB.tmp");

	return true;
}

void CResultsDlg::SetCurrentType(AuditType nType)
{
	m_nCurrentType = nType;
	if(GetSafeHwnd())
		ShowType(m_nCurrentType);
}

void CResultsDlg::OnContextMenu(CWnd* pWnd, CPoint point) 
{
	    //This block was added by the Pop-up Menu component
    if (point.x == -1 && point.y == -1)
    {
        //keystroke invocation
        CRect rect;
        GetClientRect(rect);
        ClientToScreen(rect);

        point = rect.TopLeft();
        point.Offset(5, 5);
    }

    CMenu menu;
    VERIFY(menu.LoadMenu(IDR_CONTEXT_MENU));

    CMenu* pPopup = menu.GetSubMenu(0);
    ASSERT(pPopup != NULL);

    pPopup->TrackPopupMenu(TPM_LEFTALIGN | TPM_RIGHTBUTTON, point.x, point.y,this);
}

void CResultsDlg::OnExport() 
{
    CFileDialog dlg(FALSE,"txt","Audit.csv",OFN_HIDEREADONLY | OFN_OVERWRITEPROMPT,"Comma Separated Values (*.csv)|*.csv||",this);
    if(dlg.DoModal() != IDOK)
        return;

    CString strPath(dlg.GetPathName());

    CStdioFile f;
    if(!f.Open(strPath,CFile::modeWrite|CFile::modeCreate))
    {
        CString strMsg;
        strMsg.Format("Failed to open the export file \"%s\"",strPath);
        MessageBox(strMsg,"Audit",MB_ICONEXCLAMATION|MB_OK);
        return;
    }

    CString strLine;
    int nNumCols = m_ctlResultsList.GetColumnCount();
	int nNumRows = m_ctlResultsList.GetRowCount();
	for(int nRow=0;nRow<nNumRows;nRow++)
	{
		strLine.Empty();

		for(int nCol=0;nCol<nNumCols;nCol++)
		{
			strLine += m_ctlResultsList.GetItemText(nRow,nCol);
			if(nCol!=nNumCols-1)
				strLine += ",";
		}        
        strLine += "\n";
	    f.WriteString(strLine);
    }
    f.Close();
    MessageBox("Export complete","Audit",MB_ICONINFORMATION|MB_OK);
}

CString CResultsDlg::ConvertToTime(long seconds)
{
	CString answer;
	int remainder;
	div_t temp;
	int h=0;
	int m=0;
	int s=0;
	int days=0;

	temp=div(seconds,3600*24);
	days=temp.quot;
	remainder=temp.rem;

	temp=div(remainder,3600);
	h=temp.quot;
	remainder=temp.rem;

	temp=div(remainder,60);
	m=temp.quot;
	s=temp.rem;


	if(days>0)
	{
		answer.Format("%d days %02d:%02d:%02d",days,h,m,s);
	}
	else
	{
		answer.Format("%02d:%02d:%02d",h,m,s);
	}
	return answer;
}

CString CResultsDlg::ConvertToTimeMilliseconds(long milliseconds)
{
	CString answer;
	int remainder;
	div_t temp;
	int h=0;
	int m=0;
	int s=0;
	int ms=0;
	int days=0;
	

	temp=div(milliseconds,3600*24*1000);
	days=temp.quot;
	remainder=temp.rem;

	temp=div(remainder,3600*1000);
	h=temp.quot;
	remainder=temp.rem;

	temp=div(remainder,60*1000);
	m=temp.quot;
	remainder=temp.rem;

	temp=div(remainder,1000);
	s=temp.quot;
	ms=temp.rem;


	if(days>0)
	{
		answer.Format("%d days %02d:%02d:%02d.%03d",days,h,m,s,ms);
	}
	else
	{
		answer.Format("%02d:%02d:%02d.%03d",h,m,s,ms);
	}
	return answer;
}


CString CResultsDlg::ConvertToSeconds(long milliseconds)
{
	CString answer;
	div_t temp;

	int s=0;
	int ms=0;
	temp=div(milliseconds,1000);
	s=temp.quot;
	ms=temp.rem;

	answer.Format("%d.%03d",s,ms);

	return answer;
}


BOOL CResultsDlg::MergeFiles(CString input1, CString input2, CString output)
{
	//check both the inputs exist
	CHTTPData fInput1;
	CHTTPData fInput2;
	BOOL bOpened1=FALSE;
	BOOL bOpened2=FALSE;

	CFileFind ff;
	if((!input1.IsEmpty())&&(ff.FindFile(input1)))
	{
		if(fInput1.LoadFile(input1))
		{
			bOpened1=TRUE;
		}

	}

	if((!input2.IsEmpty())&&(ff.FindFile(input2)))
	{
		if(fInput2.LoadFile(input2))
		{
			bOpened2=TRUE;
		}
	}

	//create a new output
	//ensure creates new every time
	CFile fOutput(output,CFile::modeCreate|CFile::modeWrite|CFile::typeBinary);
	//if only one opened just copy it to output
	if(bOpened1==FALSE &&bOpened2==TRUE)
	{
		HTTPSnapFile answer;

		int version=fInput2.GetFileVersion();

		if(version>=0)
		{
			fOutput.Write(&version,sizeof(int));
			while(fInput2.FindNextSnapShot()>=0)
			{
				answer=fInput2.GetCurrentSnapShot();
				fOutput.Write(answer.strAnswer,answer.length);
			}
			fInput2.CloseFile();
			fOutput.Close();
			return true;
		}
		fInput2.CloseFile();
		fOutput.Close();
		return false;
	}

	if(bOpened1==TRUE && bOpened2==FALSE)
	{
		HTTPSnapFile answer;

		int version=fInput1.GetFileVersion();

		if(version>=0)
		{
			fOutput.Write(&version,sizeof(int));
			while(fInput1.FindNextSnapShot()>=0)
			{
				answer=fInput1.GetCurrentSnapShot();
				fOutput.Write(answer.strAnswer,answer.length);
			}
			fInput1.CloseFile();
			fOutput.Close();
			return true;
		}
		fInput1.CloseFile();
		fOutput.Close();
		return false;
	}

	//read in to two buffers, compare time

	HTTPSnapFile answer1;
	HTTPSnapFile answer2;

	int version1=fInput1.GetFileVersion();
	int version2=fInput2.GetFileVersion();

	if((version1>=0)&&(version1>=0))
	{
		if(version1!=version2)
		{
			//cant merge two different versions
			fInput1.CloseFile();
			fInput2.CloseFile();
			fOutput.Close();
			return false;
		}
		
		fOutput.Write(&version1,sizeof(int));

		//get both answers and decide which one to output based on the time


		long one=fInput1.FindNextSnapShot();
		long two=fInput2.FindNextSnapShot();

		BOOL bFinished;
		if((one>=0) || (two>0))
		{
			bFinished=FALSE;
		}
		else
		{
			bFinished=TRUE;
		}

		while(!bFinished)
		{
			answer1=fInput1.GetCurrentSnapShot();
			answer2=fInput2.GetCurrentSnapShot();

			if((answer1.length<0) && (answer2.length<0))
			{
				//both files are finished
				bFinished=TRUE;
			}
			else if((answer1.length)<0)
			{
				//from now on all from file 2
				fOutput.Write(answer2.strAnswer,answer2.length);
				while(fInput2.FindNextSnapShot()>=0)
				{
					answer2=fInput2.GetCurrentSnapShot();
					fOutput.Write(answer2.strAnswer,answer2.length);
				}
				bFinished=TRUE;
			}
			else if((answer2.length)<0)
			{
				//from now on all from file 1
				fOutput.Write(answer1.strAnswer,answer1.length);
	
				while(fInput1.FindNextSnapShot()>=0)
				{
					answer1=fInput1.GetCurrentSnapShot();
					fOutput.Write(answer1.strAnswer,answer1.length);
				}
			}
			else
			{	//comparing times of two snapshots
				if(fInput1.TimeOfCurrentSnapshot()<fInput2.TimeOfCurrentSnapshot())
				{
					fOutput.Write(answer1.strAnswer,answer1.length);
					fInput1.FindNextSnapShot();
				}
				else
				{
					fOutput.Write(answer2.strAnswer,answer2.length);
					fInput2.FindNextSnapShot();
				}
			}
		
		}
		fOutput.Close();
		fInput1.CloseFile();
		fInput2.CloseFile();
		return true;
	}

	return false;
}

