/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

// ChartDlg.cpp : implementation file
//

#include "stdafx.h"
#include "XChartCtrl.h"
#include "ChartDlg.h"
#include "comdef.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CChartDlg dialog
#define MIN_DOUBLE (double)(-1.7E+304)

CChartDlg::CChartDlg(CWnd* pParent /*=NULL*/)
        : CDialog(CChartDlg::IDD, pParent),
        covOptional((long)DISP_E_PARAMNOTFOUND, VT_ERROR)
{
    m_nStep = -1;
    m_nCurrentGroup = 0;
    m_colBackground = RGB(255,255,255);
    m_nZoomStart=0;
    m_nZoomEnd=-1;
    m_nDragStart = -1;
    m_bZoomEnabled = true;
	m_nMaxRollingPoints = -1;
    //{{AFX_DATA_INIT(CChartDlg)
    // NOTE: the ClassWizard will add member initialization here
    //}}AFX_DATA_INIT
}


CChartDlg::~CChartDlg()
{
    for(int nGroup=0;nGroup<m_arGroupData.GetSize();nGroup++)
    {
        m_arGroupData[nGroup]->RemoveAll();
        delete m_arGroupData[nGroup];
    }
    m_arGroupData.RemoveAll();
}

void CChartDlg::DoDataExchange(CDataExchange* pDX)
{
    CDialog::DoDataExchange(pDX);
    //{{AFX_DATA_MAP(CChartDlg)
    DDX_Control(pDX, IDC_MSCHART, m_ctlChart);
    //}}AFX_DATA_MAP
}



BEGIN_MESSAGE_MAP(CChartDlg, CDialog)
//{{AFX_MSG_MAP(CChartDlg)
ON_WM_ERASEBKGND()
ON_WM_LBUTTONDOWN()
ON_WM_SIZE()
ON_WM_MOUSEMOVE()
ON_WM_LBUTTONUP()
//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CChartDlg message handlers

void CChartDlg::OnSize(UINT nType, int cx, int cy)
{
    CDialog::OnSize(nType, cx, cy);

    if(m_ctlChart.GetSafeHwnd())
	{
		m_ctlChart.MoveWindow(0,0,cx,cy);
		LayoutChart();
		
		LayoutGrid();
//		m_ctlChart.Refresh();
	}
}

BOOL CChartDlg::OnInitDialog()
{
    CDialog::OnInitDialog();

    //  m_ctlChart.SetEnabled(true);
    m_ctlChart.SetChartType(VtChChartType2dCombination);
    m_ctlChart.SetDrawMode(VtChDrawModeBlit);

    LayoutChart();

    return TRUE;  // return TRUE unless you set the focus to a control
    // EXCEPTION: OCX Property Pages should return FALSE
}


void CChartDlg::SetTitleText(const CString& strTitle)
{
    CVcTitle &Title = m_ctlChart.GetTitle();
    Title.SetText(strTitle);
    Title.GetVtFont().SetSize(16);

    LayoutChart();
}

void CChartDlg::SetTitleBkColor(COLORREF colTitleBk)
{
    CVcFill &fill = m_ctlChart.GetTitle().GetBackdrop().GetFill();
    fill.SetStyle(VtFillStyleBrush);
    fill.GetBrush().GetFillColor().Set(GetRValue(colTitleBk),GetGValue(colTitleBk),GetBValue(colTitleBk));
}

void CChartDlg::SetPlotBkColor(COLORREF colPlotBk)
{
    m_colBackground = colPlotBk;
    CVcFill &fill = m_ctlChart.GetPlot().GetBackdrop().GetFill();
    fill.SetStyle(VtFillStyleBrush);
    fill.GetBrush().GetFillColor().Set(GetRValue(colPlotBk),GetGValue(colPlotBk),GetBValue(colPlotBk));
}

void CChartDlg::SetSeriesStyle(int nSeries,VtChSeriesType nStyle)
{
    CVcSeriesCollection &collection = m_ctlChart.GetPlot().GetSeriesCollection();

    int nCount = collection.Count();
    if(nSeries>nCount || nSeries<=0)
        return;

    CVcSeries series = collection.GetItem(nSeries);
    long nOldStyle = series.GetSeriesType();
    if(nOldStyle == nStyle)
        return;

    series.SetSeriesType(nStyle);
    CVcSeriesPosition pos = series.GetPosition();
    pos.SetExcluded(FALSE);
    pos.SetHidden(FALSE);
    series.SetShowLine(TRUE);
    series.GetPen().SetWidth(2);

    if((nOldStyle==VtChChartType2dBar) || (nStyle==VtChChartType2dBar))
        UpdateChartData();
}

VtChSeriesType CChartDlg::GetSeriesStyle(int nSeries)
{
    CVcSeriesCollection &collection = m_ctlChart.GetPlot().GetSeriesCollection();

    int nCount = collection.Count();
    if(nCount<nSeries || nSeries<=0)
        return (VtChSeriesType)-1;

    return (VtChSeriesType)collection.GetItem(nSeries).GetSeriesType();
}

void CChartDlg::SetChartData(const VARIANT& varNewData)
{
    m_nCurrentGroup = m_arGroupData.GetSize();

    CVcDataGrid &dataGrid = m_ctlChart.GetDataGrid();

    COleSafeArray arNewData(varNewData);

    long lNumPoints,lNumGroups;
    arNewData.GetUBound(1,&lNumPoints);
    arNewData.GetUBound(2,&lNumGroups);
    lNumPoints++;
    lNumGroups++;

    if(lNumPoints<=0)
    {
        dataGrid.SetRowCount(0);
        dataGrid.SetColumnCount(0);
        dataGrid.SetColumnLabelCount(0);
        dataGrid.SetRowLabelCount(0);

        for(int nGroup=0;nGroup<m_arGroupData.GetSize();nGroup++)
        {
            m_arGroupData[nGroup]->RemoveAll();
            delete m_arGroupData[nGroup];
        }
        m_arGroupData.RemoveAll();
        return;
    }

    VARIANT *pVarNewData;
    arNewData.AccessData((void**)&pVarNewData);

    if(m_arAnnotation.GetSize() < lNumPoints)
        m_arAnnotation.SetSize(lNumPoints);

    int nFirstGroup=0;
    //Check if labels are included in data
    if(pVarNewData[0].vt == VT_BSTR)
    {
        for(int nNdx=0;nNdx<lNumPoints;nNdx++)
            if(pVarNewData[nNdx].vt == VT_BSTR)
            {
                char *psz = _com_util::ConvertBSTRToString(pVarNewData[nNdx].bstrVal);
                m_arAnnotation[nNdx] = psz;
                free(psz);
            }

        nFirstGroup=1;
    }

    while(m_arGroupData.GetSize() < lNumGroups-nFirstGroup+m_nCurrentGroup)
        m_arGroupData.Add(new DataList);

    for(int nGroup=nFirstGroup;nGroup<lNumGroups;nGroup++)
    {
        int nIndex = (nGroup*lNumPoints);

        //If there are more than nMaxPoints points cull the excess data
        for(int nPoint=0;nPoint<lNumPoints;nPoint++,nIndex++)
        {
            DataPoint data;
            if(pVarNewData[nIndex].vt == VT_EMPTY)
                data.bNull = true;
            else
            {
                data.bNull = false;
                data.dValue = pVarNewData[nIndex].dblVal;
            }

            m_arGroupData[nGroup-nFirstGroup+m_nCurrentGroup]->AddTail(data);
        }
    }

    arNewData.UnaccessData();

	if(m_nMaxRollingPoints != -1)
		SetRollingGraph(m_nMaxRollingPoints);

    UpdateChartData();
}

void CChartDlg::UpdateChartData()
{
    CVcDataGrid &dataGrid = m_ctlChart.GetDataGrid();
    dataGrid.SetRowCount(0);

    int nNumGroups = m_arGroupData.GetSize();
    int nNumPoints = 0;
    for(int nGroup=0;nGroup<nNumGroups;nGroup++)
    {
        if(m_arGroupData[nGroup]->GetCount() > nNumPoints)
            nNumPoints = m_arGroupData[nGroup]->GetCount();
    }

    dataGrid.SetRowLabelCount(1);
    m_ctlChart.SetColumnCount(nNumGroups);

    const int nMaxPoints = 800;
    if(m_nZoomEnd==-1)
    {
        if(nNumPoints-m_nZoomStart > nMaxPoints)
            m_ctlChart.SetRowCount((short)nMaxPoints);
        else
            m_ctlChart.SetRowCount((short)nNumPoints-m_nZoomStart);
		nNumPoints = nNumPoints-m_nZoomStart;
    }
    else
    {
		nNumPoints = m_nZoomEnd-m_nZoomStart;
        if((m_nZoomEnd-m_nZoomStart) > nMaxPoints)
            m_ctlChart.SetRowCount((short)nMaxPoints);
        else
		{
            m_ctlChart.SetRowCount((short)nNumPoints);
		}
    }

    for(nGroup=0;nGroup<nNumGroups;nGroup++)
    {
        bool bAverageNulls = (GetSeriesStyle(nGroup+1)==VtChChartType2dBar ? false : true);

        int nNumMissing=0;
        int nLastPoint=0;
        double dPrevValue=0;
        if(nNumPoints > nMaxPoints)
        {
            double dStep=(double)nNumPoints/nMaxPoints;

            int nIndex=0;
            int nPoint=1;
            double dPosition=dStep;
            double dMaxVal=MIN_DOUBLE;
            DataList *pDataList = m_arGroupData[nGroup];

            POSITION pos = pDataList->FindIndex(m_nZoomStart);
            POSITION posEnd = NULL;
            if(m_nZoomEnd != -1)
                posEnd = pDataList->FindIndex(m_nZoomEnd);

            while(pos != posEnd)
            {
                DataPoint& Data = pDataList->GetNext(pos);

                if(nIndex++>=dPosition)
                {
                    dataGrid.SetRowLabel(nPoint,1,m_arAnnotation[nIndex+m_nZoomStart-1]);
                    if(bAverageNulls)
                    {
                        if(dMaxVal==MIN_DOUBLE)
                            nNumMissing++;
                        else
                        {
                            if(nNumMissing)
                            {
                                if(nLastPoint)
                                {
                                    double dAverage = (dMaxVal-dPrevValue)/(nNumMissing+1);
                                    for(int n=nLastPoint+1;n<nPoint;n++)
                                    {
                                        dPrevValue += dAverage;
                                        dataGrid.SetData(n,nGroup+1,dPrevValue,false);
                                    }
                                }
                                nNumMissing = 0;
                            }
                            dataGrid.SetData(nPoint,nGroup+1,dMaxVal,false);

                            nLastPoint = nPoint;
                            dPrevValue = dMaxVal;
                        }
                        nPoint++;
                    }
                    else
                        dataGrid.SetData(nPoint++,nGroup+1,dMaxVal,dMaxVal==MIN_DOUBLE);
                    dMaxVal=MIN_DOUBLE;
                    dPosition += dStep;
                }

                if(!Data.bNull && (dMaxVal < Data.dValue))
                    dMaxVal = Data.dValue;
            }
        }
        else
        {
            int nPoint=1;
            DataList *pDataList = m_arGroupData[nGroup];

            POSITION pos = pDataList->FindIndex(m_nZoomStart);
            POSITION posEnd = NULL;
            if(m_nZoomEnd != -1)
                posEnd = pDataList->FindIndex(m_nZoomEnd);

            while(pos != posEnd)
            {
                DataPoint& Data = pDataList->GetNext(pos);

                int nAnnotationNdx = nPoint+m_nZoomStart-1;
                if(m_arAnnotation.GetSize()>nAnnotationNdx)
                    dataGrid.SetRowLabel(nPoint,1,m_arAnnotation[nAnnotationNdx]);

                if(bAverageNulls)
                {
                    if(Data.bNull)
                        nNumMissing++;
                    else
                    {
                        if(nNumMissing)
                        {
                            if(nLastPoint)
                            {
                                double dAverage = (Data.dValue-dPrevValue)/(nNumMissing+1);
                                for(int n=nLastPoint+1;n<nPoint;n++)
                                {
                                    dPrevValue += dAverage;
                                    dataGrid.SetData(n,nGroup+1,dPrevValue,false);
                                }
                            }
                            nNumMissing = 0;
                        }
                        dataGrid.SetData(nPoint,nGroup+1,Data.dValue,false);

                        nLastPoint = nPoint;
                        dPrevValue = Data.dValue;
                    }
                    nPoint++;
                }
                else
                    dataGrid.SetData(nPoint++,nGroup+1,Data.dValue,Data.bNull);
            }
        }
    }

	LayoutGrid();
}

void CChartDlg::SetAxisLabel(VtChAxisId nAxis,const CString& strText)
{
    m_ctlChart.GetPlot().GetAxis(nAxis,covOptional).GetAxisTitle().SetText(strText);
}

void CChartDlg::SetAxisMinimum(VtChAxisId nAxis,int nMinimum)
{
    CVcValueScale &valScale = m_ctlChart.GetPlot().GetAxis(nAxis,covOptional).GetValueScale();
    valScale.SetAuto(FALSE);
    valScale.SetMinimum(nMinimum);
}

void CChartDlg::SetAxisMaximum(VtChAxisId nAxis,int nMaximum)
{
    CVcValueScale &valScale = m_ctlChart.GetPlot().GetAxis(nAxis,covOptional).GetValueScale();
    valScale.SetAuto(FALSE);
    valScale.SetMaximum(nMaximum);
}

void CChartDlg::GetTitleText(CString& strTitle)
{
    strTitle = m_ctlChart.GetTitle().GetText();
}

COLORREF CChartDlg::GetTitleBkColor()
{
    CVcColor &color = m_ctlChart.GetTitle().GetBackdrop().GetFill().GetBrush().GetFillColor();
    return RGB(color.GetRed(),color.GetGreen(),color.GetBlue());
}

COLORREF CChartDlg::GetPlotBkColor()
{
    CVcColor &color = m_ctlChart.GetPlot().GetBackdrop().GetFill().GetBrush().GetFillColor();
    return RGB(color.GetRed(),color.GetGreen(),color.GetBlue());
}

void CChartDlg::GetAxisLabel(VtChAxisId nAxis,CString& strText)
{
    strText = m_ctlChart.GetPlot().GetAxis(nAxis,covOptional).GetAxisTitle().GetText();
}

int CChartDlg::GetAxisMinimum(VtChAxisId nAxis)
{
    return (int)m_ctlChart.GetPlot().GetAxis(nAxis,covOptional).GetValueScale().GetMinimum();
}

int CChartDlg::GetAxisMaximum(VtChAxisId nAxis)
{
    return (int)m_ctlChart.GetPlot().GetAxis(nAxis,covOptional).GetValueScale().GetMaximum();
}

void CChartDlg::SetTitleTextColor(COLORREF colTitleText)
{
    m_ctlChart.GetTitle().GetVtFont().GetVtColor().Set(GetRValue(colTitleText),GetGValue(colTitleText),GetBValue(colTitleText));
}

void CChartDlg::SetTitleFont(const CString& strFont)
{
    m_ctlChart.GetTitle().GetVtFont().SetName(strFont);
}

COLORREF CChartDlg::GetTitleTextColor()
{
    CVcColor &color = m_ctlChart.GetTitle().GetVtFont().GetVtColor();
    return RGB(color.GetRed(),color.GetGreen(),color.GetBlue());
}

void CChartDlg::GetTitleFont(CString& strFont)
{
    strFont = m_ctlChart.GetTitle().GetVtFont().GetName();
}

void CChartDlg::GetAxisLabelFont(CString& strFont)
{
    strFont = m_ctlChart.GetPlot().GetAxis(VtChAxisIdX,covOptional).GetAxisTitle().GetVtFont().GetName();
}

void CChartDlg::SetAxisLabelFont(const CString& strFont)
{
    CVcPlot &plot = m_ctlChart.GetPlot();
    plot.GetAxis(VtChAxisIdX,covOptional).GetAxisTitle().GetVtFont().SetName(strFont);
    plot.GetAxis(VtChAxisIdY,covOptional).GetAxisTitle().GetVtFont().SetName(strFont);
    plot.GetAxis(VtChAxisIdY2,covOptional).GetAxisTitle().GetVtFont().SetName(strFont);
}

COLORREF CChartDlg::GetGridColor()
{
    CVcColor col = m_ctlChart.GetPlot().GetAxis(VtChAxisIdX,covOptional).GetAxisGrid().GetMajorPen().GetVtColor();
    return RGB(col.GetRed(),col.GetGreen(),col.GetBlue());
}

void CChartDlg::SetGridColor(COLORREF colGrid)
{
    CVcPlot &plot = m_ctlChart.GetPlot();

    short red = GetRValue(colGrid),
                green = GetGValue(colGrid),
                        blue = GetBValue(colGrid);

    plot.GetAxis(VtChAxisIdX,covOptional).GetAxisGrid().GetMajorPen().GetVtColor().Set(red,green,blue);
    plot.GetAxis(VtChAxisIdX,covOptional).GetAxisGrid().GetMinorPen().GetVtColor().Set(red,green,blue);
    plot.GetAxis(VtChAxisIdY,covOptional).GetAxisGrid().GetMajorPen().GetVtColor().Set(red,green,blue);
    plot.GetAxis(VtChAxisIdY,covOptional).GetAxisGrid().GetMinorPen().GetVtColor().Set(red,green,blue);
    plot.GetAxis(VtChAxisIdY2,covOptional).GetAxisGrid().GetMajorPen().GetVtColor().Set(red,green,blue);
    plot.GetAxis(VtChAxisIdY2,covOptional).GetAxisGrid().GetMinorPen().GetVtColor().Set(red,green,blue);
}

COLORREF CChartDlg::GetAxisLabelColor()
{
    CVcColor col = m_ctlChart.GetPlot().GetAxis(VtChAxisIdX,covOptional).GetAxisTitle().GetVtFont().GetVtColor();
    return RGB(col.GetRed(),col.GetGreen(),col.GetBlue());
}

void CChartDlg::SetAxisLabelColor(COLORREF colLabels)
{
    short red = GetRValue(colLabels),
                green = GetGValue(colLabels),
                        blue = GetBValue(colLabels);

    m_ctlChart.GetPlot().GetAxis(VtChAxisIdX,covOptional).GetAxisTitle().GetVtFont().GetVtColor().Set(red,green,blue);
    m_ctlChart.GetPlot().GetAxis(VtChAxisIdY,covOptional).GetAxisTitle().GetVtFont().GetVtColor().Set(red,green,blue);
    m_ctlChart.GetPlot().GetAxis(VtChAxisIdY2,covOptional).GetAxisTitle().GetVtFont().GetVtColor().Set(red,green,blue);
}

COLORREF CChartDlg::GetAxisTextColor()
{
    CVcLabels labels = m_ctlChart.GetPlot().GetAxis(VtChAxisIdX,covOptional).GetLabels();
    if(labels.GetCount())
    {
        CVcColor col = labels.GetItem(1).GetVtFont().GetVtColor();
        return RGB(col.GetRed(),col.GetGreen(),col.GetBlue());
    }
    return RGB(0,0,0);
}

void CChartDlg::SetAxisTextColor(COLORREF colAxisText)
{
    short red = GetRValue(colAxisText),
                green = GetGValue(colAxisText),
                        blue = GetBValue(colAxisText);

    CVcLabels labels = m_ctlChart.GetPlot().GetAxis(VtChAxisIdX,covOptional).GetLabels();
    int nNumLabels = labels.GetCount();
    for(int nLabel=1;nLabel<=nNumLabels;nLabel++)
        labels.GetItem(nLabel).GetVtFont().GetVtColor().Set(red,green,blue);

    labels = m_ctlChart.GetPlot().GetAxis(VtChAxisIdY,covOptional).GetLabels();
    nNumLabels = labels.GetCount();
    for(nLabel=1;nLabel<=nNumLabels;nLabel++)
        labels.GetItem(nLabel).GetVtFont().GetVtColor().Set(red,green,blue);

    labels = m_ctlChart.GetPlot().GetAxis(VtChAxisIdY2,covOptional).GetLabels();
    nNumLabels = labels.GetCount();
    for(nLabel=1;nLabel<=nNumLabels;nLabel++)
        labels.GetItem(nLabel).GetVtFont().GetVtColor().Set(red,green,blue);
}

BOOL CChartDlg::OnEraseBkgnd(CDC *pDC)
{
    return true;
}

void CChartDlg::LayoutChart()
{
    CVcTitle &Title = m_ctlChart.GetTitle();
    BOOL bShowTitle = !Title.GetText().IsEmpty();
    BOOL bShowLegend = m_ctlChart.GetShowLegend();

    CRect rc;
    HWND hWnd = (HWND)m_ctlChart.GetHWnd();
    ::GetWindowRect(hWnd,&rc);

    if(bShowTitle)
    {
        m_ctlChart.GetTitle().GetLocation().SetLocationType(VtChLocationTypeCustom);
        CVcRect vrcTitle =  m_ctlChart.GetTitle().GetLocation().GetRect();
        vrcTitle.GetMin().SetX(0);
        vrcTitle.GetMin().SetY((float)(rc.Height() * 14.0));
        vrcTitle.GetMax().SetX((float)(rc.Width() * 15.0));
        vrcTitle.GetMax().SetY((float)(rc.Height() * 15.0));
    }

    float fLegendHeight = 0;
    if(bShowLegend)
    {
		CVcLegend vcLegend = m_ctlChart.GetLegend();

		CVcLocation vcLegendLocation = vcLegend.GetLocation();
		vcLegendLocation.SetLocationType(VtChLocationTypeCustom);
		CVcRect vrcLegend = vcLegendLocation.GetRect();

		fLegendHeight = (float)rc.Width();

        vcLegendLocation.GetRect().GetMax().SetX((float)(rc.Width() * 15.0));
        vcLegendLocation.GetRect().GetMax().SetY(fLegendHeight);
        vcLegendLocation.GetRect().GetMin().SetX(0);
        vcLegendLocation.GetRect().GetMin().SetY(0);
    }

	m_ctlChart.GetPlot().SetAutoLayout(FALSE);
    CVcRect vrcPlot = m_ctlChart.GetPlot().GetLocationRect();
    vrcPlot.GetMin().SetX(0);
    vrcPlot.GetMin().SetY(fLegendHeight);
    vrcPlot.GetMax().SetX((float)(rc.Width() * 15.0));
	vrcPlot.GetMax().SetY((float)(rc.Height() * (bShowTitle ? 14.0 : 15.0)));
}

void CChartDlg::ShowLegend(BOOL bShow)
{
    m_ctlChart.SetShowLegend(bShow);
    LayoutChart();

    //set legend background to same color as the plot
	CVcLegend vcLegend = m_ctlChart.GetLegend();
    CVcColor &colorPlot = m_ctlChart.GetPlot().GetBackdrop().GetFill().GetBrush().GetFillColor();
    
	vcLegend.GetBackdrop().GetFill().SetStyle(VtFillStyleBrush);
	vcLegend.GetBackdrop().GetFrame().GetFrameColor().Set(0,0,0);
	vcLegend.GetTextLayout().SetVertAlignment(VtVerticalAlignmentCenter);
	vcLegend.GetTextLayout().SetHorzAlignment(VtHorizontalAlignmentFlush);

    CVcColor &colorLegend = vcLegend.GetBackdrop().GetFill().GetBrush().GetFillColor();
    colorLegend.Set(colorPlot.GetRed(),colorPlot.GetGreen(),colorPlot.GetBlue());
}

void CChartDlg::SetLegendText(int nSeries,const CString& strText)
{
    CVcSeriesCollection &collection = m_ctlChart.GetPlot().GetSeriesCollection();

    int nCount = collection.Count();
    if(nSeries>nCount || nSeries<=0)
        return;

    CVcSeries series = collection.GetItem(nSeries);
    series.SetLegendText(strText);

	if(m_arSeriesAnnotation.GetSize() <= nSeries-1)
		m_arSeriesAnnotation.SetAtGrow(nSeries-1,strText);
}

void CChartDlg::GetLegendText(int nSeries,CString& strText)
{
    CVcSeriesCollection &collection = m_ctlChart.GetPlot().GetSeriesCollection();

    int nCount = collection.Count();
    if(nSeries>nCount || nSeries<=0)
        return;

    CVcSeries series = collection.GetItem(nSeries);
    strText = series.GetLegendText();
}

BEGIN_EVENTSINK_MAP(CChartDlg, CDialog)
//{{AFX_EVENTSINK_MAP(CChartDlg)
//}}AFX_EVENTSINK_MAP
END_EVENTSINK_MAP()

void CChartDlg::SetDataPoint(int nIndex,int nSeries,double dValue)
{
    CVcDataGrid &dataGrid = m_ctlChart.GetDataGrid();
    dataGrid.SetData(nIndex,nSeries,dValue,FALSE);

    while(m_arGroupData.GetSize() < nSeries-1)
        m_arGroupData.Add(new DataList);

    DataList *pData = m_arGroupData[nSeries-1];

    if(pData->GetCount() < nIndex)
    {
        DataPoint dataPoint;
        dataPoint.dValue=0;
        dataPoint.bNull=true;
        while(pData->GetCount()<nIndex-1)
        {
            m_arAnnotation.Add("");
            pData->AddTail(dataPoint);
        }

        dataPoint.dValue = dValue;
        dataPoint.bNull=false;
        pData->AddTail(dataPoint);
    }
    else
    {
        POSITION pos = pData->FindIndex(nIndex-1);
        DataPoint &dataPoint = pData->GetAt(pos);
        dataPoint.bNull = false;
        dataPoint.dValue = dValue;
    }
}

double CChartDlg::GetDataPoint(int nIndex,int nSeries)
{
    short null;
    double dValue=0;
    CVcDataGrid &dataGrid = m_ctlChart.GetDataGrid();
    dataGrid.GetData(nIndex,nSeries,&dValue,&null);
    return dValue;
}

void CChartDlg::AddDataPoint(int nSeries,double dValue)
{
    CVcDataGrid &dataGrid = m_ctlChart.GetDataGrid();
    int nNextNdx = dataGrid.GetRowCount()+1;
    int nNumSeries = dataGrid.GetColumnCount();
    if(nNumSeries < nSeries)
        dataGrid.SetColumnCount(nSeries);

	if((m_nMaxRollingPoints > 0) && (nNextNdx > m_nMaxRollingPoints))
	{
		int nNumCols = dataGrid.GetColumnCount();
		for(int nGroup=0;nGroup<m_arGroupData.GetSize();nGroup++)
		{
			DataList *pDataList = m_arGroupData[nGroup];
			while(pDataList->GetCount() > m_nMaxRollingPoints)
				pDataList->RemoveHead();
		}

		nNextNdx = m_nMaxRollingPoints;
		dataGrid.MoveData(1,0,nNextNdx-1,nNumSeries-1,0,-1);
		if(m_arAnnotation.GetSize())
			m_arAnnotation.RemoveAt(0);
	}
	
	dataGrid.SetRowCount(nNextNdx);
	dataGrid.SetData(nNextNdx,nSeries,dValue,false);
    dataGrid.SetRowLabelCount(1);
    dataGrid.SetRowLabel(nNextNdx,nSeries,"");

    while(m_arGroupData.GetSize() <= nSeries-1)
        m_arGroupData.Add(new DataList);

    DataList *pData = m_arGroupData[nSeries-1];

    if(pData->GetCount() < nNextNdx)
    {
        DataPoint dataPoint;
        dataPoint.dValue=0;
        dataPoint.bNull=true;
        while(pData->GetCount()<nNextNdx-1)
        {
            pData->AddTail(dataPoint);
            m_arAnnotation.Add("");
        }

        dataPoint.dValue = dValue;
        dataPoint.bNull=false;
        m_arAnnotation.Add("");
        pData->AddTail(dataPoint);
    }
    else
    {
        POSITION pos = pData->FindIndex(nNextNdx-1);
        DataPoint &dataPoint = pData->GetAt(pos);
        dataPoint.bNull = false;
        dataPoint.dValue = dValue;
    }

	LayoutGrid();
}

void CChartDlg::SetDataColors(const VARIANT& varColors)
{
    CVcDataGrid &dataGrid = m_ctlChart.GetDataGrid();

    COleSafeArray arColors(varColors);

    COLORREF colData;
    long lUBound=0;
    arColors.GetUBound(1,&lUBound);

    int nNumSeries=0;
    CVcSeriesCollection collection;
    try
    {
        collection = m_ctlChart.GetPlot().GetSeriesCollection();
        nNumSeries = collection.Count();
    }
    catch(...)
    {
    }


    for(long lSeries=0;lSeries<=lUBound && lSeries<MAX_DATA_COLORS;lSeries++)
    {
        arColors.GetElement(&lSeries,&colData);
        m_colData[lSeries] = colData;

        if(lSeries+1>nNumSeries)
            continue;

        short red = GetRValue(colData),
                    green = GetGValue(colData),
                            blue = GetBValue(colData);

        collection.GetItem(lSeries+1).GetPen().GetVtColor().Set(red,green,blue);
        collection.GetItem(lSeries+1).GetDataPoints().GetItem(-1).GetBrush().GetFillColor().Set(red,green,blue);
    }
}

void CChartDlg::SetSeriesOnSecondaryAxis(int nSeries,BOOL bSet)
{
    CVcSeriesCollection &collection = m_ctlChart.GetPlot().GetSeriesCollection();
    int nNumSeries = collection.Count();
    if(nSeries>nNumSeries)
        return;

    collection.GetItem(nSeries).SetSecondaryAxis(bSet);
}

BOOL CChartDlg::GetSeriesOnSecondaryAxis(int nSeries)
{
    CVcSeriesCollection &collection = m_ctlChart.GetPlot().GetSeriesCollection();
    int nNumSeries = collection.Count();
    if(nSeries>nNumSeries)
        return false;

    return collection.GetItem(nSeries).GetSecondaryAxis();
}

void CChartDlg::GetDataColors(VARIANT& varColors)
{
    COleSafeArray saData;
    saData.CreateOneDim(VT_I4,MAX_DATA_COLORS,m_colData);
    varColors = saData;
}


void CChartDlg::OnLButtonDown(UINT nFlags, CPoint point)
{
    if(m_bZoomEnabled)
    {
        SetCapture();
        m_nDragStart = point.x;
        m_nPrevPos = point.x;
    }

    CDialog::OnLButtonDown(nFlags, point);
}

void CChartDlg::SetZoomRange(int nStartPoint,int nEndPoint)
{
    m_nZoomStart = nStartPoint;
    m_nZoomEnd = nEndPoint;

    UpdateChartData();
}

void CChartDlg::OnMouseMove(UINT nFlags, CPoint point)
{
    if(m_nDragStart >= 0)
    {
        CClientDC dc(this);
        CRect rc;
        GetClientRect(&rc);
        rc.left = m_nDragStart;
        rc.right =m_nPrevPos;
        dc.InvertRect(rc);

        rc.right = point.x;
        dc.InvertRect(rc);
        m_nPrevPos = point.x;
    }

    CDialog::OnMouseMove(nFlags, point);
}

void CChartDlg::OnLButtonUp(UINT nFlags, CPoint point)
{
    if(m_nDragStart >= 0)
    {
        ReleaseCapture();

        int nEndPoint = GetDataPointFromPoint(point.x);
        int nStartPoint = GetDataPointFromPoint(m_nDragStart);
        if(nStartPoint > nEndPoint)
        {
            int n = nStartPoint;
            nStartPoint = nEndPoint;
            nEndPoint = n;
        }

        if(nEndPoint == nStartPoint)
            SetZoomRange(0,-1);
        else
            SetZoomRange(nStartPoint,nEndPoint);

        m_nDragStart = -1;
    }

    CDialog::OnLButtonUp(nFlags, point);
}

int CChartDlg::GetDataPointFromPoint(int nXpos)
{
    CRect rc;
    m_ctlChart.GetWindowRect(&rc);

    //bodge to guess where axes are when there is a y label
    CString str;
    GetAxisLabel(VtChAxisIdY,str);
    const int nStartPoint = str.IsEmpty() ? 45 :71;

    GetAxisLabel(VtChAxisIdY2,str);
    int nLastPoint = rc.Width() - (str.IsEmpty() ? 45:71);

	int nNumPoints = 0;
	if(m_nZoomEnd == -1)
	{
		int nNumGroups = m_arGroupData.GetSize();
		for(int nGroup=0;nGroup<nNumGroups;nGroup++)
			if(m_arGroupData[nGroup]->GetCount() > nNumPoints)
				nNumPoints = m_arGroupData[nGroup]->GetCount();
	}
	else
		nNumPoints = m_nZoomEnd-m_nZoomStart;

    if(nXpos<nStartPoint)
        nXpos = nStartPoint;
    if(nXpos>nLastPoint)
        nXpos = nLastPoint;

    return m_nZoomStart + MulDiv((nXpos-nStartPoint),nNumPoints,(nLastPoint-nStartPoint));
}

void CChartDlg::SetSeriesAnnotation(int nSeries,CString strSeries)
{
    m_arSeriesAnnotation.SetAtGrow(nSeries-1,strSeries);
}

void CChartDlg::GetSeriesAnnotation(int nSeries,CString& strSeries)
{
	if(nSeries <= m_arSeriesAnnotation.GetSize())
		strSeries = m_arSeriesAnnotation[nSeries-1];
}

void CChartDlg::RemoveSeries(CString strSeries)
{
    for(int n=0;n<m_arSeriesAnnotation.GetSize();n++)
    {
        if(m_arSeriesAnnotation[n] == strSeries)
        {
            m_arSeriesAnnotation.RemoveAt(n);
            delete m_arGroupData[n];
            m_arGroupData.RemoveAt(n);
            n--;
        }
    }
    UpdateChartData();
}

void CChartDlg::GetChartData(VARIANT& vaData)
{
    COleSafeArray saData;

    int nNumGroups = m_arGroupData.GetSize();

    int nNumPoints = 0;
    for(int nGroup=0;nGroup<nNumGroups;nGroup++)
        if(nNumPoints < m_arGroupData[nGroup]->GetCount())
            nNumPoints = m_arGroupData[nGroup]->GetCount();

    DWORD dwNumElements[]= {nNumPoints , nNumGroups+1};
    saData.Create(VT_VARIANT,2,dwNumElements);

    VARIANT *pVar;
    saData.AccessData((void**)&pVar);

	for(int nIndex=0;nIndex<nNumPoints;nIndex++)
	{
		pVar[nIndex].vt = VT_BSTR;
		pVar[nIndex].bstrVal = _com_util::ConvertStringToBSTR(m_arAnnotation[nIndex]);
	}

    for(nGroup=1;nGroup<=nNumGroups;nGroup++)
    {
        int nIndex = nGroup*nNumPoints;

        DataList *pDataList = m_arGroupData[nGroup-1];
        POSITION pos = pDataList->GetHeadPosition();
        while(pos)
        {
            DataPoint& data = pDataList->GetNext(pos);

            if(data.bNull)
                pVar[nIndex].vt = VT_EMPTY;
            else
            {
                pVar[nIndex].vt = VT_R8;
                pVar[nIndex].dblVal = data.dValue;
            }
            nIndex++;
        }
    }
    saData.UnaccessData();

    vaData = saData.Detach();
}

void CChartDlg::CopyToClipboard()
{
    int nNumGroups = m_arGroupData.GetSize();

    POSITION *pGroupPos = new POSITION[nNumGroups];
    for(int nGroup=0;nGroup<nNumGroups;nGroup++)
        pGroupPos[nGroup] = m_arGroupData[nGroup]->FindIndex(m_nZoomStart);

    POSITION EndPos=NULL;
    if(m_nZoomEnd!=-1)
        EndPos = m_arGroupData[0]->FindIndex(m_nZoomEnd);

    CString strData;
	
    for(nGroup=0;nGroup<nNumGroups;nGroup++)
    {
        CString strLegend;
	GetLegendText(nGroup+1, strLegend);
        strData += "\t";
	if(m_arSeriesAnnotation.GetSize() > nGroup)
	    strData += strLegend;
    }

    strData += "\r\n";

    int nIndex=m_nZoomStart;
    while(pGroupPos[0] != EndPos)
    {
        CString strTemp;
        BOOL bAllNull = true;
        for(int nGroup=0;nGroup<nNumGroups;nGroup++)
        {
			if(pGroupPos[nGroup])
			{
				DataPoint& Data = m_arGroupData[nGroup]->GetNext(pGroupPos[nGroup]);

				if(Data.bNull)
					strTemp += "\t";
				else
				{
					bAllNull = false;

					CString str;
					str.Format("\t%f",Data.dValue);
					strTemp += str;
				}
			}
        }

        if(!bAllNull)
        {
            if(m_arAnnotation.GetSize() > nIndex)
                strData += m_arAnnotation[nIndex];

            strData += strTemp + "\r\n";
        }
        nIndex++;
    }

    delete []pGroupPos;

    m_ctlChart.EditCopy();

    if(!OpenClipboard())
        return;

    DWORD dw = strData.GetLength()+1;
    HGLOBAL hData = GlobalAlloc(GMEM_ZEROINIT|GMEM_MOVEABLE,dw);

    char *sz = (char*)GlobalLock(hData);

    strcpy(sz,strData);

    GlobalUnlock(hData);

    HANDLE hResult = SetClipboardData(CF_TEXT,hData);

    ASSERT(hResult != NULL);

    CloseClipboard();
}

void CChartDlg::LayoutGrid()
{
	int nRowCount = m_ctlChart.GetRowCount();
	int nNumLabels = 20;

	CRect rc;
	m_ctlChart.GetWindowRect(&rc);

	if(rc.Width() < 200)
		nNumLabels = 5;
	else
	if(rc.Width() < 400)
		nNumLabels = 10;
	else
	if(rc.Width() < 700)
		nNumLabels = 20;
	else
		nNumLabels = 25;

	int nDivisions = (int)(nRowCount/nNumLabels);

	CVcCategoryScale catScale = m_ctlChart.GetPlot().GetAxis(VtChAxisIdX,covOptional).GetCategoryScale();
	catScale.SetAuto(FALSE);
    if(catScale.GetDivisionsPerLabel() != nDivisions)
	{
        catScale.SetDivisionsPerLabel(nDivisions);
        catScale.SetDivisionsPerTick(nDivisions);
    }
}

void CChartDlg::SetRollingGraph(int nNumPoints)
{
	m_nMaxRollingPoints = nNumPoints;

    CVcDataGrid &dataGrid = m_ctlChart.GetDataGrid();
    int nNumRows = dataGrid.GetRowCount();
	if(nNumRows > nNumPoints)
	{
		int nNumCols = dataGrid.GetColumnCount();
		dataGrid.MoveData(nNumRows-nNumPoints,0,nNumRows-1,nNumCols-1,0,-(nNumRows-nNumPoints));
		dataGrid.SetRowCount(nNumPoints);
		if(m_arAnnotation.GetSize() >= nNumRows-nNumPoints)
			m_arAnnotation.RemoveAt(0,nNumRows-nNumPoints);
		else
			m_arAnnotation.RemoveAll();

		for(int nGroup=0;nGroup<m_arGroupData.GetSize();nGroup++)
		{
			DataList *pDataList = m_arGroupData[nGroup];
			while(pDataList->GetCount() > nNumPoints)
				pDataList->RemoveHead();
		}
	}
}