/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

#ifndef __MSCHART_DEFS__
#define __MSCHART_DEFS__

#include "VcTitle.h"
#include "VcBackdrop.h"
#include "VcFill.h"
#include "VcBrush.h"
#include "VcColor.h"
#include "VcPlot.h"
#include "VcSeriesCollection.h"
#include "VcSeries.h"
#include "VcPen.h"
#include "VcFont.h"
#include "VcAxis.h"
#include "VcAxisTitle.h"
#include "VcValueScale.h"
#include "VcCategoryScale.h"
#include "VcLabels.h"
#include "VcLabel.h"
#include "VcDataGrid.h"
#include "VcSeriesPosition.h"
#include "VcAxisGrid.h"
#include "VcRect.h"
#include "VcCoor.h"
#include "VcLocation.h"
#include "VcLegend.h"
#include "VcIntersection.h"
#include "VcDataPoints.h"
#include "VcDataPoint.h"
#include "VcTextLayout.h"
#include "VcFrame.h"

typedef enum {
    VtChChartType3dBar = 0,
    VtChChartType2dBar = 1,
    VtChChartType3dLine = 2,
    VtChChartType2dLine = 3,
    VtChChartType3dArea = 4,
    VtChChartType2dArea = 5,
    VtChChartType3dStep = 6,
    VtChChartType2dStep = 7,
    VtChChartType3dCombination = 8,
    VtChChartType2dCombination = 9,
    VtChChartType3dHorizontalBar = 10,
    VtChChartType2dHorizontalBar = 11,
    VtChChartType3dClusteredBar = 12,
    VtChChartType3dPie = 13,
    VtChChartType2dPie = 14,
    VtChChartType3dDoughnut = 15,
    VtChChartType2dXY = 16,
    VtChChartType2dPolar = 17,
    VtChChartType2dRadar = 18,
    VtChChartType2dBubble = 19,
    VtChChartType2dHiLo = 20,
    VtChChartType2dGantt = 21,
    VtChChartType3dGantt = 22,
    VtChChartType3dSurface = 23,
    VtChChartType2dContour = 24,
    VtChChartType3dScatter = 25,
    VtChChartType3dXYZ = 26,
    VtChChartTypeCount = 27
} VtChChartType;

typedef enum {
    VtChSeriesTypeVaries = -2,
    VtChSeriesTypeDefault = -1,
    VtChSeriesType3dBar = 0,
    VtChSeriesType2dBar = 1,
    VtChSeriesType3dHorizontalBar = 2,
    VtChSeriesType2dHorizontalBar = 3,
    VtChSeriesType3dClusteredBar = 4,
    VtChSeriesType3dLine = 5,
    VtChSeriesType2dLine = 6,
    VtChSeriesType3dArea = 7,
    VtChSeriesType2dArea = 8,
    VtChSeriesType3dStep = 9,
    VtChSeriesType2dStep = 10,
    VtChSeriesType2dXY = 11,
    VtChSeriesType2dPolar = 12,
    VtChSeriesType2dRadarLine = 13,
    VtChSeriesType2dRadarArea = 14,
    VtChSeriesType2dBubble = 15,
    VtChSeriesType2dHiLo = 16,
    VtChSeriesType2dHLC = 17,
    VtChSeriesType2dHLCRight = 18,
    VtChSeriesType2dOHLC = 19,
    VtChSeriesType2dOHLCBar = 20,
    VtChSeriesType2dGantt = 21,
    VtChSeriesType3dGantt = 22,
    VtChSeriesType3dPie = 23,
    VtChSeriesType2dPie = 24,
    VtChSeriesType3dDoughnut = 25,
    VtChSeriesType2dDates = 26,
    VtChSeriesType3dBarHiLo = 27,
    VtChSeriesType2dBarHiLo = 28,
    VtChSeriesType3dHorizontalBarHiLo = 29,
    VtChSeriesType2dHorizontalBarHiLo = 30,
    VtChSeriesType3dClusteredBarHiLo = 31,
    VtChSeriesType3dSurface = 32,
    VtChSeriesType2dContour = 33,
    VtChSeriesType3dXYZ = 34,
    VtChSeriesTypeCount = 35
} VtChSeriesType;

typedef enum {
    VtFrameStyleNull = 0,
    VtFrameStyleSingleLine = 1,
    VtFrameStyleDoubleLine = 2,
    VtFrameStyleThickInner = 3,
    VtFrameStyleThickOuter = 4
} VtFrameStyle;

typedef enum {
    VtFillStyleNull = 0,
    VtFillStyleBrush = 1,
    VtFillStyleGradient = 2
} VtFillStyle;

typedef enum {
    VtChAxisIdX = 0,
    VtChAxisIdY = 1,
    VtChAxisIdY2 = 2,
    VtChAxisIdZ = 3,
    VtChAxisIdNone = 4
} VtChAxisId;

typedef enum {
    VtChLocationTypeTopLeft = 0,
    VtChLocationTypeTop = 1,
    VtChLocationTypeTopRight = 2,
    VtChLocationTypeLeft = 3,
    VtChLocationTypeRight = 4,
    VtChLocationTypeBottomLeft = 5,
    VtChLocationTypeBottom = 6,
    VtChLocationTypeBottomRight = 7,
    VtChLocationTypeCustom = 8
} VtChLocationType;

typedef enum {
    VtChDrawModeDraw = 0,
    VtChDrawModeBlit = 1
} VtChDrawMode;

typedef enum {
    VtChPartTypeChart = 0,
    VtChPartTypeTitle = 1,
    VtChPartTypeFootnote = 2,
    VtChPartTypeLegend = 3,
    VtChPartTypePlot = 4,
    VtChPartTypeSeries = 5,
    VtChPartTypeSeriesLabel = 6,
    VtChPartTypePoint = 7,
    VtChPartTypePointLabel = 8,
    VtChPartTypeAxis = 9,
    VtChPartTypeAxisLabel = 10,
    VtChPartTypeAxisTitle = 11,
    VtChPartTypeSeriesName = 12,
    VtChPartTypePointName = 13,
    VtChPartTypeCount = 14
} VtChPartType;

typedef enum {
    VtVerticalAlignmentTop = 0,
    VtVerticalAlignmentBottom = 1,
    VtVerticalAlignmentCenter = 2
} VtVerticalAlignment;

typedef enum {
    VtHorizontalAlignmentLeft = 0,
    VtHorizontalAlignmentRight = 1,
    VtHorizontalAlignmentCenter = 2,
    VtHorizontalAlignmentFill = 3,
    VtHorizontalAlignmentFlush = 4
} VtHorizontalAlignment;

#endif //__MSCHART_DEFS__

