#ifndef __SUMARRY_TASK__
#define __SUMARRY_TASK__


#include <string>
#include <vector>
using std::string;
using std::vector;

#define TASKSUMMARY_VERSION 0

class CSummaryTask
{
public:
	CSummaryTask();
	CSummaryTask(	bool bScript, 
					string name,
					string target,
					string description,
					int iDuration, 
					bool bIsDuration, 
					int iNumIterations,
					int iMaxIterDelay,
					int iMinIterDelay,
					int iFixedIterDelay,
					bool bConstantDelay);

	void	display();

	bool		bScript_;			// True if script, false if collector
	string		name_;
	string		target_;
	string		description_;
	int			iDuration_;
	bool		bIsDuration_;
	int			iNumIterations_;
	int			iMaxIterDelay_;
	int			iMinIterDelay_;
	int			iFixedIterDelay_;
	bool		bConstantDelay_;
};

typedef vector<CSummaryTask *> CSummaryTaskVectorType;

class CSummaryTaskGroup
{
public:
	CSummaryTaskGroup() ;
	CSummaryTaskGroup(	string		hostname,
						string		name,
						string		description,
						bool		bIsDelayed,
						bool		bStopOnComplete,
						int			iVUsPerBatch,
						int			iInterBatchDelay,
						int			iBatchRampUpTime,
						bool		bShowAdvancedVUInfo,
						int			iTaskGroupIterations,
						int			iTaskGroupDuration,
						int			iStartInterval,
						int			iNumVUs,
						int			iNumVUsTimRes,
						int			iNumVUsHttpRes,
						int			iVUPerInterval,
						int			iStartDelay,
						bool		bDuration,
						bool		bScheduled,
						string		scheduledTime);
	void initialize(	string		hostname,
						string		name,
						string		description,
						bool		bIsDelayed,
						bool		bStopOnComplete,
						int			iVUsPerBatch,
						int			iInterBatchDelay,
						int			iBatchRampUpTime,
						bool		bShowAdvancedVUInfo,
						int			iTaskGroupIterations,
						int			iTaskGroupDuration,
						int			iStartInterval,
						int			iNumVUs,
						int			iNumVUsTimRes,
						int			iNumVUsHttpRes,
						int			iVUPerInterval,
						int			iStartDelay,
						bool		bDuration,
						bool		bScheduled,
						string		scheduledTime);
	~CSummaryTaskGroup() ;
	void clear();

	void	display();
	int		GetNbTasks() { return taskVector_.size() ;}

	int			iTimestamp_ ;
	string		hostname_;
	string		name_;
	string		description_;
	bool		bIsDelayed_;
	bool		bStopOnComplete_;
	int			iVUsPerBatch_;
	int			iInterBatchDelay_;
	int			iBatchRampUpTime_;
	bool		bShowAdvancedVUInfo_;

	int			iTaskGroupIterations_;
	int			iTaskGroupDuration_;
	int			iStartInterval_;
	int			iNumVUs_;
	int			iNumVUsTimRes_;
	int			iNumVUsHttpRes_;
	int			iVUPerInterval_;
	int			iStartDelay_;
	bool		bDuration_;
	bool		bScheduled_;
	string		scheduledTime_;

	CSummaryTaskVectorType taskVector_ ;
};

class CSummaryFile
{
public:
	CSummaryFile(string fileName)
		:fileName_(fileName), iVersion_(-1), iNbTaskGroup_(0) {}
	bool		openForWrite(string testName, string testDescription);
	bool		openForRead();
	bool		openForAppend();
	bool		close();

	int			GetVersion() { return iVersion_ ;}
	int			GetNbTaskGroup() { return iNbTaskGroup_; }

	bool		writeTaskGroup(int iTimestamp, CSummaryTaskGroup & summaryTaskGroup) ;
	bool		readTaskGroup(CSummaryTaskGroup & summaryTaskGroup) ;

	const string & getError() const { return error_ ;}
private:
	bool		writeString(const string & str);
	bool		writeInt(int i);
	bool		writeBool(bool b);

	bool		readString(string & str);
	bool		readInt(int & i);
	bool		readBool(bool & b);

protected:
	string	fileName_ ;
	int		iVersion_ ;
	int		iNbTaskGroup_ ;
	bool	bFileWrite_ ;		// TRUE if the file is open for write

	string	testName_;
	string	testDescription_;

	CFile	fd_ ;
	string	error_;
};


#endif