/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

#include <winsock2.h>
#include "StepByStep_TestExecuter_i.h"
#include "CECInetInfo.h"
#include "TEMainStructure.hpp"

extern CMainExecuterStructure *g_pTExecuter;


StepByStepScriptExecuterImpl::StepByStepScriptExecuterImpl(CORBAServer* pserver)
    : m_pServer(pserver), m_pObjid(NULL)
{
    char strReg[64];
    CECInetInfo inet;
    m_pObjid = m_pServer->ActivateObject(this);
    wsprintf(strReg, "%s_%8.8lX", inet.GetIPAddressAsString().c_str(), GetCurrentProcessId());
    m_strRegisterName = strReg;
    m_pServer->NameObject(this->_this(), StepByStepScriptExecuter::CLASS_NAME, strReg);

    // Connect to the step-by-step control object.
    try
    {
	    CORBA::Object_var obj = m_pServer->FindObjectWithName(StepByStepControl::CLASS_NAME, inet.GetIPAddressAsString().c_str());
	    m_refControl = StepByStepControl::_narrow(obj);
    }
    catch(const CyrCORBAException&)
    {
        // Object could not be found.
    }

    // Register this object in the step-by-step control object.
    if(!CORBA::is_nil(m_refControl))
        m_refControl->SetTestExecuter(this->_this());
}

StepByStepScriptExecuterImpl::~StepByStepScriptExecuterImpl()
{
    m_pServer->UnNameObject(StepByStepScriptExecuter::CLASS_NAME, m_strRegisterName.c_str());
    Deactivate();
}

void StepByStepScriptExecuterImpl::Deactivate()
{
    if(m_pObjid)
    {
        m_pServer->DeActivateObject(*m_pObjid);
        m_pObjid = NULL;
    }
}

CORBA::Boolean StepByStepScriptExecuterImpl::Next(CORBA::Long /*lTaskGroupID*/, CORBA::Long lVUId)
{
    g_pTExecuter->GetThreadPoolController()->StepByStep_Next((DWORD)lVUId);
    return 1;
}
