/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

#include <windows.h>
#include "ConfigFile.hpp"

CConfigFile::CConfigFile(const std::string& strFile)
     : m_strFile(strFile)
{
}

void CConfigFile::SetFile(const std::string& strFile)
{
    // Get the exe's path.
    char zPath[MAX_PATH];
    GetModuleFileNameA(NULL, zPath, sizeof(zPath));

    // Strip off the file's name and extension.
    LPSTR pEnd = zPath + lstrlenA(zPath);
    while(*pEnd != '\\')
        --pEnd;
    pEnd[1] = 0;    // The path now ends with the final backslash.
    m_strFile = zPath + strFile;
}

BOOL CConfigFile::Set(const std::string& strKey, const std::string& strValue, LPCSTR zSection) const
{
    if(zSection == NULL)
        zSection = m_strSection.c_str();
    return WritePrivateProfileString(zSection, strKey.c_str(), strValue.c_str(), m_strFile.c_str());
}

BOOL CConfigFile::Set(const std::string& strKey, long lValue, LPCSTR zSection) const
{
    char zValue[64];
    wsprintf(zValue, "%d", lValue);
    return Set(strKey, std::string(zValue), zSection);
}

const CConfigFile& CConfigFile::Get(const std::string& strKey, std::string& strResult, LPCSTR zDefault, LPCSTR zSection) const
{
    if(zSection == NULL)
        zSection = m_strSection.c_str();
    char zResult[1024];
    GetPrivateProfileString(zSection, strKey.c_str(), zDefault == NULL ? "" : zDefault, zResult, sizeof(zResult), m_strFile.c_str());
    strResult = zResult;
    return *this;
}

const CConfigFile& CConfigFile::Get(const std::string& strKey, long& lResult, long lDefault, LPCSTR zSection) const
{
    if(zSection == NULL)
        zSection = m_strSection.c_str();
    lResult = GetPrivateProfileInt(zSection, strKey.c_str(), lDefault, m_strFile.c_str());
    return *this;
}
