/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/
#ifndef __TAGPARSING__
#define __TAGPARSING__


///////////////////////
// Tag parsing helpers.
///////////////////////

#ifdef _WIN32
#   pragma warning (disable: 4786)
#endif

#pragma warning(disable:4800)
#include<map>
#pragma warning(default:4800)

// This enum contains all the tag types.
enum Tags
{
    TAG_NONE,   // No tag.
    TAG_ROOT,   // fake top level document tag
    TAG_A,
    TAG_ABBR,
    TAG_ACRONYM,
    TAG_ADDRESS,
    TAG_APPLET,
    TAG_AREA,
    TAG_B,
    TAG_BASE,
    TAG_BASEFONT,
    TAG_BDO,
    TAG_BGSOUND,
    TAG_BIG,
    TAG_BLINK,
    TAG_BLOCKQUOTE,
    TAG_BODY,
    TAG_BR,
    TAG_BUTTON,
    TAG_CAPTION,
    TAG_CENTER,
    TAG_CITE,
    TAG_CODE,
    TAG_COL,
    TAG_COLGROUP,
    TAG_COMMENT,
    TAG_DD,
    TAG_DEL,
    TAG_DFN,
    TAG_DIR,
    TAG_DIV,
    TAG_DL,
    TAG_DT,
    TAG_EM,
    TAG_EMBED,
    TAG_FIELDSET,
    TAG_FONT,
    TAG_FORM,
    TAG_FRAME,
    TAG_FRAMESET,
    TAG_H1,
    TAG_H2,
    TAG_H3,
    TAG_H4,
    TAG_H5,
    TAG_H6,
    TAG_HEAD,
    TAG_HR,
    TAG_HTML,
    TAG_I,
    TAG_IFRAME,
    TAG_ILAYER,
    TAG_IMG,
    TAG_INPUT,
    TAG_INS,
    TAG_ISINDEX,
    TAG_KBD,
    TAG_KEYGEN,
    TAG_LABEL,
    TAG_LAYER,
    TAG_LEGEND,
    TAG_LI,
    TAG_LINK,
    TAG_LISTING,
    TAG_MAP,
    TAG_MARQUEE,
    TAG_MENU,
    TAG_META,
    TAG_MULTICOL,
    TAG_NOBR,
    TAG_NOEMBED,
    TAG_NOFRAMES,
    TAG_NOLAYER,
    TAG_NOSCRIPT,
    TAG_OBJECT,
    TAG_OL,
    TAG_OPTGROUP,
    TAG_OPTION,
    TAG_P,
    TAG_PARAM,
    TAG_PLAINTEXT,
    TAG_PRE,
    TAG_Q,
    TAG_RT,
    TAG_RUBY,
    TAG_S,
    TAG_SAMP,
    TAG_SCRIPT,
    TAG_SELECT,
    TAG_SERVER,
    TAG_SMALL,
    TAG_SPACER,
    TAG_SPAN,
    TAG_STRIKE,
    TAG_STRONG,
    TAG_STYLE,
    TAG_SUB,
    TAG_SUP,
    TAG_TABLE,
    TAG_TBODY,
    TAG_TD,
    TAG_TEXTAREA,
    TAG_TFOOT,
    TAG_TH,
    TAG_THEAD,
    TAG_TITLE,
    TAG_TR,
    TAG_TT,
    TAG_U,
    TAG_UL,
    TAG_VAR,
    TAG_WBR,
    TAG_XML,
    TAG_XMP
};

// This enum defines the actual behavior adopted by an opening tag when
// followed by another opening tag.
enum TagBehavior
{
    TAG_CLOSES,
    TAG_CREATES_CHILD
};

// The collection of specific behaviors to adopt when the (opening) tag
// is followed by another opening tag.
// This collection is a concatenation of sets of entries. Each set of
// entries is composed as follows:
// - the tag (type Tags)
// - the tag's name (type const char*)
// - the tag's default behavior (type TagBehavior)
// - a sequence of tags (type Tags) adopting the opposite behavior from
//   the default.
// - the constant -1
// A final -1 ends the array.
extern const long tagBehaviors[];

struct StrCompare : public std::binary_function<const char*,const char*,BOOL>
{
    BOOL operator()(const char* s1, const char* s2) const
    {
        return lstrcmp(s1,s2) < 0;
    }
};

// The map giving access to tag IDs by tag name.
typedef std::map<const char*, Tags, StrCompare> NameToTag;
extern NameToTag nameToTag;

// The map giving access to tag names by tag ID.
typedef std::map<Tags, const char*> TagToName;
extern TagToName tagToName;

// Key built from a couple (tag1, tag2).
struct TagCouple
{
    Tags eTag1;
    Tags eTag2;

    bool operator<(const TagCouple& tc) const
    {
        if(eTag1 < tc.eTag1)
            return true;
        else if(eTag1 == tc.eTag1)
        {
            if(eTag2 < tc.eTag2)
                return true;
        }
        return false;
    }
};

// The map giving the behavior of a tag when encountering another opening tag.
typedef std::map<TagCouple, TagBehavior> TagToBehavior;
extern TagToBehavior tagToBehavior;

// The map giving the default behavior of a tag.
typedef std::map<Tags, TagBehavior> TagToDefaultBehavior;
extern TagToDefaultBehavior tagToDefaultBehavior;

// Initialization procedure for the global variables.
void InitTagParsing();

#endif  // __TAGPARSING__
