/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/


#ifndef __CAPTURE__
#define __CAPTURE__

#ifdef HTTPCAPTURECMD_EXPORTS
#  define DLLSPEC_HTTPCAPTURECMD __EXPORT__
#else
#  define DLLSPEC_HTTPCAPTURECMD __IMPORT__
#endif

#include <socket.hpp>

class DLLSPEC_HTTPCAPTURECMD HttpCaptureCmd
{

public:
    HttpCaptureCmd(port_t adminPort = 3000);
    ~HttpCaptureCmd(void);

    // Start the gateway
    bool startGateway(string _strBufDirectory, bool bConsole = false, bool bTrace = false) ;
    // Stop the gateway
    bool stopGateway()
    {
        return shutdown();
    }
    // Send browser parameters to the gateway
    bool browserParameters(string httpProxy,
                           string httpsProxy,
                           string ftpProxy,
                           string gopherProxy,
                           string socksProxy,
                           string exceptions,
                           bool bNoProxyForLocal);
    // Send browser parameters to the gateway.
    // GOPHER, FTP, SOCKS use the HTTP proxy value
    bool browserParameters(string httpProxy,
                           string httpsProxy,
                           string exceptions,
                           bool bNoProxyForLocal);
    // Send browser parameters.
    // All protocol use the same proxy
    bool browserParameters(string proxy,
                           string exceptions,
                           bool bNoProxyForLocal);
    // Set the listen task (listen port number)
    bool listen(port_t listenPort);
    // Specify to the gateway the path to save the documents
    bool setDocumentsPath(string path) ;

    // record in a tof file with name tofFileName.
    // the title tofTitle is written at the top of the file
    bool record(string tofFileName, int iConid = 0, string tofTitle = "");
    // pause the capture
    bool pause(void);
    // resume the capture
    bool resume(void);
    // stop the capture
    bool stop(void);
    // add a comment tofComment to the capture
    bool comment(string tofComment, unsigned long ulMilisecond);

	// add STimer to the capture
	bool StartTimer(string strTimer, unsigned long ulMilisecond);
	// add ETimer to the capture
	bool EndTimer(string strTimer, unsigned long ulMilisecond);
    // shutdown the gateway
    bool shutdown(void);
	// Set the cookie modelling to on or off
	bool automaticCookiesModelling(bool bModel, bool bComment) ;
    // Member access (should not be very usefull)
    void    setAdminPort(port_t adminPort)
    {
        adminPort_ = adminPort ;
    }

    port_t  getAdminPort()
    {
        return adminPort_ ;
    }
    CTSocket * getAdmingateSocket()
    {
        return adminGateSocket_ ;
    }

private:
    port_t      adminPort_ ;
    string  commandDllName_ ;
    string  decodingDllName_ ;
    CTSocket    *adminGateSocket_ ;
    string  documents_ ;
    HANDLE m_hTraceOutputPipe;
};


#endif
