/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

//==================================================================< Pragmas >
#ifdef  WIN32
#  pragma warning(disable : 4710)
#endif  // WIN32


//=================================================================< Includes >
#ifndef __CYRTOOLS_TIME__
#  include <ctime.hpp>
#endif  // __CYRTOOLS_TIME__

#ifndef __ADMINISTRATION_LISTEN_CLIENT__
#  include "adminlc.hpp"
#endif  // __ADMINISTRATION_LISTEN_CLIENT__

#ifndef __GATEWAY_LISTEN_CLIENT__
#  include "gatewlc.hpp"
#endif  // __GATEWAY_LISTEN_CLIENT__


//======================================================< Definition of class >

///////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////// Class CGCmdWhoWithAssign //
///////////////////////////////////////////////////////////////////////////////
class CGCmdWhoWithAssign : public CGCmdWho
{
    //-----------------------------------------------------------------------------
public:                         // Constructor(s) and destructor.
    CGCmdWhoWithAssign ()
: CGCmdWho ()
    { }

    CGCmdWhoWithAssign (const CGCmdWhoStlList& childrenList, enumWhoType eType, long lId, const string& clientAddress, const string& gatewayAddress, const string& targetAddress, const string& library)
: CGCmdWho (childrenList, eType, lId, clientAddress, gatewayAddress, targetAddress, library)
    { }

    CGCmdWhoWithAssign (CGCmdWhoWithAssign& who)
: CGCmdWho (who)
    { }

    ~CGCmdWhoWithAssign ()
    { }


public:                         // Operators.
    CGCmdWhoWithAssign&         operator = (CGTask& task) ;

    //-----------------------------------------------------------------------------
}

; // CGCmdWhoWithAssign

///////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////// End of Class CGCmdWhoWithAssign //
///////////////////////////////////////////////////////////////////////////////

//====================================================================< Class >

///////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////// Class CGCmdWhoWithAssign //
///////////////////////////////////////////////////////////////////////////////

//================================================================< Operators >
//-----------------------------------------------------------------------------
// CGCmdWhoWithAssign::operator =
//
// Constructor.
//
// Arguments : task : task.
// Return    : this.
//-----------------------------------------------------------------------------
CGCmdWhoWithAssign& CGCmdWhoWithAssign::operator = (CGTask& task)
{
    for (CGCmdWhoStlList::iterator itr = ( (CGCmdWho*) this)->getChildrenList ().begin(); itr != ( (CGCmdWho*) this)->getChildrenList ().end(); itr ++)
        delete *itr ;
    ( (CGCmdWho*) this)->getChildrenList ().clear () ;
    for (size_t i = 0 ; i < task.getChildrenList ().size() ; ++ i)
    {
        ( (CGCmdWho*) this)->getChildrenList ().push_back (new CGCmdWho) ;
        * ( (CGCmdWhoWithAssign*) ( ( (CGCmdWho*) this)->getChildrenList ()[i]) ) = * (task.getChildrenList ()[i]) ;
    }
    ( (CGCmdWho*) this)->getType () = (enumWhoType) task.getType () ;
    ( (CGCmdWho*) this)->getId () = task.getTaskId () ;
    if ( ( (CGCmdWho*) this)->getType () == WHO_ADMINISTRATION_LISTEN)
    {
        CGAdministrationListen* pAdministrationListen = (CGAdministrationListen*) task.getData () ;
        if (pAdministrationListen)
        {
            char szAddress [22] ;
            sprintf (szAddress, "%s:%hu", pAdministrationListen->getSocket ().getInetAddress ().getHostAddress ().c_str (), pAdministrationListen->getSocket ().getPort () ) ;
            ( (CGCmdWho*) this)->getGatewayAddress () = szAddress ;
        }
    }
    else if ( ( (CGCmdWho*) this)->getType () == WHO_ADMINISTRATION_CLIENT)
    {
        CGAdministrationClient* pAdministrationClient = (CGAdministrationClient*) task.getData () ;
        if (pAdministrationClient)
        {
            char szAddress [22] ;
            sprintf (szAddress, "%s:%hu", pAdministrationClient->getSocket ()->getInetAddress ().getHostAddress ().c_str (), pAdministrationClient->getSocket ()->getPort () ) ;
            ( (CGCmdWho*) this)->getGatewayAddress () = szAddress ;
            sprintf (szAddress, "%s:%hu", pAdministrationClient->getSocket ()->getPeerInetAddress ().getHostAddress ().c_str (), pAdministrationClient->getSocket ()->getPeerPort () ) ;
            ( (CGCmdWho*) this)->getClientAddress () = szAddress ;
            if (pAdministrationClient->getLibrary () )
            {
                ( (CGCmdWho*) this)->getLibrary () = CTLibrary::getSystemFileName (CTLibrary::getBaseFileName (pAdministrationClient->getLibrary ()->getName () ) ) ;
            }
            else
            {
                ( (CGCmdWho*) this)->getLibrary () = "null" ;
            }
        }
    }
    else if ( ( (CGCmdWho*) this)->getType () == WHO_GATEWAY_LISTEN)
    {
        CGGatewayListen*    pGatewayListen = (CGGatewayListen*) task.getData () ;
        if (pGatewayListen)
        {
            char szAddress [22] ;
            sprintf (szAddress, "%s:%hu", pGatewayListen->getSocket ().getInetAddress ().getHostAddress ().c_str (), pGatewayListen->getSocket ().getPort () ) ;
            ( (CGCmdWho*) this)->getGatewayAddress () = szAddress ;
            ( (CGCmdWho*) this)->getTargetAddress () = pGatewayListen->getGatewayInfo ().getTargetAddress () ;
            if (pGatewayListen->getLibrary () )
            {
                ( (CGCmdWho*) this)->getLibrary () = CTLibrary::getSystemFileName (CTLibrary::getBaseFileName (pGatewayListen->getLibrary ()->getName () ) ) ;
            }
        }
    }
    else if ( ( (CGCmdWho*) this)->getType () == WHO_GATEWAY_CLIENT)
    {
        CGGatewayClient*    pGatewayClient = (CGGatewayClient*) task.getData () ;
        if (pGatewayClient)
        {
            char szAddress [22] ;
            sprintf (szAddress, "%s:%hu", pGatewayClient->getClientSocket ()->getPeerInetAddress ().getHostAddress ().c_str (), pGatewayClient->getClientSocket ()->getPeerPort () ) ;
            ( (CGCmdWho*) this)->getClientAddress () = szAddress ;
            sprintf (szAddress, "%s:%hu", pGatewayClient->getClientSocket ()->getInetAddress ().getHostAddress ().c_str (), pGatewayClient->getClientSocket ()->getPort () ) ;
            ( (CGCmdWho*) this)->getGatewayAddress () = szAddress ;
            sprintf (szAddress, "%s:%hu", pGatewayClient->getTargetSocket ()->getPeerInetAddress ().getHostAddress ().c_str (), pGatewayClient->getTargetSocket ()->getPeerPort () ) ;
            ( (CGCmdWho*) this)->getTargetAddress () = szAddress ;
            if (pGatewayClient->getLibrary () )
            {
                ( (CGCmdWho*) this)->getLibrary () = CTLibrary::getSystemFileName (CTLibrary::getBaseFileName (pGatewayClient->getLibrary ()->getName () ) ) ;
            }
        }
    }

    return *this ;
} // CGCmdWhoWithAssign::operator =

///////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////// End of Class CGCmdWhoWithAssign //
///////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////// Class CGAdministrationClient //
///////////////////////////////////////////////////////////////////////////////

//============================================< Constructor(s) and destructor >
//-----------------------------------------------------------------------------
// CGAdministrationClient::CGAdministrationClient
//
// Constructor.
//
// Arguments : task : task associated.
// Return    : none.
//-----------------------------------------------------------------------------
CGAdministrationClient::CGAdministrationClient (CGTask& task)
        : task_ (task), pClient_ (NULL), pLibrary_ (NULL), newGatewayParseCommand_ (NULL), pGWParseCommand_ (NULL)
{
    // Set the task data.
    task_.setData (this) ;

    // Save this pointer.
    CTCurrentThread::setData (this) ;

    // Set the abort routine.
    CTCurrentThread::setAbortRoutine (abort) ;
} // CGAdministrationClient::CGAdministrationClient

//-----------------------------------------------------------------------------
// CGAdministrationClient::~CGAdministrationClient
//
// Destructor.
//
// Arguments : none.
// Return    : none.
//-----------------------------------------------------------------------------
CGAdministrationClient::~CGAdministrationClient ()
{
    if (pClient_)
    {
        if (GpProcessMain->getTraceMode () )
        {
            CTLockGuard<CTMutex>    guard (GpProcessMain->getLogFileList ().getMutex () ) ;
            GpProcessMain->getLogFileList () << task_.getTaskName () << ": <" << pClient_->getPeerInetAddress ().getHostAddress () << ":" << pClient_->getPeerPort () << "> disconnected." << endl ;
        }
        else
        {
            CTLockGuard<CTMutex>    guard (GpProcessMain->getLogFileList ().getMutex () ) ;
            GpProcessMain->getLogFileList () << "Disconnection from <" << pClient_->getPeerInetAddress ().getHostAddress () << ":" << pClient_->getPeerPort () << ">." << endl ;
        }

        pClient_->close () ;
        delete pClient_ ;
        pClient_ = NULL ;
    }

    if (pGWParseCommand_)
    {
        delete pGWParseCommand_ ;
        pGWParseCommand_ = NULL ;
    }

    if (pLibrary_)
    {   // Unload library.
        delete pLibrary_ ;
        pLibrary_ = NULL ;
    }
} // CGAdministrationClient::~CGAdministrationClient

//=========================================================< Control methodes >
//-----------------------------------------------------------------------------
// CGAdministrationClient::init
//
// Initialize.
//
// Arguments : none.
// Return    : none.
//-----------------------------------------------------------------------------
void CGAdministrationClient::init (void)
{
    pClient_ = new CTSocket ( ( (CTSocket*) (task_.getInitInfo () ) )->accept () ) ;
    if (pClient_ == NULL)
    {
        throw CGTaskException ("init: new CTSocket failed.", __FILE__, __LINE__) ;
    }
    if (GpProcessMain->getTraceMode () )
    {
        CTLockGuard<CTMutex>    guard (GpProcessMain->getLogFileList ().getMutex () ) ;
        GpProcessMain->getLogFileList () << task_.getTaskName () << ": <administration client -> gateway              >" << endl ;
        char szAddress1 [22] ;
        char szAddress2 [22] ;
        char szLine [80] ;
        sprintf (szAddress1, "%s:%hu", pClient_->getPeerInetAddress ().getHostAddress ().c_str (), pClient_->getPeerPort () ) ;
        sprintf (szAddress2, "%s:%hu", pClient_->getInetAddress ().getHostAddress ().c_str (), pClient_->getPort () ) ;
        sprintf (szLine, "<%-21.21s -> %-21.21s>", szAddress1, szAddress2) ;
        GpProcessMain->getLogFileList () << task_.getTaskName () << ": " << szLine << endl ;
    }
    else
    {
        CTLockGuard<CTMutex>    guard (GpProcessMain->getLogFileList ().getMutex () ) ;
        GpProcessMain->getLogFileList () << "Connection from <" << pClient_->getPeerInetAddress ().getHostAddress () << ":" << pClient_->getPeerPort () << ">." << endl ;
    }

    // Tell the parent task that initialization is terminated.
    task_.initializationTerminated () ;
} // CGAdministrationClient::init

//-----------------------------------------------------------------------------
// CGAdministrationClient::processConnection
//
// Process connection.
//
// Arguments : none.
// Return    : none.
//-----------------------------------------------------------------------------
void CGAdministrationClient::processConnection (void)
{
    try
    {   // Process connection.
        string  command ;
        bool        bContinue = true ;

        do
        {
            if (task_.hasToBeReinitialized () )
            {
                task_.reinitialized () ;
            }

            // Wait for command.
            while (pClient_->poll (_S_RECEIVE_ | _S_EXCEPTION_) == CTSocket::TIMEDOUT)
            {
                CTCurrentThread::sleep (1) ;
            }

            // Receive command from admingate.
            *pClient_ >> command ;
            // OC 240800 Put command to uppercase
            for (int iCommand = 0; iCommand<(int)command.size(); iCommand++)
                command[iCommand] = toupper((int)command[iCommand]) ;

            try
            {
                if (pGWParseCommand_ == NULL)
                {
                    if (GpProcessMain->getTraceMode () )
                    {
                        CTLockGuard<CTMutex>    guard (GpProcessMain->getLogFileList ().getMutex () ) ;
                        GpProcessMain->getLogFileList () << task_.getTaskName () << ": " << command << "." << endl ;
                    }
                    if (command.find(":PING") != string::npos)
                    {
                        // Send command result to admingate.
                        *pClient_ << string ("PONG") ;
                        continue ;
                    }
                    if (command.find (":GET_EXE_NAME") != string::npos)
                    {
                        // Send command result to admingate.
                        *pClient_ << string ("GATEWAY") ;
                        continue ;
                    }
                    else if (command.find (":GOODBYE") != string::npos )
                    {
                        bContinue = false ;
                        break ;
                    }
                    else if (command.find (":USE ") != string::npos )
                    {
                        string  libraryName = command.substr(command.find (":USE ") + 5, command.size() - (command.find(":USE ") + 5) ) ;

                        if (GpProcessMain->getTraceMode () )
                        {
                            CTLockGuard<CTMutex>    guard (GpProcessMain->getLogFileList ().getMutex () ) ;
                            GpProcessMain->getLogFileList () << task_.getTaskName () << ": try to load the library '" << CTLibrary::getSystemFileName (CTLibrary::getBaseFileName (libraryName) ) << "'." << endl ;
                        }

                        try
                        {
                            try
                            {
                                pLibrary_ = new CTLibrary (libraryName) ;

                                if (GpProcessMain->getTraceMode () )
                                {
                                    CTLockGuard<CTMutex>    guard (GpProcessMain->getLogFileList ().getMutex () ) ;
                                    GpProcessMain->getLogFileList () << task_.getTaskName () << endl ;
                                }
                            }
                            catch (CTxalloc&)
                            {   // Error from new.
                                throw CGTaskException (string ("processConnection: fail to load the library '") + CTLibrary::getSystemFileName (CTLibrary::getBaseFileName (libraryName) ) + "': new CTLibrary (libraryName) failed.", __FILE__, __LINE__) ;
                            }
                            catch (CTException& e)
                            {
                                throw CGTaskException (string ("processConnection: fail to load the library '") + CTLibrary::getSystemFileName (CTLibrary::getBaseFileName (libraryName) ) + "'.\n" + e.what (), __FILE__, __LINE__) ;
                            }

                            // Determinate if there is a new GWParseCommand function.
                            CTProcedure procNewGatewayParseCommand (*pLibrary_, NEWGATEWAYPARSECOMMAND) ;
                            newGatewayParseCommand_ = (CGWParseCommand::fnNewGatewayParseCommand) (procNewGatewayParseCommand.getAddress () ) ;

                            // Create a parse structure.
                            newGatewayParseCommand_ (CTCurrentThread::getThreadId (), task_.getTaskName (), GpProcessMain->getTraceMode (), GpProcessMain->getLogFileList (), pClient_, pGWParseCommand_) ;

                            if (GpProcessMain->getCommandLibrary ().empty () )
                            {   // Save the library name.
                                GpProcessMain->getCommandLibrary () = libraryName ;
                            }

                            // Send command result to admingate.
                            *pClient_ << string ("OK") ;
                            continue ;
                        }
                        catch (CTException&)
                        {
                            // Send command result to admingate.
                            *pClient_ << string ("FAIL") ;
                            throw ;
                        }
                    }
                    else
                    {
                        if (GpProcessMain->getTraceMode () )
                        {
                            CTLockGuard<CTMutex>    guard (GpProcessMain->getLogFileList ().getMutex () ) ;
                            GpProcessMain->getLogFileList () << task_.getTaskName () << ": try to load the library '" << CTLibrary::getSystemFileName (CTLibrary::getBaseFileName (GpProcessMain->getCommandLibrary () ) ) << "'." << endl ;
                        }

                        try
                        {
                            pLibrary_ = new CTLibrary (GpProcessMain->getCommandLibrary () ) ;
                        }
                        catch (CTxalloc&)
                        {   // Error from new.
                            throw CGTaskException (string ("processConnection: fail to load the library '") + CTLibrary::getSystemFileName (CTLibrary::getBaseFileName (GpProcessMain->getCommandLibrary () ) ) + "': new CTLibrary (libraryName) failed.", __FILE__, __LINE__) ;
                        }
                        catch (CTException& e)
                        {
                            throw CGTaskException (string ("processConnection: fail to load the library '") + CTLibrary::getSystemFileName (CTLibrary::getBaseFileName (GpProcessMain->getCommandLibrary () ) ) + "'.\n" + e.what (), __FILE__, __LINE__) ;
                        }

                        // Determinate if there is a new GWParseCommand function.
                        CTProcedure procNewGatewayParseCommand (*pLibrary_, NEWGATEWAYPARSECOMMAND) ;
                        newGatewayParseCommand_ = (CGWParseCommand::fnNewGatewayParseCommand) (procNewGatewayParseCommand.getAddress () ) ;

                        // Create a parse structure.
                        newGatewayParseCommand_ (CTCurrentThread::getThreadId (), task_.getTaskName (), GpProcessMain->getTraceMode (), GpProcessMain->getLogFileList (), pClient_, pGWParseCommand_) ;
                    }
                }

                CGCmd*  pCmd = NULL ;
                pGWParseCommand_->parseCommand (command, pCmd) ;
                try
                {
                    switch (pCmd->getCommand () )
                    {
                    case CGCmd::CMD_NEW_LISTEN_TASK :
                        {   // Create a new listen task.
                            CGGatewayInfo   gatewayInfo ( ( (CGCmdNewListenTask*) pCmd)->getIsAdministrationListen (), ( (CGCmdNewListenTask*) pCmd)->getTargetAddress (), ( (CGCmdNewListenTask*) pCmd)->getPort (), ( (CGCmdNewListenTask*) pCmd)->getLibrary (), ( (CGCmdNewListenTask*) pCmd)->getTargetAddressAutoDetectionMode () ) ;
                            newListenTask (gatewayInfo) ;
                        }
                        break ;
                    case CGCmd::CMD_KILL_TASK_ID :
                        // Stop a task.
                        stopTaskById ( ( (CGCmdKillTaskId*) pCmd)->getTaskId (), ( (CGCmdKillTaskId*) pCmd)->getChildrenAlso () ) ;
                        break ;
                    case CGCmd::CMD_SHUTDOWN :
                        // Shutdown.
                        shutdown ( ( (CGCmdShutDown*) pCmd)->getWithNoWait () ) ;
                        break ;
                    case CGCmd::CMD_TRACE :
                        // Set the trace on or off.
                        GpProcessMain->getTraceMode () = ( (CGCmdTrace*) pCmd)->getOn () ;
                        break ;
                    case CGCmd::CMD_WHO :
                        // Who is connected.
                        * ( (CGCmdWhoWithAssign*) pCmd) = * (task_.getTasksList ().getTasksList ().front () ) ;
                        break ;
                    case CGCmd::CMD_QUIT :
                        // Close connection.
                        bContinue = false ;
                        break ;
                    case CGCmd::CMD_USER :
                        // User command.
                        break ;
                    default :
                        // Error from command.
                        throw CGTaskException (string ("processConnection: command ") + command + "is unknow.", __FILE__, __LINE__) ;
                    }
                    pCmd->getResult () = true ;
                    pGWParseCommand_->commandResult (pCmd) ;
                }
                catch (CGTaskException&)
                {
                    pCmd->getResult () = false ;
                    pGWParseCommand_->commandResult (pCmd) ;
                }
                catch (CTException&)
                {
                    pCmd->getResult () = false ;
                    pGWParseCommand_->commandResult (pCmd) ;
                    throw ;
                }
            }
            catch (CTSocketException&)
            {
                throw ;
            }
        }
        while (bContinue) ;
    }
    catch (CTSocketException& e)
    {
#ifdef  WIN32
        if (e.getNumber () != WSAEINTR)
#endif  // WIN32
#ifdef  UNIX
            if (e.getNumber () != EINTR)
#endif  // UNIX
            {
                throw e ;
            }
    }
    catch (CTException& e)
    {
        throw CTException (e) ;
    }
} // CGAdministrationClient::processConnection

//-----------------------------------------------------------------------------
// CGAdministrationClient::newListenTask
//
// Create a new listen task.
//
// Arguments : gatewayInfo : gateway informations.
// Return    : none.
//-----------------------------------------------------------------------------
void CGAdministrationClient::newListenTask (const CGGatewayInfo& gatewayInfo)
{
    // Search for the task.
    int iIndex = task_.getTasksList ().index ( (CGTasksList::fnTestTask) findTaskFromData, (void*) &gatewayInfo) ;

    if (iIndex == (int) string::npos)
    {   // Create a new listen task.
        if (gatewayInfo.getIsAdministrationListen () )
        {
            char    szPort[6] ;
            sprintf (szPort, "%hu", gatewayInfo.getPort () ) ;

            string  adminPort (szPort) ;
            task_.getTasksList ().newTask (CGTask::TASK_ADMINISTRATION_LISTEN, (void*) &adminPort, task_) ;
        }
        else
        {
            task_.getTasksList ().newTask (CGTask::TASK_GATEWAY_LISTEN, (void*) &gatewayInfo, task_) ;
        }
    }
    else
    {
        if (gatewayInfo.getIsAdministrationListen () == true)
        {
            CGAdministrationListen* pAdministrationListen = (CGAdministrationListen*) (task_.getTasksList ().getTasksList () [iIndex]->getData () ) ;
            if (pAdministrationListen->getSocket ().getPort () == gatewayInfo.getPort () )
            {   // Reinitialize the listen task.
                task_.getTasksList ().reinitializeTask (iIndex, true) ;
            }
            else
            {
                throw CGTaskException ("newListenTask: gateway configuration isn't the same.", __FILE__, __LINE__) ;
            }
        }
        else
        {
            CGGatewayListen*    pGatewayListen = (CGGatewayListen*) (task_.getTasksList ().getTasksList () [iIndex]->getData () ) ;
            if (   (pGatewayListen->getGatewayInfo ().getIsAdministrationListen ()          == gatewayInfo.getIsAdministrationListen () )
                    && (pGatewayListen->getGatewayInfo ().getTargetAddress ()                   == gatewayInfo.getTargetAddress () )
                    && (pGatewayListen->getGatewayInfo ().getPort ()                            == gatewayInfo.getPort () )
                    && (pGatewayListen->getGatewayInfo ().getLibrary ()                         == gatewayInfo.getLibrary () )
                    && (pGatewayListen->getGatewayInfo ().getTargetAddressAutoDetectionMode ()  == gatewayInfo.getTargetAddressAutoDetectionMode () ) )
            {   // Reinitialize the listen task.
                task_.getTasksList ().reinitializeTask (iIndex, true) ;
            }
            else
            {
                throw CGTaskException ("newListenTask: gateway configuration isn't the same.", __FILE__, __LINE__) ;
            }
        }
    }
} // CGAdministrationClient::newListenTask

//-----------------------------------------------------------------------------
// CGAdministrationClient::stopTaskById
//
// Stop a task.
//
// Arguments : iTaskId      : Task identifier.
//             bClientsAlso : kill chidren also.
// Return    : none.
//-----------------------------------------------------------------------------
void CGAdministrationClient::stopTaskById (const int iTaskId, const bool bClientsAlso /* = false */)
{
    // Verify task to kill is not the control task.
    int i = 0;
    for (i=0; i<(int)task_.getTasksList ().getTasksList ().size(); i++)
    {
        if (findTaskFromId(task_.getTasksList ().getTasksList ()[i],  &iTaskId))
            break ;
    }

    CGTask* pTask = task_.getTasksList ().getTasksList () [i];

    if (pTask == NULL)
    {
        throw CGTaskException ("stopTaskById: unknow task.", __FILE__, __LINE__) ;
    }

    if (pTask->getType () == CGTask::TASK_CONTROL)
    {
        throw CGTaskException ("stopTaskById: can't kill the control task.", __FILE__, __LINE__) ;
    }

    // Verify task to kill is not the current task.
    if (task_.getTaskId () == iTaskId)
    {
        throw CGTaskException ("stopTaskById: can't kill the current task.", __FILE__, __LINE__) ;
    }

    int iFatherId = 0 ;
    int iIndex ;

    // Remove temporaly from task list and from father chidren list.
    for (CGTaskStlList::iterator itr = task_.getTasksList ().getTasksList ().begin(); itr != task_.getTasksList ().getTasksList ().end(); itr++)
    {
        if (*itr == &task_)
        {
            task_.getTasksList ().getTasksList ().erase(itr);
            break ;
        }
    }
    if (task_.getFather () )
    {
        iFatherId = task_.getFather () ->getTaskId () ;
        //      task_.getFather () ->getChildrenList ().remove ( (CGTasksList::fnTestTask) findTaskFromPointer, &task_) ;

        for (CGTaskStlList::iterator itr = task_.getFather () ->getChildrenList ().begin(); itr != task_.getFather () ->getChildrenList ().end(); itr++)
        {
            if (*itr == &task_)
            {
                task_.getFather () ->getChildrenList ().erase(itr);
                break ;
            }
        }

        for (size_t i = 0 ; i < task_.getChildrenList ().size () ; ++ i)
        {
            task_.getFather () ->getChildrenList ().push_back (task_.getChildrenList ()[i]) ;
            task_.getChildrenList ()[i]->setFather (task_.getFather () ) ;
        }
    }

    // Search for the task.
    iIndex = task_.getTasksList ().index ( (CGTasksList::fnTestTask) findTaskFromId, (void*) &iTaskId) ;

    if (iIndex != (int) string::npos)
    {   // Stop the task.
        task_.getTasksList ().killTask (iIndex, bClientsAlso) ;
    }

    // Restore in task list and in father chidren list.
    if (task_.getFather () )
    {
        // Clean the children list.
        int i=0;
        for (CGTaskStlList::iterator itrChild = task_.getChildrenList ().begin();itrChild != task_.getChildrenList ().end(); itrChild++, i++)
        {
            bool bFound = false ;
            for (CGTaskStlList::iterator itr = task_.getTasksList ().getTasksList ().begin(); itr != task_.getTasksList ().getTasksList ().end(); itr++)
            {
                if (*itr == *itrChild)
                {
                    bFound = true ;
                    break ;
                }
            }
            if (!bFound)
            {
                task_.getChildrenList ().erase(itrChild);
                --i ;
                itrChild = task_.getChildrenList ().begin();
                itrChild += i;
            }
            else
            {
                for (CGTaskStlList::iterator itr2 = task_.getChildrenList ()[i]->getFather () ->getChildrenList ().begin(); itr2 != task_.getChildrenList ()[i]->getFather () ->getChildrenList ().end(); itr2++)
                {
                    if (*itr2 == task_.getChildrenList ()[i])
                    {
                        task_.getChildrenList ()[i]->getFather () ->getChildrenList ().erase(itr2) ;
                        task_.getChildrenList ()[i]->setFather (&task_) ;
                        break ;
                    }
                }
            }
        }
        // Search for the parent task.
        iIndex = task_.getTasksList ().index ( (CGTasksList::fnTestTask) findTaskFromId, (void*) &iFatherId) ;
        if (iIndex == string::npos)
        {
            task_.setFather (task_.getTasksList ().getTasksList ().front () ) ;
            task_.getFather () ->getChildrenList ().push_back (&task_) ;
        }
        else
        {
            task_.getFather () ->getChildrenList ().push_back (&task_) ;
        }
    }
    task_.getTasksList ().getTasksList ().push_back (&task_) ;
} // CGAdministrationClient::stopTaskById

//-----------------------------------------------------------------------------
// CGAdministrationClient::shutdown
//
// Stop the gateway.
//
// Arguments : bWithNoWait : shutdown now if true !
// Return    : none.
//-----------------------------------------------------------------------------
void CGAdministrationClient::shutdown (bool bWithNoWait /* = false */)
{
    // Get the tasks list.
    CGTasksList&    tasksList = task_.getTasksList () ;

    // Search for the gateway listen tasks.
    CGTask::enumTaskType    eTaskType = CGTask::TASK_GATEWAY_LISTEN ;
    int                     iIndex = tasksList.index ( (CGTasksList::fnTestTask) findTaskFromType, (void*) &eTaskType) ;

    while (iIndex != (int) string::npos)
    {   // Stop the gateway listen task and its children.
        tasksList.killTask (iIndex, bWithNoWait) ;

        // Search for the next gateway listen task.
        iIndex = tasksList.index ( (CGTasksList::fnTestTask) findTaskFromType, (void*) &eTaskType) ;
    }

    // Search for the administration listen tasks.
    eTaskType = CGTask::TASK_ADMINISTRATION_LISTEN ;
    iIndex = tasksList.index ( (CGTasksList::fnTestTask) findTaskFromType, (void*) &eTaskType) ;

    while (iIndex != (int) string::npos)
    {   // Stop the administration listen task and its children.
        tasksList.killTask (iIndex, bWithNoWait) ;

        // Search for the next gateway listen task.
        iIndex = tasksList.index ( (CGTasksList::fnTestTask) findTaskFromType, (void*) &eTaskType) ;
    }

    GpProcessMain->waitForTasks () = !bWithNoWait ;

    GpProcessMain->abort () ;
} // CGAdministrationClient::shutdown

//-----------------------------------------------------------------------------
// CGAdministrationClient::findTaskFromData
//
// Find a task from data.
//
// Arguments : pTask        : pointer to task..
//             pGatewayInfo : pointer to gateway informations.
// Return    : bEqual       : TRUE if the task is found.
//-----------------------------------------------------------------------------
bool CGAdministrationClient::findTaskFromData (const CGTask* pTask, const CGGatewayInfo* pGatewayInfo)
{
    bool    bEqual = FALSE ;

    if ( (pTask) && (pTask->isValid () ) )
    {
        if (pGatewayInfo->getIsAdministrationListen () == true)
        {
            if (pTask->getType () == CGTask::TASK_ADMINISTRATION_LISTEN)
            {
                CGAdministrationListen* pAdministrationListen = (CGAdministrationListen*) (pTask->getData () ) ;
                bEqual = (pAdministrationListen->getSocket ().getPort () == pGatewayInfo->getPort () ) ;
            }
        }
        else
        {
            if (pTask->getType () == CGTask::TASK_GATEWAY_LISTEN)
            {
                CGGatewayListen*    pGatewayListen = (CGGatewayListen*) (pTask->getData () ) ;
                bEqual = (pGatewayListen->getGatewayInfo ().getPort () == pGatewayInfo->getPort () ) ;
            }
        }
    }

    // Return bEqual.
    return bEqual ;
} // CGAdministrationClient::findTaskFromData

//-----------------------------------------------------------------------------
// CGAdministrationClient::findTaskByPointer
//
// Find a task from pointer.
//
// Arguments : pTask       : pointer to task..
//             pSearchTask : pointer to search task.
// Return    : bEqual      : TRUE if the task is found.
//-----------------------------------------------------------------------------
bool CGAdministrationClient::findTaskFromPointer (const CGTask* pTask, const CGTask* pSearchTask)
{
    return pTask == pSearchTask ;
} // CGAdministrationClient::findTaskFromPointer

//-----------------------------------------------------------------------------
// CGAdministrationClient::findTaskFromId
//
// Find a task from identifier.
//
// Arguments : pTask   : pointer to task..
//             pTaskId : pointer to task identifier.
// Return    : bEqual  : TRUE if the task is found.
//-----------------------------------------------------------------------------
bool CGAdministrationClient::findTaskFromId (const CGTask* pTask, const int* pTaskId)
{
    bool    bEqual = FALSE ;

    if ( (pTask) && (pTask->isValid () ) )
    {
        bEqual = (pTask->getTaskId () == *pTaskId) ;
    }

    // Return bEqual.
    return bEqual ;
} // CGAdministrationClient::findTaskFromId

//-----------------------------------------------------------------------------
// CGAdministrationClient::findTaskFromType
//
// Find a task from type.
//
// Arguments : pTask      : pointer to task..
//             peTaskType : pointer to task type.
// Return    : bEqual     : TRUE if the task is found.
//-----------------------------------------------------------------------------
bool CGAdministrationClient::findTaskFromType (const CGTask* pTask, const CGTask::enumTaskType* peTaskType)
{
    bool    bEqual = FALSE ;

    if ( (pTask) && (pTask->isValid () ) )
    {
        bEqual = (pTask->getType () == *peTaskType) ;
    }

    // Return bEqual.
    return bEqual ;
} // CGAdministrationClient::findTaskFromType

//-----------------------------------------------------------------------------
// CGAdministrationClient::abort
//
// Abort Routine.
//
// Arguments : pStatus : pointer to the exit status.
// Return    : none.
//-----------------------------------------------------------------------------
void CGAdministrationClient::abort (void*)
{
    // Delete the CGAdministrationClient pointer.
    ( (CGAdministrationClient*) CTCurrentThread::getData () )->~CGAdministrationClient () ;
} // CGAdministrationClient::abort

///////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////// End of Class CGAdministrationClient //
///////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////// Globals Functions //
///////////////////////////////////////////////////////////////////////////////

void administrationClientTask (CGTask& task)
{
    try
    {
        if (GpProcessMain->getTraceMode () )
        {
            CTLockGuard<CTMutex>    guard (GpProcessMain->getLogFileList ().getMutex () ) ;
            GpProcessMain->getLogFileList () << task.getTaskName () << ": new administration client task." << endl ;
        }

        // Destruction of this object must occur before killing the task.
        CGAdministrationClient  administrationClient (task) ;
        administrationClient.init () ;
        administrationClient.processConnection () ;

        if (GpProcessMain->getTraceMode () )
        {
            CTLockGuard<CTMutex>    guard (GpProcessMain->getLogFileList ().getMutex () ) ;
            GpProcessMain->getLogFileList () << task.getTaskName () << ": administration client task terminated." << endl ;
        }
    }
    catch (CTException& e)
    {
        if (GpProcessMain->getTraceMode () )
        {
            CTLockGuard<CTMutex>    guard (GpProcessMain->getLogFileList ().getMutex () ) ;
            GpProcessMain->getLogFileList () << task.getTaskName () << ": !! " << e.what () << endl ;
            GpProcessMain->getLogFileList () << task.getTaskName () << ": administration client task terminated." << endl ;
        }
        else
        {
            CTLockGuard<CTMutex>    guard (GpProcessMain->getLogFileList ().getMutex () ) ;
            GpProcessMain->getLogFileList () << "!! " << e.what () << endl ;
        }
    }
    catch (...)
    {
        if (GpProcessMain->getTraceMode () )
        {
            CTLockGuard<CTMutex>    guard (GpProcessMain->getLogFileList ().getMutex () ) ;
            GpProcessMain->getLogFileList () << task.getTaskName () << ": !! unknown exception." << endl ;
            GpProcessMain->getLogFileList () << task.getTaskName () << ": administration client task terminated." << endl ;
        }
        else
        {
            CTLockGuard<CTMutex>    guard (GpProcessMain->getLogFileList ().getMutex () ) ;
            GpProcessMain->getLogFileList () << "!! unknown exception." << endl ;
        }
    }
    if (task.initializationIsTerminated () )
    {   // Remove the task from the list.
        task.invalidate () ;

        for (int iIndex=0; iIndex<(int)task.getTasksList ().getTasksList ().size(); iIndex++)
        {
            if (task.getTasksList ().getTasksList ()[iIndex] == &task)
            {
                task.getTasksList ().killTask (iIndex) ;
                break ;
            }
        }
    }
    else
    {   // Tell the control task that initialization is terminated.
        task.initializationTerminated (false) ;
    }
} // administrationClientTask


///////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////// End of Globals Functions //
///////////////////////////////////////////////////////////////////////////////
