/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

#ifdef  WIN32
#pragma warning(disable : 4100 4201 4214 4209 4115 4512 4514 4516 4705 4710)
#endif

#include  "cstof.hpp"

CTTime              CTofReference::timePreviousWrite_ ;
stringStlMap		CTofReference::scriptCookieMap_ ;
stringStlMap		CTofReference::fakedCookieMap_ ;
extern string		gListeDocuments ;
extern int          giSpid ;
bool				CTofReference::bAutomaticModel_ = true ;
bool				CTofReference::bAddCookieComment_ = true ;

void CTofReference::addContext(CContext *context)
{
    contextList_.add(context);
}

void CTofReference::removeContext(CContext *context)
{
    contextList_.remove(context);
}

bool CTofReference::analyse(const string & conId, const string & tofFileName, const string & strMode, const string & strCommand, const string & strMessage)
{
    enumSelectStatus status;
    bool oneTof = FALSE;

    if (strCommand == "record")
        status = ESL_RECORD;
    else if (strCommand == "stop")
        status = ESL_STOP;
    else if (strCommand == "pause")
        status = ESL_PAUSE;
    else if (strCommand == "resume")
        status = ESL_RESUME;
    else if (strCommand == "comment")
        status = ESL_COMMENT;
    else if (strCommand == "starttimer")
        status = ESL_STARTTIMER;
    else if (strCommand == "endtimer")
        status = ESL_ENDTIMER;

    if (strMode == "only")
        oneTof = TRUE;

    return analyse(tofFileName, oneTof, status, strMessage);
}

bool CTofReference::incTimePreviousWrite(unsigned long ulMillisecond)
{
    CTLockGuard <CTMutex> guard(tofMutex_);

	timePreviousWrite_ += ulMillisecond ;
	return true ;
}

bool CTofReference::analyse (const string & tofFileName, bool oneTof, enumSelectStatus selectStatus, const string & message)
{
    CTLockGuard <CTMutex> guard(tofMutex_);
    CContext    *context    =   NULL;
    CContextStlList  liste = contextList_.getList();

    CContextStlList :: iterator ilistedeb=liste.begin();
    size_t lastSlash = string::npos;

    switch(selectStatus)
    {
        //
        // RECORD Action
        //
    case ESL_RECORD :
        gListeDocuments = "" ;
        // The precedent action was record
        if (selectStatus_ == ESL_RECORD)
            break;
        selectStatus_ = ESL_RECORD;
        tofFileName_ = tofFileName;
        bOutOfOrder_ = false ;
        /* tof Timer Name from tof File Name */
        lastSlash = tofFileName_.find_last_of("\\");
        if (lastSlash != string::npos)
        {
            tofTimerName_ = tofFileName_.substr(lastSlash + 1, tofFileName_.length() - lastSlash - 5);
        }
        else
        {
            tofTimerName_ = tofFileName_.substr(0, tofFileName_.length() - 4);
        }

        // We want one TOF file for all connections
        oneTof_ = oneTof;

        // Tof Title
        tofTitle_ = message;

        // Create the common TOF file
        if (oneTof_)
            createTofFile(tofFileName_);

        //
        // If OneTof == TRUE, AssociateTofFile does not create files,
        // else, it creates one file per connecion.
        //

        while (ilistedeb != liste.end())
        {
            if ((*ilistedeb)->getConnectId != 0)
                associateTofFile(*ilistedeb);
            ilistedeb++;
        }
        break;
        //
        // STOP Action
        //
    case ESL_STOP :
        // the precedent action was stop
        if (selectStatus_ == ESL_STOP)
            break;
        CGWHttp::Concatenate() ;

        selectStatus_ = ESL_STOP;

        tofFileName_ = "";
        tofTitle_ = "";
        spid_ = 0;
        timePreviousWrite_.reset() ;
        //
        // Before, there were one TOF far all connections,
        // We close this file.
        //
        if (oneTof_)
        {
            closeTofFile();
            oneTof_ = FALSE;

            while (ilistedeb != liste.end())
            {
                (*ilistedeb)->setTofStream(NULL);
                ilistedeb++;
            }


        }
        //
        // Before, there were one TOF per connection,
        // We close all these files.
        //
        else
        {
            while (ilistedeb != liste.end())
            {
                if ((*ilistedeb)->getConnectId != 0)
                    closeTofFile(*ilistedeb);
                ilistedeb++;
            }
        }
        break;
        //
        // PAUSE Action
        //
    case ESL_PAUSE :
        // the precedent action was pause
        if (selectStatus_ == ESL_PAUSE)
            break;
        selectStatus_ = ESL_PAUSE;
        break;
        //
        // RESUME Action
        //
    case ESL_RESUME :
        // the precedent action was pause
        if (selectStatus_ == ESL_RECORD)
            break;
        selectStatus_ = ESL_RECORD;
        break;
        //
        // COMMENT Action
        //
    case ESL_COMMENT :
		{ // stop compiler complainig about declaring vars in here
        if (message == "")
            break;
		string tofseparator(SEPARATOR);
		string sanitizedmessage(message);
		string::size_type pos = 0;
		// erase characters until we have no instance of the SEPERATOR in message
		while((pos=sanitizedmessage.find(tofseparator, pos)) != string::npos)
            sanitizedmessage.erase(pos, 1);  
        if (oneTof_)
        {
            writeCommentInTofFile(NULL, "USC:" + sanitizedmessage);
            break;
        }
        while (ilistedeb != liste.end())
        {
            writeCommentInTofFile((*ilistedeb), sanitizedmessage);
            ilistedeb++;
        }
        break;
		}
        //
        // STARTTIMER Action
        //
    case ESL_STARTTIMER :
        if (message == "")
            break;
        if (oneTof_)
        {
            writeStartTimerInTofFile(NULL, message);
            break;
        }
        while (ilistedeb != liste.end())
        {
            writeStartTimerInTofFile((*ilistedeb), message);
            ilistedeb++;
        }
        break;
        //
        // STARTTIMER Action
        //
    case ESL_ENDTIMER :
        if (message == "")
            break;
        if (oneTof_)
        {
            writeEndTimerInTofFile(NULL, message);
            break;
        }
        while (ilistedeb != liste.end())
        {
            writeEndTimerInTofFile((*ilistedeb), message);
            ilistedeb++;
        }
        break;
    }

    return(TRUE);
}


bool CTofReference::createTofFile(string & tofFileName)
{
    CTLockGuard <CTMutex > guard(tofMutex_);

    time_t          ltime;
    struct tm       *stm;
    char            Msg[255];
    char            day[12][4] = {  "jan","feb","mar","apr","may","jun",
                                    "jul","aug","sep","oct","nov","dec"};

    tofStream_.open(tofFileName.data(), ios::out | ios::binary);

    if (tofStream_.fail() != 0)
    {
        char zBuf[255] ;
        FormatMessage (FORMAT_MESSAGE_FROM_SYSTEM, NULL, GetLastError (), MAKELANGID (LANG_NEUTRAL, SUBLANG_DEFAULT), (LPTSTR) zBuf, sizeof (zBuf), NULL) ;
        string message = "";
        message += "cannot open ";
        message += tofFileName;
        message += "\r\n";
        message += zBuf;
        throw message;
    }

    tofStream_ << "tof.5\r\n" \
    "dbtype 1\r\n" \
    "dbversion 10\r\n";

    time(&ltime);
    stm = localtime(&ltime);

    sprintf(Msg,    "date %02.2d-%s-%d %02.2d:%02.2d:%02.2d\r\n",
            stm->tm_mday, day[stm->tm_mon], stm->tm_year + 1900,
            stm->tm_hour, stm->tm_min, stm->tm_sec);
    if (serverName_ != "FILEGate" )
        tofStream_ << Msg;

    if (tofTitle_ != "")
        tofStream_ << "name " << tofTitle_ << "\r\n";

    tofStream_ << "\r\n";

    tofStream_ << "STimer\r\n" << "name T_" << tofTimerName_ << "\r\n\r" << std::endl;

    return TRUE;
}

bool CTofReference::createTofFile(CContext *context, string & tofFileName, string & tofTimerName)
{
    CTLockGuard <CTMutex > guard(tofMutex_);

    time_t          ltime;
    struct tm       *stm;
    char            Msg[255];
    char            day[12][4] = {  "jan","feb","mar","apr","may","jun",
                                    "jul","aug","sep","oct","nov","dec"};

    ofstream * f = context->getTofStream();
    f = new ofstream;
    f->open(tofFileName.data(), ios::out | ios::binary);
    context->setTofStream(f);
    if (f->fail() != 0)
    {
        char zBuf[255] ;
        FormatMessage (FORMAT_MESSAGE_FROM_SYSTEM, NULL, GetLastError (), MAKELANGID (LANG_NEUTRAL, SUBLANG_DEFAULT), (LPTSTR) zBuf, sizeof (zBuf), NULL) ;
        string message = "";
        message += "cannot open ";
        message += tofFileName;
        message += "\r\n";
        message += zBuf;
        throw message;
    }

    *f << "tof.5\r\n" \
    "dbtype 1\r\n" \
    "dbversion 10\r\n";

    time(&ltime);
    stm = localtime(&ltime);

    sprintf(Msg,    "date %02.2d-%s-%d %02.2d:%02.2d:%02.2d\r\n",
            stm->tm_mday, day[stm->tm_mon], stm->tm_year + 1900,
            stm->tm_hour, stm->tm_min, stm->tm_sec);
    if (serverName_ != "FILEGate" )
        *f << Msg;

    if (tofTitle_ != "")
        *f << "name " << tofTitle_ << "\r\n";

    *f << "\r\n";

    *f << "STimer\r\n" << "name T_" << tofTimerName << "\r\n\r" << std::endl;

    return TRUE;
}

void CTofReference::associateTofFile(CContext *context)
{
    CTLockGuard <CTMutex > guard(tofMutex_);
    string tofFileName;
    // One TOF file for all connections
    if (oneTof_)
    {
        context->setTofStream(&tofStream_);
    }
    // One TOF file per connection
    else
    {
        if (selectStatus_ != ESL_STOP)
        {
            //
            // The first file has the specified TOF file name
            // others have the name : FileName + spid number
            //
            spid_ ++;

            string tofTimerName = tofTimerName_;

            if (spid_ == 1)
                tofFileName = tofFileName_;
            else
            {
                char bufTmp[10];
                char fileNameTmp[256];

                memset(fileNameTmp, 0, sizeof(fileNameTmp));
                sprintf(bufTmp, "%d", context->getConnectId());
                strncpy(fileNameTmp, tofFileName_.data(), tofFileName_.length() - 4);
                tofFileName = fileNameTmp;
                tofFileName += bufTmp;
                tofFileName += ".tof";
                tofTimerName += bufTmp;
            }
            createTofFile(context, tofFileName, tofTimerName);
        }
    }
}

void CTofReference::closeTofFile(CContext *context)
{
    CTLockGuard <CTMutex > guard(tofMutex_);

    if (!oneTof_)
    {
        ofstream * f = context->getTofStream();
        if (f != NULL)
        {
            *f << "\r\nSynchronize\r\n\r\n" ;
            if (context->getConnectId() == 1)
                *f << "\r\nETimer\r\n" << "name T_" << tofTimerName_ << "\r\n\r" << std::endl;
            else
                *f << "\r\nETimer\r\n" << "name T_" << tofTimerName_ << context->getConnectId() << "\r\n\r" << std::endl;

            if (f->is_open() != 0)
                f->close();
            delete f;
            f = NULL;
        }
        context->setTofStream(NULL);
    }
    else
    {
		
        tofStream_ << "\r\nSynchronize\r\n\r\n" ;

		// Close the timers added manually during the capture and not closed 
		// yet.
		stringStlMap::iterator itrMap;
		for (itrMap = timerVarMap_.begin(); itrMap != timerVarMap_.end(); ++itrMap)
		{
			if (itrMap->second == "STARTED")
				tofStream_ << "\r\nETimer\r\n" \
					       << "name " << itrMap->first << "\r" << std::endl;
		}
        
		tofStream_ << "\r\nETimer\r\n" << "name T_" << tofTimerName_ << "\r" << std::endl;
        tofStream_ << "\r\nTimdef\r\n" \
        << "name T_" << tofTimerName_ << "\r" << std::endl ;

		// declare timer variables
		for (itrMap = timerVarMap_.begin(); itrMap != timerVarMap_.end(); ++itrMap)
		{
            tofStream_ << "\r\nTimdef\r\n" \
				       << "name " << itrMap->first << "\r" << std::endl;
		}
		timerVarMap_.clear();

        string varName ;
        stringStlList::iterator iter ;
		// declare the cookie variables
		for (iter = varList_.begin(); iter != varList_.end(); iter++)
        {
			string vName = (*iter);
            tofStream_ << "\r\nVardef\r\n" \
            << "name " << vName << "\r\n" ;

			if (vName.substr(0, 8) != "S_cookie")
	            tofStream_ << "option 4225\r\n" \
				<< "size 1024\r" << std::endl;
			else
			{
				// Declare s_cookie variables as constants
				// Add the const entry in the tof object
				// format :
				// const cookie_value
				for (stringStlMap::iterator scmap = scriptCookieMap_.begin(); scmap != scriptCookieMap_.end(); ++scmap)
				{
					if (scmap->second == vName)
					{
						tofStream_ << "const " << scmap->first << "\r" << std::endl;
						break ;
					}
				}
			}
        }
        varList_.clear();

		// declare the authentication variables
		for (iter = authVarList_.begin(); iter != authVarList_.end(); iter++)
		{
            tofStream_ << "\r\nVardef\r\n" \
            << "name " << (*iter) << "\r\n" \
            << "option 4225\r\n" \
            << "size 256\r" << std::endl;
        }
        authVarList_.clear();
        authVarMap_.clear();

		// remove the faked cookies
		for (itrMap = fakedCookieMap_.begin(); itrMap != fakedCookieMap_.end(); itrMap++)
		{
			string fakedCookie = itrMap->first ;
			int iIndexEqual = fakedCookie.find_first_of("=") ;
			int iIndexSemiColon = fakedCookie.find_first_of(";") ;
			if (iIndexEqual != string::npos)
			{
				if (iIndexSemiColon != string::npos)
					fakedCookie.replace(iIndexEqual + 1, iIndexSemiColon - iIndexEqual - 1, ";expires=\"Tuesday, 31-Dec-1980 12:00:00 GMT\"") ;
				else
					fakedCookie.replace(iIndexEqual + 1, fakedCookie.size() - iIndexEqual - 1, ";expires=\"Tuesday, 31-Dec-1980 12:00:00 GMT\"") ;
			}
			InternetSetCookie(itrMap->second.c_str(), NULL, fakedCookie.c_str());
#ifdef _DEBUG
			DWORD dwSizeAcol ;
			if (InternetGetCookie(itrMap->second.c_str(), NULL, NULL, &dwSizeAcol))
			{
				char * lpszDataAcol = new char[dwSizeAcol];
				InternetGetCookie(itrMap->second.c_str(), NULL, lpszDataAcol, &dwSizeAcol);
			}
#endif
		}

		// Clear the cookie maps.
		scriptCookieMap_.clear();
		fakedCookieMap_.clear();

        if (context == NULL)
            tofStream_.close();
    }
}

void CTofReference::writeCommentInTofFile(CContext *context, const string & message)
{
    CTLockGuard <CTMutex > guard(tofMutex_);

    if (selectStatus_ == ESL_RECORD)
    {
        if (oneTof_)
            tofStream_ << "\r\nComment " << SEPARATOR << "\r\n" << message << " " << SEPARATOR << "\r" << std::endl ;
        else
        {
            ofstream * f = context->getTofStream();
            if (f == NULL)
                return;
            *f << "\r\nComment " << SEPARATOR << "\r\n" << message << " " << SEPARATOR << "\r" << std::endl ;
        }
    }
}

void CTofReference::writeStartTimerInTofFile(CContext *context, const string & timerName)
{
    CTLockGuard <CTMutex > guard(tofMutex_);

    if (selectStatus_ == ESL_RECORD)
    {
		// Declare the timer name in the timer list
		timerVarMap_.insert(stringStlMap::value_type(timerName, "STARTED"));
        if (oneTof_)
            tofStream_ << "\r\nSTimer\r\nname " <<  timerName << "\r" << std::endl ;
        else
        {
            ofstream * f = context->getTofStream();
            if (f == NULL)
                return;
			*f << "\r\nSTimer\r\nname " <<  timerName << "\r" << std::endl ;
        }
    }
}

void CTofReference::writeEndTimerInTofFile(CContext *context, const string & timerName)
{
    CTLockGuard <CTMutex > guard(tofMutex_);

    if (selectStatus_ == ESL_RECORD)
    {
		// check if the timer has been started
		stringStlMap::iterator itrMap ;
		if ((itrMap = timerVarMap_.find(timerName)) == timerVarMap_.end())
			return ;
		itrMap->second = "ENDED" ;
        if (oneTof_)
            tofStream_ << "\r\nETimer\r\nname " <<  timerName << "\r" << std::endl ;
        else
        {
            ofstream * f = context->getTofStream();
            if (f == NULL)
                return;
			*f << "\r\nETimer\r\nname " <<  timerName << "\r" << std::endl ;
        }
    }
}

string CTofReference::modifyStringAsScl(string input)
{
    int iIndex = string::npos ;
    string output = "";
    char zBuf[10] ;
    for(iIndex=0; iIndex<(int)input.length(); iIndex++) {
        if(((unsigned char)input[(size_t)iIndex] >= (unsigned char)0x7E) ||
           ((unsigned char)input[(size_t)iIndex] < (unsigned char)0x20))
        {
            switch((unsigned char)input[(size_t)iIndex]) {
              case (unsigned char)0x07:
                sprintf(zBuf, "~<BEL>");
                break;
              case (unsigned char)0x08:
                sprintf(zBuf, "~<BS>");
                break;
              case (unsigned char)0x09:
                sprintf(zBuf, "~<HT>");
                break;
              case (unsigned char)0x0A:
                sprintf(zBuf, "~<LF>");
                break;
              case (unsigned char)0x0B:
                sprintf(zBuf, "~<VT>");
                break;
              case (unsigned char)0x0C:
                sprintf(zBuf, "~<FF>");
                break;
              case (unsigned char)0x0D:
                if(((iIndex+1)<((int)input.length())) &&
                   ((unsigned char)input[(size_t)iIndex+1] == (unsigned char)0x0A)) {
                    sprintf(zBuf, "^J");
                    iIndex++;
                } else {
                    sprintf(zBuf, "~<CR>");
                }
                break;
              case (unsigned char)0x1B:
                sprintf(zBuf, "~<ESC>");
                break;
              case (unsigned char)0x7F:
                sprintf(zBuf, "~<DEL>");
                break;
              case (unsigned char)0xFF:
                sprintf(zBuf, "~<0FF>");
                break;
              default:
                sprintf(zBuf, "~<%2.2X>", (unsigned char)input[(size_t)iIndex]);
                break;
            }      
            output += zBuf ;
        } else if((unsigned char)input[(size_t)iIndex] == (unsigned char)0x22) {
            output += "~<22>";
        } else if((unsigned char)input[(size_t)iIndex] == (unsigned char)0x27) {
            output += "~<27>";
        } else if((unsigned char)input[(size_t)iIndex] == (unsigned char)0x5E) {
            output += "~^";
        } else {
            output += input[(size_t)iIndex] ;
        }
    }

    return output;
}

string CTofReference::modifyBodyAsScl(char * input , int iBodySize)
{
    string output = "";
    char zBuf[10] ;
    for (int iIndex=0; iIndex<iBodySize; iIndex++)
    {
        if(((unsigned char)input[(size_t)iIndex] >= (unsigned char)0x7E) ||
           ((unsigned char)input[(size_t)iIndex] < (unsigned char)0x20))
        {
            switch((unsigned char)input[(size_t)iIndex]) {
              case (unsigned char)0x07:
                sprintf(zBuf, "~<BEL>");
                break;
              case (unsigned char)0x08:
                sprintf(zBuf, "~<BS>");
                break;
              case (unsigned char)0x09:
                sprintf(zBuf, "~<HT>");
                break;
              case (unsigned char)0x0A:
                sprintf(zBuf, "~<LF>");
                break;
              case (unsigned char)0x0B:
                sprintf(zBuf, "~<VT>");
                break;
              case (unsigned char)0x0C:
                sprintf(zBuf, "~<FF>");
                break;
              case (unsigned char)0x0D:
                sprintf(zBuf, "~<CR>");
                break;
              case (unsigned char)0x1B:
                sprintf(zBuf, "~<ESC>");
                break;
              case (unsigned char)0x7F:
                sprintf(zBuf, "~<DEL>");
                break;
              case (unsigned char)0xFF:
                sprintf(zBuf, "~<0FF>");
                break;
              default:
                sprintf(zBuf, "~<%2.2X>", (unsigned char)input[(size_t)iIndex]);
                break;
            }      
            output += zBuf;
        } else if((unsigned char)input[(size_t)iIndex] == (unsigned char)0x22) {
            output += "~<22>";
        } else if((unsigned char)input[(size_t)iIndex] == (unsigned char)0x27) {
            output += "~<27>";
        } else if((unsigned char)input[(size_t)iIndex] == (unsigned char)0x5E) {
            output += "~^";
        } else {
            output += input[(size_t)iIndex];
        }
    }
    return output ;
}

void CTofReference::writeConnectInTofFile(CContext *context)
{
    CTLockGuard <CTMutex > guard(tofMutex_);

    if (selectStatus_ == ESL_RECORD)
    {
        ofstream * f = context->getTofStream();
        if (f == NULL)
            return;

        try
        {
            CTTime timeWrite;
            timeWrite.now();
            CTTime idleTime ;

            if (serverName_ != "FILEGate" && !timePreviousWrite_.isNull() )
                idleTime = timeWrite - timePreviousWrite_;
            if (idleTime != 0)
            {
                *f << "Wait\r\n" ;
                *f << "tdelay " << (unsigned long)idleTime << "\r\n\r" << std::endl ;
            }

            *f << "Connect\r\n" \
            "server \"" << context->getServerName() << ":" << context->getPort() << "\"\r\n" \
            "conid " << context->getConnectId() << "\r\n\r" << std::endl;
        }
        catch(...)
        {
            perror("");
        }
    }
    timePreviousWrite_.now() ;
    eObjectType_ = EOT_CONNECT ;
}

void CTofReference::writeDisconnectInTofFile(CContext *context, bool bFullDocument)
{
    CTLockGuard <CTMutex > guard(tofMutex_);

    if (selectStatus_ == ESL_RECORD)
    {
        ofstream * f = context->getTofStream();
        if (f == NULL)
            return;

		// For conid on which no  request has been written in the tof,
		// don't write a disconnect.
		if (context->getTofRequestId() == 0)
			return ;
		// For disconnect with cancel, add a WAIT before the disconnect
		// to be sure to load a part of the document. Indeed, when
		// a disconnect with cancel is executed, it disconnects 
		// without waiting for the document to be loaded.
		// The wait will let the injector receiving some 
		// of the documents before canceling it.
        if (bFullDocument == FALSE)
		{
			CTTime timeWrite;
			timeWrite.now();
			CTTime idleTime ;

			if (serverName_ != "FILEGate" && !timePreviousWrite_.isNull() )
				idleTime = timeWrite - timePreviousWrite_;

			if (idleTime != 0)
			{
				*f << "Wait\r\n" ;
				*f << "tdelay " << (unsigned long)idleTime << "\r" << std::endl ;
			}
		}

        *f << "Disconnect\r\n" ;
        *f << "conid " << context->getConnectId() << "\r\n" ;
        if (bFullDocument == FALSE)
            *f << "options 1\r\n";
        else
            *f << "options 0\r\n";

        *f << "\r" << std::endl ;
    }
    eObjectType_ = EOT_DISCONNECT ;
}


string CTofReference::modelCookies(CContext *context, string header, bool & bAddQuotes)
{
    DWORD dwSize = 0;               // variable to get the buffer size needed
    LPSTR cookieAllFaked = NULL;   
    LPSTR cookieAllRequest = NULL; 
	int iBeginCookie = 0 ;
	int iLengthCookie = 0 ;
	string variableName ;
	string cookieValue ;
 
    if ((iBeginCookie  = CGWHttp::find_in_string(header,"Cookie:",false, 0)) == string::npos)
		return header ;
	iBeginCookie += strlen("Cookie:");

	if (InternetGetCookie(context->getRequestUrl().c_str(), NULL, NULL, &dwSize) )
	{
		cookieAllFaked = new char[dwSize];
		InternetGetCookie(context->getRequestUrl().c_str(), NULL, cookieAllFaked, &dwSize);
	}

	if (dwSize == 0)
	{
		cookieAllFaked = new char[2] ;
		strcpy(cookieAllFaked, "");
	}
	// Insert the faked cookies in a map.
	// The map value is the variable name, the map key is the cookie 
	// associated with the variable (in the form name=value)
	string commentfakes = "Fake Cookies:\n";
	stringStlMap httpCookieMap;
	char * token = strtok(cookieAllFaked, ";");
	while(token != NULL) { // While we have Cookies seperated by ;'s
		while((*token != NULL) && isspace(*token)) token++; // strip leading whitespace
		if((*token != NULL) &&  // NOT empty string AND faked Cookie found
		   (strncmp(token, AUTOMATIC_COOKIE_PREFIX, strlen(AUTOMATIC_COOKIE_PREFIX)) == 0)) {
		    commentfakes += "> " + string(token) + "\n";
		    char * pRealCookie = strchr(token, '=');
		    *pRealCookie = 0;
		    pRealCookie++;
		    variableName = &token[strlen(AUTOMATIC_COOKIE_PREFIX)];
		    cookieValue = pRealCookie;
		    httpCookieMap[cookieValue] = variableName;
		}
		token = strtok(NULL, ";");
	}

	if(cookieAllFaked != NULL) delete [] cookieAllFaked;

	// extract the cookie header from the http header
	string cookieHeader = header.substr(iBeginCookie, header.length() - iBeginCookie);
	iLengthCookie = cookieHeader.find("^J", 0) ;
	// The cookie is the last header of the http header
	if (iLengthCookie == string::npos)
	{
		iLengthCookie = cookieHeader.length() ;
		bAddQuotes = false ;
	}
	cookieHeader.resize(iLengthCookie) ;

	// prepare a string to add the original cookie value as comment in the tof file
	string comment = "Cookie Header:\n";
	comment += "> " + cookieHeader;

	// Insert the original cookies of the request in a stl vector container
	stringStlList cookieList ;
	stringStlList::iterator itrList ;
	cookieAllRequest = new char[cookieHeader.length() + 1] ;
	strcpy(cookieAllRequest, cookieHeader.c_str());
	token = strtok(cookieAllRequest, ";");
	while(token != NULL) {
		while((*token != NULL) && isspace(*token)) token++; // strip leading whitespace
		if(*token != NULL) { // Not empty string
		    cookieList.push_back(token) ;
		    token = strtok(NULL, ";");
		}
	}

	if (cookieAllRequest != NULL)
		delete [] cookieAllRequest ;

	// We are going to build anew cookie http header.
	cookieHeader.erase();

	stringStlMap::iterator itrHttpMap ;
	stringStlMap::iterator itrMap ;
	ofstream * f = context->getTofStream();
	for (itrList = cookieList.begin(); itrList != cookieList.end(); itrList++)
	{	
		cookieValue = *itrList ;
		variableName = "" ;
		// The cookie is an HTTP cookie
		if ((itrHttpMap = httpCookieMap.find(cookieValue)) != httpCookieMap.end())
			variableName = itrHttpMap->second ;
		// Cookie is not a HTTP cookie
		else
		{
			// the cookie is not already declared, we declare and initialise it.
			if ((itrMap = scriptCookieMap_.find(cookieValue)) == scriptCookieMap_.end())
			{
				char bufConid[20] ;
				sprintf(bufConid, "%d_%d", context->getConnectId(), context->getCookieId(ECT_SCRIPT)) ;
				variableName = "S_cookie_" ;
				variableName += bufConid ;
				context->incCookieId(ECT_SCRIPT);
				scriptCookieMap_[cookieValue] = variableName ;
// Now, the s_cookie variables are declared as constant

//				*f << "\r\nSetvar\r\n" ;
//				*f << "name " << variableName << "\r\n" ;
//				*f << "value " << cookieValue << "\r\n" ;
				varList_.push_back(variableName) ;
			}
			else
				variableName = itrMap->second ;
		}
		if (!variableName.empty())
		{
			if (!cookieHeader.empty())
				cookieHeader += "+\"; \"+" ;
			else 
				cookieHeader += " \"+" ;
			cookieHeader += variableName ;
		}
		else
			cookieHeader += cookieValue ;
	}
	
	if (bAddQuotes)
		cookieHeader += "^J" ;

	if (!cookieHeader.empty())
		header.replace(iBeginCookie, iLengthCookie, cookieHeader.c_str()) ;

	// Add the original cookie value as comment in the tof file
	if (bAddCookieComment_) {
		writeCommentInTofFile(context, comment);
		writeCommentInTofFile(context, commentfakes);
	}
	
	return header ;
}

const int pr2six[256]={
    64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,
    64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,62,64,64,64,63,
    52,53,54,55,56,57,58,59,60,61,64,64,64,64,64,64,64,0,1,2,3,4,5,6,7,8,9,
    10,11,12,13,14,15,16,17,18,19,20,21,22,23,24,25,64,64,64,64,64,64,26,27,
    28,29,30,31,32,33,34,35,36,37,38,39,40,41,42,43,44,45,46,47,48,49,50,51,
    64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,
    64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,
    64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,
    64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,
    64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,
    64,64,64,64,64,64,64,64,64,64,64,64,64
};

// uudecode function.
// pcInputBuf: [in]: input buffer to decode.
// pdwDecoded: [out]: number of bytes in returned buffer.
// IMPORTANT: The caller must issue a "delete []" on the return value after use.
BYTE* CTofReference::uudecode(const char* pcInputBuf, DWORD* pdwDecoded)
{
    int iDecoded;
    const char *pcIn = pcInputBuf;
    unsigned char *pcOut;
    int iBytes;
    BYTE* pReturn;

    pcIn = pcInputBuf;
    while(pr2six[*(pcIn++)] <= 63);
    iBytes = pcIn - pcInputBuf - 1;
    iDecoded = ((iBytes + 3) / 4) * 3;

    pReturn = new BYTE[iDecoded + 1];
    *pdwDecoded = iDecoded;

    pcOut = (unsigned char *)pReturn;
    pcIn = pcInputBuf;

    while(iBytes > 0)
    {
        *(pcOut++) = (unsigned char) (pr2six[*pcIn]   << 2 | pr2six[pcIn[1]] >> 4);
        *(pcOut++) = (unsigned char) (pr2six[pcIn[1]] << 4 | pr2six[pcIn[2]] >> 2);
        *(pcOut++) = (unsigned char) (pr2six[pcIn[2]] << 6 | pr2six[pcIn[3]]);
        pcIn += 4;
        iBytes -= 4;
    }

    if(iBytes & 03)
    {
        if(pr2six[pcIn[-2]] > 63)
            iDecoded -= 2;
        else
            iDecoded -= 1;
    }

    pReturn[iDecoded] = '\0';

    return pReturn;
}


void CTofReference::BASIC_authentication(CContext *context, string & headScl, bool & bAddQuotes)
{
    try
    {
		string authStr ;
		string suffix = " + \"" ;
		int iIndexAuth = CGWHttp::find_in_string(headScl, "Authorization: BASIC", false, 0) ;
		if (iIndexAuth == string::npos)
			return ;
		int iIndexRet = CGWHttp::find_in_string(headScl, "^J", false, iIndexAuth) ;
		if (iIndexRet == string::npos)
		{
			iIndexRet = headScl.size() ;
			bAddQuotes = false;
			suffix = "" ;
		}

		authStr = headScl.substr(iIndexAuth + strlen("Authorization: BASIC "), iIndexRet - iIndexAuth - strlen("Authorization: BASIC ")) ;
		
		int iBlankCar = 0;
		for (iBlankCar =0; authStr[iBlankCar] == ' ' || authStr[iBlankCar] == '\t'; iBlankCar++) ;
		authStr = authStr.substr(iBlankCar) ;

		char zBuf[50];
		// SCL Variable name for the Authentication

		stringStlMap::iterator itrMap ;
		if ((itrMap = authVarMap_.find(authStr)) == authVarMap_.end())
		{
			// Fill the list used to declare the variables
			context->setAuthenticationIndex(context->getAuthenticationIndex() + 1);
			sprintf(zBuf, "blob_%d_%d",  context->getConnectId(), context->getAuthenticationIndex() );
			authVarMap_.insert(stringStlMap::value_type(authStr, zBuf)) ;
			// Fill the list used to declare the variables
			authVarList_.push_back(zBuf) ;
			
			ofstream * f = context->getTofStream();
			if (f == NULL)
				return;
			
			DWORD dwLen = 0; 
			string domain ;
			string user ;
			string password ;

			char * pDecoded = (char *)uudecode(authStr.c_str(), &dwLen) ;
			if (pDecoded != NULL)
			{
				char * pKeepDecoded = pDecoded ;

				// Domain
				char * p = strchr(pDecoded, '\\');
				
				if (p != NULL)
				{
					*p = 0 ;
					domain = pDecoded ;
					pDecoded = p + 1 ;
				}
				// User and Password
				p = strchr(pDecoded, ':');
				if (p != NULL)
				{
					*p = 0 ;
					user = pDecoded ;
					password = p + 1 ;
				}

				delete [] pKeepDecoded ;
			}

			*f << "\r\nBAuthentication\r\n" ;
			*f << "type 0\r\n" ;
			*f << "tpargument 1\r\n" ;
			*f << "user \"" << user << "\"\r\n" ;
			*f << "password \"" << password << "\"\r\n" ;
			*f << "domain \"" << domain << "\"\r\n" ;
			*f << "into " << zBuf << "\r\n" ;
			headScl.replace(iIndexAuth, iIndexRet - iIndexAuth, "Authorization: \"+" + string(zBuf) + suffix) ; 
		}
		else
		{
			headScl.replace(iIndexAuth, iIndexRet - iIndexAuth, "Authorization: \"+" + itrMap->second + suffix) ; 
		}

	}
    catch (...)
    {
        perror("Error in write BASIC Authentication in tof file");
    }
}

void CTofReference::NTLM_authentication_client_1(CContext *context, string & headScl, bool & bAddQuotes)
{
    try
    {
		char zBuf[50];
		// SCL Variable name for the Authentication
		context->setAuthenticationIndex(context->getAuthenticationIndex() + 1);
		sprintf(zBuf, "blob_%d_%d",  context->getConnectId(), context->getAuthenticationIndex() );

		// Fill the list used to declare the variables
		authVarList_.push_back(zBuf) ;

		ofstream * f = context->getTofStream();
		if (f == NULL)
			return;
		*f << "\r\nBAuthentication\r\n" ;
		*f << "type 1\r\n" ;
		*f << "tpargument 0\r\n" ;
		*f << "into " << zBuf << "\r\n" ;

		int iIndexAuth = CGWHttp::find_in_string(headScl, "Authorization: NTLM", false, 0) ;
		if (iIndexAuth == string::npos)
			return ;
		int iIndexRet = CGWHttp::find_in_string(headScl, "^J", false, iIndexAuth) ;
		if (iIndexRet == string::npos)
		{
			headScl.replace(iIndexAuth, headScl.size() - iIndexAuth, "Authorization: \"+" + string(zBuf)) ; 
			bAddQuotes = false;
		}
		else
			headScl.replace(iIndexAuth, iIndexRet - iIndexAuth, "Authorization: \"+" + string(zBuf) + "+\"") ; 
	}
    catch (...)
    {
        perror("Error in write NTLM Authentication 1 in tof file");
    }
}

void CTofReference::NTLM_authentication_client_2(CContext *context, string & headScl, bool & bAddQuotes)
{
    try
    {
		string authStr ;
		string suffix = " + \"" ;

		int iIndexAuth = CGWHttp::find_in_string(headScl, "Authorization: NTLM", false, 0) ;
		if (iIndexAuth == string::npos)
			return ;
		int iIndexRet = CGWHttp::find_in_string(headScl, "^J", false, iIndexAuth) ;
		if (iIndexRet == string::npos)
		{
			iIndexRet = headScl.size() ;
			bAddQuotes = false;
			suffix = "" ;
		}

		authStr = headScl.substr(iIndexAuth + strlen("Authorization:"), iIndexRet - iIndexAuth - strlen("Authorization:")) ;

		char zBuf[50];

		stringStlMap::iterator itrMap ;
		if ((itrMap = authVarMap_.find(authStr)) == authVarMap_.end())
		{
			// Fill the list used to declare the variables
			sprintf(zBuf, "blob_%d_%d",  context->getConnectId(), context->getAuthenticationIndex() );
			authVarMap_.insert(stringStlMap::value_type(authStr, zBuf)) ;
			// Fill the list used to declare the variables

			ofstream * f = context->getTofStream();
			if (f == NULL)
				return;
			*f << "\r\nLload\r\n" ;
			*f << "conid " << context->getConnectId() << "\r\n" ;
			*f << "data 0\r\n" ;
			if (context->getThroughProxy())
				*f << "identifier \"Proxy-Authenticate\"\r\n";
			else
				*f << "identifier \"WWW-Authenticate\"\r\n";
			*f << "into " << zBuf << "\r\n" ;

			*f << "\r\nBAuthentication\r\n" ;
			*f << "type 1\r\n" ;
			*f << "tpargument 2\r\n" ;
			*f << "blob " << zBuf << "\r\n" ;
			*f << "into " << zBuf << "\r\n" ;

			headScl.replace(iIndexAuth, iIndexRet - iIndexAuth, "Authorization: \"+" + string(zBuf) + suffix) ; 
		}
		else
		{
			headScl.replace(iIndexAuth, iIndexRet - iIndexAuth, "Authorization: \"+" + itrMap->second + suffix) ; 
		}
	}
    catch (...)
    {
        perror("Error in write NTLM Authentication 2 in tof file");
    }
}

void CTofReference::Negotiate_authentication_client_1(CContext *context, string & headScl, bool & bAddQuotes)
{
    try
    {
		char zBuf[50];
		// SCL Variable name for the Authentication
		context->setAuthenticationIndex(context->getAuthenticationIndex() + 1);
		sprintf(zBuf, "blob_%d_%d",  context->getConnectId(), context->getAuthenticationIndex() );

		// Fill the list used to declare the variables
		authVarList_.push_back(zBuf) ;

		ofstream * f = context->getTofStream();
		if (f == NULL)
			return;
		*f << "\r\nBAuthentication\r\n" ;
		*f << "type 3\r\n" ;
		*f << "tpargument 0\r\n" ;
		*f << "into " << zBuf << "\r\n" ;

		int iIndexAuth = CGWHttp::find_in_string(headScl, "Authorization: Negotiate", false, 0) ;
		if (iIndexAuth == string::npos)
			return ;
		int iIndexRet = CGWHttp::find_in_string(headScl, "^J", false, iIndexAuth) ;
		if (iIndexRet == string::npos)
		{
			headScl.replace(iIndexAuth, headScl.size() - iIndexAuth, "Authorization: \"+" + string(zBuf)) ; 
			bAddQuotes = false;
		}
		else
			headScl.replace(iIndexAuth, iIndexRet - iIndexAuth, "Authorization: \"+" + string(zBuf) + "+\"") ; 
	}
    catch (...)
    {
        perror("Error in write Negotiate Authentication 1 in tof file");
    }
}

void CTofReference::Negotiate_authentication_client_2(CContext *context, string & headScl, bool & bAddQuotes)
{
    try
    {
		string authStr ;
		string suffix = " + \"" ;

		int iIndexAuth = CGWHttp::find_in_string(headScl, "Authorization: Negotiate", false, 0) ;
		if (iIndexAuth == string::npos)
			return ;
		int iIndexRet = CGWHttp::find_in_string(headScl, "^J", false, iIndexAuth) ;
		if (iIndexRet == string::npos)
		{
			iIndexRet = headScl.size() ;
			bAddQuotes = false;
			suffix = "" ;
		}

		authStr = headScl.substr(iIndexAuth + strlen("Authorization:"), iIndexRet - iIndexAuth - strlen("Authorization:")) ;

		char zBuf[50];

		stringStlMap::iterator itrMap ;
		if ((itrMap = authVarMap_.find(authStr)) == authVarMap_.end())
		{
			// Fill the list used to declare the variables
			sprintf(zBuf, "blob_%d_%d",  context->getConnectId(), context->getAuthenticationIndex() );
			authVarMap_.insert(stringStlMap::value_type(authStr, zBuf)) ;
			// Fill the list used to declare the variables

			ofstream * f = context->getTofStream();
			if (f == NULL)
				return;
			*f << "\r\nLload\r\n" ;
			*f << "conid " << context->getConnectId() << "\r\n" ;
			*f << "data 0\r\n" ;
			if (context->getThroughProxy())
				*f << "identifier \"Proxy-Authenticate\"\r\n";
			else
				*f << "identifier \"WWW-Authenticate\"\r\n";
			*f << "into " << zBuf << "\r\n" ;

			*f << "\r\nBAuthentication\r\n" ;
			*f << "type 3\r\n" ;
			*f << "tpargument 2\r\n" ;
			*f << "blob " << zBuf << "\r\n" ;
			*f << "into " << zBuf << "\r\n" ;

			headScl.replace(iIndexAuth, iIndexRet - iIndexAuth, "Authorization: \"+" + string(zBuf) + suffix) ; 
		}
		else
		{
			headScl.replace(iIndexAuth, iIndexRet - iIndexAuth, "Authorization: \"+" + itrMap->second + suffix) ; 
		}
	}
    catch (...)
    {
        perror("Error in write Negotiate Authentication 2 in tof file");
    }
}

void CTofReference::writeRequestInTofFile(CContext *context, const string & header, char * pBody, int iBodyLen)
{
    try
    {
        CTLockGuard <CTMutex > guard(tofMutex_);
		int iOptions = 0 ;

        if (selectStatus_ == ESL_RECORD)
        {
            ofstream * f = context->getTofStream();
            if (f == NULL)
                return;

			context->incTofRequestId() ;

            CTTime timeWrite;
            timeWrite.now();
            CTTime idleTime ;

            if (serverName_ != "FILEGate" && !timePreviousWrite_.isNull() )
                idleTime = timeWrite - timePreviousWrite_;

            if (idleTime != 0)
            {
                *f << "Wait\r\n" ;
                *f << "tdelay " << (unsigned long)idleTime << "\r" << std::endl ;
            }

			bool bAddQuotes = true ;
            string headScl = modifyStringAsScl(header) ;

			// ADD  Authentication entries
			switch(context->getAuthenticationState())
			{
			case EAS_NTLM_CLIENT_1 :		NTLM_authentication_client_1(context, headScl, bAddQuotes) ;
											break;
			case EAS_NTLM_CLIENT_2  :		NTLM_authentication_client_2(context, headScl, bAddQuotes) ;
											break ;
			case EAS_NEGOTIATE_CLIENT_1 :	Negotiate_authentication_client_1(context, headScl, bAddQuotes) ;
											break;
			case EAS_NEGOTIATE_CLIENT_2  :	Negotiate_authentication_client_2(context, headScl, bAddQuotes) ;
											break ;
			case EAS_BASIC :				BASIC_authentication(context, headScl, bAddQuotes);
											break ;
			}
			string headCookie = headScl ;
			// Model cookies only if the automatic cookie option is on.
			if (bAutomaticModel_)
				headCookie = modelCookies(context, headScl, bAddQuotes) ;

            *f << "\r\nRequest\r\n";
            *f << "conid " << context->getConnectId() << "\r\n" ;
            if (CGWHttp::find_in_string(context->getMethod(),"GET", false, 0) == 0)
                *f << "type 0\r\n" ;
            else if (CGWHttp::find_in_string(context->getMethod(),"HEAD",false,0) == 0)
                *f << "type 1\r\n" ;
            else if (CGWHttp::find_in_string(context->getMethod(),"POST",false,0) == 0)
                *f << "type 2\r\n" ;

            string url = modifyStringAsScl(context->getRequestUrl()) ;
            *f << "uri \"" << url << " " << context->getHttpVersion() << "\"\r\n" ;

            *f << "header " << SEPARATOR << "\r\n";
            *f << "\"" << headCookie ;
			if (bAddQuotes)
				*f << "\"" ;
			*f << SEPARATOR << "\r\n" ;


			if (context->getClientCertificateName() != "")
			{
				*f << "certificate \"" << context->getClientCertificateName() << "\"\r\n" ;
//				iOptions += 4 ; The default is a certificate name, not file.
			}
            if (iBodyLen > 0)
            {
                *f << "body " << SEPARATOR << "\r\n" ;
                string bod = modifyBodyAsScl(pBody, iBodyLen);
                *f << "\"" << bod << "\"" << SEPARATOR << "\r\n" ;
            }
			// Write option for primary urls

			if (context->isPrimary())
				iOptions += 2 ;
			
			if (iOptions > 0)
				*f << "option " << iOptions << "\r\n" ;

            *f << "\r" << std::endl ;
        }
        timePreviousWrite_.now() ;
    }
    catch (...)
    {
        perror("Error in write request in tof file");
    }
    eObjectType_ = EOT_REQUEST ;
}

void CTofReference::writeLoadResponseHeader(CContext * context, string & header, string & variable_prefix)
{
    try
    {
        CTLockGuard <CTMutex > guard(tofMutex_);

        if (selectStatus_ == ESL_RECORD)
        {
            ofstream * f = context->getTofStream();
            if (f == NULL)
                return;

			int iIndexEqual = CGWHttp::find_in_string(header,"=",false,0) ;
			string cookieName = header.substr(0, iIndexEqual);
			string valueName = string("Set-Cookie,") + cookieName;
            string vName; // The SCL variable name for this Cookie
            DWORD dwSize = 0;

			// See if we already have a variable for this Cookie using our faked Cookies
	        if(InternetGetCookie(context->getRequestUrl().c_str(), NULL, NULL, &dwSize)) {
		        LPSTR cookies = new char[dwSize];
		        InternetGetCookie(context->getRequestUrl().c_str(), NULL, cookies, &dwSize);
	            char* token = strtok(cookies, ";");
	            while((token != NULL) && (vName == "")) {
		            while((*token != NULL) && isspace(*token)) token++; // strip leading whitespace
		            if((*token != NULL) && // NOT empty string AND starts with faking string
					   (strncmp(token, AUTOMATIC_COOKIE_PREFIX, strlen(AUTOMATIC_COOKIE_PREFIX)) == 0)) {
			            char* pRealCookie = strchr(token, '=');
						if(pRealCookie != NULL) { // Sanity check
			                *pRealCookie = NULL; // token string terminated at end of fake Cookie name
			                char* variableName = &token[strlen(AUTOMATIC_COOKIE_PREFIX)] ;
			                pRealCookie++; // Point at fake Cookies value, = real Cookie 'name=value'
						    char* pEndCookieName = strchr(pRealCookie, '=');
						    if(pEndCookieName != NULL) *pEndCookieName = NULL; // terminate, lose value
			                if(cookieName == pRealCookie) vName = variableName; // FOUND Variable name
						}
					}
					token = strtok(NULL, ";");
				}
				delete [] cookies;
			}

			if(vName == "") {
                char bufConid[20] ;
                sprintf(bufConid, "%d_%d", context->getConnectId(), context->getCookieId(ECT_HTTP)) ;
                vName = variable_prefix + "_" + bufConid;
                context->incCookieId(ECT_HTTP);
				// Create a faked Cookie to track this SCL variable
			    string fakedCookie = AUTOMATIC_COOKIE_PREFIX + vName + "=" + modifyStringAsScl(header);
			    // remove 'expires' so our faked cookies are always 'session'
			    int iIndexExpires = string::npos ;
			    if((iIndexExpires = CGWHttp::find_in_string(fakedCookie, "expires=", false, 0)) != string::npos) {
				    int iIndexSemiColon = string::npos ;
			 	    if((iIndexSemiColon = fakedCookie.find_first_of(";", iIndexExpires)) != string::npos)
				 	    fakedCookie.erase(iIndexExpires, iIndexSemiColon - iIndexExpires + 1) ;
				    else
					    fakedCookie.erase(iIndexExpires) ;
			    }
			    InternetSetCookie(context->getRequestUrl().c_str(), NULL, fakedCookie.c_str());

			    // Insert the faked cookie in map. This map will be used to remove all the faked cookie.
			    fakedCookieMap_[fakedCookie] = context->getRequestUrl() ;
                varList_.push_back(vName) ;
			}

			string valueScl = modifyStringAsScl(valueName) ;
            *f << "\r\nLload\r\n";
            *f << "conid " << context->getConnectId() << "\r\n" ;
            *f << "data 0\r\n" ;
            *f << "identifier \"" << valueScl << "\"\r\n";
            *f << "into " << vName << "\r\n" ;
            *f << "\r" << std::endl ;
        }
    }
    catch (...)
    {
        perror("Error in write LoadResponseHeader in tof file");
    }

}
