/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

#ifdef  WIN32
#  pragma warning(disable : 4710)
#endif  // WIN32


//=================================================================< Includes >
#ifndef __DEFINITION__
#  include "definition.h"
#endif  // __DEFINITION__

#ifdef BUILDDLLGWHTTP
#  define DLLSPEC_GWHTTP __EXPORT__
#  define DLLSPEC_GWCONSCMD __EXPORT__
#else
#  define DLLSPEC_GWHTTP __IMPORT__
#  define DLLSPEC_GWCONSCMD __IMPORT__
#endif  // BUILDDLLGWHTTP

#include <ctime.hpp>

#ifndef __GATEWAY_CMD__
#define DLLSPEC_GWCMD   DLLSPEC_GWHTTP
#  include "gwcmd.hpp"
#endif  // __GATEWAY_CMD__


#ifndef __GATEWAY_CONSOLE_COMMAND__
#  include "gwconscmd.hpp"
#endif  // __GATEWAY_CONSOLE_COMMAND__

#ifndef __CSTOF__
#  include "cstof.hpp"
#endif


enum enumUserCommand
{
    EUC_UNKNOWN = 0,
    EUC_CAPTURE = 1,
    EUC_CONNECTION = 2,
    EUC_AUTOMATICMODEL = 3,
    EUC_SETDOCUMENTSPATH = 4
};


//======================================================< Definition of class >

///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////// Class CGSelect //
///////////////////////////////////////////////////////////////////////////////
class DLLSPEC_GWCMD CGSelect : public CGWParseCommand
{
    //-----------------------------------------------------------------------------
public:                         // Virtual methodes.
    void                        parseCommand (const string& command, CGCmd*& pCmd) ;
    void                        commandResult (CGCmd*& cmd) ;

    //-----------------------------------------------------------------------------
public:                         // Constructor(s) and destructor.
    CGSelect (const void* pConnectionId, const string& taskName, bool& bTraceOn, CTOLogFileList& logFileList, CTSocket* pClient, CGWParseCommand* pNext = NULL) ;
    ~CGSelect ()
    { }
    ;

public:                         // Operators.
    CGSelect&                   operator = (const CGSelect&) ;

    //-----------------------------------------------------------------------------
protected:                      // Control Methodes.
    void                        setNext (CGWParseCommand* pNext)
    {
        pNext_ = pNext ;
    } ;

protected:                      // Get Methodes.

    //-----------------------------------------------------------------------------
private:                        // Members.
    int                         iConnectionId_ ;
    static int                  iNbConnections_ ;
    CGWParseCommand*            pNext_ ;
    bool                        bIsCommandForNext_ ;

    //-----------------------------------------------------------------------------
}

; // CGSelect

///////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////// End of Class CGSelect //
///////////////////////////////////////////////////////////////////////////////

//====================================================================< Class >
//CTofReference tofReference ;

///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////// Class CGSelect //
///////////////////////////////////////////////////////////////////////////////

int CGSelect::iNbConnections_ = 0 ;

//============================================< Constructor(s) and destructor >
//-----------------------------------------------------------------------------
// CGSelect::CGSelect
//
// Constructor.
//
// Arguments : pConnectionId : connection identifier.
//             taskName      : name of the task.
//             bTraceOn      : reference on the trace mode.
//             logFileList   : reference on the log file list.
//             pClient       : pointer to client socket.
//             pNext         : pointer to the next CGWParseCommand object.
// Return    : none.
//-----------------------------------------------------------------------------
CGSelect::CGSelect (const void* pConnectionId, const string& taskName, bool& bTraceOn, CTOLogFileList& logFileList, CTSocket* pClient, CGWParseCommand* pNext /* = NULL */)
        : CGWParseCommand (pConnectionId, taskName, bTraceOn, logFileList, pClient), pNext_ (pNext), bIsCommandForNext_ (FALSE)
{
    try
    {
        iConnectionId_ = ++ iNbConnections_ ;

        if (pClient == NULL)
        {
            extern CTofReference tofReference;
            tofReference.analyse("FILEGate", taskName, "only", "record", "");
        }

        if (bTraceOn_)
        {
            logFileList_ << taskName_.data () << ": new select command object." << endl ;
        }
    }
    catch (CTException& e)
    {
        throw CGCmdException ( (CGCmdException&) e) ;
    }
} // CGSelect::CGSelect

extern string                       gListeDocuments ;
extern string                       gDocumentPath ;


//=========================================================< Control methodes >
//-----------------------------------------------------------------------------
// CGSelect::parseCommand
//
// Parse the command.
//
// Arguments : command : command string to parse.
//             pCmd    : reference to pointer to command to execute at return.
// Return    : none.
//-----------------------------------------------------------------------------
void CGSelect::parseCommand (const string& command, CGCmd*& pCmd)
{
    try
    {
        if ( (pNext_ == NULL) || (CGWHttp::find_in_string(command,"SELECT:",false, 0) == 0) )
        {
            bIsCommandForNext_ = FALSE ;
            if (bTraceOn_)
            {
                logFileList_ << taskName_.data () << ": " << command.data () << flush ;
            }
            extern CTofReference tofReference;

            if (command == "SELECT:CAPTURE")
            {   // Close connection.
                string strConid = "";
                string strCommand = "";
                string strTofFileName = "";
                string strMode = "";
                string strMessage = "";

                client_ >> strConid;
                client_ >> strCommand;
                if (strCommand == "record")
                {
                    client_ >> strTofFileName;
                    client_ >> strMode;
                    client_ >> strMessage;
                }
                else if (strCommand == "comment")
				{
					unsigned long ulTimeForComment ;
                    client_ >> strMessage;
					client_ >> ulTimeForComment ;

					tofReference.incTimePreviousWrite(ulTimeForComment) ;
				}	
                else if (strCommand == "starttimer" || strCommand == "endtimer")
				{
					unsigned long ulTimeForAddTimer ;
                    client_ >> strMessage;
					client_ >> ulTimeForAddTimer ;

					tofReference.incTimePreviousWrite(ulTimeForAddTimer) ;
				}	
                if (
                    (strCommand != "record" &&
                     strCommand != "stop" &&
                     strCommand != "pause" &&
                     strCommand != "comment" &&
                     strCommand != "resume" && 
					 strCommand != "starttimer" && 
					 strCommand != "endtimer")
                    ||
                    strConid == ""
                )
                    pCmd = new CGCmdUser (EUC_UNKNOWN) ;
                else
                    pCmd = new CGCmdUser (EUC_CAPTURE) ;

                tofReference.analyse(strConid, strTofFileName, strMode, strCommand, strMessage);

                if (pCmd == NULL)
                {   // Error from new.
                    throw CGCmdException ("selectParseCommand: new CGCmdUser failed.", __FILE__, __LINE__) ;
                }
            }
            else if (command == "SELECT:AUTOMATICMODEL")
            {
                client_ >> CTofReference::bAutomaticModel_ ;
                client_ >> CTofReference::bAddCookieComment_ ;
                pCmd = new CGCmdUser (EUC_AUTOMATICMODEL) ;
                if (pCmd == NULL)
                {   // Error from new.
                    throw CGCmdException ("selectParseCommand: new CGCmdUser failed.", __FILE__, __LINE__) ;
                }
            }
            else if (command == "SELECT:SETDOCUMENTSPATH")
            {
                client_ >> gDocumentPath;
                if (gDocumentPath[(size_t)0] == '\"' && gDocumentPath[(size_t)(gDocumentPath.length() - 1)] == '\"')
                    gDocumentPath = gDocumentPath.substr(1, gDocumentPath.length() - 2) ;

                pCmd = new CGCmdUser (EUC_SETDOCUMENTSPATH) ;
                if (pCmd == NULL)
                {   // Error from new.
                    throw CGCmdException ("selectParseCommand: new CGCmdUser failed.", __FILE__, __LINE__) ;
                }
            }
            else if (command == "SELECT:WEBCONNECTION")
            {   // Close connection.
                string strProxy = "";
                string strExceptions = "";
                string strLocal = "";

                client_ >> strProxy ;
                client_ >> strExceptions ;
                client_ >> strLocal ;
                pCmd = new CGCmdUser (EUC_CONNECTION) ;

                logFileList_  << "\nOriginal Browser configuration"
                << "\n------------------------------"
                << "\nProxy Configuration : " << strProxy.c_str()
                << "\nExceptions : " << strExceptions.c_str()
                << "\nLocal Proxy Value : " << strLocal.data ()
                << "\n" << endl ;
                extern CTargetEvaluation * gTargetEvaluation ;
                bool bLocal = (strLocal == "no_proxy_for_local")?TRUE:FALSE ;
                gTargetEvaluation = new CTargetEvaluation(strProxy, strExceptions, bLocal) ;

                if (pCmd == NULL)
                {   // Error from new.
                    throw CGCmdException ("selectParseCommand: new CGCmdUser failed.", __FILE__, __LINE__) ;
                }
            }
            else if (command == "SELECT:KILL_TASK_DATA")
            {
                CGListen    listenInfo ;
                client_ >> listenInfo ;

                port_t  listenPort ;
                sscanf (listenInfo.getPort ().data (), "%hu", &listenPort) ;
                pCmd = new CGCmdKillTaskData (listenInfo.getIsAdministrationListen(), listenInfo.getTargetAddress (), listenPort, listenInfo.getLibrary (), listenInfo.getTargetAddressAutoDetectionMode()) ;
                if (pCmd == NULL)
                {   // Error from new.
                    throw CGCmdException ("selectParseCommand: new CGCmdKillTaskData failed.", __FILE__, __LINE__) ;
                }
            }
            else if (command == "SELECT:SHUTDOWN")
            {   // Shutdown all the gateways.
                exit(0) ;
            }
            else
            {   // Error from command.
                throw CGCmdException (string ("selectParseCommand: command ") + command + " is unknow.", __FILE__, __LINE__) ;
            }
            if (bTraceOn_)
            {
                logFileList_ << "." << endl ;
            }
        }
        else
        {
            bIsCommandForNext_ = TRUE ;
            pNext_->parseCommand (command, pCmd) ;
        }
    }
    catch (CTException& e)
    {
        throw CGCmdException ( (CGCmdException&) e) ;
    }
} // CGSelect::parseCommand

//-----------------------------------------------------------------------------
// CGSelect::commandResult
//
// Send the result of the command.
//
// Arguments : pCmd    : reference to pointer to command to execute at return.
// Return    : none.
//-----------------------------------------------------------------------------
void CGSelect::commandResult (CGCmd*& pCmd)
{
    try
    {
        if (!bIsCommandForNext_)
        {
            string  command ;

            if (pCmd == NULL)
            {   // Error from pCmd.
                throw CGCmdException ("commandResult: pCmd is null.", __FILE__, __LINE__) ;
            }
            if (pCmd->getCommand () == CGCmd::CMD_NEW_LISTEN_TASK)
            {   // Create a new gateway.
                // Send command execution status.
                if (pCmd->getResult ())
                    command = "OK";
                else
                {
                    command = "FAILED : " ;
                }
                client_ << command ;
            }
            else if (pCmd->getCommand () == CGCmd::CMD_KILL_TASK_DATA)
            {   // Stop a gateway.
                // Send command execution status.
                if (pCmd->getResult ())
                    command = "OK";
                else
                {
                    command = "FAILED : " ;
                }
                client_ << command ;
            }
            else if (pCmd->getCommand () == CGCmd::CMD_SHUTDOWN)
            {   // Shutdown.
                // Send command execution status.
                if (pCmd->getResult ())
                    command = "OK";
                else
                {
                    command = "FAILED : " ;
                }
                client_ << command ;
            }
            else if (pCmd->getCommand () == CGCmd::CMD_USER)
            {
                if (((CGCmdUser *)pCmd)->getUserCommand() == EUC_CAPTURE)
                    command = "OK";
                else if (((CGCmdUser *)pCmd)->getUserCommand() == EUC_CONNECTION)
                    command = "OK";
                else if (((CGCmdUser *)pCmd)->getUserCommand() == EUC_AUTOMATICMODEL)
                    command = "OK";
                else if (((CGCmdUser *)pCmd)->getUserCommand() == EUC_SETDOCUMENTSPATH)
                    command = "OK";
                // Error from command.
                // Send command execution status.
                else
                    command = "FAIL" ;
                client_ << command ;
            }
            else
            {   // Error from command.
                // Send command execution status.
                command = "FAIL" ;
                client_ << command ;
            }
            delete pCmd ;
            pCmd = NULL ;
        }
        else
        {
            pNext_->commandResult (pCmd) ;
        }
    }
    catch (CTException& e)
    {
        throw CGCmdException ( (CGCmdException&) e) ;
    }
} // CGSelect::commandResult

///////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////// End of Class CGSelect //
///////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////// Globals Functions //
///////////////////////////////////////////////////////////////////////////////

//================================================================< Functions >
extern "C"
{

    void DLLSPEC_GWHTTP LGW_NewGatewaySelectParseCommand (const void* pConnectionId, const string& taskName, bool& bTraceOn, CTOLogFileList& logFileList, CTSocket* pClient, CGWParseCommand*& pGWParseCommand)
    {
        try
        {
            pGWParseCommand = (CGWParseCommand*) new CGSelect (pConnectionId, taskName, bTraceOn, logFileList, pClient) ;
            if (pGWParseCommand == NULL)
            {   // Error from new.
                throw CGCmdException ("LGW_NewGatewaySelectParseCommand: new failed.", __FILE__, __LINE__) ;
            }
        }
        catch (CTException& e)
        {
            pGWParseCommand = NULL ;
            throw CGCmdException ( (CGCmdException&) e) ;
        }
    } // LGW_NewGatewaySelectParseCommand

} // extern "C"

///////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////// End of Globals Functions //
///////////////////////////////////////////////////////////////////////////////
