/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

// BrowserInformation.cpp: implementation of the CBrowserInformation class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "BrowserInformation.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

#include "concertlib.h"
#include "ConstantesLib.h"
#include <winsock2.h>

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////
//-------------------------------------------------------------------

CBrowserInformation::CBrowserInformation(CWnd* _pWnd)
        :   m_pWnd(_pWnd)
{}

//-------------------------------------------------------------------


CBrowserInformation::~CBrowserInformation()
{
}

//-------------------------------------------------------------------


BOOL CBrowserInformation::GetDefaultParameters( _TPreferencesProxyData *pData )
{
    return false;
}

//--------------------------------------------------------------------

void CBrowserInformation::ModifyProxyForCapture( CString &strProxy , long lPort )
{
    CString strLeft ;
    CString strRight ;
    CString strPort ;
    //---
    strLeft.Empty() ;
    strRight.Empty() ;
    strPort.Format( "%ld" , lPort ) ;

    int nPos = strProxy.Find( '=' ) ;
    if( nPos >= 0 ) //All the proxy types do not have the same addresses
    {
        ModifySpecificProxyForCapture( strProxy , _T("http" ) , lPort ) ;
        ModifySpecificProxyForCapture( strProxy , _T("https" ) , lPort ) ;
    }
    else //there is only one proxy for all the protocols
    {
        TCHAR *pczBuffer[ID_LOCALHOST_SIZE + 1 ] ;
        //---
        gethostname( (char*)pczBuffer , ID_LOCALHOST_SIZE ) ;
        strProxy = (char*)pczBuffer ;
        strProxy +=  _T(":") ;
        strProxy += strPort ;
    }
}

//--------------------------------------------------------------------

void CBrowserInformation::ModifySpecificProxyForCapture( CString &strProxy , TCHAR *pczType , long lPort)
{
    CString strLeft ;
    CString strRight ;
    CString strPort ;
    TCHAR *pczBuffer[ID_LOCALHOST_SIZE + 1 ] ;
    //---
    strLeft.Empty() ;
    strRight.Empty() ;
    strPort.Format( "%ld" , lPort ) ;
    gethostname( (char*)pczBuffer , ID_LOCALHOST_SIZE ) ;

    int nPos = strProxy.Find( pczType ) ;
    if( nPos < 0 )
        return;

    strLeft = strProxy.Left( nPos + strlen( pczType ) + 1 ) ;
    strLeft += (char*)pczBuffer ;
    strLeft += _T(":" ) ;
    nPos = strProxy.Find( ':' , nPos ) ;
    strRight = strProxy.Right( strProxy.GetLength() - nPos - 1 ) ;
    nPos = strRight.Find( ';' ) ;
    if( nPos >= 0 )
    {
        strRight = strRight.Right( strRight.GetLength() - nPos ) ;
        strProxy = strLeft ;
        strProxy += strPort ;
        strProxy += strRight ;
    }
    else
    {
        strProxy = strLeft ;
        strProxy += strPort ;
    }
}


//--------------------------------------------------------------------

BOOL CBrowserInformation::IsKeyValueExist( HKEY _hKey , LPCTSTR _lpSubKey , LPCTSTR _szEntry )
{
    LONG    lResult;
    DWORD   dwType;
    DWORD   dwCount;
    HKEY    hSoftKey;
    //-----

    BOOL bRet = FALSE ;
    if ( ::RegOpenKeyEx(_hKey, _lpSubKey, 0, KEY_WRITE|KEY_READ, &hSoftKey) == ERROR_SUCCESS )
    {
        // test if the data exist
        lResult = RegQueryValueEx(hSoftKey, (LPTSTR)_szEntry, NULL, &dwType,
                                  NULL, &dwCount);
        bRet = ( lResult == ERROR_SUCCESS );
    }

    return bRet;
}

//--------------------------------------------------------------------

BOOL CBrowserInformation::IsKeyExist( HKEY _hKey , LPCTSTR _lpSubKey )
{
    HKEY    hSoftKey;
    //-----
    return ( ::RegOpenKeyEx(_hKey, _lpSubKey, 0, KEY_WRITE|KEY_READ, &hSoftKey) == ERROR_SUCCESS ) ;
}
