/* Copyright (C) 2000-2001 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/
// ScriptMonDlg.cpp : implementation file
//

#include "stdafx.h"
#include "ScriptMonDlg.h"
#include "CTime.hpp"
#include "HTTPInfo.h"
#include "..\Include\HttpHelpID.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

extern HINSTANCE m_hInstance;
extern CHTTPInfo* m_pHttpInfo;
#define ID_POLL_DATA 1
/////////////////////////////////////////////////////////////////////////////
// CScriptMonDlg dialog


CScriptMonDlg::CScriptMonDlg(CWnd* pParent /*=NULL*/)
	: CDialog(CScriptMonDlg::IDD, pParent)
{
	//{{AFX_DATA_INIT(CScriptMonDlg)
		// NOTE: the ClassWizard will add member initialization here
	//}}AFX_DATA_INIT
}


void CScriptMonDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CScriptMonDlg)
	DDX_Control(pDX, IDC_LIST, m_ctlList);
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(CScriptMonDlg, CDialog)
	//{{AFX_MSG_MAP(CScriptMonDlg)
	ON_WM_SIZE()
	ON_WM_TIMER()
	ON_WM_DESTROY()
	ON_WM_HELPINFO()
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CScriptMonDlg message handlers
BOOL CScriptMonDlg::OnInitDialog() 
{
	CDialog::OnInitDialog();
	
	SetWindowText(m_strTaskName);

	SetIcon(LoadIcon( m_hInstance, MAKEINTRESOURCE(IDI_MONITOR)), FALSE);

	CRect r;
	GetClientRect(r);
	m_ctlList.InsertColumn(0,"VU ID",LVCFMT_LEFT, r.Width()/4);
	m_ctlList.InsertColumn(1,"Duration",LVCFMT_LEFT, r.Width()/4);
	m_ctlList.InsertColumn(2,"Current Iteration",LVCFMT_LEFT, r.Width()/4);
	m_ctlList.InsertColumn(3,"Note Text",LVCFMT_LEFT, r.Width()/4);
	
	m_pHttpInfo->SetManagerReg(m_strManagerReg);
	TaskId task;
	task.lTaskGroupId = m_nTaskGroup;
	task.lTaskId = m_nTaskID;
	m_pHttpInfo->AddTaskId(task);
	m_pHttpInfo->StartCollector();

	m_nRefreshRate = m_pHttpInfo->GetRefreshRate();

	UpdateList();
	SetTimer(ID_POLL_DATA, m_nRefreshRate, NULL);
	return TRUE;  // return TRUE unless you set the focus to a control
	              // EXCEPTION: OCX Property Pages should return FALSE
}

void CScriptMonDlg::InitMonitor(CString strTaskName,int nTaskGroup, int nTaskID, CString strManagerReg)
{
	m_strTaskName = strTaskName;
	m_nTaskGroup = nTaskGroup;
	m_nTaskID = nTaskID;
	m_strManagerReg = strManagerReg;
}

void CScriptMonDlg::OnSize(UINT nType, int cx, int cy) 
{
	CDialog::OnSize(nType, cx, cy);
	
	if (m_ctlList.m_hWnd != NULL)
	{
		m_ctlList.MoveWindow(0,0,cx,cy);
	}
}

void CScriptMonDlg::OnTimer(UINT nIDEvent) 
{
	UpdateList();
	
	int nRefresh = m_pHttpInfo->GetRefreshRate();
	if (m_nRefreshRate != nRefresh)
	{
		m_nRefreshRate = nRefresh;
		KillTimer(ID_POLL_DATA);
		SetTimer(ID_POLL_DATA, m_nRefreshRate, NULL);
	}

	CDialog::OnTimer(nIDEvent);
}

void CScriptMonDlg::UpdateList()
{
	m_ctlList.SetRedraw(FALSE);
	m_ctlList.SetFocus();
	int highestVisible=m_ctlList.GetTopIndex();

	m_ctlList.DeleteAllItems();

	VUSnapList userInfo;
	for (int z = 0; z<(int)m_pHttpInfo->m_allVUSnaps.length(); z++)
	{
		TaskId id = m_pHttpInfo->m_allVUSnaps[z].task;
		if (m_nTaskGroup == id.lTaskGroupId &&
			m_nTaskID == id.lTaskId)
		{
			VUSnapList userInfo = m_pHttpInfo->m_allVUSnaps[z].VUSnaps;
			for (int i =0; i<(int)userInfo.length(); i++)
			{
				CString strVUId;
				strVUId.Format("%d-%d",m_nTaskGroup+1,userInfo[i].lVUId);
				m_ctlList.InsertItem(i, strVUId);
	
				m_ctlList.SetItemText(i,1,ConvertTimeToString(userInfo[i].lRunTime));

				CString strTGIter;
				strTGIter.Format("%d",userInfo[i].lCompletedIterations);
				m_ctlList.SetItemText(i,2,strTGIter);

				m_ctlList.SetItemText(i,3,userInfo[i].strNoteText);
	
			}
		}
	}

	int nHeightInPixels;
	
	CHeaderCtrl* head=m_ctlList.GetHeaderCtrl();
	if(head->GetItemCount()>0)
	{
		CRect rc;
		head->GetItemRect(0,rc);
		nHeightInPixels=rc.Height()-3;
			
		//CString height;
		//height.Format("first item in view was %d",nHeightInPixels);
		//AfxMessageBox(height);
	}
	else
	{
		nHeightInPixels=14;	//estimate if using w2k + tahoma?
	}
	
	CSize movement;
	movement.cx=0;
	movement.cy=highestVisible*nHeightInPixels;

	m_ctlList.SetRedraw(TRUE);
	m_ctlList.Scroll(movement);

}



void CScriptMonDlg::OnDestroy() 
{
	TaskId task;
	task.lTaskGroupId = m_nTaskGroup;
	task.lTaskId = m_nTaskID;
	m_pHttpInfo->RemoveTaskId(task);
	m_pHttpInfo->StopCollector();

	CDialog::OnDestroy();
}

CString CScriptMonDlg::ConvertTimeToString(long duration)
{

	CString answer;
	int remainder;
	div_t temp;
	int h=0;
	int m=0;
	int s=0;
	int days=0;

	temp=div(duration,3600*24);
	days=temp.quot;
	remainder=temp.rem;

	temp=div(remainder,3600);
	h=temp.quot;
	remainder=temp.rem;

	temp=div(remainder,60);
	m=temp.quot;
	s=temp.rem;

	if(days>0)
	{
		answer.Format("%d %02d:%02d:%02d",days,h,m,s);
	}
	else
	{
		answer.Format("%02d:%02d:%02d",h,m,s);
	}
	return answer;
}

BOOL CScriptMonDlg::OnHelpInfo(HELPINFO* pHelpInfo) 
{
	pHelpInfo->dwContextId = HIDD_HTTP_SCRIPTMON;	
	return CDialog::OnHelpInfo(pHelpInfo);
}
