/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

// DomTreeView.h : interface of the CDomTreeView class
//
/////////////////////////////////////////////////////////////////////////////

#if !defined(AFX_DOMTREEVIEW_H__87BE3BB1_1B48_11D4_AC8D_00105A483E3C__INCLUDED_)
#define AFX_DOMTREEVIEW_H__87BE3BB1_1B48_11D4_AC8D_00105A483E3C__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include <afxdlgs.h>
#include <afxcview.h>
#include <PhParse.hpp>

class CDomTreeView : public CTreeView
{

    CImageList              m_ImageList ;
    int                     m_nidTag;
    int                     m_nidAttribute ;
    int                     m_nidText ;
	int						m_nidScript;
    CFindReplaceDialog *    m_pFindDlg ;
	HTREEITEM				m_hMenuSelection;
    BOOL                    m_bInitialized;

protected: // create from serialization only
    CDomTreeView();

    DECLARE_DYNCREATE(CDomTreeView)

    // Attributes
public:

    void ShowDom();

    void InitTreeCtrlDisplay()  ;
    void InsertTag( CTag *pTag ,HTREEITEM hItem);
    HTREEITEM InsertTagInTree( CTag* pTag ,HTREEITEM hItem);
    void InsertAttribute( CTag* pTag,HTREEITEM hItem) ;
    void InsertText( CTag* pTag,HTREEITEM hItem) ;
    void InsertScript( CTag* pTag,HTREEITEM hItem) ;
	void ExpandAllChildren(HTREEITEM hItem);

    // Operations
public:

    // Overrides
    // ClassWizard generated virtual function overrides
    //{{AFX_VIRTUAL(CDomTreeView)
public:
    virtual void OnDraw(CDC* pDC);  // overridden to draw this view
    virtual BOOL PreCreateWindow(CREATESTRUCT& cs);
    virtual void OnInitialUpdate();
protected:
    virtual void OnUpdate( CView* pSender, LPARAM lHint, CObject* pHint );
    //}}AFX_VIRTUAL

    //Add for search functionnality
public:
    HTREEITEM GetPrevItem( HTREEITEM hItem );
    HTREEITEM GetNextItem( HTREEITEM hItem, UINT nCode );
    HTREEITEM GetNextItem( HTREEITEM hItem );
    HTREEITEM GetLastItem( HTREEITEM hItem );
    virtual HTREEITEM FindItem( CString &sSearch,
                                BOOL bCaseSensitive = FALSE,
                                BOOL bDownDir = TRUE,
                                BOOL bWholeWord = FALSE,
                                HTREEITEM hItem = NULL );

protected:
    virtual BOOL IsFindValid( HTREEITEM );

    // Implementation
public:
    virtual ~CDomTreeView();
#ifdef _DEBUG
    virtual void AssertValid() const;
    virtual void Dump(CDumpContext& dc) const;
#endif

protected:

    // Generated message map functions
protected:
    //{{AFX_MSG(CDomTreeView)
    afx_msg void OnAddressing() ;
    afx_msg void OnUpdateSearch( CCmdUI* pCmdUI ) ;
    afx_msg void OnInitSearch() ;
	afx_msg void OnContextMenu(CWnd* pWnd, CPoint point);
	afx_msg void OnRclick(NMHDR* pNMHDR, LRESULT* pResult);
	afx_msg void OnClipboardAddress();
	//}}AFX_MSG
    afx_msg LRESULT OnDoSearch( WPARAM w , LPARAM pText ) ;

    DECLARE_MESSAGE_MAP()
};


/////////////////////////////////////////////////////////////////////////////

//{{AFX_INSERT_LOCATION}}
// Microsoft Visual C++ will insert additional declarations immediately before the previous line.

#endif // !defined(AFX_DOMTREEVIEW_H__87BE3BB1_1B48_11D4_AC8D_00105A483E3C__INCLUDED_)
