/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

// GatewayObject.cpp: implementation of the CGatewayObject class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include <malloc.h>
#include "modeller.h"
#include "GatewayObject.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

#include <ConstantesLib.h>
#include "Constantes.h"
#include "CSerialization.h"
#include "ModellerDoc.h"
#include <IE4Information.h>
#include <IE5Information.h>
#include <IE6Information.h>
#include <NS47Information.h>
#include "Tof2Scl.h"

extern CModellerApp theApp ;


//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////
//--------------------------------------------------------------------

CGatewayObject::CGatewayObject(CModellerDoc *_pDoc )
        :   m_pDoc( _pDoc )                      ,
        m_bPause( TRUE )                     ,
        m_pBrowserInfo( NULL )
{
}

//--------------------------------------------------------------------


CGatewayObject::~CGatewayObject()
{
    if( m_pBrowserInfo )
        delete m_pBrowserInfo ;

}

//--------------------------------------------------------------------

BOOL CGatewayObject::GetDefaultParameters( _TPreferencesProxyData *pData )
{
    if( m_pBrowserInfo )
    {
        delete m_pBrowserInfo ;
        m_pBrowserInfo  = NULL ;
    }
    switch( m_pDoc->GetSCLDocInfo()->GetBrowserID() )
    {
    case ID_BROWSER_IE4:
        m_pBrowserInfo = static_cast<CBrowserInformation*> (new CIE4Information(AfxGetMainWnd())) ;
        break ;
    case ID_BROWSER_IE5 :
        m_pBrowserInfo = static_cast<CBrowserInformation*> (new CIE5Information(AfxGetMainWnd())) ;
        break ;
    case ID_BROWSER_IE6 :
        m_pBrowserInfo = static_cast<CBrowserInformation*> (new CIE6Information(AfxGetMainWnd())) ;
        break ;
    case ID_BROWSER_NS47 :
        m_pBrowserInfo = static_cast<CBrowserInformation*> (new CNS47Information(AfxGetMainWnd())) ;
        break ;
    default:
        ASSERT(FALSE);
        m_pBrowserInfo = static_cast<CBrowserInformation*> (new CIE5Information(AfxGetMainWnd())) ;
        break;
    }
    return m_pBrowserInfo->GetDefaultParameters( pData );
}

//--------------------------------------------------------------------
#pragma optimize("g", off)
BOOL CGatewayObject::ConfigureGateway(BOOL _bConsole, BOOL _bTrace,BOOL _bAutoCookieGen, BOOL _bCommentCookie)
{
    BOOL bRet;

    //1) chemin court de la gateway
    TCHAR  szModellerPath[_MAX_PATH];
    TCHAR  szShortGatewayPath[_MAX_PATH];
    char   szdrive[_MAX_DRIVE];
    char   szdir[_MAX_DIR];

    CString     strGatewayPath;
    string   strShortGatewayPath;
    // ----

    VERIFY(::GetModuleFileName(AfxGetApp()->m_hInstance, szModellerPath, _MAX_PATH));

    _splitpath( szModellerPath, szdrive, szdir, NULL, NULL );

    strcpy(szModellerPath,szdrive);
    strcat(szModellerPath,szdir);


    strGatewayPath = GetExtendedProfileString(HKEY_CURRENT_USER, theApp.GetRegistryPath(),
                     STR_SCL_GATEWAY_ENTRY, szModellerPath);

    GetShortPathName( (LPCSTR)strGatewayPath,
                      szShortGatewayPath,
                      _MAX_PATH);

    strShortGatewayPath = szShortGatewayPath;

    try
    {
        theApp.m_CaptureGateway.setAdminPort((port_t)m_Data.m_lAdministrationPort ) ;
    }
    catch( CTException &e)
    {
        DisplayGatewayMessage( e ) ;
        return FALSE;
    }

    try
    {
        bRet = theApp.m_CaptureGateway.startGateway(strShortGatewayPath, _bConsole?true:false, _bTrace?true:false) ;
		theApp.m_CaptureGateway.automaticCookiesModelling(_bAutoCookieGen ? true:false, _bCommentCookie ? true:false);
    }
    catch( CTException &e)
    {
        DisplayGatewayMessage( e ) ;
        bRet = FALSE ;
    }

    if( !bRet )
    {
        DisplayErrorCaptureMessage( IDS_CANNOT_START_GATEWAY ) ;
        return bRet ;
    }

    try
    {
        bRet = theApp.m_CaptureGateway.listen((port_t)m_Data.m_lListenPort ) ;
    }
    catch( CTException &e)
    {
        DisplayGatewayMessage( e ) ;
        bRet = FALSE ;
    }

    if( !bRet )
    {
        DisplayErrorCaptureMessage( IDS_BAD_LISTEN_PORT ) ;
        return bRet ;
    }

    CString strPath ;
    strPath  = m_pDoc->GetSCLDocInfo()->GetWorkingFolder();
    strPath += m_pDoc->GetSCLDocInfo()->GetSCLName() ;

    try
    {
        theApp.m_CaptureGateway.setDocumentsPath((string) strPath ) ;
    }
    catch( CTException &e)
    {
        DisplayGatewayMessage( e ) ;
        return bRet ;
    }

    CString strHTTP ;
    CString strHTTPS ;
    CString strException ;
    BOOL    bNoProxyForLocal ;

    strHTTP  = _T(":0");
    strHTTPS = _T(":0");

    if( m_Data.m_bLocal )
    {
        if( !m_Data.m_ProxyInfo.m_LocalProxy.m_strProxyAddress.IsEmpty() )
            strHTTP.Format( "%s:%ld", m_Data.m_ProxyInfo.m_LocalProxy.m_strProxyAddress ,
                            m_Data.m_ProxyInfo.m_LocalProxy.m_lAddressPort ) ;


        if( !m_Data.m_ProxyInfo.m_LocalProxy.m_strProxySecure.IsEmpty() )
            strHTTPS.Format( "%s:%ld", m_Data.m_ProxyInfo.m_LocalProxy.m_strProxySecure ,
                             m_Data.m_ProxyInfo.m_LocalProxy.m_lSecurePort ) ;

        bNoProxyForLocal = m_Data.m_ProxyInfo.m_LocalProxy.m_bByPassProxyLocal ;
        strException = m_Data.m_ProxyInfo.m_LocalProxy.m_strAddressesNoProxy ;
    }
    else
    {
        if( !m_Data.m_ProxyInfo.m_RemoteProxy.m_strProxyAddress.IsEmpty() )
            strHTTP.Format( "%s:%ld", m_Data.m_ProxyInfo.m_RemoteProxy.m_strProxyAddress ,
                            m_Data.m_ProxyInfo.m_RemoteProxy.m_lAddressPort ) ;

        if( !m_Data.m_ProxyInfo.m_RemoteProxy.m_strProxySecure.IsEmpty() )
            strHTTPS.Format( "%s:%ld", m_Data.m_ProxyInfo.m_RemoteProxy.m_strProxySecure ,
                             m_Data.m_ProxyInfo.m_RemoteProxy.m_lSecurePort ) ;

        bNoProxyForLocal = m_Data.m_ProxyInfo.m_RemoteProxy.m_bByPassProxyLocal ;
        strException = m_Data.m_ProxyInfo.m_RemoteProxy.m_strAddressesNoProxy ;
    }

    try
    {
		// This call is the reason to turn "g" optimization off
		// STLport 4.5.3 string causes a C1001 compiler error in VC6sp5
        bRet = theApp.m_CaptureGateway.browserParameters( (string)strHTTP,
                (string)strHTTPS,
                (string)strException,
                bNoProxyForLocal?true:false ) ;
    }
    catch( CTException &e)
    {
        DisplayGatewayMessage( e ) ;
        bRet = FALSE ;
    }

    if( !bRet )
    {
        DisplayErrorCaptureMessage( IDS_BAD_BROWSER_PARAM ) ;
        return bRet ;
    }
    return bRet ;
}
#pragma optimize("", on)
//--------------------------------------------------------------------

BOOL CGatewayObject::RunGateway()
{
    CString strTofFile ;
    BOOL bRet = TRUE ;
    //---
    strTofFile  = m_pDoc->GetSCLDocInfo()->GetWorkingFolder();
    strTofFile += m_pDoc->GetSCLDocInfo()->GetTOFFileName() ;

    CStdioFile file( strTofFile , CFile::modeCreate|CFile::modeWrite| CFile::typeText ) ;
    //---
    file.Close() ;

    try
    {
        bRet = theApp.m_CaptureGateway.record( (string)strTofFile , 0 , (string)m_pDoc->GetSCLDocInfo()->GetSCLFileName() ) ;
    }
    catch( CTException &e)
    {
        DisplayGatewayMessage( e ) ;
        bRet = FALSE ;
    }

    if( !bRet )
    {
        DisplayErrorCaptureMessage( IDS_CANNOT_RECORD ) ;
        return bRet;
    }
    return bRet ;

}

//--------------------------------------------------------------------

void CGatewayObject::ExecuteCommand( const CString &strExe )
{
    STARTUPINFO     StartupInfo ;
    CString         strCommandLine ;
    HANDLE          hProcess;
    TCHAR           *pczCommandLine ;
    TCHAR           pczBuffer[_MAX_PATH] ;
    TCHAR           pczShortPath[_MAX_PATH];

    //---
    strCommandLine = strExe ;
    GetCurrentDirectory( _MAX_PATH - 1 ,
                         pczBuffer);

    GetShortPathName(   (LPCSTR)pczBuffer,
                        pczShortPath,
                        _MAX_PATH);

    memset(&StartupInfo, 0, sizeof(StartupInfo));
    StartupInfo.cb = sizeof(STARTUPINFO);
    StartupInfo.wShowWindow = (WORD)SW_SHOW;
    StartupInfo.dwFlags = STARTF_USESHOWWINDOW;

    hProcess = OpenProcess(   PROCESS_QUERY_INFORMATION,
                              TRUE,
                              theApp.m_ProcessInformation.dwProcessId
                          );


    if ( hProcess == NULL )
    {
        // Copy the command line, because CreateProcess may modify it directly!
        pczCommandLine = (char*)_alloca(strCommandLine.GetLength() + 1);
        lstrcpy(pczCommandLine, strCommandLine);

        if( !CreateProcess(  NULL ,               // Application Name
                             pczCommandLine,       // command line string
                             NULL,                 // process security attributes
                             NULL,                 // thread security attributes
                             FALSE,                // handle inheritance flag
                             NORMAL_PRIORITY_CLASS,// creation flags
                             NULL ,  // pointer to new environment block
                             pczShortPath,   // pointer to current directory name
                             &StartupInfo,  // pointer to STARTUPINFO
                             &theApp.m_ProcessInformation  // pointer to PROCESS_INFORMATION
                          ) )
        {
            CString strText ;
            CString strTemp ;
            CString strError ;
            //-----
            strTemp.LoadString( IDS_CANNOT_LAUNCH_BROWSER ) ;
            strText.Format( strTemp , strCommandLine ) ;
            strError.LoadString( IDS_ERROR ) ;
            AfxGetMainWnd()->MessageBox( strText , strError , MB_OK ) ;
        }
    }
    else
        CloseHandle(hProcess);
}

//--------------------------------------------------------------------

BOOL CGatewayObject::LaunchBrowser()
{
    CString strBrowserPath ;
    BOOL bRet = FALSE;

    //-----
    if( m_pBrowserInfo->GetBrowserFullPath( strBrowserPath ) )
    {
        ExecuteCommand( strBrowserPath ) ;
        bRet = TRUE ;
    }
    return bRet;
}

//--------------------------------------------------------------------

BOOL CGatewayObject::Record(BOOL _bConsole, BOOL _bTrace,BOOL _bAutoCookieGen, BOOL _bCommentCookie)
{
    if( !m_pBrowserInfo->IsInternetSettingOK() )
        return FALSE;

    m_bPause = TRUE ;
    //Get information provided by the preferences of the script
    theApp.GetGatewayInfomation( &m_Data ) ;
    //if the capture is local : save browser registry info and modify registry for capture
    if(theApp.IsCaptureLocal())
    {
        m_pBrowserInfo->SaveDefaultRegistryInfo() ;
        m_pBrowserInfo->ModifyRegistryForCapture(m_Data.m_lListenPort) ;
    }

    //configure the gateway with the new parameters and run it
    if( !ConfigureGateway(_bConsole, _bTrace,_bAutoCookieGen, _bCommentCookie) )
        return FALSE;

    if( !RunGateway() )
        return FALSE;

    //launch the browser if the capture is local
    if(theApp.IsCaptureLocal())
    {
		LoadCaptureBHO();
	    
		if( !LaunchBrowser() )
        {
            StopRecord() ;
            return FALSE;
        }
    }
    return TRUE;
}

//--------------------------------------------------------------------

void CGatewayObject::PauseRecord()
{
    BOOL bRet = TRUE;
    //---
    //pause gateway
    if( m_bPause )
    {
        try
        {
            bRet = theApp.m_CaptureGateway.pause() ;
        }
        catch( CTException &e)
        {
            DisplayGatewayMessage( e ) ;
            bRet = FALSE ;
        }

        if( !bRet)
        {
            DisplayErrorCaptureMessage( IDS_CANNOT_PAUSE ) ;
            return;
        }
    }
    else
    {
        try
        {
            bRet = theApp.m_CaptureGateway.resume() ;
        }
        catch( CTException &e)
        {
            DisplayGatewayMessage( e ) ;
            bRet = FALSE ;
        }

        if( !bRet )
        {
            DisplayErrorCaptureMessage( IDS_CANNOT_RESUME_PAUSE ) ;
            return;
        }
    }
    m_bPause = !m_bPause ;
}

//--------------------------------------------------------------------

BOOL CGatewayObject::QuitCommand()
{
    return TRUE ;
}

//--------------------------------------------------------------------

BOOL CGatewayObject::StopRecord()
{
    BOOL bRet = TRUE ;
    //----
    //stop gateway
    try
    {
        bRet = theApp.m_CaptureGateway.stop()  ;
    }
    catch( CTException &e)
    {
        DisplayGatewayMessage( e ) ;
        bRet = FALSE ;
    }

	LoadCaptureBHO(false);

    if( !bRet )
    {
        DisplayErrorCaptureMessage( IDS_CANNOT_STOP ) ;
        return FALSE;
    }

    //Close Explorer
    QuitCommand();

    //Reinit registry parameters
    m_pBrowserInfo->SetDefaultParametersInRegistry() ;
    return TRUE ;
}

//--------------------------------------------------------------------

void CGatewayObject::GetSCLRecordedScript( CString & strScript, bool bAddPageTimers )
{
    //1 - Copy Header.ini file in the project directory
    //We suppose Header.ini file is in the exe directory
    TCHAR pczBuffer[_MAX_PATH] ;
    CString strFromHeaderFile ;
    CString strToHeaderFile ;
    CString strFromSCLExe ;
    //CString strToSCLExe ;
    CString strTmpProjectPath ;
    //---
    GetCurrentDirectory( _MAX_PATH - 1 ,
                         pczBuffer);

    strFromHeaderFile   = pczBuffer ;
    strFromHeaderFile   += _T("\\" ) ;
    strFromHeaderFile   += STR_SCL_HEADER_FILE ;
    strTmpProjectPath   = m_pDoc->GetSCLDocInfo()->GetWorkingFolder() ;
    strToHeaderFile     += strTmpProjectPath ;
    strToHeaderFile     += STR_SCL_HEADER_FILE ;

    CopyFile(   strFromHeaderFile,
                strToHeaderFile, FALSE);

    //convert tof script to SCL
    CString strTofFile = strTmpProjectPath ;
    strTofFile += m_pDoc->GetSCLDocInfo()->GetTOFFileName() ;
    CString strSCLFile = strTmpProjectPath ;
    strSCLFile += m_pDoc->GetSCLDocInfo()->GetSCLFileName() ;

    //info sur le chemin du headers.ini
    TCHAR  szModellerPath[_MAX_PATH];
    TCHAR  szShortModellerPath[_MAX_PATH];
    char szdrive[_MAX_DRIVE];
    char szdir[_MAX_DIR];


    // ----
    VERIFY(::GetModuleFileName(AfxGetApp()->m_hInstance, szModellerPath, _MAX_PATH));

    _splitpath( szModellerPath, szdrive, szdir, NULL, NULL );

    strcpy(szModellerPath,szdrive);
    strcat(szModellerPath,szdir);


    GetShortPathName( szModellerPath,
                      szShortModellerPath,
                      _MAX_PATH);

    ::SetHeadersIniPath(szShortModellerPath);

    if( ParseTof(strTofFile, bAddPageTimers ) != 1 )
        return;

    CFile File ;
    //---
    if( !File.Open( strSCLFile , CFile::shareExclusive|CFile::modeReadWrite) )
        return;

    //Get and init script
    strScript.Empty() ;
    TCHAR *pczBrowser = NULL ;
    TCHAR *pczDate = NULL ;
    CString strDate ;
    int nCount = File.GetLength() ;
    TCHAR *pczBufferRead = new TCHAR[nCount + 1] ;
    //----
    File.SeekToBegin( );
    File.Read( pczBufferRead , nCount ) ;
    pczBufferRead[nCount] = '\0' ;

    //Write Browser
    switch( m_pDoc->GetSCLDocInfo()->GetBrowserID() )
    {
    case ID_BROWSER_IE4 :
        pczBrowser = new TCHAR[ strlen( STR_BROWSER_IE4 ) + 3] ;
        strcpy( pczBrowser , STR_BROWSER_IE4 ) ;
        strcat( pczBrowser , _T("\r\n") ) ;
        break ;
    case ID_BROWSER_IE5 :
        pczBrowser = new TCHAR[ strlen( STR_BROWSER_IE5 ) + 3] ;
        strcpy( pczBrowser , STR_BROWSER_IE5 ) ;
        strcat( pczBrowser , _T("\r\n") ) ;
        break ;
    case ID_BROWSER_IE6 :
        pczBrowser = new TCHAR[ strlen( STR_BROWSER_IE6 ) + 3] ;
        strcpy( pczBrowser , STR_BROWSER_IE6 ) ;
        strcat( pczBrowser , _T("\r\n") ) ;
        break ;
    case ID_BROWSER_NS47 :
        pczBrowser = new TCHAR[ strlen( STR_BROWSER_NS47 ) + 3] ;
        strcpy( pczBrowser , STR_BROWSER_NS47 ) ;
        strcat( pczBrowser , _T("\r\n") ) ;
        break ;
    default :
        ASSERT(FALSE);
        pczBrowser = NULL ;
        break ;
    }

    //Write Date
    strDate.LoadString( IDS_DATE ) ;
    strDate += COleDateTime::GetCurrentTime().Format( VAR_DATEVALUEONLY ) ;
    pczDate = new TCHAR[strDate.GetLength() + 3 ] ;
    strcpy( pczDate , strDate ) ;
    strcat( pczDate , _T("\r\n" ) );

    if( pczBrowser )
        nCount += strlen( pczBrowser ) ;
    if( pczDate )
        nCount += strlen( pczDate ) ;

    //wrtie new Buffer
    TCHAR *pczBufferWrite = new TCHAR[nCount + 1] ;
    //---
    if( pczBrowser )
        strcpy( pczBufferWrite , pczBrowser ) ;
    if( pczDate )
        strcat( pczBufferWrite , pczDate ) ;
    if( pczBufferRead )
        strcat( pczBufferWrite , pczBufferRead ) ;

    File.SeekToBegin( );
    File.Write( pczBufferWrite ,nCount ) ;

    strScript = pczBufferWrite ;
    File.Close() ;

    if(pczBufferWrite )
        delete pczBufferWrite ;
    if(pczBufferRead )
        delete pczBufferRead ;
    if(pczBrowser )
        delete pczBrowser ;
    if(pczDate )
        delete pczDate ;

}

//--------------------------------------------------------------------

void CGatewayObject::InsertComments( const CString &strComments, unsigned long ulMillisecond )
{
    BOOL bRet = TRUE ;
    //send comments to gateway
    try
    {
        bRet = theApp.m_CaptureGateway.comment( (string)strComments, ulMillisecond ) ;
    }
    catch( CTException &e)
    {
        DisplayGatewayMessage( e ) ;
        bRet = FALSE ;
    }
    if( !bRet )
    {
        DisplayErrorCaptureMessage( IDS_CANNOT_INSERT_COMMENT ) ;
        return;
    }

}

//--------------------------------------------------------------------

void CGatewayObject::DisplayErrorCaptureMessage( long lID )
{
    CString strText ;
    CString strError ;
    //----
    strText.LoadString( lID ) ;
    strError.LoadString( IDS_ERROR ) ;
    AfxGetMainWnd()->MessageBox( strText , strError , MB_OK ) ;
}

//--------------------------------------------------------------------

void CGatewayObject::DisplayGatewayMessage( const CTException& e )
{
    CString strMessage ;
    CString strWarning ;
    //---
    strWarning.LoadString( IDS_WARNING ) ;
    strMessage = (CString)( e.getMsg().c_str() );
    AfxGetMainWnd()->MessageBox( strMessage , strWarning , MB_OK ) ;
}

//--------------------------------------------------------------------

void CGatewayObject::LoadCaptureBHO(bool bLoad /*=true*/)
{
	typedef HRESULT (__stdcall *PFNCALL) ();
	HMODULE hDLL = LoadLibrary("CaptureBHO.dll");
	if(hDLL)
	{
		CString strFn = (bLoad ? "DllRegisterServer" : "DllUnregisterServer");
		
		PFNCALL pfn = (PFNCALL)GetProcAddress(hDLL,strFn);

		if(pfn)
			pfn();

		FreeLibrary(hDLL);
	}
	else
	{
		AfxMessageBox("Could not load CaptureBHO.dll",MB_ICONEXCLAMATION);
	}
}

void CGatewayObject::StartTimer(const CString &strTimer, unsigned long ulMillisecond )
{
    BOOL bRet = TRUE ;
    //send Start Timer to the gateway
    try
    {
        bRet = theApp.m_CaptureGateway.StartTimer( (string)strTimer, ulMillisecond ) ;
    }
    catch( CTException &e)
    {
        DisplayGatewayMessage( e ) ;
        bRet = FALSE ;
    }
    if( !bRet )
    {
        DisplayErrorCaptureMessage( IDS_CANNOT_START_TIMER ) ;
        return;
    }
}

void CGatewayObject::EndTimer(const CString &strTimer, unsigned long ulMillisecond )
{
    BOOL bRet = TRUE ;
    //send End Timer to the gateway
    try
    {
        bRet = theApp.m_CaptureGateway.EndTimer( (string)strTimer, ulMillisecond ) ;
    }
    catch( CTException &e)
    {
        DisplayGatewayMessage( e ) ;
        bRet = FALSE ;
    }
    if( !bRet )
    {
        DisplayErrorCaptureMessage( IDS_CANNOT_END_TIMER ) ;
        return;
    }
}