/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

// Modeller.h : main header file for the MODELLER application
//

#if !defined(AFX_MODELLER_H__9D2B815B_F0E1_11D3_AC83_00105A483E3C__INCLUDED_)
#define AFX_MODELLER_H__9D2B815B_F0E1_11D3_AC83_00105A483E3C__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#ifndef __AFXWIN_H__
#error include 'stdafx.h' before including this file for PCH
#endif

#include "resource.h"       // main symbols
#include <CSerialization.h>
#include <RepositoryAPI.h>
#include "GatewayObject.h"

#include "EnumModellerData.h"

/////////////////////////////////////////////////////////////////////////////
// CModellerApp:
// See Modeller.cpp for the implementation of this class
//


class CModellerApp : public CWinApp
{
    //Attributes
public:

	void OpenFile(const CString& strPath);

    HttpCaptureCmd          m_CaptureGateway ;
    PROCESS_INFORMATION     m_ProcessInformation;
    BOOL                    m_bRegisterClass ;
    CString                 m_strClassName ;

    //State of the browser before the application is launched
    BOOL                    m_bOffLine ;

    CRepositoryAPI         m_RepositoryAPI ;
    CMultiDocTemplate*      m_pCaptureTemplate;
    CMultiDocTemplate*      m_pSubroutineTemplate;

    CCSerialization         m_Serialization ;

    BOOL                    m_bRecording;
    BOOL                    m_bReplay;
    BOOL                    m_bTrace;
    BOOL                    m_bConsole;
	BOOL					m_bAutoCookieGeneration;
	BOOL					m_bCommentCookie;
	BOOL					m_bAddPageTimers;

    CString                 m_strVariablePrefix;
    int                     m_nBrowserID;

protected:

    BOOL InitializeApp();

    void ReadPreferences();
    void SavePreferences();
public:
    CModellerApp();
    ~CModellerApp();

    _TPreferencesGatewayData             m_GatewayData;

    BOOL IsCaptureLocal();

    //fill the gateway parameters defined by the user for the gateway object.
    void GetGatewayInfomation( _TPreferencesGatewayData *pData ) ;

    BOOL DoesDocumentExist(CString _strFileName);
    void OpenDocument(CDocTemplate *pTemplate ,CString _strFileName);
    void OpenDocumentFromArgs(const CString& _strFileName) ;
    void UpdateWindowText();

    CString GetRegistryPath() ;

    inline CRepositoryAPI* GetRepositoryAPI()
    {
        return &m_RepositoryAPI ;
    } ;
    inline CCSerialization* GetSerialization()
    {
        return &m_Serialization ;
    };

    CWnd *FirstInstance() ;
    BOOL ApplyProjectInfo( const CString& strArgList ) ;

    inline const CString& GetClassName()
    {
        return m_strClassName ;
    } ;

    void SetGlobalOffLine( BOOL bOffLine ) ;
    BOOL IsGlobalOffLine() ;

    BOOL IsDocumentOpen(CString _strFileName, BOOL bActivate = FALSE);
    // Overrides
    // ClassWizard generated virtual function overrides
    //{{AFX_VIRTUAL(CModellerApp)
public:
    virtual BOOL InitInstance();
    virtual int ExitInstance();
    //}}AFX_VIRTUAL

    // Implementation
    //{{AFX_MSG(CModellerApp)
    afx_msg void OnAppAbout();
    afx_msg void OnFileNew( );
    afx_msg void OnFileOpen( );
    afx_msg void OnOptionsBrowser();
    afx_msg void OnOptionsGateway();
    afx_msg void OnOptionsVariables();
    afx_msg void OnToolsEdit();
    //}}AFX_MSG
    DECLARE_MESSAGE_MAP()
};


/////////////////////////////////////////////////////////////////////////////

//{{AFX_INSERT_LOCATION}}
// Microsoft Visual C++ will insert additional declarations immediately before the previous line.

#endif // !defined(AFX_MODELLER_H__9D2B815B_F0E1_11D3_AC83_00105A483E3C__INCLUDED_)
