/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

// OutputCheckView.cpp : implementation file
//

#include "stdafx.h"
#include "Modeller.h"
#include "VariousObject.h"
#include "OutputCheckView.h"

#include "Constantes.h"
#include "ModellerDoc.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

extern CModellerApp theApp ;


//---------------------------------------------------------------------------

BEGIN_MESSAGE_MAP(COutputCheckExtendedEditCtrl, COSEditCtrl)
//{{AFX_MSG_MAP(COutputCheckExtendedEditCtrl)
ON_WM_LBUTTONDBLCLK()
//}}AFX_MSG_MAP
END_MESSAGE_MAP()

//---------------------------------------------------------------------------

COutputCheckExtendedEditCtrl::COutputCheckExtendedEditCtrl(CView* pParent ,LPCSTR pczRegistryPath  ):
        COSEditCtrl( pczRegistryPath ),
        m_pParent( pParent )
{}

//---------------------------------------------------------------------------


COutputCheckExtendedEditCtrl::~COutputCheckExtendedEditCtrl()
{}



//---------------------------------------------------------------------------


void COutputCheckExtendedEditCtrl::DoOwnerContextMenu( CMenu *pMenu )
{
    /*$$$OSCExtendedEditCtrl::DoOwnerContextMenu( pMenu ) ;
    //---
    int nCount = pMenu->GetMenuItemCount( ) ;
    pMenu->RemoveMenu( nCount - 2 , MF_BYPOSITION );
    pMenu->RemoveMenu( nCount - 2 , MF_BYPOSITION );*/
}

//---------------------------------------------------------------------------

void COutputCheckExtendedEditCtrl::OnLButtonDblClk(UINT nFlags, CPoint point)
{
    int nCurLine;

    nCurLine = GetCurrentLine();
    SelectLine( nCurLine );

    CString strText = GetString( nCurLine, nCurLine );
    if( strText.Find( STR_ERROR ) >= 0 )
    {
		char *szFile = strtok(strText.GetBuffer(0),"(");
		char *szLine = strtok(NULL,")");
		if(!szFile || !szLine)
			return;

		int nRow = atoi(szLine);

		CModellerDoc* pDoc = static_cast<CModellerDoc*>(m_pParent->GetDocument() );
		if(pDoc->GetSCLDocInfo()->GetSCLFileName().CompareNoCase(szFile)==0)
	        pDoc->SetErrorLine( nRow ) ;
    }

    COSEditCtrl::OnLButtonDblClk(nFlags, point);
}


//---------------------------------------------------------------------------

IMPLEMENT_DYNCREATE(COutputCheckView, CView)

//---------------------------------------------------------------------------

BEGIN_MESSAGE_MAP(COutputCheckView, CView)
//{{AFX_MSG_MAP(COutputCheckView)
ON_WM_CREATE()
ON_WM_SIZE()
ON_COMMAND(ID_EDIT_COPY, OnEditCopy)
ON_COMMAND(ID_EDIT_CUT, OnEditCut)
ON_COMMAND(ID_EDIT_FIND, OnEditFind)
ON_COMMAND(ID_EDIT_PASTE, OnEditPaste)
ON_COMMAND(ID_EDIT_REDO, OnEditRedo)
ON_COMMAND(ID_EDIT_REPLACE, OnEditReplace)
ON_COMMAND(ID_EDIT_UNDO, OnEditUndo)
ON_UPDATE_COMMAND_UI(ID_EDIT_COPY, OnUpdateEditCopy)
ON_UPDATE_COMMAND_UI(ID_EDIT_CUT, OnUpdateEditCut)
ON_UPDATE_COMMAND_UI(ID_EDIT_FIND, OnUpdateEditFind)
ON_UPDATE_COMMAND_UI(ID_EDIT_PASTE, OnUpdateEditPaste)
ON_UPDATE_COMMAND_UI(ID_EDIT_REDO, OnUpdateEditRedo)
ON_UPDATE_COMMAND_UI(ID_EDIT_REPLACE, OnUpdateEditReplace)
ON_UPDATE_COMMAND_UI(ID_EDIT_UNDO, OnUpdateEditUndo)
//}}AFX_MSG_MAP
END_MESSAGE_MAP()

//---------------------------------------------------------------------------
#pragma warning(disable: 4355) // We know we shouldn't pass "this" back yet
COutputCheckView::COutputCheckView()
        :m_Editor( this , theApp.GetRegistryPath() )
{}
#pragma warning(default: 4355)

//---------------------------------------------------------------------------


COutputCheckView::~COutputCheckView()
{}

/////////////////////////////////////////////////////////////////////////////
// COutputCheckView drawing
//---------------------------------------------------------------------------


void COutputCheckView::OnDraw(CDC* pDC)
{
    CDocument* pDoc = GetDocument();
    // TODO: add draw code here
}

/////////////////////////////////////////////////////////////////////////////
// COutputCheckView diagnostics
//---------------------------------------------------------------------------


#ifdef _DEBUG
void COutputCheckView::AssertValid() const
{
    CView::AssertValid();
}

//---------------------------------------------------------------------------

void COutputCheckView::Dump(CDumpContext& dc) const
{
    CView::Dump(dc);
}
#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// COutputCheckView message handlers
//---------------------------------------------------------------------------

void COutputCheckView::OnUpdate(CView* pSender, LPARAM lHint, CObject* pHint)
{
    if( lHint == ID_SCL_CHECKER )
	{
		CString str(((CStringObject*)pHint)->m_strObject);
		if(!str.IsEmpty())
			str = m_Editor.GetWindowText() + "\n" + str;

		m_Editor.SetWindowText(str) ;
	}
}

//---------------------------------------------------------------------------

COSEditCtrl* COutputCheckView::GetEditCtrl()
{
    return &m_Editor ;
}

//---------------------------------------------------------------------------

BOOL COutputCheckView::OnCmdMsg(UINT nID, int nCode, void* pExtra, AFX_CMDHANDLERINFO* pHandlerInfo)
{
    if(m_Editor.OnCmdMsg(nID,nCode,pExtra,pHandlerInfo))
        return true;

    return CView::OnCmdMsg(nID, nCode, pExtra, pHandlerInfo);
}

//---------------------------------------------------------------------------

int COutputCheckView::OnCreate(LPCREATESTRUCT lpCreateStruct)
{
    if (CView::OnCreate(lpCreateStruct) == -1)
        return -1;

    if ( m_Editor.Create( WS_CHILD | WS_VISIBLE | WS_VSCROLL, CRect( 0, 0, 100, 100 ), this, 100 ) == -1 )
        return -1;

    m_Editor.SetReadOnly(TRUE);

    return 0;
}

//---------------------------------------------------------------------------

void COutputCheckView::OnSize(UINT nType, int cx, int cy)
{
    CView::OnSize(nType, cx, cy);

    if ( m_Editor.m_hWnd )
        m_Editor.SetWindowPos( NULL, -1, -1, cx, cy, SWP_NOZORDER | SWP_NOMOVE );
}

//---------------------------------------------------------------------------

void COutputCheckView::OnEditCopy()
{
    m_Editor.Copy();

}

//---------------------------------------------------------------------------

void COutputCheckView::OnEditCut()
{
    m_Editor.Cut();
}

//---------------------------------------------------------------------------

void COutputCheckView::OnEditFind()
{
    m_Editor.SendMessage(CMM_EXECUTECMD, CMD_FIND, 0 );
}

//---------------------------------------------------------------------------

void COutputCheckView::OnEditPaste()
{
    m_Editor.Paste();
}

//---------------------------------------------------------------------------

void COutputCheckView::OnEditRedo()
{
    m_Editor.Redo();
}

//---------------------------------------------------------------------------

void COutputCheckView::OnEditReplace()
{
    ::SendMessage( m_Editor.m_hWnd, CMM_EXECUTECMD, CMD_FINDREPLACE, 0 );
}

//---------------------------------------------------------------------------

void COutputCheckView::OnEditUndo()
{
    m_Editor.Undo();

}

//---------------------------------------------------------------------------

void COutputCheckView::OnUpdateEditCopy(CCmdUI* pCmdUI)
{
    pCmdUI->Enable(m_Editor.CanCopy());
}

//---------------------------------------------------------------------------

void COutputCheckView::OnUpdateEditCut(CCmdUI* pCmdUI)
{
    pCmdUI->Enable(m_Editor.CanCut());
}

//---------------------------------------------------------------------------

void COutputCheckView::OnUpdateEditFind(CCmdUI* pCmdUI)
{
    pCmdUI->Enable(TRUE);
}

//---------------------------------------------------------------------------

void COutputCheckView::OnUpdateEditPaste(CCmdUI* pCmdUI)
{
    pCmdUI->Enable(m_Editor.CanPaste());

}

//---------------------------------------------------------------------------

void COutputCheckView::OnUpdateEditRedo(CCmdUI* pCmdUI)
{
    pCmdUI->Enable(m_Editor.CanRedo());
}

//---------------------------------------------------------------------------

void COutputCheckView::OnUpdateEditReplace(CCmdUI* pCmdUI)
{
    pCmdUI->Enable(!m_Editor.IsReadOnly());
}

//---------------------------------------------------------------------------

void COutputCheckView::OnUpdateEditUndo(CCmdUI* pCmdUI)
{
    pCmdUI->Enable(m_Editor.CanUndo());
}
