/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

// SCLCaptureScriptView.cpp : implementation file
//

#include "stdafx.h"
#include <ConstantesLib.h>
#include "Modeller.h"
#include "SCLCaptureScriptView.h"
#include "Constantes.h"
#include "ModellerDoc.h"
#include "VariousObject.h"
#include "ChildFrm.h"
#include "FindInStringsDlg.h"


#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

extern CModellerApp theApp ;

#define ID_UPDATE_TIMER 2

/////////////////////////////////////////////////////////////////////////////
// CSCLCaptureScriptView
//---------------------------------------------------------------------------

IMPLEMENT_DYNCREATE(CSCLCaptureScriptView, CView)

//---------------------------------------------------------------------------

CSCLCaptureScriptView::CSCLCaptureScriptView():
        m_Editor(theApp.GetRegistryPath() )
{}

//---------------------------------------------------------------------------


CSCLCaptureScriptView::~CSCLCaptureScriptView()
{}

//---------------------------------------------------------------------------


BEGIN_MESSAGE_MAP(CSCLCaptureScriptView, CView)
//{{AFX_MSG_MAP(CSCLCaptureScriptView)
ON_COMMAND(IDM_TOGGLEBOOKMARK, OnTogglebookmark)
ON_UPDATE_COMMAND_UI(IDM_TOGGLEBOOKMARK, OnUpdateTogglebookmark)
ON_COMMAND(IDM_GOTO_URL , OnGoToURL )
ON_UPDATE_COMMAND_UI(IDM_GOTO_URL , OnUpdateGoToURL )
ON_WM_CREATE()
ON_WM_SIZE()
ON_COMMAND(ID_EDIT_COPY, OnEditCopy)
ON_COMMAND(ID_EDIT_CUT, OnEditCut)
ON_COMMAND(ID_EDIT_FIND, OnEditFind)
ON_COMMAND(ID_EDIT_PASTE, OnEditPaste)
ON_COMMAND(ID_EDIT_REDO, OnEditRedo)
ON_COMMAND(ID_EDIT_REPLACE, OnEditReplace)
ON_COMMAND(ID_EDIT_UNDO, OnEditUndo)
ON_UPDATE_COMMAND_UI(ID_EDIT_COPY, OnUpdateEditCopy)
ON_UPDATE_COMMAND_UI(ID_EDIT_CUT, OnUpdateEditCut)
ON_UPDATE_COMMAND_UI(ID_EDIT_FIND, OnUpdateEditFind)
ON_UPDATE_COMMAND_UI(ID_EDIT_PASTE, OnUpdateEditPaste)
ON_UPDATE_COMMAND_UI(ID_EDIT_REDO, OnUpdateEditRedo)
ON_UPDATE_COMMAND_UI(ID_EDIT_REPLACE, OnUpdateEditReplace)
ON_UPDATE_COMMAND_UI(ID_EDIT_UNDO, OnUpdateEditUndo)
ON_WM_TIMER()
	ON_COMMAND(ID_FIND_REPLACE_STRINGS, OnFindReplaceStrings)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CSCLCaptureScriptView drawing
//---------------------------------------------------------------------------

void CSCLCaptureScriptView::OnDraw(CDC* pDC)
{
    CDocument* pDoc = GetDocument();
    // TODO: add draw code here
}

/////////////////////////////////////////////////////////////////////////////
// CSCLCaptureScriptView diagnostics
//---------------------------------------------------------------------------



#ifdef _DEBUG
void CSCLCaptureScriptView::AssertValid() const
{
    CView::AssertValid();
}

//---------------------------------------------------------------------------

void CSCLCaptureScriptView::Dump(CDumpContext& dc) const
{
    CView::Dump(dc);
}
#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// CSCLCaptureScriptView message handlers
//---------------------------------------------------------------------------

COSEditCtrl* CSCLCaptureScriptView::GetEditCtrl()
{
    return &m_Editor ;
}

//---------------------------------------------------------------------------

void CSCLCaptureScriptView::OnUpdate(CView* pSender, LPARAM lHint, CObject* pHint)
{
    switch( lHint )
    {
    case ID_CHECKER_ERROR :
        {
			((CChildFrame*)GetParentFrame())->MDIActivate();
            long lRow = ((CLongObject*) (pHint ))->m_lObject ;
            m_Editor.GoToLine( lRow ) ;
            m_Editor.SelectLine(lRow,TRUE) ;
            m_Editor.SetFocus() ;
            break ;
        }
    }

}

//---------------------------------------------------------------------------

void CSCLCaptureScriptView::SetText(CString _strScript, BOOL _bModify)
{
    m_Editor.SetWindowText( _strScript) ;
    m_Editor.SetModified(_bModify);

    CModellerDoc* pDoc = static_cast<CModellerDoc*>(GetDocument());
    pDoc->Parse(TRUE);
    
    m_Editor.Invalidate();
}

//---------------------------------------------------------------------------

void CSCLCaptureScriptView::MarkLine(int nLine,BOOL bMark)
{
	//Can only have one marked line so clear current mark
	m_Editor.ClearAllMarks();

	//mark the new line
    m_Editor.ShowMarkOnLine(nLine,bMark);
}

//---------------------------------------------------------------------------

int CSCLCaptureScriptView::GetMarkedLine()
{
	int nNumLines = m_Editor.GetLineCount();
	for(int nLine=1;nLine<=nNumLines;nLine++)
		if(m_Editor.IsLineMarked(nLine))
			return nLine;

	return -1;
}

//---------------------------------------------------------------------------

void CSCLCaptureScriptView::OnInitialUpdate()
{
    CModellerDoc* pDoc = static_cast<CModellerDoc*>(GetDocument());
    ASSERT(pDoc != NULL);

    CView::OnInitialUpdate();

    m_Editor.ShowScrollBar( TRUE );
    m_Editor.SetWindowText("") ;

	// CL : 11/06/2002
	m_Editor.EnableSelBounds(TRUE);
	CM_FONTSTYLES fontStyle;
	memset(&fontStyle, 0, sizeof(CM_FONTSTYLES));
	fontStyle.byKeyword = CM_FONT_NORMAL;
	fontStyle.byComment = CM_FONT_ITALIC;
	m_Editor.SetFontStyles(&fontStyle);
	HFONT hFont;
	hFont = CreateFont( -12, 7, 0, 0, FW_NORMAL, 0, 0, 0, 0, OUT_STROKE_PRECIS, CLIP_STROKE_PRECIS, DRAFT_QUALITY, FF_MODERN | FIXED_PITCH, _T("Courier New") );
	::SendMessage( m_Editor.m_hWnd, WM_SETFONT, ( WPARAM ) hFont, 0 );
	CM_HOTKEY hotKey;
	hotKey.byModifiers1 = HOTKEYF_CONTROL;
	hotKey.nVirtKey1 = 'A';
	hotKey.byModifiers2 = 0;
	hotKey.nVirtKey2 = 0;
	CMRegisterHotKey( &hotKey, CMD_SELECTALL);
	// 

    pDoc->SetCaptureView(this);

    SetTimer(ID_UPDATE_TIMER,3000,NULL);
}

//---------------------------------------------------------------------------

void CSCLCaptureScriptView::OnActivateView(BOOL bActivate, CView* pActivateView, CView* pDeactiveView)
{
    CView::OnActivateView(bActivate, pActivateView, pDeactiveView);

	// TODO: Add your message handler code here
	// CL : 11/06/2002
	if ( bActivate != FALSE && pActivateView == this )
		m_Editor.SetFocus();
}

//---------------------------------------------------------------------------

void CSCLCaptureScriptView::OnTogglebookmark()
{
    int nCurLine;
    BOOL BookmarkSet;
    //---
    nCurLine    = m_Editor.GetCurrentLine();
    BookmarkSet = m_Editor.GetBookmark( nCurLine - 1 );
	m_Editor.SetBookmark( nCurLine - 1, !BookmarkSet );
}

//---------------------------------------------------------------------------

void CSCLCaptureScriptView::OnUpdateTogglebookmark(CCmdUI* pCmdUI)
{
    pCmdUI->Enable(TRUE);
}

//---------------------------------------------------------------------------

void CSCLCaptureScriptView::OnGoToURL()
{
    long lLine = m_Editor.GetCurrentLine();
    CModellerDoc* pDoc = static_cast<CModellerDoc*>(GetDocument());
    pDoc->GoToURL(lLine);

	// CL : 11/06/2002
	m_Editor.UpdateWindow();
}

//---------------------------------------------------------------------------

void CSCLCaptureScriptView::OnUpdateGoToURL( CCmdUI *pCmdUI )
{
    long lLine = m_Editor.GetCurrentLine() ;

    CModellerDoc* pDoc = static_cast<CModellerDoc*>(GetDocument());
    pDoc->OnUpdateGoToURL(lLine, pCmdUI );
}

//---------------------------------------------------------------------------

BOOL CSCLCaptureScriptView::OnCmdMsg(UINT nID, int nCode, void* pExtra, AFX_CMDHANDLERINFO* pHandlerInfo)
{
    if(m_Editor.OnCmdMsg(nID,nCode,pExtra,pHandlerInfo))
        return true;

    return CView::OnCmdMsg(nID, nCode, pExtra, pHandlerInfo);
}

//---------------------------------------------------------------------------

int CSCLCaptureScriptView::OnCreate(LPCREATESTRUCT lpCreateStruct)
{
	if (CView::OnCreate(lpCreateStruct) == -1)
        return -1;

    if (!m_Editor.Create( WS_CHILD | WS_VISIBLE | WS_VSCROLL, CRect( 0, 0, 100, 100 ), this, 100 ))
		return -1;

    return 0;
}

//---------------------------------------------------------------------------

void CSCLCaptureScriptView::OnSize(UINT nType, int cx, int cy)
{
    CView::OnSize(nType, cx, cy);

    if ( m_Editor.m_hWnd )
        m_Editor.SetWindowPos( NULL, -1, -1, cx, cy, SWP_NOZORDER | SWP_NOMOVE );

}

//---------------------------------------------------------------------------

void CSCLCaptureScriptView::OnEditCopy()
{
    m_Editor.Copy();

}

//---------------------------------------------------------------------------

void CSCLCaptureScriptView::OnEditCut()
{
    m_Editor.Cut();
}

//---------------------------------------------------------------------------

void CSCLCaptureScriptView::OnEditFind()
{
    m_Editor.SendMessage(CMM_EXECUTECMD, CMD_FIND, 0 );
}

//---------------------------------------------------------------------------

void CSCLCaptureScriptView::OnEditPaste()
{
    m_Editor.Paste();
}

//---------------------------------------------------------------------------

void CSCLCaptureScriptView::OnEditRedo()
{
    m_Editor.Redo();
}

//---------------------------------------------------------------------------

void CSCLCaptureScriptView::OnEditReplace()
{
    ::SendMessage( m_Editor.m_hWnd, CMM_EXECUTECMD, CMD_FINDREPLACE, 0 );
}

//---------------------------------------------------------------------------

void CSCLCaptureScriptView::OnEditUndo()
{
    m_Editor.Undo();

}

//---------------------------------------------------------------------------

void CSCLCaptureScriptView::OnUpdateEditCopy(CCmdUI* pCmdUI)
{
    pCmdUI->Enable(m_Editor.CanCopy());
}

//---------------------------------------------------------------------------

void CSCLCaptureScriptView::OnUpdateEditCut(CCmdUI* pCmdUI)
{
    pCmdUI->Enable(m_Editor.CanCut());
}

//---------------------------------------------------------------------------

void CSCLCaptureScriptView::OnUpdateEditFind(CCmdUI* pCmdUI)
{
    pCmdUI->Enable(TRUE);
}

//---------------------------------------------------------------------------

void CSCLCaptureScriptView::OnUpdateEditPaste(CCmdUI* pCmdUI)
{
    pCmdUI->Enable(m_Editor.CanPaste());

}

//---------------------------------------------------------------------------

void CSCLCaptureScriptView::OnUpdateEditRedo(CCmdUI* pCmdUI)
{
    pCmdUI->Enable(m_Editor.CanRedo());
}

//---------------------------------------------------------------------------

void CSCLCaptureScriptView::OnUpdateEditReplace(CCmdUI* pCmdUI)
{
    pCmdUI->Enable(TRUE);
}

//---------------------------------------------------------------------------

void CSCLCaptureScriptView::OnUpdateEditUndo(CCmdUI* pCmdUI)
{
    pCmdUI->Enable(m_Editor.CanUndo());
}

//---------------------------------------------------------------------------

void CSCLCaptureScriptView::OnTimer(UINT nIDEvent)
{
    switch(nIDEvent)
    {
    case ID_UPDATE_TIMER:
//        CModellerDoc* pDoc = static_cast<CModellerDoc*>(GetDocument());
  //      pDoc->Parse();
    //    ShowPrimaryURL();
        break;
    }

    CView::OnTimer(nIDEvent);
}

void CSCLCaptureScriptView::OnFindReplaceStrings() 
{
	CFindInStringsDlg dlg;
	if(dlg.DoModal() != IDOK)
		return;
	
	CString szScript = m_Editor.GetWindowText();

	char *pszCopy = new char[szScript.GetLength()+1];
	memcpy(pszCopy,szScript.GetBuffer(0),szScript.GetLength()+1);
	
	register char *pPos=pszCopy;

	char *pPrevPos=NULL;
	char *pStartPos=NULL;
	
	char *szExpanded=NULL;
	int nExpandedLen=0;

	while (*pPos)
	{
		if((*pPos == '\"') || (*pPos == '\'') )
		{
			if(!pStartPos)
			{
				pStartPos = pPos;
				pPrevPos = pPos;
			}
			else
			{
				int nNewLen = 0;
				if(!nExpandedLen)
					nNewLen = (pPos-pPrevPos)+2;	//Added one extra to avoid reallocation when adding terminating quote
				else
				{
					nNewLen = nExpandedLen+(pPos-pPrevPos);
					nExpandedLen -= 2;
				}

				char *szBuffer = new char[nNewLen];
				char *szBufferPos = szBuffer;
				memcpy(szBuffer,szExpanded,nExpandedLen);
				memcpy(&szBuffer[nExpandedLen],pPrevPos,(pPos-pPrevPos));
				szBuffer[nNewLen-2] = NULL;
				delete []szExpanded;
				
				nExpandedLen = nNewLen;
				szExpanded = szBuffer;
				
				//skip whitespace
				while(*(++pPos) && ((*pPos == '\t') || (*pPos == ' ')));

				if(*pPos == '&')	//Got a continuation mark so need to check if the string is continued
				{
					//skip to new line
					while(*(++pPos) && (*pPos != '\n'));
				
					//skip whitespace
					while(*(++pPos) && ((*pPos == '\t') || (*pPos == ' ')));
		
					//If first char on line is a quote the string is continued
					//otherwise do the replace
					if((*pPos != '\"') && (*pPos != '\''))
					{
						szExpanded[nExpandedLen-2] = szExpanded[0];	//Add terminating quote (string should be long enough)
						szExpanded[nExpandedLen-1] = '\0';
						
						//Do search and replace
						pPos = DoStringReplace(szExpanded,dlg.m_strFindText,dlg.m_strReplaceText,pStartPos,pPos,&pszCopy,dlg.m_bCaseSensitive);
						pStartPos = NULL;

						delete []szExpanded;
						szExpanded = NULL;
						nExpandedLen = 0;
					}
					else
						pPrevPos = pPos+1;
				}
				else	//If the next character is not a &, string is complete at this point
				{
					szExpanded[nExpandedLen-2] = szExpanded[0];	//Add terminating quote (string should be long enough)
					szExpanded[nExpandedLen-1] = '\0';

					pPos = DoStringReplace(szExpanded,dlg.m_strFindText,dlg.m_strReplaceText,pStartPos,pPos,&pszCopy,dlg.m_bCaseSensitive);

					pStartPos = NULL;

					delete []szExpanded;
					szExpanded = NULL;
					nExpandedLen = 0;
				}
			}
		}



		pPos++;
	}
	szScript = pszCopy;
	delete []pszCopy;

	CM_RANGE range;
	memset(&range,0,sizeof(range));
    range.posEnd.nLine = m_Editor.GetLineCount()-1;
	range.posEnd.nCol = m_Editor.GetLineLength(range.posEnd.nLine);
	m_Editor.SetSel(&range,false);
	m_Editor.ReplaceSel(szScript);
}

char* CSCLCaptureScriptView::DoStringReplace(char *szString,	//String to search and replace in
									  const char *szFind,		//string to find
									  const char* szReplace,	//string to replace with
									  char* szInsert,		//Insert point in script
									  char* szEnd,		//End point of string in script
									  char** szScript,	//entire script
									  BOOL bCaseSensitive)		
{
	char *szCurrentPos=szEnd;

	//Do search and replace
	int nNumFound =0;

	int nFindLen = strlen(szFind);
	int nReplaceLen = strlen(szReplace);

	char *szStartPos = NULL;
	const char *szFindPos = szFind;
	char *szStringToSearch = new char[strlen(szString)+1];
	strcpy(szStringToSearch,szString);
	char *szStringPos = szStringToSearch;
	while(*szStringPos)
	{
		if((*szStringPos == *szFindPos)		||
			(!bCaseSensitive && (
				( isupper(*szStringPos) && islower(*szFindPos) && ((*szStringPos+('a'-'A'))==*szFindPos)) ||
				( islower(*szStringPos) && isupper(*szFindPos) && ((*szStringPos-('a'-'A'))==*szFindPos)))))
		{
			if(!szStartPos)
				szStartPos = szStringPos;
			szFindPos++;		//matched another char in search string
		}
		else
		{
			szStartPos = NULL;
			szFindPos = szFind;	//reset search
		}

		szStringPos++;

		if(!(*szFindPos))
		{
			//found text so do the replace
			nNumFound++;

			if(nFindLen == nReplaceLen)
			{
				memcpy(szStartPos,szReplace,nReplaceLen);
			}
			else
			{
				int nNewLen = strlen(szStringToSearch) - nFindLen + nReplaceLen + 1;
				char *szNew = new char[nNewLen];
				char *szPos = szNew;
				memcpy(szPos,szStringToSearch,(szStartPos-szStringToSearch));
				szPos += szStartPos-szStringToSearch;

				memcpy(szPos,szReplace,strlen(szReplace));
				szPos += strlen(szReplace);

				strcpy(szPos,szStringPos);

				delete []szStringToSearch;
				szStringToSearch = szNew;
				szStringPos = szPos;
			}
			szStartPos = NULL;
			szFindPos = szFind;
		}
	}

	if(nNumFound)
	{
		//Limit length of lines in search string  
		BOOL bInQuote=false;
		int nMaxLength = 50;
		char *pPos = szStringToSearch;
		char *pPrevPos = pPos;
		char *pPrevQuote=NULL;

		int nDoubleQuotes = -1;
		while(*pPos)
		{
			if(nDoubleQuotes == -1)
			{
				if((*pPos == '\"') || (*pPos == '\''))
				{
					nDoubleQuotes = (*pPos == '\"');
				}
			}
			
			if( ((nDoubleQuotes==1) && (*pPos == '\"')) || (!nDoubleQuotes && (*pPos=='\'')))
			{
				pPrevQuote = pPos;
				bInQuote = !bInQuote;
			}
			
			if(pPos - pPrevPos > nMaxLength)
			{
				if( ((nDoubleQuotes==1) && (*pPos == '\"')) || (!nDoubleQuotes && (*pPos=='\'')))
				{
					if(bInQuote)	//just started string
						pPos--;
					else		//just finisihed string
						pPos++;

					bInQuote = false;
				}

				//create a new string with space for quotes,tabs,& and newline
				char *szNew = new char[strlen(szStringToSearch)+9]; 
				char *szNewPos = szNew;
				memcpy(szNewPos,szStringToSearch,pPos-szStringToSearch+1);
				szNewPos += pPos-szStringToSearch+1;

				if(bInQuote)
				{
					if(nDoubleQuotes)
						strcpy(szNewPos,"\"\t&\n\t\t\t\"");
					else
						strcpy(szNewPos,"\'\t&\n\t\t\t\'");
					szNewPos += 8;
				}
				else
				{
					strcpy(szNewPos," \t&\n\t\t\t ");
					szNewPos += 8;
				}

				strcpy(szNewPos,pPos+1);
				pPos = szNewPos;
				delete [] szStringToSearch;
				szStringToSearch = szNew;
				pPrevPos = pPos+1;
			}

			pPos++;
		}




		//Replace string in script
		int nNewStringLen = strlen(szStringToSearch);

		//If we replace anything the script will need to be resized
		int nNewLen = strlen(*szScript)+1-(szEnd-szInsert)+nNewStringLen;
	
		char *szBuffer = new char[nNewLen];
		char *szBufferPos = szBuffer;
		
		memcpy(szBufferPos,*szScript,szInsert-(*szScript));	//copy everything up to beginning of the string
		szBufferPos += szInsert-(*szScript);

		memcpy(szBufferPos,szStringToSearch,nNewStringLen);
		szBufferPos += nNewStringLen;

		strcpy(szBufferPos,szEnd);	//copy remainder of script back

		delete [](*szScript);
		
		szCurrentPos = &szBuffer[(szInsert-(*szScript))+nNewStringLen];
		*szScript = szBuffer;
	}
	delete []szStringToSearch;

	return szCurrentPos;
}