/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/


#include "stdafx.h"
#include "IECache.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CIECache::CIECache(LPCTSTR zCaptureFile)
        : m_strCaptureFile(zCaptureFile), m_pbCaptureData(NULL), m_pbBody(0),
        m_pcClientHeader(0), m_pcServerHeader(0), m_pbIndex(NULL)
{
    m_strCaptureFile.TrimLeft();
    m_strCaptureFile.TrimRight();
}

CIECache::~CIECache()
{
    if(m_pbCaptureData)
        UnmapViewOfFile(m_pbCaptureData);   // Releases the file mapping and file handle kernel objects.
}


BOOL CIECache::Init()
{
    BOOL bRes = FALSE;

    //// If a capture file was specified, then map it into memory,
    //// build the index.
    if(!m_strCaptureFile.IsEmpty())
    {
        HANDLE hCaptureFile = CreateFile(m_strCaptureFile, GENERIC_READ, FILE_SHARE_READ,
                                         NULL, OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL|FILE_FLAG_RANDOM_ACCESS, NULL);
        if(hCaptureFile != INVALID_HANDLE_VALUE)
        {
            HANDLE hMap = CreateFileMapping(hCaptureFile, NULL, PAGE_READONLY, 0, 0, NULL);
            if(hMap)
            {
                m_pbCaptureData = (LPBYTE)MapViewOfFile(hMap, FILE_MAP_READ, 0, 0, 0);
                if(m_pbCaptureData)
                {
                    //// The mapped view is ready. Now, build the index.
                    //// Here, all URLs are assumed to be canonical.

                    // Set pointers to the body, client headers and server headers sections.
                    LPDWORD pdwData  = (LPDWORD)m_pbCaptureData;
                    // pdwData[0] contains the last connection id used in the capture.
                    // This is useless here.
                    m_pbBody         = (LPBYTE)(m_pbCaptureData + pdwData[1]);
                    m_pcClientHeader = (LPCSTR)(m_pbCaptureData + pdwData[2]);
                    m_pcServerHeader = (LPCSTR)(m_pbCaptureData + pdwData[3]);

                    // Set a pointer to the index section.
                    m_pbIndex = m_pbCaptureData + 4 * sizeof(DWORD);

                    // Loop on all URLs, and for each, insert a new record in the map.
                    m_index.clear();
                    LPBYTE pIx = m_pbIndex;
                    CString strURL;
                    while(*pIx)
                    {
                        strURL = (LPCTSTR)pIx;
                        DWORD dwLen = strURL.GetLength();
                        LPDWORD pdw = (LPDWORD)(pIx + dwLen + 1);

                        // Find the last used RId for this couple (URL, connection id): our
                        // new RId will follow.
                        URLKey key(strURL, pdw[0], 1);
                        while(m_index.find(key) != m_index.end())
                            key.m_dwRId++;
                        URLData data(m_pbBody + pdw[1], m_pcClientHeader + pdw[2], m_pcServerHeader + pdw[3]);
                        m_index[key] = data;

                        // Skip to the next index record.
                        pIx += dwLen + 1 + 4 * sizeof(DWORD);
                    }
                    bRes = TRUE;
                }
                CloseHandle(hMap);
            }
            CloseHandle(hCaptureFile);
        }
    }
    else
        bRes = TRUE;

    return bRes;
}

BOOL CIECache::CacheURL(
    const CString& strOrigURL, DWORD dwConnId, DWORD dwRId,
    const BYTE* pData, DWORD dwDataLength)
{
    BOOL bRes = FALSE;

    CString strURL = strOrigURL;

    //// First, crack the URL to try to determine the file extension.
    URL_COMPONENTS urlComp;
    ZeroMemory(&urlComp, sizeof(urlComp));
    urlComp.dwStructSize = sizeof(urlComp);

    // Buffers.
    char zExtraInfo[256], zUrlPath[256];
    urlComp.lpszExtraInfo     = zExtraInfo;
    urlComp.lpszUrlPath       = zUrlPath;
    urlComp.dwExtraInfoLength = sizeof zExtraInfo;
    urlComp.dwUrlPathLength   = sizeof zUrlPath;

    if(InternetCrackUrl(strURL, 0, ICU_DECODE, &urlComp))
    {
        CString strExtension;
        if(urlComp.lpszUrlPath)
        {
            CString strPath = urlComp.lpszUrlPath;
            if(urlComp.lpszExtraInfo)
            {
                // Remove the extra info from the path.
                strPath = strPath.Left(strPath.GetLength() - lstrlen(urlComp.lpszExtraInfo));
            }

            // Look for an extension.
            if(strPath.Right(4).CompareNoCase(".htm") == 0)
                strExtension = "htm";
            else if(strPath.Right(5).CompareNoCase(".html") == 0)
                strExtension = "html";
            else if(strPath.Right(4).CompareNoCase(".gif") == 0)
                strExtension = "gif";
            else if(strPath.Right(4).CompareNoCase(".jpg") == 0)
                strExtension = "jpg";
        }
        else
            strExtension = "htm";

        TCHAR zFile[MAX_PATH];
        if(!pData)
            dwDataLength = 0;
        if(CreateUrlCacheEntry(strURL, dwDataLength, strExtension, zFile, 0))
        {
            if(!pData)
            {
                // If a capture file was specified, then extract the data from it.
                if(dwConnId)
                {
                    BOOL bNULLRid = dwRId == 0;
                    if(bNULLRid)
                        dwRId = 1;  // Try.
                    URLKey key(strURL, dwConnId, dwRId);
                    Index::const_iterator it1 = m_index.find(key);
                    if(bNULLRid)
                    {
                        key.m_dwRId = 2;
                        Index::const_iterator it2 = m_index.find(key);
                        if(it1 == m_index.end() || it2 != m_index.end())
                        {
                            // Ambiguity.
                            it1 = m_index.end();
                        }
                    }
                    if(it1 != m_index.end())
                    {
                        URLData data = it1->second;
                        pData = data.m_pbBody;
                        dwDataLength = *(DWORD*)pData;
                        pData += sizeof(DWORD);
                    }
                }
            }

            if(pData)
            {
                HANDLE hF = CreateFile(zFile, GENERIC_WRITE, 0, NULL,
                                       CREATE_ALWAYS, FILE_ATTRIBUTE_NORMAL|FILE_FLAG_SEQUENTIAL_SCAN, NULL);
                if(hF != INVALID_HANDLE_VALUE)
                {
                    DWORD dwWritten;
                    BOOL bOK = WriteFile(hF, pData, dwDataLength, &dwWritten, NULL);
                    CloseHandle(hF);
                    if(bOK)
                    {
                        FILETIME ft;
                        ZeroMemory(&ft,sizeof(ft));
                        if(CommitUrlCacheEntry(strURL, zFile, ft, ft, NORMAL_CACHE_ENTRY, NULL, 0, NULL, NULL))
                            bRes = TRUE;
                    }
                }
            }
        }
    }

    return bRes;
}

BOOL CIECache::MergeCaptureFile(LPCTSTR zCaptureFile1, LPCTSTR zCaptureFile2)
{
    BOOL bRes = FALSE;

    // Copy file zCaptureFile1 to a temporary file.
    TCHAR zBuffer[MAX_PATH], zTmp[MAX_PATH];
    LPTSTR pFileName = NULL;
    if(GetFullPathName(zCaptureFile1, sizeof(zBuffer)/sizeof(TCHAR), zBuffer, &pFileName) == 0)
        return FALSE;
    *pFileName = _T('\0');
    if(GetTempFileName(zBuffer, _T("tmp"), 0, zTmp) == 0)
        return FALSE;
    if(!CopyFile(zCaptureFile1, zTmp, FALSE))
        return FALSE;

    // Map zCaptureFile2 into memory.
    HANDLE hCaptureFile2 = CreateFile(zCaptureFile2, GENERIC_READ, FILE_SHARE_READ,
                                      NULL, OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL|FILE_FLAG_RANDOM_ACCESS, NULL);
    if(hCaptureFile2 != INVALID_HANDLE_VALUE)
    {
        HANDLE hMap2 = CreateFileMapping(hCaptureFile2, NULL, PAGE_READONLY, 0, 0, NULL);
        if(hMap2)
        {
            LPBYTE pbFile2 = (LPBYTE)MapViewOfFile(hMap2, FILE_MAP_READ, 0, 0, 0);
            DWORD dwFileSize2 = GetFileSize(hCaptureFile2, NULL);
            if(pbFile2)
            {
                // Get the section bases.
                LPDWORD pdw = (LPDWORD)pbFile2;
                DWORD dwBodyBase2 = pdw[1];
                DWORD dwClientHeaderBase2 = pdw[2];
                DWORD dwServerHeaderBase2 = pdw[3];

                // Get the size in bytes of the index section (without the trailing \0).
                DWORD dwIndexSectionSize2 = dwBodyBase2 - 4 * sizeof(DWORD) - 1;

                // Get a pointer to the index section.
                LPBYTE pIndexSection2 = pbFile2 + 4 * sizeof(DWORD);

                // Get the size in bytes of the body section.
                DWORD dwBodySectionSize2 = dwClientHeaderBase2 - dwBodyBase2;

                // Get a pointer to the body section.
                LPBYTE pBodySection2 = pIndexSection2 + dwIndexSectionSize2 + 1;

                // Get the size in bytes of the client header section.
                DWORD dwClientHeaderSectionSize2 = dwServerHeaderBase2 - dwClientHeaderBase2;

                // Get a pointer to the client header section.
                LPBYTE pClientHeaderSection2 = pBodySection2 + dwBodySectionSize2;

                // Get the size in bytes of the server header section.
                DWORD dwServerHeaderSectionSize2 = dwFileSize2 - dwServerHeaderBase2;

                // Get a pointer to the server header section.
                LPBYTE pServerHeaderSection2 = pClientHeaderSection2 + dwClientHeaderSectionSize2;

                // Map zTmp into memory, with write access, resizing the file to
                // fit the contents of zCaptureFile2.
                HANDLE hCaptureFile1 = CreateFile(zTmp, GENERIC_READ|GENERIC_WRITE, 0,
                                                  NULL, OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL|FILE_FLAG_RANDOM_ACCESS, NULL);
                if(hCaptureFile1 != INVALID_HANDLE_VALUE)
                {
                    DWORD dwFileSize1Original = GetFileSize(hCaptureFile1, NULL);
                    DWORD dwFileSize1 = dwFileSize1Original
                                        + dwIndexSectionSize2 + dwBodySectionSize2
                                        + dwClientHeaderSectionSize2 + dwServerHeaderSectionSize2;
                    {
                        HANDLE hMap1 = CreateFileMapping(hCaptureFile1, NULL, PAGE_READWRITE, 0, dwFileSize1, NULL);
                        if(hMap1)
                        {
                            LPBYTE pbFile1 = (LPBYTE)MapViewOfFile(hMap1, FILE_MAP_READ|FILE_MAP_WRITE, 0, 0, 0);
                            if(pbFile1)
                            {
                                // Get the section bases.
                                LPDWORD pdw = (LPDWORD)pbFile1;
                                DWORD dwBodyBase1 = pdw[1];
                                DWORD dwClientHeaderBase1 = pdw[2];
                                DWORD dwServerHeaderBase1 = pdw[3];

                                // Get the size in bytes of the index section (without the trailing \0).
                                DWORD dwIndexSectionSize1 = dwBodyBase1 - 4 * sizeof(DWORD) - 1;

                                // Get a pointer to the index section.
                                LPBYTE pIndexSection1 = pbFile1 + 4 * sizeof(DWORD);

                                // Get the size in bytes of the body section.
                                DWORD dwBodySectionSize1 = dwClientHeaderBase1 - dwBodyBase1;

                                // Get a pointer to the body section.
                                LPBYTE pBodySection1 = pIndexSection1 + dwIndexSectionSize1 + 1;

                                // Get the size in bytes of the client header section.
                                DWORD dwClientHeaderSectionSize1 = dwServerHeaderBase1 - dwClientHeaderBase1;

                                // Get a pointer to the client header section.
                                LPBYTE pClientHeaderSection1 = pBodySection1 + dwBodySectionSize1;

                                // Get the size in bytes of the server header section.
                                DWORD dwServerHeaderSectionSize1 = dwFileSize1Original - dwServerHeaderBase1;

                                // Get a pointer to the server header section.
                                LPBYTE pServerHeaderSection1 = pClientHeaderSection1 + dwClientHeaderSectionSize1;

                                // Move the server header section forward to its new place.
                                MoveMemory(pServerHeaderSection1 + dwIndexSectionSize2 + dwBodySectionSize2
                                           + dwClientHeaderSectionSize2, pServerHeaderSection1, dwServerHeaderSectionSize1);

                                // Update the server header base and pointer.
                                dwServerHeaderBase1 += dwIndexSectionSize2 + dwBodySectionSize2 + dwClientHeaderSectionSize2;
                                pdw[3] = dwServerHeaderBase1;
                                pServerHeaderSection1 += dwIndexSectionSize2 + dwBodySectionSize2 + dwClientHeaderSectionSize2;

                                // Move the client header section forward to its new place.
                                MoveMemory(pClientHeaderSection1 + dwIndexSectionSize2 + dwBodySectionSize2,
                                           pClientHeaderSection1, dwClientHeaderSectionSize1);

                                // Update the client header base and pointer.
                                dwClientHeaderBase1 += dwIndexSectionSize2 + dwBodySectionSize2;
                                pdw[2] = dwClientHeaderBase1;
                                pClientHeaderSection1 += dwIndexSectionSize2 + dwBodySectionSize2;

                                // Move the body section forward to its new place, along with the
                                // trailing zero from the index section!
                                MoveMemory(pBodySection1 - 1 + dwIndexSectionSize2, pBodySection1 - 1, dwBodySectionSize1 + 1);

                                // Update the body base and pointer.
                                dwBodyBase1 += dwIndexSectionSize2;
                                pdw[1] = dwBodyBase1;
                                pBodySection1 += dwIndexSectionSize2;

                                // Append the index section from zCaptureFile2 to the index section.
                                CopyMemory(pIndexSection1 + dwIndexSectionSize1, pIndexSection2, dwIndexSectionSize2);

                                // Append the body section from zCaptureFile2 to the body section.
                                CopyMemory(pBodySection1 + dwBodySectionSize1, pBodySection2, dwBodySectionSize2);

                                // Append the client header section from zCaptureFile2 to the client header section.
                                CopyMemory(pClientHeaderSection1 + dwClientHeaderSectionSize1, pClientHeaderSection2, dwClientHeaderSectionSize2);

                                // Append the server header section from zCaptureFile2 to the server header section.
                                CopyMemory(pServerHeaderSection1 + dwServerHeaderSectionSize1, pServerHeaderSection2, dwServerHeaderSectionSize2);

                                // Update the offsets in the appended records of the index section.
                                LPBYTE pIndex1 = pIndexSection1 + dwIndexSectionSize1;
                                while(*pIndex1)
                                {
                                    DWORD dwLen = lstrlenA((LPCSTR)pIndex1);
                                    LPDWORD pdwOff = (LPDWORD)(pIndex1 + dwLen + 1);
                                    pdwOff[1] += dwBodySectionSize1;
                                    pdwOff[2] += dwClientHeaderSectionSize1;
                                    pdwOff[3] += dwServerHeaderSectionSize1;

                                    // Move on to the next record.
                                    pIndex1 += dwLen + 1 + 4 * sizeof(DWORD);
                                }

                                // Update the last connection id.
                                pdw[0] = *(LPDWORD)pbFile2;

                                UnmapViewOfFile(pbFile1);
                                pbFile1 = NULL;

                                // Indicate partial success.
                                bRes = TRUE;
                            }
                            CloseHandle(hMap1);
                        }
                    }
                    CloseHandle(hCaptureFile1);
                }

                UnmapViewOfFile(pbFile2);
                pbFile2 = NULL;
            }
            CloseHandle(hMap2);
        }
        CloseHandle(hCaptureFile2);
    }

    if(bRes)    // Partial success.
    {
        // Delete the original files.
        bRes = DeleteFile(zCaptureFile1)
               && DeleteFile(zCaptureFile2)
               && MoveFile(zTmp, zCaptureFile1);
    }

    return bRes;
}

BOOL CIECache::CacheAllURLsFromCaptureFile()
{
    bool bRes = TRUE;

    for(Index::const_iterator it = m_index.begin(); it != m_index.end(); it++)
    {
        URLKey key = it->first;
        if(!CacheURL(key.m_strURL, key.m_dwConnId, key.m_dwRId))
        {
            bRes = FALSE;
            break;
        }
    }

    return bRes;
}

DWORD CIECache::GetLastConnId() const
{
    return *(LPDWORD)m_pbCaptureData;
}

LPBYTE CIECache::GetBody(const CString& strOrigURL, DWORD dwConnId, LPDWORD pdwDataLength, DWORD dwRId) const
{
    LPBYTE pbRes = NULL;
    BOOL bNULLRid = dwRId == 0;

    if(bNULLRid)
        dwRId = 1;  // Try.

    URLKey key(strOrigURL, dwConnId, dwRId);
    Index::const_iterator it1 = m_index.find(key);
    if(bNULLRid)
    {
        key.m_dwRId = 2;
        Index::const_iterator it2 = m_index.find(key);
        if(it1 == m_index.end() || it2 != m_index.end())
        {
            // Ambiguity.
            it1 = m_index.end();
        }
    }
    if(it1 != m_index.end())
    {
        URLData data = it1->second;
        pbRes = data.m_pbBody;
        *pdwDataLength = *(DWORD*)pbRes;
        pbRes += sizeof(DWORD);
    }
    else
        *pdwDataLength = 0;

    return pbRes;
}

LPCSTR CIECache::GetClientHeader(const CString& strOrigURL, DWORD dwConnId, LPDWORD pdwDataLength, DWORD dwRId) const
{
    LPCSTR pcRes = NULL;
    BOOL bNULLRid = dwRId == 0;

    if(bNULLRid)
        dwRId = 1;  // Try.

    URLKey key(strOrigURL, dwConnId, dwRId);
    Index::const_iterator it1 = m_index.find(key);
    if(bNULLRid)
    {
        key.m_dwRId = 2;
        Index::const_iterator it2 = m_index.find(key);
        if(it1 == m_index.end() || it2 != m_index.end())
        {
            // Ambiguity.
            it1 = m_index.end();
        }
    }
    if(it1 != m_index.end())
    {
        URLData data = it1->second;
        pcRes = data.m_pcClient;
        *pdwDataLength = *(DWORD*)pcRes;
        pcRes += sizeof(DWORD);
    }
    else
        *pdwDataLength = 0;

    return pcRes;
}

LPCSTR CIECache::GetServerHeader(const CString& strOrigURL, DWORD dwConnId, LPDWORD pdwDataLength, DWORD dwRId) const
{
    LPCSTR pcRes = NULL;
    BOOL bNULLRid = dwRId == 0;

    if(bNULLRid)
        dwRId = 1;  // Try.

    URLKey key(strOrigURL, dwConnId, dwRId);
    Index::const_iterator it1 = m_index.find(key);
    if(bNULLRid)
    {
        key.m_dwRId = 2;
        Index::const_iterator it2 = m_index.find(key);
        if(it1 == m_index.end() || it2 != m_index.end())
        {
            // Ambiguity.
            it1 = m_index.end();
        }
    }
    if(it1 != m_index.end())
    {
        URLData data = it1->second;
        pcRes = data.m_pcServer;
        *pdwDataLength = *(DWORD*)pcRes;
        pcRes += sizeof(DWORD);
    }
    else
        *pdwDataLength = 0;

    return pcRes;
}

void CIECache::QueryURL(const CString& strQuery, DWORD dwConnId, std::list<URLKey>& outList) const
{
    // First, empty the list.
    outList.clear();

    // Convert to a string.
    std::string strQuery2 = strQuery;

    // Tokenize the query.
    stringList strPatterns;
    std::string strTmp;
    std::string::size_type newpos = std::string::npos;
    BOOL bLeadingStar = FALSE, bTrailingStar = FALSE;
    for(std::string::size_type pos = 0; pos != std::string::npos; pos = newpos)
    {
        // Get the next star.
        if(newpos != std::string::npos)
            pos++;  // Skip the preceding star.
        newpos = strQuery2.find('*', pos);

        // Extract the substring.
        std::string::size_type length;
        if(newpos == std::string::npos)
        {
            // Extract til the end.
            length = newpos;
        }
        else
        {
            // Normal substring extraction.
            length = newpos - pos;
        }
        strTmp = strQuery2.substr(pos, length);

        // Append the substring to the list.
        if(!strTmp.empty())
            strPatterns.push_back(strTmp);
        else
        {
            if(newpos == 0)
                bLeadingStar = TRUE;
        }
    }

    if(strQuery2.size() > 0 && strTmp.empty())
        bTrailingStar = TRUE;

    if(!strPatterns.empty())
    {
        // Search the map for keys greater or equal to the lower bound.
        for(Index::const_iterator it = m_index.begin(); it != m_index.end(); it++)
        {
            // Look for a match.
            if(Match(strPatterns, dwConnId, it, bLeadingStar, bTrailingStar))
                outList.push_back(it->first);
        }
    }

	if ( outList.empty() )
	{
        for(Index::const_iterator it = m_index.begin(); it != m_index.end(); it++)
        {
			outList.push_back(it->first);
		}
	}
}

BOOL CIECache::Match(const stringList& strPatterns, DWORD dwConnId,
                     const Index::const_iterator it,
                     BOOL bLeadingStar, BOOL bTrailingStar) const
{
    BOOL bRes = (dwConnId == it->first.m_dwConnId);

    if(bRes)
    {
        // Get a reference to the URL.
        const std::string strURL = it->first.m_strURL;

        // Loop on the patterns, and check if they are all included (in order)
        // in the URL.
        std::string::size_type pos = 0;
        for(stringList::const_iterator strIt = strPatterns.begin();
                strIt != strPatterns.end(); strIt++)
        {
            const std::string& strCurrentPattern = *strIt;
            std::string::size_type newpos = strURL.find(strCurrentPattern, pos);
            if(newpos == std::string::npos || newpos > 0 && pos == 0 && !bLeadingStar)
            {
                bRes = FALSE;
                break;
            }
            else
                pos = newpos + strCurrentPattern.size();    // Skip the matched string.
        }

        if(!bTrailingStar && pos < strURL.size())
            bRes = FALSE;
    }

    return bRes;
}

void CIECache::QueryHTMLURL(const CString& strQuery, DWORD dwConnId, std::list<URLKey>& outList) const
{
    URLKey              key ;
    std::list<URLKey>   tmpURLList ;
    //---

    // First, empty the list.
    outList.clear();

    QueryURL(strQuery, dwConnId, tmpURLList) ;

    for(std::list<URLKey>::iterator it = tmpURLList.begin();
            it != tmpURLList.end(); it++ )
    {
        key     = *it ;

        //if ( IsHTMLURL(key.m_strURL, key.m_dwConnId, key.m_dwRId) )
            outList.insert(outList.end(), key);
    }
}

BOOL CIECache::IsHTMLURL(const CString& strOrigURL, DWORD dwConnId, DWORD dwRId) const
{
    LPCSTR              pcHeader;
    DWORD               dwLength ;
    int                 nPos ;

    pcHeader = GetServerHeader(strOrigURL, dwConnId, &dwLength, dwRId);
    CString strServerHeader(pcHeader, dwLength);

    strServerHeader.MakeUpper();

    nPos = strServerHeader.Find("CONTENT-TYPE" );
    if( nPos >= 0 )
    {
        strServerHeader = strServerHeader.Right( strServerHeader.GetLength() - nPos ) ;
        nPos = strServerHeader.Find("\n");
        if( nPos >= 0 )
        {
            strServerHeader = strServerHeader.Left( nPos ) ;
        }
    }

    nPos = strServerHeader.Find("TEXT/HTML");

    return (nPos >= 0);
}

BOOL CIECache::IsPrimaryURL(const CString& strOrigURL, DWORD dwConnId, DWORD dwRId) const
{
    CString strClientHeader;
    DWORD   dwLength ;
    int     nPos;

    //client
    strClientHeader = GetClientHeader(strOrigURL,
                                      dwConnId,
                                      &dwLength,dwRId);
    strClientHeader.MakeUpper();
    nPos = strClientHeader.Find("ACCEPT" );
    if( nPos >= 0 )
    {
        strClientHeader = strClientHeader.Right( strClientHeader.GetLength() - nPos ) ;

        nPos = strClientHeader.Find(":");
        if( nPos >= 0 )
            strClientHeader = strClientHeader.Right( strClientHeader.GetLength() - nPos - 1) ;

        nPos = strClientHeader.Find("\n");
        if( nPos >= 0 )
        {
            strClientHeader = strClientHeader.Left( nPos ) ;
        }
    }

    nPos = strClientHeader.Find("*/*");
    strClientHeader.TrimLeft();
    strClientHeader.TrimRight();
    return ( (!( (nPos >= 0) && (strClientHeader.GetLength() == 3 ) )) && (!strClientHeader.IsEmpty()));

}

BOOL CIECache::CacheAllForPrimaryURL(const CString& strURL,DWORD dwConnId,DWORD dwRId)
{
	if(!IsPrimaryURL(strURL,dwConnId,dwRId))
	{
		return CacheURL(strURL,dwConnId,dwRId);
	}

    for(Index::const_iterator it = m_index.begin(); it != m_index.end(); it++)
    {
        URLKey key = it->first;
		
		//Look for the Primary URL
		if((key.m_strURL == strURL) && (key.m_dwConnId == dwConnId) && (key.m_dwRId == dwRId))
		{
			if(!CacheURL(key.m_strURL, key.m_dwConnId, key.m_dwRId))
				return false;
			
			it++;
			break;
		}
	}

    for(; it != m_index.end(); it++)
    {
        URLKey key = it->first;
		if(IsPrimaryURL(key.m_strURL, key.m_dwConnId, key.m_dwRId))
			return TRUE;

		if(!CacheURL(key.m_strURL, key.m_dwConnId, key.m_dwRId))
            return FALSE;
    }
	return TRUE;
}
