// EditQueryDlg.cpp : implementation file
//

#include "stdafx.h"
#include "ntperfplugin.h"
#include "EditQueryDlg.h"
#include "pdh.h"
#include "PdhMsg.h"
#include <malloc.h>
#include "..\include\NTPerfHelpID.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CEditQueryDlg dialog


CEditQueryDlg::CEditQueryDlg(CWnd* pParent /*=NULL*/)
	: CDialog(CEditQueryDlg::IDD, pParent)
{
	//{{AFX_DATA_INIT(CEditQueryDlg)
	m_bDelta = FALSE;
	m_strDescription = _T("");
	m_strInterval = _T("5");
	m_strQuery = _T("");
	//}}AFX_DATA_INIT
	m_bInit = FALSE;
}


void CEditQueryDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CEditQueryDlg)
	DDX_Control(pDX, IDC_EDIT_QUERY, m_ctlEditQuery);
	DDX_Control(pDX, IDC_INTERVAL_SPIN, m_ctlIntervalSpin);
	DDX_Check(pDX, IDC_DELTA, m_bDelta);
	DDX_Text(pDX, IDC_DESCRIPTION, m_strDescription);
	DDX_Text(pDX, IDC_INTERVAL, m_strInterval);
	DDX_Text(pDX, IDC_QUERY, m_strQuery);
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(CEditQueryDlg, CDialog)
	//{{AFX_MSG_MAP(CEditQueryDlg)
	ON_BN_CLICKED(IDC_EDIT_QUERY, OnEditQuery)
	ON_WM_HELPINFO()
	ON_EN_UPDATE(IDC_INTERVAL, OnUpdateInterval)
	ON_EN_CHANGE(IDC_INTERVAL, OnChangeInterval)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CEditQueryDlg message handlers

BOOL CEditQueryDlg::EditNTPerformanceQuery(CString& strQuery)
{
    PDH_BROWSE_DLG_CONFIG PDHBrowseDlgConfig;

    DWORD dwQuerySize = 1024;

    //Select a configuration for the performance monitor config dialog
    ZeroMemory(&PDHBrowseDlgConfig, sizeof(PDHBrowseDlgConfig));

    PDHBrowseDlgConfig.bIncludeInstanceIndex = 1;
    PDHBrowseDlgConfig.bSingleCounterPerAdd = 0;
    PDHBrowseDlgConfig.bSingleCounterPerDialog = 1;
    PDHBrowseDlgConfig.bLocalCountersOnly = 0;
    PDHBrowseDlgConfig.bWildCardInstances = 0;
    PDHBrowseDlgConfig.bHideDetailBox = 1;
    PDHBrowseDlgConfig.bInitializePath = !strQuery.IsEmpty();
    PDHBrowseDlgConfig.hWndOwner = m_hWnd;
    PDHBrowseDlgConfig.pCallBack = NULL;
    PDHBrowseDlgConfig.szReturnPathBuffer = strQuery.GetBuffer(dwQuerySize);
    PDHBrowseDlgConfig.dwDefaultDetailLevel = 4;
	PDHBrowseDlgConfig.bDisableMachineSelection = 0;
    PDHBrowseDlgConfig.cchReturnPathLength = dwQuerySize;

    HINSTANCE hPDH = AfxLoadLibrary("PDH.dll");
    if(hPDH != NULL)
    {
        typedef PDH_STATUS  (CALLBACK* LPBROWSECOUNTERS)    (PDH_BROWSE_DLG_CONFIG*);
        LPBROWSECOUNTERS lpfnBrowseCounters =  (LPBROWSECOUNTERS)GetProcAddress(hPDH, "PdhBrowseCountersA");

        if (lpfnBrowseCounters != NULL)
        {
            //Display PerfMon config dialog

            if(lpfnBrowseCounters(&PDHBrowseDlgConfig) != ERROR_SUCCESS)
            {
                return FALSE;
            }
        }
        AfxFreeLibrary(hPDH);
        hPDH = NULL;
    }

    strQuery.ReleaseBuffer();
    return TRUE;
}

void CEditQueryDlg::OnEditQuery() 
{
	UpdateData();
	EditNTPerformanceQuery(m_strQuery);
	PDH_COUNTER_PATH_ELEMENTS *pCounterPathElements ; 
	DWORD dwBufferSize = 0; 


    // Call to find buffer size by calling with buffer size to zero 
	PDH_STATUS status = PdhParseCounterPath(m_strQuery, NULL, &dwBufferSize, 0);

    // The call will result in a status of not enough buffer space on XP 
    // (PDH_MORE_DATA) but ERROR_SUCCESS on NT/2000


	if ((status != PDH_MORE_DATA)&&(status != ERROR_SUCCESS))
     {
      return ;
     }

	BYTE * pBuffer = (BYTE *)_alloca(dwBufferSize) ;
	pCounterPathElements = (PDH_COUNTER_PATH_ELEMENTS *)pBuffer ;
	status = PdhParseCounterPath(m_strQuery, pCounterPathElements, &dwBufferSize, 0);

	if (status != ERROR_SUCCESS)
     {
    	return ;
     }		

	m_strDescription = pCounterPathElements->szCounterName ;
	
	UpdateData(FALSE);
}

void CEditQueryDlg::OnOK() 
{
	UpdateData();

	if(atoi(m_strInterval)<1)
	{
		AfxMessageBox("Please enter an interval greater than zero");
		return;
	}

	if(m_strDescription.IsEmpty())
	{
		AfxMessageBox("Please enter a name for this query");
		return;
	}

	for(int n=0;n<m_arstrNamesInUse.GetSize();n++)
	{
		if(m_strDescription == m_arstrNamesInUse[n])
		{
			AfxMessageBox("The name of this query is already in use.\nPlease enter a name which is unique within this profile.");
			return;
		}
	}

	if(m_strQuery.IsEmpty())
	{
		AfxMessageBox("Please enter a query");
		return;
	}

	CDialog::OnOK();
}

BOOL CEditQueryDlg::OnInitDialog() 
{
	CDialog::OnInitDialog();

    OSVERSIONINFO OSInfo;
    OSInfo.dwOSVersionInfoSize = sizeof(OSVERSIONINFO);

    if(!GetVersionEx(&OSInfo) || (OSInfo.dwPlatformId!=VER_PLATFORM_WIN32_NT))
    {
		m_ctlEditQuery.EnableWindow(FALSE);
	}

	if(m_strDescription.IsEmpty())
	{
		int nNdx=1;
		bool bContinue=true;
		while(bContinue)
		{
			bContinue= false;
			m_strDescription.Format("Query %d",nNdx++);
			for(int nName=0;nName<m_arstrNamesInUse.GetSize();nName++)
			{
				if(m_arstrNamesInUse[nName] == m_strDescription)
				{
					bContinue=true;
					break;
				}	
			}	
		}
	}

	UDACCEL AL[2];
	AL[0].nInc = 1;
	AL[0].nSec = 2;
	AL[1].nInc = 3;
	AL[1].nSec = 4;
	m_ctlIntervalSpin.SetAccel(2,AL);
	m_ctlIntervalSpin.SetRange(1,32000);

	UpdateData(FALSE);
	m_bInit = TRUE ;
	return TRUE;  // return TRUE unless you set the focus to a control
	              // EXCEPTION: OCX Property Pages should return FALSE
}

BOOL CEditQueryDlg::OnHelpInfo(HELPINFO* pHelpInfo) 
{
	pHelpInfo->dwContextId = HIDD_NTPERF_EDIT_QUERY;
	return CDialog::OnHelpInfo(pHelpInfo);
}

void CEditQueryDlg::OnUpdateInterval() 
{
}

void CEditQueryDlg::OnChangeInterval() 
{
	if (!m_bInit)
		return ;
	UpdateData();
	if (m_strInterval == "0")
	{
		m_strInterval = _T("1") ;
		UpdateData(FALSE);
	}
	
}
