/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/


#include "stdafx.h"
#include <afxdllx.h>
#include "PluginTypes.h"
#include "resource.h"       // main symbols
#include "MonitorGraph.h"
#include "TestFile.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif


static AFX_EXTENSION_MODULE CSNMPMonDLL = { NULL, NULL };

HINSTANCE m_hInstance = NULL;
int currentTaskGroup=-1;

extern "C" int APIENTRY
    DllMain(HINSTANCE hInstance, DWORD dwReason, LPVOID lpReserved)
{
	m_hInstance = hInstance;
    // Remove this if you use lpReserved
    UNREFERENCED_PARAMETER(lpReserved);

    if (dwReason == DLL_PROCESS_ATTACH)
    {
        TRACE0("CSNMPMonDLL.DLL Initializing!\n");

        // Extension DLL one-time initialization
        if (!AfxInitExtensionModule(CSNMPMonDLL, hInstance))
            return 0;

        // Insert this DLL into the resource chain
        // NOTE: If this Extension DLL is being implicitly linked to by
        //  an MFC Regular DLL (such as an ActiveX Control)
        //  instead of an MFC application, then you will want to
        //  remove this line from DllMain and put it in a separate
        //  function exported from this Extension DLL.  The Regular DLL
        //  that uses this Extension DLL should then explicitly call that
        //  function to initialize this Extension DLL.  Otherwise,
        //  the CDynLinkLibrary object will not be attached to the
        //  Regular DLL's resource chain, and serious problems will
        //  result.

    }
    else if (dwReason == DLL_PROCESS_DETACH)
    {
        TRACE0("CSNMPMonDLL.DLL Terminating!\n");

        // Terminate the library before destructors are called
        AfxTermExtensionModule(CSNMPMonDLL);

    }

    return 1;   // ok
}

extern "C" AFX_EXT_API
BOOL OpenSTAVerify()
{
	return true;
}

extern "C" AFX_EXT_API
void GetTitle(CString& strTitle)
{
	strTitle = "SNMP Monitor Plugin";
}

extern "C" AFX_EXT_API
int GetType()
{
	return TEST_MONITOR_PLUGIN;
}

extern "C" AFX_EXT_API
void GetPackage(CString& strPackage)
{
	strPackage = "SMP";
}

extern "C" AFX_EXT_API
BOOL IsSupportedExtension(CString strExt)
{
	if (strExt.CompareNoCase("SMP") == 0)
	{
		return true;
	}
    return false;
}

extern "C" AFX_EXT_API
void GetAvailableScreens(CTestFile* pTestFile, int nTaskGroup, CStringArray& arrScreens)
{
	if(pTestFile->GetNumTasks(nTaskGroup)!=1)
		return;

	currentTaskGroup=nTaskGroup;
	//set current task group

	CString strTaskPath = pTestFile->GetTaskPath(nTaskGroup, 0);

	//Open profile file and get array of descriptions
	CStdioFile profile;
	if(profile.Open(strTaskPath, CFile::modeRead | CFile::shareDenyNone))
	{
		CString line;

		while(profile.ReadString(line))
		{
			//get up to the first comma
			char *pszToken = strtok(line.GetBuffer(0),",");
			if(pszToken)
			{
				CString name;
				name.Format("%s",pszToken);
				arrScreens.Add(name);
			}
		}
	}
}

extern "C" AFX_EXT_API
CWnd* ViewScreen(CString strManagerReg, CString strTestRunPath, CStringArray& arrScreens, CWnd* pParent)
{
	// Get the current Resource handle
	HINSTANCE hInst = AfxGetResourceHandle();

	// Set the resource handle to be this dll while opening a document
	AfxSetResourceHandle(m_hInstance);

	CMonitorGraph* pGraph = new CMonitorGraph();
	pGraph->InitMonitor(strTestRunPath, arrScreens);
	pGraph->Create(IDD_GRAPH_DLG, pParent);
	pGraph->ShowWindow(SW_SHOW);

	// Restore the resource handle
	AfxSetResourceHandle(hInst);

    return pGraph;	
}
