/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/
// CECSNMPQuery.cpp: implementation of the CECSNMPQuery class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "CECSNMPQuery.h"



#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CECSNMPQuery::CECSNMPQuery()
{
//
//  Start Winsock
//
	WORD      VersionRequested;
	WSADATA   stWSAData;

	VersionRequested = MAKEWORD(1, 1);
	WSAStartup(VersionRequested, &stWSAData);

	// Define environment variable specifying the location of MIB files
	// NB On Unix, putenv requires a static string.
	char     *pHome = getenv("OPENSTA_HOME");

	if (pHome == NULL)
	{
		AfxMessageBox("Unable to determine MIB file path!");
	}
	else
	{
		// note that the separator in MIBDIRS is a colon in UNIX,
		// but a semicolon under Windows, to avoid conflict
		// with "drive letters"

		// only set MIBDIRS if not already set
		if (getenv("MIBDIRS") == NULL)
		{
			CString    setmibdir;
			setmibdir.Format("MIBDIRS=%s/Engines/Mon/mibs", pHome);
			static char putenvString[512];

			strcpy(putenvString, setmibdir.GetBuffer(0));
			putenv(putenvString);
		}

	}

	// snmp_set_do_debugging(1);
	// Initialize the SNMP library
	init_snmp("snmpapp");

}

CECSNMPQuery::~CECSNMPQuery()
{

}

long CECSNMPQuery::ExecuteSimple(QUERY_DEF& def)
{
	CString errMsg="";
	long      value = 0;

	// Initialize a "session" that defines who we're going to talk to
	struct snmp_session session;
	void *sessp;

	snmp_sess_init(&session);
	session.version = (long)SNMP_VERSION_1;
	session.peername = (char *) def.server.GetBuffer(0);
	session.community = (unsigned char *) def.community.GetBuffer(0);
	session.community_len = strlen((const char *) session.community);
	session.timeout = def.nTimeout;
	session.remote_port = def.nPort;

	// Open the session
	sessp = snmp_sess_open(&session);
	if (sessp == NULL)
	{
		char     *err;

		snmp_sess_error(sessp, NULL, NULL, &err);
		errMsg.Format("snmp_sess_open failed: %s", err);
		AfxMessageBox(errMsg);
		free(err);
		return -1;
	}

	// Create the PDU for the data for our request.
	struct snmp_pdu *pdu;

	pdu = snmp_pdu_create(SNMP_MSG_GET);

	// Convert the OID string representation to an internal representation.
	oid       reqOid[MAX_OID_LEN];
	size_t    reqOid_len = MAX_OID_LEN;

    if (  get_node(def.query.GetBuffer(0), reqOid, &reqOid_len) != 1)
    {
		char     *err;

		snmp_sess_error(sessp, NULL, NULL, &err);
		errMsg.Format( "read_objid failed: %s", err);
		AfxMessageBox(errMsg);
		free(err);
		return -1;
    }

	// Add this oid, with a NULL value, to the PDU
	snmp_add_null_var(pdu, reqOid, reqOid_len);

	// Send the request and get the response
	int       status;
	struct snmp_pdu *response;

	status = snmp_sess_synch_response(sessp, pdu, &response);
	if (status != STAT_SUCCESS)
	{
		char     *err;

		snmp_sess_error(sessp, NULL, NULL, &err);
		errMsg.Format( "snmp_sess_synch_response failed: %s", err);
		free(err);
		AfxMessageBox(errMsg);
		return -1;
	}
	else if (response->errstat != SNMP_ERR_NOERROR)
	{
		errMsg.Format( "Error in packet: %s",
				snmp_errstring(response->errstat));
		AfxMessageBox(errMsg);
		return -1;
	}

	struct variable_list *vars;
	int       count = 0;

	for (vars = response->variables; vars; vars = vars->next_variable)
	{
		count += 1;
		if (count > 1)
		{
			AfxMessageBox("More than 1 value returned for simple query");
			return -1;
		}

		if (vars->type == ASN_INTEGER 
		 || vars->type == ASN_COUNTER )
		{
			value = *(vars->val.integer);
		}
		else
		{
			AfxMessageBox("Non-integer value returned for simple query");
			return -1;
		}
	}

	// Free the response and close the SNMP session
	if (response)
	{
		snmp_free_pdu(response);
	}
	snmp_sess_close(sessp);

	// Return result
	return value;
}

bool  CECSNMPQuery::ExecuteDetailed(QUERY_DEF& def, CString& result)
{
	result = "";

	// Initialize a "session" that defines who we're going to talk to
	struct snmp_session session;
	void *sessp;

	snmp_sess_init(&session);
	session.version = (long)SNMP_VERSION_1;
	session.peername = (char *) def.server.GetBuffer(0);
	session.community = (unsigned char *) def.community.GetBuffer(0);
	session.community_len = strlen((const char *) session.community);
	session.timeout = def.nTimeout;
	session.remote_port = def.nPort;

	// Open the session
	sessp = snmp_sess_open(&session);
	if (sessp == NULL)
	{
		char     *err;
		snmp_sess_error(&session, NULL, NULL, &err);
		result.Format("snmp_sess_open failed: %s", err);
		return false;
	}

	// Create the PDU for the data for our request.
	struct snmp_pdu *pdu;

	pdu = snmp_pdu_create(SNMP_MSG_GET);

	// Convert the OID string representation to an internal representation.
	oid       reqOid[MAX_OID_LEN];
	size_t    reqOid_len = MAX_OID_LEN;

	if (get_node(def.query.GetBuffer(0), reqOid, &reqOid_len) == NULL)
	{
		char     *err;
		snmp_sess_error(sessp, NULL, NULL, &err);
		result.Format("read_objid failed: %s", err);
		return false;
	}

	// Add this oid, with a NULL value, to the PDU
	snmp_add_null_var(pdu, reqOid, reqOid_len);

	// Send the request and get the response
	int       status;
	struct snmp_pdu *response;

	status = snmp_sess_synch_response(sessp, pdu, &response);
	if (status != STAT_SUCCESS)
	{
		// The call failed
		char     *err;
		snmp_sess_error(sessp, NULL, NULL, &err);
		result.Format("snmp_sess_synch_response failed: %s", err);
		return false;
	}
	else if (response->errstat != SNMP_ERR_NOERROR)
	{
		// The call succeeded but an error status was returned
		result.Format( "Error in packet: %s",
					snmp_errstring(response->errstat));
		return false;
	}

	// Format the value(s) and append to result string
	struct variable_list *vars;
	char      buf[2560];
	int       count = 0;

	for (vars = response->variables; vars; vars = vars->next_variable)
	{
		def.nResultType = vars->type;

		sprint_value(buf, reqOid, reqOid_len, vars);
		result += buf;
	}

	// Free the response and close the SNMP session
	if (response)
	{
		snmp_free_pdu(response);
	}
	snmp_sess_close(sessp);

	// Return result
	return true;
}

BOOL CECSNMPQuery::ExecuteWalk(QUERY_DEF def, CStringArray& resultArray)
{
    struct snmp_session  session, *sessp;
    struct snmp_pdu *pdu, *response;
    struct variable_list *vars;
    oid    name[MAX_OID_LEN];
    size_t name_length;
    int    count;
    int    running;
    int    status;
	CString result="";
	int numprinted = 1;
	char buf[256];

	snmp_sess_init(&session);
	session.version = (long)SNMP_VERSION_1;
	session.peername = (char *) def.server.GetBuffer(0);
	session.community = (unsigned char *) def.community.GetBuffer(0);
	session.community_len = strlen((const char *) session.community);
	session.timeout = def.nTimeout;
	session.remote_port = def.nPort;

	// Open the session
	sessp = (struct snmp_session*)snmp_sess_open(&session);
	if (sessp == NULL)
	{
		char     *err;
		snmp_sess_error(&session, NULL, NULL, &err);
		result.Format("snmp_sess_open failed: %s", err);
		return false;
	}


	// Convert the OID string representation to an internal representation.
	oid       reqOid[MAX_OID_LEN];
	size_t    reqOid_len = MAX_OID_LEN;

	if (get_node(def.query.GetBuffer(0), reqOid, &reqOid_len) == NULL)
	{
		char     *err;
		snmp_sess_error(sessp, NULL, NULL, &err);
		result.Format("read_objid failed: %s", err);
		return false;
	}

    /* get first object to start walk */
    memmove(name, reqOid, reqOid_len * sizeof(oid));
    name_length = reqOid_len;

    running = -1;

    
    while(running){
      /* create PDU for GETNEXT request and add object name to request */
      pdu = snmp_pdu_create(SNMP_MSG_GETNEXT);
      snmp_add_null_var(pdu, name, name_length);

      /* do the request */
      status = snmp_sess_synch_response(sessp, pdu, &response);
      if (status == STAT_SUCCESS){
        if (response->errstat == SNMP_ERR_NOERROR){
          /* check resulting variables */
          for(vars = response->variables; vars; vars = vars->next_variable){
            if ((vars->name_length < reqOid_len) ||
                (memcmp(reqOid, vars->name, reqOid_len * sizeof(oid))!=0)) {
              /* not part of this subtree */

              running = 0;
              continue;
            }
            numprinted++;
            print_variable(vars->name, vars->name_length, vars);

	            sprint_variable(buf, vars->name, vars->name_length, vars);
				result = buf;
				resultArray.Add(result);

            if ((vars->type != SNMP_ENDOFMIBVIEW) &&
                (vars->type != SNMP_NOSUCHOBJECT) &&
                (vars->type != SNMP_NOSUCHINSTANCE)){
              /* not an exception value */
              memmove((char *)name, (char *)vars->name,
              vars->name_length * sizeof(oid));
              name_length = vars->name_length;
            } else
              /* an exception value, so stop */
              running = 0;
          }
        } else {
          /* error in response, print it */
          running = 0;
          if (response->errstat == SNMP_ERR_NOSUCHNAME){
            printf("End of MIB\n");
          } else {
            fprintf(stderr, "Error in packet.\nReason: %s\n",
		    snmp_errstring(response->errstat));
            if (response->errstat == SNMP_ERR_NOSUCHNAME){
              fprintf(stderr, "The request for this object identifier failed: ");
              for(count = 1, vars = response->variables;
                    vars && count != response->errindex;
                    vars = vars->next_variable, count++)
                /*EMPTY*/;
              if (vars)
                fprint_objid(stderr, vars->name, vars->name_length);
              fprintf(stderr, "\n");
            }
          }
        }
      } else if (status == STAT_TIMEOUT){
        fprintf(stderr, "Timeout: No Response from %s\n", session.peername);
        running = 0;
      } else {    /* status == STAT_ERROR */
        snmp_sess_perror("snmpapp", sessp);
        running = 0;
      }
      if (response)
        snmp_free_pdu(response);
    }


    snmp_close(sessp);

    return TRUE;
}
