/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/
// QueryEditDlg.cpp : implementation file
//

#include "stdafx.h"
#include "QueryEditDlg.h"
#include "CECSNMPQuery.h"
#include "..\include\SNMPHelpID.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CQueryEditDlg dialog


CQueryEditDlg::CQueryEditDlg(CString strServer, CString strQuery, int nPort, CWnd* pParent /*=NULL*/)
	: CDialog(CQueryEditDlg::IDD, pParent)
{
	//{{AFX_DATA_INIT(CQueryEditDlg)
	m_strWalkPoint = _T("");
	//}}AFX_DATA_INIT

	m_szQuery = strQuery;
	m_szServer = strServer;
	m_nPort = nPort;
}


void CQueryEditDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CQueryEditDlg)
	DDX_Control(pDX, IDC_QUERY_LIST, m_ctlWalkList);
	DDX_Control(pDX, IDC_CATEGORY, m_ctlCategory);
	DDX_Text(pDX, IDC_WALKPOINT, m_strWalkPoint);
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(CQueryEditDlg, CDialog)
	//{{AFX_MSG_MAP(CQueryEditDlg)
	ON_CBN_SELCHANGE(IDC_CATEGORY, OnSelchangeCategory)
	ON_WM_HELPINFO()
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CQueryEditDlg message handlers

BOOL CQueryEditDlg::OnInitDialog() 
{
	CDialog::OnInitDialog();
	ReadCategories();

	m_ctlCategory.SetCurSel(0);
	m_ctlCategory.EnableWindow();

	CRect r;
	m_ctlWalkList.GetWindowRect(r);
	m_ctlWalkList.InsertColumn(0,"Query", LVCFMT_LEFT, r.Width()/2-2);
	m_ctlWalkList.InsertColumn(1,"Current Value", LVCFMT_LEFT, r.Width()/2-2);
	m_ctlWalkList.EnableWindow();
	m_ctlWalkList.SetExtendedStyle(LVS_EX_FULLROWSELECT);

	if (m_szQuery != "")
	{
		CString strQ = m_szQuery;
		int nPos = strQ.Find(" ", 0);
		CString community = strQ.Left(nPos);
		strQ = strQ.Right(strQ.GetLength() - nPos -1);
		strQ.TrimRight();

		m_ctlCategory.EnableWindow();
		m_ctlWalkList.EnableWindow();

		int cat = 0;
		for (int nCat = 0; nCat<m_arrCat.GetSize(); nCat++)
		{
			if (m_arrCat.GetAt(nCat).strWalkStart == strQ)
			{
				cat=nCat;
				break;
			}
		}
		m_ctlCategory.SetCurSel(cat);
		WalkTable(cat);

		for(int i =0; i<m_ctlWalkList.GetItemCount(); i++)
		{
			if (m_ctlWalkList.GetItemText(i,0) == strQ)
			{
				m_ctlWalkList.SetFocus();
				m_ctlWalkList.SetItemState(i, LVIS_SELECTED|LVIS_DROPHILITED, LVIS_SELECTED);
			}
		}
	}
	else
	{
		WalkTable(0);
	}
	return TRUE;  // return TRUE unless you set the focus to a control
	              // EXCEPTION: OCX Property Pages should return FALSE
}

void CQueryEditDlg::WalkTable(int nTable)
{
	CWaitCursor wc;
	m_ctlWalkList.DeleteAllItems();

	// Test that an SNMP server is running
	try
	{
		CECSNMPQuery query;
		QUERY_DEF def;
		def.header = m_szServer;
		def.community = "public";
		def.server = m_szServer;
		def.nTimeout = 5000;
		def.query = m_arrCat.GetAt(nTable).strWalkStart;
		def.nPort = m_nPort;

		m_strWalkPoint = def.query;

		int nTrimPoint = m_strWalkPoint.GetLength()+1;

		CStringArray results;
		if(query.ExecuteWalk(def, results))
		{
			for (int i =0; i<results.GetSize(); i++)
			{
				CString strRes = results.GetAt(i).Mid(nTrimPoint);
				int nPos = strRes.Find(" =",0);
				int nLen = strRes.GetLength();
				m_ctlWalkList.InsertItem(i,strRes.Left(nPos));
				m_ctlWalkList.SetItemText(i,1,strRes.Right(nLen-nPos-3));
			}
		}
	}
	catch (...)
	{
		m_ctlCategory.EnableWindow(FALSE);
		m_ctlWalkList.EnableWindow(FALSE);
		m_ctlWalkList.DeleteAllItems();
		m_strWalkPoint.Empty();
	}

	UpdateData(FALSE);
}

void CQueryEditDlg::OnSelchangeCategory() 
{
	int nCat = m_ctlCategory.GetCurSel();
	WalkTable(nCat);	
}

void CQueryEditDlg::OnOK() 
{
	// Get the selected query
	POSITION pos = m_ctlWalkList.GetFirstSelectedItemPosition();
	if (pos == NULL)
	{
		AfxMessageBox("Nothing Selected!!");
		return;
	}
	CWaitCursor wc;

	int nItem = m_ctlWalkList.GetNextSelectedItem(pos);
	CString strQuery = m_strWalkPoint + "." +  m_ctlWalkList.GetItemText(nItem,0);

	// Test query to see if it returns an integer value
	try
	{
		CECSNMPQuery query;
		QUERY_DEF def;
		def.header = m_szServer;
		def.community = "public";
		def.server = m_szServer;
		def.nTimeout = 5000;
		def.query = strQuery;
		def.nPort = m_nPort;


		if (query.ExecuteSimple(def) == -1)
		{
			return;
		}

		m_szQuery = "public "+ strQuery;
	}
	catch (...)
	{
		m_ctlCategory.EnableWindow(FALSE);
		m_ctlWalkList.EnableWindow(FALSE);
		m_ctlWalkList.DeleteAllItems();
	}
	
	CDialog::OnOK();
}

CString CQueryEditDlg::GetQuery() 
{
	return m_szQuery;
}

void CQueryEditDlg::ReadCategories()
{
	m_arrCat.RemoveAll();
	DWORD dwMaxBufferSize=255;

	CString strIniFile = getenv("OPENSTA_HOME");
	strIniFile += "\\BaseUI\\snmpcat.ini";

	int nCat = GetPrivateProfileInt("CATEGORY","NUMCAT", 0,strIniFile);

	for (int i = 0; i<nCat; i++)
	{
		CString strKey;
		strKey.Format("CAT%d",i+1);
		CString strCat;
		GetPrivateProfileString("CATEGORY",strKey,"",strCat.GetBuffer(dwMaxBufferSize),dwMaxBufferSize,strIniFile);
		strCat.ReleaseBuffer();

		int nLen = strCat.GetLength();
		int nPos = strCat.Find(',');
		CATEGORY cat;
		cat.strCatName = strCat.Left(nPos);
		cat.strWalkStart = strCat.Right(nLen - nPos -1);
		m_arrCat.Add(cat);

		m_ctlCategory.InsertString(i,cat.strCatName);
	}
}

BOOL CQueryEditDlg::OnHelpInfo(HELPINFO* pHelpInfo) 
{
	pHelpInfo->dwContextId = HIDD_SNMP_QUERY_EDIT;
	return CDialog::OnHelpInfo(pHelpInfo);
}
